{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_admin_presets/settings_rollback

    Template for rolling preset back result.

    Example context (json):
    {
        "nopresets": 0,
        "showactions": 1
    }
}}

{{#appliedchanges.show}}
    <div class="alert alert-success">{{#str}}rollbackresults, tool_admin_presets{{/str}}</div>
    {{#appliedchanges}}{{> tool_admin_presets/local/tables/old_and_new_value_settings_table}}{{/appliedchanges}}
{{/appliedchanges.show}}
{{#skippedchanges.show}}
    <div class="alert alert-info">{{#str}}rollbackfailures, tool_admin_presets{{/str}}</div>
    {{#skippedchanges}}{{> tool_admin_presets/local/tables/old_and_new_value_settings_table}}{{/skippedchanges}}
{{/skippedchanges.show}}
