{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_grades/bulkactions/edit/tree/bulk_move_grade_tree

    Renders the tree of categories and items for the bulk move action.

    Context variables required for this template:
    * id - The id of the category.
    * name - The name of the category.
    * children - An array of child categories.
    * iscategory - Whether the item is a category or not.

    Example context (json):
    {
        "id": 1,
        "name": "Category 1",
        "children": [
            {
                "id": 2,
                "name": "Category 2",
                "children": [
                    {
                        "id": 3,
                        "name": "Category 3",
                        "children": [
                            {
                                "id": 4,
                                "name": "Category 4",
                                "children": [],
                                "iscategory": true
                            }
                        ],
                        "iscategory": true
                    }
                ],
                "iscategory": true
            }
        ],
        "iscategory": true
    }
}}
<p data-for="sectionname">{{#str}} moveselectedto, grades {{/str}}</p>
<nav class="collapse-list" id="destination-selector">
    <ul class="unlist p-0" role="tree">
        {{#iscategory}}
            {{> core_grades/bulkactions/edit/tree/bulk_move_category_item }}
        {{/iscategory}}
    </ul>
</nav>
