{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template gradereport_singleview/dropdown_attribute

    Dropdown attribute.

    Example context (json):
    {
        "name": "Awesome-test",
        "disabled": "true",
        "options": "true",
        "value": "1",
        "selected": "true"
    }
}}
{{#readonly}}
    {{text}}
{{/readonly}}
{{^readonly}}
    <label for="{{name}}" class="accesshide">{{label}}</label>
    <select id="{{name}}" name="{{name}}" class="form-select" {{#disabled}}disabled{{/disabled}}>
        {{#options}}
            <option value="{{value}}" {{#selected}}selected{{/selected}}>{{name}}</option>
        {{/options}}
    </select>
    <input type="hidden" name="old{{name}}" value="{{value}}">
{{/readonly}}
