{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_lti/tool_config_modal_body

    This template provides the layout for the body of the configuration summary modal for
    a single tool opened from the tool configuration page.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * platformid
    * clientid
    * deploymentid
    * urls
    ** publickeyset
    ** accesstoken
    ** authrequest

    Example context (json):
    {
        "platformid": "htps://...",
        "clientid": "123",
        "deploymentid": "1",
        "urls": {
            "publickeyset": "https://...",
            "accesstoken": "https://...",
            "authrequest": "https://..."
        }
    }

}}
<ul>
  <li><strong>{{#str}} tooldetailsplatformid, mod_lti {{/str}}:</strong> {{platformid}}</li>
  <li><strong>{{#str}} tooldetailsclientid, mod_lti {{/str}}:</strong> {{clientid}}</li>
  <li><strong>{{#str}} tooldetailsdeploymentid, mod_lti {{/str}}:</strong> {{deploymentid}}</li>
  <li><strong>{{#str}} tooldetailspublickeyseturl, mod_lti {{/str}}:</strong> {{urls.publickeyset}}</li>
  <li><strong>{{#str}} tooldetailsaccesstokenurl, mod_lti {{/str}}:</strong> {{urls.accesstoken}}</li>
  <li><strong>{{#str}} tooldetailsauthrequesturl, mod_lti {{/str}}:</strong> {{urls.authrequest}}</li>
</ul>
