{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_data/fields/requiredfield

    The purpose of this template is to enable the user to define the 'required' field.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * none

    Example context (json):
    {
        "required": false
    }
}}

<div class="row mb-2 fitem">
    <label for="required" class="col-auto col-md-3 col-form-label">{{#str}}requiredfield, data{{/str}}</label>
    <div class="col col-md-9 align-self-center felement">
        <input class="requiredfield form-check" type="checkbox" name="required" id="required" {{#required}}
            checked="checked" {{/required}}>
    </div>
</div>
