{{!
    This file is part of Moodle - http://moodle.org/
    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template gradereport_grader/action_bar

    Actions bar for the grader report.

    Context variables required for this template:
    * generalnavselector - The data object containing the required properties to render the general navigation selector.
    * groupselector - (optional) HTML that outputs the group selector

    Example context (json):
    {
        "generalnavselector": {
            "name": "Gradebook tertiary navigation selector",
            "value": "opt2",
            "baseid": "select-menu56789",
            "selectedoption": "Gradebook setup",
            "options": [
                {
                    "selected": false,
                     "isgroup": {
                         "name": "View",
                         "id": "select-menu-group1",
                         "options": [
                             {
                                 "name": "Grader report",
                                 "value": "opt1",
                                 "id": "select-menu-option1",
                                 "selected": false
                             }
                         ]
                     }
                },
                {
                    "selected": false,
                    "isgroup": {
                        "name": "Setup",
                        "id": "select-menu-group2",
                        "options": [
                            {
                                "name": "Gradebook setup",
                                "value": "opt2",
                                "id": "select-menu-option2",
                                "selected": true
                            }
                        ]
                    }
                }
            ]
        },
        "searchdropdown": {"search dropdown": "context here"},
        "groupselector": {"group selector": "context here"},
        "initialselector": {"initial selector": "context here"},
        "collapsedcolumns": "<div class='collapse-columns'></div>",
        "pagereset": "http://moodle.local/grade/report/grader/index.php?id=2&userid=2&reset=1"
    }
}}
<div class="container-fluid tertiary-navigation full-width-bottom-border">
    <div class="d-flex flex-wrap">
        {{#generalnavselector}}
            <div class="navitem">
                {{>core/tertiary_navigation_selector}}
            </div>
            <div class="navitem-divider"></div>
        {{/generalnavselector}}
        {{#searchdropdown}}
            <div class="navitem">
                {{>core/comboboxsearch}}
            </div>
            <div class="navitem-divider"></div>
        {{/searchdropdown}}
        {{#groupselector}}
            <div class="navitem">
                {{>core/comboboxsearch}}
            </div>
            <div class="navitem-divider"></div>
        {{/groupselector}}
        {{#initialselector}}
            <div class="navitem d-flex flex-column align-self-center">
                {{>core/comboboxsearch}}
            </div>
            <div class="navitem-divider"></div>
        {{/initialselector}}
        {{#pagereset}}
            <div class="navitem align-self-center">
                <a href="{{{.}}}" class="btn btn-link">{{#str}}clearall, core{{/str}}</a>
            </div>
            <div class="navitem-divider"></div>
        {{/pagereset}}
        {{#collapsedcolumns}}
            <div class="navitem flex-column align-self-center ms-auto {{#classes}}{{.}}{{/classes}}" aria-live="polite">
                {{#content}}
                    {{>core/comboboxsearch}}
                {{/content}}
            </div>
        {{/collapsedcolumns}}
    </div>
</div>
