{{!
    This file is part of Moodle - http://moodle.org/
    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_feedback/item_edit_enhanced_title

    The enhanced item title template when editing questions in the feedback activity.

    Context variables required for this template:
    * movetitle - The string for the title attribute of the move button.
    * label - The question text.
    * actionsmenu - Context data for the actions menu.

    Example context (json):
    {
        "movetitle": "Move item",
        "label": "Hello world!",
        "actionsmenu": {}
    }
}}
<div class="itemtitle d-flex mb-3">
    <div class="itemhandle" data-drag-type="move">
        {{> core/drag_handle}}
    </div>
    <div class="itemname" data-region="item-title">
        {{{label}}}
    </div>
    {{#actionsmenu}}
    <div class="itemactions">
        {{> core/action_menu}}
    </div>
    {{/actionsmenu}}
</div>
