<div id="blocks-drawer" data-region="drawer" class="moodle-has-zindex {{^blocksdraweropen}}closed{{/blocksdraweropen}}" aria-hidden="{{#blocksdraweropen}}false{{/blocksdraweropen}}{{^blocksdraweropen}}true{{/blocksdraweropen}}">
    {{{ sidepreblocks }}}
</div>
