<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * mod_choice data generator.
 *
 * @package mod_choice
 * @category test
 * @copyright 2013 Adrian Greeve <adrian@moodle.com>
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

defined('MOODLE_INTERNAL') || die();

/**
 * mod_choice data generator class.
 *
 * @package mod_choice
 * @category test
 * @copyright 2013 Adrian Greeve <adrian@moodle.com>
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class mod_choice_generator extends testing_module_generator {

    public function create_instance($record = null, ?array $options = null) {
        $record = (object) (array) $record;

        if (!isset($record->timemodified)) {
            $record->timemodified = time();
        }
        if (isset($record->timeclose) && is_string($record->timeclose)) {
            $record->timeclose = strtotime($record->timeclose);
        }
        if (!isset($record->option)) {
            $record->option = array();
            $record->option[] = 'Soft Drink';
            $record->option[] = 'Beer';
            $record->option[] = 'Wine';
            $record->option[] = 'Spirits';
        } else if (!is_array($record->option)) {
            $record->option = preg_split('/\s*,\s*/', trim($record->option), -1, PREG_SPLIT_NO_EMPTY);
        }
        return parent::create_instance($record, (array) $options);
    }

    /**
     * Submit a choice for a user (or current user))
     *
     * @param array $record composed of choiceid, choiceoptionids and userid
     * @return void
     */
    public function create_response(array $record): void {
        global $USER, $DB;
        [
            'choiceid' => $choiceid,
            'responses' => $responses,
            'userid' => $userid,
        ] = $record;
        if (empty($userid)) {
            $userid = $USER->id;
        }
        [$course, $cm] = get_course_and_cm_from_instance($choiceid, 'choice');
        $choice = $DB->get_record('choice', ['id' => $choiceid], '*', MUST_EXIST);
        choice_user_submit_response($responses, $choice, $userid, $course, $cm);
    }
}
