/* Rubric - grid layout */

.path-mod-workshop .mform.frozen #id_rubric-grid-wrapper,
.path-mod-workshop .assessmentform.rubric #id_rubric-grid-wrapper {
    border: none;
}

.path-mod-workshop .mform.frozen #id_rubric-grid-wrapper legend,
.path-mod-workshop .assessmentform.rubric #id_rubric-grid-wrapper legend {
    display: none;
}

.path-mod-workshop .mform.frozen #id_rubric-grid-wrapper th,
.path-mod-workshop .mform.frozen #id_rubric-grid-wrapper td,
.path-mod-workshop .assessmentform.rubric.grid #id_rubric-grid-wrapper th,
.path-mod-workshop .assessmentform.rubric.grid #id_rubric-grid-wrapper td {
    border: 1px solid #ddd;
    padding: 5px;
    vertical-align: top;
}

.path-mod-workshop .mform.frozen #id_rubric-grid-wrapper,
.path-mod-workshop .assessmentform.rubric.grid #id_rubric-grid-wrapper .criterion {
    text-align: center;
}

.path-mod-workshop .assessmentform.rubric.grid #id_rubric-grid-wrapper .fitem {
    text-align: center;
}

.path-mod-workshop .mform.frozen #id_rubric-grid-wrapper .fitem .fitemtitle,
.path-mod-workshop .assessmentform.rubric.grid #id_rubric-grid-wrapper .fitem .fitemtitle {
    display: none;
}

.path-mod-workshop .mform.frozen #id_rubric-grid-wrapper,
.path-mod-workshop #id_rubric-grid-wrapper {
    margin-left: auto;
    margin-right: auto;
    width: 100%;
}

.path-mod-workshop .mform.frozen #id_rubric-grid-wrapper .checkbox,
.path-mod-workshop .assessmentform.rubric.grid #id_rubric-grid-wrapper .checkbox {
    max-width: 100%;
    flex: 0 0 100%;
    text-align: left;
}

.path-mod-workshop .mform.frozen #id_rubric-grid-wrapper .fitem .felement,
.path-mod-workshop .assessmentform.rubric.grid #id_rubric-grid-wrapper .fitem .felement {
    width: 100%;
    margin-left: auto;
    margin-right: auto;
}

.path-mod-workshop .mform.frozen #id_rubric-grid-wrapper .fitem .felement {
    border: none;
}

.path-mod-workshop .assessmentform.rubric.grid #id_rubric-grid-wrapper .fitem .felement span {
    display: block;
    text-align: center;
}

.path-mod-workshop .assessmentform.rubric.grid #id_rubric-grid-wrapper .fitem .felement span label {
    display: block;
    text-align: center;
}

/* Rubric - list layout */

.path-mod-workshop .mform.frozen .fitem.description.rubric + .fitem .fitemtitle,
.path-mod-workshop .assessmentform.rubric.list #id_rubric-grid-wrapper .fitem .fitemtitle {
    display: none;
}

.path-mod-workshop .mform.frozen .fitem.description.rubric + .fitem .fitemtitle + .felement,
.path-mod-workshop .assessmentform.rubric.list .fitem .felement {
    width: auto;
    border: none;
}

.path-mod-workshop .assessmentform.rubric.list .fitem .felement span input {
    display: block;
    float: left;
}

.path-mod-workshop .assessmentform.rubric.list .fitem .felement.fgroup span label {
    display: block;
    margin-left: 30px;
}
