{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_customlang/translator

    Template for the custom language translator page.

    Classes required for JS:
    -

    Data attributes required for JS:
    -

    Context variables required for this template:
    * strings

    Example context (json):
    {
        "hasstrings": true,
        "formurl": "admin/tool/customlang/edit.php?lng=en",
        "currentpage": 0,
        "sesskey" : "AZyeeQgmcs",
        "strings": [
            {
                "id": 11,
                "component": "core",
                "componentid": 1,
                "stringid": "course",
                "original": "Course",
                "master": "Cursus",
                "local": "Hoofdstuk",
                "outdated": 0,
                "modified": 1
            }
        ]
    }
}}

{{^hasstrings}}
    {{{ nostrings }}}
{{/hasstrings}}
{{#hasstrings}}
<form method="post" action="{{{formurl}}}">
    <input type="hidden" name="translatorsubmitted" value="1">
    <input type="hidden" name="sesskey" value="{{{ sesskey }}}">
    <input type="hidden" name="p" value="{{ currentpage }}">

    <fieldset class="m-3">
        <button type="submit" name="savecontinue" class="btn btn-secondary">
            {{#str}}savecontinue, tool_customlang{{/str}}
        </button>
        <button type="submit" name="savecheckin" class="btn btn-secondary">
            {{#str}}savecheckin, tool_customlang{{/str}}
        </button>
    </fieldset>

    <div class="list-group">
        <div class="container-fluid d-none d-md-block list-group-item border-bottom-0">
            <div class="row">
                <div class="col-sm-4 col-md-2">
                    <strong>{{#str}}headingcomponent, tool_customlang{{/str}}</strong>
                </div>
                <div class="col-sm-4 col-md-2">
                    <strong>{{#str}}headingstringid, tool_customlang{{/str}}</strong>
                </div>
                <div class="col-sm-4 col-md-2">
                    <strong>{{#str}}headingstandard, tool_customlang{{/str}}</strong>
                </div>
                <div class="col-sm-12 col-md-6">
                    <span class="ps-3">
                        <strong>{{#str}}headinglocal, tool_customlang{{/str}}</strong>
                    </span>
                </div>
            </div>
        </div>
    </div>

    <div class="list-group">
    {{#strings}}
        <div class="container-fluid list-group-item
                {{#local}}list-group-item-info{{/local}}
                {{#outdated}}list-group-item-warning{{/outdated}}
                {{#modified}}list-group-item-info{{/modified}}"
            >
            <div class="row">
                <div class="col-sm-4 col-md-2 text-break">
                    <div class="d-md-none">
                        <strong>{{#str}}headingcomponent, tool_customlang{{/str}}</strong>
                    </div>
                    {{{ component }}}
                </div>
                <div class="col-sm-4 col-md-2 text-break">
                    <div class="d-md-none">
                        <strong>{{#str}}headingstringid, tool_customlang{{/str}}</strong>
                    </div>
                    {{{ stringid }}}
                </div>
                <div class="col-sm-4 col-md-2">
                    <div class="d-md-none">
                        <strong>{{#str}}headingstandard, tool_customlang{{/str}}</strong>
                    </div>
                    {{ master }}
                    <div class="info">
                        {{{ placeholderhelp }}}
                        {{{ outdatedhelp}}}
                    </div>
                    {{#showoriginalvsmaster}}
                    <div class="alert-secondary mt-3 mt-1">
                        {{ original }}
                    </div>
                    {{/showoriginalvsmaster}}
                </div>
                <div class="col-sm-12 col-md-6 mt-sm-3 mt-md-0">
                    <div class="d-md-none">
                        <strong>{{#str}}headinglocal, tool_customlang{{/str}}</strong>
                    </div>
                    <div class="py-2 py-md-0 px-md-3">
                        <label for="{{id}}" class="visually-hidden-focusable">{{{ component }}}/{{{ stringid }}}</label>
                        <textarea class="form-control w-100 border-box" id="{{id}}" name="cust[{{id}}]" cols="40" rows="3">{{{ local }}}</textarea>
                        {{#checkupdated}}
                        <div class="uptodatewrapper">
                            <div class="form-check">
                                <input id="update_{{id}}" class="form-check-input" name="updates[]" type="checkbox" value="{{id}}">
                                <label for="update_{{id}}" class="form-check-label">{{#str}}markuptodate, tool_customlang{{/str}}</label>
                                {{{ outdatedhelp }}}
                            </div>
                        </div>
                        {{/checkupdated}}
                    </div>
                </div>
            </div>
        </div>
    {{/strings}}
    </div>

    <fieldset class="m-3">
        <button type="submit" name="savecontinue" class="btn btn-secondary">
            {{#str}}savecontinue, tool_customlang{{/str}}
        </button>
        <button type="submit" name="savecheckin" class="btn btn-secondary">
            {{#str}}savecheckin, tool_customlang{{/str}}
        </button>
    </fieldset>
</form>
{{/hasstrings}}
