{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_forum/local/grades/local/grader/user_picker/user

    Classes required for JS:
    * none

    Data attributes required for JS:
    * data-userid="42"
    * data-region="index"

    Context variables required for this template:
    * id: The ID of the user
    * fullname: The user's full name
    * displayIndex: The index of the user in the userpicker array + 1
    * total: The length of the user array in the userpicker

    Example context (json):
    {
        "id": 4,
        "fullname": "Phillip J. Fry",
        "displayIndex": 3,
        "total": 7
    }
}}
{{#profileimageurl}}
<img
    class="rounded-circle userpicture me-2"
    src="{{.}}"
    aria-hidden="true"
>
{{/profileimageurl}}
<div class="overflow-hidden">
    <h5
        class="mb-0 fw-bold user-full-name text-truncate"
        data-region="name"
        data-userid="{{id}}"
    >
        {{fullname}}
    </h5>
</div>

