<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

namespace mod_assign;

use core\shortlink_handler_interface;

/**
 * Shortlink handler for mod_assign.
 *
 * @package    mod_assign
 * @copyright  2025 Andrew Lyons <andrew@nicols.co.uk>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class shortlink_handler implements shortlink_handler_interface {
    #[\Override]
    public function get_valid_linktypes(): array {
        return [
            'view',
        ];
    }

    #[\Override]
    public function process_shortlink(
        string $type,
        string $identifier,
    ): ?\core\url {
        return match ($type) {
            'view' => new \core\url('/mod/assign/view.php', [
                'id' => $identifier,
            ]),
            default => null,
        };
    }
}
