<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'communication', language 'en'.
 *
 * @package    core_communication
 * @copyright  2022 Huong Nguyen <huongnv13@gmail.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['communication'] = 'Communication';
$string['communicationdisabled'] = 'Communication is disabled.';
$string['communicationprovidernotfound'] = 'The \'{$a}\' communication provider doesn\'t exist or is not recognised.';
$string['communicationroomname'] = 'Room name';
$string['communicationroomnameinfo'] = 'If the field is left blank, the course name is used as the room name.';
$string['communicationroompending'] = 'Your {$a} room will be ready soon.';
$string['communicationroomready'] = 'Your {$a} room is ready.';
$string['communicationgrouproomnameformat'] = '{$a->groupname} ({$a->baseroomname})';
$string['managecommunicationproviders'] = 'Manage communication providers';
$string['nocommunicationinstance'] = 'No communication instance found.';
$string['nocommunicationprovider'] = 'No communication provider found.';
$string['nocommunicationselected'] = 'None';
$string['privacy:metadata:communication_user'] = 'The communication user table stores the user ID and communication ID for communication provider mapping.';
$string['privacy:metadata:communication_user:commid'] = 'The communication ID for user mapping';
$string['privacy:metadata:communication_user:userid'] = 'The user ID for communication provider mapping for users';
$string['privacy:metadata:communication_user:synced'] = 'The user is synced with the provider or not';
$string['selectcommunicationprovider'] = 'Provider';
$string['selectcommunicationprovider_help'] = 'The tool available for chat and related communication methods.';
$string['synchroniseproviders'] = 'Synchronise communication providers';
