{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_contentbank/copycontent

    Example context (json):
    {
        "contentid": 17,
        "name": "Accordion"
    }

}}
    <div class="form-check w-100 justify-content-start">
        <label for="newname">{{#str}}contentname, core_contentbank{{/str}}</label>
        <input type="text" size="5" id="newname" name="newname"  value="{{ name }}" class="form-control">
    </div>
