{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_message/message_drawer_conversations_list_items_placeholder

    This template will render a placeholder loading element for a message in
    the message drawer.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * All data attributes are required

    Context variables required for this template:
    * userid The logged in user id
    * urls The URLs for the popover

    Example context (json):
    {}

}}
{{#placeholders}}
    <div class="p-2 d-flex list-group-item border-0" data-region="placeholder">
        <div
            class="rounded-circle bg-pulse-grey"
            style="height: 38px; width: 38px"
        >
        </div>
        <div class="ms-2 align-self-center" style="flex: 1">
            <div
                class="bg-pulse-grey w-50"
                style="height: 15px;"
            >
            </div>
            <div
                class="bg-pulse-grey mt-1"
                style="height: 15px; width: 90%"
            >
            </div>
        </div>
        <div
            class="bg-pulse-grey align-self-center"
            style="height: 15px; width: 15px"
        >
        </div>
        <div
            class="ms-2 bg-pulse-grey align-self-center"
            style="height: 15px; width: 15px"
        >
        </div>
    </div>
{{/placeholders}}