{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template qbank_columnsortorder/resize_modal

    This template renders the content of the resize modal, a number input to set the column width in pixels.

    Example context (json):
    {
    }
}}
<form class="has-validation">
    <label for="columnwidth">{{#str}}columnwidth, qbank_columnsortorder{{/str}}</label>
    <input class="form-control" id="columnwidth" type="number" value="{{width}}" min="{{min}}">
    <div class="invalid-feedback">
        {{#str}}invalidwidth, qbank_columnsortorder, {{min}}{{/str}}
    </div>
</form>

