{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_quiz/random_question_form_preview_question_list

    @deprecated since Moodle 5.1
    @todo MDL-84777 Final deprecation on Moodle 6.0

    A preview section for the list of questions that match the random
    question criteria in the random question form.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * title A cleaned string (use clean_text()) to display.
    * body HTML content for the boday

    Example context (json):
    {
        "questions": [
            {
                "name":"Example question 1"
            },
            {
                "name":"Example question 2"
            },
            {
                "name":"Example question 3"
            },
            {
                "name":"Example question 4"
            }
        ]
    }
}}
<ul class="list-group" data-region="question-list">
    {{#questions}}
        <li class="list-group-item text-truncate">
            {{#icon}}{{#pix}} {{key}}, {{component}}, {{{alttext}}} {{/pix}}{{/icon}}
            {{{name}}}
        </li>
    {{/questions}}
</ul>
