{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_lp/action_selector

    Select an action to execute.

    Classes required for JS:
    * none

    Data attibutes required for JS:
    * none

    Context variables required for this template:
    * message - The message to explain why we ask user to select an action
    * choices - List of possible actions
    * confirm - Text for button confirms
    * cancel -  Text for button cancel

    Example context (json):
    {
        "message": "Message",
        "choices": [
            { "value": "1", "text": "One" },
            { "value": "2", "text": "Two" }
        ],
        "confirm": "Confirm",
        "cancel": "Cancel"
    }
}}
<div data-region="action-selector">
    <div data-region="action-selector-message">
       {{message}}<br>
    </div><br>
    <div data-region="action-selector-radio-buttons">
       {{#choices}}
        <input id="action-selection-option-{{value}}" type="radio" class="me-1" name="choice" value="{{value}}"/><label for="action-selection-option-{{value}}">{{text}}</label><br>
       {{/choices}}
    </div><br>
    <div data-region="action_selector-buttons">
       <input type="button" data-action="action-selector-confirm" class="btn btn-primary" value="{{confirm}}"/>
       <input type="button" data-action="action-selector-cancel" class="btn btn-secondary" value="{{cancel}}"/>
    </div>
</div>
