<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'profilefield_social'
 *
 * @package   profilefield_social
 * @copyright 2020 Bas Brands <bas@moodle.com>
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['invalidnetwork'] = 'Invalid social network';
$string['networkinuse'] = 'Social network has already been added';
$string['pluginname'] = 'Social';
$string['networktype'] = 'Network type';
$string['privacy:metadata:profile_field_social:data'] = 'Text area user profile field user data';
$string['privacy:metadata:profile_field_social:dataformat'] = 'The format of Text area user profile field user data';
$string['privacy:metadata:profile_field_social:fieldid'] = 'The ID of the profile field';
$string['privacy:metadata:profile_field_social:tableexplanation'] = 'Additional profile data';
$string['privacy:metadata:profile_field_social:userid'] = 'The ID of the user whose data is stored by the social user profile field';
$string['aimid'] = 'AIM ID';
$string['yahooid'] = 'Yahoo ID';
$string['skypeid'] = 'Skype ID';
$string['icqnumber'] = 'ICQ number';
$string['msnid'] = 'MSN ID';
$string['webpage'] = 'Website';
