{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template gradereport_singleview/bulk_insert

    Bulk insert attribute.

    Example context (json):
    {
        "applyname": "Apply-name",
        "applylabel": "Perform bulk insert",
        "label": "Bulk insert",
        "menuname": "Menu-name",
        "menulabel": "For",
        "menuoptions": "JSON object",
        "value": "all",
        "selected": "",
        "name": "All grades",
        "valuename": "Value-name",
        "valuelabel": "Insert value",
        "valuefield": "PARSED HTML"
    }
}}
<div class="enable">
    <input type="checkbox" name="{{applyname}}" value="1" id="{{applyname}}" class="ms-0">
    <label for="{{applyname}}">{{applylabel}}</label>
</div>
<fieldset class="d-flex flex-wrap align-items-center">
    <legend class="accesshide">{{label}}</legend>
    <label for="{{menuname}}">{{menulabel}}</label>
    <select name="{{menuname}}" id="{{menuname}}" class="form-control form-select">
        {{#menuoptions}}
            <option value="{{value}}" {{#selected}}selected{{/selected}}>{{name}}</option>
        {{/menuoptions}}
    </select>
    <label for="{{valuename}}">{{valuelabel}}</label>
    {{{valuefield}}}
</fieldset>
