/*!
 * Bootstrap Docs (https://getbootstrap.com/)
 * Copyright 2011-2020 The Bootstrap Authors
 * Copyright 2011-2020 Twitter, Inc.
 * Licensed under the Creative Commons Attribution 3.0 Unported License.
 * For details, see https://creativecommons.org/licenses/by/3.0/.
 */
.bd-content > h2[id],
.bd-content > h3[id],
.bd-content > h4[id] {
  pointer-events: none;
}
.bd-content > h2[id]::before,
.bd-content > h3[id]::before,
.bd-content > h4[id]::before {
  display: block;
  height: 6rem;
  margin-top: -6rem;
  content: "";
}
.bd-content > .table {
  max-width: 100%;
  margin-bottom: 1.5rem;
}
@media (max-width: 767.98px) {
  .bd-content > .table {
    display: block;
    overflow-x: auto;
  }
  .bd-content > .table.table-bordered {
    border: 0;
  }
}
.bd-content > .table th:first-child,
.bd-content > .table td:first-child {
  padding-left: 0;
}
.bd-content > .table th:not(:last-child),
.bd-content > .table td:not(:last-child) {
  padding-right: 1.5rem;
}
.bd-content > .table td:first-child > code {
  white-space: nowrap;
}

.bd-content-title {
  display: block;
  pointer-events: auto;
}

.bd-content > h2:not(:first-child) {
  margin-top: 3rem;
}
.bd-content > h3 {
  margin-top: 1.5rem;
}
.bd-content > ul li,
.bd-content > ol li {
  margin-bottom: 0.25rem;
}

.bd-title {
  margin-bottom: 0.5rem;
  font-size: calc(1.425rem + 2.1vw);
}
@media (min-width: 1200px) {
  .bd-title {
    font-size: 3rem;
  }
}

.bd-lead {
  font-size: calc(1.275rem + 0.3vw);
  font-weight: 300;
}
@media (min-width: 1200px) {
  .bd-lead {
    font-size: 1.5rem;
  }
}

.bd-text-purple {
  color: #563d7c;
}

.bd-text-purple-bright {
  color: #7952b3;
}

.skippy {
  display: block;
  padding: 1em;
  color: #fff;
  text-align: center;
  background-color: #563d7c;
  outline: 0;
}
.skippy:hover {
  color: #fff;
}

.skippy-text {
  padding: 0.5em;
  outline: 1px dotted;
}

.sidebar {
  line-height: 1.8;
  font-size: 90%;
}

@media (min-width: 576px) {
  .sidebar {
    border-right: 1px solid #dee2e6;
    border-bottom: 1px solid #dee2e6;
  }
}
.sidebar .dropdown-menu .dropdown-item {
  white-space: normal;
  text-transform: capitalize;
}

.docs-sidebar ul {
  list-style: none;
  padding: 0;
  margin: 0;
}

.docs-sidebar > ul {
  padding: 0 2px;
}

.docs-sidebar a {
  position: relative;
  display: flex;
  align-items: center;
  color: #212529;
  text-decoration: none;
  padding: 0.3rem 0.5rem;
  border-top: solid 1px #dee2e6;
}
.docs-sidebar a.dropdown-toggle::after {
  margin-left: auto;
  font-size: 14px;
  color: #ced4da;
}

.docs-sidebar a:hover {
  background-color: #e9ecef;
}

.docs-sidebar > ul > li li a {
  padding-left: 1rem;
  border-top: solid 1px transparent;
}

.docs-sidebar > ul ul > li li a {
  padding-left: 2rem;
  border-top: solid 1px transparent;
}

@media screen and (max-width: 480px) {
  .docs-sidebar > ul > li:last-child a {
    border-bottom: solid 1px #dee2e6;
  }
}
.docs-sidebar .active > a {
  background: #dee2e6;
  font-weight: bold;
}

.docs-sidebar .slide-menu .has-sub-menu:not(.parent) ul {
  display: none;
}

.secondnav {
  z-index: 0;
  position: fixed;
  width: 100%;
  height: calc(100vh - 50px);
  top: 50px;
  overflow-y: auto;
}

#TableOfContents > ul {
  font-size: 90%;
  padding-left: 0;
  padding-top: 1rem;
  border-left: 1px solid #dee2e6;
}
#TableOfContents > ul ul {
  padding-left: 1rem;
}
#TableOfContents > ul li {
  display: block;
}
#TableOfContents > ul li a {
  display: block;
  padding: 0.125rem 1.5rem;
  color: #212529;
}
#TableOfContents > ul li a:hover {
  color: #0d6efd;
  text-decoration: none;
}

.bd-toc nav {
  padding-top: 0.125em;
  padding-bottom: 0.125em;
  border-left: 0.25em solid #e9ecef;
}
.bd-toc nav > ul {
  padding-left: 0.5rem;
  margin-bottom: 0;
}
.bd-toc nav > ul > li {
  display: block;
}
.bd-toc nav li ul li {
  margin-bottom: 0.25rem;
  list-style-type: disc;
}
.bd-toc nav a code {
  font: inherit;
}

.bd-footer {
  font-size: 0.875rem;
  color: #63707c;
}
.bd-footer a {
  font-weight: 600;
  color: #495057;
}
.bd-footer a:hover, .bd-footer a:focus {
  color: #0d6efd;
}
.bd-footer p {
  margin-bottom: 0;
}

.bd-footer-links {
  padding-left: 0;
  margin-bottom: 1rem;
}
.bd-footer-links li {
  display: inline-block;
}
.bd-footer-links li + li {
  margin-left: 1rem;
}

.bd-example-row .row > .col,
.bd-example-row .row > [class^=col-] {
  padding-top: 0.75rem;
  padding-bottom: 0.75rem;
  background-color: rgba(86, 61, 124, 0.15);
  border: 1px solid rgba(86, 61, 124, 0.2);
}
.bd-example-row .row + .row {
  margin-top: 1rem;
}

.bd-example-row-flex-cols .row {
  min-height: 10rem;
  background-color: rgba(255, 0, 0, 0.1);
}

.bd-highlight {
  background-color: rgba(86, 61, 124, 0.15);
  border: 1px solid rgba(86, 61, 124, 0.15);
}

.bd-example-responsive-containers [class^=container] {
  padding-top: 0.75rem;
  padding-bottom: 0.75rem;
  background-color: rgba(86, 61, 124, 0.15);
  border: 1px solid rgba(86, 61, 124, 0.2);
}

.example-container {
  width: 800px;
  --bs-gutter-x: 1.5rem;
  --bs-gutter-y: 0;
  width: 100%;
  padding-right: calc(var(--bs-gutter-x) * 0.5);
  padding-left: calc(var(--bs-gutter-x) * 0.5);
  margin-right: auto;
  margin-left: auto;
}

.example-row {
  --bs-gutter-x: 1.5rem;
  --bs-gutter-y: 0;
  display: flex;
  flex-wrap: wrap;
  margin-top: calc(-1 * var(--bs-gutter-y));
  margin-right: calc(-0.5 * var(--bs-gutter-x));
  margin-left: calc(-0.5 * var(--bs-gutter-x));
}

.example-content-main {
  flex-shrink: 0;
  width: 100%;
  max-width: 100%;
  padding-right: calc(var(--bs-gutter-x) * 0.5);
  padding-left: calc(var(--bs-gutter-x) * 0.5);
  margin-top: var(--bs-gutter-y);
}
@media (min-width: 576px) {
  .example-content-main {
    flex: 0 0 auto;
    width: 50%;
  }
}
@media (min-width: 992px) {
  .example-content-main {
    flex: 0 0 auto;
    width: 66.66666667%;
  }
}

.example-content-secondary {
  flex-shrink: 0;
  width: 100%;
  max-width: 100%;
  padding-right: calc(var(--bs-gutter-x) * 0.5);
  padding-left: calc(var(--bs-gutter-x) * 0.5);
  margin-top: var(--bs-gutter-y);
}
@media (min-width: 576px) {
  .example-content-secondary {
    flex: 0 0 auto;
    width: 50%;
  }
}
@media (min-width: 992px) {
  .example-content-secondary {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
}

.bd-example {
  position: relative;
  padding: 1rem;
  margin: 1rem -0.75rem 0;
  border: solid #f8f9fa;
  border-width: 0.2rem 0 0;
}
.bd-example::after {
  display: block;
  clear: both;
  content: "";
}
@media (min-width: 576px) {
  .bd-example {
    padding: 1.5rem;
    margin-right: 0;
    margin-left: 0;
    border-width: 0.2rem;
  }
}
.bd-example + .highlight,
.bd-example + .clipboard + .highlight {
  margin-top: 0;
}
.bd-example + p {
  margin-top: 2rem;
}
.bd-example > .form-control + .form-control {
  margin-top: 0.5rem;
}
.bd-example > .nav + .nav,
.bd-example > .alert + .alert,
.bd-example > .navbar + .navbar,
.bd-example > .progress + .progress {
  margin-top: 1rem;
}
.bd-example > .dropdown-menu {
  position: static;
  display: block;
}

.bd-example > svg + svg,
.bd-example > img + img {
  margin-left: 0.5rem;
}

.bd-example > .btn,
.bd-example > .btn-group {
  margin-top: 0.25rem;
  margin-bottom: 0.25rem;
}
.bd-example > .btn-toolbar + .btn-toolbar {
  margin-top: 0.5rem;
}

.bd-example > .list-group {
  max-width: 400px;
}

.bd-example > [class*=list-group-horizontal] {
  max-width: 100%;
}

.bd-example .fixed-top,
.bd-example .sticky-top {
  position: static;
  margin: -1rem -1rem 1rem;
}
.bd-example .fixed-bottom {
  position: static;
  margin: 1rem -1rem -1rem;
}
@media (min-width: 576px) {
  .bd-example .fixed-top,
  .bd-example .sticky-top {
    margin: -1.5rem -1.5rem 1rem;
  }
  .bd-example .fixed-bottom {
    margin: 1rem -1.5rem -1.5rem;
  }
}

.bd-example .pagination {
  margin-top: 0.5rem;
  margin-bottom: 0.5rem;
}

.bd-example-modal {
  background-color: #fafafa;
}
.bd-example-modal .modal {
  position: static;
  display: block;
}

.tooltip-demo a {
  white-space: nowrap;
}

.scrollspy-example {
  position: relative;
  height: 200px;
  margin-top: 0.5rem;
  overflow: auto;
}

.scrollspy-example-2 {
  position: relative;
  height: 350px;
  overflow: auto;
}

.bd-example-border-utils [class^=border] {
  display: inline-block;
  width: 5rem;
  height: 5rem;
  margin: 0.25rem;
  background-color: #f5f5f5;
}

.bd-example-border-utils-0 [class^=border] {
  border: 1px solid #dee2e6;
}

.highlight {
  padding: 1rem;
  margin-top: 1rem;
  margin-bottom: 1rem;
  background-color: #f8f9fa;
  -ms-overflow-style: -ms-autohiding-scrollbar;
}
@media (min-width: 576px) {
  .highlight {
    padding: 1.5rem;
  }
}

.bd-content .highlight {
  margin-right: -0.75rem;
  margin-left: -0.75rem;
}
@media (min-width: 576px) {
  .bd-content .highlight {
    margin-right: 0;
    margin-left: 0;
  }
}

.highlight pre {
  padding: 0;
  margin-top: 0;
  margin-bottom: 0;
  background-color: transparent;
  border: 0;
}
.highlight pre code {
  font-size: inherit;
  color: #212529;
}

.bd-callout {
  padding: 1.25rem;
  margin-top: 1.25rem;
  margin-bottom: 1.25rem;
  border: 1px solid #e9ecef;
  border-left-width: 0.25rem;
  border-radius: 0.375rem;
}
.bd-callout h4 {
  margin-bottom: 0.25rem;
}
.bd-callout p:last-child {
  margin-bottom: 0;
}
.bd-callout code {
  border-radius: 0.375rem;
}
.bd-callout + .bd-callout {
  margin-top: -0.25rem;
}

.bd-callout-info {
  border-left-color: #5bc0de;
}
.bd-callout-info h4 {
  color: #5bc0de;
}

.bd-callout-warning {
  border-left-color: #f0ad4e;
}
.bd-callout-warning h4 {
  color: #f0ad4e;
}

.bd-callout-danger {
  border-left-color: #d9534f;
}
.bd-callout-danger h4 {
  color: #d9534f;
}

.swatch-blue {
  color: #fff;
  background-color: #0d6efd;
}

.swatch-indigo {
  color: #fff;
  background-color: #6610f2;
}

.swatch-purple {
  color: #fff;
  background-color: #6f42c1;
}

.swatch-pink {
  color: #fff;
  background-color: #d63384;
}

.swatch-red {
  color: #fff;
  background-color: #dc3545;
}

.swatch-orange {
  color: #000;
  background-color: #fd7e14;
}

.swatch-yellow {
  color: #000;
  background-color: #ffc107;
}

.swatch-green {
  color: #fff;
  background-color: #198754;
}

.swatch-teal {
  color: #000;
  background-color: #20c997;
}

.swatch-cyan {
  color: #000;
  background-color: #0dcaf0;
}

.swatch-black {
  color: #fff;
  background-color: #000;
}

.swatch-white {
  color: #000;
  background-color: #fff;
}

.swatch-gray {
  color: #fff;
  background-color: #6c757d;
}

.swatch-gray-dark {
  color: #fff;
  background-color: #343a40;
}

.swatch-100 {
  color: #000;
  background-color: #f8f9fa;
}

.swatch-200 {
  color: #000;
  background-color: #e9ecef;
}

.swatch-300 {
  color: #000;
  background-color: #dee2e6;
}

.swatch-400 {
  color: #000;
  background-color: #ced4da;
}

.swatch-500 {
  color: #000;
  background-color: #adb5bd;
}

.swatch-600 {
  color: #fff;
  background-color: #6c757d;
}

.swatch-700 {
  color: #fff;
  background-color: #495057;
}

.swatch-800 {
  color: #fff;
  background-color: #343a40;
}

.swatch-900 {
  color: #fff;
  background-color: #212529;
}

.bd-clipboard {
  position: relative;
  display: none;
  float: right;
}
.bd-clipboard + .highlight {
  margin-top: 0;
}
@media (min-width: 768px) {
  .bd-clipboard {
    display: block;
  }
}

.btn-clipboard {
  position: absolute;
  top: 0.5rem;
  right: 0.5rem;
  z-index: 10;
  display: block;
  padding: 0.25rem 0.5rem;
  font-size: 0.75em;
  color: #343a40;
  background-color: transparent;
  border: 0;
  border-radius: 0.375rem;
}
.btn-clipboard:hover {
  color: #fff;
  background-color: #0d6efd;
}

.bd-placeholder-img {
  font-size: 1.125rem;
  text-anchor: middle;
  user-select: none;
}

.bd-placeholder-img-lg {
  font-size: calc(1.475rem + 2.7vw);
}
@media (min-width: 1200px) {
  .bd-placeholder-img-lg {
    font-size: 3.5rem;
  }
}

/* NEW Background .chroma { background-color: #f0f0f0; } */
/* NEW Error .chroma .err { } */
/* NEW LineTableTD .chroma .lntd { vertical-align: top; padding: 0; margin: 0; border: 0; } */
/* NEW LineTable .chroma .lntable { border-spacing: 0; padding: 0; margin: 0; border: 0; width: auto; overflow: auto; display: block; } */
/* NEW LineHighlight .chroma .hl { display: block; width: 100%; background-color: #ffffcc; } */
/* NEW LineNumbersTable .chroma .lnt { margin-right: .4em; padding: 0 .4em; } */
/* NEW LineNumbers .chroma .ln { margin-right: .4em; padding: 0 .4em; } */
/* LiteralStringDelimiter */
.chroma .dl {
  color: #4070a0;
} /* NEW */
/* Comment */
.chroma .c {
  color: #727272;
}

/* CommentMultiline */
.chroma .c1 {
  color: #727272;
}

/* CommentHashbang */
.chroma .ch {
  font-style: italic;
  color: #60a0b0;
} /* NEW */
/* CommentMultiline */
.chroma .cm {
  color: #727272;
}

/* CommentPreproc */
.chroma .cp {
  color: #008085;
}

/* CommentPreprocFile */
.chroma .cpf {
  color: #007020;
} /* NEW */
/* CommentSpecial */
.chroma .cs {
  color: #727272;
}

/* GenericDeleted */
.chroma .gd {
  background-color: #fcc;
  border: 1px solid #c00;
}

/* GenericEmph */
.chroma .ge {
  font-style: italic;
}

/* GenericHeading */
.chroma .gh {
  color: #030;
}

/* GenericInserted */
.chroma .gi {
  background-color: #cfc;
  border: 1px solid #0c0;
}

/* GenericUnderline */
.chroma .gl {
  text-decoration: underline;
} /* NEW */
/* GenericOutput */
.chroma .go {
  color: #aaa;
}

/* GenericPrompt */
.chroma .gp {
  color: #009;
}

/* GenericError */
.chroma .gr {
  color: #f00;
}

/* GenericStrong */
.chroma .gs {
  font-weight: 700;
} /* NEW */
/* GenericTraceback */
.chroma .gt {
  color: #9c6;
}

/* GenericSubheading */
.chroma .gu {
  color: #030;
}

/* LiteralNumberIntegerLong */
.chroma .il {
  color: #c24f19;
}

/* Keyword */
.chroma .k {
  color: #069;
}

/* KeywordConstant */
.chroma .kc {
  color: #069;
}

/* KeywordDeclaration */
.chroma .kd {
  color: #069;
}

/* KeywordNamespace */
.chroma .kn {
  color: #069;
}

/* KeywordPseudo */
.chroma .kp {
  color: #069;
}

/* KeywordReserved */
.chroma .kr {
  color: #069;
}

/* KeywordType */
.chroma .kt {
  color: #078;
}

/* LiteralNumber */
.chroma .m {
  color: #c24f19;
}

/* LiteralNumberFloat */
.chroma .mf {
  color: #c24f19;
}

/* LiteralNumberBin */
.chroma .mb {
  color: #40a070;
} /* NEW */
/* LiteralNumberHex */
.chroma .mh {
  color: #c24f19;
}

/* LiteralNumberInteger */
.chroma .mi {
  color: #c24f19;
}

/* LiteralNumberOct */
.chroma .mo {
  color: #c24f19;
}

/* NameAttribute */
.chroma .na {
  color: #006ee0;
}

/* NameBuiltin */
.chroma .nb {
  color: #366;
}

/* NameClass */
.chroma .nc {
  color: #168174;
}

/* NameDecorator */
.chroma .nd {
  color: #6b62de;
}

/* NameException */
.chroma .ne {
  color: #c00;
}

/* NameFunction */
.chroma .nf {
  color: #b715f4;
}

/* NameEntity */
.chroma .ni {
  color: #727272;
}

/* NameLabel */
.chroma .nl {
  color: #6b62de;
}

/* NameNamespace */
.chroma .nn {
  color: #007ca5;
}

/* NameConstant */
.chroma .no {
  color: #360;
}

/* NameTag */
.chroma .nt {
  color: #2f6f9f;
}

/* NameVariable */
.chroma .nv {
  color: #033;
}

/* Operator */
.chroma .o {
  color: #555;
}

/* OperatorWord */
.chroma .ow {
  color: #000;
}

/* LiteralString */
.chroma .s {
  color: #d73038;
}

/* LiteralStringSingle */
.chroma .s1 {
  color: #c30;
}

/* LiteralStringDouble */
.chroma .s2 {
  color: #c30;
}

/* LiteralStringAffix */
.chroma .sa {
  color: #4070a0;
} /* NEW */
/* LiteralStringBacktick */
.chroma .sb {
  color: #c30;
}

/* LiteralStringChar */
.chroma .sc {
  color: #c30;
}

/* LiteralStringDoc */
.chroma .sd {
  font-style: italic;
  color: #c30;
}

/* LiteralStringEscape */
.chroma .se {
  color: #c30;
}

/* LiteralStringHeredoc */
.chroma .sh {
  color: #c30;
}

/* LiteralStringInterpol */
.chroma .si {
  color: #a00;
}

/* LiteralStringRegex */
.chroma .sr {
  color: #337e7e;
}

/* LiteralStringSymbol */
.chroma .ss {
  color: #fc3;
}

/* LiteralStringOther */
.chroma .sx {
  color: #c30;
}

/* TextWhitespace */
.chroma .w {
  color: #bbb;
}

.chroma .language-bash::before,
.chroma .language-sh::before {
  color: #009;
  content: "$ ";
  user-select: none;
}
.chroma .language-bash .m,
.chroma .language-sh .m {
  color: inherit;
}
.chroma .language-powershell::before {
  color: #009;
  content: "PM> ";
  user-select: none;
}

.anchorjs-link {
  font-weight: 400;
  color: rgba(13, 110, 253, 0.5);
  transition: color 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .anchorjs-link {
    transition: none;
  }
}
.anchorjs-link:hover {
  color: #0d6efd;
  text-decoration: none;
}

.path-admin-tool-componentlibrary #page.drawers {
  padding-left: 0;
  padding-right: 0;
}
.path-admin-tool-componentlibrary #page.drawers .main-inner {
  margin: 0;
  padding: 0;
  border-radius: 0;
}
.path-admin-tool-componentlibrary #page.drawers div[role=main] {
  padding: 0;
}
.path-admin-tool-componentlibrary #page-header,
.path-admin-tool-componentlibrary #maincontent,
.path-admin-tool-componentlibrary .secondarynavigation {
  display: none;
}
.path-admin-tool-componentlibrary #region-main {
  padding: 0;
  border: 0;
}

/*# sourceMappingURL=docs.css.map */
