<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'datafield_picture', language 'en', branch 'master'
 *
 * @package    datafield
 * @subpackage picture
 * @copyright  2011 onwards Eloy Lafuente (stronk7) {@link http://stronk7.com}
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['pluginname'] = 'Image';
$string['fieldtypelabel'] = 'Image';
$string['error_invalidparam1'] = 'Width in single view needs to be a numeric value.';
$string['error_invalidparam2'] = 'Height in single view needs to be a numeric value.';
$string['error_invalidparam4'] = 'Width in list view needs to be a numeric value.';
$string['error_invalidparam5'] = 'Height in list view needs to be a numeric value.';
$string['privacy:metadata'] = 'The Image field component doesn\'t store any personal data; it uses tables defined in mod_data.';
$string['sample'] = 'Image description placeholder';
