/** General Styles **/
.path-mod-glossary .glossarypost {
    width: 95%;
    border-collapse: separate;
    margin: 0 auto;
    text-align: left;
}

.path-mod-glossary .glossarypost.entrylist {
    border-width: 0;
}

.path-mod-glossary .glossarypost.continuous .concept {
    display: inline;
}

.path-mod-glossary .glossarypost .commands {
    width: 200px;
    white-space: nowrap;
}

.path-mod-glossary .glossarypost td.picture {
    width: 35px;
}

.path-mod-glossary .glossarypost .entrylowersection .icons {
    text-align: right;
    padding-right: 5px;
}

.path-mod-glossary .glossarypost .entrylowersection .ratings {
    text-align: right;
    padding-right: 5px;
    padding-bottom: 2px;
}

.path-mod-glossary .glossarypost .glossary-hidden-note {
    margin: 0 .45em;
}

.path-mod-glossary .glossarydisplay {
    margin-left: auto;
    margin-right: auto;
}

.path-mod-glossary .glossarydisplay .tabs {
    width: 100%;
    margin-bottom: 0;
}

.path-mod-glossary .glossarydisplay .tabs .side {
    border-style: none;
    border-width: 0;
    width: auto;
}

.path-mod-glossary .glossarydisplay .separator {
    width: 4px;
}

.path-mod-glossary table.glossarypopup {
    width: 95%;
}

.path-mod-glossary .entrybox,
.path-mod-glossary table.glossaryapproval,
.path-mod-glossary .glossarypost .entrylowersection table {
    width: 100%;
    margin-bottom: 0;
}

.glossary-activity-picture {
    float: left;
}

.glossary-activity-content {
    margin-left: 40px;
}

/** Page specific styles **/
#page-mod-glossary-view .glossarycontrol {
    float: right;
    text-align: right;
    white-space: nowrap;
    margin: 5px 0;
}

#page-mod-glossary-view table.glossarycategoryheader,
#page-mod-glossary-import table.glossaryimportexport {
    margin-left: auto;
    margin-right: auto;
}

#page-mod-glossary-view table.glossarycategoryheader {
    margin-bottom: 0;
}

#page-mod-glossary-view table.glossarycategoryheader th {
    padding: 0;
}

#page-mod-glossary-view td.glossarysearchbox label {
    display: inline-block;
}

@media (min-width: 768px) { /* Only apply this rule for larger devices (tablets, 768px and up). */
    #page-mod-glossary-showentry #page-content {
        min-width: 600px;
    }
}

#page-mod-glossary-print .mod-glossary-entrylist .mod-glossary-entry {
    vertical-align: top;
}

#page-mod-glossary-print .displayprinticon,
#page-mod-glossary-print .displaydate {
    text-align: right;
    font-size: 0.75em;
}

#page-mod-glossary-print .strong {
    font-weight: bold;
}

.path-mod-glossary .printicon {
    background: url([[pix:t/print]]) no-repeat scroll 2px center transparent;
    /*rtl:ignore*/
    padding-left: 20px;
}
