YUI.add('moodle-course-util-base', function (Y, NAME) {

/**
 * The Moodle.core_course.util classes provide course-related utility functions.
 *
 * TODO: remove this module as part of MDL-83627.
 *
 * @module moodle-course-util
 * @main
 */

Y.namespace('Moodle.core_course.util');

Y.log(
    'YUI Moodle.core_course.util is deprecated. Please, add support_components to your course format.',
    'warn',
    'moodle-course-coursebase'
);

/**
 * A collection of general utility functions for use in course.
 *
 * @class Moodle.core_course.util
 * @static
 */


}, '@VERSION@');
