{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template aiplacement_courseassist/response

    Template to display reponse content for AI placement.

    Context variables required for this template:
    * content - Content to display
    * heading - The heading to display
    * action - The action being performed

    Example context (json):
    {
        "content": "<p>Content to display</p>",
        "heading": "AI Explain",
        "action": "explain_text"
    }
}}
<section class="course-assist-response card mb-3" data-action-performed="{{action}}">
    <div class="card-body p-3">
        <h3 class="course-assist-response-header h5 card-title d-inline">
            <span class="ai-course-assist-sparkles-icon">
                {{#pix}} sparkles, aiplacement_courseassist {{/pix}}
            </span>
            {{heading}}
        </h3>
        <div class="card-text content mt-3">
            <div id="course-assist-response-content-{{uniqid}}" class="course-assist-response-content mb-3">
                {{{content}}}
            </div>
            <div class="course-assist-response-watermark">
                <small class="text-muted">
                    <span class="ai-course-assist-sparkles-icon">
                        {{#pix}} sparkles, aiplacement_courseassist {{/pix}}
                    </span>
                    {{#str}} contentwatermark, core_ai {{/str}}
                </small>
            </div>
            <div class="course-assist-controls d-block pt-3">
                <button class="btn btn-sm btn-outline-secondary" data-action="regenerate">
                    {{#pix}} a/refresh, core {{/pix}}
                    {{#str}} regenerate, aiplacement_courseassist {{/str}}
                </button>
                <button class="btn btn-sm btn-outline-secondary" data-action="copytoclipboard" data-clipboard-target="#course-assist-response-content-{{uniqid}}">
                    {{#pix}} e/copy, core {{/pix}}
                    {{#str}} copy, aiplacement_courseassist {{/str}}
                </button>
            </div>
        </div>
    </div>
</section>
