.que.multianswer .feedbackspan {
    display: block;
    max-width: 70%;
    background: #fff3bf;
    padding: 0.5em;
    margin-top: 1em;
    box-shadow: 0.5em 0.5em 1em #000;
}

body.ie6 .que.multianswer .feedbackspan,
body.ie7 .que.multianswer .feedbackspan,
body.ie8 .que.multianswer .feedbackspan,
body.ie9 .que.multianswer .feedbackspan {
    width: 70%;
}

.que.multianswer .answer .specificfeedback {
    display: inline;
    padding: 0 0.7em;
    background: #fff3bf;
}

.que.multianswer .answer .specificfeedback * {
    display: inline;
    background: #fff3bf;
}

.que.multianswer .answer .specificfeedback script {
    display: none;
}

.que.multianswer .answer div.r0,
.que.multianswer .answer div.r1 {
    padding: 0.3em;
}

.que.multianswer table.answer {
    margin-bottom: 0;
    width: 100%;
}
