{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_courseformat/local/content/bulkedittoggler

    Displays the bulk actions button in the page header.

    Example context (json):
    {
    }
}}
<button
    id="bulk-enable-{{uniqid}}"
    class="bulkEnable btn text-primary text-nowrap"
    data-for="enableBulk" data-coursename="{{coursename}}"
    {{#sectiontitle}} data-sectiontitle="{{.}}" {{/sectiontitle}}
    {{#sectionname}} data-sectionname="{{.}}" {{/sectionname}}
>
    {{#str}} bulkedit, core_courseformat {{/str}} {{#pix}} i/bulk_edit, core {{/pix}}
</button>
{{#js}}
require(['core_courseformat/local/content/bulkedittoggler'], function(component) {
    component.init('#bulk-enable-{{uniqid}}');
});
{{/js}}
