YUI.add("moodle-mod_quiz-quizbase",function(e,t){var i=function(){i.superclass.constructor.apply(this,arguments)};e.extend(i,e.Base,{registermodules:[],register_module:function(e){return this.registermodules.push(e),this},invoke_function:function(e,t){for(var i in this.registermodules)e in this.registermodules[i]&&this.registermodules[i][e](t);return this}},{NAME:"mod_quiz-quizbase",ATTRS:{}}),M.mod_quiz=M.mod_quiz||{},M.mod_quiz.quizbase=M.mod_quiz.quizbase||new i,M.mod_quiz.edit=M.mod_quiz.edit||{},M.mod_quiz.edit.swap_sections=function(e,t,i){var s="mod-quiz-edit-content",o="section_add_menus",s=e.Node.all("."+s+" li.section");s.item(t).one("."+o).swap(s.item(i).one("."+o))},M.mod_quiz.edit.process_sections=function(a,e,t,i,s){var o,n,u,r,d,m="sectionname",c=".left .section-handle .icon";if("move"===t.action){for(s<i&&(o=s,s=i,i=o),d=i;d<=s;d++)e.item(d).one("."+m).setContent(t.sectiontitles[d]),r=(u=(n=e.item(d).one(c)).getAttribute("alt")).lastIndexOf(" "),r=u.substr(0,r+1)+d,n.setAttribute("alt",r),n.setAttribute("title",r),e.item(d).removeClass("current");-1!==t.current&&e.item(t.current).addClass("current")}}},"@VERSION@",{requires:["base","node"]});