{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_componentlibrary/examples/toggle/example

    Template for toggle element used as a block example

    Example context (json):
    {
    }
}}
<div id="toggle-example">
    <h1>Toggle block</h1>
    <span>This is a simple example using toggle template as a block.</span>
    {{< core/toggle }}
        {{$id}}example-toggle-3{{/id}}
        {{$extraclasses}}mt-3{{/extraclasses}}
        {{$attributes}}
            data-action="toggle-status"
            data-action-id="3"
            checked
            disabled
        {{/attributes}}
        {{$label}}Enable/disable status{{/label}}
        {{$labelclasses}}visually-hidden{{/labelclasses}}
    {{/ core/toggle }}
</div>
