{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template qbank_viewcreator/version_info

    Example context (json):
    {
        "addquestiondata": [
          {
                "versionnumber": 1,
                "historyurl": "www.example.com/question/bank/history/history.php",
                "createdby": "Admin User on Wednesday, 20 October 2021, 5:33 AM"
          }
        ]
    }
}}
<div class="question-version-number">
    <span> {{versionnumber}}. </span> <a href="{{{historyurl}}}"> {{#pix}} t/log, core {{/pix}} {{#str}}history, qbank_viewcreator{{/str}} </a>
</div>
{{#createdby}}
    <div class="question-creator-info">
        <span>{{{createdby}}}</span>
    </div>
{{/createdby}}
