/******  List View CSS ******/

#imagegallery-list img.list_picture {
    width: 100%;
    height: 15vw;
    object-fit: cover;
}

#imagegallery-list .card {
    margin-top: 30px;
    margin-bottom: 15px;
}

.imagegallery-listentry .card-title {
    font-size: larger;
    text-overflow: ellipsis;
    white-space: nowrap;
    overflow: hidden;
}

/*****  Single View CSS *****/

.imagegallery-single-body .col {
    padding: 10px 15px;
}

.imagegallery-single img.list_picture {
    max-width: 100%;
    height: auto;
}
