{{!
    This file is part of Moodle - http://moodle.org/
    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_course/completion_action_bar

    Actions bar in the course completion pages.

    Context variables required for this template:
    * navigation - The data object containing the required properties to render core/tertiary_navigation_selector.

    Example context (json):
    {
        "navigation": {
            "name":"coursecompletionnavigation",
            "label": "Course completion tertiary navigation",
            "value": "\/course\/completion.php?id=4",
            "baseid": "select-menu64a40e3ea86ed8",
            "selectedoption": "Grader report",
            "options": [
                {
                    "id": "select-menu-option1",
                    "name": "Course completion settings",
                    "value": "/course/completion.php?id=4",
                    "selected": true
                }
            ],
            "labelattributes":[
                {
                    "name":"class",
                    "value":"visually-hidden"
                }
            ]
        }
    }
}}
<div class="container-fluid tertiary-navigation border-bottom" id="tertiary-navigation">
    <div class="d-flex">
        {{#navigation}}
            <div class="navitem">
                {{>core/tertiary_navigation_selector}}
            </div>
        {{/navigation}}
    </div>
</div>
