<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * The english language pack used in this profile field type.
 *
 * @package profilefield_datetime
 * @copyright 2010 Mark Nelson <markn@moodle.com>
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
 */

$string['currentdatedefault'] = 'Check to use current date as default';
$string['defaultdate'] = 'Default date';
$string['endyear'] = 'End year';
$string['notset'] = 'Not set';
$string['pluginname'] = 'Date/Time';
$string['privacy:metadata:profilefield_datetime:userid'] = 'The ID of the user whose data is stored by the Date/time user profile field';
$string['privacy:metadata:profilefield_datetime:fieldid'] = 'The ID of the profile field';
$string['privacy:metadata:profilefield_datetime:data'] = 'Date/time user profile field user data';
$string['privacy:metadata:profilefield_datetime:dataformat'] = 'The format of Date/time user profile field user data';
$string['privacy:metadata:profilefield_datetime:tableexplanation'] = 'Additional profile data';
$string['specifydatedefault'] = 'or specify a date';
$string['startyearafterend'] = 'The start year can\'t occur after the end year';
$string['startyear'] = 'Start year';
$string['wanttime'] = 'Include time?';
