{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_message/message_popover

    This template will render the message popover for the navigation bar.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * All data attributes are required

    Context variables required for this template:
    * userid The logged in user id
    * urls The URLs for the popover

    Example context (json):
    {
        "unreadcount": 3
    }

}}
<div class="popover-region collapsed" data-region="popover-region-messages">
    <a
        id="message-drawer-toggle-{{uniqid}}"
        class="nav-link popover-region-toggle position-relative icon-no-margin"
        href="#"
        aria-label="{{#str}} togglemessagemenu, message {{/str}}"
        title="{{#str}} togglemessagemenu, message {{/str}}"
        role="button"
        aria-expanded="false"
        aria-describedby="unread-messages-count-{{uniqid}}"
    >
        {{#pix}} t/message, core {{/pix}}
        <div
            class="count-container {{^unreadcount}}hidden{{/unreadcount}}"
            data-region="count-container"
        >
            <span aria-hidden="true">{{unreadcount}}</span>
            <span class="visually-hidden" id="unread-messages-count-{{uniqid}}">{{#str}} unreadconversations, core_message, {{unreadcount}} {{/str}}</span>
        </div>
    </a>
    {{> core_message/message_jumpto }}
</div>

{{#js}}
require(
[
    'jquery',
    'core_message/message_popover'
],
function(
    $,
    Popover
) {
    var toggle = $('#message-drawer-toggle-{{uniqid}}');
    Popover.init(toggle);
});
{{/js}}
