{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template auth_lti/local/ltiadvantage/account_binding_complete

    Template which displays the confirmation after the user has either signed in and has their account linked, or
    has had an account automatically provisioned and linked.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * notification
    * returnurl

    Example context (json):
    {
        "notification": {
            "message": "Your account was successfully linked!",
            "extraclasses": "",
            "announce": true,
            "closebutton": false,
            "issuccess": true,
            "isinfo": false,
            "iswarning": false,
            "iserror": false
        },
        "returnurl": "https://your.site/enrol/lti/launch_deeplink.php?id=123abc"
    }
}}
<div id="lti_adv_account_binding_complete">
    {{#notification}}
        {{> core/notification}}
    {{/notification}}
    <a class="btn btn-primary" href="{{returnurl}}">{{#str}}continue, core{{/str}}</a>
</div>
