{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_forum/local/grades/local/grader/status

    Classes required for JS:
    * none

    Data attributes required for JS:
    *

    Context variables required for this template:
    *

    Example context (json):
    {
        "status": "Graded",
        "index": 1,
        "total": 10
    }
}}
<h2 class="fw-bold h5 mb-0">
    {{#status}}
        {{#str}}graded, forum{{/str}}
    {{/status}}
    {{^status}}
        {{#str}}notgraded, forum{{/str}}
    {{/status}}
</h2>
<div>{{#str}} indexoutoftotal, mod_forum, {"index": "{{index}}", "total":"{{total}}"}{{/str}}</div>
