{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_quiz/update_filter_condition_form

    From to add random questions.

    Example context (json):
    {
        "questionbank": "<pre>Question bank HTML goes here</pre>",
        "returnurl": "https://example.com",
        "cmid": "1234",
        "slotid": "5678"
    }
}}
<div>
    <!-- Question bank -->
    <div id="questionbank_container">
        {{{questionbank}}}
    </div>


    <!-- submit buttons -->
    <form action="{{returnurl}}" method="POST" id="update_filter_condition_form" data-cmid="{{cmid}}" data-slotid="{{slotid}}">
        <input type="hidden" name="message" value="">
        <input type="submit" class="btn btn-primary" name="update" value="{{#str}} updatefilterconditon, mod_quiz {{/str}}">
        <input type="submit" class="btn btn-secondary" name="cancel" value="{{#str}} cancel {{/str}}">
    </form>
</div>
