{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_forum/forum_search_results

    Template to render results for the forum post search.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Example context (json):
    {
        "posts": [
            {
                "id": 1,
                "subject": "Re: Test discussion",
                "message": "Reply to post by Sam Student",
                "author": {
                    "id": 2,
                    "fullname": "Sam Student"
                },
                "discussionid": 3,
                "capabilities": {
                    "view": true,
                    "edit": true,
                    "delete": true,
                    "split": true,
                    "reply": true
                },
                "html": {
                    "authorsubheading": "by <a href=\"#\">Sam Student<\/a> - <time>Friday, 7 February 2020, 11:55 AM<\/time>"
                },
                "forumname": "Forum",
                "discussionname": "<span class=\"highlight\">Test<\/span> discussion",
                "showdiscussionname": true
            }
        ]
    }
}}
{{< mod_forum/forum_posts_with_context_links}}
    {{$subjectheading}}
        <h4 {{!
            }}class="h6 fw-bold mb-0" {{!
            }}data-region-content="forum-post-core-subject" {{!
            }}data-reply-subject="{{replysubject}}" {{!
            }}>
            {{$subject}}
                {{> mod_forum/forum_post_subject_with_context_links }}
            {{/subject}}
        </h4>
    {{/subjectheading}}
{{/ mod_forum/forum_posts_with_context_links}}
