@import 'variables';

.h5peditor {
  .h5p-editing-image-button {
    background: linear-gradient(to bottom, #fff 0, #f2f2f2 100%);
    font-size: $font-size-small;
    color: $black;
    line-height: $form-item-height-small;
    padding-right: $padding;
    margin-right: 0.5em;
    height: 30px;
    padding-left: 0;
    font-weight: normal;

    &:hover:not([disabled]) {
      background: linear-gradient(to bottom, #fff 0, #d0d0d1 100%);
      border-color: #999;
    }
  }

  .h5p-editing-image-button:before {
    font-family: $icon-font-family;
    content: "\e900";
    color: #666;
    padding-right: 0.3em;
    padding-left: 0.45em;
    vertical-align: middle;
    font-size: 1.5em;
    line-height: 0.9;
  }

  .h5p-editing-image-button.loading:before {
    content: "\e901";
  }

  .h5p-editing-image-button.hidden,
  .h5p-copyright-button.hidden {
    display: none !important;
  }

  .ui-dialog .h5p-editing-image-button,
  .ui-dialog .h5p-copyright-button {
    padding-left: 0.5em;
  }
}
