{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_moodlenet/view-cards

    This template renders the cards view for the MoodleNet tool.

    Example context (json):
    {
        "courses": [
            {
                "name": "Assignment due 1",
                "viewurl": "https://moodlesite/course/view.php?id=2",
                "courseimage": "https://moodlesite/pluginfile/123/course/overviewfiles/123.jpg",
                "fullname": "course 3",
                "coursecategory": "Category 1",
                "visible": true
            }
        ]
    }
}}

{{< core_course/coursecards }}
    {{$coursename}}
        <span class="multiline">
            {{#shortentext}}50, {{{fullname}}} {{/shortentext}}
        </span>
    {{/coursename}}
    {{$coursecategory}}
        <span class="visually-hidden">
            {{#str}}aria:coursecategory, core_course{{/str}}
        </span>
        <span class="categoryname text-truncate">
            {{{coursecategory}}}
        </span>
    {{/coursecategory}}
    {{$divider}}
        <div class="px-1">|</div>
    {{/divider}}
{{/ core_course/coursecards }}
