{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_courseformat/local/activitychooser/error

    Chooser error template.

    Variables required for this template:
    * errormessage - The error message

    Example context (json):
    {
        "errormessage": "Error"
    }
}}
<div class="p-2 px-sm-5 py-sm-4">
    <div class="alert alert-danger" role="alert">
        <h5 class="alert-heading">
            <i class="fa fa-exclamation-circle fa-fw text-danger"></i>
            {{#str}} error, error {{/str}}
        </h5>
        <hr>
        <p class="text-break">{{{errormessage}}}</p>
    </div>
</div>
