<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Language strings.
 *
 * @package availability_group
 * @copyright 2014 The Open University
 * @license http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['anygroup'] = '(Any group)';
$string['description'] = 'Allow only students who belong to a specified group, or all groups.';
$string['missing'] = '(Missing group)';
$string['pluginname'] = 'Restriction by group';
$string['error_selectgroup'] = 'You must select a group.';
$string['requires_anygroup'] = 'You belong to any group';
$string['requires_group'] = 'You belong to <strong>{$a}</strong>';
$string['requires_notanygroup'] = 'You do not belong to any group';
$string['requires_notgroup'] = 'You do not belong to <strong>{$a}</strong>';
$string['title'] = 'Group';
$string['privacy:metadata'] = 'The Restriction by group plugin does not store any personal data.';
