{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_message/message_drawer_view_overview_section_favourites

    This template will render the favourites section of the overview page
    in the message drawer.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * All data attributes are required

    Context variables required for this template:
    * userid The logged in user id
    * urls The URLs for the popover

    Example context (json):
    {}

}}

{{< core_message/message_drawer_view_overview_section }}
    {{$region}}view-overview-favourites{{/region}}
    {{$title}}{{#str}} favourites {{/str}}{{/title}}
    {{$placeholder}}
        <div class="text-center py-2">{{> core/loading }}</div>
    {{/placeholder}}
    {{$emptymessage}}
        <p class="text-muted mt-2">{{#str}} nofavourites, core_message {{/str}}</p>
    {{/emptymessage}}
{{/ core_message/message_drawer_view_overview_section }}