{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_admin/setting_heading

    Admin heading setting template.

    Context variables required for this template:
    * title - Title text
    * description - Description containing descriptionformatted

    Example context (json):
    {
        "title": "Favourite cheeses",
        "description": true,
        "descriptionformatted": "Stilton, Cheddar, Edam"
    }
}}
{{#title}}
    <h3 class="main">{{{title}}}</h3>
{{/title}}
{{#description}}
    <div class="box generalbox formsettingheading">{{{descriptionformatted}}}</div>
{{/description}}
