//
// Footer
//

.bd-footer {
    @include font-size(.875rem);
    color: #63707c;

    a {
        font-weight: 600;
        color: $gray-700;

        &:hover,
        &:focus {
            color: $link-color;
        }
    }

    p {
        margin-bottom: 0;
    }
}

.bd-footer-links {
    padding-left: 0;
    margin-bottom: 1rem;

    li {
        display: inline-block;

        + li {
            margin-left: 1rem;
        }
    }
}
