{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_lp/linked_courses_summary

    Moodle template for the list of linked courses to a competency

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * courses array
      * viewurl
      * fullname
      * shortname

    Example context (json):
    { "courses":
        [
            {
                "viewurl": "http://example.com",
                "fullname": "Course 1",
                "shortname": "C1"
            }
        ]
    }
}}

<p>
{{#str}}coursesusingthiscompetency, tool_lp{{/str}}
</p>
<ol>
{{#courses}}
    <li><a href="{{viewurl}}">{{{fullname}}} {{{shortname}}}</a></li>
{{/courses}}
{{^courses}}
    {{#str}}nolinkedcourses, tool_lp{{/str}}
{{/courses}}
</ol>
