{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more comments.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_dataprivacy/data_request_modal

    Data request details

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * none

    Example context (json):
    {
        "approvedeny": true,
        "allowfiltering": true,
        "canmarkcomplete": true,
        "title": "Data request modal title"
    }
}}
{{< core/modal }}
    {{$footer}}
        {{#approvedeny}}
            {{#allowfiltering}}
                <button type="button" class="btn btn-primary" data-action="approve-selected-courses">{{#str}} approveselectedcourses, tool_dataprivacy {{/str}}</button>
            {{/allowfiltering}}
            <button type="button" class="btn btn-primary" data-action="approve">{{#str}} approve, tool_dataprivacy {{/str}}</button>
            <button type="button" class="btn btn-secondary" data-action="deny">{{#str}} deny, tool_dataprivacy {{/str}}</button>
        {{/approvedeny}}
        {{#canmarkcomplete}}
            <button type="button" class="btn btn-primary" data-action="complete">{{#str}} markcomplete, tool_dataprivacy {{/str}}</button>
        {{/canmarkcomplete}}
    {{/footer}}
{{/ core/modal }}
