<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'core_grades', language 'en', branch 'MOODLE_20_STABLE'
 *
 * @package   core_grades
 * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['activities'] = 'Activities';
$string['addcategory'] = 'Add category';
$string['addcategoryerror'] = 'Could not add category.';
$string['addexceptionerror'] = 'Error occurred while adding exception for userid:gradeitem';
$string['addfeedback'] = 'Add feedback';
$string['addgradeletter'] = 'Add a grade letter';
$string['addidnumbers'] = 'Add ID numbers';
$string['additem'] = 'Add grade item';
$string['additionalfeedback'] = 'Additional feedback';
$string['addoutcome'] = 'Add an outcome';
$string['addoutcomeitem'] = 'Add outcome item';
$string['addscale'] = 'Add a scale';
$string['adjustedweight'] = 'Weight adjusted';
$string['aggregateextracreditmean'] = 'Mean of grades (with extra credits)';
$string['aggregatemax'] = 'Highest grade';
$string['aggregatemean'] = 'Mean of grades';
$string['aggregatemedian'] = 'Median of grades';
$string['aggregatemin'] = 'Lowest grade';
$string['aggregatemode'] = 'Mode of grades';
$string['aggregatenotonlygraded'] = 'Include empty grades';
$string['aggregateonlygraded'] = 'Exclude empty grades';
$string['aggregateonlygraded_help'] = 'An empty grade is a grade which is missing from the gradebook. It may be from an assignment submission which has not yet been graded or from a quiz which has not yet been attempted etc.

This setting determines whether empty grades are not included in the aggregation or are counted as minimal grades, for example 0 for an assignment graded between 0 and 100.';
$string['aggregateoutcomes'] = 'Include outcomes in aggregation';
$string['aggregateoutcomes_help'] = 'If enabled, outcomes are included in the aggregation. This may result in an unexpected category total.';
$string['aggregatesonly'] = 'Show totals only';
$string['aggregatesum'] = 'Natural';
$string['aggregateweightedmean'] = 'Weighted mean of grades';
$string['aggregateweightedmean2'] = 'Simple weighted mean of grades';
$string['aggregation'] = 'Aggregation';
$string['aggregation_help'] = 'The aggregation determines how grades in a category are combined, such as

* Mean of grades - The sum of all grades divided by the total number of grades
* Median of grades - The middle grade when grades are arranged in order of size
* Lowest grade
* Highest grade
* Mode of grades - The grade that occurs the most frequently
* Natural - The sum of all grade values scaled by weight';
$string['aggregationhintnovalue'] = '( Empty )';
$string['aggregationhintdropped'] = '( Dropped )';
$string['aggregationhintexcluded'] = '( Excluded )';
$string['aggregationhintextra'] = '( Extra credit )';
$string['aggregation_link'] = 'grade/aggregation';
$string['aggregationcoef'] = 'Aggregation coefficient';
$string['aggregationcoefextra'] = 'Extra credit'; // For the header of the table on the 'Gradebook setup' page.
$string['aggregationcoefextra_help'] = 'If the aggregation is \'Natural\' or \'Simple weighted mean\' and the extra credit checkbox is ticked, the grade item\'s maximum grade is not added to the category\'s maximum grade. This will result in the possibility of achieving the maximum grade in the category without having the maximum grade in all the grade items. If the site administrator has enabled grades over the maximum, there might be grades over the maximum.

If the aggregation is \'Mean of grades (with extra credits)\' and the extra credit is set to a value greater than zero, the extra credit is the factor by which the grade is multiplied before adding it to the total after the computation of the mean.';
$string['aggregationcoefextra_link'] = 'grade/aggregation';
$string['aggregationcoefextrasum'] = 'Extra credit'; // For the form with checkboxes: Natural or Simple weighted mean.
$string['aggregationcoefextrasumabbr'] = '+';
$string['aggregationcoefextrasum_help'] = 'If the extra credit checkbox is ticked, the grade item\'s maximum grade is not added to the category\'s maximum grade, resulting in the possibility of achieving the maximum grade (or grades over the maximum if enabled by the site administrator) in the category without having the maximum grade in all the grade items.';
$string['aggregationcoefextrasum_link'] = 'grade/aggregation';
$string['aggregationcoefextraweight'] = 'Extra credit weight'; // For the form with input: Mean of grades (with extra credits) only.
$string['aggregationcoefextraweight_help'] = 'If the extra credit weight is set to a value greater than zero, the grade acts as extra credit during aggregation. The number is the factor by which the grade is multiplied before adding it to the total for the computation of the mean.';
$string['aggregationcoefextraweight_link'] = 'grade/aggregation';
$string['aggregationcoefweight'] = 'Item weight';
$string['aggregationcoefweight_help'] = 'The item weight is used in the category aggregation to influence the importance of the item compared with other grade items in the same category.';
$string['aggregationcoefweight_link'] = 'grade/aggregation';
$string['aggregationofa'] = 'Aggregation of {$a}';
$string['aggregationposition'] = 'Aggregation position';
$string['aggregationposition_help'] = 'This setting determines whether the category and course total columns are displayed first or last in the gradebook reports.';
$string['aggregationsvisible'] = 'Available aggregation types';
$string['aggregationsvisiblehelp'] = 'Select all aggregation types that should be available. Hold down the Ctrl key to select multiple items.';
$string['allgrades'] = 'All grades by category';
$string['allstudents'] = 'All students';
$string['allusers'] = 'All users';
$string['autosort'] = 'Auto-sort';
$string['availableidnumbers'] = 'Available ID numbers';
$string['average'] = 'Average';
$string['averagesdecimalpoints'] = 'Decimals in column averages';
$string['averagesdecimalpoints_help'] = 'This setting determines the number of decimal places to display for each average or whether the overall decimal places setting for the category or grade item is used (inherit).';
$string['averagesdisplaytype'] = 'Column averages display type';
$string['averagesdisplaytype_help'] = 'This setting determines whether the average (mean) is displayed as real grades, percentages or letters, or whether the display type for the category or grade item is used (inherit).';
$string['backupwithoutgradebook'] = 'Backup does not contain gradebook configuration';
$string['badgrade'] = 'Supplied grade is invalid';
$string['badlyformattedscale'] = 'Please enter a comma-separated list of values (at least two values required).';
$string['baduser'] = 'Supplied user is invalid';
$string['bonuspoints'] = 'Bonus points';
$string['bulkcheckboxes'] = 'Bulk checkboxes';
$string['calculatedgrade'] = 'Calculated grade';
$string['calculation'] = 'Calculation';
$string['calculation_help'] = 'A grade calculation is a formula used to determine grades. The formula should start with an equal (=) sign and may use common mathematical operators, such as max, min and sum. If desired, other grade items may be included in the calculation by typing the ID numbers in double square brackets.';
$string['calculation_link'] = 'grade/calculation';
$string['calculationadd'] = 'Add calculation';
$string['calculationedit'] = 'Edit calculation';
$string['calculationsaved'] = 'Calculation saved';
$string['calculationview'] = 'View calculation';
$string['calculationwarning'] = 'This category uses a grade calculation for the category total rather than an aggregation.';
$string['cannotaccessgroup'] = 'Can not access grades of selected group, sorry.';
$string['categories'] = 'Categories';
$string['category'] = 'Category';
$string['categoryedit'] = 'Edit category';
$string['categoryname'] = 'Category name';
$string['categorytotal'] = 'Category total';
$string['categorytotalname'] = 'Category total name';
$string['categorytotalfull'] = '{$a->category} total';
$string['selectuserlink'] = 'Select a user';
$string['collapsecriterion'] = 'Collapse criterion';
$string['compact'] = 'Compact';
$string['componentcontrolsvisibility'] = 'Whether this grade item is hidden is controlled by the activity settings.';
$string['contract'] = 'Contract category';
$string['contributiontocoursetotal'] = 'Contribution to course total';
$string['courseavg'] = 'Course average';
$string['coursegradecategory'] = 'Course grade category';
$string['coursegradedisplaytype'] = 'Course grade display type';
$string['coursegradedisplayupdated'] = 'The course grade display type has been updated.';
$string['coursegradesettings'] = 'Course grade settings';
$string['coursename'] = 'Course name';
$string['coursesiamtaking'] = 'Courses I am taking';
$string['coursesiamteaching'] = 'Courses I am teaching';
$string['coursescales'] = 'Course scales';
$string['coursesettings'] = 'Course settings';
$string['coursesettingsexplanation'] = 'Course grade settings determine how the gradebook appears for all participants in the course.';
$string['coursetotal'] = 'Course total';
$string['createcategory'] = 'Create category';
$string['createcategoryerror'] = 'Could not create a new category';
$string['creatinggradebooksettings'] = 'Creating gradebook settings';
$string['csv'] = 'CSV';
$string['currentgrade'] = 'Current grade';
$string['currentparentaggregation'] = 'Current parent aggregation';
$string['curveto'] = 'Curve to';
$string['decimalpoints'] = 'Overall decimal places';
$string['decimalpoints_help'] = 'This setting determines the number of decimal places to display for each grade. It has no effect on grade calculations, which are made with an accuracy of 5 decimal places.';
$string['default'] = 'Default';
$string['defaultprev'] = 'Default ({$a})';
$string['deletecategory'] = 'Delete category';
$string['deletescale'] = 'Delete scale';
$string['disablegradehistory'] = 'Disable grade history';
$string['disablegradehistory_help'] = 'Disable history tracking of changes in grades related tables. This may speed up the server a little and conserve space in database.';
$string['displaylettergrade'] = 'Display letter grades';
$string['displaypercent'] = 'Display percents';
$string['displaypoints'] = 'Display points';
$string['displayweighted'] = 'Display weighted grades';
$string['droplow'] = 'Drop the lowest';
$string['droplow_help'] = 'This setting enables a specified number of the lowest grades to be excluded from the aggregation.';
$string['droplowestvalue'] = 'Set drop lowest grade value';
$string['dropped'] = 'Dropped';
$string['droplowestvalues'] = 'Drop {$a} lowest values';
$string['dropxlowest'] = 'Drop X lowest';
$string['dropxlowestwarning'] = 'Note: If you use drop x lowest the grading assumes that all items in the category have the same point value. If point values differ results will be unpredictable';
$string['duplicatedgradeitem'] = '{$a} (copy)';
$string['duplicatescale'] = 'Duplicate scale';
$string['edit'] = 'Edit';
$string['editcalculation'] = 'Edit calculation';
$string['editfeedback'] = 'Edit feedback';
$string['editgrade'] = 'Edit grade';
$string['editgradeletters'] = 'Edit grade letters';
$string['editoutcome'] = 'Edit outcome';
$string['editoutcomes'] = 'Edit outcomes';
$string['editscale'] = 'Edit scale';
$string['edittree'] = 'Setup';
$string['enableoutcomes'] = 'Enable outcomes';
$string['enableoutcomes_help'] = 'If enabled, grade items may be graded using one or more scales tied to outcome statements.';
$string['encoding'] = 'Encoding';
$string['encoding_help'] = 'Select the character encoding used for the data. (The standard encoding is UTF-8.) If the wrong encoding is selected by mistake, it will be noticeable when previewing the data for import.';
$string['errorcalculationnoequal'] = 'Formula must start with equal sign (=1+2)';
$string['errorcalculationunknown'] = 'Invalid formula';
$string['errorcalculationbroken'] = 'Error in the calculation of grade item {$a}.';
$string['errorgradevaluenonnumeric'] = 'Received non-numeric for low or high grade for';
$string['errornegativepenalty'] = 'Negative value received from penalty aggregation for grade ID {$a}.';
$string['errornocalculationallowed'] = 'Calculations are not allowed for this item';
$string['errornocategorisedid'] = 'Could not get an uncategorised id!';
$string['errornocourse'] = 'Could not get course information';
$string['erroroverweight'] = 'Weight total exceeds 100%.';
$string['errorreprintheadersnonnumeric'] = 'Received non-numeric value for reprint-headers';
$string['errorsavegrade'] = 'Could not save grade, sorry.';
$string['errorsettinggrade'] = 'Error saving "{$a->itemname}" grade for userid {$a->userid}';
$string['errorunderweight'] = 'Weight total is less than 100%.';
$string['errorupdatinggradecategoryaggregateonlygraded'] = 'Error updating the "Aggregate only non-empty grades" setting of grade category ID {$a->id}';
$string['errorupdatinggradecategoryaggregateoutcomes'] = 'Error updating the "Include outcomes in aggregation" setting of grade category ID {$a->id}';
$string['errorupdatinggradecategoryaggregation'] = 'Error updating the aggregation type of grade category ID {$a->id}';
$string['errorupdatinggradeitemaggregationcoef'] = 'Error updating the aggregation coefficient (weight or extra credit) of grade item ID {$a->id}';
$string['eventgradedeleted'] = 'Grade deleted';
$string['eventgradeexported'] = 'Grade exported';
$string['eventgradeitemcreated'] = 'Grade item created';
$string['eventgradeitemdeleted'] = 'Grade item deleted';
$string['eventgradeitemupdated'] = 'Grade item updated';
$string['eventgradelettercreated'] = 'Grade letter created';
$string['eventgradeletterdeleted'] = 'Grade letter deleted';
$string['eventgradeletterupdated'] = 'Grade letter updated';
$string['eventgradeviewed'] = 'Grades were viewed in the gradebook';
$string['eventscalecreated'] = 'Scale created';
$string['eventscaledeleted'] = 'Scale deleted';
$string['eventscaleupdated'] = 'Scale updated';
$string['eventusergraded'] = 'User graded';
$string['excluded'] = 'Excluded';
$string['excluded_help'] = 'If ticked, the grade will not be included in any aggregation.';
$string['expand'] = 'Expand category';
$string['expandcriterion'] = 'Expand criterion';
$string['export'] = 'Export';
$string['exportas'] = 'Export as';
$string['exportalloutcomes'] = 'Export all outcomes';
$string['exportfeedback'] = 'Include feedback in export';
$string['exportfeedback_desc'] = 'This can be overridden during export.';
$string['exportformatoptions'] = 'Export format options';
$string['exportplugins'] = 'Export plugins';
$string['exportsettings'] = 'Export settings';
$string['exportonlyactive'] = 'Exclude suspended users';
$string['exportonlyactive_help'] = 'Only include students in the export whose enrolment is active and has not been suspended';
$string['exportto'] = 'Export to';
$string['externalurl'] = 'External URL';
$string['externalurl_desc'] = 'If an external gradebook is used, the URL should be specified here.';
$string['extracreditvalue'] = 'Extra credit value for {$a}';
$string['extracreditwarning'] = 'Note: Setting all items for a category to extra credit will effectively remove them from the grade calculation. Since there will be no point total';
$string['fail'] = 'Fail';
$string['feedback'] = 'Feedback';
$string['feedback_help'] = 'This box enables any comments about the grade to be added.';
$string['feedbackadd'] = 'Add feedback';
$string['feedbackedit'] = 'Edit feedback';
$string['feedbackfiles'] = 'Feedback files';
$string['feedbackforgradeitems'] = 'Feedback for {$a}';
$string['feedbackhistoryfiles'] = 'Feedback history files';
$string['feedbackprovided'] = 'Feedback provided';
$string['feedbacks'] = 'Feedbacks';
$string['feedbacksaved'] = 'Feedback saved';
$string['feedbackview'] = 'View feedback';
$string['finalgrade'] = 'Final grade';
$string['finalgrade_help'] = 'If the overridden checkbox is ticked, a grade may be added or amended.';
$string['fixedstudents'] = 'Static students column';
$string['fixedstudents_help'] = 'Allows grades to scroll horizontally without losing sight of the students column, by making it static.';
$string['forceimport'] = 'Force import';
$string['forceimport_help'] = 'Force import of grades even if the grades were updated after the import file was exported';
$string['forceoff'] = 'Force: Off';
$string['forceon'] = 'Force: On';
$string['forelementtypes'] = 'for the selected {$a}';
$string['forstudents'] = 'For students';
$string['full'] = 'Full';
$string['fullmode'] = 'Show grades and totals';
$string['generalsettings'] = 'General settings';
$string['grade_help'] = 'The grade to award the student for their work.';
$string['gradeadministration'] = 'Grade administration';
$string['gradealreadyupdated'] = '{$a} grades have not been imported because the grades in the import file are older than in the grader report. To proceed with the grade import anyway, use the force import option.';
$string['gradeanalysis'] = 'Grade analysis';
$string['gradebook'] = 'Gradebook';
$string['gradebookcalculationsuptodate'] = 'The calculations in the gradebook are up to date. You may need to reload this page to see changes.';
$string['gradebookcalculationsfixbutton'] = 'Accept grade changes and fix calculation errors';
$string['gradebookcalculationswarning'] = 'Note: Some errors have been detected in calculating the grades displayed in the gradebook. If your course has not started or is in progress, it is recommended that the errors are fixed by clicking the button below, though this will result in some grades being changed. If your course has already ended and grades have been submitted, you probably do not want to fix this issue.

The newer version is {$a->currentversion}; you are using gradebook version {$a->gradebookversion}. You can see a list of changes in <a href="{$a->url}">Gradebook calculation changes</a>.';
$string['gradebookhiddenerror'] = 'The gradebook is currently set to hide everything from students.';
$string['gradebooknavigationmenu'] = 'Gradebook navigation menu';
$string['gradebookhistories'] = 'Grade histories';
$string['gradebooksetup'] = 'Gradebook setup';
$string['gradeboundary'] = 'Letter grade boundary';
$string['gradeboundary_help'] = 'This setting determines the minimum percentage over which grades will be assigned the grade letter.';
$string['gradecategories'] = 'Grade categories';
$string['gradecategory'] = 'Grade category';
$string['gradecategoryonmodform'] = 'Grade category';
$string['gradecategoryonmodform_help'] = 'This setting controls the category in which this activity\'s grades are placed in the gradebook.';
$string['gradecategorysettings'] = 'Grade category settings';
$string['gradedisplay'] = 'Grade display';
$string['gradedisplaytype'] = 'Grade display type';
$string['gradedisplaytype_help'] = 'This setting determines how grades are displayed in the grader and user reports.

* Letter - Letters or words are used to represent a range of grades, as defined in \'Letters\' in the gradebook setup
* Percentage - Relative to maximum and minimum grades
* Real - Actual grades or scale values';
$string['gradedon'] = 'Graded: {$a}';
$string['gradeexport'] = 'Grade export';
$string['gradeexportcolumntype'] = '{$a->name} ({$a->extra})';
$string['gradeexportcustomprofilefields'] = 'Grade export custom profile fields';
$string['gradeexportcustomprofilefields_desc'] = 'Include these custom profile fields in the grade export, separated by commas.';
$string['gradeexportdecimalpoints'] = 'Grade export decimal places';
$string['gradeexportdecimalpoints_desc'] = 'The number of decimal places to display for export. This can be overridden during export.';
$string['gradeexportdisplaytype'] = 'Grade export display type';
$string['gradeexportdisplaytype_desc'] = 'Grades can be shown as real grades, as percentages (in reference to the minimum and maximum grades) or as letters (A, B, C etc..) during export. This can be overridden during export.';
$string['gradeexportdisplaytypes'] = 'Grade export display types';
$string['gradeexportuserprofilefields'] = 'Grade export user profile fields';
$string['gradeexportuserprofilefields_desc'] = 'Include these user profile fields in the grade export, separated by commas.';
$string['gradeforstudent'] = '{$a->student}<br />{$a->item}{$a->feedback}';
$string['gradegrademinmax'] = 'Initial min and max grades';
$string['gradehelp'] = 'Grade help';
$string['gradehistorylifetime'] = 'Grade history lifetime';
$string['gradehistorylifetime_help'] = 'This specifies the length of time you want to keep history of changes in grade related tables. It is recommended to keep it as long as possible. If you experience performance problems or have limited database space, try to set lower value.';
$string['gradeimport'] = 'Grade import';
$string['gradeimportfailed'] = 'Grade Import failed during commit. Details:';
$string['gradeitem'] = 'Grade item';
$string['gradeitemaddusers'] = 'Exclude from grading';
$string['gradeitemislocked'] = 'This activity is locked in the gradebook. Changes that are made to grades in this activity will not be copied to the gradebook until it is unlocked.';
$string['gradeitemlocked'] = 'Grading locked';
$string['gradeitemminmax'] = 'Min and max grades as specified in grade item settings';
$string['gradeitemmembersselected'] = 'Excluded from grading';
$string['gradeitemnonmembers'] = 'Included in grading';
$string['gradeitemremovemembers'] = 'Include in grading';
$string['gradeitems'] = 'Grade items';
$string['gradeitemsettings'] = 'Grade item settings';
$string['gradeitemsinc'] = 'Grade items to be included';
$string['gradeletter'] = 'Grade letter';
$string['gradeletter_help'] = 'Grade letters are letters, A, B, C, ..., or words, for example Distinction, Merit, Pass, ..., used to represent a range of grades.';
$string['gradeletternote'] = 'To delete a grade letter just empty any of the<br /> three text areas for that letter and click submit.';
$string['gradeletteroverridden'] = 'The default grade letters are currently overridden.';
$string['gradeletters'] = 'Grade letters';
$string['gradelocked'] = 'Grade is locked';
$string['gradelong'] = '{$a->grade} / {$a->max}';
$string['grademax'] = 'Maximum grade';
$string['grademax_help'] = 'This setting determines the maximum grade when using the value grade type. The maximum grade for an activity-based grade item is set on the activity settings page.';
$string['grademin'] = 'Minimum grade';
$string['grademin_help'] = 'This setting determines the minimum grade when using the value grade type.';
$string['gradenotificationmessage'] = 'You have new feedback on your work for "{$a}"';
$string['gradenotificationsubject'] = 'You have been graded';
$string['gradeoutcomeitem'] = 'Grade outcome item';
$string['gradeoutcomes'] = 'Outcomes';
$string['gradeoutcomescourses'] = 'Course outcomes';
$string['gradepass'] = 'Grade to pass';
$string['gradepass_help'] = 'This setting determines the minimum grade required to pass. The value is used in activity and course completion, and in the gradebook, where pass grades are highlighted in green and fail grades in red.';
$string['gradepassgreaterthangrade'] = 'The grade to pass can not be greater than the maximum possible grade {$a}';
$string['gradepenalty'] = 'Grade penalties';
$string['gradepenalty_enabledmodules'] = 'Enabled activities';
$string['gradepenalty_enabledmodules_help'] = 'Enable grade penalties for the selected activities.';
$string['gradepenalty_indicator_info'] = 'Late penalty applied -{$a} marks';
$string['gradepenalty_supportedplugins'] = 'Manage supported activities';
$string['gradepointdefault'] = 'Grade point default';
$string['gradepointdefault_help'] = 'This setting determines the default value for the grade point value available in a grade item.';
$string['gradepointdefault_validateerror'] = 'This setting must be an integer between 1 and the grade point maximum.';
$string['gradepointmax'] = 'Grade point maximum';
$string['gradepointmax_help'] = 'This setting determines the maximum grade point value available in an activity.';
$string['gradepointmax_validateerror'] = 'This setting must be an integer between 1 and 10000.';
$string['gradepreferences'] = 'Grade preferences';
$string['gradepreferenceshelp'] = 'Grade preferences Help';
$string['gradepublishing'] = 'Enable publishing';
$string['gradepublishinglink'] = 'Download: {$a}';
$string['gradepublishing_help'] = 'Grade publishing is a way of importing and exporting grades via a URL without being logged in to Moodle. If enabled, administrators and users with the permission to publish grades (by default, users with the role of manager only) are provided with grade export publishing settings in each course gradebook.';
$string['gradereport'] = 'Grade report';
$string['graderreport'] = 'Grader report';
$string['grades'] = 'Grades';
$string['gradesforuser'] = 'Grades for {$a->user}';
$string['gradesonly'] = 'Show grades only';
$string['gradesmoduledeletionpendingwarning'] = 'Warning: Activity deletion in progress! Some grades are about to be removed.';
$string['gradesmoduledeletionprefix'] = '[Deletion in progress]';
$string['gradessettings'] = 'Grade settings';
$string['gradetype'] = 'Grade type';
$string['gradetype_help'] = 'There are 4 grade types:

* None - No grading possible
* Value - A numerical value with a maximum and minimum
* Scale - An item in a list
* Text - Feedback only

Only value and scale grade types may be aggregated. The grade type for an activity-based grade item is set on the activity settings page.';
$string['gradevaluetoobig'] = 'One of the grade values is larger than the allowed grade maximum of {$a}';
$string['gradevaluetoosmall'] = 'One of the grade values is smaller than the allowed grade minimum of {$a}';
$string['gradeview'] = 'View grade';
$string['gradewasmodifiedduringediting'] = 'The grade entered for {$a->itemname} for {$a->username} was ignored because it was more recently updated by someone else.';
$string['gradeweighthelp'] = 'Grade weight help';
$string['gradingmodulename'] = 'Grading ({$a})';
$string['groupavg'] = 'Group average';
$string['hidden'] = 'Hidden';
$string['hidden_help'] = 'If ticked, grades are hidden from students. A hidden until date may be set if desired, to release grades after grading is completed.';
$string['hiddenasdate'] = 'Show submitted date for hidden grades';
$string['hiddenasdate_help'] = 'If user can not see hidden grades show date of submission instead of \'-\'.';
$string['hiddenuntil'] = 'Hidden until';
$string['hideadvanced'] = 'Hide advanced features';
$string['hideaverages'] = 'Hide averages';
$string['hidecalculations'] = 'Hide calculations';
$string['hidecategory'] = 'Hidden';
$string['hideeyecons'] = 'Hide show/hide icons';
$string['hidefeedback'] = 'Hide feedback';
$string['hideforcedsettings'] = 'Hide forced settings';
$string['hideforcedsettings_help'] = 'Do not show forced settings in grading UI.';
$string['hidegroups'] = 'Hide groups';
$string['hidelocks'] = 'Hide locks';
$string['hidenooutcomes'] = 'Show outcomes';
$string['hidequickfeedback'] = 'Hide quick feedback';
$string['hideranges'] = 'Hide ranges';
$string['hidetotalifhiddenitems'] = 'Hide totals if they contain hidden items';
$string['hidetotalifhiddenitems_help'] = 'This setting specifies whether totals which contain hidden grade items are shown to students or replaced with a hyphen (-). If shown, the total may be calculated either excluding or including hidden items.

If hidden items are excluded, the total will be different to the total seen by the teacher in the grader report, since the teacher always sees totals calculated from all items, hidden or unhidden. If hidden items are included, students may be able to calculate the hidden items.';
$string['hidetotalshowexhiddenitems'] = 'Show totals excluding hidden items';
$string['hidetotalshowinchiddenitems'] = 'Show totals including hidden items';
$string['highgradeascending'] = 'Sort by high grade ascending';
$string['highgradedescending'] = 'Sort by high grade descending';
$string['highgradeletter'] = 'High';
$string['changedefaults'] = 'Change defaults';
$string['changereportdefaults'] = 'Change report defaults';
$string['chooseaction'] = 'Choose an action ...';
$string['choosecategory'] = 'Select category';
$string['identifier'] = 'Identify user by';
$string['idnumbers'] = 'ID numbers';
$string['ignore'] = 'Ignore';
$string['import'] = 'Import';
$string['importas'] = 'Import as';
$string['importcsv'] = 'Import CSV';
$string['importcsv_help'] = 'Grades can be imported via a CSV file with format as follows:

* Each line of the file contains one record
* Each record is a series of data separated by the selected separator
* The first record contains a list of fieldnames defining the format of the rest of the file
* A fieldname containing user identity data is required - either username or ID number or email address

A file of the correct format can be obtained by first exporting some grades. The file can then be edited and saved as a CSV file.';
$string['importcsv_link'] = 'grade/import/csv/index';
$string['importcustom'] = 'Import as custom outcomes (only this course)';
$string['importdata'] = 'Data';
$string['importdata_help'] = 'Grades may be pasted directly from an OpenDocument or Excel spreadsheet with format as follows:

* At least one column should contain user identity data - username, email address or ID number.
* Other columns should have headings as for existing activities or grade items.

A correctly formatted spreadsheet can be obtained by first exporting some grades. The spreadsheet can then be edited to add more grades and saved.

Select and copy the columns you want to import from the spreadsheet and then paste them into this field.';
$string['importerror'] = 'An error occurred, this script wasn\'t called with the right parameters.';
$string['importfailed'] = 'Import failed. No data was imported.';
$string['importfeedback'] = 'Import feedback';
$string['importfile'] = 'Import file';
$string['importfilemissing'] = 'No file was received, go back to the form and make sure to upload a valid file.';
$string['importfrom'] = 'Import from';
$string['importoutcomenofile'] = 'The uploaded file is empty or corrupted.  Please verify this is a valid file. The problem was detected at line {$a}; this is triggered by the data lines not having as many columns as the first line (the header line) or if the imported file is missing expected headers.  Look at the exported file for an example of a file with valid header.';
$string['importoutcomes'] = 'Import outcomes';
$string['importoutcomes_help'] = 'Outcomes can be imported via csv file with format as for the export outcomes csv file.';
$string['importoutcomes_link'] = 'grade/outcome';
$string['importoutcomesuccess'] = 'Imported outcome "{$a->name}" with ID #{$a->id}';
$string['importplugins'] = 'Import plugins';
$string['importpreview'] = 'Import preview';
$string['importsettings'] = 'Import settings';
$string['importskippednomanagescale'] = 'You don\'t have permission to add a new scale, so outcome "{$a}" was skipped as it required creating a new scale';
$string['importskippedoutcome'] = 'An outcome with shortname "{$a}" already exists in this context, the one in the imported file was skipped.';
$string['importstandard'] = 'Import as standard outcomes';
$string['importsuccess'] = 'Grade import success';
$string['importxml'] = 'Import XML';
$string['includescalesinaggregation'] = 'Include scales in aggregation';
$string['includescalesinaggregation_help'] = 'You can change whether scales are to be included as numbers in all aggregated grades across all gradebooks in all courses. CAUTION: changing this setting will force all aggregated grades to be recalculated.';
$string['incorrectcourseid'] = 'Course ID was incorrect';
$string['incorrectcustomscale'] = '(Incorrect custom scale, please change.)';
$string['incorrectminmax'] = 'The minimum must be lower than the maximum';
$string['inherit'] = 'Inherit';
$string['intersectioninfo'] = 'Student/Grade info';
$string['invalidgradeexporteddate'] = 'The export date is invalid because it is more than a year ago, or in the future, or because the format is invalid.';
$string['item'] = 'Item';
$string['iteminfo'] = 'Item info';
$string['iteminfo_help'] = 'This setting provides space for entering information about the item. The information is not displayed anywhere else.';
$string['itemname'] = 'Item name';
$string['itemnamehelp'] = 'The name of this item, pushed in by the module.';
$string['items'] = 'Items';
$string['itemsedit'] = 'Edit grade item';
$string['keephigh'] = 'Keep the highest';
$string['keephighestvalues'] = 'Keep the {$a} highest values';
$string['keephigh_help'] = 'If set, this option will only keep the X highest grades, X being the selected value for this option.';
$string['keymanager'] = 'Key manager';
$string['lessthanmin'] = 'The grade entered for {$a->itemname} for {$a->username} is less than the minimum allowed';
$string['letter'] = 'Letter';
$string['lettergrade'] = 'Letter grade';
$string['lettergradenonnumber'] = 'Low and/or High grade were non-numeric for';
$string['letterpercentage'] = 'Letter (percentage)';
$string['letterreal'] = 'Letter (real)';
$string['letters'] = 'Letters';
$string['linkedactivity'] = 'Linked activity';
$string['linkedactivity_help'] = 'This setting specifies an activity to which this outcome item is linked. This may be used to measure student performance on criteria not assessed by the activity grade.';
$string['linktoactivity'] = '{$a->name} activity {$a->title}';
$string['lock'] = 'Lock';
$string['locked'] = 'Locked';
$string['locked_help'] = 'If ticked, grades can no longer be automatically updated by the related activity.';
$string['locktime'] = 'Lock after';
$string['lowest'] = 'Lowest';
$string['lowgradeletter'] = 'Low';
$string['manageoutcomes'] = 'Manage outcomes';
$string['managepenaltyplugins'] = 'Manage penalty plugins';
$string['manualitem'] = 'Manual item';
$string['mapfrom'] = 'Map from';
$string['mapfrom_help'] = 'Select the column in the spreadsheet containing data for identifying the user, such as username, user ID or email address.';
$string['mappings'] = 'Grade item mappings';
$string['mappings_help'] = 'For each column of grades in the spreadsheet, select the corresponding grade item to import the grades into.';
$string['mapto'] = 'Map to';
$string['mapto_help'] = 'Select the same identifying data as selected for \'Map from\'.';
$string['max'] = 'Highest';
$string['maxgrade'] = 'Max grade';
$string['meanall'] = 'All grades';
$string['meangraded'] = 'Non-empty grades';
$string['meanselection'] = 'Grades selected for column averages';
$string['meanselection_help'] = 'This setting determines whether cells with no grade should be included when calculating the average (mean) for each category or grade item.';
$string['median'] = 'Median';
$string['min'] = 'Lowest';
$string['minmaxtouse'] = 'Min and max grades used in calculation';
$string['minmaxtouse_desc'] = 'This setting determines whether to use the initial minimum and maximum grades from when the grade was given, or the minimum and maximum grades as specified in the settings for the grade item, when calculating the grade displayed in the gradebook. It is recommended that this setting is modified at an off-peak time, as all grades will be recalculated, which may result in a high server load.';
$string['minmaxtouse_help'] = 'This setting determines whether to use the initial minimum and maximum grades from when the grade was given, or the minimum and maximum grades as specified in the settings for the grade item, when calculating the grade displayed in the gradebook.';
$string['minmaxtouse_link'] = 'Grades_min_max';
$string['minmaxupgradedgrades'] = 'Note: Some grades have been changed in order to resolve an inconsistency in the gradebook caused by a change in the minimum and maximum grades used when calculating the grade displayed. It is recommended that the changes are reviewed and accepted.';
$string['minmaxupgradefixbutton'] = 'Resolve inconsistencies';
$string['minmaxupgradewarning'] = 'Note: An inconsistency has been detected with some grades due to a change in the minimum and maximum grades used when calculating the grade displayed in the gradebook. It is recommended that the inconsistency is resolved by clicking the button below, though this will result in some grades being changed.';
$string['minimum_show'] = 'Show minimum grade';
$string['minimum_show_help'] = 'Minimum grade is used in calculating grades and weights. If not shown, minimum grade will default to zero and cannot be edited.';
$string['missingitemtypeoreid'] = 'Array key (itemtype or eid) missing';
$string['missingscale'] = 'Scale must be selected';
$string['mode'] = 'Mode';
$string['modgrade'] = 'Grade';
$string['modgrade_help'] = '* Point - A numerical value
* None - Feedback only
* Scale - An item from a list';
$string['modgradecantchangegradetype'] = 'You cannot change the type, as grades already exist for this item.';
$string['modgradecantchangegradetypemsg'] = 'Some grades have already been awarded, so the grade type cannot be changed. If you wish to change the maximum grade, you must first choose whether or not to rescale existing grades.';
$string['modgradecantchangegradetyporscalemsg'] = 'Some grades have already been awarded, so the grade type and scale cannot be changed.';
$string['modgradecategorycantchangegradetypemsg'] = 'This category has associated grade items which have been overridden. Therefore some grades have already been awarded, so the grade type cannot be changed. If you wish to change the maximum grade, you must first choose whether or not to rescale existing grades.';
$string['modgradecategorycantchangegradetyporscalemsg'] = 'This category has associated grade items which have been overridden. Therefore some grades have already been awarded, so the grade type and scale cannot be changed.';
$string['modgradecantchangescale'] = 'You cannot change the scale, as grades already exist for this item.';
$string['modgradecantchangeratingmaxgrade'] = 'You cannot change the maximum grade when grades already exist for an activity with ratings.';
$string['modgradedonotmodify'] = 'Do not modify existing grades';
$string['modgradeerrorbadpoint'] = 'Invalid grade value. This must be an integer between 1 and {$a}';
$string['modgradeerrorbadscale'] = 'Invalid scale selected. Please make sure you select a scale from the selections below.';
$string['modgrademaxgrade'] = 'Maximum grade';
$string['modgraderecalculatepenalty'] = 'Recalculate penalty';
$string['modgraderecalculatepenalty_help'] = 'The penalty will be recalculated for all users.';
$string['modgraderescalegrades'] = 'Rescale existing grades';
$string['modgraderescalegrades_help'] = 'When changing the maximum grades on a gradebook item you need to specify whether or not this will cause existing percentage grades to change as well.

If this is set to \'Yes\', any existing grades will be rescaled so that the percentage grade remains the same.

For example, if this option is set to \'Yes\', changing the maximum grade on an item from 10 to 20 would cause a grade of 6/10 (60%) to be rescaled to 12/20 (60%). With this option set to \'No\', the grade would change from 6/10 (60%) to 6/20 (30%), requiring manual adjustment of the grade items to ensure correct scores.';
$string['modgradecategoryrescalegrades'] = 'Rescale overridden grades';
$string['modgradecategoryrescalegrades_help'] = 'When changing the maximum grades on a gradebook item you need to specify whether or not this will cause existing percentage grades to change as well.

If this is set to \'Yes\', any existing overridden grades will be rescaled so that the percentage grade remains the same.

For example, if this option is set to \'Yes\', changing the maximum grade on an item from 10 to 20 would cause a grade of 6/10 (60%) to be rescaled to 12/20 (60%). With this option set to \'No\', the grade will remain unchanged, requiring manual adjustment of the grade items to ensure correct scores.';
$string['modgradetype'] = 'Type';
$string['modgradetypenone'] = 'None';
$string['modgradetypepoint'] = 'Point';
$string['modgradetypescale'] = 'Scale';
$string['morethanmax'] = 'The grade entered for {$a->itemname} for {$a->username} is more than the maximum allowed';
$string['movesitems'] = 'Move items';
$string['moveselectedto'] = 'Move selected items to:';
$string['movingelement'] = 'Moving {$a}';
$string['multfactor'] = 'Multiplicator';
$string['multfactorvalue'] = 'Multiplicator value for {$a}';
$string['multfactor_help'] = 'The multiplicator is the factor by which all grades for this grade item will be multiplied, with a maximum value of the maximum grade. For example, if the multiplicator is 2 and the maximum grade is 100, then all grades less than 50 are multiplied by 2, and all grades 50 and above are changed to 100.';
$string['mygrades'] = 'User menu grades link';
$string['mygrades_desc'] = 'This setting allows for the option of linking to an external gradebook from the user menu.';
$string['mypreferences'] = 'My preferences';
$string['myreportpreferences'] = 'My report preferences';
$string['myself'] = 'Myself';
$string['mustchooserescaleyesorno'] = 'You must choose whether to rescale existing grades or not.';
$string['neverdeletehistory'] = 'Never delete history';
$string['newcategory'] = 'New category';
$string['newitem'] = 'New grade item';
$string['newoutcomeitem'] = 'New outcome item';
$string['no'] = 'No';
$string['nocategories'] = 'Grade categories could not be added or found for this course';
$string['nocategoryname'] = 'No category name was given.';
$string['nocategoryview'] = 'No category to view by';
$string['nocourses'] = 'There are no courses yet';
$string['noexistingscales'] = 'There are no existing scales.';
$string['noexistingoutcomes'] = 'There are no existing outcomes.';
$string['noforce'] = 'Do not force';
$string['nogradeletters'] = 'No grade letters set';
$string['nogradesreturned'] = 'No grades returned';
$string['noidnumber'] = 'No ID number';
$string['nolettergrade'] = 'No letter grade for';
$string['nomode'] = 'NA';
$string['nonnumericweight'] = 'Received non-numeric value for';
$string['nonunlockableverbose'] = 'To unlock this grade, you need to unlock {$a->itemname}.';
$string['nonunhideableverbose'] = 'To unhide this grade, you need to unhide {$a->itemname}.';
$string['nonweightedpct'] = 'non-weighted %';
$string['nooutcome'] = 'No outcome';
$string['nooutcomes'] = 'Outcome items must be linked to a course outcome, but there are no outcomes for this course. Would you like to add one?';
$string['nopermissiontoresetweights'] = 'No permission to reset the weights';
$string['nopublish'] = 'Do not publish';
$string['noreports'] = 'You are not enrolled in, nor teaching any courses on this site.';
$string['norolesdefined'] = 'No roles defined in Administration > Grades > General settings > Graded roles';
$string['noscales'] = 'Outcomes must be linked to a course scale or global scale, but there are none. Would you like to add one?';
$string['noselectedcategories'] = 'no categories were selected.';
$string['noselecteditems'] = 'no items were selected.';
$string['notteachererror'] = 'You must be a teacher to use this feature.';
$string['notenrolled'] = 'You are currently not enrolled in any courses.';
$string['nousersloaded'] = 'No users loaded';
$string['nouserstograde'] = 'No users to grade';
$string['numberofgrades'] = 'Number of grades';
$string['onascaleof'] = 'on a scale of {$a->grademin} to {$a->grademax}';
$string['operations'] = 'Operations';
$string['options'] = 'Options';
$string['originalgrade'] = 'Original grade';
$string['others'] = 'Others';
$string['otheruser'] = 'User';
$string['outcome'] = 'Outcome';
$string['outcome_help'] = 'This setting determines the outcome which this grade item will represent in the gradebook.';
$string['outcomeassigntocourse'] = 'Assign another outcome to this course';
$string['outcomecategory'] = 'Create outcomes in category';
$string['outcomecategorynew'] = 'New category';
$string['outcomeconfirmdelete'] = 'Are you sure you wish to delete the outcome "{$a}"?';
$string['outcomecreate'] = 'Add a new outcome';
$string['outcomedelete'] = 'Delete outcome';
$string['outcomefullname'] = 'Full name';
$string['outcomeitem'] = 'Outcome item';
$string['outcomeitemsedit'] = 'Edit outcome item';
$string['outcomereport'] = 'Outcome report';
$string['outcomes'] = 'Outcomes';
$string['outcomescourse'] = 'Outcomes used in course';
$string['outcomescoursecustom'] = 'Custom used (no remove)';
$string['outcomescoursenotused'] = 'Standard not used';
$string['outcomescourseused'] = 'Standard used (no remove)';
$string['outcomescustom'] = 'Custom outcomes';
$string['outcomesdisabled'] = 'Outcomes are not enabled';
$string['outcomeshortname'] = 'Short name';
$string['outcomesstandard'] = 'Standard outcomes';
$string['outcomesstandardavailable'] = 'Available standard outcomes';
$string['outcomestandard'] = 'Standard outcome';
$string['outcomestandard_help'] = 'A standard outcome is available site-wide, for all courses.';
$string['overallaverage'] = 'Overall average';
$string['overridecat'] = 'Allow category grades to be manually overridden';
$string['overridecat_help'] = 'Disabling this setting makes it impossible for users to override category grades.';
$string['overridden'] = 'Overridden';
$string['overridden_help'] = 'If ticked, the grade can no longer be changed from within the related activity.

When a grade is edited in the grader report, the overridden checkbox is ticked automatically. However it may be un-ticked to allow the grade to be changed via the related activity.';
$string['overriddennotice'] = 'Your final grade from this activity was manually adjusted.';
$string['overridesitedefaultgradedisplaytype'] = 'Override site defaults';
$string['overridesitedefaultgradedisplaytype_help'] = 'If ticked, grade letters and boundaries for the course may be set, rather than using the site defaults.';
$string['overrideweightofa'] = 'Override weight of {$a}';
$string['parentcategory'] = 'Parent category';
$string['pctoftotalgrade'] = '% of total grade';
$string['pass'] = 'Pass';
$string['percent'] = 'Percent';
$string['percentage'] = 'Percentage';
$string['percentageletter'] = 'Percentage (letter)';
$string['percentagereal'] = 'Percentage (real)';
$string['percentascending'] = 'Sort by percent ascending';
$string['percentdescending'] = 'Sort by percent descending';
$string['percentshort'] = '%';
$string['plusfactor'] = 'Offset';
$string['plusfactor_help'] = 'The offset is a number that is added to every grade for this grade item, after the multiplicator is applied.';
$string['plusfactorvalue'] = 'Offset value for {$a}';
$string['points'] = 'points';
$string['pointsascending'] = 'Sort by points ascending';
$string['pointsdescending'] = 'Sort by points descending';
$string['positionfirst'] = 'First';
$string['positionlast'] = 'Last';
$string['preferences'] = 'Preferences';
$string['prefgeneral'] = 'General';
$string['prefletters'] = 'Grade letters and boundaries';
$string['prefrows'] = 'Special rows';
$string['prefshow'] = 'Show/hide toggles';
$string['previewrows'] = 'Preview rows';
$string['privacy:metadata:categorieshistory'] = 'A record of previous versions of grade categories';
$string['privacy:metadata:filepurpose'] = 'Feedback files stored in the gradebook for a user.';
$string['privacy:metadata:grade_import_newitem'] = 'Temporary table for storing new grade_item names from grade import';
$string['privacy:metadata:grade_import_newitem:importcode'] = 'A unique batch code for identifying one batch of imports';
$string['privacy:metadata:grade_import_newitem:importer'] = 'User importing the data';
$string['privacy:metadata:grade_import_newitem:itemname'] = 'New grade item name';
$string['privacy:metadata:grade_import_values'] = 'Temporary table for importing grades';
$string['privacy:metadata:grade_import_values:feedback'] = 'Grade feedback';
$string['privacy:metadata:grade_import_values:finalgrade'] = 'Raw grade value';
$string['privacy:metadata:grade_import_values:importcode'] = 'A unique batch code for identifying one batch of imports';
$string['privacy:metadata:grade_import_values:importer'] = 'User importing the data';
$string['privacy:metadata:grade_import_values:importonlyfeedback'] = 'Flag if only feedback was imported';
$string['privacy:metadata:grade_import_values:userid'] = 'User whose grade was imported';
$string['privacy:metadata:grades'] = 'A record of grades';
$string['privacy:metadata:grades:aggregationstatus'] = 'The aggregation status';
$string['privacy:metadata:grades:aggregationweight'] = 'The weight in aggregation';
$string['privacy:metadata:grades:feedback'] = 'The feedback';
$string['privacy:metadata:grades:finalgrade'] = 'The grade';
$string['privacy:metadata:grades:information'] = 'Additional information about the grade';
$string['privacy:metadata:grades:timemodified'] = 'The time when the grade was last modified';
$string['privacy:metadata:grades:userid'] = 'The ID of the user whose grade it is';
$string['privacy:metadata:grades:usermodified'] = 'The ID of the user who last modified the record';
$string['privacy:metadata:gradeshistory'] = 'A record of the previous grades';
$string['privacy:metadata:history:loggeduser'] = 'The ID of the user who was logged in when the versioning occurred';
$string['privacy:metadata:history:timemodified'] = 'The time when grade versioning occurred';
$string['privacy:metadata:itemshistory'] = 'A record of previous versions of grade items';
$string['privacy:metadata:outcomes'] = 'A record of outcomes';
$string['privacy:metadata:outcomes:timemodified'] = 'The time when the record was modified';
$string['privacy:metadata:outcomes:usermodified'] = 'The user who last modified the record';
$string['privacy:metadata:outcomeshistory'] = 'A record of previous versions of outcomes';
$string['privacy:metadata:scale'] = 'A record of scales';
$string['privacy:metadata:scale:timemodified'] = 'The time when the record was last modified';
$string['privacy:metadata:scale:userid'] = 'The user who last modified the record';
$string['privacy:metadata:scalehistory'] = 'A record of previous versions of scales';
$string['privacy:path:relatedtome'] = 'Related to me';
$string['privacy:request:historyactiondelete'] = 'Delete';
$string['privacy:request:historyactioninsert'] = 'Insert';
$string['privacy:request:historyactionupdate'] = 'Update';
$string['privacy:request:unknowndeletedgradeitem'] = 'Unknown (the grade item was deleted)';
$string['profilereport'] = 'User profile report';
$string['profilereport_help'] = 'Grade report used on user profile page.';
$string['publishing'] = 'Publishing';
$string['publishingoptions'] = 'Grade publishing options';
$string['quickfeedback'] = 'Quick feedback';
$string['quickgrading'] = 'Quick grading';
$string['quickgrading_help'] = 'If enabled, when editing is turned on, a text input box appears for each grade, allowing many grades to be edited at the same time. Changes are saved and highlighted when the update button is clicked.

Note that when a grade is edited in the grader report, an overridden flag is set, meaning that the grade can no longer be changed from within the related activity.';
$string['range'] = 'Range';
$string['rangesdecimalpoints'] = 'Decimals shown in ranges';
$string['rangesdecimalpoints_help'] = 'This setting determines the number of decimal places to display for each range or whether the overall decimal places setting for the category or grade item is used (inherit).';
$string['rangesdisplaytype'] = 'Range display type';
$string['rangesdisplaytype_help'] = 'This setting determines whether the range is displayed as real grades, percentages or letters, or whether the display type for the category or grade item is used (inherit).';
$string['rank'] = 'Rank';
$string['rawpct'] = 'Raw %';
$string['real'] = 'Real';
$string['realletter'] = 'Real (letter)';
$string['realpercentage'] = 'Real (percentage)';
$string['recalculatepenalty'] = 'Recalculate penalties';
$string['recalculatepenaltyconfirm'] = 'This will recalculate and apply penalties for all submissions in this context. Are you sure you want to continue?';
$string['recalculatepenaltysuccess'] = 'Penalty recalculation in progress. There may be a delay before grades are updated.';
$string['recalculatinggrades'] = 'Recalculating grades';
$string['recalculatinggradesadhoc'] = 'The report will update automatically. You don\'t need to do anything.';
$string['recovergradesdefault'] = 'Recover grades default';
$string['recovergradesdefault_help'] = 'By default recover old grades when re-enrolling a user in a course.';
$string['refreshpreview'] = 'Refresh preview';
$string['regradeanyway'] = 'Regrade anyway';
$string['removeallcoursegrades'] = 'All grades';
$string['removeallcoursegrades_help'] = 'If ticked, all grade items and grades that were manually added to the gradebook will be deleted, as well as data on overridden, excluded, hidden and locked grades. Only grade items and grades associated with activities will remain.';
$string['removeallcourseitems'] = 'All items and categories';
$string['removeallcourseitems_help'] = 'If ticked, all categories and grade items which were manually added to the gradebook will be deleted, together with grades and data on overridden, excluded, hidden and locked grades. Only grade items associated with activities will remain.';
$string['report'] = 'Report';
$string['reportdefault'] = 'Report default ({$a})';
$string['reportplugins'] = 'Report plugins';
$string['reportsettings'] = 'Report settings';
$string['reprintheaders'] = 'Reprint headers';
$string['resetweightsshort'] = 'Reset weights';
$string['resetweights'] = 'Reset weights of {$a->itemname}';
$string['respectingcurrentdata'] = 'leaving current configuration unmodified';
$string['rowpreviewnum'] = 'Preview rows';
$string['rowpreviewnum_help'] = 'Data for import may be previewed before confirming the import. This setting determines how many rows are displayed in the preview.';
$string['savechanges'] = 'Save changes';
$string['savepreferences'] = 'Save preferences';
$string['scaleconfirmdelete'] = 'Are you sure you wish to delete the scale "{$a}"?';
$string['scaledpct'] = 'Scaled %';
$string['searchcollapsedcolumns'] = 'Search collapsed columns';
$string['seeallcoursegrades'] = 'See all course grades';
$string['select'] = 'Select {$a}';
$string['selectalloroneuser'] = 'Select all or one user';
$string['selectauser'] = 'Select a user';
$string['selectdestination'] = 'Select destination of {$a}';
$string['selectgrouplink'] = 'Select a group';
$string['separator'] = 'Separator';
$string['separator_help'] = 'Select the separator used in the CSV file. (Normally this is a comma.)';
$string['sepcolon'] = 'Colon';
$string['sepcomma'] = 'Comma';
$string['sepsemicolon'] = 'Semicolon';
$string['septab'] = 'Tab';
$string['setcategories'] = 'Set categories';
$string['setcategorieserror'] = 'You must first set the categories for your course before you can give weights to them.';
$string['setgradeletters'] = 'Set grade letters';
$string['setpreferences'] = 'Set preferences';
$string['setting'] = 'Setting';
$string['settings'] = 'Settings';
$string['setup'] = 'Setup';
$string['setweights'] = 'Set weights';
$string['showaverage'] = 'Show average';
$string['showaverage_help'] = 'Whether to show an average column. Note that participants may be able to estimate others\' grades if the average is calculated from a small number of grades. For performance reasons the average is approximate if it is dependent on any hidden items.';
$string['showcontributiontocoursetotal'] = 'Show contribution to course total';
$string['showcontributiontocoursetotal_help'] = 'Whether to show a column of percentages indicating how much each grade item contributes to the user\'s course total percentage (after weighting is applied).';
$string['showfeedback'] = 'Show feedback';
$string['showfeedback_help'] = 'Whether to show a column for feedback.';
$string['showgrade'] = 'Show grades';
$string['showgrade_help'] = 'Whether to show a column for grades.';
$string['showingaggregatesonly'] = 'Showing totals only';
$string['showingfullmode'] = 'Showing grades and totals';
$string['showinggradesonly'] = 'Showing grades only';
$string['showlettergrade'] = 'Show letter grades';
$string['showlettergrade_help'] = 'Whether to show a column for letter grades.';
$string['showrange'] = 'Show ranges';
$string['showrange_help'] = 'Whether to show a column for the range.';
$string['showweight'] = 'Show weightings';
$string['showweight_help'] = 'Whether to show a column for the grade weight.';
$string['rangedecimals'] = 'Range decimal places';
$string['rangedecimals_help'] = 'The number of decimal places to display for the range.';
$string['showallgrades'] = 'Show all grades';
$string['showallhidden'] = 'Show hidden';
$string['showallstudents'] = 'Show all Students';
$string['showaverages'] = 'Show column averages';
$string['showaverages_help'] = 'If enabled, the grader report will contain an additional row displaying the average (mean) for each category and grade item.';
$string['showgroups'] = 'Show groups';
$string['showhiddenitems'] = 'Show hidden items';
$string['showhiddenitems_help'] = 'Whether hidden grade items are hidden entirely or if the names of hidden grade items are visible to students.

* Show hidden - Hidden grade item names are shown but student grades are hidden
* Only hidden until - Grade items with a "hide until" date set are hidden completely until the set date, after which the whole item is shown
* Do not show - Hidden grade items are completely hidden';
$string['showhiddenuntilonly'] = 'Only hidden until';
$string['shownohidden'] = 'Do not show';
$string['shownooutcomes'] = 'Hide outcomes';
$string['shownumberofgrades'] = 'Show number of grades in averages';
$string['shownumberofgrades_help'] = 'If enabled, the number of grades used when calculating the average (mean) is displayed in brackets after each average.';
$string['showonlyactiveenrol'] = 'Show only active enrolments';
$string['showonlyactiveenrol_help'] = 'This setting determines, if only active enrolled users will be visible in gradebook report. If enabled suspended users will not be shown in gradebook.';
$string['showpercentage'] = 'Show percentage';
$string['showpercentage_help'] = 'Whether to show the percentage value of each grade item.';
$string['showranges'] = 'Show ranges';
$string['showranges_help'] = 'If enabled, the grader report will contain an additional row displaying the range for each category and grade item.';
$string['showrank'] = 'Show rank';
$string['showrank_help'] = 'Whether to show the position of the student in relation to the rest of the class for each grade item.';
$string['showuserimage'] = 'Show user profile images';
$string['showuserimage_help'] = 'Whether to show the user\'s profile image next to the name in the grader report.';
$string['simpleview'] = 'Single view';
$string['sitewide'] = 'Site-wide';
$string['sort'] = 'sort';
$string['sortasc'] = 'Sort in ascending order';
$string['sortbyfirstname'] = 'Sort by first name';
$string['sortbylastname'] = 'Sort by last name';
$string['sortdesc'] = 'Sort in descending order';
$string['standarddeviation'] = 'Standard deviation';
$string['stats'] = 'Statistics';
$string['statslink'] = 'Stats';
$string['student'] = 'Student';
$string['studentsperpagereduced'] = 'Reduced maximum students per page from {$a->originalstudentsperpage} to {$a->studentsperpage}. Consider increasing the PHP setting max_input_vars from {$a->maxinputvars}.';
$string['subcategory'] = 'Normal category';
$string['submissions'] = 'Submissions';
$string['submittedon'] = 'Submitted: {$a}';
$string['sumofgradesupgradedgrades'] = 'Note: The aggregation method "Sum of grades" has been changed to "Natural" as part of a site upgrade. Since "Sum of grades" was previously used in this course, it is recommended that you review this change in the gradebook.';
$string['aggregatesubcatsupgradedgrades'] = 'Note: The aggregation setting "Aggregate including subcategories" has been removed as part of a site upgrade. Since "Aggregate including subcategories" was previously used in this course, it is recommended that you review this change in the gradebook.';
$string['upgradedgradeshidemessage'] = 'Dismiss notice';
$string['switchtofullview'] = 'Switch to full view';
$string['switchtosimpleview'] = 'Switch to simple view';
$string['topcategory'] = 'Super category';
$string['total'] = 'Total';
$string['totalweightnot100'] = 'The total weight is not equal to 100';
$string['totalweight100'] = 'The total weight is equal to 100';
$string['turnfeedbackoff'] = 'Turn feedback off';
$string['turnfeedbackon'] = 'Turn feedback on';
$string['typenone'] = 'None';
$string['typescale'] = 'Scale';
$string['typescale_help'] = 'This setting determines the scale used when using the scale grade type. The scale for an activity-based grade item is set on the activity settings page.';
$string['typetext'] = 'Text';
$string['typevalue'] = 'Value';
$string['upgradedminmaxrevertmessage'] = 'Revert the changes';
$string['uncategorised'] = 'Uncategorised';
$string['unenrolledusersinimport'] = 'This import included the following grades for users not currently enrolled in this course: {$a}';
$string['unchangedgrade'] = 'Grade unchanged';
$string['unlimitedgrades'] = 'Unlimited grades';
$string['unlimitedgrades_help'] = 'By default, grades are limited by the maximum and minimum values of the grade item. Enabling this setting removes this limit, and allows grades of over 100% to be entered directly in the gradebook.';
$string['unlock'] = 'Unlock';
$string['unused'] = 'Unused';
$string['updatedgradesonly'] = 'Export new or updated grades only';
$string['uploadgrades'] = 'Upload grades';
$string['useadvanced'] = 'Use advanced features';
$string['usedcourses'] = 'Used courses';
$string['usedgradeitem'] = 'Used grade item';
$string['usenooutcome'] = 'Use no outcome';
$string['usenoscale'] = 'Use no scale';
$string['usepercent'] = 'Use percent';
$string['user'] = 'User';
$string['userfields_show'] = 'Show user fields';
$string['userfields_show_help'] = 'Show additional user fields like email address on the grader report. The specific fields displayed are controlled by the site setting showuseridentity';
$string['usergrade'] = 'User {$a->fullname} ({$a->useridnumber}) on item {$a->gradeidnumber}';
$string['userid'] = 'User ID';
$string['useridnumberwarning'] = 'Users without an ID number are excluded from the export as they cannot be imported';
$string['usermappingerror'] = 'User mapping error: Could not find user with {$a->field} of "{$a->value}".';
$string['usermappingerrormultipleusersfound'] = 'User mapping error: Multiple users found with {$a->field} of "{$a->value}". Please use a more unique mapping field.';
$string['usermappingerrorusernotfound'] = 'User mapping error. Could not find user.';
$string['usermappingerrorcurrentgroup'] = 'User is not a member of current group.';
$string['userpreferences'] = 'User preferences';
$string['userenrolmentsuspended'] = 'User enrolment suspended';
$string['useweighted'] = 'Use weighted';
$string['verbosescales'] = 'Verbose scales';
$string['verbosescales_help'] = 'A verbose scale uses words rather than numbers. Set to \'Yes\' if both numerical and verbose scales are to be imported. Set to \'No\' if only numerical scales are to be imported.';
$string['viewas'] = 'View report as';
$string['viewbygroup'] = 'Group';
$string['viewfeedback'] = 'View feedback';
$string['viewgrades'] = 'View grades';
$string['weight'] = 'weight';
$string['weight_help'] = 'A value used to determine the relative value of multiple grade items in a category or course.';
$string['weightcourse'] = 'Use weighted grades for course';
$string['weightedascending'] = 'Sort by weighted percent ascending';
$string['weighteddescending'] = 'Sort by weighted percent descending';
$string['weightoverride'] = 'weight adjustment';
$string['weightoverride_help'] = 'Uncheck this to reset a grade item weight to its automatically calculated value. Checking this will prevent the weight being automatically adjusted.';
$string['weightedpct'] = 'weighted %';
$string['weightedpctcontribution'] = 'weighted % contribution';
$string['weightofa'] = 'Weight of {$a}';
$string['weightorextracredit'] = 'Weight or extra credit';
$string['weight'] = 'Weight';
$string['weights'] = 'Weights';
$string['weightsadjusted'] = 'Your weights have been adjusted to total 100.';
$string['weightsedit'] = 'Edit weights and extra credits';
$string['weightuc'] = 'Calculated weight';
$string['writinggradebookinfo'] = 'Writing gradebook settings';
$string['xml'] = 'XML';
$string['yes'] = 'Yes';
$string['yourgrade'] = 'Your grade';

// Deprecated since Moodle 4.5.
$string['filterbothactive'] = 'First ({$a->first}) Last ({$a->last})';
$string['filterbyname'] = 'Filter by name';
$string['filterfirstactive'] = 'First ({$a->first})';
$string['filterlastactive'] = 'Last ({$a->last})';

// Deprecated since Moodle 5.1.
$string['editcalculationverbose'] = 'Edit calculation for {$a->category} {$a->itemmodule} {$a->itemname}';
$string['editverbose'] = 'Edit {$a->category} {$a->itemmodule} {$a->itemname}';
$string['hiddenuntildate'] = 'Hidden until: {$a}';
$string['hideverbose'] = 'Hide {$a->category} {$a->itemmodule} {$a->itemname}';
$string['locktimedate'] = 'Locked after: {$a}';
$string['lockverbose'] = 'Lock {$a->category} {$a->itemmodule} {$a->itemname}';
$string['showverbose'] = 'Show {$a->category} {$a->itemmodule} {$a->itemname}';
$string['unlockverbose'] = 'Unlock {$a->category} {$a->itemmodule} {$a->itemname}';
