{{!
    This file is part of Moodle - http://moodle.org/
    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_quiz/attempt_summary_information

    This template renders summary information about a single quiz attempt.

    Example context (json):
    {
        "hasitems": true,
        "items": [
            {
                "title": "<img src='https://qa.moodledemo.net/pluginfile.php/27/user/icon/boost/f2?rev=5522' class='userpicture' width='35' height='35' alt='It is me!'>",
                "content": "<a href='https://qa.moodledemo.net/user/view.php?id=4&amp;course=2'>Sam Student</a>"
            },
            {"title": "Status", "content": "Finished"},
            {"title": "Started", "content": "Thursday, 23 November 2023, 9:29 AM"},
            {"title": "Completed", "content": "Thursday, 23 November 2023, 9:32 AM"},
            {"title": "Duration", "content": "3 minutes"},
            {"title": "Grade", "content": "Not yet graded"}
        ],
        "caption": "Summary of attempt"
    }
}}
{{#hasitems}}
<table class="table generaltable generalbox quizreviewsummary mb-0">
       {{#caption}}
           <caption class="visually-hidden">{{caption}}</caption>
       {{/caption}}
    <tbody>
    {{#items}}
        <tr>
            <th class="cell" scope="row">{{{title}}}</th>
            <td class="cell">{{{content}}}</td>
        </tr>
    {{/items}}
    </tbody>
</table>
{{/hasitems}}
