{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_assign/override_form_user_defaults

    HTML for showing user default dates in override form.
}}
<dl>
    <dt>{{#str}} allowsubmissionsfromdate, mod_assign {{/str}}</dt>
    <dl>
        {{#allowsubmissionsfromdate}} {{#userdate}} {{.}}, {{#str}} strftimedatetime, core_langconfig {{/str}} {{/userdate}} {{/allowsubmissionsfromdate}}
        {{^allowsubmissionsfromdate}} {{#str}} disabled, mod_assign {{/str}} {{/allowsubmissionsfromdate}}
    </dl>

    <dt>{{#str}} duedate, mod_assign {{/str}}</dt>
    <dl>
        {{#duedate}} {{#userdate}} {{.}}, {{#str}} strftimedatetime, core_langconfig {{/str}} {{/userdate}} {{/duedate}}
        {{^duedate}} {{#str}} disabled, mod_assign {{/str}} {{/duedate}}
    </dl>

    <dt>{{#str}} cutoffdate, mod_assign {{/str}}</dt>
    <dl>
        {{#cutoffdate}} {{#userdate}} {{cutoffdate}}, {{#str}} strftimedatetime, core_langconfig {{/str}} {{/userdate}} {{/cutoffdate}}
        {{^cutoffdate}} {{#str}} disabled, mod_assign {{/str}} {{/cutoffdate}}
    </dl>
</dl>
