<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'core_langconfig', language 'en', branch 'MOODLE_20_STABLE'
 *
 * @package   core
 * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['alphabet'] = 'A,B,C,D,E,F,G,H,I,J,K,L,M,N,O,P,Q,R,S,T,U,V,W,X,Y,Z';
$string['am'] = 'am';
$string['amcaps'] = 'AM';
$string['backupnameformat'] = '%Y%m%d-%H%M';
// See https://www.php.net/manual/en/dateinterval.format.php for format used in dateintervalxx strings.
$string['dateintervalday'] = '%ad';
$string['dateintervaldayfull'] = '%a days';
$string['dateintervaldayhr'] = '%ad %hh';
$string['dateintervaldayhrfull'] = '%a days %h hours';
$string['dateintervaldayhrmin'] = '%ad %hh %im';
$string['dateintervaldayhrminfull'] = '%a days %h hours %i minutes';
$string['dateintervaldayhrminsec'] = '%ad %hh %im %ss';
$string['dateintervaldayhrminsecfull'] = '%a days %h hours %i minutes %s seconds';
$string['dateintervaldayhrsec'] = '%ad %hh %ss';
$string['dateintervaldayhrsecfull'] = '%a days %h hours %s seconds';
$string['dateintervaldaymin'] = '%ad %im';
$string['dateintervaldayminfull'] = '%a days %i minutes';
$string['dateintervaldayminsec'] = '%ad %im %ss';
$string['dateintervaldayminsecfull'] = '%a days %i minutes %s seconds';
$string['dateintervaldaysec'] = '%ad %ss';
$string['dateintervaldaysecfull'] = '%a days %s seconds';
$string['dateintervalhr'] = '%hh';
$string['dateintervalhrfull'] = '%h hours';
$string['dateintervalhrmin'] = '%hh %im';
$string['dateintervalhrminfull'] = '%h hours %i minutes';
$string['dateintervalhrminsec'] = '%hh %im %ss';
$string['dateintervalhrminsecfull'] = '%h hours %i minutes %s seconds';
$string['dateintervalhrsec'] = '%hh %ss';
$string['dateintervalhrsecfull'] = '%h hours %s seconds';
$string['dateintervalmin'] = '%im';
$string['dateintervalminfull'] = '%i minutes';
$string['dateintervalminsec'] = '%im %ss';
$string['dateintervalminsecfull'] = '%i minutes %s seconds';
$string['dateintervalmo'] = '%mmo';
$string['dateintervalmoday'] = '%mmo %ad';
$string['dateintervalmodayfull'] = '%m months %a days';
$string['dateintervalmodayhr'] = '%mmo %ad %hh';
$string['dateintervalmodayhrfull'] = '%m months %a days %h hours';
$string['dateintervalmodayhrmin'] = '%mmo %ad %hh %im';
$string['dateintervalmodayhrminfull'] = '%m months %a days %h hours %i minutes';
$string['dateintervalmodayhrminsec'] = '%mmo %ad %hh %im %ss';
$string['dateintervalmodayhrminsecfull'] = '%m months %a days %h hours %i minutes %s seconds';
$string['dateintervalmodayhrsec'] = '%mmo %ad %hh %ss';
$string['dateintervalmodayhrsecfull'] = '%m months %a days %h hours %s seconds';
$string['dateintervalmodaymin'] = '%mmo %ad %im';
$string['dateintervalmodayminfull'] = '%m months %a days %i minutes';
$string['dateintervalmodayminsec'] = '%mmo %ad %im %ss';
$string['dateintervalmodayminsecfull'] = '%m months %a days %i minutes %s seconds';
$string['dateintervalmodaysec'] = '%mmo %ad %ss';
$string['dateintervalmodaysecfull'] = '%m months %a days %s seconds';
$string['dateintervalmofull'] = '%m months';
$string['dateintervalmohr'] = '%mmo %hh';
$string['dateintervalmohrfull'] = '%m months %h hours';
$string['dateintervalmohrmin'] = '%mmo %hh %im';
$string['dateintervalmohrminfull'] = '%m months %h hours %i minutes';
$string['dateintervalmohrminsec'] = '%mmo %hh %im %ss';
$string['dateintervalmohrminsecfull'] = '%m months %h hours %i minutes %s seconds';
$string['dateintervalmohrsec'] = '%mmo %hh %ss';
$string['dateintervalmohrsecfull'] = '%m months %h hours %s seconds';
$string['dateintervalmomin'] = '%mmo %im';
$string['dateintervalmominfull'] = '%m months %i minutes';
$string['dateintervalmominsec'] = '%mmo %im %ss';
$string['dateintervalmominsecfull'] = '%m months %i minutes %s seconds';
$string['dateintervalmosec'] = '%mmo %ss';
$string['dateintervalmosecfull'] = '%m months %s seconds';
$string['dateintervalsec'] = '%ss';
$string['dateintervalsecfull'] = '%s seconds';
$string['dateintervalyr'] = '%yy';
$string['dateintervalyrday'] = '%yy %ad';
$string['dateintervalyrdayfull'] = '%y years %a days';
$string['dateintervalyrdayhr'] = '%yy %ad %hh';
$string['dateintervalyrdayhrfull'] = '%y years %a days %h hours';
$string['dateintervalyrdayhrmin'] = '%yy %ad %hh %im';
$string['dateintervalyrdayhrminfull'] = '%y years %a days %h hours %i minutes';
$string['dateintervalyrdayhrminsec'] = '%yy %ad %hh %im %ss';
$string['dateintervalyrdayhrminsecfull'] = '%y years %a days %h hours %i minutes %s seconds';
$string['dateintervalyrdayhrsec'] = '%yy %ad %hh %ss';
$string['dateintervalyrdayhrsecfull'] = '%y years %a days %h hours %s seconds';
$string['dateintervalyrdaymin'] = '%yy %ad %im';
$string['dateintervalyrdayminfull'] = '%y years %a days %i minutes';
$string['dateintervalyrdayminsec'] = '%yy %ad %im %ss';
$string['dateintervalyrdayminsecfull'] = '%y years %a days %i minutes %s seconds';
$string['dateintervalyrdaysec'] = '%yy %ad %ss';
$string['dateintervalyrdaysecfull'] = '%y years %a days %s seconds';
$string['dateintervalyrfull'] = '%y years';
$string['dateintervalyrhr'] = '%yy %hh';
$string['dateintervalyrhrfull'] = '%y years %h hours';
$string['dateintervalyrhrmin'] = '%yy %hh %im';
$string['dateintervalyrhrminfull'] = '%y years %h hours %i minutes';
$string['dateintervalyrhrminsec'] = '%yy %hh %im %ss';
$string['dateintervalyrhrminsecfull'] = '%y years %h hours %i minutes %s seconds';
$string['dateintervalyrhrsec'] = '%yy %hh %ss';
$string['dateintervalyrhrsecfull'] = '%y years %h hours %s seconds';
$string['dateintervalyrmin'] = '%yy %im';
$string['dateintervalyrminfull'] = '%y years %i minutes';
$string['dateintervalyrminsec'] = '%yy %im %ss';
$string['dateintervalyrminsecfull'] = '%y years %i minutes %s seconds';
$string['dateintervalyrmo'] = '%yy %mmo';
$string['dateintervalyrmoday'] = '%yy %mmo %ad';
$string['dateintervalyrmodayfull'] = '%y years %m months %a days';
$string['dateintervalyrmodayhr'] = '%yy %mmo %ad %hh';
$string['dateintervalyrmodayhrfull'] = '%y years %m months %a days %h hours';
$string['dateintervalyrmodayhrmin'] = '%yy %mmo %ad %hh %im';
$string['dateintervalyrmodayhrminfull'] = '%y years %m months %a days %h hours %i minutes';
$string['dateintervalyrmodayhrminsec'] = '%yy %mmo %ad %hh %im %ss';
$string['dateintervalyrmodayhrminsecfull'] = '%y years %m months %a days %h hours %i minutes %s seconds';
$string['dateintervalyrmodayhrsec'] = '%yy %mmo %ad %hh %ss';
$string['dateintervalyrmodayhrsecfull'] = '%y years %m months %a days %h hours %s seconds';
$string['dateintervalyrmodaymin'] = '%yy %mmo %ad %im';
$string['dateintervalyrmodayminfull'] = '%y years %m months %a days %i minutes';
$string['dateintervalyrmodayminsec'] = '%yy %mmo %ad %im %ss';
$string['dateintervalyrmodayminsecfull'] = '%y years %m months %a days %i minutes %s seconds';
$string['dateintervalyrmodaysec'] = '%yy %mmo %ad %ss';
$string['dateintervalyrmodaysecfull'] = '%y years %m months %a days %s seconds';
$string['dateintervalyrmofull'] = '%y years %m months';
$string['dateintervalyrmohr'] = '%yy %mmo %hh';
$string['dateintervalyrmohrfull'] = '%y years %m months %h hours';
$string['dateintervalyrmohrmin'] = '%yy %mmo %hh %im';
$string['dateintervalyrmohrminfull'] = '%y years %m months %h hours %i minutes';
$string['dateintervalyrmohrminsec'] = '%yy %mmo %hh %im %ss';
$string['dateintervalyrmohrminsecfull'] = '%y years %m months %h hours %i minutes %s seconds';
$string['dateintervalyrmohrsec'] = '%yy %mmo %hh %ss';
$string['dateintervalyrmohrsecfull'] = '%y years %m months %h hours %s seconds';
$string['dateintervalyrmomin'] = '%yy %mmo %im';
$string['dateintervalyrmominfull'] = '%y years %m months %i minutes';
$string['dateintervalyrmominsec'] = '%yy %mmo %im %ss';
$string['dateintervalyrmominsecfull'] = '%y years %m months %i minutes %s seconds';
$string['dateintervalyrmosec'] = '%yy %mmo %ss';
$string['dateintervalyrmosecfull'] = '%y years %m months %s seconds';
$string['dateintervalyrsec'] = '%yy %ss';
$string['dateintervalyrsecfull'] = '%y years %s seconds';
$string['decsep'] = '.';
$string['firstdayofweek'] = '1';
$string['iso6391'] = 'en';
$string['iso6392'] = 'eng';
$string['labelsep'] = ': ';
$string['listsep'] = ',';
$string['locale'] = 'en_AU.UTF-8';
$string['localecldr'] = 'en-AU';
$string['localewin'] = 'English_Australia.1252';
$string['localewincharset'] = '';
$string['oldcharset'] = 'ISO-8859-1';
$string['parentlanguage'] = '';
$string['pm'] = 'pm';
$string['pmcaps'] = 'PM';
// Whether the question icon follows the language direction.
// E.g. In some RTL languages, like Arabic, it needs to be set to 'yes' so the question mark is flipped.
// But for Hebrew, the question mark character is not flipped, so it needs to be set to 'no'.
$string['questioniconfollowlangdirection'] = 'yes';
$string['strftimedate'] = '%d %B %Y';
$string['strftimedatemonthabbr'] = '%d %b %Y';
$string['strftimedatemonthtimeshort'] = '%d %b %Y, %I:%M';
$string['strftimedatefullshort'] = '%d/%m/%y';
$string['strftimedateshort'] = '%d %B';
$string['strftimedateshortmonthabbr'] = '%d %b';
$string['strftimedatetime'] = '%d %B %Y, %I:%M %p';
$string['strftimedatetimeaccurate'] = '%d %B %Y, %I:%M:%S %p';
$string['strftimedatetimeshort'] = '%d/%m/%y, %H:%M';
$string['strftimedatetimeshortaccurate'] = '%d/%m/%y, %H:%M:%S';
$string['strftimedaydate'] = '%A, %d %B %Y';
$string['strftimedaydatetime'] = '%A, %d %B %Y, %I:%M %p';
$string['strftimedayshort'] = '%A, %d %B';
$string['strftimedaytime'] = '%a, %H:%M';
$string['strftimemonth'] = '%B';
$string['strftimemonthyear'] = '%B %Y';
$string['strftimerecent'] = '%d %b, %H:%M';
$string['strftimerecentfull'] = '%a, %d %b %Y, %I:%M %p';
$string['strftimetime'] = '%I:%M %p';
$string['strftimetime12'] = '%I:%M %p';
$string['strftimetime24'] = '%H:%M';
$string['thisdirection'] = 'ltr';
$string['thisdirectionvertical'] = 'btt';
$string['thislanguage'] = 'English';
$string['thislanguageint'] = 'English';
$string['thousandssep'] = ',';
