{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_lti/tool_deeplinking_results

    This template lists the items that will be added to the course after
    a deep linking flow.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    *

    Example context (json):
    {
        "items": [
            {
                "name": "Chapter 1",
                "instructorchoiceacceptgrades": 0
            },
            {
                "name": "Quiz 1",
                "instructorchoiceacceptgrades": 1,
                "grade_modgrade_point": 20.5
            }
        ]
    }
}}
<p>
    {{#str}} contentitem_multiple_description, mod_lti {{/str}}
</p>
<ul>
    {{#items}}
        <li>
            <strong>{{{name}}}</strong>
            {{#instructorchoiceacceptgrades}}
                <em>{{#str}} contentitem_multiple_graded , mod_lti, {{{grade_modgrade_point}}} {{/str}}</em>
            {{/instructorchoiceacceptgrades}}
        </li>
    {{/items}}
</ul>
