{{!
    This file is part of Moodle - https://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_workshop/action_bar

    Actions bar at the top of the workshop pages.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * urlselect - The data object containing the required properties to render core/url_select.
    * heading - The heading of the page.

    Example context (json):
    {
        "urlselect": {
            "id": "url_select_test",
            "action": "https://example.com/post",
            "formid": "url_select_form",
            "sesskey": "sesskey",
            "classes": "urlselect",
            "label": "",
            "helpicon": false,
            "showbutton": null,
            "options": [
                {
                    "name": "Some name",
                    "value": "/mod/data/someurl.php",
                    "selected": false
                }
            ],
            "disabled": false,
            "title": null
        },
        "heading": "Print"
    }
}}
<div class="container-fluid tertiary-navigation">
    <div class="d-flex">
        {{#urlselect}}
            <div class="navitem">
                {{>core/url_select}}
            </div>
        {{/urlselect}}
    </div>
</div>
{{#heading}}<h3>{{heading}}</h3>{{/heading}}
