H5PEditor.language.core = {
  missingTranslation: '[변환 누락 :key]',
  loading: '로드하는 중, 잠시 기다리십시오...',
  selectLibrary: '콘텐츠를 위해 사용할 라이브러리를 선택하십시오.',
  unknownFieldPath: '":path" 를 찾을 수 없음.',
  notImageField: '":path"는 이미지가 아님',
  notImageOrDimensionsField: '":path"는 이미지 또는 치수 영역이 아님',
  requiredProperty: ':property 는 필수이며 값이 있어야 함',
  onlyNumbers: ':property 값은 숫자만 포함할 수 있음',
  illegalDecimalNumber: ':property는 최대  :decimals decimals를 가진 숫자만 포함할 수 있다.',
  exceedsMax: ':property 값이 최대값 :max 를 초과함.',
  listExceedsMax: '목록이 최대값 :max 항목을 초과함.',
  belowMin: ' :property 값이 최소값인 :min보다 낮음.',
  listBelowMin: '목록은 콘텐츠가 제대로 작동하려면 최소 :min 항목이 필요하다.',
  outOfStep: ':property 값은 :step의 단계에서만 변경할 수 있다.',
  add: '추가',
  addFile: '파일 추가',
  removeFile: '파일 제거',
  confirmRemoval: '이 :type을 제거하시겠습니까?',
  removeImage: '이미지 제거',
  confirmImageRemoval: '이렇게 하면 이미지가 제거됩니다. 정말 계속하시겠습니까?',
  changeFile: '파일 변경',
  changeLibrary: '콘텐츠 유형을 변경하시겠습니까?',
  semanticsError: 'Semantics error: :error',
  missingProperty: '영역 :index 에 :property 속성이 없습니다.',
  expandCollapse: '확대/축소',
  addEntity: ' :entity 더하기',
  tooLong: '영역 값이 너무 김. 최대 문자 또는 그 이하를 포함해야 함',
  invalidFormat: '영역 값에 금지된 형식 혹은 금지된 문자가 포함됨.',
  confirmChangeLibrary:
    '이 작업을 수행하면 현재 콘텐츠 유형으로 수행된 모든 작업이 손실됨. 콘텐츠 유형을 변경하시겠습니까?',
  commonFields: '텍스트 재정의 및 변환',
  commonFieldsDescription: '여기에서 이 콘텐츠의 설정을 편집하거나 텍스트를 변환할 수 있다.',
  uploading: '업로드 중, 잠시 기다리십시오...',
  noFollow: '필드 ":path"를 따를 수 없다.',
  editCopyright: '저작권 편집',
  close: '닫기',
  tutorial: '튜토리얼',
  editMode: '편집모드',
  listLabel: '목록',
  uploadError: '파일 업로드 오류',
  fileToLarge: '업로드하려는 파일이 너무 클 수 있음.',
  unknownFileUploadError: '알 수 없는 파일 업로드 오류',
  noSemantics: '오류, 콘텐츠 유형 양식을 로드할 수 없음.',
  editImage: '이미지 편집',
  saveLabel: '저장',
  cancelLabel: '취소',
  resetToOriginalLabel: '원본으로 돌아가기',
  loadingImageEditor: '이미지 편집기 로드 중, 잠시 기다리십시오...',
  selectFiletoUpload: '업로드할 파일 선택',
  or: '또는',
  enterAudioUrl: '오디오 소스 URL 입력',
  enterVideoUrl: '비디오 소스 URL 또는 YouTube 링크 입력',
  enterAudioTitle: '링크 혹은 다른 오디오 소스 URL 붙여넣기',
  enterVideoTitle: '유튜브 게시 링크 또는 기타 비디오 소스 URL 붙여넣기',
  uploadAudioTitle: '오디오 파일 업로드',
  uploadVideoTitle: '비디오 파일 업로드',
  addVideoDescription:
    'H5P는 비메오 프로처럼 mp4, webm 또는 ogv로 포맷된 모든 외부 비디오 소스를 지원하며, 유튜브 링크를 지원함.',
  insert: '삽입',
  cancel: '취소',
  height: '높이',
  width: '너비',
  textField: '텍스트 영역',
  numberField: '숫자 영역',
  orderItemUp: '아이템 위로',
  orderItemDown: '아이템 아래로',
  expandAllContent: 'Expand all content',
  collapseAllContent: 'Collapse all content',
  removeItem: '아이템 제거',
  hubPanelLabel: '콘텐츠 유형 선택',
  importantInstructions: '중요 지침',
  showImportantInstructions: '지침 보이기',
  hideImportantInstructions: '중요 지침 감추기',
  hide: '숨기기',
  example: '예시',
  createContentTabLabel: '콘텐츠 만들기',
  uploadTabLabel: '업로드',
  uploadPlaceholder: '파일 선택 없음',
  uploadInstructionsTitle: 'H5P 파일 업로드',
  uploadInstructionsContent:
    '<a href="https://h5p.org/content-types-and-applications" target="blank"의 예로부터 시작할 수 있습니다.',
  uploadFileButtonLabel: '파일 업로드',
  uploadFileButtonChangeLabel: '파일 바꾸기',
  uploadingThrobber: '업로드 중...',
  uploadSuccess: ':title 이 성공적으로 업로드 되었습니다!',
  unableToInterpretError: '응답 해석 불가',
  unableToInterpretSolution: '오류 로그를 확인하십시오.',
  h5pFileWrongExtensionTitle: '선택한 파일을 업로드할 수 없음',
  h5pFileWrongExtensionContent: '.h5p 확장명을 가진 파일만 허용됨.',
  h5pFileValidationFailedTitle: 'H5P 파일의 유효성을 검사할 수 없음.',
  h5pFileValidationFailedContent:
    '업로드된 H5P에 유효한 H5P 콘텐츠가 포함되어 있는지 확인하십시오. H5P 파일은 라이브러리를 포함하고 있을 때 H5P 라이브러리 페이지를 통해 업로드 되어야 함.',
  h5pFileUploadServerErrorTitle: 'H5P 파일을 업로드할 수 없음',
  h5pFileUploadServerErrorContent: '예상하지 않은 오류가 발생함. 서버 오류 로그 확인 더 자세한 내용.',
  contentTypeSectionAll: '모든 콘텐츠 유형',
  searchResults: '검색 결과',
  contentTypeSearchFieldPlaceholder: '콘텐츠 유형 검색',
  contentTypeInstallButtonLabel: '설치',
  contentTypeInstallingButtonLabel: '설치 중',
  contentTypeUseButtonLabel: '사용',
  contentTypeDetailButtonLabel: '세부내용',
  contentTypeUpdateButtonLabel: '업데이트',
  contentTypeUpdatingButtonLabel: '업데이트중',
  contentTypeGetButtonLabel: '가져오기',
  contentTypeBackButtonLabel: '돌아가기',
  contentTypeIconAltText: '아이콘',
  contentTypeInstallSuccess: ':contentType 가 성공적으로 설치됨',
  contentTypeUpdateSuccess: ':contentType 가 성곡적으로 업데이트 됨!',
  contentTypeInstallError: ':contentType 를 설치할 수 없음. 관리자에게 문의',
  contentTypeLicensePanelTitle: '라이선스',
  contentTypeDemoButtonLabel: '콘텐츠 데모',
  numResults: ':num 결과',
  show: '보여주기',
  recentlyUsedFirst: '최근 사용 항목 먼저 보여주기',
  popularFirst: '인기 있는 항목 먼저 보여주기',
  newestFirst: '새 항목 먼저 보여주기',
  aToZ: 'A에서Z 순서',
  noResultsFound: '결과 없음',
  noResultsFoundDesc: '검색 기준에 맞는 콘텐츠 유형이 없음',
  readMore: '더 읽기',
  readLess: '덜 읽기',
  contentTypeOwner: '소유자 :owner',
  contentTypeUnsupportedApiVersionTitle: '이 컨텐츠 유형에는 새로운 코어 버전이 필요함',
  contentTypeUnsupportedApiVersionContent: '필요한 업데이트 제공을 위해 시스템 관리자에게 문의하십시오.',
  contentTypeUpdateAvailable: '업데이트 사용 가능',
  contentTypeRestricted: '제한된 콘텐츠 유형',
  contentTypeRestrictedDesc: '관리자에 의해 이 컨텐츠 유형의 사용이 제한됨',
  contentTypeNotInstalled: '콘텐츠 유형이 설치되지 않음',
  contentTypeNotInstalledDesc: '콘텐츠 유형을 설치할 수 있는 권한이 없음.',
  theContentType: '콘텐츠 유형',
  currentMenuSelected: '현재 선택',
  errorCommunicatingHubTitle: '허브와 통신할 수 없음.',
  errorCommunicatingHubContent: '에러가 발생함. 다시 시도하십시오.',
  warningNoContentTypesInstalled: '설치된 콘텐츠 유형이 없습니다.',
  warningChangeBrowsingToSeeResults: '설치할 수 있는 모든 콘텐츠 유형 목록을 보려면 <em>All</em>(모두)을 클릭하십시오.',
  warningUpdateAvailableTitle: '새로운 버전의 :content 유형을 사용할 수 있음',
  warningUpdateAvailableBody: '더 나은 경험을 위해 최신 버전으로 업데이트',
  licenseDescription:
    '이 라이선스의 일부 기능은 다음과 같다. 원래 라이센스 텍스트를 읽으려면 위의 정보 아이콘을 클릭하십시오.',
  licenseModalTitle: '라이선스 세부 정보',
  licenseModalSubtitle: '적절한 사용에 대한 정보를 보려면 라이선스를 선택하십시오.',
  licenseUnspecified: '지정되지 않음',
  licenseCanUseCommercially: '상업적으로 사용할 수 있음',
  licenseCanModify: '수정 가능',
  licenseCanDistribute: '배포할 수 있음',
  licenseCanSublicense: '하위 라이선스(sublicense) 가능 ',
  licenseCanHoldLiable: '책임(liable) 포함 가능',
  licenseCannotHoldLiable: '책임(liable) 포함 불가능',
  licenseMustIncludeCopyright: '저작권을 포함해야 함',
  licenseMustIncludeLicense: '라이선스를 포함해야 함',
  licenseFetchDetailsFailed: '라이선스 세부 정보 가져오기 실패',
  imageLightboxTitle: '이미지',
  imageLightBoxProgress: '총 :total 중 :num ',
  nextImage: '다음 이미지',
  previousImage: '이전 이미지',
  screenshots: '스크린샷',
  reloadButtonLabel: '재로드',
  videoQuality: '비디오 품질 라벨',
  videoQualityDescription:
    '이 라벨은 사용자가 동영상의 현재 품질을 식별하는 데 도움이 됨. 예를 들어 1080p, 720p, HD 또는 Mobile',
  videoQualityDefaultLabel: '품질 :index',
  noContentTypesAvailable: '사용할 수 있는 콘텐츠 유형 없음',
  noContentTypesAvailableDesc:
    '당신의 사이트는 H5P.org에 접속하여 사용 가능한 콘텐츠 유형을 나열하는 데 어려움을 겪고 있음.',
  contentTypeCacheOutdated: '콘텐츠 유형 목록이 오래됨',
  contentTypeCacheOutdatedDesc:
    '당신의 사이트는 컨텐츠 유형 업데이트를 확인하기 위해 H5P.org에 연결하는 데 어려움을 겪고 있음. 새로운 콘텐츠 유형을 업데이트하거나 설치하지 못할 수 있다.',
  tryAgain: '재시도',
  getHelp: '도움',
  untitled: '제목없음 :libraryTitle',
  title: '제목',
  metadata: '메타데이터',
  addTitle: '제목추가',
  usedForSearchingReportsAndCopyrightInformation: '검색, 보고서 및 저작권 정보에 사용됨',
  metadataSharingAndLicensingInfo: '메타데이터 (공유 및 라이센싱 정보)',
  fillInTheFieldsBelow: '아래 영역에 입력',
  saveMetadata: '메타데이터 저장',
  addAuthor: '작성자 저장',
  confirmRemoveAuthor: '이 작성자를 제거하시겠습니까?',
  addNewChange: '새 변화 추가',
  confirmDeleteChangeLog: '이 변경 로그 항목을 삭제하시겠습니까?',
  changelogDescription:
    '일부 라이선스는 원본 작업에 대한 변경 사항을 요구하거나 파생 모델을 기록하여 표시해야 한다. 라이센싱상의 이유로 변경사항을 여기에 기록하거나 자신과 다른 사람이 이 콘텐츠에 대한 변경사항을 추적할 수 있도록 허용하십시오.',
  logThisChange: '변경 기록',
  newChangeHasBeenLogged: '새 변경 사항이 기록됨',
  loggedChanges: '변경 기록',
  noChangesHaveBeenLogged: '변경 사항이 기록되지 않음',
  errorHeader: '에러가 발생함',
  errorCalculatingMaxScore:
    '이 컨텐츠의 최대 점수를 계산할 수 없음. 최대 점수는 0으로 가정한다. 올바르지 않으면 관리자에게 문의하십시오.',
  maxScoreSemanticsMissing: '콘텐츠에서 기대되는 의미 있는 문장(semantics)를 찾을 수 없음.',
  copyButton: '복사',
  copiedButton: '복사완료',
  pasteButton: '붙여넣기',
  pasteAndReplaceButton: '붙여넣기 및 대체하기',
  pasteContent: '콘텐츠 대체하기',
  confirmPasteContent:
    '이 작업을 수행하면 현재 내용을 클립보드의 내용으로 바꿀 수 있음. 현재 내용은 손실됨. 계속하시겠습니까?',
  confirmPasteButtonText: '콘텐츠 대체하기',
  copyToClipboard: 'H5P콘텐츠를 클립보드에 복사하기',
  copiedToClipboard: '콘텐츠가 클립보드로 복사되었습니다.',
  pasteFromClipboard: '클립보드에서 H5P 콘텐츠 붙여넣기',
  pasteAndReplaceFromClipboard: '클립보드의 H5P 콘텐츠로 기존 콘텐츠 바꾸기',
  pasteNoContent: '클립보드에 H5P 콘텐츠 없음',
  pasteError: '클립보드에서 붙여넣을 수 없음',
  pasteContentNotSupported: 'H5P 클립보드의 콘텐츠는 이 컨텍스트에서 지원되지 않음',
  pasteContentRestricted: '클립보드의 콘텐츠가 이 사이트에서 제한됨',
  pasteTooOld:
    'H5P 클립보드의 콘텐츠는 이 컨텍스트(:local)에서 지원되는 것보다 낮은 버전(:clip)이며, 가능하면 붙여넣을 콘텐츠를 업그레이드한 후 다시 복사한 후 여기에 붙여 넣으십시오.',
  pasteTooNew:
    'H5P 클립보드의 콘텐츠는 이 컨텍스트에서 지원되는 것보다 더 높은 버전(:clip)이며, 가능하면 이 콘텐츠를 먼저 업그레이드한 다음 여기에 다시 붙여 넣으십시오.',
  ok: 'OK',
  avTablistLabel: '삽입하기',
  tabTitleBasicFileUpload: '파일업로드',
  tabTitleInputLinkURL: '링크/URL',
  errorTooHighVersion: '%Supported 혹은 이전 버전이 지원됨에도 매개 변수가 %used를 포함함.',
  errorNotSupported: '매개변수는 지원되지 않는 %used를 포함한다.',
  errorParamsBroken: '매개변수 오류',
  libraryMissing: '필요한 라이브러리 %lib 누락',
  scriptMissing: '%lib에 대한 업그레이드 스크립트를 로드할 수 없음.',
  language: '언어',
  noLanguagesSupported: '지원되는 언어 없음',
  changeLanguage: '언어를 :language 로 변경하시겠습니까?',
  thisWillPotentially:
    '이것은 잠재적으로 모든 텍스트와 변환을 재설정할 것이다. 이는 되돌릴 수 없고 콘텐츠 자체가 바뀌지 않을 것입니다. 계속하시겠습니까?',
  notAllTextsChanged: '모든 텍스트가 변경된 것은 아니며 :language 에 대한 부분적인 적용 범위만 있을 뿐임.',
  contributeTranslations:
    ':language 에 대한 번역을 완료하려면, <a href=":url" target="_new">contributing translations to H5P</a> (H5P 번역에 기여) 에 대해 배울 수 있음.',
  unknownLibrary: "불행히도 선택한 콘텐츠 유형 '%lib'이(가) 이 시스템에 설치되지 않았음.",
  proceedButtonLabel: '저장 진행',
  enterFullscreenButtonLabel: '전체화면',
  exitFullscreenButtonLabel: '전체화면 종료',
  noContentHeader: '알맞은 콘텐츠가 없습니까?',
  noContentSuggestion: '직접 콘텐츠를 만들어 보세요!',
  tutorials: '튜토리얼',
  contentSectionAll: '모든 공유 콘텐츠',
  popularContent: '인기 콘텐츠',
  allPopular: '모든 인기 콘텐츠',
  newOnTheHub: '허브의 새 콘텐츠',
  allNew: '모든 새 콘텐츠',
  filterBy: '검색 조건',
  filter: '검색',
  filters: {
    level: { dropdownLabel: '레벨', dialogHeader: '교육 레벨 선택', dialogButtonLabel: '교육 레벨에 따라 검색' },
    language: {
      dropdownLabel: '언어',
      dialogHeader: '언어 선택',
      dialogButtonLabel: '언어에 따라 검색',
      searchPlaceholder: '하나 이상의 언어 선택',
    },
    reviewed: {
      dropdownLabel: '검토됨',
      dialogHeader: '검토가 된 콘텐츠',
      dialogButtonLabel: '검색',
      optionLabel: '검토된 내용만 표시',
    },
    contentTypes: {
      dropdownLabel: '콘텐츠 유형',
      dialogHeader: '콘텐츠 유형 선택',
      dialogButtonLabel: '콘텐츠 유형 검색',
      searchPlaceholder: '하나 이상의 콘텐츠 유형 선택',
    },
    disciplines: {
      dropdownLabel: '과목',
      dialogHeader: '과목 선택',
      dialogButtonLabel: '과목 검색',
      searchPlaceholder: '하나 이상의 과목 선택',
    },
    licenses: {
      dropdownLabel: '라이선스',
      dialogHeader: '선호하는 사용 권한 선택',
      dialogButtonLabel: '라이선스에 따라 검색',
      options: { modified: '수정할 수 있음', commercial: '상업적 사용 허용' },
    },
  },
  clearFilters: '모든 검색 조건 지우기',
  contentSearchFieldPlaceholder: '콘텐츠 검색',
  loadingContentTitle: '컨텐츠를 로드하는 중...',
  loadingContentSubtitle: '잠시만 기다리십시오',
  by: '저작자',
  dropdownButton: '드랍다운 열기',
  paginationNavigation: '페이징 네비게이션',
  page: '페이지',
  currentPage: '현재 페이지',
  nextPage: '다음 페이지로 이동',
  previousPage: '이전 페이지로 이동',
  contentPreviewButtonLabel: '미리보기',
  contentDownloadButtonLabel: '콘텐츠 가져오기',
  reuseContentTabLabel: '공유 콘텐츠 가져오기',
  contentPublisherPanelHeader: '게시자 정보',
  noContentFoundDesc: '검색 기준에 맞는 콘텐츠가 없음.',
  h5pType: 'H5P 유형',
  level: '레벨',
  size: '크기',
  failedFetchingData: '데이터 가져오기 실패',
  filterErrorMessage: '오류가 있습니다. 페이지를 다시 로드하십시오.',
  in: 'in',
  navigateToParent: '상위 탐색',
  a11yTitleShowLabel: 'AT 에 대한 라벨 표시',
  a11yTitleHideLabel: 'AT 에 대한 레이블 숨기기',
  reuseSuccess: ':title 이 H5P 허브에서 성공적으로 가져오기 되었습니다.',
};
