{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_message/add_contact_button

    Template for the contents of the add contact button on the user's profile page.

    Context variables required for this template:
    * displaytextlabel - Whether to display text next to the action icon.

    Example context (json):
    {
        "displaytextlabel": true
    }
}}
<span>
    {{^displaytextlabel}}
        {{#pix}} t/addcontact, core, {{#str}} addtoyourcontacts, message {{/str}} {{/pix}}
    {{/displaytextlabel}}
    {{#displaytextlabel}}
        {{#pix}} t/addcontact, core {{/pix}}
        <span class="header-button-title">{{#str}} addtoyourcontacts, message {{/str}}</span>
    {{/displaytextlabel}}
</span>
{{> core/loading }}
