/* Administration interface styling */

.h5p-content {
  border: 1px solid #DDD;
  border-radius: 3px;
  padding: 10px;
}

.h5p-admin-table,
.h5p-admin-table > tbody {
  border: none;
  width: 100%;
}

.h5p-admin-table tr:nth-child(odd),
.h5p-data-view tr:nth-child(odd) {
  background-color: #F9F9F9;
}
.h5p-admin-table tbody tr:hover {
  background-color: #EEE;
}
.h5p-admin-table.empty {
  padding: 1em;
  background-color: #EEE;
  font-size: 1.2em;
  font-weight: bold;
}

.h5p-admin-table.libraries th:last-child,
.h5p-admin-table.libraries td:last-child {
  text-align: right;
}

.h5p-admin-buttons-wrapper {
  white-space: nowrap;
}

.h5p-admin-table.libraries button {
  font-size: 2em;
  cursor: pointer;
  border: 1px solid #AAA;
  border-radius: .2em;
  background-color: #e0e0e0;
  text-shadow: 0 0 0.5em #fff;
  padding: 0;
  line-height: 1em;
  width: 1.125em;
  height: 1.05em;
  text-indent: -0.125em;
  margin: 0.125em 0.125em 0 0.125em;
}
.h5p-admin-upgrade-library:before {
  font-family: 'H5P';
  content: "\e888";
}
.h5p-admin-view-library:before {
  font-family: 'H5P';
  content: "\e889";
}
.h5p-admin-delete-library:before {
  font-family: 'H5P';
  content: "\e890";
}

.h5p-admin-table.libraries button:hover {
  background-color: #d0d0d0;
}
.h5p-admin-table.libraries button:disabled:hover {
  background-color: #e0e0e0;
  cursor: default;
}

.h5p-admin-upgrade-library {
  color: #339900;
}
.h5p-admin-view-library {
  color: #0066cc;
}
.h5p-admin-delete-library {
  color: #990000;
}
.h5p-admin-delete-library:disabled,
.h5p-admin-upgrade-library:disabled {
  cursor: default;
  color: #c0c0c0;
}

.h5p-library-info {
  padding: 1em 1em;
  margin: 1em 0;

  width: 350px;

  border: 1px solid #DDD;
  border-radius: 3px;
}

/* Labeled field (label + value) */
.h5p-labeled-field {
  border-bottom: 1px solid #ccc;
}
.h5p-labeled-field:last-child {
  border-bottom: none;
}

.h5p-labeled-field .h5p-label {
  display: inline-block;
  min-width: 150px;
  font-size: 1.2em;
  font-weight: bold;
  padding: 0.2em;
}

.h5p-labeled-field .h5p-value {
  display: inline-block;
  padding: 0.2em;
}

/* Search element */
.h5p-content-search {
  display: inline-block;
  position: relative;

  width: 100%;
  padding: 5px 0;
  margin-top: 10px;

  border: 1px solid #CCC;
  border-radius: 3px;
  box-shadow: 2px 2px 5px #888888;
}
.h5p-content-search:before {
  font-family: 'H5P';
  vertical-align: bottom;
  content: "\e88a";
  font-size: 2em;
  line-height: 1.25em;
}
.h5p-content-search input {
  font-size: 120%;
  line-height: 120%;
}
.h5p-admin-search-results {
  margin-left: 10px;
  color: #888;
}

.h5p-admin-pager-size-selector {
  position: absolute;
  right: 10px;
  top: .75em;
  display: inline-block;
}
.h5p-admin-pager-size-selector > span {
  padding: 5px;
  margin-left: 10px;
  cursor: pointer;
  border: 1px solid #CCC;
  border-radius: 3px;
}
.h5p-admin-pager-size-selector > span.selected {
  background-color: #edf5fa;
}
.h5p-admin-pager-size-selector > span:hover {
  background-color: #555;
  color: #FFF;
}

/* Generic "javascript"-action button */
button.h5p-admin {
  border: 1px solid #AAA;
  border-radius: 5px;
  padding: 3px 10px;
  background-color: #EEE;
  cursor: pointer;
  display: inline-block;
  text-align: center;
  color: #222;
}
button.h5p-admin:hover {
  background-color: #555;
  color: #FFF;
}
button.h5p-admin.disabled,
button.h5p-admin.disabled:hover {
  cursor: auto;
  color: #CCC;
  background-color: #FFF;
}

/* Pager element */
.h5p-content-pager {
  display: inline-block;
  border: 1px solid #CCC;
  border-radius: 3px;
  box-shadow: 2px 2px 5px #888888;
  width: 100%;
  text-align: center;
  padding: 3px 0;
}
.h5p-content-pager > button {
  min-width: 80px;
  font-size: 130%;
  line-height: 130%;
  border: none;
  background: none;
  font-family: 'H5P';
  font-size: 1.4em;
}
.h5p-content-pager > button:focus {
  outline: 0;
}
.h5p-content-pager > button:last-child {
  margin-left: 10px;
}
.h5p-content-pager > .pager-info {
  cursor: pointer;
  padding: 5px;
  border-radius: 3px;
}
.h5p-content-pager > .pager-info:hover {
  background-color: #555;
  color: #FFF;
}
.h5p-content-pager > .pager-info,
.h5p-content-pager > .h5p-pager-goto {
  margin: 0 10px;
  line-height: 130%;
  display: inline-block;
}

.h5p-admin-header {
  margin-top: 1.5em;
}
#h5p-library-upload-form.h5p-admin-upload-libraries-form,
#h5p-content-type-cache-update-form.h5p-admin-upload-libraries-form {
  position: relative;
  margin: 0;

}
.h5p-admin-upload-libraries-form .form-submit {
  position: absolute;
  top: 0;
  right: 0;
}
.h5p-spinner {
  padding: 0 0.5em;
  font-size: 1.5em;
  font-weight: bold;
}
#h5p-admin-container .h5p-admin-center {
  text-align: center;
}
.h5p-pagination {
  text-align: center;
}
.h5p-pagination > span, .h5p-pagination > input {
  margin: 0 1em;
}
.h5p-data-view input[type="text"] {
  margin-bottom: 0.5em;
  margin-right: 0.5em;
  float: left;
}
.h5p-data-view input[type="text"]::-ms-clear {
  display: none;
}

.h5p-data-view .h5p-others-contents-toggler-wrapper {
  float: right;
  line-height: 2;
  margin-right: 0.5em;
}

.h5p-data-view .h5p-others-contents-toggler-label {
  font-size: 14px;
}

.h5p-data-view .h5p-others-contents-toggler {
  margin-right: 0.5em;
}

.h5p-data-view th[role="button"] {
  cursor: pointer;
}
.h5p-data-view th[role="button"].h5p-sort:after,
.h5p-data-view th[role="button"]:hover:after,
.h5p-data-view th[role="button"].h5p-sort.h5p-reverse:hover:after {
  content: "\25BE";
  position: relative;
  left: 0.5em;
  top: -1px;
}
.h5p-data-view th[role="button"].h5p-sort.h5p-reverse:after,
.h5p-data-view th[role="button"].h5p-sort:hover:after {
  content: "\25B4";
  top: -2px;
}
.h5p-data-view th[role="button"]:hover:after,
.h5p-data-view th[role="button"].h5p-sort.h5p-reverse:hover:after,
.h5p-data-view th[role="button"].h5p-sort:hover:after {
  color: #999;
}
.h5p-data-view .h5p-facet {
  cursor: pointer;
  color: #0073aa;
  outline: none;
}
.h5p-data-view .h5p-facet:hover,
.h5p-data-view .h5p-facet:active {
  color: #00a0d2;
}
.h5p-data-view .h5p-facet:focus {
  color: #124964;
  box-shadow: 0 0 0 1px #5b9dd9,0 0 2px 1px rgba(30,140,190,.8);
}
.h5p-data-view .h5p-facet-wrapper {
  line-height: 23px;
}
.h5p-data-view .h5p-facet-tag {
  margin: 2px 0 0 0.5em;
  font-size: 12px;
  background: #e8e8e8;
  border: 1px solid #cbcbcc;
  border-radius: 5px;
  color: #5d5d5d;
  padding: 0 24px 0 10px;
  display: inline-block;
  position: relative;
}
.h5p-data-view .h5p-facet-tag > span {
  position: absolute;
  right: 0;
  top: auto;
  bottom: auto;
  font-size: 18px;
  color: #a2a2a2;
  outline: none;
  width: 21px;
  text-indent: 4px;
  letter-spacing: 10px;
  overflow: hidden;
  cursor: pointer;
}
.h5p-data-view .h5p-facet-tag > span:before {
  content: "×";
  font-weight: bold;
}
.h5p-data-view .h5p-facet-tag > span:hover,
.h5p-data-view .h5p-facet-tag > span:focus {
  color: #a20000;
}
.h5p-data-view .h5p-facet-tag > span:active {
  color: #d20000;
}
.content-upgrade-log {
  color: red;
}
