{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_group/group_details

    Template for the Groups page.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * name string Group Name
    * pictureurl string Group image url
    * description string Group description
    * edit string edit link to edit the group

    Example context (json):
    {
        "name": "Group Name",
        "pictureurl": "https://raw.githubusercontent.com/moodle/moodle/master/pix/g/f1.png",
        "description": "This is the description for Group Name",
        "editurl": "http://www.moodle.org"
    }
}}
{{#name}}
<div class="groupinfobox container-fluid card p-3">
    {{#pictureurl}}
    <div class="group-image"><img class="grouppicture" src="{{{pictureurl}}}" alt="{{{name}}}" title="{{{name}}}"/></div>
    {{/pictureurl}}
    {{#editurl}}
    <div class="group-edit"><a href="{{editurl}}">{{#pix}}t/edit, core, {{#str}}editgroupprofile{{/str}}{{/pix}}</a></div>
    {{/editurl}}
    <h3 class="">{{{name}}}</h3>
    <div class="group-description">{{{description}}}</div>
</div>
{{/name}}
