{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_forum/setting_switch

    This template will render a toggle switch, which can be used for the forum settings drawer.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * none

    Example context (json):
    {}

}}
<div>
    <div class="form-check form-switch mb-1">
        <input type="checkbox" class="form-check-input" id="{{$switchid}}{{uniqid}}{{/switchid}}"
            data-type="{{$type}}toggle-switch{{/type}}" data-action="toggle"
            {{$otherattributes}}{{/otherattributes}}>
        <label class="form-check-label" for="{{$switchid}}{{uniqid}}{{/switchid}}">
            {{$labeltext}}{{/labeltext}}
        </label>
    </div>
</div>
