{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_h5pactivity/reportlink

    This template will render the report link inside a H5P activity.

    Variables required for this template:
    * url - The URL to the report page
    * message - The link message

    Example context (json):
    {
        "url": "#",
        "message": "View attempts list (3 submitted)"
    }

}}
<div class="reportlink mb-3">
    <a href="{{{url}}}">{{message}}</a>
</div>
