<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 *
 * Strings describing subject classification (Codes are from ASCED 2001)
 *
 * @package   core
 * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['field101'] = 'Natural and Physical Science';
$string['field10101'] = 'Mathematical Sciences';
$string['field1010101'] = 'Mathematics';
$string['field1010103'] = 'Statistics';
$string['field1010199'] = 'Mathematical Sciences (Other)';
$string['field10103'] = 'Physics and Astronomy';
$string['field1010301'] = 'Physics';
$string['field1010303'] = 'Astronomy';
$string['field10105'] = 'Chemical Sciences';
$string['field1010501'] = 'Organic Chemistry';
$string['field1010503'] = 'Inorganic Chemistry';
$string['field1010599'] = 'Chemical Sciences (Other)';
$string['field10107'] = 'Earth Sciences';
$string['field1010701'] = 'Atmospheric Sciences';
$string['field1010703'] = 'Geology';
$string['field1010705'] = 'Geophysics';
$string['field1010707'] = 'Geochemistry';
$string['field1010709'] = 'Soil Science';
$string['field1010711'] = 'Hydrology';
$string['field1010713'] = 'Oceanography';
$string['field1010799'] = 'Earth Sciences (Other)';
$string['field10109'] = 'Biological Sciences';
$string['field1010901'] = 'Biochemistry and Cell Biology';
$string['field1010903'] = 'Botany';
$string['field1010905'] = 'Ecology and Evolution';
$string['field1010907'] = 'Marine Science';
$string['field1010909'] = 'Genetics';
$string['field1010911'] = 'Microbiology';
$string['field1010913'] = 'Human Biology';
$string['field1010915'] = 'Zoology';
$string['field1010999'] = 'Biological Sciences (Other)';
$string['field10199'] = 'Other Natural and Physical Sciences';
$string['field1019901'] = 'Medical Science';
$string['field1019903'] = 'Forensic Science';
$string['field1019905'] = 'Food Science and Biotechnology';
$string['field1019907'] = 'Pharmacology';
$string['field1019909'] = 'Laboratory Technology';
$string['field1019999'] = 'Natural and Physical Sciences (Other)';
$string['field102'] = 'Information Technology';
$string['field10201'] = 'Computer Science';
$string['field1020101'] = 'Formal Language Theory';
$string['field1020103'] = 'Programming';
$string['field1020105'] = 'Computational Theory';
$string['field1020107'] = 'Compiler Construction';
$string['field1020109'] = 'Algorithms';
$string['field1020111'] = 'Data Structures';
$string['field1020113'] = 'Networks and Communications';
$string['field1020115'] = 'Computer Graphics';
$string['field1020117'] = 'Operating Systems';
$string['field1020119'] = 'Artificial Intelligence';
$string['field1020199'] = 'Computer Science (Other)';
$string['field10203'] = 'Information Systems';
$string['field1020301'] = 'Conceptual Modelling';
$string['field1020303'] = 'Database Management';
$string['field1020305'] = 'Systems Analysis and Design';
$string['field1020307'] = 'Decision Support Systems';
$string['field1020399'] = 'Information Systems (Other)';
$string['field10299'] = 'Other Information Technology';
$string['field1029901'] = 'Security Science';
$string['field1029999'] = 'Information Technology (Other)';
$string['field103'] = 'Engineering and Related Technologies';
$string['field10301'] = 'Manufacturing Engineering and Technology';
$string['field1030101'] = 'Manufacturing Engineering';
$string['field1030103'] = 'Printing';
$string['field1030105'] = 'Textile Making';
$string['field1030107'] = 'Garment Making';
$string['field1030109'] = 'Footwear Making';
$string['field1030111'] = 'Wood Machining and Turning';
$string['field1030113'] = 'Cabinet Making';
$string['field1030115'] = 'Furniture Upholstery and Renovation';
$string['field1030117'] = 'Furniture Polishing';
$string['field1030199'] = 'Manufacturing Engineering and Technology (Other)';
$string['field10303'] = 'Process and Resources Engineering';
$string['field1030301'] = 'Chemical Engineering';
$string['field1030303'] = 'Mining Engineering';
$string['field1030305'] = 'Materials Engineering';
$string['field1030307'] = 'Food Processing Technology';
$string['field1030399'] = 'Process and Resources Engineering (Other)';
$string['field10305'] = 'Automotive Engineering and Technology';
$string['field1030501'] = 'Automotive Engineering';
$string['field1030503'] = 'Vehicle Mechanics';
$string['field1030505'] = 'Automotive Electrics and Electronics';
$string['field1030507'] = 'Automotive Vehicle Refinishing';
$string['field1030509'] = 'Automotive Body Construction';
$string['field1030511'] = 'Panel Beating';
$string['field1030513'] = 'Upholstery and Vehicle Trimming';
$string['field1030515'] = 'Automotive Vehicle Operations';
$string['field1030599'] = 'Automotive Engineering and Technology (Other)';
$string['field10307'] = 'Mechanical and Industrial Engineering and Technology';
$string['field1030701'] = 'Mechanical Engineering';
$string['field1030703'] = 'Industrial Engineering';
$string['field1030705'] = 'Toolmaking';
$string['field1030707'] = 'Metal Fitting, Turning and Machining';
$string['field1030709'] = 'Sheetmetal Working';
$string['field1030711'] = 'Boilermaking and Welding';
$string['field1030713'] = 'Metal Casting and Patternmaking';
$string['field1030715'] = 'Precision Metalworking';
$string['field1030717'] = 'Plant and Machine Operations';
$string['field1030799'] = 'Mechanical and Industrial Engineering and Technology (Other)';
$string['field10309'] = 'Civil Engineering';
$string['field1030901'] = 'Construction Engineering';
$string['field1030903'] = 'Structural Engineering';
$string['field1030905'] = 'Building Services Engineering';
$string['field1030907'] = 'Water and Sanitary Engineering';
$string['field1030909'] = 'Transport Engineering';
$string['field1030911'] = 'Geotechnical Engineering';
$string['field1030913'] = 'Ocean Engineering';
$string['field1030999'] = 'Civil Engineering (Other)';
$string['field10311'] = 'Geomatic Engineering';
$string['field1031101'] = 'Surveying';
$string['field1031103'] = 'Mapping Science';
$string['field1031199'] = 'Geomatic Engineering (Other)';
$string['field10313'] = 'Electrical and Electronic Engineering and Technology';
$string['field1031301'] = 'Electrical Engineering';
$string['field1031303'] = 'Electronic Engineering';
$string['field1031305'] = 'Computer Engineering';
$string['field1031307'] = 'Communications Technologies';
$string['field1031309'] = 'Communications Equipment Installation and Maintenance';
$string['field1031311'] = 'Powerline Installation and Maintenance';
$string['field1031313'] = 'Electrical Fitting, Electrical Mechanics';
$string['field1031315'] = 'Refrigeration and Air Conditioning Mechanics';
$string['field1031317'] = 'Electronic Equipment Servicing';
$string['field1031399'] = 'Electrical and Electronic Engineering and Technology (Other)';
$string['field10315'] = 'Aerospace Engineering and Technology';
$string['field1031501'] = 'Aerospace Engineering';
$string['field1031503'] = 'Aircraft Maintenance Engineering';
$string['field1031505'] = 'Aircraft Operation';
$string['field1031507'] = 'Air Traffic Control';
$string['field1031599'] = 'Aerospace Engineering and Technology (Other)';
$string['field10317'] = 'Maritime Engineering and Technology';
$string['field1031701'] = 'Maritime Engineering';
$string['field1031703'] = 'Marine Construction';
$string['field1031705'] = 'Marine Craft Operation';
$string['field1031799'] = 'Maritime Engineering and Technology (Other)';
$string['field10399'] = 'Other Engineering and Related Technologies';
$string['field1039901'] = 'Environmental Engineering';
$string['field1039903'] = 'Biomedical Engineering';
$string['field1039905'] = 'Fire Technology';
$string['field1039907'] = 'Rail Operations';
$string['field1039909'] = 'Cleaning';
$string['field1039999'] = 'Engineering and Related Technologies (Other)';
$string['field104'] = 'Architecture and Building';
$string['field10401'] = 'Architecture and Urban Environment';
$string['field1040101'] = 'Architecture';
$string['field1040103'] = 'Urban Design and Regional Planning';
$string['field1040105'] = 'Landscape Architecture';
$string['field1040107'] = 'Interior and Environmental Design';
$string['field1040199'] = 'Architecture and Urban Environment (Other)';
$string['field10403'] = 'Building';
$string['field1040301'] = 'Building Science and Technology';
$string['field1040303'] = 'Building Construction Management';
$string['field1040305'] = 'Building Surveying';
$string['field1040307'] = 'Building Construction Economics';
$string['field1040309'] = 'Bricklaying and Stonemasonry';
$string['field1040311'] = 'Carpentry and Joinery';
$string['field1040313'] = 'Ceiling, Wall and Floor Fixing';
$string['field1040315'] = 'Roof Fixing';
$string['field1040317'] = 'Plastering';
$string['field1040319'] = 'Furnishing Installation';
$string['field1040321'] = 'Floor Coverings';
$string['field1040323'] = 'Glazing';
$string['field1040325'] = 'Painting, Decorating and Sign Writing';
$string['field1040327'] = 'Plumbing';
$string['field1040329'] = 'Scaffolding and Rigging';
$string['field1040399'] = 'Building (Other)';
$string['field105'] = 'Agriculture, Environmental and Related Studies';
$string['field10501'] = 'Agriculture';
$string['field1050101'] = 'Agricultural Science';
$string['field1050103'] = 'Wool Science';
$string['field1050105'] = 'Animal Husbandry';
$string['field1050199'] = 'Agriculture (Other)';
$string['field10503'] = 'Horticulture and Viticulture';
$string['field1050301'] = 'Horticulture';
$string['field1050303'] = 'Viticulture';
$string['field10505'] = 'Forestry Studies';
$string['field1050501'] = 'Forestry Studies';
$string['field10507'] = 'Fisheries Studies';
$string['field1050701'] = 'Aquaculture';
$string['field1050799'] = 'Fisheries Studies (Other)';
$string['field10509'] = 'Environmental Studies';
$string['field1050901'] = 'Land, Parks and Wildlife Management';
$string['field1050999'] = 'Environmental Studies (Other)';
$string['field10599'] = 'Other Agriculture, Environmental and Related Studies';
$string['field1059901'] = 'Pest and Weed Control';
$string['field1059999'] = 'Agriculture, Environmental and Related Studies (Other)';
$string['field106'] = 'Health';
$string['field10601'] = 'Medical Studies';
$string['field1060101'] = 'General Medicine';
$string['field1060103'] = 'Surgery';
$string['field1060105'] = 'Psychiatry';
$string['field1060107'] = 'Obstetrics and Gynaecology';
$string['field1060109'] = 'Paediatrics';
$string['field1060111'] = 'Anaesthesiology';
$string['field1060113'] = 'Pathology';
$string['field1060115'] = 'Radiology';
$string['field1060117'] = 'Internal Medicine';
$string['field1060119'] = 'General Practice';
$string['field1060199'] = 'Medical Studies (Other)';
$string['field10603'] = 'Nursing';
$string['field1060301'] = 'General Nursing';
$string['field1060303'] = 'Midwifery';
$string['field1060305'] = 'Mental Health Nursing';
$string['field1060307'] = 'Community Nursing';
$string['field1060309'] = 'Critical Care Nursing';
$string['field1060311'] = 'Aged Care Nursing';
$string['field1060313'] = 'Palliative Care Nursing';
$string['field1060315'] = 'Mothercraft Nursing and Family and Child Health Nursing';
$string['field1060399'] = 'Nursing (Other)';
$string['field10605'] = 'Pharmacy';
$string['field1060501'] = 'Pharmacy';
$string['field10607'] = 'Dental Studies';
$string['field1060701'] = 'Dentistry';
$string['field1060703'] = 'Dental Assisting';
$string['field1060705'] = 'Dental Technology';
$string['field1060799'] = 'Dental Studies (Other)';
$string['field10609'] = 'Optical Science';
$string['field1060901'] = 'Optometry';
$string['field1060903'] = 'Optical Technology';
$string['field1060999'] = 'Optical Science (Other)';
$string['field10611'] = 'Veterinary Studies';
$string['field1061101'] = 'Veterinary Science';
$string['field1061103'] = 'Veterinary Assisting';
$string['field1061199'] = 'Veterinary Studies (Other)';
$string['field10613'] = 'Public Health';
$string['field1061301'] = 'Occupational Health and Safety';
$string['field1061303'] = 'Environmental Health';
$string['field1061305'] = 'Indigenous Health';
$string['field1061307'] = 'Health Promotion';
$string['field1061309'] = 'Community Health';
$string['field1061311'] = 'Epidemiology';
$string['field1061399'] = 'Public Health (Other)';
$string['field10615'] = 'Radiography';
$string['field1061501'] = 'Radiography';
$string['field10617'] = 'Rehabilitation Therapies';
$string['field1061701'] = 'Physiotherapy';
$string['field1061703'] = 'Occupational Therapy';
$string['field1061705'] = 'Chiropractic and Osteopathy';
$string['field1061707'] = 'Speech Pathology';
$string['field1061709'] = 'Audiology';
$string['field1061711'] = 'Massage Therapy';
$string['field1061713'] = 'Podiatry';
$string['field1061799'] = 'Rehabilitation Therapies (Other)';
$string['field10619'] = 'Complementary Therapies';
$string['field1061901'] = 'Naturopathy';
$string['field1061903'] = 'Acupuncture';
$string['field1061905'] = 'Traditional Chinese Medicine';
$string['field1061999'] = 'Complementary Therapies (Other)';
$string['field10699'] = 'Other Health';
$string['field1069901'] = 'Nutrition and Dietetics';
$string['field1069903'] = 'Human Movement';
$string['field1069905'] = 'Paramedical Studies';
$string['field1069907'] = 'First Aid';
$string['field1069999'] = 'Health (Other)';
$string['field107'] = 'Education';
$string['field10701'] = 'Teacher Education';
$string['field1070101'] = 'Teacher Education: Early Childhood';
$string['field1070103'] = 'Teacher Education: Primary';
$string['field1070105'] = 'Teacher Education: Secondary';
$string['field1070107'] = 'Teacher-Librarianship';
$string['field1070109'] = 'Teacher Education: Vocational Education and Training';
$string['field1070111'] = 'Teacher Education: Higher Education';
$string['field1070113'] = 'Teacher Education: Special Education';
$string['field1070115'] = 'English As A Second Language Teaching';
$string['field1070117'] = 'Nursing Education Teacher Training';
$string['field1070199'] = 'Teacher Education (Other)';
$string['field10703'] = 'Curriculum and Education Studies';
$string['field1070301'] = 'Curriculum Studies';
$string['field1070303'] = 'Education Studies';
$string['field10799'] = 'Other Education';
$string['field1079999'] = 'Education (Other)';
$string['field108'] = 'Management and Commerce';
$string['field10801'] = 'Accounting';
$string['field1080101'] = 'Accounting';
$string['field10803'] = 'Business and Management';
$string['field1080301'] = 'Business Management';
$string['field1080303'] = 'Human Resource Management';
$string['field1080305'] = 'Personal Management Training';
$string['field1080307'] = 'Organisation Management';
$string['field1080309'] = 'Industrial Relations';
$string['field1080311'] = 'International Business';
$string['field1080313'] = 'Public and Health Care Administration';
$string['field1080315'] = 'Project Management';
$string['field1080317'] = 'Quality Management';
$string['field1080319'] = 'Hospitality Management';
$string['field1080321'] = 'Farm Management and Agribusiness';
$string['field1080323'] = 'Tourism Management';
$string['field1080399'] = 'Business and Management (Other)';
$string['field10805'] = 'Sales and Marketing';
$string['field1080501'] = 'Sales';
$string['field1080503'] = 'Real Estate';
$string['field1080505'] = 'Marketing';
$string['field1080507'] = 'Advertising';
$string['field1080509'] = 'Public Relations';
$string['field1080599'] = 'Sales and Marketing (Other)';
$string['field10807'] = 'Tourism';
$string['field1080701'] = 'Tourism';
$string['field10809'] = 'Office Studies';
$string['field1080901'] = 'Secretarial and Clerical Studies';
$string['field1080903'] = 'Keyboard Skills';
$string['field1080905'] = 'Practical Computing Skills';
$string['field1080999'] = 'Office Studies (Other)';
$string['field10811'] = 'Banking, Finance and Related Fields';
$string['field1081101'] = 'Banking and Finance';
$string['field1081103'] = 'Insurance and Actuarial Studies';
$string['field1081105'] = 'Investment and Securities';
$string['field1081199'] = 'Banking, Finance and Related Fields (Other)';
$string['field10899'] = 'Other Management and Commerce';
$string['field1089901'] = 'Purchasing, Warehousing and Distribution';
$string['field1089903'] = 'Valuation';
$string['field1089999'] = 'Management and Commerce (Other)';
$string['field109'] = 'Society and Culture';
$string['field10901'] = 'Political Science and Policy Studies';
$string['field1090101'] = 'Political Science';
$string['field1090103'] = 'Policy Studies';
$string['field10903'] = 'Studies In Human Society';
$string['field1090301'] = 'Sociology';
$string['field1090303'] = 'Anthropology';
$string['field1090305'] = 'History';
$string['field1090307'] = 'Archaeology';
$string['field1090309'] = 'Human Geography';
$string['field1090311'] = 'Indigenous Studies';
$string['field1090313'] = 'Gender Specific Studies';
$string['field1090399'] = 'Studies In Human Society (Other)';
$string['field10905'] = 'Human Welfare Studies and Services';
$string['field1090501'] = 'Social Work';
$string['field1090503'] = 'Children\'S Services';
$string['field1090505'] = 'Youth Work';
$string['field1090507'] = 'Care For The Aged';
$string['field1090509'] = 'Care For The Disabled';
$string['field1090511'] = 'Residential Client Care';
$string['field1090513'] = 'Counselling';
$string['field1090515'] = 'Welfare Studies';
$string['field1090599'] = 'Human Welfare Studies and Services (Other)';
$string['field10907'] = 'Behavioural Science';
$string['field1090701'] = 'Psychology';
$string['field1090799'] = 'Behavioural Science (Other)';
$string['field10909'] = 'Law';
$string['field1090901'] = 'Business and Commercial Law';
$string['field1090903'] = 'Constitutional Law';
$string['field1090905'] = 'Criminal Law';
$string['field1090907'] = 'Family Law';
$string['field1090909'] = 'International Law';
$string['field1090911'] = 'Taxation Law';
$string['field1090913'] = 'Legal Practice';
$string['field1090999'] = 'Law (Other)';
$string['field10911'] = 'Justice and Law Enforcement';
$string['field1091101'] = 'Justice Administration';
$string['field1091103'] = 'Legal Studies';
$string['field1091105'] = 'Police Studies';
$string['field1091199'] = 'Justice and Law Enforcement (Other)';
$string['field10913'] = 'Librarianship, Information Management and Curatorial Studies';
$string['field1091301'] = 'Librarianship and Information Management';
$string['field1091303'] = 'Curatorial Studies';
$string['field10915'] = 'Language and Literature';
$string['field1091501'] = 'English Language';
$string['field1091503'] = 'Northern European Languages';
$string['field1091505'] = 'Southern European Languages';
$string['field1091507'] = 'Eastern European Languages';
$string['field1091509'] = 'Southwest Asian and North African Languages';
$string['field1091511'] = 'Southern Asian Languages';
$string['field1091513'] = 'Southeast Asian Languages';
$string['field1091515'] = 'Eastern Asian Languages';
$string['field1091517'] = 'Australian Indigenous Languages';
$string['field1091519'] = 'Translating and Interpreting';
$string['field1091521'] = 'Linguistics';
$string['field1091523'] = 'Literature';
$string['field1091599'] = 'Language and Literature (Other)';
$string['field10917'] = 'Philosophy and Religious Studies';
$string['field1091701'] = 'Philosophy';
$string['field1091703'] = 'Religious Studies';
$string['field10919'] = 'Economics and Econometrics';
$string['field1091901'] = 'Economics';
$string['field1091903'] = 'Econometrics';
$string['field10921'] = 'Sport and Recreation';
$string['field1092101'] = 'Sport and Recreation Activities';
$string['field1092103'] = 'Sports Coaching, Officiating and Instruction';
$string['field1092199'] = 'Sport and Recreation (Other)';
$string['field10999'] = 'Other Society and Culture';
$string['field1099901'] = 'Family and Consumer Studies';
$string['field1099903'] = 'Criminology';
$string['field1099905'] = 'Security Services';
$string['field1099999'] = 'Society and Culture (Other)';
$string['field110'] = 'Creative Arts';
$string['field11001'] = 'Performing Arts';
$string['field1100101'] = 'Music';
$string['field1100103'] = 'Drama and Theatre Studies';
$string['field1100105'] = 'Dance';
$string['field1100199'] = 'Performing Arts (Other)';
$string['field11003'] = 'Visual Arts and Crafts';
$string['field1100301'] = 'Fine Arts';
$string['field1100303'] = 'Photography';
$string['field1100305'] = 'Crafts';
$string['field1100307'] = 'Jewellery Making';
$string['field1100309'] = 'Floristry';
$string['field1100399'] = 'Visual Arts and Crafts (Other)';
$string['field11005'] = 'Graphic and Design Studies';
$string['field1100501'] = 'Graphic Arts and Design Studies';
$string['field1100503'] = 'Textile Design';
$string['field1100505'] = 'Fashion Design';
$string['field1100599'] = 'Graphic and Design Studies (Other)';
$string['field11007'] = 'Communication and Media Studies';
$string['field1100701'] = 'Audio Visual Studies';
$string['field1100703'] = 'Journalism';
$string['field1100705'] = 'Written Communication';
$string['field1100707'] = 'Verbal Communication';
$string['field1100799'] = 'Communication and Media Studies (Other)';
$string['field11099'] = 'Other Creative Arts';
$string['field1109999'] = 'Creative Arts (Other)';
$string['field111'] = 'Food, Hospitality and Personal Services';
$string['field11101'] = 'Food and Hospitality';
$string['field1110101'] = 'Hospitality';
$string['field1110103'] = 'Food and Beverage Service';
$string['field1110105'] = 'Butchery';
$string['field1110107'] = 'Baking and Pastrymaking';
$string['field1110109'] = 'Cookery';
$string['field1110111'] = 'Food Hygiene';
$string['field1110199'] = 'Food and Hospitality (Other)';
$string['field11103'] = 'Personal Services';
$string['field1110301'] = 'Beauty Therapy';
$string['field1110303'] = 'Hairdressing';
$string['field1110399'] = 'Personal Services (Other)';
$string['field112'] = 'Mixed Field Programmes';
$string['field11201'] = 'General Education Programmes';
$string['field1120101'] = 'General Primary and Secondary Education Programmes';
$string['field1120103'] = 'Literacy and Numeracy Programmes';
$string['field1120105'] = 'Learning Skills Programmes';
$string['field1120199'] = 'General Education Programmes (Other)';
$string['field11203'] = 'Social Skills Programmes';
$string['field1120301'] = 'Social and Interpersonal Skills Programmes';
$string['field1120303'] = 'Survival Skills Programmes';
$string['field1120305'] = 'Parental Education Programmes';
$string['field1120399'] = 'Social Skills Programmes (Other)';
$string['field11205'] = 'Employment Skills Programmes';
$string['field1120501'] = 'Career Development Programmes';
$string['field1120503'] = 'Job Search Skills Programmes';
$string['field1120505'] = 'Work Practices Programmes';
$string['field1120599'] = 'Employment Skills Programmes (Other)';
$string['field11299'] = 'Other Mixed Field Programmes';
$string['field1129999'] = 'Mixed Field Programmes (Other)';
