{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_courseformat/local/content/frontpagesection

    Displays the frontpage section 1 (site home).

    Example context (json):
    {
        "editing": true,
        "sections": [
            "<li>This is the section content</li>"
        ],
        "showsettings": true,
        "settingsurl": "#"
    }
}}
<div class="course-content" id="courseformat-frontpage-main-topic">
    {{#showsettings}}
    <div class="mb-2">
        <a href="{{{settingsurl}}}" title="{{#str}} edit {{/str}}" aria-label="{{#str}} edit {{/str}}">
            {{#pix}} i/settings, moodle {{/pix}}
        </a>
    </div>
    {{/showsettings}}
    <div class="sitetopic">
        <ul class="topics section-list frontpage" data-for="course_sectionlist">
            {{#sections}}
                {{> core_courseformat/local/content/section }}
            {{/sections}}
        </ul>
    </div>
</div>
{{#js}}
{{! The home page should be as fast as possible, we only load the editor when needed.}}
{{#editing}}
require(['core_courseformat/local/content'], function(component) {
    component.init('#courseformat-frontpage-main-topic', {});
});
{{/editing}}
{{/js}}
