{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_message/message_drawer_lazy_load_list

    This template will render a lazy loaded list for the message drawer.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * All data attributes are required

    Context variables required for this template:
    * userid The logged in user id
    * urls The URLs for the popover

    Example context (json):
    {}

}}
<div
    class="{{$rootclasses}}{{/rootclasses}} lazy-load-list"
    aria-live="polite"
    data-region="lazy-load-list"
    data-user-id="{{loggedinuser.id}}"
    {{$rootattributes}}{{/rootattributes}}
>
    {{$notification}}{{/notification}}
    <div class="hidden text-center p-2" data-region="empty-message-container">
        {{$emptymessage}}{{/emptymessage}}
    </div>
    <div class="hidden list-group" data-region="content-container">
        {{$content}}{{/content}}
    </div>
    <div class="list-group" data-region="placeholder-container">
        {{$placeholder}}{{/placeholder}}
    </div>
    <div class="w-100 text-center p-3 hidden" data-region="loading-icon-container" >
        {{> core/loading }}
    </div>
</div>
