<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'iso6392', language 'en', branch 'MOODLE_20_STABLE'
 *
 * Note:
 *  - the list of languages was downloaded from http://www.loc.gov/standards/iso639-2/ascii_8bits.html
 *  - array indexes are the ISO 639-2 "T" values (terminology)
 *  - some language names are shortened
 *
 * @package    core
 * @subpackage iso6392
 * @copyright  2010 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['aar'] = 'Afar';
$string['abk'] = 'Abkhazian';
$string['ace'] = 'Achinese';
$string['ach'] = 'Acoli';
$string['ada'] = 'Adangme';
$string['ady'] = 'Adyghe; Adygei';
$string['afa'] = 'Afro-Asiatic languages';
$string['afh'] = 'Afrihili';
$string['afr'] = 'Afrikaans';
$string['ain'] = 'Ainu';
$string['aka'] = 'Akan';
$string['akk'] = 'Akkadian';
$string['sqi'] = 'Albanian';
$string['ale'] = 'Aleut';
$string['alg'] = 'Algonquian languages';
$string['alt'] = 'Southern Altai';
$string['amh'] = 'Amharic';
$string['ang'] = 'English, Old (ca.450-1100)';
$string['anp'] = 'Angika';
$string['apa'] = 'Apache languages';
$string['ara'] = 'Arabic';
$string['arc'] = 'Official Aramaic (700-300 BCE)';
$string['arg'] = 'Aragonese';
$string['hye'] = 'Armenian';
$string['arn'] = 'Mapudungun; Mapuche';
$string['arp'] = 'Arapaho';
$string['art'] = 'Artificial languages';
$string['arw'] = 'Arawak';
$string['asm'] = 'Assamese';
$string['ast'] = 'Asturian';
$string['ath'] = 'Athapascan languages';
$string['aus'] = 'Australian languages';
$string['ava'] = 'Avaric';
$string['ave'] = 'Avestan';
$string['awa'] = 'Awadhi';
$string['aym'] = 'Aymara';
$string['aze'] = 'Azerbaijani';
$string['bad'] = 'Banda languages';
$string['bai'] = 'Bamileke languages';
$string['bak'] = 'Bashkir';
$string['bal'] = 'Baluchi';
$string['bam'] = 'Bambara';
$string['ban'] = 'Balinese';
$string['eus'] = 'Basque';
$string['bas'] = 'Basa';
$string['bat'] = 'Baltic languages';
$string['bej'] = 'Beja; Bedawiyet';
$string['bel'] = 'Belarusian';
$string['bem'] = 'Bemba';
$string['ben'] = 'Bengali';
$string['ber'] = 'Berber languages';
$string['bho'] = 'Bhojpuri';
$string['bih'] = 'Bihari languages';
$string['bik'] = 'Bikol';
$string['bin'] = 'Bini; Edo';
$string['bis'] = 'Bislama';
$string['bla'] = 'Siksika';
$string['bnt'] = 'Bantu (Other)';
$string['bos'] = 'Bosnian';
$string['bra'] = 'Braj';
$string['bre'] = 'Breton';
$string['btk'] = 'Batak languages';
$string['bua'] = 'Buriat';
$string['bug'] = 'Buginese';
$string['bul'] = 'Bulgarian';
$string['mya'] = 'Burmese';
$string['byn'] = 'Blin; Bilin';
$string['cad'] = 'Caddo';
$string['cai'] = 'Central American Indian languages';
$string['car'] = 'Galibi Carib';
$string['cat'] = 'Catalan; Valencian';
$string['cau'] = 'Caucasian languages';
$string['ceb'] = 'Cebuano';
$string['cel'] = 'Celtic languages';
$string['cha'] = 'Chamorro';
$string['chb'] = 'Chibcha';
$string['che'] = 'Chechen';
$string['chg'] = 'Chagatai';
$string['zho'] = 'Chinese';
$string['chk'] = 'Chuukese';
$string['chm'] = 'Mari';
$string['chn'] = 'Chinook jargon';
$string['cho'] = 'Choctaw';
$string['chp'] = 'Chipewyan; Dene Suline';
$string['chr'] = 'Cherokee';
$string['chu'] = 'Church Slavic; Old Slavonic';
$string['chv'] = 'Chuvash';
$string['chy'] = 'Cheyenne';
$string['cmc'] = 'Chamic languages';
$string['cop'] = 'Coptic';
$string['cor'] = 'Cornish';
$string['cos'] = 'Corsican';
$string['cpe'] = 'Creoles and pidgins, English based';
$string['cpf'] = 'Creoles and pidgins, French-based ';
$string['cpp'] = 'Creoles and pidgins, Portuguese-based ';
$string['cre'] = 'Cree';
$string['crh'] = 'Crimean Tatar; Crimean Turkish';
$string['crp'] = 'Creoles and pidgins ';
$string['csb'] = 'Kashubian';
$string['cus'] = 'Cushitic languages';
$string['ces'] = 'Czech';
$string['dak'] = 'Dakota';
$string['dan'] = 'Danish';
$string['dar'] = 'Dargwa';
$string['day'] = 'Land Dayak languages';
$string['del'] = 'Delaware';
$string['den'] = 'Slave (Athapascan)';
$string['dgr'] = 'Dogrib';
$string['din'] = 'Dinka';
$string['div'] = 'Divehi; Dhivehi; Maldivian';
$string['doi'] = 'Dogri';
$string['dra'] = 'Dravidian languages';
$string['dsb'] = 'Lower Sorbian';
$string['dua'] = 'Duala';
$string['dum'] = 'Dutch, Middle (ca.1050-1350)';
$string['nld'] = 'Dutch; Flemish';
$string['dyu'] = 'Dyula';
$string['dzo'] = 'Dzongkha';
$string['efi'] = 'Efik';
$string['egy'] = 'Egyptian (Ancient)';
$string['eka'] = 'Ekajuk';
$string['elx'] = 'Elamite';
$string['eng'] = 'English';
$string['enm'] = 'English, Middle (1100-1500)';
$string['epo'] = 'Esperanto';
$string['est'] = 'Estonian';
$string['ewe'] = 'Ewe';
$string['ewo'] = 'Ewondo';
$string['fan'] = 'Fang';
$string['fao'] = 'Faroese';
$string['fat'] = 'Fanti';
$string['fij'] = 'Fijian';
$string['fil'] = 'Filipino; Pilipino';
$string['fin'] = 'Finnish';
$string['fiu'] = 'Finno-Ugrian languages';
$string['fon'] = 'Fon';
$string['fra'] = 'French';
$string['frm'] = 'French, Middle (ca.1400-1600)';
$string['fro'] = 'French, Old (842-ca.1400)';
$string['frr'] = 'Northern Frisian';
$string['frs'] = 'Eastern Frisian';
$string['fry'] = 'Western Frisian';
$string['ful'] = 'Fulah';
$string['fur'] = 'Friulian';
$string['gaa'] = 'Ga';
$string['gay'] = 'Gayo';
$string['gba'] = 'Gbaya';
$string['gem'] = 'Germanic languages';
$string['kat'] = 'Georgian';
$string['deu'] = 'German';
$string['gez'] = 'Geez';
$string['gil'] = 'Gilbertese';
$string['gla'] = 'Gaelic; Scottish Gaelic';
$string['gle'] = 'Irish';
$string['glg'] = 'Galician';
$string['glv'] = 'Manx';
$string['gmh'] = 'German, Middle High (ca.1050-1500)';
$string['goh'] = 'German, Old High (ca.750-1050)';
$string['gon'] = 'Gondi';
$string['gor'] = 'Gorontalo';
$string['got'] = 'Gothic';
$string['grb'] = 'Grebo';
$string['grc'] = 'Greek, Ancient (to 1453)';
$string['ell'] = 'Greek, Modern (1453-)';
$string['grn'] = 'Guarani';
$string['gsw'] = 'Swiss German; Alemannic; Alsatian';
$string['guj'] = 'Gujarati';
$string['gwi'] = 'Gwich\'in';
$string['hai'] = 'Haida';
$string['hat'] = 'Haitian; Haitian Creole';
$string['hau'] = 'Hausa';
$string['haw'] = 'Hawaiian';
$string['heb'] = 'Hebrew';
$string['her'] = 'Herero';
$string['hil'] = 'Hiligaynon';
$string['him'] = 'Himachali; Western Pahari';
$string['hin'] = 'Hindi';
$string['hit'] = 'Hittite';
$string['hmn'] = 'Hmong; Mong';
$string['hmo'] = 'Hiri Motu';
$string['hrv'] = 'Croatian';
$string['hsb'] = 'Upper Sorbian';
$string['hun'] = 'Hungarian';
$string['hup'] = 'Hupa';
$string['iba'] = 'Iban';
$string['ibo'] = 'Igbo';
$string['isl'] = 'Icelandic';
$string['ido'] = 'Ido';
$string['iii'] = 'Sichuan Yi; Nuosu';
$string['ijo'] = 'Ijo languages';
$string['iku'] = 'Inuktitut';
$string['ile'] = 'Interlingue; Occidental';
$string['ilo'] = 'Iloko';
$string['ina'] = 'Interlingua (IALA)';
$string['inc'] = 'Indic languages';
$string['ind'] = 'Indonesian';
$string['ine'] = 'Indo-European languages';
$string['inh'] = 'Ingush';
$string['ipk'] = 'Inupiaq';
$string['ira'] = 'Iranian languages';
$string['iro'] = 'Iroquoian languages';
$string['ita'] = 'Italian';
$string['jav'] = 'Javanese';
$string['jbo'] = 'Lojban';
$string['jpn'] = 'Japanese';
$string['jpr'] = 'Judeo-Persian';
$string['jrb'] = 'Judeo-Arabic';
$string['kaa'] = 'Kara-Kalpak';
$string['kab'] = 'Kabyle';
$string['kac'] = 'Kachin; Jingpho';
$string['kal'] = 'Kalaallisut; Greenlandic';
$string['kam'] = 'Kamba';
$string['kan'] = 'Kannada';
$string['kar'] = 'Karen languages';
$string['kas'] = 'Kashmiri';
$string['kau'] = 'Kanuri';
$string['kaw'] = 'Kawi';
$string['kaz'] = 'Kazakh';
$string['kbd'] = 'Kabardian';
$string['kha'] = 'Khasi';
$string['khi'] = 'Khoisan languages';
$string['khm'] = 'Central Khmer';
$string['kho'] = 'Khotanese; Sakan';
$string['kik'] = 'Kikuyu; Gikuyu';
$string['kin'] = 'Kinyarwanda';
$string['kir'] = 'Kirghiz; Kyrgyz';
$string['kmb'] = 'Kimbundu';
$string['kok'] = 'Konkani';
$string['kom'] = 'Komi';
$string['kon'] = 'Kongo';
$string['kor'] = 'Korean';
$string['kos'] = 'Kosraean';
$string['kpe'] = 'Kpelle';
$string['krc'] = 'Karachay-Balkar';
$string['krl'] = 'Karelian';
$string['kro'] = 'Kru languages';
$string['kru'] = 'Kurukh';
$string['kua'] = 'Kuanyama; Kwanyama';
$string['kum'] = 'Kumyk';
$string['kur'] = 'Kurdish';
$string['kut'] = 'Kutenai';
$string['lad'] = 'Ladino';
$string['lah'] = 'Lahnda';
$string['lam'] = 'Lamba';
$string['lao'] = 'Lao';
$string['lat'] = 'Latin';
$string['lav'] = 'Latvian';
$string['lez'] = 'Lezghian';
$string['lim'] = 'Limburgan';
$string['lin'] = 'Lingala';
$string['lit'] = 'Lithuanian';
$string['lol'] = 'Mongo';
$string['loz'] = 'Lozi';
$string['ltz'] = 'Luxembourgish';
$string['lua'] = 'Luba-Lulua';
$string['lub'] = 'Luba-Katanga';
$string['lug'] = 'Ganda';
$string['lui'] = 'Luiseno';
$string['lun'] = 'Lunda';
$string['luo'] = 'Luo (Kenya and Tanzania)';
$string['lus'] = 'Lushai';
$string['mkd'] = 'Macedonian';
$string['mad'] = 'Madurese';
$string['mag'] = 'Magahi';
$string['mah'] = 'Marshallese';
$string['mai'] = 'Maithili';
$string['mak'] = 'Makasar';
$string['mal'] = 'Malayalam';
$string['man'] = 'Mandingo';
$string['mri'] = 'Maori';
$string['map'] = 'Austronesian languages';
$string['mar'] = 'Marathi';
$string['mas'] = 'Masai';
$string['msa'] = 'Malay';
$string['mdf'] = 'Moksha';
$string['mdr'] = 'Mandar';
$string['men'] = 'Mende';
$string['mga'] = 'Irish, Middle (900-1200)';
$string['mic'] = 'Mi\'kmaq; Micmac';
$string['min'] = 'Minangkabau';
$string['mis'] = 'Uncoded languages';
$string['mkh'] = 'Mon-Khmer languages';
$string['mlg'] = 'Malagasy';
$string['mlt'] = 'Maltese';
$string['mnc'] = 'Manchu';
$string['mni'] = 'Manipuri';
$string['mno'] = 'Manobo languages';
$string['moh'] = 'Mohawk';
$string['mon'] = 'Mongolian';
$string['mos'] = 'Mossi';
$string['mul'] = 'Multiple languages';
$string['mun'] = 'Munda languages';
$string['mus'] = 'Creek';
$string['mwl'] = 'Mirandese';
$string['mwr'] = 'Marwari';
$string['myn'] = 'Mayan languages';
$string['myv'] = 'Erzya';
$string['nah'] = 'Nahuatl languages';
$string['nai'] = 'North American Indian languages';
$string['nap'] = 'Neapolitan';
$string['nau'] = 'Nauru';
$string['nav'] = 'Navajo; Navaho';
$string['nbl'] = 'Ndebele, South';
$string['nde'] = 'Ndebele, North';
$string['ndo'] = 'Ndonga';
$string['nds'] = 'German Low (Saxon)';
$string['nep'] = 'Nepali';
$string['new'] = 'Nepal Bhasa; Newari';
$string['nia'] = 'Nias';
$string['nic'] = 'Niger-Kordofanian languages';
$string['niu'] = 'Niuean';
$string['nno'] = 'Norwegian Nynorsk';
$string['nob'] = 'Norwegian Bokmål';
$string['nog'] = 'Nogai';
$string['non'] = 'Norse, Old';
$string['nor'] = 'Norwegian';
$string['nqo'] = 'N\'Ko';
$string['nso'] = 'Pedi';
$string['nub'] = 'Nubian languages';
$string['nwc'] = 'Classical Newari';
$string['nya'] = 'Chichewa';
$string['nym'] = 'Nyamwezi';
$string['nyn'] = 'Nyankole';
$string['nyo'] = 'Nyoro';
$string['nzi'] = 'Nzima';
$string['oci'] = 'Occitan (post 1500); Provençal';
$string['oji'] = 'Ojibwa';
$string['ori'] = 'Oriya';
$string['orm'] = 'Oromo';
$string['osa'] = 'Osage';
$string['oss'] = 'Ossetian; Ossetic';
$string['ota'] = 'Turkish, Ottoman (1500-1928)';
$string['oto'] = 'Otomian languages';
$string['paa'] = 'Papuan languages';
$string['pag'] = 'Pangasinan';
$string['pal'] = 'Pahlavi';
$string['pam'] = 'Pampanga; Kapampangan';
$string['pan'] = 'Panjabi; Punjabi';
$string['pap'] = 'Papiamento';
$string['pau'] = 'Palauan';
$string['peo'] = 'Persian, Old (ca.600-400 B.C.)';
$string['fas'] = 'Persian';
$string['phi'] = 'Philippine languages';
$string['phn'] = 'Phoenician';
$string['pli'] = 'Pali';
$string['pol'] = 'Polish';
$string['pon'] = 'Pohnpeian';
$string['por'] = 'Portuguese';
$string['pra'] = 'Prakrit languages';
$string['pro'] = 'Provençal, Old (to 1500)';
$string['pus'] = 'Pushto; Pashto';
$string['que'] = 'Quechua';
$string['raj'] = 'Rajasthani';
$string['rap'] = 'Rapanui';
$string['rar'] = 'Rarotongan; Cook Islands Maori';
$string['roa'] = 'Romance languages';
$string['roh'] = 'Romansh';
$string['rom'] = 'Romany';
$string['ron'] = 'Romanian; Moldavian; Moldovan';
$string['run'] = 'Rundi';
$string['rup'] = 'Aromanian';
$string['rus'] = 'Russian';
$string['sad'] = 'Sandawe';
$string['sag'] = 'Sango';
$string['sah'] = 'Yakut';
$string['sai'] = 'South American Indian (Other)';
$string['sal'] = 'Salishan languages';
$string['sam'] = 'Samaritan Aramaic';
$string['san'] = 'Sanskrit';
$string['sas'] = 'Sasak';
$string['sat'] = 'Santali';
$string['scn'] = 'Sicilian';
$string['sco'] = 'Scots';
$string['sel'] = 'Selkup';
$string['sem'] = 'Semitic languages';
$string['sga'] = 'Irish, Old (to 900)';
$string['sgn'] = 'Sign Languages';
$string['shn'] = 'Shan';
$string['sid'] = 'Sidamo';
$string['sin'] = 'Sinhala; Sinhalese';
$string['sio'] = 'Siouan languages';
$string['sit'] = 'Sino-Tibetan languages';
$string['sla'] = 'Slavic languages';
$string['slk'] = 'Slovak';
$string['slv'] = 'Slovenian';
$string['sma'] = 'Southern Sami';
$string['sme'] = 'Northern Sami';
$string['smi'] = 'Sami languages';
$string['smj'] = 'Lule Sami';
$string['smn'] = 'Inari Sami';
$string['smo'] = 'Samoan';
$string['sms'] = 'Skolt Sami';
$string['sna'] = 'Shona';
$string['snd'] = 'Sindhi';
$string['snk'] = 'Soninke';
$string['sog'] = 'Sogdian';
$string['som'] = 'Somali';
$string['son'] = 'Songhai languages';
$string['sot'] = 'Sotho, Southern';
$string['spa'] = 'Spanish; Castilian';
$string['srd'] = 'Sardinian';
$string['srn'] = 'Sranan Tongo';
$string['srp'] = 'Serbian';
$string['srr'] = 'Serer';
$string['ssa'] = 'Nilo-Saharan languages';
$string['ssw'] = 'Swati';
$string['suk'] = 'Sukuma';
$string['sun'] = 'Sundanese';
$string['sus'] = 'Susu';
$string['sux'] = 'Sumerian';
$string['swa'] = 'Swahili';
$string['swe'] = 'Swedish';
$string['syc'] = 'Classical Syriac';
$string['syr'] = 'Syriac';
$string['tah'] = 'Tahitian';
$string['tai'] = 'Tai languages';
$string['tam'] = 'Tamil';
$string['tat'] = 'Tatar';
$string['tel'] = 'Telugu';
$string['tem'] = 'Timne';
$string['ter'] = 'Tereno';
$string['tet'] = 'Tetum';
$string['tgk'] = 'Tajik';
$string['tgl'] = 'Tagalog';
$string['tha'] = 'Thai';
$string['bod'] = 'Tibetan';
$string['tig'] = 'Tigre';
$string['tir'] = 'Tigrinya';
$string['tiv'] = 'Tiv';
$string['tkl'] = 'Tokelau';
$string['tlh'] = 'Klingon';
$string['tli'] = 'Tlingit';
$string['tmh'] = 'Tamashek';
$string['tog'] = 'Tonga (Nyasa)';
$string['ton'] = 'Tonga (Tonga Islands)';
$string['tpi'] = 'Tok Pisin';
$string['tsi'] = 'Tsimshian';
$string['tsn'] = 'Tswana';
$string['tso'] = 'Tsonga';
$string['tuk'] = 'Turkmen';
$string['tum'] = 'Tumbuka';
$string['tup'] = 'Tupi languages';
$string['tur'] = 'Turkish';
$string['tut'] = 'Altaic languages';
$string['tvl'] = 'Tuvalu';
$string['twi'] = 'Twi';
$string['tyv'] = 'Tuvinian';
$string['udm'] = 'Udmurt';
$string['uga'] = 'Ugaritic';
$string['uig'] = 'Uighur; Uyghur';
$string['ukr'] = 'Ukrainian';
$string['umb'] = 'Umbundu';
$string['und'] = 'Undetermined';
$string['urd'] = 'Urdu';
$string['uzb'] = 'Uzbek';
$string['vai'] = 'Vai';
$string['ven'] = 'Venda';
$string['vie'] = 'Vietnamese';
$string['vol'] = 'Volapük';
$string['vot'] = 'Votic';
$string['wak'] = 'Wakashan languages';
$string['wal'] = 'Walamo';
$string['war'] = 'Waray';
$string['was'] = 'Washo';
$string['cym'] = 'Welsh';
$string['wen'] = 'Sorbian languages';
$string['wln'] = 'Walloon';
$string['wol'] = 'Wolof';
$string['xal'] = 'Kalmyk; Oirat';
$string['xho'] = 'Xhosa';
$string['yao'] = 'Yao';
$string['yap'] = 'Yapese';
$string['yid'] = 'Yiddish';
$string['yor'] = 'Yoruba';
$string['ypk'] = 'Yupik languages';
$string['zap'] = 'Zapotec';
$string['zbl'] = 'Blissymbols';
$string['zen'] = 'Zenaga';
$string['zha'] = 'Zhuang; Chuang';
$string['znd'] = 'Zande languages';
$string['zul'] = 'Zulu';
$string['zun'] = 'Zuni';
$string['zxx'] = 'No linguistic content';
$string['zza'] = 'Zaza';
