<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'gradepenalty_duedate', language 'en'.
 *
 * @package   gradepenalty_duedate
 * @copyright 2024 Catalyst IT Australia Pty Ltd
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['addrule'] = 'Add rule';
$string['deleteallrules'] = 'Delete all rules';
$string['duedate:manage'] = 'Permission to manage penalty rules';
$string['duedaterule'] = 'Penalty rules';
$string['editduedaterule'] = 'Edit penalty rules';
$string['error_overdueby_abovevalue'] = 'The overdue amount must be greater than the amount for the rule above ({$a}).';
$string['error_overdueby_maxvalue'] = 'The overdue amount cannot be greater than {$a}.';
$string['error_overdueby_minvalue'] = 'The overdue amount must be greater than or equal to {$a}.';
$string['error_penalty_abovevalue'] = 'The penalty must be greater than the penalty for the rule above ({$a}%).';
$string['error_penalty_maxvalue'] = 'The penalty cannot be greater than {$a}%.';
$string['error_penalty_minvalue'] = 'The penalty must be greater than or equal to {$a}%.';
$string['existingrule'] = 'Existing rules';
$string['finalpenaltyrule'] = 'Final penalty rule';
$string['finalpenaltyrule_help'] = 'The final penalty rule specifies the penalty that is applied to submissions overdue longer than for any other penalty rule.';
$string['insertrule'] = 'Insert below';
$string['overdueby'] = 'Overdue';
$string['overdueby_help'] = 'The time in seconds after the due date that the penalty will be applied.';
$string['overdueby_label'] = 'Overdue:';
$string['overdueby_lastrow'] = '&gt; {$a}';
$string['overdueby_onerow'] = 'All late submissions';
$string['overdueby_row'] = '&le; {$a}';
$string['penalty'] = 'Penalty';
$string['penalty_help'] = 'The penalty in percent that will be applied for late submissions.';
$string['penalty_label'] = 'Penalty:';
$string['penaltyrule'] = 'Penalty rules';
$string['penaltyrule_group'] = 'Penalty rule {no}';
$string['penaltyrule_inherited'] = 'The penalty rules in this context are inherited from a parent context. Click Edit to override the values.';
$string['penaltyrule_not_inherited'] = 'Click Edit to change or add rules.';
$string['penaltyrule_overridden'] = 'The penalty rules are overridden. Click Reset to remove overridden rules. This will remove all rules if there is no rule in the parent context.';
$string['pluginname'] = 'Late submission penalties';
$string['privacy:metadata:gradepenalty_duedate_rule'] = 'Grade penalty due date table';
$string['privacy:metadata:gradepenalty_duedate_rule:usermodified'] = 'User who modified the rule';
$string['resetconfirm'] = 'This will remove all rules in this context. Are you sure you want to continue?';
