{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_cohortroles/cohort-in-list

    Moodle template for displaying a cohort in a list.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * id cohort id field
    * name cohort name field
    * idnumber cohort idnumber field
    * description cohort description field
    * visible cohort visible field
    * theme cohort theme field

    Example context (json):
    { "id": "1",
      "name": "Cohort 1",
      "visible": true,
      "idnumber": "014",
      "description": "Some users",
      "theme": "classic"
    }
}}
<span>
    <span>{{name}}</span>
    {{#idnumber}}
    <span><small>{{idnumber}}</small></span>
    {{/idnumber}}
    <span><small>( {{contextname}} )</small></span>
</span>
