{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_assign/submission_actionmenu

    Actions for grading submissions in the assignment.

    Classes required for JS:
    * none

    Context variables required for this template:
    * see mod/assign/classes/output/actionmenu.php

    Example context (json):
    {
        "gradelink": "https://moodle.org"
    }

}}
<div class="container-fluid tertiary-navigation">
    <div class="d-flex">
        {{#gradelink}}
        <div class="navitem">
            <a class="btn btn-primary" href="{{gradelink}}">{{#str}}gradeverb, core{{/str}}</a>
        </div>
        {{/gradelink}}
    </div>
</div>
