{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template gradereport_singleview/text_attribute

    Text attribute.

    Example context (json):
    {
        "name": "Awesome-report",
        "label": "Text label",
        "value": "Text information",
        "disabled": false,
        "type": "number",
        "isnumeric": true,
        "min": "0",
        "max": "100"
    }
}}
{{#readonly}}
    {{value}}
{{/readonly}}
{{^readonly}}
    {{#label}}<label for="{{name}}" class="accesshide">{{label}}</label>{{/label}}
    <input id="{{name}}" data-uielement="text" name="{{name}}" type="{{#type}}{{.}}{{/type}}{{^type}}text{{/type}}" value="{{value}}" class="form-control" {{#disabled}}disabled{{/disabled}}{{!
        }} {{#isnumeric}} {{#min}}min="{{.}}" {{/min}} {{#max}}max="{{.}}" {{/max}} step="any" {{/isnumeric}}>
    <input type="hidden" name="old{{name}}" value="{{value}}">
{{/readonly}}
