<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'format_singleactivity'
 *
 * @package    format_singleactivity
 * @copyright  2012 Marina Glancy
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['activitytype'] = 'Type of activity';
$string['activitytype_help'] = 'Choose the type of activity or resource to use in this course';
$string['defactivitytype'] = 'Default type of activity';
$string['defactivitytypedesc'] = 'Specify the activity type that will be selected by default when creating a new course';
$string['erroractivitytype'] = 'Type of activity is not set up in Course settings';
$string['plugin_description'] = 'The course displays only one activity on the course page. You can add more activities in the Additional activities block.';
$string['pluginname'] = 'Single activity';
$string['sectionname'] = '';
$string['privacy:metadata'] = 'The Single activity format plugin does not store any personal data.';
