{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_message/message_drawer_view_settings_body_placeholder

    This template will render the body of the settings page in the message drawer.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * All data attributes are required

    Context variables required for this template:
    * userid The logged in user id
    * urls The URLs for the popover

    Example context (json):
    {}

}}

<div class="d-flex flex-column p-3">
    <div class="w-25 bg-pulse-grey h6" style="height: 18px"></div>
    <div class="w-75 bg-pulse-grey mb-4" style="height: 18px"></div>
    <div class="mb-3">
        <div class="w-100 d-flex mb-3">
            <div class="bg-pulse-grey rounded-circle" style="width: 18px; height: 18px"></div>
            <div class="bg-pulse-grey w-50 ms-2" style="height: 18px"></div>
        </div>
        <div class="w-100 d-flex mb-3">
            <div class="bg-pulse-grey rounded-circle" style="width: 18px; height: 18px"></div>
            <div class="bg-pulse-grey w-50 ms-2" style="height: 18px"></div>
        </div>
        <div class="w-100 d-flex mb-3">
            <div class="bg-pulse-grey rounded-circle" style="width: 18px; height: 18px"></div>
            <div class="bg-pulse-grey w-50 ms-2" style="height: 18px"></div>
        </div>
    </div>
    <div class="w-50 bg-pulse-grey h6 mb-3 mt-2" style="height: 18px"></div>
    <div class="mb-4">
        <div class="w-100 d-flex mb-2 align-items-center">
            <div class="bg-pulse-grey w-25" style="width: 18px; height: 27px"></div>
            <div class="bg-pulse-grey w-25 ms-2" style="height: 18px"></div>
        </div>
        <div class="w-100 d-flex mb-2 align-items-center">
            <div class="bg-pulse-grey w-25" style="width: 18px; height: 27px"></div>
            <div class="bg-pulse-grey w-25 ms-2" style="height: 18px"></div>
        </div>
    </div>
    <div class="w-25 bg-pulse-grey h6 mb-3 mt-2" style="height: 18px"></div>
    <div class="mb-3">
        <div class="w-100 d-flex mb-2 align-items-center">
            <div class="bg-pulse-grey w-25" style="width: 18px; height: 27px"></div>
            <div class="bg-pulse-grey w-50 ms-2" style="height: 18px"></div>
        </div>
    </div>
</div>