{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_forum/discussion_pin_toggle

    Template to display the discussion subscription toggle.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * TODO

    Example context (json):
    {
    }
}}
{{#capabilities.pin}}
    <a
        class="dropdown-item menu-action"
        role="menuitem"
        data-type="pin-toggle"
        data-action="toggle"
        data-discussionid="{{id}}"
        data-forumid="{{forumid}}"
        href="{{urls.pin}}"
        tabindex="-1"
        {{#pinned}}
            data-targetstate="0"
            title="{{#str}}unpindiscussion, mod_forum{{/str}}"
        {{/pinned}}
        {{^pinned}}
            data-targetstate="1"
            title="{{#str}}pindiscussion, mod_forum{{/str}}"
        {{/pinned}}
    >
        <span class="menu-action-text">
            {{#pinned}}
                {{#str}}unpindiscussion, mod_forum{{/str}}
            {{/pinned}}
            {{^pinned}}
                {{#str}}pindiscussion, mod_forum{{/str}}
            {{/pinned}}
        </span>
    </a>
{{/capabilities.pin}}