H5PEditor.language.core = {
  missingTranslation: '[Falta a tradución :key]',
  loading: 'Cargando, agarda, por favor...',
  selectLibrary: 'Selecciona a librería que queres usar para o teu contido.',
  unknownFieldPath: 'Non se atopa ":path".',
  notImageField: '":path" non é unha imaxe.',
  notImageOrDimensionsField: '":path" non é unha imaxe ou un cammpo de dimensións.',
  requiredProperty: 'Requírese a :property e debe ter un valor.',
  onlyNumbers: 'O valor de :property so pode conter números.',
  illegalDecimalNumber: ':property so pode conter números cun máximo de :decimals decimais.',
  exceedsMax: 'O valor de :property excede o máximo de :max.',
  listExceedsMax: 'A lista excede o máximo de :max elementos.',
  belowMin: 'O valor de :property está por baixo do mínimo de :min.',
  listBelowMin: 'A lista necesita polo menos :min elementos para que o contido funcione axeitadamente.',
  outOfStep: 'O valor de :property so se pode cambiar en saltos de :step.',
  add: 'Engadir',
  addFile: 'Engadir ficheiro',
  removeFile: 'Borrar ficheiro',
  confirmRemoval: 'Seguro que queres borrar este :type?',
  removeImage: 'Borrar imaxe',
  confirmImageRemoval: 'Isto borrará a tua imaxe. Seguro que queres seguir?',
  changeFile: 'Cambiar ficheiro',
  changeLibrary: 'Cambiar tipo de contido?',
  semanticsError: 'Erro semántico: :error',
  missingProperty: 'Ao campo :index fátalle a propiedade :property',
  expandCollapse: 'Expandir/Encoller',
  addEntity: 'Engadir :entity',
  tooLong: 'O valor do campo é demasiado longo, debería conter un máximo de :max letras ou menos.',
  invalidFormat: 'O valor do campo contén un formato non válido ou caracteres non permitidos.',
  confirmChangeLibrary: 'Ao facer isto, perderás todo o traballo feito co tipo de contido actual. Seguro que queres cambiar o tipo de contido?',
  commonFields: 'Cambios de texto e traducións',
  commonFieldsDescription: 'Aquí podes editar a configuración ou traducir os textos usados neste contido.',
  uploading: 'Cargando, espera, por favor...',
  noFollow: 'Non se pode seguir o campo ":path".',
  editCopyright: 'Editar copyright',
  close: 'Pechar',
  tutorial: 'Tutorial',
  editMode: 'Modo de edición',
  listLabel: 'Lista',
  uploadError: 'Erro na carga de ficheiros',
  fileToLarge: 'Poida que o ficheiro que estás tentando cargar sexa demasiado grande.',
  unknownFileUploadError: 'Erro de carga de ficheiros descoñecido',
  noSemantics: 'Erro, non se puido cargar o formulario do tipo de contido.',
  editImage: 'Editar imaxe',
  saveLabel: 'Gardar',
  cancelLabel: 'Cancelar',
  resetToOriginalLabel: 'Voltar á orixinal',
  loadingImageEditor: 'Cargando o editor de imaxes. Agarda, por favor...',
  selectFiletoUpload: 'Seleccionar ficheiro a cargar',
  or: 'ou',
  enterAudioUrl: 'Fornecer URL da fonte de audio',
  enterVideoUrl: 'Fornecer URL da fonte de vídeo',
  enterAudioTitle: 'Pegar enlace ou outra URL da fonte de audio',
  enterVideoTitle: 'Pegar enlace de Youtube ou outra URL de fonte de vídeo',
  uploadAudioTitle: 'Cargar ficheiro de audio',
  uploadVideoTitle: 'Cargar ficheiro de vídeo',
  addVideoDescription: 'H5P suporta todas as fontes externas de vídeo formateado como mp4, webm ou ogv, como Vimeo Pro, e soporta enlaces de YouTube e Panopto.',
  insert: 'Inserir',
  cancel: 'Cancelar',
  height: 'altura',
  width: 'anchura',
  textField: 'campo de texto',
  numberField: 'campo numérico',
  orderItemUp: 'Subir a orde do elemento',
  orderItemDown: 'Baixar a orde do elemento',
  expandAllContent: 'Expand all content',
  collapseAllContent: 'Collapse all content',
  removeItem: 'Borrar elemento',
  hubPanelLabel: 'Seleccionar tipo de contido',
  importantInstructions: 'Instrucións importantes',
  showImportantInstructions: 'Amosar instrucións',
  hideImportantInstructions: 'Agochar instruccións importantes',
  hide: 'Agochar',
  example: 'Exemplo',
  createContentTabLabel: 'Crear Contido',
  uploadTabLabel: 'Cargar',
  uploadPlaceholder: 'Non se seleccionou ningún ficheiro',
  uploadInstructionsTitle: 'Cargar un ficheiro H5P.',
  uploadInstructionsContent: 'Podes empezar con exemplos de <a href="https://h5p.org/content-types-and-applications" target="blank">H5P.org</a>.',
  uploadFileButtonLabel: 'Cargar un ficheiro',
  uploadFileButtonChangeLabel: 'Cambiar ficheiro',
  uploadingThrobber: 'Cargando...',
  uploadSuccess: ':title cargouse correctamente!',
  unableToInterpretError: 'Non se pode interpretar a resposta.',
  unableToInterpretSolution: 'Por favor, comproba o teu rexistro de erros.',
  h5pFileWrongExtensionTitle: 'Non se puido cargar o ficheiro seleccionado',
  h5pFileWrongExtensionContent: 'Só se permiten ficheiros coa extensión .h5p.',
  h5pFileValidationFailedTitle: 'Non se puido validar o ficheiro H5P.',
  h5pFileValidationFailedContent: 'Asegúrate de que o H5P que cargaches teña contido H5P válido. Os ficheiros H5P' +
  ' que conteñan so librerías deberían cargarse na páxina de Librerías H5P.',
  h5pFileUploadServerErrorTitle: 'Non se puido cargar o ficheiro H5P',
  h5pFileUploadServerErrorContent: 'Ocorreu un erro inesperado. Comproba o teu rexistro de error para' +
  ' máis detalles.',
  contentTypeSectionAll: 'Todos os Tipos de Contido',
  searchResults: 'Resultados da Busca',
  contentTypeSearchFieldPlaceholder: 'Buscar Tipos de Contido',
  contentTypeInstallButtonLabel: 'Instalar',
  contentTypeInstallingButtonLabel: 'Instalando',
  contentTypeUseButtonLabel: 'Usar',
  contentTypeDetailButtonLabel: 'Detalles',
  contentTypeUpdateButtonLabel: 'Actualizar',
  contentTypeUpdatingButtonLabel: 'Actualizando',
  contentTypeGetButtonLabel: 'Obter',
  contentTypeBackButtonLabel: 'Voltar',
  contentTypeIconAltText: 'Icona',
  contentTypeInstallSuccess: ':contentType instalado correctamente!',
  contentTypeUpdateSuccess: ':contentType actualizado correctamente!',
  contentTypeInstallError: 'Non se puido instalar :contentType. Contacta co teu administrador.',
  contentTypeLicensePanelTitle: 'Licenza',
  contentTypeDemoButtonLabel: 'Demostración do contido',
  numResults: ':num resultados',
  show: 'Amosar',
  recentlyUsedFirst: 'Primeiro os Usados Recentemete',
  popularFirst: 'Primeiro os máis Populares',
  newestFirst: 'Primeiro os máis Recentes',
  aToZ: 'A ao Z',
  noResultsFound: 'Non se atoparon resultados',
  noResultsFoundDesc: 'Non hai tipos de contido que coincidan cos teus criterios de busca.',
  readMore: 'Ler máis',
  readLess: 'Ler menos',
  contentTypeOwner: 'Por :owner',
  contentTypeUnsupportedApiVersionTitle: 'Este tipo de contido require una versión do núcleo máis moderna',
  contentTypeUnsupportedApiVersionContent: 'Contacta co administrador do sistema para que faga as actualizacións necesarias',
  contentTypeUpdateAvailable: 'Actualización dispoñible',
  contentTypeRestricted: 'Tipo de contido restrinxido',
  contentTypeRestrictedDesc: 'Un administrador restrinxiu o uso deste tipo de contido.',
  contentTypeNotInstalled: 'Tipo de contido non instalado',
  contentTypeNotInstalledDesc: 'Non tes permiso para instalar tipos de contido.',
  theContentType: 'o tipo de contido',
  currentMenuSelected: 'selección actual',
  errorCommunicatingHubTitle: 'Imposible comunicarse co hub.',
  errorCommunicatingHubContent: 'Ocorreu un erro. Por favor, téntao de novo.',
  warningNoContentTypesInstalled: "Non tes ningún tipo de contido instalado.",
  warningChangeBrowsingToSeeResults: 'Preme en <em>Todos</em> para ver unha lista de todos os tipos de contido que podes instalar.',
  warningUpdateAvailableTitle: 'Está dispoñible unha nova versión de :contentType.',
  warningUpdateAvailableBody: 'Actualiza á nova versión para mellorar a túa experiencia.',
  licenseDescription: 'Algunhas das características desta licenza indícanse na parte inferior. Preme na icona de información da parte superior para ler o texto orixinal da licenza.',
  licenseModalTitle: 'Detalles da Licenza',
  licenseModalSubtitle: 'Seleciona a licenza para ver información sobre o seu uso axeitado',
  licenseUnspecified: 'Non especificado',
  licenseCanUseCommercially: 'Pódese usar comercialmente',
  licenseCanModify: 'Pódese modificar',
  licenseCanDistribute: 'Pódese distribuir',
  licenseCanSublicense: 'Pódese sublicenciar',
  licenseCanHoldLiable: 'Pódede esixir responsabilidade',
  licenseCannotHoldLiable: 'Non se pode esixir responsabilidade',
  licenseMustIncludeCopyright: 'Debe incluir copyright',
  licenseMustIncludeLicense: 'Debe incluir licenza',
  licenseFetchDetailsFailed: 'Non se puideron descargar os detalles da licenza',
  imageLightboxTitle: 'Imaxes',
  imageLightBoxProgress: ':num de :total',
  nextImage: 'Imaxe seguinte',
  previousImage: 'Imaxe anterior',
  screenshots: 'Capturas de Pantalla',
  reloadButtonLabel: 'Recargar',
  videoQuality: 'Etiqueta de calidade de vídeo',
  videoQualityDescription: 'Esta etiqueta axuda ao usuario a identificar a calidade de vídeo actual. P.e. 1080p, 720p, HD ou Móbil',
  videoQualityDefaultLabel: 'Calidade :index',
  noContentTypesAvailable: 'Non hai tipos de contido dispoñibles',
  noContentTypesAvailableDesc: 'O teu sitio está a ter dificultades para conectarse á H5P.org e listar os tipos de contido dispoñibles.',
  contentTypeCacheOutdated: 'A lista de tipos de contido non está actualizada',
  contentTypeCacheOutdatedDesc: 'O teu sitio está a ter dificultades para conectarse á H5P.org e comprobar as actualizacións para os tipos de contido. É posible que non poidas actualizalos ou instalar tipos de contido novos.',
  tryAgain: 'Tenta de novo',
  getHelp: 'Obter axuda',
  untitled: 'Sen título :libraryTitle',
  title: 'Título',
  metadata: 'Metadatos',
  addTitle: 'Engadir título',
  usedForSearchingReportsAndCopyrightInformation: 'Usado para a busca, informes e información de copyright',
  metadataSharingAndLicensingInfo: 'Metadatos (información de licenciamento e compartición)',
  fillInTheFieldsBelow : 'Enche os campos da aparte inferior',
  saveMetadata: 'Gardar metadatos',
  addAuthor: 'Gardar autor',
  confirmRemoveAuthor: 'Seguro que queres borrar este autor?',
  addNewChange: 'Engadir novo cambio',
  confirmDeleteChangeLog: 'Seguro que queres borrar esta entrada no rexistro de cambios?',
  changelogDescription: 'Algunhas licenzas requiren que se rexistren ou amosen os cambios feitos ao traballo orixinal ou aos seus derivados. Podes rexistrar os cambios aquí por razóns de licenza ou so permitir que ti e outras personas vexan o rexistro dos cambios feitos a este contido.',
  logThisChange: 'Rexistrar este cambio',
  newChangeHasBeenLogged: 'Rexistrouse un novo cambio',
  loggedChanges: 'Cambos rexistrados',
  noChangesHaveBeenLogged: 'Non se rexistraron cambios',
  errorHeader: 'Ocorreu un erro',
  errorCalculatingMaxScore: 'Non se puido calcular a puntuación máxima para este contido. Asúmese que a puntuación máxima é 0. Contacta co teu administrador se isto non é correcto.',
  maxScoreSemanticsMissing: 'Non se puido atopar a semántica esperada no contido.',
  copyButton: 'Copiar',
  copiedButton: 'Copiado',
  pasteButton: 'Pegar',
  pasteAndReplaceButton: 'Pegar & Substituír',
  pasteContent: 'Substituír contido',
  confirmPasteContent: 'Ao facer isto, substituirase o contido actual co contido do portapapeis. Perderase o contido actual. Seguro que queres continuar?',
  confirmPasteButtonText: 'Substituír contido',
  copyToClipboard: 'Copiar contido H5P ao portapapeis',
  copiedToClipboard: 'Copiouse o contido ao portapapeis',
  pasteFromClipboard: 'Pegar contido H5P do portapapeis',
  pasteAndReplaceFromClipboard: 'Substituír contido existente co contido H5P do portapapeis',
  pasteNoContent: 'Non hai contido H5P no portapapeis',
  pasteError: 'Non se pode pegar do portapapeis',
  pasteContentNotSupported: 'O contido do portapapeis non está soportado neste contexto',
  pasteContentRestricted: 'Restrinxiuse o contido do portapapeis neste sitio',
  pasteTooOld: 'O contido do portapapeis pertence a unha versión anterior (:clip) do que se soporta neste contexto (:local), se fose posible, intenta actualizar o contido que queres pegar, volve a copialo e tenta pegalo aquí de novo.',
  pasteTooNew: 'O contido do portapapeis H5P é dunha versión máis alta (:clip) que a que se soporta neste contexto (:local), se fose posible, intenta actualizar o contido que queres pegar, volve a copialo e tenta pegalo aquí de novo.',
  ok: 'OK',
  avTablistLabel: 'Inserir usando',
  tabTitleBasicFileUpload: 'Carga de Ficheiros',
  tabTitleInputLinkURL: 'Enlace/URL',
  errorTooHighVersion: 'Os parámteros conteñen %used mais so se soportan %supported ou anteriores.',
  errorNotSupported: 'Os parámetros conteñen %used que non está soportado.',
  errorParamsBroken: 'Os parámetros están rotos.',
  libraryMissing: 'Falta a librería requirida %lib.',
  scriptMissing: 'Non se puido cargar o script de actualización para %lib.',
  language: 'Idioma',
  noLanguagesSupported: 'Non hai soporte para ningún idioma',
  changeLanguage: 'Cambiar idioma á :language?',
  thisWillPotentially: "Isto podería borrar todos os textos e as traducions. Non se pode desfacer. Non se cambiará o contido mesmo. Queres seguir?",
  notAllTextsChanged: 'Non se cambiaron todos os textos, so hai soporte parcial para o :language.',
  contributeTranslations: 'Se queres completar a tradución do :language podes informarte sobre como <a href=":url" target="_new">contribuír ás traducións de H5P</a>',
  unknownLibrary: 'Desafortunadamente, o tipo de contido seleccionado \'%lib\' non está instalado neste sistema.',
  proceedButtonLabel: 'Seguir para gardar',
  enterFullscreenButtonLabel: 'Pasar a pantalla completa',
  exitFullscreenButtonLabel: 'Saír de pantalla completa',
  noContentHeader: 'Non atopas contido axeitado?',
  noContentSuggestion: 'Crea o teu propio!',
  tutorials: 'Titoriais',
  contentSectionAll: 'Todo o contido compartido',
  popularContent: 'Contido Popular',
  allPopular: 'Todo o Popular',
  newOnTheHub: 'Novo no Hub',
  allNew: 'Todo o Novo',
  filterBy: 'Filtar por',
  filter: 'Filtrar',
  filters: {
    level: {
      dropdownLabel: 'Nivel',
      dialogHeader: 'Seleccionar nivel educativo',
      dialogButtonLabel: 'Filtrar nivel educativo'
    },
    language: {
      dropdownLabel: 'Idioma',
      dialogHeader: 'Seleccionar idioma(s)',
      dialogButtonLabel: 'Filtrar idiomas',
      searchPlaceholder: 'Escribe para buscar idiomas'
    },
    reviewed: {
      dropdownLabel: 'Valorado',
      dialogHeader: 'Contido valorado',
      dialogButtonLabel: 'Filtrar',
      optionLabel: 'Amosar so contido valorado'
    },
    contentTypes: {
      dropdownLabel: 'Tipos de Contido',
      dialogHeader: 'Seleccionar Tipo(s) de Contido',
      dialogButtonLabel: 'Filtrar Tipos de Contido',
      searchPlaceholder: 'Escribe para buscar tipos de contido'
    },
    disciplines: {
      dropdownLabel: 'Materia',
      dialogHeader: 'Selecciona a túa Materia',
      dialogButtonLabel: 'Filtrar Materias',
      searchPlaceholder: 'Escribe para buscar materias'
    },
    licenses: {
      dropdownLabel: 'Licenza',
      dialogHeader: 'Seleccionar dereitos de uso preferidos',
      dialogButtonLabel: 'Filtrar licenzas',
      options: {
        modified: 'Pódese modificar',
        commercial: 'Permite uso comercial'
      }
    }
  },
  clearFilters: 'Borrar todos os filtros',
  contentSearchFieldPlaceholder: 'Buscar Contido...',
  loadingContentTitle: 'Estamos a cargar contido para ti...',
  loadingContentSubtitle: 'Agarda, por favor',
  by: 'Por',
  dropdownButton: 'Abrir despregable',
  paginationNavigation: 'Navegacion en páxinas',
  page: 'Páxina',
  currentPage: 'Páxina actual',
  nextPage: 'Ir á páxina seguinte',
  previousPage: 'Ir á páxina anterior',
  contentPreviewButtonLabel: 'Vista previa',
  contentDownloadButtonLabel: 'Obter contido',
  reuseContentTabLabel: 'Obter contido compartido',
  contentPublisherPanelHeader: 'Información editorial',
  noContentFoundDesc: 'Non hai contido que coincida cos teus criterios de busca.',
  h5pType: 'Tipo H5P',
  level: 'Nivel',
  size: 'Tamaño',
  failedFetchingData: 'Non se puideron obter os datos',
  filterErrorMessage: 'Algo saleu mal. Por favor, volve a cargar a páxina.',
  in: 'en',
  navigateToParent: 'Navegar ao pai',
  a11yTitleShowLabel: 'Amosar etiqueta para AT',
  a11yTitleHideLabel: 'Agochar etiqueta para AT',
  reuseSuccess: 'Importouse :title correctamente do hub H5P.',
};
