H5PEditor.language.core = {
  missingTranslation: '[缺少翻译 :key]',
  loading: '加载中，请稍候...',
  selectLibrary: '请选择用于创建内容的库。',
  unknownFieldPath: '找不到":path".',
  notImageField: '":path" 不是图片。',
  notImageOrDimensionsField: '":path" 不是图片或尺寸字段。',
  requiredProperty: ':property 是必需的，并且必须有一个值。',
  onlyNumbers: ':property 的值只能包含数字。',
  illegalDecimalNumber: ':property 只能包含最大为:decimals小数的数字。',
  exceedsMax: ':property 的值超过了:max的最大值。',
  listExceedsMax: '列表超过了:max个项目的上限。',
  belowMin: ':property 值低于:min的最小值。',
  listBelowMin: '该列表至少需要:min个项目，内容才能正常运行。',
  outOfStep: ':property 的值只能以:step为单位更改。',
  add: '添加',
  addFile: '添加文件',
  removeFile: '移除文件',
  confirmRemoval: '您确定要删除此:type吗？',
  removeImage: '移除图片',
  confirmImageRemoval: '这将删除您的图片。 您确定要继续吗？',
  changeFile: '更改文件',
  changeLibrary: '更改内容类型？',
  semanticsError: '语义错误：:error',
  missingProperty: '字段:index缺少其:property属性。',
  expandCollapse: '展开/折叠',
  addEntity: '添加:entity',
  tooLong: '字段值太长，应包含最多:max个字母。',
  invalidFormat: '字段值包含无效的格式或被禁止的字符。',
  confirmChangeLibrary: '继续操作将丢失使用当前内容类型创建的所有工作。 您确定要更改内容类型吗？',
  commonFields: '文字替代和翻译',
  commonFieldsDescription: '您可以在此处编辑设置或翻译此内容中使用的文本。',
  uploading: '正在上传，请稍候...',
  noFollow: '无法追踪字段":path".',
  editCopyright: '编辑版权',
  close: '关闭',
  tutorial: '使用教程',
  editMode: '编辑模式',
  listLabel: '列表',
  uploadError: '文件上传错误',
  fileToLarge: '您尝试上传的文件可能太大了。',
  unknownFileUploadError: '未知文件上传错误',
  noSemantics: '错误，无法加载内容类型表格。',
  editImage: '编辑图片',
  saveLabel: '保存',
  cancelLabel: '取消',
  resetToOriginalLabel: '重置为原始状态',
  loadingImageEditor: '正在加载图片编辑器，请稍候...',
  selectFiletoUpload: '请选择要上传的文件',
  or: '或',
  enterAudioUrl: '请输入音频源URL',
  enterVideoUrl: '请输入视频源URL',
  enterAudioTitle: '粘贴链接或其他音频源URL',
  enterVideoTitle: '粘贴YouTube链接或其他视频源URL',
  uploadAudioTitle: '上传音频文件',
  uploadVideoTitle: '上传视频文件',
  addVideoDescription:
    'H5P 支持格式为 mp4、webm 或 ogv 的所有外部视频源，如 Vimeo Pro，并支持 YouTube 和 Panopto 链接。',
  insert: '插入',
  cancel: '取消',
  height: '高度',
  width: '宽度',
  textField: '文本字段',
  numberField: '数字字段',
  orderItemUp: '向上',
  orderItemDown: '向下',
  expandAllContent: 'Expand all content',
  collapseAllContent: 'Collapse all content',
  removeItem: '移除',
  hubPanelLabel: '选择内容类型',
  importantInstructions: '重要说明',
  showImportantInstructions: '显示重要说明',
  hideImportantInstructions: '隐藏重要说明',
  hide: '隐藏',
  example: '示例',
  createContentTabLabel: '创建内容',
  uploadTabLabel: '上传',
  uploadPlaceholder: '没有选择文件',
  uploadInstructionsTitle: '上传H5P文件。',
  uploadInstructionsContent:
    '您可以从<a href="https://h5p.org/content-types-and-applications" target="blank">H5P.org</a>中的示例开始。',
  uploadFileButtonLabel: '上传文件',
  uploadFileButtonChangeLabel: '替换文件',
  uploadingThrobber: '正在上传...',
  uploadSuccess: ':title 已成功上传！',
  unableToInterpretError: '无法解释响应。',
  unableToInterpretSolution: '请检查您的错误日志。',
  h5pFileWrongExtensionTitle: '所选文件无法上传',
  h5pFileWrongExtensionContent: '仅允许使用扩展名为.h5p的文件。',
  h5pFileValidationFailedTitle: '无法验证H5P文件。',
  h5pFileValidationFailedContent: '确保上传的H5P包含有效的H5P内容。如果H5P仅包含库文件，则应通过“ H5P库”页面上载。',
  h5pFileUploadServerErrorTitle: '无法上传H5P文件',
  h5pFileUploadServerErrorContent: '发生意外错误。 请检查服务器错误日志中是否有更多信息。',
  contentTypeSectionAll: '所有内容类型',
  searchResults: '搜索结果',
  contentTypeSearchFieldPlaceholder: '搜索内容类型',
  contentTypeInstallButtonLabel: '安装',
  contentTypeInstallingButtonLabel: '正在安装',
  contentTypeUseButtonLabel: '应用',
  contentTypeDetailButtonLabel: '详细信息',
  contentTypeUpdateButtonLabel: '更新',
  contentTypeUpdatingButtonLabel: '正在更新',
  contentTypeGetButtonLabel: '获取',
  contentTypeBackButtonLabel: '返回',
  contentTypeIconAltText: '图标',
  contentTypeInstallSuccess: ':contentType 成功安装！',
  contentTypeUpdateSuccess: ':contentType 成功更新！',
  contentTypeInstallError: ':contentType 无法安装。请与您的管理员联系。',
  contentTypeLicensePanelTitle: '许可证',
  contentTypeDemoButtonLabel: '演示内容',
  numResults: ':num 条结果',
  show: '显示',
  recentlyUsedFirst: '最近使用',
  popularFirst: '热门',
  newestFirst: '最新',
  aToZ: '按名称',
  noResultsFound: '未找到结果',
  noResultsFoundDesc: '没有与您的搜索条件匹配的内容类型。',
  readMore: '更多',
  readLess: '更少',
  contentTypeOwner: '由 :owner',
  contentTypeUnsupportedApiVersionTitle: '此内容类型需要一个更新的核心版本',
  contentTypeUnsupportedApiVersionContent: '请与您的系统管理员联系安装必要的更新',
  contentTypeUpdateAvailable: '有可用更新',
  contentTypeRestricted: '受限内容类型',
  contentTypeRestrictedDesc: '管理员已限制使用此内容类型。',
  contentTypeNotInstalled: '内容类型未安装',
  contentTypeNotInstalledDesc: '您无权安装内容类型。',
  theContentType: '内容类型',
  currentMenuSelected: '当前选择',
  errorCommunicatingHubTitle: '无法与H5P Hub通信。',
  errorCommunicatingHubContent: '出错了，请再试一次。',
  warningNoContentTypesInstalled: '您没有安装任何内容类型。',
  warningChangeBrowsingToSeeResults: '单击<em>全部</em>以获取可以安装的所有内容类型的列表。',
  warningUpdateAvailableTitle: '有:contentType的更新可用。',
  warningUpdateAvailableBody: '更新到最新版本以获得更好的使用体验。',
  licenseDescription: '该许可证的某些功能如下所示。 单击上方的信息图标以阅读原始许可证文本。',
  licenseModalTitle: '许可证详细信息',
  licenseModalSubtitle: '选择一个许可证以查看有关正确使用的信息',
  licenseUnspecified: '未指定',
  licenseCanUseCommercially: '可以商业使用',
  licenseCanModify: '可以修改',
  licenseCanDistribute: '可以分发',
  licenseCanSublicense: '可以再许可',
  licenseCanHoldLiable: '承担责任',
  licenseCannotHoldLiable: '不承担责任',
  licenseMustIncludeCopyright: '必须包含版权信息',
  licenseMustIncludeLicense: '必须包含许可证',
  licenseFetchDetailsFailed: '获取许可证详细信息失败',
  imageLightboxTitle: '图片',
  imageLightBoxProgress: ':num / :total',
  nextImage: '下一个图片',
  previousImage: '上一个图片',
  screenshots: '屏幕截图',
  reloadButtonLabel: '重新加载',
  videoQuality: '视频质量标签',
  videoQualityDescription: '此标签可帮助用户识别视频的当前视频质量。 例如。 1080p，720p，高清或手机端',
  videoQualityDefaultLabel: '视频质量 :index',
  noContentTypesAvailable: '没有可用的内容类型',
  noContentTypesAvailableDesc: '您的网站在连接到H5P.org并列出可用的内容类型时遇到问题。',
  contentTypeCacheOutdated: '内容类型列表已过时',
  contentTypeCacheOutdatedDesc: '您的网站无法连接到H5P.org以检查内容类型更新。 您可能无法更新或安装新的内容类型。',
  tryAgain: '再试一次',
  getHelp: '获取帮助',
  untitled: '未命名:libraryTitle',
  title: '标题',
  metadata: '元数据',
  addTitle: '添加标题',
  usedForSearchingReportsAndCopyrightInformation: '用于搜索，报告和版权信息',
  metadataSharingAndLicensingInfo: '元数据（共享和许可信息）',
  fillInTheFieldsBelow: '请填写下面的字段',
  saveMetadata: '保存元数据',
  addAuthor: '保存作者',
  confirmRemoveAuthor: '您确定要删除这个作者吗？',
  addNewChange: '新增变更',
  confirmDeleteChangeLog: '您确定要删除这个更新日志条目吗？',
  changelogDescription:
    '有些许可证要求记录并显示对原始作品或衍生作品所做的更改。 您可以根据许可证要求的原因在此处记录您的更改，或者只是为了让自己和他人跟踪对此内容所做的更改。',
  logThisChange: '记录此更改',
  newChangeHasBeenLogged: '已记录新的更改',
  loggedChanges: '已记录的更改',
  noChangesHaveBeenLogged: '尚未记录任何更改',
  errorHeader: '发生错误',
  errorCalculatingMaxScore: '无法计算此内容的最高分。 当前最高分数设定为0。如果您觉得设置不正确，请与管理员联系。',
  maxScoreSemanticsMissing: '在内容中找不到预期的语义。',
  copyButton: '复制',
  copiedButton: '已复制',
  pasteButton: '粘贴',
  pasteAndReplaceButton: '粘贴并替换',
  pasteContent: '替换内容',
  confirmPasteContent: '通过此操作，您将用剪贴板中的内容替换当前内容。 当前内容将丢失。 您确定要继续吗？',
  confirmPasteButtonText: '替换内容',
  copyToClipboard: '将H5P内容复制到剪贴板',
  copiedToClipboard: '内容已复制到剪贴板',
  pasteFromClipboard: '从剪贴板粘贴H5P内容',
  pasteAndReplaceFromClipboard: '用剪贴板中的H5P内容替换现有内容',
  pasteNoContent: '剪贴板中没有H5P内容',
  pasteError: '无法从剪贴板粘贴',
  pasteContentNotSupported: '当前环境不支持H5P剪贴板中的内容',
  pasteContentRestricted: '剪贴板中的内容已在此站点上受到限制',
  pasteTooOld:
    'H5P剪贴板中的内容（:clip）的版本低于当前环境（:local）中支持的版本，如果可能，请尝试升级要粘贴的内容，然后再次复制并尝试将其粘贴到此处。',
  pasteTooNew:
    'H5P剪贴板中的内容（：clip）的版本高于当前环境（：local）中支持的版本，如果可能，请尝试先升级此内容，然后再尝试将其粘贴到此处。',
  ok: '我知道了',
  avTablistLabel: '插入使用',
  tabTitleBasicFileUpload: '上传文件',
  tabTitleInputLinkURL: '链接/URL',
  errorTooHighVersion: '参数中包含%used，而仅支持%supported或更早的版本。',
  errorNotSupported: '参数中包含不支持的%used。',
  errorParamsBroken: '参数已损坏。',
  libraryMissing: '缺少必需的库 %lib。',
  scriptMissing: '无法加载 %lib的升级脚本。',
  language: '语言',
  noLanguagesSupported: '没有支持的语言',
  changeLanguage: '更改语言为 :language？',
  thisWillPotentially: '这可能会重置所有文本和翻译。 您无法撤消此操作。 内容本身不会被更改。 您要继续吗？',
  notAllTextsChanged: '并非所有文本都已更改，:language仅部分覆盖。',
  contributeTranslations:
    '如果您想完善或帮助 :language的语言翻译，可以了解有关<a href=":url" target="_new">为H5P贡献翻译的信息</a>',
  unknownLibrary: "很遗憾，此系统上未安装选定的内容类型 '%lib'。",
  proceedButtonLabel: '继续保存',
  enterFullscreenButtonLabel: '进入全屏',
  exitFullscreenButtonLabel: '退出全屏',
  noContentHeader: '没有合适的内容？',
  noContentSuggestion: '自己创建一个吧！',
  tutorials: '使用教程',
  contentSectionAll: '所有共享内容',
  popularContent: '热门内容',
  allPopular: '所有热门',
  newOnTheHub: '最新添加',
  allNew: '所有最新内容',
  filterBy: '筛选条件',
  filter: '筛选',
  filters: {
    level: { dropdownLabel: '程度', dialogHeader: '选择教育程度', dialogButtonLabel: '按教育程度' },
    language: {
      dropdownLabel: '语言',
      dialogHeader: '选择语言',
      dialogButtonLabel: '按语言',
      searchPlaceholder: '键入以搜索语言',
    },
    reviewed: {
      dropdownLabel: '已审核',
      dialogHeader: '已审核内容',
      dialogButtonLabel: '筛选',
      optionLabel: '仅显示已审核的内容',
    },
    contentTypes: {
      dropdownLabel: '内容类型',
      dialogHeader: '选择内容类型',
      dialogButtonLabel: '按内容类型',
      searchPlaceholder: '键入以搜索内容类型',
    },
    disciplines: {
      dropdownLabel: '学科',
      dialogHeader: '选择学科',
      dialogButtonLabel: '按学科',
      searchPlaceholder: '键入以搜索学科',
    },
    licenses: {
      dropdownLabel: '许可证',
      dialogHeader: '选择许可证类型',
      dialogButtonLabel: '按许可证类型',
      options: { modified: '可以修改', commercial: '允许商业用途' },
    },
  },
  clearFilters: '清除所有过滤器',
  contentSearchFieldPlaceholder: '搜索内容',
  loadingContentTitle: '正在为您加载内容...',
  loadingContentSubtitle: '请稍等',
  by: '作者',
  dropdownButton: '打开下拉菜单',
  paginationNavigation: '分页导航',
  page: '页',
  currentPage: '当前页面',
  nextPage: '转到下一页',
  previousPage: '转到上一页',
  contentPreviewButtonLabel: '预览',
  contentDownloadButtonLabel: '获取内容',
  reuseContentTabLabel: '获取共享内容',
  contentPublisherPanelHeader: '出版商信息',
  noContentFoundDesc: '没有找到符合您的搜索条件的内容。',
  h5pType: 'H5P类型',
  level: '等级',
  size: '大小',
  failedFetchingData: '获取数据失败',
  filterErrorMessage: '出问题了，请重新加载页面。',
  in: 'in',
  navigateToParent: '导航到父级',
  a11yTitleShowLabel: '显示 AT 的标签',
  a11yTitleHideLabel: '隐藏 AT 的标签',
  reuseSuccess: ':title 已成功从 H5P Hub 导入。',
};
