{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_message/message_drawer_view_conversation_header_edit_mode

    This template will render the header while the conversation is in edit mode on
    the conversation page of the message drawer.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * All data attributes are required

    Context variables required for this template:
    * userid The logged in user id
    * urls The URLs for the popover

    Example context (json):
    {}

}}

<div class="d-flex p-2 align-items-center">
    {{#str}} messagesselected:, core_message {{/str}}
    <span class="ms-1" data-region="message-selected-court">1</span>
    <button type="button" class="ms-auto btn-close" aria-label="{{#str}} cancelselection, core_message {{/str}}"
        data-action="cancel-edit-mode">
    </button>
</div>
