{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_courseformat/local/content/bulkedittools

    Displays the bulk actions button in the page header.

    Example context (json):
    {
        "id": 42,
        "hasactions": true,
        "actions": [
            {
                "icon": "i/delete",
                "action": "cmDelete",
                "name": "delete",
                "bulk": "cm",
                "title": "Delete activities"
            }
        ]
    }
}}
{{< core/sticky_footer }}
    {{$ stickyclasses }} justify-content-center bulkactions {{/ stickyclasses }}
    {{$ disable }} data-disable="true" {{/ disable }}
    {{$ extradata }} data-for="bulkedittools" {{/ extradata }}
    {{$ stickycontent }}
    <div class="form-check">
        <input type="checkbox" class="form-check-input" id="selectall" data-for="selectall" disabled>
        <label class="form-check-label" for="selectall">
            {{#str}} selectall {{/str}}
        </label>
    </div>
    <div class="mx-auto" data-for="bulktools">
        {{^hasactions}}
        {{#str}} nobulkaction, core_courseformat {{/str}}
        {{/hasactions}}
        {{#hasactions}}
        <ul class="actions nav flex-nowrap" data-for="bulkactions">
            {{#actions}}
            <li class="nav-item">
                <button
                    class="btn py-0 d-flex flex-column"
                    data-action="{{action}}"
                    data-bulk="{{bulk}}"
                    data-for="bulkaction"
                    {{#title}} title="{{title}}" {{/title}}
                >
                    <span class="bulkaction-icon w-100 ps-2">{{#pix}}{{icon}}{{/pix}}</span>
                    <span class="bulkaction-name">{{name}}</span>
                </button>
            </li>
            {{/actions}}
        </ul>
        {{/hasactions}}
    </div>
    <div class="d-flex flex-column">
        <div class="bulkcancel ms-auto">
            <button
                class="btn pe-0 pb-0"
                data-action="bulkcancel"
                data-for="bulkcancel"
                title="{{#str}} bulkcancel, core_courseformat {{/str}}"
            >
                {{#pix}} e/cancel, core {{/pix}}
            </button>
        </div>
        <div class="bulkcount" data-for="bulkcount" role="status">
            {{#str}} bulkselection, core_courseformat, 0 {{/str}}
        </div>
    </div>
    {{/ stickycontent }}
{{/ core/sticky_footer }}
{{#js}}
require(['core_courseformat/local/content/bulkedittools'], function(component) {
    component.init('[data-for="bulkedittools"]');
});
{{/js}}
