{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template gradereport_singleview/report

    Report table and sticky footer.

    Context variables required for this template:
    * sesskey - string - The session key.
    * table - string - HTML content of the grade items or users table.
    * stickyfooter - string - HTML content of the sticky footer.

    Example context (json):
    {
        "sesskey": "fakesesskey",
        "table": "<div><div class='reporttable'><div class='tableresponsive'><table class='generaltable' id='singleview-grades'><thead> <tr><th>Name</th><th>User fullname</th><th>Grade</th><th>Actions</th> </tr></thead><tbody></tbody></table></div></div></div>",
        "stickyfooter": "<div id='sticky-footer'></div>"
    }
}}
<form method="post">
    {{{table}}}
    {{{stickyfooter}}}
    <input type="hidden" name="sesskey" value="{{sesskey}}">
</form>
