{{!
    This file is part of Moodle - http://moodle.org/
    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_feedback/create_template
    Trigger button for the create template modal form.
    Classes required for JS:
    * none
    Data attributes required for JS:
    * All data attributes are required
    Context variables required for this template:
    * id - The feedback id.
    Example context (json):
    {
        "id": 1
    }
}}
<button type="button" class="btn btn-secondary" data-action="createtemplate" data-dataid={{id}}>
    {{#str}}save_as_new_template, mod_feedback{{/str}}
</button>
{{#js}}
    require(['mod_feedback/createtemplate'], function(createtemplate) {
    createtemplate.init();
    });
{{/js}}
