{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template tool_lp/competency_rule_config

    Configuration dialogue for competency rules.

    Context variables required for this template:
    * config - boolean
    * outcomes - array
      * code
      * name
      * selected
    * rules - array
      * type
      * name
      * selected

    Example context (json):
    {
        "config": true,
        "outcomes": [
            {
                "code": "C1",
                "name": "Complete",
                "selected": true
            }
        ],
        "rules": [
            {
                "type": "Type",
                "name": "Something happens",
                "selected": true
            }
        ]
    }
}}
<div data-region="competencyruleconfig">
    <div data-region="content">

        {{^config}}
        <p>{{#str}}cannotaddrules, tool_lp{{/str}}</p>
        {{/config}}

        {{#config}}
        <div data-region="rule-base" class="form">
            <div data-region="rule-outcome" class="mb-3">
                <label>{{#str}}outcome, tool_lp{{/str}}</label>
                <select name="outcome" class="form-select">
                    {{#outcomes}}
                    <option value="{{code}}" {{#selected}}selected{{/selected}}>{{name}}</option>
                    {{/outcomes}}
                </select>
            </div>
            <div data-region="rule-type" class="mb-3">
                <label>{{#str}}when, tool_lp{{/str}}</label>
                <select name="rule" class="form-select">
                    <option value="-1">{{#str}}choosedots{{/str}}</option>
                    {{#rules}}
                    <option value="{{type}}" {{#selected}}selected{{/selected}}>{{name}}</option>
                    {{/rules}}
                </select>
            </div>
        </div>
        <div data-region="rule-config">
        </div>
        {{/config}}
    </div>

    <div data-region="footer" class="float-sm-end mt-1">
        {{#config}}
        <input type="button" class="btn btn-primary" data-action="save" value="{{#str}}savechanges{{/str}}"/>
        {{/config}}
        <input type="button" class="btn btn-secondary" data-action="cancel" value="{{#str}}cancel{{/str}}"/>
    </div>
    <div class="clearfix"></div>
</div>
