//
// Bootstrap variables
//

// Prefix for all CSS variables
$prefix: bs- !default;

// Colors
$white:    #fff !default;
$gray-100: #f8f9fa !default;
$gray-200: #e9ecef !default;
$gray-300: #dee2e6 !default;
$gray-400: #ced4da !default;
$gray-500: #8f959e !default;
$gray-600: #6a737b !default;
$gray-700: #495057 !default;
$gray-800: #343a40 !default;
$gray-900: #1d2125 !default;
$black:    #000 !default;

$blue:    #0f6cbf !default;
$indigo:  #6610f2 !default;
$purple:  #613d7c !default;
$pink:    #db1a74 !default;
$red:     #ca3120 !default;
$orange:  #ff7518 !default;
$yellow:  #f0ad4e !default;
$green:   #357a32 !default;
$teal:    #20c997 !default;
$cyan:    #008196 !default;

$primary:       $blue !default;
$secondary:     $gray-400 !default;
$success:       $green !default;
$info:          $cyan !default;
$warning:       $yellow !default;
$danger:        $red !default;
$light:         $gray-100 !default;
$dark:          $gray-900 !default;

// stylelint-disable
$theme-colors: () !default;
$theme-colors: map-merge((
    "primary":    $primary,
    "secondary":  $secondary,
    "success":    $success,
    "info":       $info,
    "warning":    $warning,
    "danger":     $danger,
    "light":      $light,
    "dark":       $dark
), $theme-colors);
// stylelint-enable

// Fonts
$font-size-base: 0.9375rem !default;

// Links
$link-decoration: none !default;
$link-hover-decoration: underline !default;

// Border radius
$border-radius: .5rem !default;
$border-radius-lg: .6rem !default;

// HR
$hr-opacity: .15 !default;

// Breadcrumbs
$breadcrumb-padding-y:      .5rem !default;
$breadcrumb-margin-bottom:  0 !default;

// Tables
$table-striped-bg-factor: .03 !default;

// Cards
$card-group-margin: .25rem;

// Inputs
$focus-ring-opacity: .75 !default;
$input-border-color: var(--#{$prefix}gray-500) !default;
$form-check-input-border: var(--#{$prefix}border-width) solid var(--#{$prefix}gray-500) !default;

// Dropdowns
$dropdown-link-hover-color: $white;
$dropdown-link-hover-bg: $primary;

// Popovers
$popover-max-width: 300px !default;

// Spacers
$spacer: 1rem !default;
$spacers: (
    0: 0,
    1: ($spacer * .25),
    2: ($spacer * .5),
    3: $spacer,
    4: ($spacer * 1.5),
    5: ($spacer * 2),
    6: ($spacer * 3)
) !default;

// Behat site specific variable values.
$behatsite: false !default;
$enable-transitions: not $behatsite !default;
$enable-smooth-scroll: not $behatsite !default;

// Import FontAwesome.
@import "fontawesome";

// Import All of Bootstrap
@import "bootstrap";

// Import Core moodle CSS
@import "moodle";

//
// Preset CSS
//

body {
    -webkit-font-smoothing: antialiased;
    -moz-osx-font-smoothing: grayscale;
}

.navbar {
    box-shadow: 0 2px 4px rgba(0, 0, 0, .08);
}

// Rounded user pictures
.userpicture {
    border-radius: 50%;
}

// Reset the default styling back to the bootstrap defaults for
// the secondary outline button because gray-200 is much too light
// for an outline button.

$info-outline:    #1f7e9a;
$warning-outline: #a6670e;

.btn-outline-secondary {
    @include button-outline-variant($gray-600);
    border-color: $gray-600;
}

.btn-outline-info {
    @include button-outline-variant($info-outline);
}

.btn-outline-warning {
    @include button-outline-variant($warning-outline);
}
