{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_calendar/add_event_button

    Button to launch the "Add new event" dialogue.

    The purpose of this template is to render the button used to generate the new event dialogue.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Example context (json):
    {
        "defaulteventcontext": "2"
    }
}}
{{#defaulteventcontext}}
<button{{!
    }} class="btn btn-primary float-sm-end float-end ms-1 mb-2"{{!
    }} data-context-id="{{defaulteventcontext}}"{{!
    }} data-action="new-event-button"{{!
    }}>
    {{#str}}newevent, core_calendar{{/str}}
</button>
{{/defaulteventcontext}}
