{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_message/message_drawer_view_conversation_footer_require_unblock

    This template will render the footer content when the user must unblock the other
    user in a conversation in the message drawer.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * All data attributes are required

    Context variables required for this template:
    * userid The logged in user id
    * urls The URLs for the popover

    Example context (json):
    {}

}}

<div class="p-3 bg-white">
    <p class="text-muted" data-region="text">{{#str}} youhaveblockeduser, core_message {{/str}}</p>
    <button type="button" class="btn btn-primary w-100" data-action="request-unblock">
        <span data-region="dialogue-button-text">{{#str}} unblockuser, core_message {{/str}}</span>
        <span class="hidden" data-region="loading-icon-container">{{> core/loading }}</span>
    </button>
</div>
