{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_calendar/event_details

    Calendar event details.

    The purpose of this template is to render the event details.

    This template is used in the summary modal, day and upcoming views to output event information consistently
    across the calendar.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Example context (json):
    {
        "formattedtime": "Wednesday, 17 April, 9:27 AM",
        "formattedlocation": "Moodle HQ",
        "normalisedeventtype": "Group",
        "normalisedeventtypetext": "Group event",
        "description": "An random event description",
        "isactionevent": "true",
        "course": {
            "viewurl": "http://mymoodlesite/course/view.php?id=1",
            "fullname": "Course name"
        },
        "source": "Ical imported",
        "groupname": "Group 1",
        "iscategoryevent": "true",
        "category": {
            "nestedname": "Category name"
        },
        "iscourseevent": "true",
        "groupname": "Group name",
        "subscription": "true",
        "displayeventsource": "true",
        "subscriptionname": "Subscription name",
        "subscriptionurl": "http://subscriptionurl"
    }
}}
<div class="row">
    <div class="col-1">{{#pix}} i/calendareventtime, core, {{#str}} when, core_calendar {{/str}} {{/pix}}</div>
    <div class="col-11">{{{formattedtime}}}</div>
</div>
{{#normalisedeventtypetext}}
    <div class="row mt-1">
        <div class="col-1">{{#pix}} i/calendar, core, {{#str}} eventtype, core_calendar {{/str}} {{/pix}}</div>
        <div class="col-11">{{normalisedeventtypetext}}</div>
    </div>
{{/normalisedeventtypetext}}
{{#description}}
    <div class="row mt-1">
        <div class="col-1">{{#pix}} i/calendareventdescription, core, {{#str}} description {{/str}} {{/pix}}</div>
        <div class="description-content col-11">{{{.}}}</div>
    </div>
{{/description}}
{{#formattedlocation}}
    <div class="row mt-1">
        <div class="col-1">{{#pix}} i/location, core, {{#str}} location {{/str}} {{/pix}}</div>
        <div class="location-content col-11">{{{.}}}</div>
    </div>
{{/formattedlocation}}
{{#iscategoryevent}}
    <div class="row mt-1">
        <div class="col-1">{{#pix}} i/categoryevent, core, {{#str}} category {{/str}} {{/pix}}</div>
        <div class="col-11">{{{category.nestedname}}}</div>
    </div>
{{/iscategoryevent}}
{{#course.viewurl}}
    <div class="row mt-1">
        <div class="col-1">{{#pix}} i/courseevent, core, {{#str}} course {{/str}} {{/pix}}</div>
        <div class="col-11"><a href="{{course.viewurl}}">{{{course.fullname}}}</a></div>
    </div>
{{/course.viewurl}}
{{#groupname}}
    <div class="row mt-1">
        <div class="col-1">{{#pix}} i/groupevent, core, {{#str}} group {{/str}} {{/pix}}</div>
        <div class="col-11">{{{groupname}}}</div>
    </div>
{{/groupname}}
{{#subscription}}
    {{#displayeventsource}}
        <div class="row mt-1">
            <div class="col-1">{{#pix}} i/rss, core, {{#str}} eventsource, core_calendar {{/str}} {{/pix}}</div>
            <div class="col-11">
                {{#subscriptionurl}}
                    <p><a href="{{subscriptionurl}}">{{#str}}subscriptionsource, core_calendar, {{{subscriptionname}}}{{/str}}</a></p>
                {{/subscriptionurl}}
                {{^subscriptionurl}}
                    <p>{{#str}}subscriptionsource, core_calendar, {{{subscriptionname}}}{{/str}}</p>
                {{/subscriptionurl}}
            </div>
        </div>
    {{/displayeventsource}}
{{/subscription}}
