{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_ai/policymodal

    Modal to allow acceptance of the AI Policy.

    To be used in combination with core_ai/policy ESM monly.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * elementid: The id of the element that will be used to create the modal.

    Example context (json):
    {
        "elementid": "exampleId"
    }
}}
{{< core/modal }}

    {{$title}}
        {{#str}} aiusagepolicy, core_ai{{/str}}
    {{/title}}

    {{$body}}
        <div id="{{elementid}}_tiny_aiplacement_blur" class="">
            <div id="{{elementid}}_tiny_aiplacement_overlay" class="tiny-aiplacement-overlay hidden"></div>
            <div class="ai-policy-display">
                {{#str}} userpolicy, core_ai{{/str}}
            </div>
        </div>
    {{/body}}

    {{$footer}}
        <button type="button" class="btn btn-secondary" data-action="cancel">{{#str}} declineaipolicy, core_ai {{/str}}</button>
        <button type="button" class="btn btn-primary" data-action="save">{{#str}} acceptai, core_ai {{/str}}</button>
    {{/footer}}

{{/ core/modal }}
