{{!
    This file is part of Moodle - http://moodle.org/
    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_courseformat/fileuploader
    Displays the complete course format.
    Example context (json):
    {
        "filename": "course-format.html",
        "uploadid": "example",
        "handlers": [
            {
                "module": "resource",
                "labelid": "resource_file1",
                "message": "Create a ressource",
                "selected": true,
                "value": 1
            },
            {
                "module": "h5pactivity",
                "labelid": "resource_file2",
                "message": "Create an H5P activity",
                "selected": false,
                "value": 2
            }
        ]
    }
}}
<p>
    {{#str}} actionchoice, moodle, {{filename}}{{/str}}
</p>
<div id="dndupload_handlers{{uploadid}}">
{{#handlers}}
    <input
        type="radio"
        name="handler"
        value="{{value}}"
        id="{{labelid}}"
        {{#selected}} checked="checked" {{/selected}}
    />
    <label for="{{labelid}}">{{message}}</label>
    <br/>
{{/handlers}}
</div>
