{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_courseformat/local/content/sectionselector

    Displays the course section navigation.

    Example context (json):
    {
        "hasprevious": true,
        "previousurl": "#",
        "larrow": "&#x25C4;",
        "previousname": "Section 3",
        "hasnext": true,
        "rarrow": "&#x25BA;",
        "nexturl": "#",
        "nextname": "Section 5",
        "selector": "<select><option>Section 4</option></select>"
    }
}}
<div class="section-navigation mdl-bottom d-flex justify-content-between">
    <div class="prevsection">
        {{#hasprevious}}
        <a href="{{{previousurl}}}" class="{{#previoushidden}} dimmed_text {{/previoushidden}}">
            <span class="larrow">{{{larrow}}}</span>{{{previousname}}}
        </a>
        {{/hasprevious}}
    </div>
    <div>
        {{{selector}}}
    </div>
    <div class="nextsection">
        {{#hasnext}}
        <a href="{{{nexturl}}}" class="{{#nexthidden}} dimmed_text {{/nexthidden}}">
            <span class="rarrow">{{{rarrow}}}</span>{{{nextname}}}
        </a>
        {{/hasnext}}
    </div>
</div>
