.h5p-metadata-popup-overlay {
  position: absolute;
  top: 0;
  left: 0;
  right: 0;
  bottom: 0;
  background-color: rgba(0, 0, 0, 0.85);
  // Need to be higer than the different z-indexes in the custom editors
  // Like eg popups in CP, DnD and IV
  z-index: 101;
  text-align: center;
}
