H5PEditor.language.core = {
  missingTranslation: '[Manglar oversettelse :key]',
  loading: 'Lastar, ver venleg og vent...',
  selectLibrary: 'Vel biblioteket du ønskjer å bruke for innhaldet ditt.',
  unknownFieldPath: 'Kan ikkje finne ":path".',
  notImageField: '":path" er ikkje eit bilete.',
  notImageOrDimensionsField: '":path" er korkje eit bilete- eller dimensjonsfelt.',
  requiredProperty: '":property" er påkravd og må ha ein verdi.',
  onlyNumbers: '":property" kan berre innhalde tal.',
  illegalDecimalNumber: ':property can only contain numbers with max :decimals decimals.',
  exceedsMax: '":property" overstig maksverdien på :max.',
  listExceedsMax: 'Antallet elementer i listen overstiger maksantallet på :max elementer.',
  belowMin: '":property" er mindre enn minimumsverdien på :min.',
  listBelowMin: 'Listen trenger minst :min elementer for at innholdet skal virke.',
  outOfStep: '":property" kan berre endrast i steg på :step.',
  add: 'Legg til',
  addFile: 'Legg til fil',
  removeFile: 'Fjern fil',
  confirmRemoval: 'Er du sikker på at du ønskjer å fjerne denne :type?',
  removeImage: 'Fjern bilde',
  confirmImageRemoval: 'Er du sikker på at du ønskjer å fjerne dette bildet?',
  changeFile: 'Bytt fil',
  changeLibrary: 'Endre innholdstype',
  semanticsError: 'Semantikkfeil: :error',
  missingProperty: 'Feltet :index manglar :property attributten sin.',
  expandCollapse: 'Utvid/Slå saman',
  addEntity: 'Legg til :entity',
  tooLong: 'Feltets verdi er for lang, den må vere på :max tegn eller mindre.',
  invalidFormat: 'Feltets verdi er på eit ugyldig format eller bruker ulovlege tegn.',
  confirmChangeLibrary:
    'Gjør du dette mister du alt arbeid gjort med nåværande innhaldstype. Er du sikker på at du ønskjer å byte innhaldstype?',
  commonFields: 'Innstillinger og tekstar',
  commonFieldsDescription: 'Her kan du redigere innstillinger eller oversette tekstar som brukast i dette innhaldet.',
  uploading: 'Lastar opp fil, ver venleg og vent...',
  noFollow: 'Kunne ikkje følge feltet ":path".',
  editCopyright: 'Rediger opphavsrett',
  close: 'Lukk',
  tutorial: 'Veiledning',
  editMode: 'Redigeringsmodus',
  listLabel: 'Liste',
  uploadError: 'Filopplasting feilet',
  fileToLarge: 'Filen du prøver å laste opp kan være for stor.',
  unknownFileUploadError: 'Ukjent filopplastingsfeil',
  noSemantics: 'Feil, kunne ikke laste skjemaet for innholdstypen.',
  editImage: 'Rediger bilete',
  saveLabel: 'Lagre',
  cancelLabel: 'Avbryt',
  resetToOriginalLabel: 'Tilbakestill bilde',
  loadingImageEditor: 'Laster inn bilderedigering, vennligst vent...',
  selectFiletoUpload: 'Velg fil som skal lastes opp',
  or: 'eller',
  enterAudioUrl: 'Skriv inn nettadresse til lydkjelde',
  enterVideoUrl: 'Skriv inn nettadresse til video',
  enterAudioTitle: 'Lim inn link eller annan lydkjelde URL',
  enterVideoTitle: 'Lim inn YouTube lenke eller annen videokjelde URL',
  uploadAudioTitle: 'Last opp lydfil',
  uploadVideoTitle: 'Last opp videofil',
  addVideoDescription:
    'H5P støtter alle eksterne videokilder på formatene mp4, webm eller ogv, slik som Vimeo Pro, og har støtte for YouTube-lenker og Panopto-lenker.',
  insert: 'Sett inn',
  cancel: 'Avbryt',
  height: 'Høyde',
  width: 'Bredde',
  textField: 'Tekstfelt',
  numberField: 'Nummerfelt',
  orderItemUp: 'Flytt element opp',
  orderItemDown: 'Flytt element ned',
  expandAllContent: 'Expand all content',
  collapseAllContent: 'Collapse all content',
  removeItem: 'Fjern element',
  hubPanelLabel: 'Select content type',
  importantInstructions: 'Viktige instruksjonar',
  showImportantInstructions: 'Vis instruksjonar',
  hideImportantInstructions: 'Skjul viktige instruksjonar',
  hide: 'Skjul',
  example: 'Eksempel',
  createContentTabLabel: 'Create Content',
  uploadTabLabel: 'Upload',
  uploadPlaceholder: 'No file chosen',
  uploadInstructionsTitle: 'Upload an H5P file.',
  uploadInstructionsContent:
    'You may start with examples from <a href="https://h5p.org/content-types-and-applications" target="blank">H5P.org</a>.',
  uploadFileButtonLabel: 'Upload a file',
  uploadFileButtonChangeLabel: 'Change file',
  uploadingThrobber: 'Now uploading...',
  uploadSuccess: ':title ble lastet opp!',
  unableToInterpretError: 'Kan ikke tolke respons.',
  unableToInterpretSolution: 'Vennligst sjekk feilloggen din.',
  h5pFileWrongExtensionTitle: 'The selected file could not be uploaded',
  h5pFileWrongExtensionContent: 'Only files with the .h5p extension are allowed.',
  h5pFileValidationFailedTitle: 'Could not validate H5P file.',
  h5pFileValidationFailedContent:
    'Make sure the uploaded H5P contains valid H5P content. H5P files containing only libraries should be uploaded through the H5P Libraries page.',
  h5pFileUploadServerErrorTitle: 'The H5P file could not be uploaded',
  h5pFileUploadServerErrorContent: 'An unexpected error occured. Check your server error log for more details.',
  contentTypeSectionAll: 'Alle innhaldstypar',
  searchResults: 'Søkeresultater',
  contentTypeSearchFieldPlaceholder: 'Search for Content Types',
  contentTypeInstallButtonLabel: 'Install',
  contentTypeInstallingButtonLabel: 'Installing',
  contentTypeUseButtonLabel: 'Use',
  contentTypeDetailButtonLabel: 'Details',
  contentTypeUpdateButtonLabel: 'Update',
  contentTypeUpdatingButtonLabel: 'Updating',
  contentTypeGetButtonLabel: 'Get',
  contentTypeBackButtonLabel: 'Back',
  contentTypeIconAltText: 'Icon',
  contentTypeInstallSuccess: ':contentType successfully installed!',
  contentTypeUpdateSuccess: ':contentType successfully updated!',
  contentTypeInstallError: ':contentType could not be installed. Contact your administrator.',
  contentTypeLicensePanelTitle: 'License',
  contentTypeDemoButtonLabel: 'Content Demo',
  numResults: ':num treff',
  show: 'Vis',
  recentlyUsedFirst: 'Nylig brukt først',
  popularFirst: 'Populære først',
  newestFirst: 'Nyeste først',
  aToZ: 'A til Z',
  noResultsFound: 'Ingen resultater',
  noResultsFoundDesc: 'Det finnes ingen innholdstype som samsvarer med søkekriteriene dine.',
  readMore: 'Read more',
  readLess: 'Read less',
  contentTypeOwner: 'By :owner',
  contentTypeUnsupportedApiVersionTitle: 'This content type requires a newer core version',
  contentTypeUnsupportedApiVersionContent:
    'Contact your system administrator to provide you with the necessary updates',
  contentTypeUpdateAvailable: 'Update available',
  contentTypeRestricted: 'Begrenset tilgong til innhaldstype',
  contentTypeRestrictedDesc: 'Bruken av denne innhaldstypen er begrenset av ein administrator.',
  contentTypeNotInstalled: 'Innehaldstype ikkje installert',
  contentTypeNotInstalledDesc: 'Du har ikkje tilgong til å installere innhaldstypar.',
  theContentType: 'the content type',
  currentMenuSelected: 'current selection',
  errorCommunicatingHubTitle: 'Not able to communicate with hub.',
  errorCommunicatingHubContent: 'An error occured. Please try again.',
  warningNoContentTypesInstalled: "You don't have any content types installed.",
  warningChangeBrowsingToSeeResults: 'Click <em>All</em> to get the list of all the content types you can install.',
  warningUpdateAvailableTitle: 'A new version of :contentType is available.',
  warningUpdateAvailableBody: 'Update to the latest version for an improved experience.',
  licenseDescription:
    'Some of the features of this license are indicated below. Click the info icon above to read the original license text.',
  licenseModalTitle: 'License Details',
  licenseModalSubtitle: 'Select a license to view information about proper usage',
  licenseUnspecified: 'Unspecified',
  licenseCanUseCommercially: 'Can use commercially',
  licenseCanModify: 'Can modify',
  licenseCanDistribute: 'Can distribute',
  licenseCanSublicense: 'Can sublicense',
  licenseCanHoldLiable: 'Can hold liable',
  licenseCannotHoldLiable: 'Cannot hold liable',
  licenseMustIncludeCopyright: 'Must include copyright',
  licenseMustIncludeLicense: 'Must include license',
  licenseFetchDetailsFailed: 'Failed fetching license details',
  imageLightboxTitle: 'Images',
  imageLightBoxProgress: ':num of :total',
  nextImage: 'Next image',
  previousImage: 'Previous image',
  screenshots: 'Screenshots',
  reloadButtonLabel: 'Reload',
  videoQuality: 'Navn på videokvalitet',
  videoQualityDescription:
    'Dette navnet hjelper brukeren med å identifisere den aktuelle videokvaliteten. F.eks. 1080p, 720p, HD or Mobile',
  videoQualityDefaultLabel: 'Kvalitet :index',
  noContentTypesAvailable: 'Ingen innhaldstyper er tilgjengeleg',
  noContentTypesAvailableDesc:
    'Nettstedet ditt har problemer med å koble til H5P.org og liste tilgjengelege innhaldstyper.',
  contentTypeCacheOutdated: 'Innhaldstype-lista er utdatert',
  contentTypeCacheOutdatedDesc:
    'Nettstedet ditt har problemer med å koble til H5P.org for å se etter innhaldstype-oppdateringer. Det kan hende at du ikke får oppdatert eller installert nye innhaldstyper.',
  tryAgain: 'Prøv på nytt',
  getHelp: 'Get help',
  untitled: 'Untitled :libraryTitle',
  title: 'Title',
  metadata: 'Metadata',
  addTitle: 'Add title',
  usedForSearchingReportsAndCopyrightInformation: 'Used for searching, reports and copyright information',
  metadataSharingAndLicensingInfo: 'Metadata (sharing and licensing info)',
  fillInTheFieldsBelow: 'Fill in the fields below',
  saveMetadata: 'Save metadata',
  addAuthor: 'Save author',
  confirmRemoveAuthor: 'Are you sure you would like to remove this author?',
  addNewChange: 'Add new change',
  confirmDeleteChangeLog: 'Are you sure you would like to delete this change log entry?',
  changelogDescription:
    'Some licenses require that changes made to the original work, or derivatives are logged and displayed. You may log your changes here for licensing reasons or just to allow yourself and others to keep track of the changes made to this content.',
  logThisChange: 'Log this change',
  newChangeHasBeenLogged: 'New change has been logged',
  loggedChanges: 'Logged changes',
  noChangesHaveBeenLogged: 'No changes have been logged',
  errorHeader: 'An error occured',
  errorCalculatingMaxScore:
    'Could not calculate the max score for this content. The max score is assumed to be 0. Contact your administrator if this isn’t correct.',
  maxScoreSemanticsMissing: 'Could not find the expected semantics in the content.',
  copyButton: 'Kopier',
  copiedButton: 'Kopiert',
  pasteButton: 'Lim inn',
  pasteAndReplaceButton: 'Paste & Replace',
  pasteContent: 'Replace Content',
  confirmPasteContent:
    'By doing this you will replace the current content with the content from your clipboard. The current content will be lost. Are you sure you wish to continue?',
  confirmPasteButtonText: 'Replace content',
  copyToClipboard: 'Copy H5P content to the clipboard',
  copiedToClipboard: 'Content is copied to the clipboard',
  pasteFromClipboard: 'Paste H5P content from the clipboard',
  pasteAndReplaceFromClipboard: 'Replace existing content with H5P Content from the clipboard',
  pasteNoContent: 'No H5P content on the clipboard',
  pasteError: 'Cannot paste from clipboard',
  pasteContentNotSupported: 'The content in the H5P clipboard is not supported in this context',
  pasteContentRestricted: 'The content in the clipboard has been restricted on this site',
  pasteTooOld:
    'The content in the H5P clipboard is of a lower version (:clip) than what is supported in this context (:local), if possible try to have the content you want to paste upgraded, copy it again and try pasting it here.',
  pasteTooNew:
    'The content in the H5P clipboard is of a higher version (:clip) than what is supported in this context (:local), if possible try to have this content upgraded first, and then try pasting the content here again.',
  ok: 'OK',
  avTablistLabel: 'Insert using',
  tabTitleBasicFileUpload: 'File Upload',
  tabTitleInputLinkURL: 'Link/URL',
  errorTooHighVersion: 'Parameters contain %used while only %supported or earlier are supported.',
  errorNotSupported: 'Parameters contain %used which is not supported.',
  errorParamsBroken: 'Parameters are broken.',
  libraryMissing: 'Missing required library %lib.',
  scriptMissing: 'Could not load upgrades script for %lib.',
  language: 'Language',
  noLanguagesSupported: 'No languages supported',
  changeLanguage: 'Change language to :language?',
  thisWillPotentially:
    "This will potentially reset all the text and translations. You can't undo this. The content itself will not be changed. Do you want to proceed?",
  notAllTextsChanged: 'Not all texts were changed, there is only partial coverage for :language.',
  contributeTranslations:
    'If you want to complete the translation for :language you can learn about <a href=":url" target="_new">contributing translations to H5P</a>',
  unknownLibrary: "Dessverre var ikke den valgte '%lib'-innholdstypen installert på dette systemet.",
  proceedButtonLabel: 'Proceed to save',
  enterFullscreenButtonLabel: 'Enter fullscreen',
  exitFullscreenButtonLabel: 'Exit fullscreen',
  a11yTitleShowLabel: 'Show label for AT',
  a11yTitleHideLabel: 'Hide label for AT',
  reuseSuccess: ':title was successfully imported from the H5P Hub.',
  noContentHeader: 'No Fitting Content?',
  noContentSuggestion: 'Create one yourself!',
  tutorials: 'Tutorials',
  contentSectionAll: 'All shared content',
  popularContent: 'Popular Content',
  allPopular: 'All Popular',
  newOnTheHub: 'New on the Hub',
  allNew: 'All New',
  filterBy: 'Filter by',
  filter: 'Filter',
  filters: {
    level: {
      dropdownLabel: 'Level',
      dialogHeader: 'Select level of education',
      dialogButtonLabel: 'Filter level of education',
    },
    language: {
      dropdownLabel: 'Language',
      dialogHeader: 'Select language(s)',
      dialogButtonLabel: 'Filter languages',
      searchPlaceholder: 'Type to search for languages',
    },
    reviewed: {
      dropdownLabel: 'Reviewed',
      dialogHeader: 'Reviewed Content',
      dialogButtonLabel: 'Filter',
      optionLabel: 'Show only reviewed content',
    },
    contentTypes: {
      dropdownLabel: 'Content types',
      dialogHeader: 'Select Content type(s)',
      dialogButtonLabel: 'Filter Content Types',
      searchPlaceholder: 'Type to search for content types',
    },
    disciplines: {
      dropdownLabel: 'Discipline',
      dialogHeader: 'Select Your Discipline',
      dialogButtonLabel: 'Filter Disciplines',
      searchPlaceholder: 'Type to search for disciplines',
    },
    licenses: {
      dropdownLabel: 'License',
      dialogHeader: 'Select preferred rights of use',
      dialogButtonLabel: 'Filter licenses',
      options: { modified: 'Can be modified', commercial: 'Allows commercial use' },
    },
  },
  clearFilters: 'Clear all filters',
  contentSearchFieldPlaceholder: 'Search for Content',
  loadingContentTitle: 'We are loading content for you...',
  loadingContentSubtitle: 'Please wait',
  by: 'By',
  dropdownButton: 'Open dropdown',
  paginationNavigation: 'Pagination navigation',
  page: 'Page',
  currentPage: 'Current page',
  nextPage: 'Go to next page',
  previousPage: 'Go to previous page',
  contentPreviewButtonLabel: 'Preview',
  contentDownloadButtonLabel: 'Get Content',
  reuseContentTabLabel: 'Get Shared Content',
  contentPublisherPanelHeader: 'Publisher Info',
  noContentFoundDesc: 'There is no content that matches your search criteria.',
  h5pType: 'H5P Type',
  level: 'Level',
  size: 'Size',
  failedFetchingData: 'Failed fetching data',
  filterErrorMessage: 'Something went wrong. Please reload the page.',
  in: 'in',
  navigateToParent: 'Navigate to parent',
};
