{{!
    This file is part of Moodle - http://moodle.org/
    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_data/save_as_preset

    Trigger button for the save as preset modal form.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * All data attributes are required

    Context variables required for this template:
    * d - The database id.

    Example context (json):
    {
        "d": 1
    }
}}
<button type="button" class="btn btn-secondary" data-action="saveaspreset" data-dataid={{d}}>
   {{#str}}saveaspreset, mod_data{{/str}}
</button>
{{#js}}
    require(['mod_data/saveaspreset'], function(saveAsPreset) {
        saveAsPreset.init();
    });
{{/js}}
