{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_courseformat/local/content/addresourceoractivitybutton

    Displays a add resource or activity button.

    Example context (json):
    {
        "sectionnum": 0,
        "sectionid": 0,
        "modid": 1,
        "activityname": "Activity example",
        "sectionreturnnum": 0,
        "sectionreturnid": 0
    }
}}
<button class="section-modchooser section-modchooser-link {{!
    }}{{^hasactionlinks}}btn add-content d-flex justify-content-center align-items-center p-1 icon-no-margin{{/hasactionlinks}}{{!
    }}{{#hasactionlinks}}dropdown-item{{/hasactionlinks}}"
    data-action="open-chooser"
    data-sectionnum="{{sectionnum}}"
    {{!
        data-sectionid was wrongly used for the section number in Moodle 4.4,
        so we use data-section-id instead for now.
    }}
    data-section-id="{{sectionid}}"
    {{#sectionreturnnum}}data-sectionreturnnum="{{.}}"{{/sectionreturnnum}}
    {{#sectionreturnid}}data-sectionreturnid="{{.}}"{{/sectionreturnid}}
    {{#modid}}
        data-beforemod="{{modid}}"
        aria-label="{{#str}}insertresourceoractivitybefore, core, { "activityname": {{#quote}} {{activityname}} {{/quote}} } {{/str}}"
        tabindex="0"
        title="{{#str}}addresourceoractivity, core{{/str}}"
    {{/modid}}
>
    {{^hasactionlinks}}
        {{#modid}}
            {{#pix}} t/add, core {{/pix}}
        {{/modid}}
        {{^modid}}
            <div class="px-1">
                {{#pix}} t/add, core {{/pix}}
                <span class="activity-add-text pe-1">{{#str}}addresourceoractivity, core{{/str}}</span>
            </div>
        {{/modid}}
    {{/hasactionlinks}}
    {{#hasactionlinks}}
        {{#pix}} i/activities, core {{/pix}}{{#str}}activityorresource, core{{/str}}
    {{/hasactionlinks}}
</button>
