videojs.addLanguage('pl', {
  "Audio Player": "Odtwarzacz audio",
  "Video Player": "Odtwarzacz wideo",
  "Play": "Odtwórz",
  "Pause": "Wstrzymaj",
  "Replay": "Odtwórz ponownie",
  "Current Time": "Aktualny czas",
  "Duration": "Czas trwania",
  "Remaining Time": "Pozostały czas",
  "Stream Type": "Typ strumienia",
  "LIVE": "NA ŻYWO",
  "Seek to live, currently behind live": "Przejdź do transmisji na żywo, obecnie trwa odtwarzanie z archiwum",
  "Seek to live, currently playing live": "Przejdż do transmisji na żywo, obecnie trwa odtwarzanie na żywo",
  "Loaded": "Załadowany",
  "Progress": "Status",
  "Progress Bar": "Pasek postępu",
  "progress bar timing: currentTime={1} duration={2}": "{1} z {2}",
  "Fullscreen": "Pełny ekran",
  "Exit Fullscreen": "Pełny ekran niedostępny",
  "Mute": "Wycisz",
  "Unmute": "Wyłącz wyciszenie",
  "Playback Rate": "Prędkość odtwarzania",
  "Subtitles": "Napisy",
  "subtitles off": "Napisy wyłączone",
  "Captions": "Transkrypcja",
  "captions off": "Transkrypcja wyłączona",
  "Chapters": "Rozdziały",
  "Descriptions": "Opisy",
  "descriptions off": "Opisy wyłączone",
  "Audio Track": "Ścieżka audio",
  "Volume Level": "Poziom głośności",
  "You aborted the media playback": "Odtwarzanie zostało przerwane",
  "A network error caused the media download to fail part-way.": "Błąd sieci spowodował częściowe niepowodzenie pobierania materiału wideo.",
  "The media could not be loaded, either because the server or network failed or because the format is not supported.": "Materiał wideo nie może zostać załadowany, ponieważ wystąpił problem z serwerem lub siecią albo format materiału wideo nie jest obsługiwany",
  "The media playback was aborted due to a corruption problem or because the media used features your browser did not support.": "Odtwarzanie materiału wideo zostało przerwane z powodu uszkodzonego pliku wideo lub z powodu użycia funkcji multimediów nieobsługiwanych przez Twoją przeglądarkę.",
  "No compatible source was found for this media.": "Nie znaleziono kompatybilnego źródła dla tego typu materiału wideo.",
  "The media is encrypted and we do not have the keys to decrypt it.": "Materiał jest zaszyfrowany, a nie mamy kluczy do jego odszyfrowania.",
  "Play Video": "Odtwórz wideo",
  "Close": "Zamknij",
  "Close Modal Dialog": "Zamknij okno modalne",
  "Modal Window": "Okno modalne",
  "This is a modal window": "To jest okno modalne",
  "This modal can be closed by pressing the Escape key or activating the close button.": "To okno modalne możesz zamknąć, naciskając klawisz Escape albo wybierając przycisk Zamknij.",
  ", opens captions settings dialog": ", otwiera okno dialogowe ustawień transkrypcji",
  ", opens subtitles settings dialog": ", otwiera okno dialogowe napisów",
  ", opens descriptions settings dialog": ", otwiera okno dialogowe opisów",
  ", selected": ", zaznaczone",
  "captions settings": "ustawienia transkrypcji",
  "subtitles settings": "ustawienia napisów",
  "descriptions settings": "ustawienia opisów",
  "Text": "Tekst",
  "White": "Biały",
  "Black": "Czarny",
  "Red": "Czerwony",
  "Green": "Zielony",
  "Blue": "Niebieski",
  "Yellow": "Żółty",
  "Magenta": "Karmazynowy",
  "Cyan": "Cyjanowy",
  "Background": "Tło",
  "Window": "Okno",
  "Transparent": "Przezroczysty",
  "Semi-Transparent": "Półprzezroczysty",
  "Opaque": "Nieprzezroczysty",
  "Font Size": "Rozmiar czcionki",
  "Text Edge Style": "Styl krawędzi tekstu",
  "None": "Brak",
  "Raised": "Podniesiony",
  "Depressed": "Obniżony",
  "Uniform": "Równomierny",
  "Dropshadow": "Cień",
  "Font Family": "Krój czcionki",
  "Proportional Sans-Serif": "Bezszeryfowa, proporcjonalna",
  "Monospace Sans-Serif": "Bezszeryfowa, stała odległość znaków",
  "Proportional Serif": "Szeryfowa, proporcjonalna",
  "Monospace Serif": "Szeryfowa, stała odległość znaków",
  "Casual": "Casual",
  "Script": "Script",
  "Small Caps": "Kapitaliki",
  "Reset": "Ustawienia domyślne",
  "restore all settings to the default values": "zresetuj wszystkie ustawienia do domyślnych wartości",
  "Done": "Gotowe",
  "Caption Settings Dialog": "Okno dialogowe ustawień transkrypcji",
  "Beginning of dialog window. Escape will cancel and close the window.": "Początek okna dialogowego. Klawisz Escape anuluje i zamyka okno.",
  "End of dialog window.": "Koniec okna dialogowego.",
  "{1} is loading.": "Wczytywanie pliku {1}.",
  "Exit Picture-in-Picture": "Wyjdź z trybu obraz w obrazie",
  "Picture-in-Picture": "Obraz w obrazie",
  "No content": "Brak zawartości"
});