{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_bigbluebuttonbn/instance_updated_message.mustache

    This template renders a message used to inform users that an activity has been created or updated.

    Example context (json):
    {
        "link": "https://example.com/mod/bigbluebuttonbn/view.php?id=12345",
        "is_create": 1,
        "is_update": 0,
        "name": "Daily standup",
        "description": "A place to discuss daily progress, blockers, and the weather.",
        "openingtime": 1234556623,
        "closingtime": 1234559623
    }
}}

{{#is_create}}
<p>{{#str}} notification_instance_created_intro, mod_bigbluebuttonbn, {
    "name": {{#quote}}{{name}}{{/quote}},
    "link": {{#quote}}{{{link}}}{{/quote}}
} {{/str}}
{{/is_create}}

{{#is_update}}
<p>{{#str}} notification_instance_updated_intro, mod_bigbluebuttonbn, {
    "name": {{#quote}}{{name}}{{/quote}},
    "link": {{#quote}}{{link}}{{/quote}}
} {{/str}}
{{/is_update}}
<table style="margin: 5px 0 0 20px;border: 0px">
    <tbody>
    <tr>
        <th>{{#str}}notification_instance_name, mod_bigbluebuttonbn{{/str}}</th>
        <td>name</td>
    </tr>
    <tr>
        <th>{{#str}}notification_instance_description, mod_bigbluebuttonbn{{/str}}</th>
        <td>{{{description}}}</td>
    </tr>
{{#openingtime}}
    <tr>
        <th>{{#str}}notification_instance_start_date, mod_bigbluebuttonbn{{/str}}</th>
        <td>{{#userdate}} {{openingtime}} , {{#str}}strftimedatetime, langconfig{{/str}} {{/userdate}} </td>
    </tr>
{{/openingtime}}
{{#closingtime}}
    <tr>
        <th>{{#str}}notification_instance_end_date, mod_bigbluebuttonbn{{/str}}</th>
        <td>{{#userdate}} {{closingtime}} , {{#str}}strftimedatetime, langconfig{{/str}} {{/userdate}} </td>
    </tr>
{{/closingtime}}
    </tbody>
</table>
