H5PEditor.language.core = {
  missingTranslation: '[Traduzione mancante :key]',
  loading: 'Caricamento, si prega di attendere...',
  selectLibrary: 'Seleziona la libreria che userai per i tuoi contenuti.',
  unknownFieldPath: 'Impossibibile trovare :path.',
  notImageField: ":path non è un'immagine.",
  notImageOrDimensionsField: ":path non è un'immagine o dimensioni campo.",
  requiredProperty: ':property è richiesta e deve avere un valore.',
  onlyNumbers: 'Il valore di :property può contenere solo numeri.',
  illegalDecimalNumber: ':property can only contain numbers with max :decimals decimals.',
  exceedsMax: 'Il valore di :property eccede il massimo di :max.',
  listExceedsMax: 'The list exceeds the maximum of :max items.',
  belowMin: 'Il valore di :property eccede il minimo di :min.',
  listBelowMin: 'The list needs at least :min items for the content to function properly.',
  outOfStep: 'Il valore di :property può essere modificato solo negli step di :step.',
  add: 'Aggiungi',
  addFile: 'Aggiungi file',
  removeFile: 'Rimuovi file',
  confirmRemoval: 'Sei sicuro di voler rimuovere questo :type?',
  removeImage: 'Rimuovi immagine',
  confirmImageRemoval: "Questo rimuoverà l'immagine . Sei sicuro di voler procedere ?",
  changeFile: 'Cambia file',
  changeLibrary: 'Cambiare tipo di contenuto',
  semanticsError: 'Errore semantico: :error',
  missingProperty: 'Nel campo :index manca la sua proprietà :property.',
  expandCollapse: 'Espandi/Collassa',
  addEntity: 'Aggiungi :entity',
  tooLong: 'Il valore del campo è troppo lungo, dovrebbe contenere al massimo :max lettere.',
  invalidFormat: 'Il valore del campo contiene un formato non valido o caratteri non consentiti.',
  confirmChangeLibrary: 'Sei sicuro di voler cambiare libreria?',
  commonFields: 'Impostazioni e testi',
  commonFieldsDescription: 'Da qui puoi modificare le impostazioni o tradurre testi utilizzati in questo contenuto.',
  uploading: 'Caricamento, si prega di attendere...',
  noFollow: 'Impossibile seguire il campo :path.',
  editCopyright: 'Modifica copyright',
  close: 'Chiudi',
  tutorial: 'Tutorial',
  editMode: 'Modalità modifica',
  listLabel: 'Lista',
  uploadError: 'Errore nel caricamento del File',
  fileToLarge: 'Il file che stai cercando di caricare potrebbe essere troppo pesante.',
  unknownFileUploadError: 'Errore di caricamento di file sconosciuto',
  noSemantics: 'Errore, impossibile caricare il tipo del contenuto del form.',
  editImage: 'Modifica immagine',
  saveLabel: 'Salva',
  cancelLabel: 'Cancella',
  resetToOriginalLabel: 'Ripristina all\'originale',
  loadingImageEditor: 'Caricamento dell\'immagine dell\'editor, attendere per favore...',
  selectFiletoUpload: 'Selezionare il file da caricare',
  or: 'oppure',
  enterAudioUrl: 'Inserire URL della sorgente audio',
  enterVideoUrl: 'Inserire l\'URL del video',
  enterAudioTitle: 'Incolla il link o l\'URL di un\'altro sorgente audio',
  enterVideoTitle: 'Incolla il link YouTube link o un altro URL sorgente video',
  uploadAudioTitle: 'Carica file audio',
  uploadVideoTitle: 'Carica file video',
  addVideoDescription: 'H5P supporta tutte le sorgenti video esterne formattate come mp4, webm o ogv, come Vimeo Pro, e supporta i collegamenti YouTube e Panopto.',
  insert: 'Inserisci',
  cancel: 'Cancella',
  height: 'altezza',
  width: 'larghezza',
  textField: 'campo di testo',
  numberField: 'campo numerico',
  orderItemUp: 'Ordina l\'elemento in alto',
  orderItemDown: 'Ordina l\'elemento in basso',
  expandAllContent: 'Expand all content',
  collapseAllContent: 'Collapse all content',
  removeItem: 'Rimuovi elemento',
  hubPanelLabel: 'Seleziona tipo di contenuto',
  importantInstructions: 'Istruzioni importanti',
  showImportantInstructions: 'Mostra istruzioni',
  hideImportantInstructions: 'Nascondi importanti istruzioni',
  hide: 'Nascondi',
  example: 'Esempio',
  createContentTabLabel: 'Crea Contenuto',
  uploadTabLabel: 'Carica',
  uploadPlaceholder: 'Nessun file scelto',
  uploadInstructionsTitle: 'Scarica un file H5P.',
  uploadInstructionsContent:
    'Potresti iniziare con un esempio da <a href="https://h5p.org/content-types-and-applications" target="blank">H5P.org</a>.',
  uploadFileButtonLabel: 'Scarica un file',
  uploadFileButtonChangeLabel: 'Cambia file',
  uploadingThrobber: 'In fase di caricamento...',
  uploadSuccess: ':title è stato caricato con successo!',
  unableToInterpretError: 'Impossibile interpretare la risposta.',
  unableToInterpretSolution: 'Si prega di controllare il log degli errori.',
  h5pFileWrongExtensionTitle: 'Il file selezionato non può essere caricato',
  h5pFileWrongExtensionContent: 'Sono permessi solo file con estensione .h5p',
  h5pFileValidationFailedTitle: 'Non può validare il file .h5p.',
  h5pFileValidationFailedContent:'Assicurati che l\'H5P caricato contenga contenuti H5P validi. I file H5P contenenti solo librerie devono essere caricati tramite la pagina Librerie H5P.',
  h5pFileUploadServerErrorTitle: 'Il file H5P non può essere caricato',
  h5pFileUploadServerErrorContent: 'Si è verificato un errore inatteso. Controllare il log degli errori sul tuo server per ulteriori dettagli.',
  contentTypeSectionAll: 'Tutti i tipi di contenuto',
  searchResults: 'Cerca Risultati',
  contentTypeSearchFieldPlaceholder: 'Cerca Tipi di Contenuto',
  contentTypeInstallButtonLabel: 'Installa',
  contentTypeInstallingButtonLabel: 'Installazione in corso',
  contentTypeUseButtonLabel: 'Usa',
  contentTypeDetailButtonLabel: 'Dettagli',
  contentTypeUpdateButtonLabel: 'Carica',
  contentTypeUpdatingButtonLabel: 'In caricamento',
  contentTypeGetButtonLabel: 'Ottieni',
  contentTypeBackButtonLabel: 'Indietro',
  contentTypeIconAltText: 'Icona',
  contentTypeInstallSuccess: ':contentType installato con successo!',
  contentTypeUpdateSuccess: ':contentType caricato con successo!',
  contentTypeInstallError: ':contentType non può essere installato. Contattare il tuo amministratore.',
  contentTypeLicensePanelTitle: 'Licenza',
  contentTypeDemoButtonLabel: 'Contenuto Demo',
  numResults: ':num risultati',
  show: 'Mostra',
  recentlyUsedFirst: 'Prima Utilizzati di recente',
  popularFirst: 'Prima i più popolari',
  newestFirst: 'Prima i più recenti',
  aToZ: 'Dalla A alla Z',
  noResultsFound: 'Nessun risultato trovato',
  noResultsFoundDesc: 'Non ci sono tipi di contenuto che soddisfano i criteri di ricerca.',
  readMore: 'Leggi di più',
  readLess: 'Leggi di meno',
  contentTypeOwner: 'Creato da :owner',
  contentTypeUnsupportedApiVersionTitle: 'Questo contenuto richiede una versione principale più recente',
  contentTypeUnsupportedApiVersionContent:
    'Contattare il tuo amministratore di sistema per fornirti i necessari aggiornamenti',
  contentTypeUpdateAvailable: 'Aggiornamento disponibile',
  contentTypeRestricted: 'Tipo di contenuto limitato',
  contentTypeRestrictedDesc: 'L\'uso di questo contenuto è stato limitato da un amministratore.',
  contentTypeNotInstalled: 'Tipo di contenuto non installato',
  contentTypeNotInstalledDesc: 'Non hai i permessi per installare tipi di contenuto.',
  theContentType: 'il tipo di contenuto',
  currentMenuSelected: 'selezione attuale',
  errorCommunicatingHubTitle: 'Impossibile comunicare con l\'hub.',
  errorCommunicatingHubContent: 'Si è verificato un errore. Per favore riprovare.',
  warningNoContentTypesInstalled: "Non hai alcun tipo di contenuto installato.",
  warningChangeBrowsingToSeeResults: 'Clicca <em>Tutto</em> per ottenere una lista di tutti i tipi di contenuto installati.',
  warningUpdateAvailableTitle: 'Una nuova versione di :contentType è disponibile.',
  warningUpdateAvailableBody: 'Aggiorna all\'ultima versione per migliorare l\'esperienza.',
  licenseDescription:
    'Alcune delle caratteristiche di questa licenza sono indicate di seguito. Fare clic sull\'icona delle informazioni in alto per leggere il testo della licenza originale.',
  licenseModalTitle: 'Dettagli sulla Licenza',
  licenseModalSubtitle: 'Seleziona la licenza per vedere le informazione riguardo l\'uso appropriato',
  licenseUnspecified: 'Non specificata',
  licenseCanUseCommercially: 'Utilizzo commerciale concesso',
  licenseCanModify: 'Può modificare',
  licenseCanDistribute: 'Può distribuire',
  licenseCanSublicense: 'Permette sublicenze',
  licenseCanHoldLiable: 'Può ritenersi responsabile',
  licenseCannotHoldLiable: 'Non può ritenersi responsabile',
  licenseMustIncludeCopyright: 'Deve includere copyright',
  licenseMustIncludeLicense: 'Deve includere la licenza',
  licenseFetchDetailsFailed: 'Impossibile recuperare i dettagli della licenza',
  imageLightboxTitle: 'Immagini',
  imageLightBoxProgress: ':num di :total',
  nextImage: 'Prossima immagine',
  previousImage: 'Immagine precedente',
  screenshots: 'Screenshots',
  reloadButtonLabel: 'Ricarica',
  videoQuality: 'Etichetta della qualità video',
  videoQualityDescription:
    'Questa etichetta aiuta l\'utente a identificare la qualità attuale del video. Per esempio 1080p, 720p, HD o Mobile',
  videoQualityDefaultLabel: 'Qualità :index',
  noContentTypesAvailable: 'Nessun tipo di contenuto è disponibile',
  noContentTypesAvailableDesc:
    'Il tuo sito ha difficoltà a connettersi a H5P.org e ad elencare i tipi di contenuto disponibili.',
  contentTypeCacheOutdated: 'Elenco dei tipi di contenuto obsoleto',
  contentTypeCacheOutdatedDesc:
    'Il tuo sito ha difficoltà a connettersi a H5P.org per controllare gli aggiornamenti del tipo di contenuto. Potrebbe non essere possibile aggiornare o installare nuovi tipi di contenuto.',
  tryAgain: 'Prova di nuovo',
  getHelp: 'Chiedi aiuto',
  untitled: 'Senza titolo :libraryTitle',
  title: 'Titolo',
  metadata: 'Metadata',
  addTitle: 'Aggiungi titolo',
  usedForSearchingReportsAndCopyrightInformation: 'Usato per la ricerca, i report e le informazioni sui copyright',
  metadataSharingAndLicensingInfo: 'Metadata (informazioni sulla condivisione e sulle licenze)',
  fillInTheFieldsBelow: 'Completa i campi sottostanti',
  saveMetadata: 'Salva metadata',
  addAuthor: 'Salva autore',
  confirmRemoveAuthor: 'Sei sicuro di voler rimuovere questo autore?',
  addNewChange: 'Aggiungi nuova modifica',
  confirmDeleteChangeLog: 'Sei sicuro di voler eliminare questa voce del registro delle modifiche?',
  changelogDescription:
    'Alcune licenze richiedono che le modifiche apportate all\'opera originale o ai derivati ​​vengano registrate e visualizzate. Puoi registrare le tue modifiche qui per motivi di licenza o semplicemente per consentire a te stesso e ad altri di tenere traccia delle modifiche apportate a questo contenuto.',
  logThisChange: 'Registra questa modifica',
  newChangeHasBeenLogged: 'La nuova modifica è stata registrata',
  loggedChanges: 'Modifiche registrate',
  noChangesHaveBeenLogged: 'Nessun cambiamento è stato registrato',
  errorHeader: 'Si è verificato un errore',
  errorCalculatingMaxScore:
    'Impossibile calcolare il punteggio massimo per questo contenuto. Si presume che il punteggio massimo sia 0. Contatta l\'amministratore se non è corretto.',
  maxScoreSemanticsMissing: 'Impossibile trovare la semantica prevista nel contenuto.',
  copyButton: 'Copia',
  copiedButton: 'Copiato',
  pasteButton: 'Incolla',
  pasteAndReplaceButton: 'Incolla e Sostituisci',
  pasteContent: 'Sostituisci il contenuto',
  confirmPasteContent:
    'In questo modo sostituirai il contenuto corrente con il contenuto dei tuoi appunti. Il contenuto corrente andrà perso. Sei sicuro di voler continuare?',
  confirmPasteButtonText: 'Sostituisci contenuto',
  copyToClipboard: 'Copia contenuto H5P negli appunti',
  copiedToClipboard: 'Il contenuto è stato copiato negli appunti',
  pasteFromClipboard: 'Incolla contenuto H5P dagli appunti',
  pasteAndReplaceFromClipboard: 'Sostituisci contenuto esistente con contenuto H5P dagli appunti',
  pasteNoContent: 'Nessun contenuto H5P negli appunti',
  pasteError: 'Impossibile incollare dagli appunti',
  pasteContentNotSupported: 'Il contenuto negli appunti H5P non è supportato in questo contesto',
  pasteContentRestricted: 'Il contenuto negli appunti è stato limitato su questo sito',
  pasteTooOld:
    'Il contenuto negli appunti H5P è di una versione inferiore (:clip) rispetto a quella supportata in questo contesto (:local), se possibile provare ad aggiornare il contenuto che si desidera incollare, copiarlo di nuovo e provare a incollarlo qui.',
  pasteTooNew:
    'Il contenuto negli appunti H5P è di una versione superiore (:clip) rispetto a quella supportata in questo contesto (:local), se possibile, prova prima ad aggiornare questo contenuto, quindi prova a incollare nuovamente il contenuto qui.',
  ok: 'OK',
  avTablistLabel: 'Inserisci utilizzando',
  tabTitleBasicFileUpload: 'Caricamento file',
  tabTitleInputLinkURL: 'Indirizzo/URL',
  errorTooHighVersion: 'I parametri contengono %used mentre sono supportati solo %supported o versioni precedenti.',
  errorNotSupported: 'I parametri contengono %used che non è supportato.',
  errorParamsBroken: 'I parametri sono corrotti.',
  libraryMissing: 'Libreria richiesta mancante %lib.',
  scriptMissing: 'Impossibile caricare lo script degli aggiornamenti per %lib.',
  language: 'Linguaggi',
  noLanguagesSupported: 'Nessun linguaggio supportato',
  changeLanguage: 'Cambiare lingua in :language?',
  thisWillPotentially:
    "Questo potenzialmente ripristinerà tutto il testo e le traduzioni. Non puoi annullare questa operazione. Il contenuto stesso non verrà modificato. Vuoi procedere?",
  notAllTextsChanged: 'Non tutti i testi sono stati modificati, c\'è solo una copertura parziale per :language.',
  contributeTranslations: 'Se desideri completare la traduzione per :language, puoi ottenere informazioni su come <a href=":url" target="_new">contribuire alle traduzioni per H5P</a>',
  unknownLibrary: "Purtroppo, il tipo di contenuto selezionato '%lib' non è installato nel tuo sistema.",
  proceedButtonLabel: 'Procedi al salvataggio',
  enterFullscreenButtonLabel: 'Entra in modalità schermo intero',
  exitFullscreenButtonLabel: 'Esci dalla modalità schermo intero',
  a11yTitleShowLabel: 'Mostra l\'etichetta per AT',
  a11yTitleHideLabel: 'Nascondi l\'etichetta per AT',
  reuseSuccess: ':title è stato importato con successo dall\'Hub H5P.',
  noContentHeader: 'nessun contenuto adatto?',
  noContentSuggestion: 'Creane uno tu stesso!',
  tutorials: 'I Tutorial',
  contentSectionAll: 'Tutti i contenuti condivisi',
  popularContent: 'Contenuti Popolari',
  allPopular: 'Tutti i popolari',
  newOnTheHub: 'Nuovi nell\'Hub',
  allNew: 'Tutti i nuovi',
  filterBy: 'Filtra per',
  filter: 'Filtra',
  filters: {
    level: {
      dropdownLabel: 'Livello',
      dialogHeader: 'Seleziona il livello di istruzione',
      dialogButtonLabel: 'Filtra il livello di istruzione',
    },
    language: {
      dropdownLabel: 'Lingua',
      dialogHeader: 'Seleziona una o più lingue',
      dialogButtonLabel: 'Filtra lingue',
      searchPlaceholder: 'Digita per cercare le lingue',
    },
    reviewed: {
      dropdownLabel: 'Revisionato',
      dialogHeader: 'Contenuto Revisionato',
      dialogButtonLabel: 'Filtro',
      optionLabel: 'Mostra solo contenuti revisionati',
    },
    contentTypes: {
      dropdownLabel: 'Tipi di contenuto',
      dialogHeader: 'Seleziona uno o più tipi di Contenuto',
      dialogButtonLabel: 'Filtra i Tipi di Contenuto',
      searchPlaceholder: 'Digita per cercare i tipi di contenuto',
    },
    disciplines: {
      dropdownLabel: 'Disciplina',
      dialogHeader: 'Seleziona la tua disciplina',
      dialogButtonLabel: 'Filtra le tue discipline',
      searchPlaceholder: 'Digita per cercare le discipline',
    },
    licenses: {
      dropdownLabel: 'Licenza',
      dialogHeader: 'Seleziona i diritti d\'uso preferiti',
      dialogButtonLabel: 'Filtra le licenze',
      options: { modified: 'Può modificare', commercial: 'Permette l\'uso commerciale' },
    },
  },
  clearFilters: 'Cancella tutti i filtri',
  contentSearchFieldPlaceholder: 'Ricerca per Contenuto',
  loadingContentTitle: 'Stiamo caricando il contenuto per te...',
  loadingContentSubtitle: 'Attendere per favore',
  by: 'Fatto da',
  dropdownButton: 'Apri menù a discesa',
  paginationNavigation: 'Navigazione dell\'impaginazione',
  page: 'Pagina',
  currentPage: 'Pagina corrente',
  nextPage: 'Vai alla pagina seguente',
  previousPage: 'Vai alla pagina precedente',
  contentPreviewButtonLabel: 'Anteprima',
  contentDownloadButtonLabel: 'Ottieni Contenuto',
  reuseContentTabLabel: 'Ottieni contenuto condiviso',
  contentPublisherPanelHeader: 'Informazioni sull\'autore',
  noContentFoundDesc: 'Non ci sono contenuti corrispondenti ai criteri di ricerca.',
  h5pType: 'Tipo di H5P',
  level: 'Livello',
  size: 'Dimensione',
  failedFetchingData: 'Recupero dei dati non riuscito',
  filterErrorMessage: 'Qualcosa è andato storto. Si prega di ricaricare la pagina.',
  in: 'in',
  navigateToParent: 'Vai al parent',
};
