{{!
    This file is part of Moodle - http://moodle.org/
    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_group/comboboxsearch/searchbody

    Wrapping template for search input.

    Context variables required for this template:
    * courseid - The id of the course to search within.
    * groupid - The id of the group to search within.
    * currentvalue - The prefill value for the search input if provided

    Example context (json):
    {
        "courseid": 2,
        "groupid": 25,
        "currentvalue": "bar"
    }
}}
<div class="flex-column h-100 w-100">
    <span class="d-none" data-region="courseid" data-courseid="{{courseid}}"></span>
    <span class="d-none" data-region="groupid" data-groupid="{{groupid}}"></span>
    <span class="d-none" data-region="instance" data-instance="{{instance}}"></span>

    {{< core/search_input_auto }}
        {{$label}}{{#str}}
            searchgroups, core
        {{/str}}{{/label}}
        {{$placeholder}}{{#str}}
            searchgroups, core
        {{/str}}{{/placeholder}}
        {{$value}}{{currentvalue}}{{/value}}
        {{$additionalattributes}}
            role="combobox"
            aria-expanded="true"
            aria-controls="groups-{{instance}}-result-listbox"
            aria-autocomplete="list"
            data-input-element="input-{{uniqid}}-{{instance}}"
        {{/additionalattributes}}
    {{/ core/search_input_auto }}
    <input type="hidden" name="search" id="input-{{uniqid}}-{{instance}}"/>

    <div data-region="searchplaceholder"></div>
</div>
