{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_quiz/switch_bank_header

    Template for showing the currently selected bank as a header with a switch bank button.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * none

    Context variables required for this template:
    * title A cleaned string (use clean_text()) to display.
    * body HTML content for the boday

    Example context (json):
    {
        "currentbank": "Quiz 1"
    }
}}
<div class="d-flex align-items-center">
    <h5>{{#str}}currentbank, mod_quiz, {{currentbank}} {{/str}}</h5>
    <button data-action="switch-question-bank" type="button" class="btn btn-secondary ms-auto mb-2" id="switch-question-bank">{{#str}}switchbank, core_question{{/str}}</button>
</div>
