<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'gradereport_outcomes', language 'en'
 *
 * @package   gradereport_outcomes
 * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['addoutcome'] = 'Add an outcome';
$string['courseoutcomes'] = 'Course outcomes';
$string['coursespecoutcome'] = 'Course outcomes';
$string['eventgradereportviewed'] = 'Grade outcomes report viewed';
$string['pluginname'] = 'Outcomes report';
$string['privacy:metadata'] = 'The Grade outcomes report only shows data stored in other locations.';
$string['outcomes:view'] = 'View outcomes report';
$string['usedgradeitem'] = 'Number of grade items';
