{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_courseformat/local/overview/missingoverviewnotice

    Message to display when the activity plugin overview is missing.

    Example context (json):
    {
        "name": "Course name",
        "overviewurl": "https://moodle.org",
        "notification": {
            "message": "Notification message",
            "level": "info"
        },
        "missingoverview": true
    }
}}
{{#missingoverview}}
<div class="d-flex flex-column justify-content-between mb-3">
    {{#notification}}
        {{>core/notification_info}}
    {{/notification}}
</div>
{{/missingoverview}}
