<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'gradereport_history', language 'en'
 *
 * @package    gradereport_history
 * @copyright  2013 NetSpot Pty Ltd (https://www.netspot.com.au)
 * @author     Adam Olley <adam.olley@netspot.com.au>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['allgradeitems'] = 'All grade items';
$string['allgraders'] = 'All graders';
$string['datefrom'] = 'Date from';
$string['dateto'] = 'Date to';
$string['datetime'] = 'Date and time';
$string['deleteditemid'] = 'Delete item with id {$a}';
$string['errajaxsearch'] = 'Error when searching users';
$string['eventgradereportviewed'] = 'Grade history report viewed';
$string['excluded'] = 'Excluded from calculations';
$string['exportfilename'] = 'grade_history';
$string['foundoneuser'] = '1 user found';
$string['foundnusers'] = '{$a} users found';
$string['feedbacktext'] = 'Feedback text';
$string['finishselectingusers'] = 'Finish selecting users';
$string['gradenew'] = 'Revised grade';
$string['gradeold'] = 'Original grade';
$string['grader'] = 'Grader';
$string['history:view'] = 'View grade history';
$string['historyperpage'] = 'History entries per page';
$string['historyperpage_help'] = 'This setting determines the number of history entries displayed per page in the history report.';
$string['loadmoreusers'] = 'Load more users...';
$string['pluginname'] = 'Grade history';
$string['preferences'] = 'Grade history preferences';
$string['privacy:metadata'] = 'The Grade history report only shows data stored in other locations.';
$string['revisedonly'] = 'Revised grades only';
$string['revisedonly_help'] = 'Only show grades which have been revised.

This means only entries which result in the grade changing are listed.';
$string['selectuser'] = 'Select user';
$string['selectusers'] = 'Select users';
$string['selectedusers'] = 'Selected users';
$string['source'] = 'Source';
$string['useractivitygrade'] = '{$a} grade';
$string['useractivityfeedback'] = '{$a} feedback';
