<?php

// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'gradeimport_xml', language 'en', branch 'MOODLE_20_STABLE'
 *
 * @package   gradeimport_xml
 * @copyright 1999 onwards Martin Dougiamas  {@link http://moodle.com}
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['errbadxmlformat'] = 'Error - bad XML format';
$string['errduplicategradeidnumber'] = 'Error - there are two grade items with idnumber \'{$a}\' in this course. This should be impossible.';
$string['errduplicateidnumber'] = 'Error - duplicate idnumber';
$string['errincorrectgradeidnumber'] = 'Error - idnumber \'{$a}\' from the import file does not match any grade item.';
$string['errincorrectidnumber'] = 'Error - incorrect idnumber';
$string['errincorrectuseridnumber'] = 'Error - idnumber \'{$a}\' from the import file does not match any user.';
$string['error'] = 'Errors occur';
$string['errorduringimport'] = 'An error occurred when trying to import: {$a}';
$string['fileurl'] = 'Remote file URL';
$string['fileurl_help'] = 'The remote file URL field is for fetching data from a remote server, such as a student information system.';
$string['pluginname'] = 'XML file';
$string['privacy:metadata'] = 'The import grades from XML plugin does not store any personal data.';
$string['xml:publish'] = 'Publish import grades from XML';
$string['xml:view'] = 'Import grades from XML';
$string['importxml'] = 'XML import';
$string['importxml_help'] = 'Grades can be imported via an XML file containing user ID numbers and activity ID numbers. To obtain the correct format, first export some grades to XML file then view the file.';
$string['importxml_link'] = 'grade/import/xml/index';
