{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_forum/local/grades/local/grader/module_content_placeholder

    Classes required for JS:
    * none

    Data attributes required for JS:
    *

    Context variables required for this template:
    *

    Example context (json):
    {}
}}
<div class="p-4">
    <div class="d-inline-block bg-pulse-grey mb-1" style="height: 18px; width: 200px"></div>
    <div class="d-inline-block bg-pulse-grey ms-1 mb-1" style="height: 18px; width: 100px"></div>
    <div class="bg-pulse-grey mb-4" style="height: 36px; width: 500px"></div>

    <div class="d-flex align-items-center mb-2 p-2" style="height: 63px">
        <div class="bg-pulse-grey rounded-circle me-2" style="height: 45px; width: 45px"></div>
        <div class="bg-pulse-grey" style="height: 15px; width: 150px"></div>
    </div>
    <div class="mb-2 p-2">
        <div class="d-flex mb-2">
            <div class="bg-pulse-grey rounded-circle me-2" style="height: 45px; width: 45px"></div>
            <div class="mb-3">
                <div class="bg-pulse-grey mb-1" style="height: 18px; width: 200px"></div>
                <div class="bg-pulse-grey mb-1" style="height: 18px; width: 350px"></div>
            </div>
        </div>
        <div class="d-flex w-100">
            <div class="flex-shrink-0 me-2" style="height: 45px; width: 45px"></div>
            <div class="flex-grow-1">
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 95%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 100%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 97%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 93%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 100%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 75%"></div>
            </div>
        </div>
    </div>
    <div class="bg-pulse-grey" style="height: 36px; width: 150px"></div>
    <hr class="w-75 mx-auto my-5">

    <div class="d-flex align-items-center mb-2 p-2" style="height: 63px">
        <div class="bg-pulse-grey rounded-circle me-2" style="height: 45px; width: 45px"></div>
        <div class="bg-pulse-grey" style="height: 15px; width: 150px"></div>
    </div>
    <div class="mb-2 p-2">
        <div class="d-flex mb-2">
            <div class="bg-pulse-grey rounded-circle me-2" style="height: 45px; width: 45px"></div>
            <div class="mb-3">
                <div class="bg-pulse-grey mb-1" style="height: 18px; width: 200px"></div>
                <div class="bg-pulse-grey mb-1" style="height: 18px; width: 350px"></div>
            </div>
        </div>
        <div class="d-flex w-100">
            <div class="flex-shrink-0 me-2" style="height: 45px; width: 45px"></div>
            <div class="flex-grow-1">
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 95%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 100%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 97%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 93%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 100%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 75%"></div>
            </div>
        </div>
    </div>
    <div class="bg-pulse-grey" style="height: 36px; width: 150px"></div>
    <hr class="w-75 mx-auto my-5">

    <div class="d-flex align-items-center mb-2 p-2" style="height: 63px">
        <div class="bg-pulse-grey rounded-circle me-2" style="height: 45px; width: 45px"></div>
        <div class="bg-pulse-grey" style="height: 15px; width: 150px"></div>
    </div>
    <div class="mb-2 p-2">
        <div class="d-flex mb-2">
            <div class="bg-pulse-grey rounded-circle me-2" style="height: 45px; width: 45px"></div>
            <div class="mb-3">
                <div class="bg-pulse-grey mb-1" style="height: 18px; width: 200px"></div>
                <div class="bg-pulse-grey mb-1" style="height: 18px; width: 350px"></div>
            </div>
        </div>
        <div class="d-flex w-100">
            <div class="flex-shrink-0 me-2" style="height: 45px; width: 45px"></div>
            <div class="flex-grow-1">
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 95%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 100%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 97%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 93%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 100%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 75%"></div>
            </div>
        </div>
    </div>
    <div class="bg-pulse-grey" style="height: 36px; width: 150px"></div>
    <hr class="w-75 mx-auto my-5">

    <div class="d-flex align-items-center mb-2 p-2" style="height: 63px">
        <div class="bg-pulse-grey rounded-circle me-2" style="height: 45px; width: 45px"></div>
        <div class="bg-pulse-grey" style="height: 15px; width: 150px"></div>
    </div>
    <div class="mb-2 p-2">
        <div class="d-flex mb-2">
            <div class="bg-pulse-grey rounded-circle me-2" style="height: 45px; width: 45px"></div>
            <div class="mb-3">
                <div class="bg-pulse-grey mb-1" style="height: 18px; width: 200px"></div>
                <div class="bg-pulse-grey mb-1" style="height: 18px; width: 350px"></div>
            </div>
        </div>
        <div class="d-flex w-100">
            <div class="flex-shrink-0 me-2" style="height: 45px; width: 45px"></div>
            <div class="flex-grow-1">
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 95%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 100%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 97%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 93%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 100%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 75%"></div>
            </div>
        </div>
    </div>
    <div class="bg-pulse-grey" style="height: 36px; width: 150px"></div>
    <hr class="w-75 mx-auto my-5">

    <div class="d-flex align-items-center mb-2 p-2" style="height: 63px">
        <div class="bg-pulse-grey rounded-circle me-2" style="height: 45px; width: 45px"></div>
        <div class="bg-pulse-grey" style="height: 15px; width: 150px"></div>
    </div>
    <div class="mb-2 p-2">
        <div class="d-flex mb-2">
            <div class="bg-pulse-grey rounded-circle me-2" style="height: 45px; width: 45px"></div>
            <div class="mb-3">
                <div class="bg-pulse-grey mb-1" style="height: 18px; width: 200px"></div>
                <div class="bg-pulse-grey mb-1" style="height: 18px; width: 350px"></div>
            </div>
        </div>
        <div class="d-flex w-100">
            <div class="flex-shrink-0 me-2" style="height: 45px; width: 45px"></div>
            <div class="flex-grow-1">
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 95%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 100%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 97%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 93%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 100%"></div>
                <div class="bg-pulse-grey mb-1" style="height: 15px; width: 75%"></div>
            </div>
        </div>
    </div>
    <div class="bg-pulse-grey" style="height: 36px; width: 150px"></div>
</div>
