{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_assign/list_participant_user_summary

    Shown as the options for the list of users in the user picker on the grading page.

    Classes required for JS:
    * none

    Data attributes required for JS:
    * None

    Context variables required for this template:
    * fullname
    * requiregrading
    * identity - list of only the visible identity fields
    * list of fields from mod_assign_list_participants webservice

    Example context (json):
    { "id": "4",
      "fullname": "Mr T",
      "requiregrading": "true",
      "identity": "email, phone, etc"
    }
}}
<span>{{fullname}} <small>{{identity}}</small> {{#requiregrading}}<abbr title="{{#str}}filterrequiregrading, mod_assign{{/str}}">*</abbr>{{/requiregrading}}</span>
