#page-report-participation-index .participationselectform {
    margin: 10px auto;
}

#page-report-participation-index .participationselectform label {
    margin-left: 15px;
    margin-right: 5px;
}

/* Die to css conflicts with form-inline, we have to create a specific class to fix submit button alignment on clean */
#page-report-participation-index .participationselectform input[type="submit"] {
    margin-bottom: 0;
}
