{{!
    This file is part of Moodle - http://moodle.org/
    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_question/question_version_info

    Displays a badge containing the version information of a question
    * versioninfo - The version information string, including the number and whether it is the latest verision.

    Example context (json):
    {
        "versioninfo": "Version 2 (latest)"
    }
}}

<span class="badge bg-primary text-light">{{versioninfo}}</span>
