{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_courseformat/local/content/cm

    Displays a course module instance inside a course section.

    Example context (json):
    {
        "cmname": {
            "displayvalue" : "<a class=\"aalink\" href=\"#\"><span class=\"instancename\">Activity example</span></a>"
        },
        "hasname": "true",
        "afterlink": "<span class=\"badge bg-primary\">30 unread messages</span>",
        "hasextras": true,
        "extras": ["<span class=\"badge bg-secondary\">[extras]</span>"],
        "activityinfo": {
            "hasmodavailability": true,
            "activityname": "Activity example",
            "hascompletion": true,
            "uservisible": true,
            "hasdates": true,
            "isautomatic": true,
            "istrackeduser": true,
            "activitydates": [
                {
                    "label": "Opens:",
                    "datestring": "6 April 2021, 6:46 PM"
                }
            ],
            "completiondetails": [
                {
                    "statuscomplete": 1,
                    "description": "Viewed"
                },
                {
                    "statusincomplete": 1,
                    "description": "Receive a grade"
                }
            ]
        },
        "modstealth": true,
        "indent": 1
    }
}}
{{#editing}}
    {{< core_courseformat/local/content/divider}}
        {{$content}}
            {{#activitychooserbutton}}{{> core_courseformat/local/content/activitychooserbutton}}{{/activitychooserbutton}}
        {{/content}}
    {{/ core_courseformat/local/content/divider}}
{{/editing}}
<div class="activity-item focus-control {{#modstealth}}hiddenactivity{{/modstealth}}{{!
        }}{{#modhiddenfromstudents}}hiddenactivity{{/modhiddenfromstudents}}{{!
        }}{{#modinline}}activityinline{{/modinline}}" data-activityname="{{activityname}}" data-region="activity-card">
    {{#cmbulk}}
        {{$ core_courseformat/local/content/cm/bulkselect }}
            {{> core_courseformat/local/content/cm/bulkselect }}
        {{/ core_courseformat/local/content/cm/bulkselect }}
    {{/cmbulk}}
    {{!
      Place the actual content of the activity-item in a separate template to make it easier for other formats to add
      additional content to the activity wrapper.
    }}
    {{$ core_courseformat/local/content/cm/activity }}
        {{> core_courseformat/local/content/cm/activity }}
    {{/ core_courseformat/local/content/cm/activity }}
</div>
