<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Strings for component 'contentbank', language 'en'
 *
 * @package    core_contentbank
 * @copyright  2020 Amaia Anabitarte <amaia@moodle.com>
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

$string['author'] = 'Author';
$string['contentbank'] = 'Content bank';
$string['choosecontext'] = 'Choose course or category...';
$string['contentbankcustomfields'] = 'Content bank custom fields';
$string['contentbankpreferences'] = 'Content bank preferences';
$string['contentcopied'] = 'Content copied.';
$string['contentdeleted'] = 'Content deleted.';
$string['contentname'] = 'Content name';
$string['contentnotdeleted'] = 'An error was encountered while trying to delete the content.';
$string['contentnotrenamed'] = 'An error was encountered while trying to rename the content.';
$string['contentrenamed'] = 'Content renamed.';
$string['contentsmoved'] = 'Content bank contents moved to {$a}.';
$string['contenttypenoaccess'] = 'You cannot view this {$a} instance.';
$string['contenttypenoedit'] = 'You can not edit this content';
$string['contentvisibilitychanged'] = 'Content is now {$a}.';
$string['contentvisibilitynotset'] = 'An error was encountered while trying to set the content visibility.';
$string['contentcreated'] = 'Content created.';
$string['contentupdated'] = 'Content updated.';
$string['contextnotallowed'] = 'You are not allowed to access the content bank in this context.';
$string['copycontent'] = 'Copy content';
$string['copyof'] = 'Copy of {$a}';
$string['displayunlisted'] = 'Show unlisted content';
$string['emptynamenotallowed'] = 'Empty name is not allowed';
$string['eventcontentcreated'] = 'Content created';
$string['eventcontentdeleted'] = 'Content deleted';
$string['eventcontentreplaced'] = 'Content replaced with file';
$string['eventcontentupdated'] = 'Content updated';
$string['eventcontentuploaded'] = 'Content uploaded';
$string['eventcontentviewed'] = 'Content viewed';
$string['errordeletingcontentfromcategory'] = 'Error deleting content from category {$a}.';
$string['errornofile'] = 'A compatible file is needed to create content.';
$string['exit'] = 'Exit';
$string['deletecontent'] = 'Delete content';
$string['deletecontentconfirm'] = 'Are you sure you want to delete the content <em>\'{$a->name}\'</em> and all associated files? This action cannot be undone.';
$string['deletecontentconfirmlinked'] = 'The content will only be deleted from the content bank. Any places which currently link to it will be automatically updated to use a copy of the content instead.';
$string['displaydetails'] = 'Display content bank with file details';
$string['displayicons'] = 'Display content bank with icons';
$string['file'] = 'Upload content';
$string['file_help'] = 'Files may be stored in the content bank for use in courses. Only files used by content types enabled on the site may be uploaded.';
$string['itemsfound'] = '{$a} items found';
$string['lastmodified'] = 'Last modified';
$string['more'] = 'More';
$string['name'] = 'Content';
$string['nocontentavailable'] = 'No content available';
$string['nocontenttypes'] = 'No content types available';
$string['notavailable'] = 'Sorry, this content is not available.';
$string['nopermissiontocopy'] = 'You do not have permission to copy content.';
$string['nopermissiontodelete'] = 'You do not have permission to delete content.';
$string['nopermissiontomanage'] = 'You do not have permission to manage content.';
$string['notvalidpackage'] = 'Sorry, this file is not valid.';
$string['privacy:metadata:content:contenttype'] = 'The contenttype plugin of the content in the content bank.';
$string['privacy:metadata:content:name'] = 'Name of the content in the content bank.';
$string['privacy:metadata:content:timecreated'] = 'The time when the content was created.';
$string['privacy:metadata:content:timemodified'] = 'The time when the content was modified.';
$string['privacy:metadata:content:usercreated'] = 'The user has created the content.';
$string['privacy:metadata:content:usermodified'] = 'The last user who modified the content.';
$string['privacy:metadata:contentbankcontent'] = 'Stores the content of the content bank.';
$string['privacy:metadata:userid'] = 'The ID of the user creating or modifying content bank content.';
$string['privacy:request:preference:set'] = 'The value of the setting \'{$a->name}\' was \'{$a->value}\'';
$string['rename'] = 'Rename';
$string['renamecontent'] = 'Rename content';
$string['replacecontent'] = 'Replace with file';
$string['searchcontentbankbyname'] = 'Search for content by name';
$string['size'] = 'Size';
$string['timecreated'] = 'Time created';
$string['type'] = 'Type';
$string['unsupported'] = 'This content type is not supported.';
$string['upload'] = 'Upload';
$string['uses'] = 'Places linked';
$string['visibilitychoicepublic'] = 'Public';
$string['visibilitychoiceunlisted'] = 'Unlisted';
$string['public'] = 'public';
$string['unlisted'] = 'unlisted';
$string['visibilitypref'] = 'Default content visibility';
$string['visibilitypref_help'] = 'Content you create in the content bank will use this visibility setting by default.';
$string['visibilitysetpublic'] = 'Make public';
$string['visibilitysetunlisted'] = 'Make unlisted';
$string['visibilitytitleunlisted'] = '{$a} (Unlisted)';
