{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template core_grades/searchwidget/searchresults

    The wrapper in which our search results will be rendered

    Example context (json):
    {
        "searchresults": [
            {
                "id": "1",
                "name": "Quiz 1",
                "fullname": "Cameron Greeve",
                "sendmessage": "http://foo.bar/message/index.php?id=25",
                "addcontact": "http://foo.bar/message/index.php?user1=2&user2=14&addcontact=14&sesskey=XXXXX",
                "currentuser": "2"
            }
        ]
    }
}}
<div class="searchresultitemscontainer-wrapper dropdown">
    <ul class="searchresultitemscontainer d-flex flex-column mw-100 position-relative py-2 list-group rounded-0" role="none" data-region="search-result-items-container">
        {{#searchresults}}
            {{>core_grades/searchwidget/searchitem}}
        {{/searchresults}}
        {{^searchresults}}
            <p>{{#str}} resultsfound, core, 0 {{/str}}</p>
        {{/searchresults}}
    </ul>
</div>
