{{!
    This file is part of Moodle - http://moodle.org/

    Moodle is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Moodle is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
}}
{{!
    @template mod_assign/grading_app

    Actions panel at the bottom of the assignment grading UI.

    Classes required for JS:
    * Uses some bootstrap classes, but for visual appeal only.

    Data attributes required for JS:
    * data-region, data-first-userid, data-courseid, data-contextid, data-assignmentid

    Context variables required for this template:
    * see mod/assign/classes/output/grading_app.php

    This template includes ajax functionality, so it cannot be shown in the template library.
}}
<div data-region="grading-navigation-panel" data-first-userid="{{userid}}" data-courseid="{{courseid}}" data-showuseridentity="{{showuseridentity}}">
{{> mod_assign/grading_navigation }}
</div>
{{#showreview}}
<div data-region="review-panel">
    <a href="#" data-region="review-panel-toggle">
        <div class="collapse-icon">{{#pix}} t/expanded, core, {{#str}} collapsereviewpanel, mod_assign {{/str}} {{/pix}}</div>
        <div class="expand-icon">
            <div class="ltr-icon">{{#pix}} t/collapsed, core, {{#str}} expandreviewpanel, mod_assign {{/str}} {{/pix}}</div>
            <div class="rtl-icon">{{#pix}} t/collapsed_rtl, core, {{#str}} expandreviewpanel, mod_assign {{/str}} {{/pix}}</div>
            <div class="toggle-text">{{#str}} clickexpandreviewpanel, mod_assign {{/str}}</div>
        </div>
    </a>
    <div data-region="review-panel-content">
        <div data-region="review">
            {{> mod_assign/review_panel }}
        </div>
    </div>
</div>
<div data-region="resizer" tabindex="0" aria-label="{{#str}} resizelayout, mod_assign {{/str}}" title="{{#str}} resizelayout, mod_assign {{/str}}"></div>
{{/showreview}}
<div data-region="grade-panel" {{^showreview}}class="fullwidth"{{/showreview}}>
<div data-region="grade" data-contextid="{{contextid}}" data-assignmentid="{{assignmentid}}">
{{> mod_assign/grading_panel }}
</div>
</div>
<div data-region="grade-actions-panel">
<div data-region="grade-actions">
<div class="tool_usertours-resettourcontainer float-start"></div>
{{> mod_assign/grading_actions }}
</div>
</div>
<div data-region="overlay" class="moodle-has-zindex">
{{> mod_assign/grading_save_in_progress }}
</div>
