@charset "UTF-8";
/*!
 * Font Awesome Free 6.7.2 by @fontawesome - https://fontawesome.com
 * License - https://fontawesome.com/license/free (Icons: CC BY 4.0, Fonts: SIL OFL 1.1, Code: MIT License)
 * Copyright 2024 Fonticons, Inc.
 */
:root, :host {
  --fa-style-family-brands: "Font Awesome 6 Brands";
  --fa-font-brands: normal 400 1em/1 "Font Awesome 6 Brands";
}

@font-face {
  font-family: "Font Awesome 6 Brands";
  font-style: normal;
  font-weight: 400;
  font-display: block;
  src: url("[[font:core|fa-brands-400.woff2]]") format("woff2"), url("[[font:core|fa-brands-400.ttf]]") format("truetype");
}
.fab,
.fa-brands {
  font-weight: 400;
}

.fa-monero {
  --fa: "\f3d0";
}

.fa-hooli {
  --fa: "\f427";
}

.fa-yelp {
  --fa: "\f1e9";
}

.fa-cc-visa {
  --fa: "\f1f0";
}

.fa-lastfm {
  --fa: "\f202";
}

.fa-shopware {
  --fa: "\f5b5";
}

.fa-creative-commons-nc {
  --fa: "\f4e8";
}

.fa-aws {
  --fa: "\f375";
}

.fa-redhat {
  --fa: "\f7bc";
}

.fa-yoast {
  --fa: "\f2b1";
}

.fa-cloudflare {
  --fa: "\e07d";
}

.fa-ups {
  --fa: "\f7e0";
}

.fa-pixiv {
  --fa: "\e640";
}

.fa-wpexplorer {
  --fa: "\f2de";
}

.fa-dyalog {
  --fa: "\f399";
}

.fa-bity {
  --fa: "\f37a";
}

.fa-stackpath {
  --fa: "\f842";
}

.fa-buysellads {
  --fa: "\f20d";
}

.fa-first-order {
  --fa: "\f2b0";
}

.fa-modx {
  --fa: "\f285";
}

.fa-guilded {
  --fa: "\e07e";
}

.fa-vnv {
  --fa: "\f40b";
}

.fa-square-js {
  --fa: "\f3b9";
}

.fa-js-square {
  --fa: "\f3b9";
}

.fa-microsoft {
  --fa: "\f3ca";
}

.fa-qq {
  --fa: "\f1d6";
}

.fa-orcid {
  --fa: "\f8d2";
}

.fa-java {
  --fa: "\f4e4";
}

.fa-invision {
  --fa: "\f7b0";
}

.fa-creative-commons-pd-alt {
  --fa: "\f4ed";
}

.fa-centercode {
  --fa: "\f380";
}

.fa-glide-g {
  --fa: "\f2a6";
}

.fa-drupal {
  --fa: "\f1a9";
}

.fa-jxl {
  --fa: "\e67b";
}

.fa-dart-lang {
  --fa: "\e693";
}

.fa-hire-a-helper {
  --fa: "\f3b0";
}

.fa-creative-commons-by {
  --fa: "\f4e7";
}

.fa-unity {
  --fa: "\e049";
}

.fa-whmcs {
  --fa: "\f40d";
}

.fa-rocketchat {
  --fa: "\f3e8";
}

.fa-vk {
  --fa: "\f189";
}

.fa-untappd {
  --fa: "\f405";
}

.fa-mailchimp {
  --fa: "\f59e";
}

.fa-css3-alt {
  --fa: "\f38b";
}

.fa-square-reddit {
  --fa: "\f1a2";
}

.fa-reddit-square {
  --fa: "\f1a2";
}

.fa-vimeo-v {
  --fa: "\f27d";
}

.fa-contao {
  --fa: "\f26d";
}

.fa-square-font-awesome {
  --fa: "\e5ad";
}

.fa-deskpro {
  --fa: "\f38f";
}

.fa-brave {
  --fa: "\e63c";
}

.fa-sistrix {
  --fa: "\f3ee";
}

.fa-square-instagram {
  --fa: "\e055";
}

.fa-instagram-square {
  --fa: "\e055";
}

.fa-battle-net {
  --fa: "\f835";
}

.fa-the-red-yeti {
  --fa: "\f69d";
}

.fa-square-hacker-news {
  --fa: "\f3af";
}

.fa-hacker-news-square {
  --fa: "\f3af";
}

.fa-edge {
  --fa: "\f282";
}

.fa-threads {
  --fa: "\e618";
}

.fa-napster {
  --fa: "\f3d2";
}

.fa-square-snapchat {
  --fa: "\f2ad";
}

.fa-snapchat-square {
  --fa: "\f2ad";
}

.fa-google-plus-g {
  --fa: "\f0d5";
}

.fa-artstation {
  --fa: "\f77a";
}

.fa-markdown {
  --fa: "\f60f";
}

.fa-sourcetree {
  --fa: "\f7d3";
}

.fa-google-plus {
  --fa: "\f2b3";
}

.fa-diaspora {
  --fa: "\f791";
}

.fa-foursquare {
  --fa: "\f180";
}

.fa-stack-overflow {
  --fa: "\f16c";
}

.fa-github-alt {
  --fa: "\f113";
}

.fa-phoenix-squadron {
  --fa: "\f511";
}

.fa-pagelines {
  --fa: "\f18c";
}

.fa-algolia {
  --fa: "\f36c";
}

.fa-red-river {
  --fa: "\f3e3";
}

.fa-creative-commons-sa {
  --fa: "\f4ef";
}

.fa-safari {
  --fa: "\f267";
}

.fa-google {
  --fa: "\f1a0";
}

.fa-square-font-awesome-stroke {
  --fa: "\f35c";
}

.fa-font-awesome-alt {
  --fa: "\f35c";
}

.fa-atlassian {
  --fa: "\f77b";
}

.fa-linkedin-in {
  --fa: "\f0e1";
}

.fa-digital-ocean {
  --fa: "\f391";
}

.fa-nimblr {
  --fa: "\f5a8";
}

.fa-chromecast {
  --fa: "\f838";
}

.fa-evernote {
  --fa: "\f839";
}

.fa-hacker-news {
  --fa: "\f1d4";
}

.fa-creative-commons-sampling {
  --fa: "\f4f0";
}

.fa-adversal {
  --fa: "\f36a";
}

.fa-creative-commons {
  --fa: "\f25e";
}

.fa-watchman-monitoring {
  --fa: "\e087";
}

.fa-fonticons {
  --fa: "\f280";
}

.fa-weixin {
  --fa: "\f1d7";
}

.fa-shirtsinbulk {
  --fa: "\f214";
}

.fa-codepen {
  --fa: "\f1cb";
}

.fa-git-alt {
  --fa: "\f841";
}

.fa-lyft {
  --fa: "\f3c3";
}

.fa-rev {
  --fa: "\f5b2";
}

.fa-windows {
  --fa: "\f17a";
}

.fa-wizards-of-the-coast {
  --fa: "\f730";
}

.fa-square-viadeo {
  --fa: "\f2aa";
}

.fa-viadeo-square {
  --fa: "\f2aa";
}

.fa-meetup {
  --fa: "\f2e0";
}

.fa-centos {
  --fa: "\f789";
}

.fa-adn {
  --fa: "\f170";
}

.fa-cloudsmith {
  --fa: "\f384";
}

.fa-opensuse {
  --fa: "\e62b";
}

.fa-pied-piper-alt {
  --fa: "\f1a8";
}

.fa-square-dribbble {
  --fa: "\f397";
}

.fa-dribbble-square {
  --fa: "\f397";
}

.fa-codiepie {
  --fa: "\f284";
}

.fa-node {
  --fa: "\f419";
}

.fa-mix {
  --fa: "\f3cb";
}

.fa-steam {
  --fa: "\f1b6";
}

.fa-cc-apple-pay {
  --fa: "\f416";
}

.fa-scribd {
  --fa: "\f28a";
}

.fa-debian {
  --fa: "\e60b";
}

.fa-openid {
  --fa: "\f19b";
}

.fa-instalod {
  --fa: "\e081";
}

.fa-files-pinwheel {
  --fa: "\e69f";
}

.fa-expeditedssl {
  --fa: "\f23e";
}

.fa-sellcast {
  --fa: "\f2da";
}

.fa-square-twitter {
  --fa: "\f081";
}

.fa-twitter-square {
  --fa: "\f081";
}

.fa-r-project {
  --fa: "\f4f7";
}

.fa-delicious {
  --fa: "\f1a5";
}

.fa-freebsd {
  --fa: "\f3a4";
}

.fa-vuejs {
  --fa: "\f41f";
}

.fa-accusoft {
  --fa: "\f369";
}

.fa-ioxhost {
  --fa: "\f208";
}

.fa-fonticons-fi {
  --fa: "\f3a2";
}

.fa-app-store {
  --fa: "\f36f";
}

.fa-cc-mastercard {
  --fa: "\f1f1";
}

.fa-itunes-note {
  --fa: "\f3b5";
}

.fa-golang {
  --fa: "\e40f";
}

.fa-kickstarter {
  --fa: "\f3bb";
}

.fa-square-kickstarter {
  --fa: "\f3bb";
}

.fa-grav {
  --fa: "\f2d6";
}

.fa-weibo {
  --fa: "\f18a";
}

.fa-uncharted {
  --fa: "\e084";
}

.fa-firstdraft {
  --fa: "\f3a1";
}

.fa-square-youtube {
  --fa: "\f431";
}

.fa-youtube-square {
  --fa: "\f431";
}

.fa-wikipedia-w {
  --fa: "\f266";
}

.fa-wpressr {
  --fa: "\f3e4";
}

.fa-rendact {
  --fa: "\f3e4";
}

.fa-angellist {
  --fa: "\f209";
}

.fa-galactic-republic {
  --fa: "\f50c";
}

.fa-nfc-directional {
  --fa: "\e530";
}

.fa-skype {
  --fa: "\f17e";
}

.fa-joget {
  --fa: "\f3b7";
}

.fa-fedora {
  --fa: "\f798";
}

.fa-stripe-s {
  --fa: "\f42a";
}

.fa-meta {
  --fa: "\e49b";
}

.fa-laravel {
  --fa: "\f3bd";
}

.fa-hotjar {
  --fa: "\f3b1";
}

.fa-bluetooth-b {
  --fa: "\f294";
}

.fa-square-letterboxd {
  --fa: "\e62e";
}

.fa-sticker-mule {
  --fa: "\f3f7";
}

.fa-creative-commons-zero {
  --fa: "\f4f3";
}

.fa-hips {
  --fa: "\f452";
}

.fa-css {
  --fa: "\e6a2";
}

.fa-behance {
  --fa: "\f1b4";
}

.fa-reddit {
  --fa: "\f1a1";
}

.fa-discord {
  --fa: "\f392";
}

.fa-chrome {
  --fa: "\f268";
}

.fa-app-store-ios {
  --fa: "\f370";
}

.fa-cc-discover {
  --fa: "\f1f2";
}

.fa-wpbeginner {
  --fa: "\f297";
}

.fa-confluence {
  --fa: "\f78d";
}

.fa-shoelace {
  --fa: "\e60c";
}

.fa-mdb {
  --fa: "\f8ca";
}

.fa-dochub {
  --fa: "\f394";
}

.fa-accessible-icon {
  --fa: "\f368";
}

.fa-ebay {
  --fa: "\f4f4";
}

.fa-amazon {
  --fa: "\f270";
}

.fa-unsplash {
  --fa: "\e07c";
}

.fa-yarn {
  --fa: "\f7e3";
}

.fa-square-steam {
  --fa: "\f1b7";
}

.fa-steam-square {
  --fa: "\f1b7";
}

.fa-500px {
  --fa: "\f26e";
}

.fa-square-vimeo {
  --fa: "\f194";
}

.fa-vimeo-square {
  --fa: "\f194";
}

.fa-asymmetrik {
  --fa: "\f372";
}

.fa-font-awesome {
  --fa: "\f2b4";
}

.fa-font-awesome-flag {
  --fa: "\f2b4";
}

.fa-font-awesome-logo-full {
  --fa: "\f2b4";
}

.fa-gratipay {
  --fa: "\f184";
}

.fa-apple {
  --fa: "\f179";
}

.fa-hive {
  --fa: "\e07f";
}

.fa-gitkraken {
  --fa: "\f3a6";
}

.fa-keybase {
  --fa: "\f4f5";
}

.fa-apple-pay {
  --fa: "\f415";
}

.fa-padlet {
  --fa: "\e4a0";
}

.fa-amazon-pay {
  --fa: "\f42c";
}

.fa-square-github {
  --fa: "\f092";
}

.fa-github-square {
  --fa: "\f092";
}

.fa-stumbleupon {
  --fa: "\f1a4";
}

.fa-fedex {
  --fa: "\f797";
}

.fa-phoenix-framework {
  --fa: "\f3dc";
}

.fa-shopify {
  --fa: "\e057";
}

.fa-neos {
  --fa: "\f612";
}

.fa-square-threads {
  --fa: "\e619";
}

.fa-hackerrank {
  --fa: "\f5f7";
}

.fa-researchgate {
  --fa: "\f4f8";
}

.fa-swift {
  --fa: "\f8e1";
}

.fa-angular {
  --fa: "\f420";
}

.fa-speakap {
  --fa: "\f3f3";
}

.fa-angrycreative {
  --fa: "\f36e";
}

.fa-y-combinator {
  --fa: "\f23b";
}

.fa-empire {
  --fa: "\f1d1";
}

.fa-envira {
  --fa: "\f299";
}

.fa-google-scholar {
  --fa: "\e63b";
}

.fa-square-gitlab {
  --fa: "\e5ae";
}

.fa-gitlab-square {
  --fa: "\e5ae";
}

.fa-studiovinari {
  --fa: "\f3f8";
}

.fa-pied-piper {
  --fa: "\f2ae";
}

.fa-wordpress {
  --fa: "\f19a";
}

.fa-product-hunt {
  --fa: "\f288";
}

.fa-firefox {
  --fa: "\f269";
}

.fa-linode {
  --fa: "\f2b8";
}

.fa-goodreads {
  --fa: "\f3a8";
}

.fa-square-odnoklassniki {
  --fa: "\f264";
}

.fa-odnoklassniki-square {
  --fa: "\f264";
}

.fa-jsfiddle {
  --fa: "\f1cc";
}

.fa-sith {
  --fa: "\f512";
}

.fa-themeisle {
  --fa: "\f2b2";
}

.fa-page4 {
  --fa: "\f3d7";
}

.fa-hashnode {
  --fa: "\e499";
}

.fa-react {
  --fa: "\f41b";
}

.fa-cc-paypal {
  --fa: "\f1f4";
}

.fa-squarespace {
  --fa: "\f5be";
}

.fa-cc-stripe {
  --fa: "\f1f5";
}

.fa-creative-commons-share {
  --fa: "\f4f2";
}

.fa-bitcoin {
  --fa: "\f379";
}

.fa-keycdn {
  --fa: "\f3ba";
}

.fa-opera {
  --fa: "\f26a";
}

.fa-itch-io {
  --fa: "\f83a";
}

.fa-umbraco {
  --fa: "\f8e8";
}

.fa-galactic-senate {
  --fa: "\f50d";
}

.fa-ubuntu {
  --fa: "\f7df";
}

.fa-draft2digital {
  --fa: "\f396";
}

.fa-stripe {
  --fa: "\f429";
}

.fa-houzz {
  --fa: "\f27c";
}

.fa-gg {
  --fa: "\f260";
}

.fa-dhl {
  --fa: "\f790";
}

.fa-square-pinterest {
  --fa: "\f0d3";
}

.fa-pinterest-square {
  --fa: "\f0d3";
}

.fa-xing {
  --fa: "\f168";
}

.fa-blackberry {
  --fa: "\f37b";
}

.fa-creative-commons-pd {
  --fa: "\f4ec";
}

.fa-playstation {
  --fa: "\f3df";
}

.fa-quinscape {
  --fa: "\f459";
}

.fa-less {
  --fa: "\f41d";
}

.fa-blogger-b {
  --fa: "\f37d";
}

.fa-opencart {
  --fa: "\f23d";
}

.fa-vine {
  --fa: "\f1ca";
}

.fa-signal-messenger {
  --fa: "\e663";
}

.fa-paypal {
  --fa: "\f1ed";
}

.fa-gitlab {
  --fa: "\f296";
}

.fa-typo3 {
  --fa: "\f42b";
}

.fa-reddit-alien {
  --fa: "\f281";
}

.fa-yahoo {
  --fa: "\f19e";
}

.fa-dailymotion {
  --fa: "\e052";
}

.fa-affiliatetheme {
  --fa: "\f36b";
}

.fa-pied-piper-pp {
  --fa: "\f1a7";
}

.fa-bootstrap {
  --fa: "\f836";
}

.fa-odnoklassniki {
  --fa: "\f263";
}

.fa-nfc-symbol {
  --fa: "\e531";
}

.fa-mintbit {
  --fa: "\e62f";
}

.fa-ethereum {
  --fa: "\f42e";
}

.fa-speaker-deck {
  --fa: "\f83c";
}

.fa-creative-commons-nc-eu {
  --fa: "\f4e9";
}

.fa-patreon {
  --fa: "\f3d9";
}

.fa-avianex {
  --fa: "\f374";
}

.fa-ello {
  --fa: "\f5f1";
}

.fa-gofore {
  --fa: "\f3a7";
}

.fa-bimobject {
  --fa: "\f378";
}

.fa-brave-reverse {
  --fa: "\e63d";
}

.fa-facebook-f {
  --fa: "\f39e";
}

.fa-square-google-plus {
  --fa: "\f0d4";
}

.fa-google-plus-square {
  --fa: "\f0d4";
}

.fa-web-awesome {
  --fa: "\e682";
}

.fa-mandalorian {
  --fa: "\f50f";
}

.fa-first-order-alt {
  --fa: "\f50a";
}

.fa-osi {
  --fa: "\f41a";
}

.fa-google-wallet {
  --fa: "\f1ee";
}

.fa-d-and-d-beyond {
  --fa: "\f6ca";
}

.fa-periscope {
  --fa: "\f3da";
}

.fa-fulcrum {
  --fa: "\f50b";
}

.fa-cloudscale {
  --fa: "\f383";
}

.fa-forumbee {
  --fa: "\f211";
}

.fa-mizuni {
  --fa: "\f3cc";
}

.fa-schlix {
  --fa: "\f3ea";
}

.fa-square-xing {
  --fa: "\f169";
}

.fa-xing-square {
  --fa: "\f169";
}

.fa-bandcamp {
  --fa: "\f2d5";
}

.fa-wpforms {
  --fa: "\f298";
}

.fa-cloudversify {
  --fa: "\f385";
}

.fa-usps {
  --fa: "\f7e1";
}

.fa-megaport {
  --fa: "\f5a3";
}

.fa-magento {
  --fa: "\f3c4";
}

.fa-spotify {
  --fa: "\f1bc";
}

.fa-optin-monster {
  --fa: "\f23c";
}

.fa-fly {
  --fa: "\f417";
}

.fa-square-bluesky {
  --fa: "\e6a3";
}

.fa-aviato {
  --fa: "\f421";
}

.fa-itunes {
  --fa: "\f3b4";
}

.fa-cuttlefish {
  --fa: "\f38c";
}

.fa-blogger {
  --fa: "\f37c";
}

.fa-flickr {
  --fa: "\f16e";
}

.fa-viber {
  --fa: "\f409";
}

.fa-soundcloud {
  --fa: "\f1be";
}

.fa-digg {
  --fa: "\f1a6";
}

.fa-tencent-weibo {
  --fa: "\f1d5";
}

.fa-letterboxd {
  --fa: "\e62d";
}

.fa-symfony {
  --fa: "\f83d";
}

.fa-maxcdn {
  --fa: "\f136";
}

.fa-etsy {
  --fa: "\f2d7";
}

.fa-facebook-messenger {
  --fa: "\f39f";
}

.fa-audible {
  --fa: "\f373";
}

.fa-think-peaks {
  --fa: "\f731";
}

.fa-bilibili {
  --fa: "\e3d9";
}

.fa-erlang {
  --fa: "\f39d";
}

.fa-x-twitter {
  --fa: "\e61b";
}

.fa-cotton-bureau {
  --fa: "\f89e";
}

.fa-dashcube {
  --fa: "\f210";
}

.fa-42-group {
  --fa: "\e080";
}

.fa-innosoft {
  --fa: "\e080";
}

.fa-stack-exchange {
  --fa: "\f18d";
}

.fa-elementor {
  --fa: "\f430";
}

.fa-square-pied-piper {
  --fa: "\e01e";
}

.fa-pied-piper-square {
  --fa: "\e01e";
}

.fa-creative-commons-nd {
  --fa: "\f4eb";
}

.fa-palfed {
  --fa: "\f3d8";
}

.fa-superpowers {
  --fa: "\f2dd";
}

.fa-resolving {
  --fa: "\f3e7";
}

.fa-xbox {
  --fa: "\f412";
}

.fa-square-web-awesome-stroke {
  --fa: "\e684";
}

.fa-searchengin {
  --fa: "\f3eb";
}

.fa-tiktok {
  --fa: "\e07b";
}

.fa-square-facebook {
  --fa: "\f082";
}

.fa-facebook-square {
  --fa: "\f082";
}

.fa-renren {
  --fa: "\f18b";
}

.fa-linux {
  --fa: "\f17c";
}

.fa-glide {
  --fa: "\f2a5";
}

.fa-linkedin {
  --fa: "\f08c";
}

.fa-hubspot {
  --fa: "\f3b2";
}

.fa-deploydog {
  --fa: "\f38e";
}

.fa-twitch {
  --fa: "\f1e8";
}

.fa-flutter {
  --fa: "\e694";
}

.fa-ravelry {
  --fa: "\f2d9";
}

.fa-mixer {
  --fa: "\e056";
}

.fa-square-lastfm {
  --fa: "\f203";
}

.fa-lastfm-square {
  --fa: "\f203";
}

.fa-vimeo {
  --fa: "\f40a";
}

.fa-mendeley {
  --fa: "\f7b3";
}

.fa-uniregistry {
  --fa: "\f404";
}

.fa-figma {
  --fa: "\f799";
}

.fa-creative-commons-remix {
  --fa: "\f4ee";
}

.fa-cc-amazon-pay {
  --fa: "\f42d";
}

.fa-dropbox {
  --fa: "\f16b";
}

.fa-instagram {
  --fa: "\f16d";
}

.fa-cmplid {
  --fa: "\e360";
}

.fa-upwork {
  --fa: "\e641";
}

.fa-facebook {
  --fa: "\f09a";
}

.fa-gripfire {
  --fa: "\f3ac";
}

.fa-jedi-order {
  --fa: "\f50e";
}

.fa-uikit {
  --fa: "\f403";
}

.fa-fort-awesome-alt {
  --fa: "\f3a3";
}

.fa-phabricator {
  --fa: "\f3db";
}

.fa-ussunnah {
  --fa: "\f407";
}

.fa-earlybirds {
  --fa: "\f39a";
}

.fa-trade-federation {
  --fa: "\f513";
}

.fa-autoprefixer {
  --fa: "\f41c";
}

.fa-whatsapp {
  --fa: "\f232";
}

.fa-square-upwork {
  --fa: "\e67c";
}

.fa-slideshare {
  --fa: "\f1e7";
}

.fa-google-play {
  --fa: "\f3ab";
}

.fa-viadeo {
  --fa: "\f2a9";
}

.fa-line {
  --fa: "\f3c0";
}

.fa-google-drive {
  --fa: "\f3aa";
}

.fa-servicestack {
  --fa: "\f3ec";
}

.fa-simplybuilt {
  --fa: "\f215";
}

.fa-bitbucket {
  --fa: "\f171";
}

.fa-imdb {
  --fa: "\f2d8";
}

.fa-deezer {
  --fa: "\e077";
}

.fa-raspberry-pi {
  --fa: "\f7bb";
}

.fa-jira {
  --fa: "\f7b1";
}

.fa-docker {
  --fa: "\f395";
}

.fa-screenpal {
  --fa: "\e570";
}

.fa-bluetooth {
  --fa: "\f293";
}

.fa-gitter {
  --fa: "\f426";
}

.fa-d-and-d {
  --fa: "\f38d";
}

.fa-microblog {
  --fa: "\e01a";
}

.fa-cc-diners-club {
  --fa: "\f24c";
}

.fa-gg-circle {
  --fa: "\f261";
}

.fa-pied-piper-hat {
  --fa: "\f4e5";
}

.fa-kickstarter-k {
  --fa: "\f3bc";
}

.fa-yandex {
  --fa: "\f413";
}

.fa-readme {
  --fa: "\f4d5";
}

.fa-html5 {
  --fa: "\f13b";
}

.fa-sellsy {
  --fa: "\f213";
}

.fa-square-web-awesome {
  --fa: "\e683";
}

.fa-sass {
  --fa: "\f41e";
}

.fa-wirsindhandwerk {
  --fa: "\e2d0";
}

.fa-wsh {
  --fa: "\e2d0";
}

.fa-buromobelexperte {
  --fa: "\f37f";
}

.fa-salesforce {
  --fa: "\f83b";
}

.fa-octopus-deploy {
  --fa: "\e082";
}

.fa-medapps {
  --fa: "\f3c6";
}

.fa-ns8 {
  --fa: "\f3d5";
}

.fa-pinterest-p {
  --fa: "\f231";
}

.fa-apper {
  --fa: "\f371";
}

.fa-fort-awesome {
  --fa: "\f286";
}

.fa-waze {
  --fa: "\f83f";
}

.fa-bluesky {
  --fa: "\e671";
}

.fa-cc-jcb {
  --fa: "\f24b";
}

.fa-snapchat {
  --fa: "\f2ab";
}

.fa-snapchat-ghost {
  --fa: "\f2ab";
}

.fa-fantasy-flight-games {
  --fa: "\f6dc";
}

.fa-rust {
  --fa: "\e07a";
}

.fa-wix {
  --fa: "\f5cf";
}

.fa-square-behance {
  --fa: "\f1b5";
}

.fa-behance-square {
  --fa: "\f1b5";
}

.fa-supple {
  --fa: "\f3f9";
}

.fa-webflow {
  --fa: "\e65c";
}

.fa-rebel {
  --fa: "\f1d0";
}

.fa-css3 {
  --fa: "\f13c";
}

.fa-staylinked {
  --fa: "\f3f5";
}

.fa-kaggle {
  --fa: "\f5fa";
}

.fa-space-awesome {
  --fa: "\e5ac";
}

.fa-deviantart {
  --fa: "\f1bd";
}

.fa-cpanel {
  --fa: "\f388";
}

.fa-goodreads-g {
  --fa: "\f3a9";
}

.fa-square-git {
  --fa: "\f1d2";
}

.fa-git-square {
  --fa: "\f1d2";
}

.fa-square-tumblr {
  --fa: "\f174";
}

.fa-tumblr-square {
  --fa: "\f174";
}

.fa-trello {
  --fa: "\f181";
}

.fa-creative-commons-nc-jp {
  --fa: "\f4ea";
}

.fa-get-pocket {
  --fa: "\f265";
}

.fa-perbyte {
  --fa: "\e083";
}

.fa-grunt {
  --fa: "\f3ad";
}

.fa-weebly {
  --fa: "\f5cc";
}

.fa-connectdevelop {
  --fa: "\f20e";
}

.fa-leanpub {
  --fa: "\f212";
}

.fa-black-tie {
  --fa: "\f27e";
}

.fa-themeco {
  --fa: "\f5c6";
}

.fa-python {
  --fa: "\f3e2";
}

.fa-android {
  --fa: "\f17b";
}

.fa-bots {
  --fa: "\e340";
}

.fa-free-code-camp {
  --fa: "\f2c5";
}

.fa-hornbill {
  --fa: "\f592";
}

.fa-js {
  --fa: "\f3b8";
}

.fa-ideal {
  --fa: "\e013";
}

.fa-git {
  --fa: "\f1d3";
}

.fa-dev {
  --fa: "\f6cc";
}

.fa-sketch {
  --fa: "\f7c6";
}

.fa-yandex-international {
  --fa: "\f414";
}

.fa-cc-amex {
  --fa: "\f1f3";
}

.fa-uber {
  --fa: "\f402";
}

.fa-github {
  --fa: "\f09b";
}

.fa-php {
  --fa: "\f457";
}

.fa-alipay {
  --fa: "\f642";
}

.fa-youtube {
  --fa: "\f167";
}

.fa-skyatlas {
  --fa: "\f216";
}

.fa-firefox-browser {
  --fa: "\e007";
}

.fa-replyd {
  --fa: "\f3e6";
}

.fa-suse {
  --fa: "\f7d6";
}

.fa-jenkins {
  --fa: "\f3b6";
}

.fa-twitter {
  --fa: "\f099";
}

.fa-rockrms {
  --fa: "\f3e9";
}

.fa-pinterest {
  --fa: "\f0d2";
}

.fa-buffer {
  --fa: "\f837";
}

.fa-npm {
  --fa: "\f3d4";
}

.fa-yammer {
  --fa: "\f840";
}

.fa-btc {
  --fa: "\f15a";
}

.fa-dribbble {
  --fa: "\f17d";
}

.fa-stumbleupon-circle {
  --fa: "\f1a3";
}

.fa-internet-explorer {
  --fa: "\f26b";
}

.fa-stubber {
  --fa: "\e5c7";
}

.fa-telegram {
  --fa: "\f2c6";
}

.fa-telegram-plane {
  --fa: "\f2c6";
}

.fa-old-republic {
  --fa: "\f510";
}

.fa-odysee {
  --fa: "\e5c6";
}

.fa-square-whatsapp {
  --fa: "\f40c";
}

.fa-whatsapp-square {
  --fa: "\f40c";
}

.fa-node-js {
  --fa: "\f3d3";
}

.fa-edge-legacy {
  --fa: "\e078";
}

.fa-slack {
  --fa: "\f198";
}

.fa-slack-hash {
  --fa: "\f198";
}

.fa-medrt {
  --fa: "\f3c8";
}

.fa-usb {
  --fa: "\f287";
}

.fa-tumblr {
  --fa: "\f173";
}

.fa-vaadin {
  --fa: "\f408";
}

.fa-quora {
  --fa: "\f2c4";
}

.fa-square-x-twitter {
  --fa: "\e61a";
}

.fa-reacteurope {
  --fa: "\f75d";
}

.fa-medium {
  --fa: "\f23a";
}

.fa-medium-m {
  --fa: "\f23a";
}

.fa-amilia {
  --fa: "\f36d";
}

.fa-mixcloud {
  --fa: "\f289";
}

.fa-flipboard {
  --fa: "\f44d";
}

.fa-viacoin {
  --fa: "\f237";
}

.fa-critical-role {
  --fa: "\f6c9";
}

.fa-sitrox {
  --fa: "\e44a";
}

.fa-discourse {
  --fa: "\f393";
}

.fa-joomla {
  --fa: "\f1aa";
}

.fa-mastodon {
  --fa: "\f4f6";
}

.fa-airbnb {
  --fa: "\f834";
}

.fa-wolf-pack-battalion {
  --fa: "\f514";
}

.fa-buy-n-large {
  --fa: "\f8a6";
}

.fa-gulp {
  --fa: "\f3ae";
}

.fa-creative-commons-sampling-plus {
  --fa: "\f4f1";
}

.fa-strava {
  --fa: "\f428";
}

.fa-ember {
  --fa: "\f423";
}

.fa-canadian-maple-leaf {
  --fa: "\f785";
}

.fa-teamspeak {
  --fa: "\f4f9";
}

.fa-pushed {
  --fa: "\f3e1";
}

.fa-wordpress-simple {
  --fa: "\f411";
}

.fa-nutritionix {
  --fa: "\f3d6";
}

.fa-wodu {
  --fa: "\e088";
}

.fa-google-pay {
  --fa: "\e079";
}

.fa-intercom {
  --fa: "\f7af";
}

.fa-zhihu {
  --fa: "\f63f";
}

.fa-korvue {
  --fa: "\f42f";
}

.fa-pix {
  --fa: "\e43a";
}

.fa-steam-symbol {
  --fa: "\f3f6";
}

/*!
 * Font Awesome Free 6.7.2 by @fontawesome - https://fontawesome.com
 * License - https://fontawesome.com/license/free (Icons: CC BY 4.0, Fonts: SIL OFL 1.1, Code: MIT License)
 * Copyright 2024 Fonticons, Inc.
 */
:root, :host {
  --fa-style-family-classic: "Font Awesome 6 Free";
  --fa-font-regular: normal 400 1em/1 "Font Awesome 6 Free";
}

@font-face {
  font-family: "Font Awesome 6 Free";
  font-style: normal;
  font-weight: 400;
  font-display: block;
  src: url("[[font:core|fa-regular-400.woff2]]") format("woff2"), url("[[font:core|fa-regular-400.ttf]]") format("truetype");
}
.far,
.fa-regular {
  font-weight: 400;
}

/*!
 * Font Awesome Free 6.7.2 by @fontawesome - https://fontawesome.com
 * License - https://fontawesome.com/license/free (Icons: CC BY 4.0, Fonts: SIL OFL 1.1, Code: MIT License)
 * Copyright 2024 Fonticons, Inc.
 */
:root, :host {
  --fa-style-family-classic: "Font Awesome 6 Free";
  --fa-font-solid: normal 900 1em/1 "Font Awesome 6 Free";
}

@font-face {
  font-family: "Font Awesome 6 Free";
  font-style: normal;
  font-weight: 900;
  font-display: block;
  src: url("[[font:core|fa-solid-900.woff2]]") format("woff2"), url("[[font:core|fa-solid-900.ttf]]") format("truetype");
}
.fas,
.fa-solid {
  font-weight: 900;
}

/*!
 * Font Awesome Free 6.7.2 by @fontawesome - https://fontawesome.com
 * License - https://fontawesome.com/license/free (Icons: CC BY 4.0, Fonts: SIL OFL 1.1, Code: MIT License)
 * Copyright 2024 Fonticons, Inc.
 */
.fa.fa-glass {
  --fa: "\f000";
}

.fa.fa-envelope-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-envelope-o {
  --fa: "\f0e0";
}

.fa.fa-star-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-star-o {
  --fa: "\f005";
}

.fa.fa-remove {
  --fa: "\f00d";
}

.fa.fa-close {
  --fa: "\f00d";
}

.fa.fa-gear {
  --fa: "\f013";
}

.fa.fa-trash-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-trash-o {
  --fa: "\f2ed";
}

.fa.fa-home {
  --fa: "\f015";
}

.fa.fa-file-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-o {
  --fa: "\f15b";
}

.fa.fa-clock-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-clock-o {
  --fa: "\f017";
}

.fa.fa-arrow-circle-o-down {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-down {
  --fa: "\f358";
}

.fa.fa-arrow-circle-o-up {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-up {
  --fa: "\f35b";
}

.fa.fa-play-circle-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-play-circle-o {
  --fa: "\f144";
}

.fa.fa-repeat {
  --fa: "\f01e";
}

.fa.fa-rotate-right {
  --fa: "\f01e";
}

.fa.fa-refresh {
  --fa: "\f021";
}

.fa.fa-list-alt {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-list-alt {
  --fa: "\f022";
}

.fa.fa-dedent {
  --fa: "\f03b";
}

.fa.fa-video-camera {
  --fa: "\f03d";
}

.fa.fa-picture-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-picture-o {
  --fa: "\f03e";
}

.fa.fa-photo {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-photo {
  --fa: "\f03e";
}

.fa.fa-image {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-image {
  --fa: "\f03e";
}

.fa.fa-map-marker {
  --fa: "\f3c5";
}

.fa.fa-pencil-square-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-pencil-square-o {
  --fa: "\f044";
}

.fa.fa-edit {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-edit {
  --fa: "\f044";
}

.fa.fa-share-square-o {
  --fa: "\f14d";
}

.fa.fa-check-square-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-check-square-o {
  --fa: "\f14a";
}

.fa.fa-arrows {
  --fa: "\f0b2";
}

.fa.fa-times-circle-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-times-circle-o {
  --fa: "\f057";
}

.fa.fa-check-circle-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-check-circle-o {
  --fa: "\f058";
}

.fa.fa-mail-forward {
  --fa: "\f064";
}

.fa.fa-expand {
  --fa: "\f424";
}

.fa.fa-compress {
  --fa: "\f422";
}

.fa.fa-eye {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-eye-slash {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-warning {
  --fa: "\f071";
}

.fa.fa-calendar {
  --fa: "\f073";
}

.fa.fa-arrows-v {
  --fa: "\f338";
}

.fa.fa-arrows-h {
  --fa: "\f337";
}

.fa.fa-bar-chart {
  --fa: "\e0e3";
}

.fa.fa-bar-chart-o {
  --fa: "\e0e3";
}

.fa.fa-twitter-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-twitter-square {
  --fa: "\f081";
}

.fa.fa-facebook-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-facebook-square {
  --fa: "\f082";
}

.fa.fa-gears {
  --fa: "\f085";
}

.fa.fa-thumbs-o-up {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-thumbs-o-up {
  --fa: "\f164";
}

.fa.fa-thumbs-o-down {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-thumbs-o-down {
  --fa: "\f165";
}

.fa.fa-heart-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-heart-o {
  --fa: "\f004";
}

.fa.fa-sign-out {
  --fa: "\f2f5";
}

.fa.fa-linkedin-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-linkedin-square {
  --fa: "\f08c";
}

.fa.fa-thumb-tack {
  --fa: "\f08d";
}

.fa.fa-external-link {
  --fa: "\f35d";
}

.fa.fa-sign-in {
  --fa: "\f2f6";
}

.fa.fa-github-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-github-square {
  --fa: "\f092";
}

.fa.fa-lemon-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-lemon-o {
  --fa: "\f094";
}

.fa.fa-square-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-square-o {
  --fa: "\f0c8";
}

.fa.fa-bookmark-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bookmark-o {
  --fa: "\f02e";
}

.fa.fa-twitter {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-facebook {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-facebook {
  --fa: "\f39e";
}

.fa.fa-facebook-f {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-facebook-f {
  --fa: "\f39e";
}

.fa.fa-github {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-credit-card {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-feed {
  --fa: "\f09e";
}

.fa.fa-hdd-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hdd-o {
  --fa: "\f0a0";
}

.fa.fa-hand-o-right {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-right {
  --fa: "\f0a4";
}

.fa.fa-hand-o-left {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-left {
  --fa: "\f0a5";
}

.fa.fa-hand-o-up {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-up {
  --fa: "\f0a6";
}

.fa.fa-hand-o-down {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-o-down {
  --fa: "\f0a7";
}

.fa.fa-globe {
  --fa: "\f57d";
}

.fa.fa-tasks {
  --fa: "\f828";
}

.fa.fa-arrows-alt {
  --fa: "\f31e";
}

.fa.fa-group {
  --fa: "\f0c0";
}

.fa.fa-chain {
  --fa: "\f0c1";
}

.fa.fa-cut {
  --fa: "\f0c4";
}

.fa.fa-files-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-files-o {
  --fa: "\f0c5";
}

.fa.fa-floppy-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-floppy-o {
  --fa: "\f0c7";
}

.fa.fa-save {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-save {
  --fa: "\f0c7";
}

.fa.fa-navicon {
  --fa: "\f0c9";
}

.fa.fa-reorder {
  --fa: "\f0c9";
}

.fa.fa-magic {
  --fa: "\e2ca";
}

.fa.fa-pinterest {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pinterest-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pinterest-square {
  --fa: "\f0d3";
}

.fa.fa-google-plus-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus-square {
  --fa: "\f0d4";
}

.fa.fa-google-plus {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus {
  --fa: "\f0d5";
}

.fa.fa-money {
  --fa: "\f3d1";
}

.fa.fa-unsorted {
  --fa: "\f0dc";
}

.fa.fa-sort-desc {
  --fa: "\f0dd";
}

.fa.fa-sort-asc {
  --fa: "\f0de";
}

.fa.fa-linkedin {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-linkedin {
  --fa: "\f0e1";
}

.fa.fa-rotate-left {
  --fa: "\f0e2";
}

.fa.fa-legal {
  --fa: "\f0e3";
}

.fa.fa-tachometer {
  --fa: "\f625";
}

.fa.fa-dashboard {
  --fa: "\f625";
}

.fa.fa-comment-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-comment-o {
  --fa: "\f075";
}

.fa.fa-comments-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-comments-o {
  --fa: "\f086";
}

.fa.fa-flash {
  --fa: "\f0e7";
}

.fa.fa-clipboard {
  --fa: "\f0ea";
}

.fa.fa-lightbulb-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-lightbulb-o {
  --fa: "\f0eb";
}

.fa.fa-exchange {
  --fa: "\f362";
}

.fa.fa-cloud-download {
  --fa: "\f0ed";
}

.fa.fa-cloud-upload {
  --fa: "\f0ee";
}

.fa.fa-bell-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bell-o {
  --fa: "\f0f3";
}

.fa.fa-cutlery {
  --fa: "\f2e7";
}

.fa.fa-file-text-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-text-o {
  --fa: "\f15c";
}

.fa.fa-building-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-building-o {
  --fa: "\f1ad";
}

.fa.fa-hospital-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hospital-o {
  --fa: "\f0f8";
}

.fa.fa-tablet {
  --fa: "\f3fa";
}

.fa.fa-mobile {
  --fa: "\f3cd";
}

.fa.fa-mobile-phone {
  --fa: "\f3cd";
}

.fa.fa-circle-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-circle-o {
  --fa: "\f111";
}

.fa.fa-mail-reply {
  --fa: "\f3e5";
}

.fa.fa-github-alt {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-folder-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-folder-o {
  --fa: "\f07b";
}

.fa.fa-folder-open-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-folder-open-o {
  --fa: "\f07c";
}

.fa.fa-smile-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-smile-o {
  --fa: "\f118";
}

.fa.fa-frown-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-frown-o {
  --fa: "\f119";
}

.fa.fa-meh-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-meh-o {
  --fa: "\f11a";
}

.fa.fa-keyboard-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-keyboard-o {
  --fa: "\f11c";
}

.fa.fa-flag-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-flag-o {
  --fa: "\f024";
}

.fa.fa-mail-reply-all {
  --fa: "\f122";
}

.fa.fa-star-half-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-star-half-o {
  --fa: "\f5c0";
}

.fa.fa-star-half-empty {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-star-half-empty {
  --fa: "\f5c0";
}

.fa.fa-star-half-full {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-star-half-full {
  --fa: "\f5c0";
}

.fa.fa-code-fork {
  --fa: "\f126";
}

.fa.fa-chain-broken {
  --fa: "\f127";
}

.fa.fa-unlink {
  --fa: "\f127";
}

.fa.fa-calendar-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-o {
  --fa: "\f133";
}

.fa.fa-maxcdn {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-html5 {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-css3 {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-unlock-alt {
  --fa: "\f09c";
}

.fa.fa-minus-square-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-minus-square-o {
  --fa: "\f146";
}

.fa.fa-level-up {
  --fa: "\f3bf";
}

.fa.fa-level-down {
  --fa: "\f3be";
}

.fa.fa-pencil-square {
  --fa: "\f14b";
}

.fa.fa-external-link-square {
  --fa: "\f360";
}

.fa.fa-compass {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-down {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-down {
  --fa: "\f150";
}

.fa.fa-toggle-down {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-toggle-down {
  --fa: "\f150";
}

.fa.fa-caret-square-o-up {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-up {
  --fa: "\f151";
}

.fa.fa-toggle-up {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-toggle-up {
  --fa: "\f151";
}

.fa.fa-caret-square-o-right {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-right {
  --fa: "\f152";
}

.fa.fa-toggle-right {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-toggle-right {
  --fa: "\f152";
}

.fa.fa-eur {
  --fa: "\f153";
}

.fa.fa-euro {
  --fa: "\f153";
}

.fa.fa-gbp {
  --fa: "\f154";
}

.fa.fa-usd {
  --fa: "\$";
}

.fa.fa-dollar {
  --fa: "\$";
}

.fa.fa-inr {
  --fa: "\e1bc";
}

.fa.fa-rupee {
  --fa: "\e1bc";
}

.fa.fa-jpy {
  --fa: "\f157";
}

.fa.fa-cny {
  --fa: "\f157";
}

.fa.fa-rmb {
  --fa: "\f157";
}

.fa.fa-yen {
  --fa: "\f157";
}

.fa.fa-rub {
  --fa: "\f158";
}

.fa.fa-ruble {
  --fa: "\f158";
}

.fa.fa-rouble {
  --fa: "\f158";
}

.fa.fa-krw {
  --fa: "\f159";
}

.fa.fa-won {
  --fa: "\f159";
}

.fa.fa-btc {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-bitcoin {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-bitcoin {
  --fa: "\f15a";
}

.fa.fa-file-text {
  --fa: "\f15c";
}

.fa.fa-sort-alpha-asc {
  --fa: "\f15d";
}

.fa.fa-sort-alpha-desc {
  --fa: "\f881";
}

.fa.fa-sort-amount-asc {
  --fa: "\f884";
}

.fa.fa-sort-amount-desc {
  --fa: "\f160";
}

.fa.fa-sort-numeric-asc {
  --fa: "\f162";
}

.fa.fa-sort-numeric-desc {
  --fa: "\f886";
}

.fa.fa-youtube-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-youtube-square {
  --fa: "\f431";
}

.fa.fa-youtube {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-xing {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-xing-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-xing-square {
  --fa: "\f169";
}

.fa.fa-youtube-play {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-youtube-play {
  --fa: "\f167";
}

.fa.fa-dropbox {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-stack-overflow {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-instagram {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-flickr {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-adn {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-bitbucket {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-bitbucket-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-bitbucket-square {
  --fa: "\f171";
}

.fa.fa-tumblr {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-tumblr-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-tumblr-square {
  --fa: "\f174";
}

.fa.fa-long-arrow-down {
  --fa: "\f309";
}

.fa.fa-long-arrow-up {
  --fa: "\f30c";
}

.fa.fa-long-arrow-left {
  --fa: "\f30a";
}

.fa.fa-long-arrow-right {
  --fa: "\f30b";
}

.fa.fa-apple {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-windows {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-android {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-linux {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-dribbble {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-skype {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-foursquare {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-trello {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-gratipay {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-gittip {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-gittip {
  --fa: "\f184";
}

.fa.fa-sun-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-sun-o {
  --fa: "\f185";
}

.fa.fa-moon-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-moon-o {
  --fa: "\f186";
}

.fa.fa-vk {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-weibo {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-renren {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pagelines {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-stack-exchange {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-right {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-right {
  --fa: "\f35a";
}

.fa.fa-arrow-circle-o-left {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-arrow-circle-o-left {
  --fa: "\f359";
}

.fa.fa-caret-square-o-left {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-caret-square-o-left {
  --fa: "\f191";
}

.fa.fa-toggle-left {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-toggle-left {
  --fa: "\f191";
}

.fa.fa-dot-circle-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-dot-circle-o {
  --fa: "\f192";
}

.fa.fa-vimeo-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-vimeo-square {
  --fa: "\f194";
}

.fa.fa-try {
  --fa: "\e2bb";
}

.fa.fa-turkish-lira {
  --fa: "\e2bb";
}

.fa.fa-plus-square-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-plus-square-o {
  --fa: "\f0fe";
}

.fa.fa-slack {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-wordpress {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-openid {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-institution {
  --fa: "\f19c";
}

.fa.fa-bank {
  --fa: "\f19c";
}

.fa.fa-mortar-board {
  --fa: "\f19d";
}

.fa.fa-yahoo {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-reddit {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-reddit-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-reddit-square {
  --fa: "\f1a2";
}

.fa.fa-stumbleupon-circle {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-stumbleupon {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-delicious {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-digg {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pied-piper-pp {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pied-piper-alt {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-drupal {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-joomla {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-behance {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-behance-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-behance-square {
  --fa: "\f1b5";
}

.fa.fa-steam {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-steam-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-steam-square {
  --fa: "\f1b7";
}

.fa.fa-automobile {
  --fa: "\f1b9";
}

.fa.fa-cab {
  --fa: "\f1ba";
}

.fa.fa-spotify {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-deviantart {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-soundcloud {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-file-pdf-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-pdf-o {
  --fa: "\f1c1";
}

.fa.fa-file-word-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-word-o {
  --fa: "\f1c2";
}

.fa.fa-file-excel-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-excel-o {
  --fa: "\f1c3";
}

.fa.fa-file-powerpoint-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-powerpoint-o {
  --fa: "\f1c4";
}

.fa.fa-file-image-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-image-o {
  --fa: "\f1c5";
}

.fa.fa-file-photo-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-photo-o {
  --fa: "\f1c5";
}

.fa.fa-file-picture-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-picture-o {
  --fa: "\f1c5";
}

.fa.fa-file-archive-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-archive-o {
  --fa: "\f1c6";
}

.fa.fa-file-zip-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-zip-o {
  --fa: "\f1c6";
}

.fa.fa-file-audio-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-audio-o {
  --fa: "\f1c7";
}

.fa.fa-file-sound-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-sound-o {
  --fa: "\f1c7";
}

.fa.fa-file-video-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-video-o {
  --fa: "\f1c8";
}

.fa.fa-file-movie-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-movie-o {
  --fa: "\f1c8";
}

.fa.fa-file-code-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-file-code-o {
  --fa: "\f1c9";
}

.fa.fa-vine {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-codepen {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-jsfiddle {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-life-bouy {
  --fa: "\f1cd";
}

.fa.fa-life-buoy {
  --fa: "\f1cd";
}

.fa.fa-life-saver {
  --fa: "\f1cd";
}

.fa.fa-support {
  --fa: "\f1cd";
}

.fa.fa-circle-o-notch {
  --fa: "\f1ce";
}

.fa.fa-rebel {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-ra {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-ra {
  --fa: "\f1d0";
}

.fa.fa-resistance {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-resistance {
  --fa: "\f1d0";
}

.fa.fa-empire {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-ge {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-ge {
  --fa: "\f1d1";
}

.fa.fa-git-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-git-square {
  --fa: "\f1d2";
}

.fa.fa-git {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-hacker-news {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-y-combinator-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-y-combinator-square {
  --fa: "\f1d4";
}

.fa.fa-yc-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-yc-square {
  --fa: "\f1d4";
}

.fa.fa-tencent-weibo {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-qq {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-weixin {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-wechat {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-wechat {
  --fa: "\f1d7";
}

.fa.fa-send {
  --fa: "\f1d8";
}

.fa.fa-paper-plane-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-paper-plane-o {
  --fa: "\f1d8";
}

.fa.fa-send-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-send-o {
  --fa: "\f1d8";
}

.fa.fa-circle-thin {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-circle-thin {
  --fa: "\f111";
}

.fa.fa-header {
  --fa: "\f1dc";
}

.fa.fa-futbol-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-futbol-o {
  --fa: "\f1e3";
}

.fa.fa-soccer-ball-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-soccer-ball-o {
  --fa: "\f1e3";
}

.fa.fa-slideshare {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-twitch {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-yelp {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-newspaper-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-newspaper-o {
  --fa: "\f1ea";
}

.fa.fa-paypal {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-wallet {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-cc-visa {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-cc-mastercard {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-cc-discover {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-cc-amex {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-cc-paypal {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-cc-stripe {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-bell-slash-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-bell-slash-o {
  --fa: "\f1f6";
}

.fa.fa-trash {
  --fa: "\f2ed";
}

.fa.fa-copyright {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-eyedropper {
  --fa: "\f1fb";
}

.fa.fa-area-chart {
  --fa: "\f1fe";
}

.fa.fa-pie-chart {
  --fa: "\f200";
}

.fa.fa-line-chart {
  --fa: "\f201";
}

.fa.fa-lastfm {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-lastfm-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-lastfm-square {
  --fa: "\f203";
}

.fa.fa-ioxhost {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-angellist {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-cc {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-cc {
  --fa: "\f20a";
}

.fa.fa-ils {
  --fa: "\f20b";
}

.fa.fa-shekel {
  --fa: "\f20b";
}

.fa.fa-sheqel {
  --fa: "\f20b";
}

.fa.fa-buysellads {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-connectdevelop {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-dashcube {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-forumbee {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-leanpub {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-sellsy {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-shirtsinbulk {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-simplybuilt {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-skyatlas {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-diamond {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-diamond {
  --fa: "\f3a5";
}

.fa.fa-transgender {
  --fa: "\f224";
}

.fa.fa-intersex {
  --fa: "\f224";
}

.fa.fa-transgender-alt {
  --fa: "\f225";
}

.fa.fa-facebook-official {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-facebook-official {
  --fa: "\f09a";
}

.fa.fa-pinterest-p {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-whatsapp {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-hotel {
  --fa: "\f236";
}

.fa.fa-viacoin {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-medium {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-y-combinator {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-yc {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-yc {
  --fa: "\f23b";
}

.fa.fa-optin-monster {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-opencart {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-expeditedssl {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-battery-4 {
  --fa: "\f240";
}

.fa.fa-battery {
  --fa: "\f240";
}

.fa.fa-battery-3 {
  --fa: "\f241";
}

.fa.fa-battery-2 {
  --fa: "\f242";
}

.fa.fa-battery-1 {
  --fa: "\f243";
}

.fa.fa-battery-0 {
  --fa: "\f244";
}

.fa.fa-object-group {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-object-ungroup {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-sticky-note-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-sticky-note-o {
  --fa: "\f249";
}

.fa.fa-cc-jcb {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-cc-diners-club {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-clone {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hourglass-o {
  --fa: "\f254";
}

.fa.fa-hourglass-1 {
  --fa: "\f251";
}

.fa.fa-hourglass-2 {
  --fa: "\f252";
}

.fa.fa-hourglass-3 {
  --fa: "\f253";
}

.fa.fa-hand-rock-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-rock-o {
  --fa: "\f255";
}

.fa.fa-hand-grab-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-grab-o {
  --fa: "\f255";
}

.fa.fa-hand-paper-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-paper-o {
  --fa: "\f256";
}

.fa.fa-hand-stop-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-stop-o {
  --fa: "\f256";
}

.fa.fa-hand-scissors-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-scissors-o {
  --fa: "\f257";
}

.fa.fa-hand-lizard-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-lizard-o {
  --fa: "\f258";
}

.fa.fa-hand-spock-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-spock-o {
  --fa: "\f259";
}

.fa.fa-hand-pointer-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-pointer-o {
  --fa: "\f25a";
}

.fa.fa-hand-peace-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-hand-peace-o {
  --fa: "\f25b";
}

.fa.fa-registered {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-creative-commons {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-gg {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-gg-circle {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-odnoklassniki {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-odnoklassniki-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-odnoklassniki-square {
  --fa: "\f264";
}

.fa.fa-get-pocket {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-wikipedia-w {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-safari {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-chrome {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-firefox {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-opera {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-internet-explorer {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-television {
  --fa: "\f26c";
}

.fa.fa-contao {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-500px {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-amazon {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-calendar-plus-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-plus-o {
  --fa: "\f271";
}

.fa.fa-calendar-minus-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-minus-o {
  --fa: "\f272";
}

.fa.fa-calendar-times-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-times-o {
  --fa: "\f273";
}

.fa.fa-calendar-check-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-calendar-check-o {
  --fa: "\f274";
}

.fa.fa-map-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-map-o {
  --fa: "\f279";
}

.fa.fa-commenting {
  --fa: "\f4ad";
}

.fa.fa-commenting-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-commenting-o {
  --fa: "\f4ad";
}

.fa.fa-houzz {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-vimeo {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-vimeo {
  --fa: "\f27d";
}

.fa.fa-black-tie {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-fonticons {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-reddit-alien {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-edge {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-credit-card-alt {
  --fa: "\f09d";
}

.fa.fa-codiepie {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-modx {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-fort-awesome {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-usb {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-product-hunt {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-mixcloud {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-scribd {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-pause-circle-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-pause-circle-o {
  --fa: "\f28b";
}

.fa.fa-stop-circle-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-stop-circle-o {
  --fa: "\f28d";
}

.fa.fa-bluetooth {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-bluetooth-b {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-gitlab {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-wpbeginner {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-wpforms {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-envira {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-wheelchair-alt {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-wheelchair-alt {
  --fa: "\f368";
}

.fa.fa-question-circle-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-question-circle-o {
  --fa: "\f059";
}

.fa.fa-volume-control-phone {
  --fa: "\f2a0";
}

.fa.fa-asl-interpreting {
  --fa: "\f2a3";
}

.fa.fa-deafness {
  --fa: "\f2a4";
}

.fa.fa-hard-of-hearing {
  --fa: "\f2a4";
}

.fa.fa-glide {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-glide-g {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-signing {
  --fa: "\f2a7";
}

.fa.fa-viadeo {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-viadeo-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-viadeo-square {
  --fa: "\f2aa";
}

.fa.fa-snapchat {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-snapchat-ghost {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-snapchat-ghost {
  --fa: "\f2ab";
}

.fa.fa-snapchat-square {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-snapchat-square {
  --fa: "\f2ad";
}

.fa.fa-pied-piper {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-first-order {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-yoast {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-themeisle {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus-official {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus-official {
  --fa: "\f2b3";
}

.fa.fa-google-plus-circle {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-google-plus-circle {
  --fa: "\f2b3";
}

.fa.fa-font-awesome {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-fa {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-fa {
  --fa: "\f2b4";
}

.fa.fa-handshake-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-handshake-o {
  --fa: "\f2b5";
}

.fa.fa-envelope-open-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-envelope-open-o {
  --fa: "\f2b6";
}

.fa.fa-linode {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-address-book-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-address-book-o {
  --fa: "\f2b9";
}

.fa.fa-vcard {
  --fa: "\f2bb";
}

.fa.fa-address-card-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-address-card-o {
  --fa: "\f2bb";
}

.fa.fa-vcard-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-vcard-o {
  --fa: "\f2bb";
}

.fa.fa-user-circle-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-user-circle-o {
  --fa: "\f2bd";
}

.fa.fa-user-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-user-o {
  --fa: "\f007";
}

.fa.fa-id-badge {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-drivers-license {
  --fa: "\f2c2";
}

.fa.fa-id-card-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-id-card-o {
  --fa: "\f2c2";
}

.fa.fa-drivers-license-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-drivers-license-o {
  --fa: "\f2c2";
}

.fa.fa-quora {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-free-code-camp {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-telegram {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-thermometer-4 {
  --fa: "\f2c7";
}

.fa.fa-thermometer {
  --fa: "\f2c7";
}

.fa.fa-thermometer-3 {
  --fa: "\f2c8";
}

.fa.fa-thermometer-2 {
  --fa: "\f2c9";
}

.fa.fa-thermometer-1 {
  --fa: "\f2ca";
}

.fa.fa-thermometer-0 {
  --fa: "\f2cb";
}

.fa.fa-bathtub {
  --fa: "\f2cd";
}

.fa.fa-s15 {
  --fa: "\f2cd";
}

.fa.fa-window-maximize {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-window-restore {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-times-rectangle {
  --fa: "\f410";
}

.fa.fa-window-close-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-window-close-o {
  --fa: "\f410";
}

.fa.fa-times-rectangle-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-times-rectangle-o {
  --fa: "\f410";
}

.fa.fa-bandcamp {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-grav {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-etsy {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-imdb {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-ravelry {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-eercast {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-eercast {
  --fa: "\f2da";
}

.fa.fa-snowflake-o {
  font-family: "Font Awesome 6 Free";
  font-weight: 400;
}

.fa.fa-snowflake-o {
  --fa: "\f2dc";
}

.fa.fa-superpowers {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-wpexplorer {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

.fa.fa-meetup {
  font-family: "Font Awesome 6 Brands";
  font-weight: 400;
}

/*!
 * Font Awesome Free 6.7.2 by @fontawesome - https://fontawesome.com
 * License - https://fontawesome.com/license/free (Icons: CC BY 4.0, Fonts: SIL OFL 1.1, Code: MIT License)
 * Copyright 2024 Fonticons, Inc.
 */
.fa {
  font-family: var(--fa-style-family, "Font Awesome 6 Free");
  font-weight: var(--fa-style, 900);
}

.fas,
.far,
.fab,
.fa-solid,
.fa-regular,
.fa-brands,
.fa {
  -moz-osx-font-smoothing: grayscale;
  -webkit-font-smoothing: antialiased;
  display: var(--fa-display, inline-block);
  font-style: normal;
  font-variant: normal;
  line-height: 1;
  text-rendering: auto;
}

.fas::before,
.far::before,
.fab::before,
.fa-solid::before,
.fa-regular::before,
.fa-brands::before,
.fa::before {
  content: var(--fa);
}

.fa-classic,
.fas,
.fa-solid,
.far,
.fa-regular {
  font-family: "Font Awesome 6 Free";
}

.fa-brands,
.fab {
  font-family: "Font Awesome 6 Brands";
}

.fa-1x {
  font-size: 1em;
}

.fa-2x {
  font-size: 2em;
}

.fa-3x {
  font-size: 3em;
}

.fa-4x {
  font-size: 4em;
}

.fa-5x {
  font-size: 5em;
}

.fa-6x {
  font-size: 6em;
}

.fa-7x {
  font-size: 7em;
}

.fa-8x {
  font-size: 8em;
}

.fa-9x {
  font-size: 9em;
}

.fa-10x {
  font-size: 10em;
}

.fa-2xs {
  font-size: 0.625em;
  line-height: 0.1em;
  vertical-align: 0.225em;
}

.fa-xs {
  font-size: 0.75em;
  line-height: 0.0833333337em;
  vertical-align: 0.125em;
}

.fa-sm {
  font-size: 0.875em;
  line-height: 0.0714285718em;
  vertical-align: 0.0535714295em;
}

.fa-lg {
  font-size: 1.25em;
  line-height: 0.05em;
  vertical-align: -0.075em;
}

.fa-xl {
  font-size: 1.5em;
  line-height: 0.0416666682em;
  vertical-align: -0.125em;
}

.fa-2xl {
  font-size: 2em;
  line-height: 0.03125em;
  vertical-align: -0.1875em;
}

.fa-fw {
  text-align: center;
  width: 1.25em;
}

.fa-ul {
  list-style-type: none;
  margin-left: var(--fa-li-margin, 2.5em);
  padding-left: 0;
}
.fa-ul > li {
  position: relative;
}

.fa-li {
  left: calc(-1 * var(--fa-li-width, 2em));
  position: absolute;
  text-align: center;
  width: var(--fa-li-width, 2em);
  line-height: inherit;
}

.fa-border {
  border-color: var(--fa-border-color, #eee);
  border-radius: var(--fa-border-radius, 0.1em);
  border-style: var(--fa-border-style, solid);
  border-width: var(--fa-border-width, 0.08em);
  padding: var(--fa-border-padding, 0.2em 0.25em 0.15em);
}

.fa-pull-left {
  float: left;
  margin-right: var(--fa-pull-margin, 0.3em);
}

.fa-pull-right {
  float: right;
  margin-left: var(--fa-pull-margin, 0.3em);
}

.fa-beat {
  animation-name: fa-beat;
  animation-delay: var(--fa-animation-delay, 0s);
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 1s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, ease-in-out);
}

.fa-bounce {
  animation-name: fa-bounce;
  animation-delay: var(--fa-animation-delay, 0s);
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 1s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, cubic-bezier(0.28, 0.84, 0.42, 1));
}

.fa-fade {
  animation-name: fa-fade;
  animation-delay: var(--fa-animation-delay, 0s);
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 1s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, cubic-bezier(0.4, 0, 0.6, 1));
}

.fa-beat-fade {
  animation-name: fa-beat-fade;
  animation-delay: var(--fa-animation-delay, 0s);
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 1s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, cubic-bezier(0.4, 0, 0.6, 1));
}

.fa-flip {
  animation-name: fa-flip;
  animation-delay: var(--fa-animation-delay, 0s);
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 1s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, ease-in-out);
}

.fa-shake {
  animation-name: fa-shake;
  animation-delay: var(--fa-animation-delay, 0s);
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 1s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, linear);
}

.fa-spin {
  animation-name: fa-spin;
  animation-delay: var(--fa-animation-delay, 0s);
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 2s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, linear);
}

.fa-spin-reverse {
  --fa-animation-direction: reverse;
}

.fa-pulse,
.fa-spin-pulse {
  animation-name: fa-spin;
  animation-direction: var(--fa-animation-direction, normal);
  animation-duration: var(--fa-animation-duration, 1s);
  animation-iteration-count: var(--fa-animation-iteration-count, infinite);
  animation-timing-function: var(--fa-animation-timing, steps(8));
}

@media (prefers-reduced-motion: reduce) {
  .fa-beat,
  .fa-bounce,
  .fa-fade,
  .fa-beat-fade,
  .fa-flip,
  .fa-pulse,
  .fa-shake,
  .fa-spin,
  .fa-spin-pulse {
    animation-delay: -1ms;
    animation-duration: 1ms;
    animation-iteration-count: 1;
    transition-delay: 0s;
    transition-duration: 0s;
  }
}
@keyframes fa-beat {
  0%, 90% {
    transform: scale(1);
  }
  45% {
    transform: scale(var(--fa-beat-scale, 1.25));
  }
}
@keyframes fa-bounce {
  0% {
    transform: scale(1, 1) translateY(0);
  }
  10% {
    transform: scale(var(--fa-bounce-start-scale-x, 1.1), var(--fa-bounce-start-scale-y, 0.9)) translateY(0);
  }
  30% {
    transform: scale(var(--fa-bounce-jump-scale-x, 0.9), var(--fa-bounce-jump-scale-y, 1.1)) translateY(var(--fa-bounce-height, -0.5em));
  }
  50% {
    transform: scale(var(--fa-bounce-land-scale-x, 1.05), var(--fa-bounce-land-scale-y, 0.95)) translateY(0);
  }
  57% {
    transform: scale(1, 1) translateY(var(--fa-bounce-rebound, -0.125em));
  }
  64% {
    transform: scale(1, 1) translateY(0);
  }
  100% {
    transform: scale(1, 1) translateY(0);
  }
}
@keyframes fa-fade {
  50% {
    opacity: var(--fa-fade-opacity, 0.4);
  }
}
@keyframes fa-beat-fade {
  0%, 100% {
    opacity: var(--fa-beat-fade-opacity, 0.4);
    transform: scale(1);
  }
  50% {
    opacity: 1;
    transform: scale(var(--fa-beat-fade-scale, 1.125));
  }
}
@keyframes fa-flip {
  50% {
    transform: rotate3d(var(--fa-flip-x, 0), var(--fa-flip-y, 1), var(--fa-flip-z, 0), var(--fa-flip-angle, -180deg));
  }
}
@keyframes fa-shake {
  0% {
    transform: rotate(-15deg);
  }
  4% {
    transform: rotate(15deg);
  }
  8%, 24% {
    transform: rotate(-18deg);
  }
  12%, 28% {
    transform: rotate(18deg);
  }
  16% {
    transform: rotate(-22deg);
  }
  20% {
    transform: rotate(22deg);
  }
  32% {
    transform: rotate(-12deg);
  }
  36% {
    transform: rotate(12deg);
  }
  40%, 100% {
    transform: rotate(0deg);
  }
}
@keyframes fa-spin {
  0% {
    transform: rotate(0deg);
  }
  100% {
    transform: rotate(360deg);
  }
}
.fa-rotate-90 {
  transform: rotate(90deg);
}

.fa-rotate-180 {
  transform: rotate(180deg);
}

.fa-rotate-270 {
  transform: rotate(270deg);
}

.fa-flip-horizontal {
  transform: scale(-1, 1);
}

.fa-flip-vertical {
  transform: scale(1, -1);
}

.fa-flip-both,
.fa-flip-horizontal.fa-flip-vertical {
  transform: scale(-1, -1);
}

.fa-rotate-by {
  transform: rotate(var(--fa-rotate-angle, 0));
}

.fa-stack {
  display: inline-block;
  height: 2em;
  line-height: 2em;
  position: relative;
  vertical-align: middle;
  width: 2.5em;
}

.fa-stack-1x,
.fa-stack-2x {
  left: 0;
  position: absolute;
  text-align: center;
  width: 100%;
  z-index: var(--fa-stack-z-index, auto);
}

.fa-stack-1x {
  line-height: inherit;
}

.fa-stack-2x {
  font-size: 2em;
}

.fa-inverse {
  color: var(--fa-inverse, #fff);
}

/* Font Awesome uses the Unicode Private Use Area (PUA) to ensure screen
readers do not read off random characters that represent icons */
.fa-0 {
  --fa: "\30 ";
}

.fa-1 {
  --fa: "\31 ";
}

.fa-2 {
  --fa: "\32 ";
}

.fa-3 {
  --fa: "\33 ";
}

.fa-4 {
  --fa: "\34 ";
}

.fa-5 {
  --fa: "\35 ";
}

.fa-6 {
  --fa: "\36 ";
}

.fa-7 {
  --fa: "\37 ";
}

.fa-8 {
  --fa: "\38 ";
}

.fa-9 {
  --fa: "\39 ";
}

.fa-fill-drip {
  --fa: "\f576";
}

.fa-arrows-to-circle {
  --fa: "\e4bd";
}

.fa-circle-chevron-right {
  --fa: "\f138";
}

.fa-chevron-circle-right {
  --fa: "\f138";
}

.fa-at {
  --fa: "\@";
}

.fa-trash-can {
  --fa: "\f2ed";
}

.fa-trash-alt {
  --fa: "\f2ed";
}

.fa-text-height {
  --fa: "\f034";
}

.fa-user-xmark {
  --fa: "\f235";
}

.fa-user-times {
  --fa: "\f235";
}

.fa-stethoscope {
  --fa: "\f0f1";
}

.fa-message {
  --fa: "\f27a";
}

.fa-comment-alt {
  --fa: "\f27a";
}

.fa-info {
  --fa: "\f129";
}

.fa-down-left-and-up-right-to-center {
  --fa: "\f422";
}

.fa-compress-alt {
  --fa: "\f422";
}

.fa-explosion {
  --fa: "\e4e9";
}

.fa-file-lines {
  --fa: "\f15c";
}

.fa-file-alt {
  --fa: "\f15c";
}

.fa-file-text {
  --fa: "\f15c";
}

.fa-wave-square {
  --fa: "\f83e";
}

.fa-ring {
  --fa: "\f70b";
}

.fa-building-un {
  --fa: "\e4d9";
}

.fa-dice-three {
  --fa: "\f527";
}

.fa-calendar-days {
  --fa: "\f073";
}

.fa-calendar-alt {
  --fa: "\f073";
}

.fa-anchor-circle-check {
  --fa: "\e4aa";
}

.fa-building-circle-arrow-right {
  --fa: "\e4d1";
}

.fa-volleyball {
  --fa: "\f45f";
}

.fa-volleyball-ball {
  --fa: "\f45f";
}

.fa-arrows-up-to-line {
  --fa: "\e4c2";
}

.fa-sort-down {
  --fa: "\f0dd";
}

.fa-sort-desc {
  --fa: "\f0dd";
}

.fa-circle-minus {
  --fa: "\f056";
}

.fa-minus-circle {
  --fa: "\f056";
}

.fa-door-open {
  --fa: "\f52b";
}

.fa-right-from-bracket {
  --fa: "\f2f5";
}

.fa-sign-out-alt {
  --fa: "\f2f5";
}

.fa-atom {
  --fa: "\f5d2";
}

.fa-soap {
  --fa: "\e06e";
}

.fa-icons {
  --fa: "\f86d";
}

.fa-heart-music-camera-bolt {
  --fa: "\f86d";
}

.fa-microphone-lines-slash {
  --fa: "\f539";
}

.fa-microphone-alt-slash {
  --fa: "\f539";
}

.fa-bridge-circle-check {
  --fa: "\e4c9";
}

.fa-pump-medical {
  --fa: "\e06a";
}

.fa-fingerprint {
  --fa: "\f577";
}

.fa-hand-point-right {
  --fa: "\f0a4";
}

.fa-magnifying-glass-location {
  --fa: "\f689";
}

.fa-search-location {
  --fa: "\f689";
}

.fa-forward-step {
  --fa: "\f051";
}

.fa-step-forward {
  --fa: "\f051";
}

.fa-face-smile-beam {
  --fa: "\f5b8";
}

.fa-smile-beam {
  --fa: "\f5b8";
}

.fa-flag-checkered {
  --fa: "\f11e";
}

.fa-football {
  --fa: "\f44e";
}

.fa-football-ball {
  --fa: "\f44e";
}

.fa-school-circle-exclamation {
  --fa: "\e56c";
}

.fa-crop {
  --fa: "\f125";
}

.fa-angles-down {
  --fa: "\f103";
}

.fa-angle-double-down {
  --fa: "\f103";
}

.fa-users-rectangle {
  --fa: "\e594";
}

.fa-people-roof {
  --fa: "\e537";
}

.fa-people-line {
  --fa: "\e534";
}

.fa-beer-mug-empty {
  --fa: "\f0fc";
}

.fa-beer {
  --fa: "\f0fc";
}

.fa-diagram-predecessor {
  --fa: "\e477";
}

.fa-arrow-up-long {
  --fa: "\f176";
}

.fa-long-arrow-up {
  --fa: "\f176";
}

.fa-fire-flame-simple {
  --fa: "\f46a";
}

.fa-burn {
  --fa: "\f46a";
}

.fa-person {
  --fa: "\f183";
}

.fa-male {
  --fa: "\f183";
}

.fa-laptop {
  --fa: "\f109";
}

.fa-file-csv {
  --fa: "\f6dd";
}

.fa-menorah {
  --fa: "\f676";
}

.fa-truck-plane {
  --fa: "\e58f";
}

.fa-record-vinyl {
  --fa: "\f8d9";
}

.fa-face-grin-stars {
  --fa: "\f587";
}

.fa-grin-stars {
  --fa: "\f587";
}

.fa-bong {
  --fa: "\f55c";
}

.fa-spaghetti-monster-flying {
  --fa: "\f67b";
}

.fa-pastafarianism {
  --fa: "\f67b";
}

.fa-arrow-down-up-across-line {
  --fa: "\e4af";
}

.fa-spoon {
  --fa: "\f2e5";
}

.fa-utensil-spoon {
  --fa: "\f2e5";
}

.fa-jar-wheat {
  --fa: "\e517";
}

.fa-envelopes-bulk {
  --fa: "\f674";
}

.fa-mail-bulk {
  --fa: "\f674";
}

.fa-file-circle-exclamation {
  --fa: "\e4eb";
}

.fa-circle-h {
  --fa: "\f47e";
}

.fa-hospital-symbol {
  --fa: "\f47e";
}

.fa-pager {
  --fa: "\f815";
}

.fa-address-book {
  --fa: "\f2b9";
}

.fa-contact-book {
  --fa: "\f2b9";
}

.fa-strikethrough {
  --fa: "\f0cc";
}

.fa-k {
  --fa: "K";
}

.fa-landmark-flag {
  --fa: "\e51c";
}

.fa-pencil {
  --fa: "\f303";
}

.fa-pencil-alt {
  --fa: "\f303";
}

.fa-backward {
  --fa: "\f04a";
}

.fa-caret-right {
  --fa: "\f0da";
}

.fa-comments {
  --fa: "\f086";
}

.fa-paste {
  --fa: "\f0ea";
}

.fa-file-clipboard {
  --fa: "\f0ea";
}

.fa-code-pull-request {
  --fa: "\e13c";
}

.fa-clipboard-list {
  --fa: "\f46d";
}

.fa-truck-ramp-box {
  --fa: "\f4de";
}

.fa-truck-loading {
  --fa: "\f4de";
}

.fa-user-check {
  --fa: "\f4fc";
}

.fa-vial-virus {
  --fa: "\e597";
}

.fa-sheet-plastic {
  --fa: "\e571";
}

.fa-blog {
  --fa: "\f781";
}

.fa-user-ninja {
  --fa: "\f504";
}

.fa-person-arrow-up-from-line {
  --fa: "\e539";
}

.fa-scroll-torah {
  --fa: "\f6a0";
}

.fa-torah {
  --fa: "\f6a0";
}

.fa-broom-ball {
  --fa: "\f458";
}

.fa-quidditch {
  --fa: "\f458";
}

.fa-quidditch-broom-ball {
  --fa: "\f458";
}

.fa-toggle-off {
  --fa: "\f204";
}

.fa-box-archive {
  --fa: "\f187";
}

.fa-archive {
  --fa: "\f187";
}

.fa-person-drowning {
  --fa: "\e545";
}

.fa-arrow-down-9-1 {
  --fa: "\f886";
}

.fa-sort-numeric-desc {
  --fa: "\f886";
}

.fa-sort-numeric-down-alt {
  --fa: "\f886";
}

.fa-face-grin-tongue-squint {
  --fa: "\f58a";
}

.fa-grin-tongue-squint {
  --fa: "\f58a";
}

.fa-spray-can {
  --fa: "\f5bd";
}

.fa-truck-monster {
  --fa: "\f63b";
}

.fa-w {
  --fa: "W";
}

.fa-earth-africa {
  --fa: "\f57c";
}

.fa-globe-africa {
  --fa: "\f57c";
}

.fa-rainbow {
  --fa: "\f75b";
}

.fa-circle-notch {
  --fa: "\f1ce";
}

.fa-tablet-screen-button {
  --fa: "\f3fa";
}

.fa-tablet-alt {
  --fa: "\f3fa";
}

.fa-paw {
  --fa: "\f1b0";
}

.fa-cloud {
  --fa: "\f0c2";
}

.fa-trowel-bricks {
  --fa: "\e58a";
}

.fa-face-flushed {
  --fa: "\f579";
}

.fa-flushed {
  --fa: "\f579";
}

.fa-hospital-user {
  --fa: "\f80d";
}

.fa-tent-arrow-left-right {
  --fa: "\e57f";
}

.fa-gavel {
  --fa: "\f0e3";
}

.fa-legal {
  --fa: "\f0e3";
}

.fa-binoculars {
  --fa: "\f1e5";
}

.fa-microphone-slash {
  --fa: "\f131";
}

.fa-box-tissue {
  --fa: "\e05b";
}

.fa-motorcycle {
  --fa: "\f21c";
}

.fa-bell-concierge {
  --fa: "\f562";
}

.fa-concierge-bell {
  --fa: "\f562";
}

.fa-pen-ruler {
  --fa: "\f5ae";
}

.fa-pencil-ruler {
  --fa: "\f5ae";
}

.fa-people-arrows {
  --fa: "\e068";
}

.fa-people-arrows-left-right {
  --fa: "\e068";
}

.fa-mars-and-venus-burst {
  --fa: "\e523";
}

.fa-square-caret-right {
  --fa: "\f152";
}

.fa-caret-square-right {
  --fa: "\f152";
}

.fa-scissors {
  --fa: "\f0c4";
}

.fa-cut {
  --fa: "\f0c4";
}

.fa-sun-plant-wilt {
  --fa: "\e57a";
}

.fa-toilets-portable {
  --fa: "\e584";
}

.fa-hockey-puck {
  --fa: "\f453";
}

.fa-table {
  --fa: "\f0ce";
}

.fa-magnifying-glass-arrow-right {
  --fa: "\e521";
}

.fa-tachograph-digital {
  --fa: "\f566";
}

.fa-digital-tachograph {
  --fa: "\f566";
}

.fa-users-slash {
  --fa: "\e073";
}

.fa-clover {
  --fa: "\e139";
}

.fa-reply {
  --fa: "\f3e5";
}

.fa-mail-reply {
  --fa: "\f3e5";
}

.fa-star-and-crescent {
  --fa: "\f699";
}

.fa-house-fire {
  --fa: "\e50c";
}

.fa-square-minus {
  --fa: "\f146";
}

.fa-minus-square {
  --fa: "\f146";
}

.fa-helicopter {
  --fa: "\f533";
}

.fa-compass {
  --fa: "\f14e";
}

.fa-square-caret-down {
  --fa: "\f150";
}

.fa-caret-square-down {
  --fa: "\f150";
}

.fa-file-circle-question {
  --fa: "\e4ef";
}

.fa-laptop-code {
  --fa: "\f5fc";
}

.fa-swatchbook {
  --fa: "\f5c3";
}

.fa-prescription-bottle {
  --fa: "\f485";
}

.fa-bars {
  --fa: "\f0c9";
}

.fa-navicon {
  --fa: "\f0c9";
}

.fa-people-group {
  --fa: "\e533";
}

.fa-hourglass-end {
  --fa: "\f253";
}

.fa-hourglass-3 {
  --fa: "\f253";
}

.fa-heart-crack {
  --fa: "\f7a9";
}

.fa-heart-broken {
  --fa: "\f7a9";
}

.fa-square-up-right {
  --fa: "\f360";
}

.fa-external-link-square-alt {
  --fa: "\f360";
}

.fa-face-kiss-beam {
  --fa: "\f597";
}

.fa-kiss-beam {
  --fa: "\f597";
}

.fa-film {
  --fa: "\f008";
}

.fa-ruler-horizontal {
  --fa: "\f547";
}

.fa-people-robbery {
  --fa: "\e536";
}

.fa-lightbulb {
  --fa: "\f0eb";
}

.fa-caret-left {
  --fa: "\f0d9";
}

.fa-circle-exclamation {
  --fa: "\f06a";
}

.fa-exclamation-circle {
  --fa: "\f06a";
}

.fa-school-circle-xmark {
  --fa: "\e56d";
}

.fa-arrow-right-from-bracket {
  --fa: "\f08b";
}

.fa-sign-out {
  --fa: "\f08b";
}

.fa-circle-chevron-down {
  --fa: "\f13a";
}

.fa-chevron-circle-down {
  --fa: "\f13a";
}

.fa-unlock-keyhole {
  --fa: "\f13e";
}

.fa-unlock-alt {
  --fa: "\f13e";
}

.fa-cloud-showers-heavy {
  --fa: "\f740";
}

.fa-headphones-simple {
  --fa: "\f58f";
}

.fa-headphones-alt {
  --fa: "\f58f";
}

.fa-sitemap {
  --fa: "\f0e8";
}

.fa-circle-dollar-to-slot {
  --fa: "\f4b9";
}

.fa-donate {
  --fa: "\f4b9";
}

.fa-memory {
  --fa: "\f538";
}

.fa-road-spikes {
  --fa: "\e568";
}

.fa-fire-burner {
  --fa: "\e4f1";
}

.fa-flag {
  --fa: "\f024";
}

.fa-hanukiah {
  --fa: "\f6e6";
}

.fa-feather {
  --fa: "\f52d";
}

.fa-volume-low {
  --fa: "\f027";
}

.fa-volume-down {
  --fa: "\f027";
}

.fa-comment-slash {
  --fa: "\f4b3";
}

.fa-cloud-sun-rain {
  --fa: "\f743";
}

.fa-compress {
  --fa: "\f066";
}

.fa-wheat-awn {
  --fa: "\e2cd";
}

.fa-wheat-alt {
  --fa: "\e2cd";
}

.fa-ankh {
  --fa: "\f644";
}

.fa-hands-holding-child {
  --fa: "\e4fa";
}

.fa-asterisk {
  --fa: "\*";
}

.fa-square-check {
  --fa: "\f14a";
}

.fa-check-square {
  --fa: "\f14a";
}

.fa-peseta-sign {
  --fa: "\e221";
}

.fa-heading {
  --fa: "\f1dc";
}

.fa-header {
  --fa: "\f1dc";
}

.fa-ghost {
  --fa: "\f6e2";
}

.fa-list {
  --fa: "\f03a";
}

.fa-list-squares {
  --fa: "\f03a";
}

.fa-square-phone-flip {
  --fa: "\f87b";
}

.fa-phone-square-alt {
  --fa: "\f87b";
}

.fa-cart-plus {
  --fa: "\f217";
}

.fa-gamepad {
  --fa: "\f11b";
}

.fa-circle-dot {
  --fa: "\f192";
}

.fa-dot-circle {
  --fa: "\f192";
}

.fa-face-dizzy {
  --fa: "\f567";
}

.fa-dizzy {
  --fa: "\f567";
}

.fa-egg {
  --fa: "\f7fb";
}

.fa-house-medical-circle-xmark {
  --fa: "\e513";
}

.fa-campground {
  --fa: "\f6bb";
}

.fa-folder-plus {
  --fa: "\f65e";
}

.fa-futbol {
  --fa: "\f1e3";
}

.fa-futbol-ball {
  --fa: "\f1e3";
}

.fa-soccer-ball {
  --fa: "\f1e3";
}

.fa-paintbrush {
  --fa: "\f1fc";
}

.fa-paint-brush {
  --fa: "\f1fc";
}

.fa-lock {
  --fa: "\f023";
}

.fa-gas-pump {
  --fa: "\f52f";
}

.fa-hot-tub-person {
  --fa: "\f593";
}

.fa-hot-tub {
  --fa: "\f593";
}

.fa-map-location {
  --fa: "\f59f";
}

.fa-map-marked {
  --fa: "\f59f";
}

.fa-house-flood-water {
  --fa: "\e50e";
}

.fa-tree {
  --fa: "\f1bb";
}

.fa-bridge-lock {
  --fa: "\e4cc";
}

.fa-sack-dollar {
  --fa: "\f81d";
}

.fa-pen-to-square {
  --fa: "\f044";
}

.fa-edit {
  --fa: "\f044";
}

.fa-car-side {
  --fa: "\f5e4";
}

.fa-share-nodes {
  --fa: "\f1e0";
}

.fa-share-alt {
  --fa: "\f1e0";
}

.fa-heart-circle-minus {
  --fa: "\e4ff";
}

.fa-hourglass-half {
  --fa: "\f252";
}

.fa-hourglass-2 {
  --fa: "\f252";
}

.fa-microscope {
  --fa: "\f610";
}

.fa-sink {
  --fa: "\e06d";
}

.fa-bag-shopping {
  --fa: "\f290";
}

.fa-shopping-bag {
  --fa: "\f290";
}

.fa-arrow-down-z-a {
  --fa: "\f881";
}

.fa-sort-alpha-desc {
  --fa: "\f881";
}

.fa-sort-alpha-down-alt {
  --fa: "\f881";
}

.fa-mitten {
  --fa: "\f7b5";
}

.fa-person-rays {
  --fa: "\e54d";
}

.fa-users {
  --fa: "\f0c0";
}

.fa-eye-slash {
  --fa: "\f070";
}

.fa-flask-vial {
  --fa: "\e4f3";
}

.fa-hand {
  --fa: "\f256";
}

.fa-hand-paper {
  --fa: "\f256";
}

.fa-om {
  --fa: "\f679";
}

.fa-worm {
  --fa: "\e599";
}

.fa-house-circle-xmark {
  --fa: "\e50b";
}

.fa-plug {
  --fa: "\f1e6";
}

.fa-chevron-up {
  --fa: "\f077";
}

.fa-hand-spock {
  --fa: "\f259";
}

.fa-stopwatch {
  --fa: "\f2f2";
}

.fa-face-kiss {
  --fa: "\f596";
}

.fa-kiss {
  --fa: "\f596";
}

.fa-bridge-circle-xmark {
  --fa: "\e4cb";
}

.fa-face-grin-tongue {
  --fa: "\f589";
}

.fa-grin-tongue {
  --fa: "\f589";
}

.fa-chess-bishop {
  --fa: "\f43a";
}

.fa-face-grin-wink {
  --fa: "\f58c";
}

.fa-grin-wink {
  --fa: "\f58c";
}

.fa-ear-deaf {
  --fa: "\f2a4";
}

.fa-deaf {
  --fa: "\f2a4";
}

.fa-deafness {
  --fa: "\f2a4";
}

.fa-hard-of-hearing {
  --fa: "\f2a4";
}

.fa-road-circle-check {
  --fa: "\e564";
}

.fa-dice-five {
  --fa: "\f523";
}

.fa-square-rss {
  --fa: "\f143";
}

.fa-rss-square {
  --fa: "\f143";
}

.fa-land-mine-on {
  --fa: "\e51b";
}

.fa-i-cursor {
  --fa: "\f246";
}

.fa-stamp {
  --fa: "\f5bf";
}

.fa-stairs {
  --fa: "\e289";
}

.fa-i {
  --fa: "I";
}

.fa-hryvnia-sign {
  --fa: "\f6f2";
}

.fa-hryvnia {
  --fa: "\f6f2";
}

.fa-pills {
  --fa: "\f484";
}

.fa-face-grin-wide {
  --fa: "\f581";
}

.fa-grin-alt {
  --fa: "\f581";
}

.fa-tooth {
  --fa: "\f5c9";
}

.fa-v {
  --fa: "V";
}

.fa-bangladeshi-taka-sign {
  --fa: "\e2e6";
}

.fa-bicycle {
  --fa: "\f206";
}

.fa-staff-snake {
  --fa: "\e579";
}

.fa-rod-asclepius {
  --fa: "\e579";
}

.fa-rod-snake {
  --fa: "\e579";
}

.fa-staff-aesculapius {
  --fa: "\e579";
}

.fa-head-side-cough-slash {
  --fa: "\e062";
}

.fa-truck-medical {
  --fa: "\f0f9";
}

.fa-ambulance {
  --fa: "\f0f9";
}

.fa-wheat-awn-circle-exclamation {
  --fa: "\e598";
}

.fa-snowman {
  --fa: "\f7d0";
}

.fa-mortar-pestle {
  --fa: "\f5a7";
}

.fa-road-barrier {
  --fa: "\e562";
}

.fa-school {
  --fa: "\f549";
}

.fa-igloo {
  --fa: "\f7ae";
}

.fa-joint {
  --fa: "\f595";
}

.fa-angle-right {
  --fa: "\f105";
}

.fa-horse {
  --fa: "\f6f0";
}

.fa-q {
  --fa: "Q";
}

.fa-g {
  --fa: "G";
}

.fa-notes-medical {
  --fa: "\f481";
}

.fa-temperature-half {
  --fa: "\f2c9";
}

.fa-temperature-2 {
  --fa: "\f2c9";
}

.fa-thermometer-2 {
  --fa: "\f2c9";
}

.fa-thermometer-half {
  --fa: "\f2c9";
}

.fa-dong-sign {
  --fa: "\e169";
}

.fa-capsules {
  --fa: "\f46b";
}

.fa-poo-storm {
  --fa: "\f75a";
}

.fa-poo-bolt {
  --fa: "\f75a";
}

.fa-face-frown-open {
  --fa: "\f57a";
}

.fa-frown-open {
  --fa: "\f57a";
}

.fa-hand-point-up {
  --fa: "\f0a6";
}

.fa-money-bill {
  --fa: "\f0d6";
}

.fa-bookmark {
  --fa: "\f02e";
}

.fa-align-justify {
  --fa: "\f039";
}

.fa-umbrella-beach {
  --fa: "\f5ca";
}

.fa-helmet-un {
  --fa: "\e503";
}

.fa-bullseye {
  --fa: "\f140";
}

.fa-bacon {
  --fa: "\f7e5";
}

.fa-hand-point-down {
  --fa: "\f0a7";
}

.fa-arrow-up-from-bracket {
  --fa: "\e09a";
}

.fa-folder {
  --fa: "\f07b";
}

.fa-folder-blank {
  --fa: "\f07b";
}

.fa-file-waveform {
  --fa: "\f478";
}

.fa-file-medical-alt {
  --fa: "\f478";
}

.fa-radiation {
  --fa: "\f7b9";
}

.fa-chart-simple {
  --fa: "\e473";
}

.fa-mars-stroke {
  --fa: "\f229";
}

.fa-vial {
  --fa: "\f492";
}

.fa-gauge {
  --fa: "\f624";
}

.fa-dashboard {
  --fa: "\f624";
}

.fa-gauge-med {
  --fa: "\f624";
}

.fa-tachometer-alt-average {
  --fa: "\f624";
}

.fa-wand-magic-sparkles {
  --fa: "\e2ca";
}

.fa-magic-wand-sparkles {
  --fa: "\e2ca";
}

.fa-e {
  --fa: "E";
}

.fa-pen-clip {
  --fa: "\f305";
}

.fa-pen-alt {
  --fa: "\f305";
}

.fa-bridge-circle-exclamation {
  --fa: "\e4ca";
}

.fa-user {
  --fa: "\f007";
}

.fa-school-circle-check {
  --fa: "\e56b";
}

.fa-dumpster {
  --fa: "\f793";
}

.fa-van-shuttle {
  --fa: "\f5b6";
}

.fa-shuttle-van {
  --fa: "\f5b6";
}

.fa-building-user {
  --fa: "\e4da";
}

.fa-square-caret-left {
  --fa: "\f191";
}

.fa-caret-square-left {
  --fa: "\f191";
}

.fa-highlighter {
  --fa: "\f591";
}

.fa-key {
  --fa: "\f084";
}

.fa-bullhorn {
  --fa: "\f0a1";
}

.fa-globe {
  --fa: "\f0ac";
}

.fa-synagogue {
  --fa: "\f69b";
}

.fa-person-half-dress {
  --fa: "\e548";
}

.fa-road-bridge {
  --fa: "\e563";
}

.fa-location-arrow {
  --fa: "\f124";
}

.fa-c {
  --fa: "C";
}

.fa-tablet-button {
  --fa: "\f10a";
}

.fa-building-lock {
  --fa: "\e4d6";
}

.fa-pizza-slice {
  --fa: "\f818";
}

.fa-money-bill-wave {
  --fa: "\f53a";
}

.fa-chart-area {
  --fa: "\f1fe";
}

.fa-area-chart {
  --fa: "\f1fe";
}

.fa-house-flag {
  --fa: "\e50d";
}

.fa-person-circle-minus {
  --fa: "\e540";
}

.fa-ban {
  --fa: "\f05e";
}

.fa-cancel {
  --fa: "\f05e";
}

.fa-camera-rotate {
  --fa: "\e0d8";
}

.fa-spray-can-sparkles {
  --fa: "\f5d0";
}

.fa-air-freshener {
  --fa: "\f5d0";
}

.fa-star {
  --fa: "\f005";
}

.fa-repeat {
  --fa: "\f363";
}

.fa-cross {
  --fa: "\f654";
}

.fa-box {
  --fa: "\f466";
}

.fa-venus-mars {
  --fa: "\f228";
}

.fa-arrow-pointer {
  --fa: "\f245";
}

.fa-mouse-pointer {
  --fa: "\f245";
}

.fa-maximize {
  --fa: "\f31e";
}

.fa-expand-arrows-alt {
  --fa: "\f31e";
}

.fa-charging-station {
  --fa: "\f5e7";
}

.fa-shapes {
  --fa: "\f61f";
}

.fa-triangle-circle-square {
  --fa: "\f61f";
}

.fa-shuffle {
  --fa: "\f074";
}

.fa-random {
  --fa: "\f074";
}

.fa-person-running {
  --fa: "\f70c";
}

.fa-running {
  --fa: "\f70c";
}

.fa-mobile-retro {
  --fa: "\e527";
}

.fa-grip-lines-vertical {
  --fa: "\f7a5";
}

.fa-spider {
  --fa: "\f717";
}

.fa-hands-bound {
  --fa: "\e4f9";
}

.fa-file-invoice-dollar {
  --fa: "\f571";
}

.fa-plane-circle-exclamation {
  --fa: "\e556";
}

.fa-x-ray {
  --fa: "\f497";
}

.fa-spell-check {
  --fa: "\f891";
}

.fa-slash {
  --fa: "\f715";
}

.fa-computer-mouse {
  --fa: "\f8cc";
}

.fa-mouse {
  --fa: "\f8cc";
}

.fa-arrow-right-to-bracket {
  --fa: "\f090";
}

.fa-sign-in {
  --fa: "\f090";
}

.fa-shop-slash {
  --fa: "\e070";
}

.fa-store-alt-slash {
  --fa: "\e070";
}

.fa-server {
  --fa: "\f233";
}

.fa-virus-covid-slash {
  --fa: "\e4a9";
}

.fa-shop-lock {
  --fa: "\e4a5";
}

.fa-hourglass-start {
  --fa: "\f251";
}

.fa-hourglass-1 {
  --fa: "\f251";
}

.fa-blender-phone {
  --fa: "\f6b6";
}

.fa-building-wheat {
  --fa: "\e4db";
}

.fa-person-breastfeeding {
  --fa: "\e53a";
}

.fa-right-to-bracket {
  --fa: "\f2f6";
}

.fa-sign-in-alt {
  --fa: "\f2f6";
}

.fa-venus {
  --fa: "\f221";
}

.fa-passport {
  --fa: "\f5ab";
}

.fa-thumbtack-slash {
  --fa: "\e68f";
}

.fa-thumb-tack-slash {
  --fa: "\e68f";
}

.fa-heart-pulse {
  --fa: "\f21e";
}

.fa-heartbeat {
  --fa: "\f21e";
}

.fa-people-carry-box {
  --fa: "\f4ce";
}

.fa-people-carry {
  --fa: "\f4ce";
}

.fa-temperature-high {
  --fa: "\f769";
}

.fa-microchip {
  --fa: "\f2db";
}

.fa-crown {
  --fa: "\f521";
}

.fa-weight-hanging {
  --fa: "\f5cd";
}

.fa-xmarks-lines {
  --fa: "\e59a";
}

.fa-file-prescription {
  --fa: "\f572";
}

.fa-weight-scale {
  --fa: "\f496";
}

.fa-weight {
  --fa: "\f496";
}

.fa-user-group {
  --fa: "\f500";
}

.fa-user-friends {
  --fa: "\f500";
}

.fa-arrow-up-a-z {
  --fa: "\f15e";
}

.fa-sort-alpha-up {
  --fa: "\f15e";
}

.fa-chess-knight {
  --fa: "\f441";
}

.fa-face-laugh-squint {
  --fa: "\f59b";
}

.fa-laugh-squint {
  --fa: "\f59b";
}

.fa-wheelchair {
  --fa: "\f193";
}

.fa-circle-arrow-up {
  --fa: "\f0aa";
}

.fa-arrow-circle-up {
  --fa: "\f0aa";
}

.fa-toggle-on {
  --fa: "\f205";
}

.fa-person-walking {
  --fa: "\f554";
}

.fa-walking {
  --fa: "\f554";
}

.fa-l {
  --fa: "L";
}

.fa-fire {
  --fa: "\f06d";
}

.fa-bed-pulse {
  --fa: "\f487";
}

.fa-procedures {
  --fa: "\f487";
}

.fa-shuttle-space {
  --fa: "\f197";
}

.fa-space-shuttle {
  --fa: "\f197";
}

.fa-face-laugh {
  --fa: "\f599";
}

.fa-laugh {
  --fa: "\f599";
}

.fa-folder-open {
  --fa: "\f07c";
}

.fa-heart-circle-plus {
  --fa: "\e500";
}

.fa-code-fork {
  --fa: "\e13b";
}

.fa-city {
  --fa: "\f64f";
}

.fa-microphone-lines {
  --fa: "\f3c9";
}

.fa-microphone-alt {
  --fa: "\f3c9";
}

.fa-pepper-hot {
  --fa: "\f816";
}

.fa-unlock {
  --fa: "\f09c";
}

.fa-colon-sign {
  --fa: "\e140";
}

.fa-headset {
  --fa: "\f590";
}

.fa-store-slash {
  --fa: "\e071";
}

.fa-road-circle-xmark {
  --fa: "\e566";
}

.fa-user-minus {
  --fa: "\f503";
}

.fa-mars-stroke-up {
  --fa: "\f22a";
}

.fa-mars-stroke-v {
  --fa: "\f22a";
}

.fa-champagne-glasses {
  --fa: "\f79f";
}

.fa-glass-cheers {
  --fa: "\f79f";
}

.fa-clipboard {
  --fa: "\f328";
}

.fa-house-circle-exclamation {
  --fa: "\e50a";
}

.fa-file-arrow-up {
  --fa: "\f574";
}

.fa-file-upload {
  --fa: "\f574";
}

.fa-wifi {
  --fa: "\f1eb";
}

.fa-wifi-3 {
  --fa: "\f1eb";
}

.fa-wifi-strong {
  --fa: "\f1eb";
}

.fa-bath {
  --fa: "\f2cd";
}

.fa-bathtub {
  --fa: "\f2cd";
}

.fa-underline {
  --fa: "\f0cd";
}

.fa-user-pen {
  --fa: "\f4ff";
}

.fa-user-edit {
  --fa: "\f4ff";
}

.fa-signature {
  --fa: "\f5b7";
}

.fa-stroopwafel {
  --fa: "\f551";
}

.fa-bold {
  --fa: "\f032";
}

.fa-anchor-lock {
  --fa: "\e4ad";
}

.fa-building-ngo {
  --fa: "\e4d7";
}

.fa-manat-sign {
  --fa: "\e1d5";
}

.fa-not-equal {
  --fa: "\f53e";
}

.fa-border-top-left {
  --fa: "\f853";
}

.fa-border-style {
  --fa: "\f853";
}

.fa-map-location-dot {
  --fa: "\f5a0";
}

.fa-map-marked-alt {
  --fa: "\f5a0";
}

.fa-jedi {
  --fa: "\f669";
}

.fa-square-poll-vertical {
  --fa: "\f681";
}

.fa-poll {
  --fa: "\f681";
}

.fa-mug-hot {
  --fa: "\f7b6";
}

.fa-car-battery {
  --fa: "\f5df";
}

.fa-battery-car {
  --fa: "\f5df";
}

.fa-gift {
  --fa: "\f06b";
}

.fa-dice-two {
  --fa: "\f528";
}

.fa-chess-queen {
  --fa: "\f445";
}

.fa-glasses {
  --fa: "\f530";
}

.fa-chess-board {
  --fa: "\f43c";
}

.fa-building-circle-check {
  --fa: "\e4d2";
}

.fa-person-chalkboard {
  --fa: "\e53d";
}

.fa-mars-stroke-right {
  --fa: "\f22b";
}

.fa-mars-stroke-h {
  --fa: "\f22b";
}

.fa-hand-back-fist {
  --fa: "\f255";
}

.fa-hand-rock {
  --fa: "\f255";
}

.fa-square-caret-up {
  --fa: "\f151";
}

.fa-caret-square-up {
  --fa: "\f151";
}

.fa-cloud-showers-water {
  --fa: "\e4e4";
}

.fa-chart-bar {
  --fa: "\f080";
}

.fa-bar-chart {
  --fa: "\f080";
}

.fa-hands-bubbles {
  --fa: "\e05e";
}

.fa-hands-wash {
  --fa: "\e05e";
}

.fa-less-than-equal {
  --fa: "\f537";
}

.fa-train {
  --fa: "\f238";
}

.fa-eye-low-vision {
  --fa: "\f2a8";
}

.fa-low-vision {
  --fa: "\f2a8";
}

.fa-crow {
  --fa: "\f520";
}

.fa-sailboat {
  --fa: "\e445";
}

.fa-window-restore {
  --fa: "\f2d2";
}

.fa-square-plus {
  --fa: "\f0fe";
}

.fa-plus-square {
  --fa: "\f0fe";
}

.fa-torii-gate {
  --fa: "\f6a1";
}

.fa-frog {
  --fa: "\f52e";
}

.fa-bucket {
  --fa: "\e4cf";
}

.fa-image {
  --fa: "\f03e";
}

.fa-microphone {
  --fa: "\f130";
}

.fa-cow {
  --fa: "\f6c8";
}

.fa-caret-up {
  --fa: "\f0d8";
}

.fa-screwdriver {
  --fa: "\f54a";
}

.fa-folder-closed {
  --fa: "\e185";
}

.fa-house-tsunami {
  --fa: "\e515";
}

.fa-square-nfi {
  --fa: "\e576";
}

.fa-arrow-up-from-ground-water {
  --fa: "\e4b5";
}

.fa-martini-glass {
  --fa: "\f57b";
}

.fa-glass-martini-alt {
  --fa: "\f57b";
}

.fa-square-binary {
  --fa: "\e69b";
}

.fa-rotate-left {
  --fa: "\f2ea";
}

.fa-rotate-back {
  --fa: "\f2ea";
}

.fa-rotate-backward {
  --fa: "\f2ea";
}

.fa-undo-alt {
  --fa: "\f2ea";
}

.fa-table-columns {
  --fa: "\f0db";
}

.fa-columns {
  --fa: "\f0db";
}

.fa-lemon {
  --fa: "\f094";
}

.fa-head-side-mask {
  --fa: "\e063";
}

.fa-handshake {
  --fa: "\f2b5";
}

.fa-gem {
  --fa: "\f3a5";
}

.fa-dolly {
  --fa: "\f472";
}

.fa-dolly-box {
  --fa: "\f472";
}

.fa-smoking {
  --fa: "\f48d";
}

.fa-minimize {
  --fa: "\f78c";
}

.fa-compress-arrows-alt {
  --fa: "\f78c";
}

.fa-monument {
  --fa: "\f5a6";
}

.fa-snowplow {
  --fa: "\f7d2";
}

.fa-angles-right {
  --fa: "\f101";
}

.fa-angle-double-right {
  --fa: "\f101";
}

.fa-cannabis {
  --fa: "\f55f";
}

.fa-circle-play {
  --fa: "\f144";
}

.fa-play-circle {
  --fa: "\f144";
}

.fa-tablets {
  --fa: "\f490";
}

.fa-ethernet {
  --fa: "\f796";
}

.fa-euro-sign {
  --fa: "\f153";
}

.fa-eur {
  --fa: "\f153";
}

.fa-euro {
  --fa: "\f153";
}

.fa-chair {
  --fa: "\f6c0";
}

.fa-circle-check {
  --fa: "\f058";
}

.fa-check-circle {
  --fa: "\f058";
}

.fa-circle-stop {
  --fa: "\f28d";
}

.fa-stop-circle {
  --fa: "\f28d";
}

.fa-compass-drafting {
  --fa: "\f568";
}

.fa-drafting-compass {
  --fa: "\f568";
}

.fa-plate-wheat {
  --fa: "\e55a";
}

.fa-icicles {
  --fa: "\f7ad";
}

.fa-person-shelter {
  --fa: "\e54f";
}

.fa-neuter {
  --fa: "\f22c";
}

.fa-id-badge {
  --fa: "\f2c1";
}

.fa-marker {
  --fa: "\f5a1";
}

.fa-face-laugh-beam {
  --fa: "\f59a";
}

.fa-laugh-beam {
  --fa: "\f59a";
}

.fa-helicopter-symbol {
  --fa: "\e502";
}

.fa-universal-access {
  --fa: "\f29a";
}

.fa-circle-chevron-up {
  --fa: "\f139";
}

.fa-chevron-circle-up {
  --fa: "\f139";
}

.fa-lari-sign {
  --fa: "\e1c8";
}

.fa-volcano {
  --fa: "\f770";
}

.fa-person-walking-dashed-line-arrow-right {
  --fa: "\e553";
}

.fa-sterling-sign {
  --fa: "\f154";
}

.fa-gbp {
  --fa: "\f154";
}

.fa-pound-sign {
  --fa: "\f154";
}

.fa-viruses {
  --fa: "\e076";
}

.fa-square-person-confined {
  --fa: "\e577";
}

.fa-user-tie {
  --fa: "\f508";
}

.fa-arrow-down-long {
  --fa: "\f175";
}

.fa-long-arrow-down {
  --fa: "\f175";
}

.fa-tent-arrow-down-to-line {
  --fa: "\e57e";
}

.fa-certificate {
  --fa: "\f0a3";
}

.fa-reply-all {
  --fa: "\f122";
}

.fa-mail-reply-all {
  --fa: "\f122";
}

.fa-suitcase {
  --fa: "\f0f2";
}

.fa-person-skating {
  --fa: "\f7c5";
}

.fa-skating {
  --fa: "\f7c5";
}

.fa-filter-circle-dollar {
  --fa: "\f662";
}

.fa-funnel-dollar {
  --fa: "\f662";
}

.fa-camera-retro {
  --fa: "\f083";
}

.fa-circle-arrow-down {
  --fa: "\f0ab";
}

.fa-arrow-circle-down {
  --fa: "\f0ab";
}

.fa-file-import {
  --fa: "\f56f";
}

.fa-arrow-right-to-file {
  --fa: "\f56f";
}

.fa-square-arrow-up-right {
  --fa: "\f14c";
}

.fa-external-link-square {
  --fa: "\f14c";
}

.fa-box-open {
  --fa: "\f49e";
}

.fa-scroll {
  --fa: "\f70e";
}

.fa-spa {
  --fa: "\f5bb";
}

.fa-location-pin-lock {
  --fa: "\e51f";
}

.fa-pause {
  --fa: "\f04c";
}

.fa-hill-avalanche {
  --fa: "\e507";
}

.fa-temperature-empty {
  --fa: "\f2cb";
}

.fa-temperature-0 {
  --fa: "\f2cb";
}

.fa-thermometer-0 {
  --fa: "\f2cb";
}

.fa-thermometer-empty {
  --fa: "\f2cb";
}

.fa-bomb {
  --fa: "\f1e2";
}

.fa-registered {
  --fa: "\f25d";
}

.fa-address-card {
  --fa: "\f2bb";
}

.fa-contact-card {
  --fa: "\f2bb";
}

.fa-vcard {
  --fa: "\f2bb";
}

.fa-scale-unbalanced-flip {
  --fa: "\f516";
}

.fa-balance-scale-right {
  --fa: "\f516";
}

.fa-subscript {
  --fa: "\f12c";
}

.fa-diamond-turn-right {
  --fa: "\f5eb";
}

.fa-directions {
  --fa: "\f5eb";
}

.fa-burst {
  --fa: "\e4dc";
}

.fa-house-laptop {
  --fa: "\e066";
}

.fa-laptop-house {
  --fa: "\e066";
}

.fa-face-tired {
  --fa: "\f5c8";
}

.fa-tired {
  --fa: "\f5c8";
}

.fa-money-bills {
  --fa: "\e1f3";
}

.fa-smog {
  --fa: "\f75f";
}

.fa-crutch {
  --fa: "\f7f7";
}

.fa-cloud-arrow-up {
  --fa: "\f0ee";
}

.fa-cloud-upload {
  --fa: "\f0ee";
}

.fa-cloud-upload-alt {
  --fa: "\f0ee";
}

.fa-palette {
  --fa: "\f53f";
}

.fa-arrows-turn-right {
  --fa: "\e4c0";
}

.fa-vest {
  --fa: "\e085";
}

.fa-ferry {
  --fa: "\e4ea";
}

.fa-arrows-down-to-people {
  --fa: "\e4b9";
}

.fa-seedling {
  --fa: "\f4d8";
}

.fa-sprout {
  --fa: "\f4d8";
}

.fa-left-right {
  --fa: "\f337";
}

.fa-arrows-alt-h {
  --fa: "\f337";
}

.fa-boxes-packing {
  --fa: "\e4c7";
}

.fa-circle-arrow-left {
  --fa: "\f0a8";
}

.fa-arrow-circle-left {
  --fa: "\f0a8";
}

.fa-group-arrows-rotate {
  --fa: "\e4f6";
}

.fa-bowl-food {
  --fa: "\e4c6";
}

.fa-candy-cane {
  --fa: "\f786";
}

.fa-arrow-down-wide-short {
  --fa: "\f160";
}

.fa-sort-amount-asc {
  --fa: "\f160";
}

.fa-sort-amount-down {
  --fa: "\f160";
}

.fa-cloud-bolt {
  --fa: "\f76c";
}

.fa-thunderstorm {
  --fa: "\f76c";
}

.fa-text-slash {
  --fa: "\f87d";
}

.fa-remove-format {
  --fa: "\f87d";
}

.fa-face-smile-wink {
  --fa: "\f4da";
}

.fa-smile-wink {
  --fa: "\f4da";
}

.fa-file-word {
  --fa: "\f1c2";
}

.fa-file-powerpoint {
  --fa: "\f1c4";
}

.fa-arrows-left-right {
  --fa: "\f07e";
}

.fa-arrows-h {
  --fa: "\f07e";
}

.fa-house-lock {
  --fa: "\e510";
}

.fa-cloud-arrow-down {
  --fa: "\f0ed";
}

.fa-cloud-download {
  --fa: "\f0ed";
}

.fa-cloud-download-alt {
  --fa: "\f0ed";
}

.fa-children {
  --fa: "\e4e1";
}

.fa-chalkboard {
  --fa: "\f51b";
}

.fa-blackboard {
  --fa: "\f51b";
}

.fa-user-large-slash {
  --fa: "\f4fa";
}

.fa-user-alt-slash {
  --fa: "\f4fa";
}

.fa-envelope-open {
  --fa: "\f2b6";
}

.fa-handshake-simple-slash {
  --fa: "\e05f";
}

.fa-handshake-alt-slash {
  --fa: "\e05f";
}

.fa-mattress-pillow {
  --fa: "\e525";
}

.fa-guarani-sign {
  --fa: "\e19a";
}

.fa-arrows-rotate {
  --fa: "\f021";
}

.fa-refresh {
  --fa: "\f021";
}

.fa-sync {
  --fa: "\f021";
}

.fa-fire-extinguisher {
  --fa: "\f134";
}

.fa-cruzeiro-sign {
  --fa: "\e152";
}

.fa-greater-than-equal {
  --fa: "\f532";
}

.fa-shield-halved {
  --fa: "\f3ed";
}

.fa-shield-alt {
  --fa: "\f3ed";
}

.fa-book-atlas {
  --fa: "\f558";
}

.fa-atlas {
  --fa: "\f558";
}

.fa-virus {
  --fa: "\e074";
}

.fa-envelope-circle-check {
  --fa: "\e4e8";
}

.fa-layer-group {
  --fa: "\f5fd";
}

.fa-arrows-to-dot {
  --fa: "\e4be";
}

.fa-archway {
  --fa: "\f557";
}

.fa-heart-circle-check {
  --fa: "\e4fd";
}

.fa-house-chimney-crack {
  --fa: "\f6f1";
}

.fa-house-damage {
  --fa: "\f6f1";
}

.fa-file-zipper {
  --fa: "\f1c6";
}

.fa-file-archive {
  --fa: "\f1c6";
}

.fa-square {
  --fa: "\f0c8";
}

.fa-martini-glass-empty {
  --fa: "\f000";
}

.fa-glass-martini {
  --fa: "\f000";
}

.fa-couch {
  --fa: "\f4b8";
}

.fa-cedi-sign {
  --fa: "\e0df";
}

.fa-italic {
  --fa: "\f033";
}

.fa-table-cells-column-lock {
  --fa: "\e678";
}

.fa-church {
  --fa: "\f51d";
}

.fa-comments-dollar {
  --fa: "\f653";
}

.fa-democrat {
  --fa: "\f747";
}

.fa-z {
  --fa: "Z";
}

.fa-person-skiing {
  --fa: "\f7c9";
}

.fa-skiing {
  --fa: "\f7c9";
}

.fa-road-lock {
  --fa: "\e567";
}

.fa-a {
  --fa: "A";
}

.fa-temperature-arrow-down {
  --fa: "\e03f";
}

.fa-temperature-down {
  --fa: "\e03f";
}

.fa-feather-pointed {
  --fa: "\f56b";
}

.fa-feather-alt {
  --fa: "\f56b";
}

.fa-p {
  --fa: "P";
}

.fa-snowflake {
  --fa: "\f2dc";
}

.fa-newspaper {
  --fa: "\f1ea";
}

.fa-rectangle-ad {
  --fa: "\f641";
}

.fa-ad {
  --fa: "\f641";
}

.fa-circle-arrow-right {
  --fa: "\f0a9";
}

.fa-arrow-circle-right {
  --fa: "\f0a9";
}

.fa-filter-circle-xmark {
  --fa: "\e17b";
}

.fa-locust {
  --fa: "\e520";
}

.fa-sort {
  --fa: "\f0dc";
}

.fa-unsorted {
  --fa: "\f0dc";
}

.fa-list-ol {
  --fa: "\f0cb";
}

.fa-list-1-2 {
  --fa: "\f0cb";
}

.fa-list-numeric {
  --fa: "\f0cb";
}

.fa-person-dress-burst {
  --fa: "\e544";
}

.fa-money-check-dollar {
  --fa: "\f53d";
}

.fa-money-check-alt {
  --fa: "\f53d";
}

.fa-vector-square {
  --fa: "\f5cb";
}

.fa-bread-slice {
  --fa: "\f7ec";
}

.fa-language {
  --fa: "\f1ab";
}

.fa-face-kiss-wink-heart {
  --fa: "\f598";
}

.fa-kiss-wink-heart {
  --fa: "\f598";
}

.fa-filter {
  --fa: "\f0b0";
}

.fa-question {
  --fa: "\?";
}

.fa-file-signature {
  --fa: "\f573";
}

.fa-up-down-left-right {
  --fa: "\f0b2";
}

.fa-arrows-alt {
  --fa: "\f0b2";
}

.fa-house-chimney-user {
  --fa: "\e065";
}

.fa-hand-holding-heart {
  --fa: "\f4be";
}

.fa-puzzle-piece {
  --fa: "\f12e";
}

.fa-money-check {
  --fa: "\f53c";
}

.fa-star-half-stroke {
  --fa: "\f5c0";
}

.fa-star-half-alt {
  --fa: "\f5c0";
}

.fa-code {
  --fa: "\f121";
}

.fa-whiskey-glass {
  --fa: "\f7a0";
}

.fa-glass-whiskey {
  --fa: "\f7a0";
}

.fa-building-circle-exclamation {
  --fa: "\e4d3";
}

.fa-magnifying-glass-chart {
  --fa: "\e522";
}

.fa-arrow-up-right-from-square {
  --fa: "\f08e";
}

.fa-external-link {
  --fa: "\f08e";
}

.fa-cubes-stacked {
  --fa: "\e4e6";
}

.fa-won-sign {
  --fa: "\f159";
}

.fa-krw {
  --fa: "\f159";
}

.fa-won {
  --fa: "\f159";
}

.fa-virus-covid {
  --fa: "\e4a8";
}

.fa-austral-sign {
  --fa: "\e0a9";
}

.fa-f {
  --fa: "F";
}

.fa-leaf {
  --fa: "\f06c";
}

.fa-road {
  --fa: "\f018";
}

.fa-taxi {
  --fa: "\f1ba";
}

.fa-cab {
  --fa: "\f1ba";
}

.fa-person-circle-plus {
  --fa: "\e541";
}

.fa-chart-pie {
  --fa: "\f200";
}

.fa-pie-chart {
  --fa: "\f200";
}

.fa-bolt-lightning {
  --fa: "\e0b7";
}

.fa-sack-xmark {
  --fa: "\e56a";
}

.fa-file-excel {
  --fa: "\f1c3";
}

.fa-file-contract {
  --fa: "\f56c";
}

.fa-fish-fins {
  --fa: "\e4f2";
}

.fa-building-flag {
  --fa: "\e4d5";
}

.fa-face-grin-beam {
  --fa: "\f582";
}

.fa-grin-beam {
  --fa: "\f582";
}

.fa-object-ungroup {
  --fa: "\f248";
}

.fa-poop {
  --fa: "\f619";
}

.fa-location-pin {
  --fa: "\f041";
}

.fa-map-marker {
  --fa: "\f041";
}

.fa-kaaba {
  --fa: "\f66b";
}

.fa-toilet-paper {
  --fa: "\f71e";
}

.fa-helmet-safety {
  --fa: "\f807";
}

.fa-hard-hat {
  --fa: "\f807";
}

.fa-hat-hard {
  --fa: "\f807";
}

.fa-eject {
  --fa: "\f052";
}

.fa-circle-right {
  --fa: "\f35a";
}

.fa-arrow-alt-circle-right {
  --fa: "\f35a";
}

.fa-plane-circle-check {
  --fa: "\e555";
}

.fa-face-rolling-eyes {
  --fa: "\f5a5";
}

.fa-meh-rolling-eyes {
  --fa: "\f5a5";
}

.fa-object-group {
  --fa: "\f247";
}

.fa-chart-line {
  --fa: "\f201";
}

.fa-line-chart {
  --fa: "\f201";
}

.fa-mask-ventilator {
  --fa: "\e524";
}

.fa-arrow-right {
  --fa: "\f061";
}

.fa-signs-post {
  --fa: "\f277";
}

.fa-map-signs {
  --fa: "\f277";
}

.fa-cash-register {
  --fa: "\f788";
}

.fa-person-circle-question {
  --fa: "\e542";
}

.fa-h {
  --fa: "H";
}

.fa-tarp {
  --fa: "\e57b";
}

.fa-screwdriver-wrench {
  --fa: "\f7d9";
}

.fa-tools {
  --fa: "\f7d9";
}

.fa-arrows-to-eye {
  --fa: "\e4bf";
}

.fa-plug-circle-bolt {
  --fa: "\e55b";
}

.fa-heart {
  --fa: "\f004";
}

.fa-mars-and-venus {
  --fa: "\f224";
}

.fa-house-user {
  --fa: "\e1b0";
}

.fa-home-user {
  --fa: "\e1b0";
}

.fa-dumpster-fire {
  --fa: "\f794";
}

.fa-house-crack {
  --fa: "\e3b1";
}

.fa-martini-glass-citrus {
  --fa: "\f561";
}

.fa-cocktail {
  --fa: "\f561";
}

.fa-face-surprise {
  --fa: "\f5c2";
}

.fa-surprise {
  --fa: "\f5c2";
}

.fa-bottle-water {
  --fa: "\e4c5";
}

.fa-circle-pause {
  --fa: "\f28b";
}

.fa-pause-circle {
  --fa: "\f28b";
}

.fa-toilet-paper-slash {
  --fa: "\e072";
}

.fa-apple-whole {
  --fa: "\f5d1";
}

.fa-apple-alt {
  --fa: "\f5d1";
}

.fa-kitchen-set {
  --fa: "\e51a";
}

.fa-r {
  --fa: "R";
}

.fa-temperature-quarter {
  --fa: "\f2ca";
}

.fa-temperature-1 {
  --fa: "\f2ca";
}

.fa-thermometer-1 {
  --fa: "\f2ca";
}

.fa-thermometer-quarter {
  --fa: "\f2ca";
}

.fa-cube {
  --fa: "\f1b2";
}

.fa-bitcoin-sign {
  --fa: "\e0b4";
}

.fa-shield-dog {
  --fa: "\e573";
}

.fa-solar-panel {
  --fa: "\f5ba";
}

.fa-lock-open {
  --fa: "\f3c1";
}

.fa-elevator {
  --fa: "\e16d";
}

.fa-money-bill-transfer {
  --fa: "\e528";
}

.fa-money-bill-trend-up {
  --fa: "\e529";
}

.fa-house-flood-water-circle-arrow-right {
  --fa: "\e50f";
}

.fa-square-poll-horizontal {
  --fa: "\f682";
}

.fa-poll-h {
  --fa: "\f682";
}

.fa-circle {
  --fa: "\f111";
}

.fa-backward-fast {
  --fa: "\f049";
}

.fa-fast-backward {
  --fa: "\f049";
}

.fa-recycle {
  --fa: "\f1b8";
}

.fa-user-astronaut {
  --fa: "\f4fb";
}

.fa-plane-slash {
  --fa: "\e069";
}

.fa-trademark {
  --fa: "\f25c";
}

.fa-basketball {
  --fa: "\f434";
}

.fa-basketball-ball {
  --fa: "\f434";
}

.fa-satellite-dish {
  --fa: "\f7c0";
}

.fa-circle-up {
  --fa: "\f35b";
}

.fa-arrow-alt-circle-up {
  --fa: "\f35b";
}

.fa-mobile-screen-button {
  --fa: "\f3cd";
}

.fa-mobile-alt {
  --fa: "\f3cd";
}

.fa-volume-high {
  --fa: "\f028";
}

.fa-volume-up {
  --fa: "\f028";
}

.fa-users-rays {
  --fa: "\e593";
}

.fa-wallet {
  --fa: "\f555";
}

.fa-clipboard-check {
  --fa: "\f46c";
}

.fa-file-audio {
  --fa: "\f1c7";
}

.fa-burger {
  --fa: "\f805";
}

.fa-hamburger {
  --fa: "\f805";
}

.fa-wrench {
  --fa: "\f0ad";
}

.fa-bugs {
  --fa: "\e4d0";
}

.fa-rupee-sign {
  --fa: "\f156";
}

.fa-rupee {
  --fa: "\f156";
}

.fa-file-image {
  --fa: "\f1c5";
}

.fa-circle-question {
  --fa: "\f059";
}

.fa-question-circle {
  --fa: "\f059";
}

.fa-plane-departure {
  --fa: "\f5b0";
}

.fa-handshake-slash {
  --fa: "\e060";
}

.fa-book-bookmark {
  --fa: "\e0bb";
}

.fa-code-branch {
  --fa: "\f126";
}

.fa-hat-cowboy {
  --fa: "\f8c0";
}

.fa-bridge {
  --fa: "\e4c8";
}

.fa-phone-flip {
  --fa: "\f879";
}

.fa-phone-alt {
  --fa: "\f879";
}

.fa-truck-front {
  --fa: "\e2b7";
}

.fa-cat {
  --fa: "\f6be";
}

.fa-anchor-circle-exclamation {
  --fa: "\e4ab";
}

.fa-truck-field {
  --fa: "\e58d";
}

.fa-route {
  --fa: "\f4d7";
}

.fa-clipboard-question {
  --fa: "\e4e3";
}

.fa-panorama {
  --fa: "\e209";
}

.fa-comment-medical {
  --fa: "\f7f5";
}

.fa-teeth-open {
  --fa: "\f62f";
}

.fa-file-circle-minus {
  --fa: "\e4ed";
}

.fa-tags {
  --fa: "\f02c";
}

.fa-wine-glass {
  --fa: "\f4e3";
}

.fa-forward-fast {
  --fa: "\f050";
}

.fa-fast-forward {
  --fa: "\f050";
}

.fa-face-meh-blank {
  --fa: "\f5a4";
}

.fa-meh-blank {
  --fa: "\f5a4";
}

.fa-square-parking {
  --fa: "\f540";
}

.fa-parking {
  --fa: "\f540";
}

.fa-house-signal {
  --fa: "\e012";
}

.fa-bars-progress {
  --fa: "\f828";
}

.fa-tasks-alt {
  --fa: "\f828";
}

.fa-faucet-drip {
  --fa: "\e006";
}

.fa-cart-flatbed {
  --fa: "\f474";
}

.fa-dolly-flatbed {
  --fa: "\f474";
}

.fa-ban-smoking {
  --fa: "\f54d";
}

.fa-smoking-ban {
  --fa: "\f54d";
}

.fa-terminal {
  --fa: "\f120";
}

.fa-mobile-button {
  --fa: "\f10b";
}

.fa-house-medical-flag {
  --fa: "\e514";
}

.fa-basket-shopping {
  --fa: "\f291";
}

.fa-shopping-basket {
  --fa: "\f291";
}

.fa-tape {
  --fa: "\f4db";
}

.fa-bus-simple {
  --fa: "\f55e";
}

.fa-bus-alt {
  --fa: "\f55e";
}

.fa-eye {
  --fa: "\f06e";
}

.fa-face-sad-cry {
  --fa: "\f5b3";
}

.fa-sad-cry {
  --fa: "\f5b3";
}

.fa-audio-description {
  --fa: "\f29e";
}

.fa-person-military-to-person {
  --fa: "\e54c";
}

.fa-file-shield {
  --fa: "\e4f0";
}

.fa-user-slash {
  --fa: "\f506";
}

.fa-pen {
  --fa: "\f304";
}

.fa-tower-observation {
  --fa: "\e586";
}

.fa-file-code {
  --fa: "\f1c9";
}

.fa-signal {
  --fa: "\f012";
}

.fa-signal-5 {
  --fa: "\f012";
}

.fa-signal-perfect {
  --fa: "\f012";
}

.fa-bus {
  --fa: "\f207";
}

.fa-heart-circle-xmark {
  --fa: "\e501";
}

.fa-house-chimney {
  --fa: "\e3af";
}

.fa-home-lg {
  --fa: "\e3af";
}

.fa-window-maximize {
  --fa: "\f2d0";
}

.fa-face-frown {
  --fa: "\f119";
}

.fa-frown {
  --fa: "\f119";
}

.fa-prescription {
  --fa: "\f5b1";
}

.fa-shop {
  --fa: "\f54f";
}

.fa-store-alt {
  --fa: "\f54f";
}

.fa-floppy-disk {
  --fa: "\f0c7";
}

.fa-save {
  --fa: "\f0c7";
}

.fa-vihara {
  --fa: "\f6a7";
}

.fa-scale-unbalanced {
  --fa: "\f515";
}

.fa-balance-scale-left {
  --fa: "\f515";
}

.fa-sort-up {
  --fa: "\f0de";
}

.fa-sort-asc {
  --fa: "\f0de";
}

.fa-comment-dots {
  --fa: "\f4ad";
}

.fa-commenting {
  --fa: "\f4ad";
}

.fa-plant-wilt {
  --fa: "\e5aa";
}

.fa-diamond {
  --fa: "\f219";
}

.fa-face-grin-squint {
  --fa: "\f585";
}

.fa-grin-squint {
  --fa: "\f585";
}

.fa-hand-holding-dollar {
  --fa: "\f4c0";
}

.fa-hand-holding-usd {
  --fa: "\f4c0";
}

.fa-chart-diagram {
  --fa: "\e695";
}

.fa-bacterium {
  --fa: "\e05a";
}

.fa-hand-pointer {
  --fa: "\f25a";
}

.fa-drum-steelpan {
  --fa: "\f56a";
}

.fa-hand-scissors {
  --fa: "\f257";
}

.fa-hands-praying {
  --fa: "\f684";
}

.fa-praying-hands {
  --fa: "\f684";
}

.fa-arrow-rotate-right {
  --fa: "\f01e";
}

.fa-arrow-right-rotate {
  --fa: "\f01e";
}

.fa-arrow-rotate-forward {
  --fa: "\f01e";
}

.fa-redo {
  --fa: "\f01e";
}

.fa-biohazard {
  --fa: "\f780";
}

.fa-location-crosshairs {
  --fa: "\f601";
}

.fa-location {
  --fa: "\f601";
}

.fa-mars-double {
  --fa: "\f227";
}

.fa-child-dress {
  --fa: "\e59c";
}

.fa-users-between-lines {
  --fa: "\e591";
}

.fa-lungs-virus {
  --fa: "\e067";
}

.fa-face-grin-tears {
  --fa: "\f588";
}

.fa-grin-tears {
  --fa: "\f588";
}

.fa-phone {
  --fa: "\f095";
}

.fa-calendar-xmark {
  --fa: "\f273";
}

.fa-calendar-times {
  --fa: "\f273";
}

.fa-child-reaching {
  --fa: "\e59d";
}

.fa-head-side-virus {
  --fa: "\e064";
}

.fa-user-gear {
  --fa: "\f4fe";
}

.fa-user-cog {
  --fa: "\f4fe";
}

.fa-arrow-up-1-9 {
  --fa: "\f163";
}

.fa-sort-numeric-up {
  --fa: "\f163";
}

.fa-door-closed {
  --fa: "\f52a";
}

.fa-shield-virus {
  --fa: "\e06c";
}

.fa-dice-six {
  --fa: "\f526";
}

.fa-mosquito-net {
  --fa: "\e52c";
}

.fa-file-fragment {
  --fa: "\e697";
}

.fa-bridge-water {
  --fa: "\e4ce";
}

.fa-person-booth {
  --fa: "\f756";
}

.fa-text-width {
  --fa: "\f035";
}

.fa-hat-wizard {
  --fa: "\f6e8";
}

.fa-pen-fancy {
  --fa: "\f5ac";
}

.fa-person-digging {
  --fa: "\f85e";
}

.fa-digging {
  --fa: "\f85e";
}

.fa-trash {
  --fa: "\f1f8";
}

.fa-gauge-simple {
  --fa: "\f629";
}

.fa-gauge-simple-med {
  --fa: "\f629";
}

.fa-tachometer-average {
  --fa: "\f629";
}

.fa-book-medical {
  --fa: "\f7e6";
}

.fa-poo {
  --fa: "\f2fe";
}

.fa-quote-right {
  --fa: "\f10e";
}

.fa-quote-right-alt {
  --fa: "\f10e";
}

.fa-shirt {
  --fa: "\f553";
}

.fa-t-shirt {
  --fa: "\f553";
}

.fa-tshirt {
  --fa: "\f553";
}

.fa-cubes {
  --fa: "\f1b3";
}

.fa-divide {
  --fa: "\f529";
}

.fa-tenge-sign {
  --fa: "\f7d7";
}

.fa-tenge {
  --fa: "\f7d7";
}

.fa-headphones {
  --fa: "\f025";
}

.fa-hands-holding {
  --fa: "\f4c2";
}

.fa-hands-clapping {
  --fa: "\e1a8";
}

.fa-republican {
  --fa: "\f75e";
}

.fa-arrow-left {
  --fa: "\f060";
}

.fa-person-circle-xmark {
  --fa: "\e543";
}

.fa-ruler {
  --fa: "\f545";
}

.fa-align-left {
  --fa: "\f036";
}

.fa-dice-d6 {
  --fa: "\f6d1";
}

.fa-restroom {
  --fa: "\f7bd";
}

.fa-j {
  --fa: "J";
}

.fa-users-viewfinder {
  --fa: "\e595";
}

.fa-file-video {
  --fa: "\f1c8";
}

.fa-up-right-from-square {
  --fa: "\f35d";
}

.fa-external-link-alt {
  --fa: "\f35d";
}

.fa-table-cells {
  --fa: "\f00a";
}

.fa-th {
  --fa: "\f00a";
}

.fa-file-pdf {
  --fa: "\f1c1";
}

.fa-book-bible {
  --fa: "\f647";
}

.fa-bible {
  --fa: "\f647";
}

.fa-o {
  --fa: "O";
}

.fa-suitcase-medical {
  --fa: "\f0fa";
}

.fa-medkit {
  --fa: "\f0fa";
}

.fa-user-secret {
  --fa: "\f21b";
}

.fa-otter {
  --fa: "\f700";
}

.fa-person-dress {
  --fa: "\f182";
}

.fa-female {
  --fa: "\f182";
}

.fa-comment-dollar {
  --fa: "\f651";
}

.fa-business-time {
  --fa: "\f64a";
}

.fa-briefcase-clock {
  --fa: "\f64a";
}

.fa-table-cells-large {
  --fa: "\f009";
}

.fa-th-large {
  --fa: "\f009";
}

.fa-book-tanakh {
  --fa: "\f827";
}

.fa-tanakh {
  --fa: "\f827";
}

.fa-phone-volume {
  --fa: "\f2a0";
}

.fa-volume-control-phone {
  --fa: "\f2a0";
}

.fa-hat-cowboy-side {
  --fa: "\f8c1";
}

.fa-clipboard-user {
  --fa: "\f7f3";
}

.fa-child {
  --fa: "\f1ae";
}

.fa-lira-sign {
  --fa: "\f195";
}

.fa-satellite {
  --fa: "\f7bf";
}

.fa-plane-lock {
  --fa: "\e558";
}

.fa-tag {
  --fa: "\f02b";
}

.fa-comment {
  --fa: "\f075";
}

.fa-cake-candles {
  --fa: "\f1fd";
}

.fa-birthday-cake {
  --fa: "\f1fd";
}

.fa-cake {
  --fa: "\f1fd";
}

.fa-envelope {
  --fa: "\f0e0";
}

.fa-angles-up {
  --fa: "\f102";
}

.fa-angle-double-up {
  --fa: "\f102";
}

.fa-paperclip {
  --fa: "\f0c6";
}

.fa-arrow-right-to-city {
  --fa: "\e4b3";
}

.fa-ribbon {
  --fa: "\f4d6";
}

.fa-lungs {
  --fa: "\f604";
}

.fa-arrow-up-9-1 {
  --fa: "\f887";
}

.fa-sort-numeric-up-alt {
  --fa: "\f887";
}

.fa-litecoin-sign {
  --fa: "\e1d3";
}

.fa-border-none {
  --fa: "\f850";
}

.fa-circle-nodes {
  --fa: "\e4e2";
}

.fa-parachute-box {
  --fa: "\f4cd";
}

.fa-indent {
  --fa: "\f03c";
}

.fa-truck-field-un {
  --fa: "\e58e";
}

.fa-hourglass {
  --fa: "\f254";
}

.fa-hourglass-empty {
  --fa: "\f254";
}

.fa-mountain {
  --fa: "\f6fc";
}

.fa-user-doctor {
  --fa: "\f0f0";
}

.fa-user-md {
  --fa: "\f0f0";
}

.fa-circle-info {
  --fa: "\f05a";
}

.fa-info-circle {
  --fa: "\f05a";
}

.fa-cloud-meatball {
  --fa: "\f73b";
}

.fa-camera {
  --fa: "\f030";
}

.fa-camera-alt {
  --fa: "\f030";
}

.fa-square-virus {
  --fa: "\e578";
}

.fa-meteor {
  --fa: "\f753";
}

.fa-car-on {
  --fa: "\e4dd";
}

.fa-sleigh {
  --fa: "\f7cc";
}

.fa-arrow-down-1-9 {
  --fa: "\f162";
}

.fa-sort-numeric-asc {
  --fa: "\f162";
}

.fa-sort-numeric-down {
  --fa: "\f162";
}

.fa-hand-holding-droplet {
  --fa: "\f4c1";
}

.fa-hand-holding-water {
  --fa: "\f4c1";
}

.fa-water {
  --fa: "\f773";
}

.fa-calendar-check {
  --fa: "\f274";
}

.fa-braille {
  --fa: "\f2a1";
}

.fa-prescription-bottle-medical {
  --fa: "\f486";
}

.fa-prescription-bottle-alt {
  --fa: "\f486";
}

.fa-landmark {
  --fa: "\f66f";
}

.fa-truck {
  --fa: "\f0d1";
}

.fa-crosshairs {
  --fa: "\f05b";
}

.fa-person-cane {
  --fa: "\e53c";
}

.fa-tent {
  --fa: "\e57d";
}

.fa-vest-patches {
  --fa: "\e086";
}

.fa-check-double {
  --fa: "\f560";
}

.fa-arrow-down-a-z {
  --fa: "\f15d";
}

.fa-sort-alpha-asc {
  --fa: "\f15d";
}

.fa-sort-alpha-down {
  --fa: "\f15d";
}

.fa-money-bill-wheat {
  --fa: "\e52a";
}

.fa-cookie {
  --fa: "\f563";
}

.fa-arrow-rotate-left {
  --fa: "\f0e2";
}

.fa-arrow-left-rotate {
  --fa: "\f0e2";
}

.fa-arrow-rotate-back {
  --fa: "\f0e2";
}

.fa-arrow-rotate-backward {
  --fa: "\f0e2";
}

.fa-undo {
  --fa: "\f0e2";
}

.fa-hard-drive {
  --fa: "\f0a0";
}

.fa-hdd {
  --fa: "\f0a0";
}

.fa-face-grin-squint-tears {
  --fa: "\f586";
}

.fa-grin-squint-tears {
  --fa: "\f586";
}

.fa-dumbbell {
  --fa: "\f44b";
}

.fa-rectangle-list {
  --fa: "\f022";
}

.fa-list-alt {
  --fa: "\f022";
}

.fa-tarp-droplet {
  --fa: "\e57c";
}

.fa-house-medical-circle-check {
  --fa: "\e511";
}

.fa-person-skiing-nordic {
  --fa: "\f7ca";
}

.fa-skiing-nordic {
  --fa: "\f7ca";
}

.fa-calendar-plus {
  --fa: "\f271";
}

.fa-plane-arrival {
  --fa: "\f5af";
}

.fa-circle-left {
  --fa: "\f359";
}

.fa-arrow-alt-circle-left {
  --fa: "\f359";
}

.fa-train-subway {
  --fa: "\f239";
}

.fa-subway {
  --fa: "\f239";
}

.fa-chart-gantt {
  --fa: "\e0e4";
}

.fa-indian-rupee-sign {
  --fa: "\e1bc";
}

.fa-indian-rupee {
  --fa: "\e1bc";
}

.fa-inr {
  --fa: "\e1bc";
}

.fa-crop-simple {
  --fa: "\f565";
}

.fa-crop-alt {
  --fa: "\f565";
}

.fa-money-bill-1 {
  --fa: "\f3d1";
}

.fa-money-bill-alt {
  --fa: "\f3d1";
}

.fa-left-long {
  --fa: "\f30a";
}

.fa-long-arrow-alt-left {
  --fa: "\f30a";
}

.fa-dna {
  --fa: "\f471";
}

.fa-virus-slash {
  --fa: "\e075";
}

.fa-minus {
  --fa: "\f068";
}

.fa-subtract {
  --fa: "\f068";
}

.fa-chess {
  --fa: "\f439";
}

.fa-arrow-left-long {
  --fa: "\f177";
}

.fa-long-arrow-left {
  --fa: "\f177";
}

.fa-plug-circle-check {
  --fa: "\e55c";
}

.fa-street-view {
  --fa: "\f21d";
}

.fa-franc-sign {
  --fa: "\e18f";
}

.fa-volume-off {
  --fa: "\f026";
}

.fa-hands-asl-interpreting {
  --fa: "\f2a3";
}

.fa-american-sign-language-interpreting {
  --fa: "\f2a3";
}

.fa-asl-interpreting {
  --fa: "\f2a3";
}

.fa-hands-american-sign-language-interpreting {
  --fa: "\f2a3";
}

.fa-gear {
  --fa: "\f013";
}

.fa-cog {
  --fa: "\f013";
}

.fa-droplet-slash {
  --fa: "\f5c7";
}

.fa-tint-slash {
  --fa: "\f5c7";
}

.fa-mosque {
  --fa: "\f678";
}

.fa-mosquito {
  --fa: "\e52b";
}

.fa-star-of-david {
  --fa: "\f69a";
}

.fa-person-military-rifle {
  --fa: "\e54b";
}

.fa-cart-shopping {
  --fa: "\f07a";
}

.fa-shopping-cart {
  --fa: "\f07a";
}

.fa-vials {
  --fa: "\f493";
}

.fa-plug-circle-plus {
  --fa: "\e55f";
}

.fa-place-of-worship {
  --fa: "\f67f";
}

.fa-grip-vertical {
  --fa: "\f58e";
}

.fa-hexagon-nodes {
  --fa: "\e699";
}

.fa-arrow-turn-up {
  --fa: "\f148";
}

.fa-level-up {
  --fa: "\f148";
}

.fa-u {
  --fa: "U";
}

.fa-square-root-variable {
  --fa: "\f698";
}

.fa-square-root-alt {
  --fa: "\f698";
}

.fa-clock {
  --fa: "\f017";
}

.fa-clock-four {
  --fa: "\f017";
}

.fa-backward-step {
  --fa: "\f048";
}

.fa-step-backward {
  --fa: "\f048";
}

.fa-pallet {
  --fa: "\f482";
}

.fa-faucet {
  --fa: "\e005";
}

.fa-baseball-bat-ball {
  --fa: "\f432";
}

.fa-s {
  --fa: "S";
}

.fa-timeline {
  --fa: "\e29c";
}

.fa-keyboard {
  --fa: "\f11c";
}

.fa-caret-down {
  --fa: "\f0d7";
}

.fa-house-chimney-medical {
  --fa: "\f7f2";
}

.fa-clinic-medical {
  --fa: "\f7f2";
}

.fa-temperature-three-quarters {
  --fa: "\f2c8";
}

.fa-temperature-3 {
  --fa: "\f2c8";
}

.fa-thermometer-3 {
  --fa: "\f2c8";
}

.fa-thermometer-three-quarters {
  --fa: "\f2c8";
}

.fa-mobile-screen {
  --fa: "\f3cf";
}

.fa-mobile-android-alt {
  --fa: "\f3cf";
}

.fa-plane-up {
  --fa: "\e22d";
}

.fa-piggy-bank {
  --fa: "\f4d3";
}

.fa-battery-half {
  --fa: "\f242";
}

.fa-battery-3 {
  --fa: "\f242";
}

.fa-mountain-city {
  --fa: "\e52e";
}

.fa-coins {
  --fa: "\f51e";
}

.fa-khanda {
  --fa: "\f66d";
}

.fa-sliders {
  --fa: "\f1de";
}

.fa-sliders-h {
  --fa: "\f1de";
}

.fa-folder-tree {
  --fa: "\f802";
}

.fa-network-wired {
  --fa: "\f6ff";
}

.fa-map-pin {
  --fa: "\f276";
}

.fa-hamsa {
  --fa: "\f665";
}

.fa-cent-sign {
  --fa: "\e3f5";
}

.fa-flask {
  --fa: "\f0c3";
}

.fa-person-pregnant {
  --fa: "\e31e";
}

.fa-wand-sparkles {
  --fa: "\f72b";
}

.fa-ellipsis-vertical {
  --fa: "\f142";
}

.fa-ellipsis-v {
  --fa: "\f142";
}

.fa-ticket {
  --fa: "\f145";
}

.fa-power-off {
  --fa: "\f011";
}

.fa-right-long {
  --fa: "\f30b";
}

.fa-long-arrow-alt-right {
  --fa: "\f30b";
}

.fa-flag-usa {
  --fa: "\f74d";
}

.fa-laptop-file {
  --fa: "\e51d";
}

.fa-tty {
  --fa: "\f1e4";
}

.fa-teletype {
  --fa: "\f1e4";
}

.fa-diagram-next {
  --fa: "\e476";
}

.fa-person-rifle {
  --fa: "\e54e";
}

.fa-house-medical-circle-exclamation {
  --fa: "\e512";
}

.fa-closed-captioning {
  --fa: "\f20a";
}

.fa-person-hiking {
  --fa: "\f6ec";
}

.fa-hiking {
  --fa: "\f6ec";
}

.fa-venus-double {
  --fa: "\f226";
}

.fa-images {
  --fa: "\f302";
}

.fa-calculator {
  --fa: "\f1ec";
}

.fa-people-pulling {
  --fa: "\e535";
}

.fa-n {
  --fa: "N";
}

.fa-cable-car {
  --fa: "\f7da";
}

.fa-tram {
  --fa: "\f7da";
}

.fa-cloud-rain {
  --fa: "\f73d";
}

.fa-building-circle-xmark {
  --fa: "\e4d4";
}

.fa-ship {
  --fa: "\f21a";
}

.fa-arrows-down-to-line {
  --fa: "\e4b8";
}

.fa-download {
  --fa: "\f019";
}

.fa-face-grin {
  --fa: "\f580";
}

.fa-grin {
  --fa: "\f580";
}

.fa-delete-left {
  --fa: "\f55a";
}

.fa-backspace {
  --fa: "\f55a";
}

.fa-eye-dropper {
  --fa: "\f1fb";
}

.fa-eye-dropper-empty {
  --fa: "\f1fb";
}

.fa-eyedropper {
  --fa: "\f1fb";
}

.fa-file-circle-check {
  --fa: "\e5a0";
}

.fa-forward {
  --fa: "\f04e";
}

.fa-mobile {
  --fa: "\f3ce";
}

.fa-mobile-android {
  --fa: "\f3ce";
}

.fa-mobile-phone {
  --fa: "\f3ce";
}

.fa-face-meh {
  --fa: "\f11a";
}

.fa-meh {
  --fa: "\f11a";
}

.fa-align-center {
  --fa: "\f037";
}

.fa-book-skull {
  --fa: "\f6b7";
}

.fa-book-dead {
  --fa: "\f6b7";
}

.fa-id-card {
  --fa: "\f2c2";
}

.fa-drivers-license {
  --fa: "\f2c2";
}

.fa-outdent {
  --fa: "\f03b";
}

.fa-dedent {
  --fa: "\f03b";
}

.fa-heart-circle-exclamation {
  --fa: "\e4fe";
}

.fa-house {
  --fa: "\f015";
}

.fa-home {
  --fa: "\f015";
}

.fa-home-alt {
  --fa: "\f015";
}

.fa-home-lg-alt {
  --fa: "\f015";
}

.fa-calendar-week {
  --fa: "\f784";
}

.fa-laptop-medical {
  --fa: "\f812";
}

.fa-b {
  --fa: "B";
}

.fa-file-medical {
  --fa: "\f477";
}

.fa-dice-one {
  --fa: "\f525";
}

.fa-kiwi-bird {
  --fa: "\f535";
}

.fa-arrow-right-arrow-left {
  --fa: "\f0ec";
}

.fa-exchange {
  --fa: "\f0ec";
}

.fa-rotate-right {
  --fa: "\f2f9";
}

.fa-redo-alt {
  --fa: "\f2f9";
}

.fa-rotate-forward {
  --fa: "\f2f9";
}

.fa-utensils {
  --fa: "\f2e7";
}

.fa-cutlery {
  --fa: "\f2e7";
}

.fa-arrow-up-wide-short {
  --fa: "\f161";
}

.fa-sort-amount-up {
  --fa: "\f161";
}

.fa-mill-sign {
  --fa: "\e1ed";
}

.fa-bowl-rice {
  --fa: "\e2eb";
}

.fa-skull {
  --fa: "\f54c";
}

.fa-tower-broadcast {
  --fa: "\f519";
}

.fa-broadcast-tower {
  --fa: "\f519";
}

.fa-truck-pickup {
  --fa: "\f63c";
}

.fa-up-long {
  --fa: "\f30c";
}

.fa-long-arrow-alt-up {
  --fa: "\f30c";
}

.fa-stop {
  --fa: "\f04d";
}

.fa-code-merge {
  --fa: "\f387";
}

.fa-upload {
  --fa: "\f093";
}

.fa-hurricane {
  --fa: "\f751";
}

.fa-mound {
  --fa: "\e52d";
}

.fa-toilet-portable {
  --fa: "\e583";
}

.fa-compact-disc {
  --fa: "\f51f";
}

.fa-file-arrow-down {
  --fa: "\f56d";
}

.fa-file-download {
  --fa: "\f56d";
}

.fa-caravan {
  --fa: "\f8ff";
}

.fa-shield-cat {
  --fa: "\e572";
}

.fa-bolt {
  --fa: "\f0e7";
}

.fa-zap {
  --fa: "\f0e7";
}

.fa-glass-water {
  --fa: "\e4f4";
}

.fa-oil-well {
  --fa: "\e532";
}

.fa-vault {
  --fa: "\e2c5";
}

.fa-mars {
  --fa: "\f222";
}

.fa-toilet {
  --fa: "\f7d8";
}

.fa-plane-circle-xmark {
  --fa: "\e557";
}

.fa-yen-sign {
  --fa: "\f157";
}

.fa-cny {
  --fa: "\f157";
}

.fa-jpy {
  --fa: "\f157";
}

.fa-rmb {
  --fa: "\f157";
}

.fa-yen {
  --fa: "\f157";
}

.fa-ruble-sign {
  --fa: "\f158";
}

.fa-rouble {
  --fa: "\f158";
}

.fa-rub {
  --fa: "\f158";
}

.fa-ruble {
  --fa: "\f158";
}

.fa-sun {
  --fa: "\f185";
}

.fa-guitar {
  --fa: "\f7a6";
}

.fa-face-laugh-wink {
  --fa: "\f59c";
}

.fa-laugh-wink {
  --fa: "\f59c";
}

.fa-horse-head {
  --fa: "\f7ab";
}

.fa-bore-hole {
  --fa: "\e4c3";
}

.fa-industry {
  --fa: "\f275";
}

.fa-circle-down {
  --fa: "\f358";
}

.fa-arrow-alt-circle-down {
  --fa: "\f358";
}

.fa-arrows-turn-to-dots {
  --fa: "\e4c1";
}

.fa-florin-sign {
  --fa: "\e184";
}

.fa-arrow-down-short-wide {
  --fa: "\f884";
}

.fa-sort-amount-desc {
  --fa: "\f884";
}

.fa-sort-amount-down-alt {
  --fa: "\f884";
}

.fa-less-than {
  --fa: "\<";
}

.fa-angle-down {
  --fa: "\f107";
}

.fa-car-tunnel {
  --fa: "\e4de";
}

.fa-head-side-cough {
  --fa: "\e061";
}

.fa-grip-lines {
  --fa: "\f7a4";
}

.fa-thumbs-down {
  --fa: "\f165";
}

.fa-user-lock {
  --fa: "\f502";
}

.fa-arrow-right-long {
  --fa: "\f178";
}

.fa-long-arrow-right {
  --fa: "\f178";
}

.fa-anchor-circle-xmark {
  --fa: "\e4ac";
}

.fa-ellipsis {
  --fa: "\f141";
}

.fa-ellipsis-h {
  --fa: "\f141";
}

.fa-chess-pawn {
  --fa: "\f443";
}

.fa-kit-medical {
  --fa: "\f479";
}

.fa-first-aid {
  --fa: "\f479";
}

.fa-person-through-window {
  --fa: "\e5a9";
}

.fa-toolbox {
  --fa: "\f552";
}

.fa-hands-holding-circle {
  --fa: "\e4fb";
}

.fa-bug {
  --fa: "\f188";
}

.fa-credit-card {
  --fa: "\f09d";
}

.fa-credit-card-alt {
  --fa: "\f09d";
}

.fa-car {
  --fa: "\f1b9";
}

.fa-automobile {
  --fa: "\f1b9";
}

.fa-hand-holding-hand {
  --fa: "\e4f7";
}

.fa-book-open-reader {
  --fa: "\f5da";
}

.fa-book-reader {
  --fa: "\f5da";
}

.fa-mountain-sun {
  --fa: "\e52f";
}

.fa-arrows-left-right-to-line {
  --fa: "\e4ba";
}

.fa-dice-d20 {
  --fa: "\f6cf";
}

.fa-truck-droplet {
  --fa: "\e58c";
}

.fa-file-circle-xmark {
  --fa: "\e5a1";
}

.fa-temperature-arrow-up {
  --fa: "\e040";
}

.fa-temperature-up {
  --fa: "\e040";
}

.fa-medal {
  --fa: "\f5a2";
}

.fa-bed {
  --fa: "\f236";
}

.fa-square-h {
  --fa: "\f0fd";
}

.fa-h-square {
  --fa: "\f0fd";
}

.fa-podcast {
  --fa: "\f2ce";
}

.fa-temperature-full {
  --fa: "\f2c7";
}

.fa-temperature-4 {
  --fa: "\f2c7";
}

.fa-thermometer-4 {
  --fa: "\f2c7";
}

.fa-thermometer-full {
  --fa: "\f2c7";
}

.fa-bell {
  --fa: "\f0f3";
}

.fa-superscript {
  --fa: "\f12b";
}

.fa-plug-circle-xmark {
  --fa: "\e560";
}

.fa-star-of-life {
  --fa: "\f621";
}

.fa-phone-slash {
  --fa: "\f3dd";
}

.fa-paint-roller {
  --fa: "\f5aa";
}

.fa-handshake-angle {
  --fa: "\f4c4";
}

.fa-hands-helping {
  --fa: "\f4c4";
}

.fa-location-dot {
  --fa: "\f3c5";
}

.fa-map-marker-alt {
  --fa: "\f3c5";
}

.fa-file {
  --fa: "\f15b";
}

.fa-greater-than {
  --fa: "\>";
}

.fa-person-swimming {
  --fa: "\f5c4";
}

.fa-swimmer {
  --fa: "\f5c4";
}

.fa-arrow-down {
  --fa: "\f063";
}

.fa-droplet {
  --fa: "\f043";
}

.fa-tint {
  --fa: "\f043";
}

.fa-eraser {
  --fa: "\f12d";
}

.fa-earth-americas {
  --fa: "\f57d";
}

.fa-earth {
  --fa: "\f57d";
}

.fa-earth-america {
  --fa: "\f57d";
}

.fa-globe-americas {
  --fa: "\f57d";
}

.fa-person-burst {
  --fa: "\e53b";
}

.fa-dove {
  --fa: "\f4ba";
}

.fa-battery-empty {
  --fa: "\f244";
}

.fa-battery-0 {
  --fa: "\f244";
}

.fa-socks {
  --fa: "\f696";
}

.fa-inbox {
  --fa: "\f01c";
}

.fa-section {
  --fa: "\e447";
}

.fa-gauge-high {
  --fa: "\f625";
}

.fa-tachometer-alt {
  --fa: "\f625";
}

.fa-tachometer-alt-fast {
  --fa: "\f625";
}

.fa-envelope-open-text {
  --fa: "\f658";
}

.fa-hospital {
  --fa: "\f0f8";
}

.fa-hospital-alt {
  --fa: "\f0f8";
}

.fa-hospital-wide {
  --fa: "\f0f8";
}

.fa-wine-bottle {
  --fa: "\f72f";
}

.fa-chess-rook {
  --fa: "\f447";
}

.fa-bars-staggered {
  --fa: "\f550";
}

.fa-reorder {
  --fa: "\f550";
}

.fa-stream {
  --fa: "\f550";
}

.fa-dharmachakra {
  --fa: "\f655";
}

.fa-hotdog {
  --fa: "\f80f";
}

.fa-person-walking-with-cane {
  --fa: "\f29d";
}

.fa-blind {
  --fa: "\f29d";
}

.fa-drum {
  --fa: "\f569";
}

.fa-ice-cream {
  --fa: "\f810";
}

.fa-heart-circle-bolt {
  --fa: "\e4fc";
}

.fa-fax {
  --fa: "\f1ac";
}

.fa-paragraph {
  --fa: "\f1dd";
}

.fa-check-to-slot {
  --fa: "\f772";
}

.fa-vote-yea {
  --fa: "\f772";
}

.fa-star-half {
  --fa: "\f089";
}

.fa-boxes-stacked {
  --fa: "\f468";
}

.fa-boxes {
  --fa: "\f468";
}

.fa-boxes-alt {
  --fa: "\f468";
}

.fa-link {
  --fa: "\f0c1";
}

.fa-chain {
  --fa: "\f0c1";
}

.fa-ear-listen {
  --fa: "\f2a2";
}

.fa-assistive-listening-systems {
  --fa: "\f2a2";
}

.fa-tree-city {
  --fa: "\e587";
}

.fa-play {
  --fa: "\f04b";
}

.fa-font {
  --fa: "\f031";
}

.fa-table-cells-row-lock {
  --fa: "\e67a";
}

.fa-rupiah-sign {
  --fa: "\e23d";
}

.fa-magnifying-glass {
  --fa: "\f002";
}

.fa-search {
  --fa: "\f002";
}

.fa-table-tennis-paddle-ball {
  --fa: "\f45d";
}

.fa-ping-pong-paddle-ball {
  --fa: "\f45d";
}

.fa-table-tennis {
  --fa: "\f45d";
}

.fa-person-dots-from-line {
  --fa: "\f470";
}

.fa-diagnoses {
  --fa: "\f470";
}

.fa-trash-can-arrow-up {
  --fa: "\f82a";
}

.fa-trash-restore-alt {
  --fa: "\f82a";
}

.fa-naira-sign {
  --fa: "\e1f6";
}

.fa-cart-arrow-down {
  --fa: "\f218";
}

.fa-walkie-talkie {
  --fa: "\f8ef";
}

.fa-file-pen {
  --fa: "\f31c";
}

.fa-file-edit {
  --fa: "\f31c";
}

.fa-receipt {
  --fa: "\f543";
}

.fa-square-pen {
  --fa: "\f14b";
}

.fa-pen-square {
  --fa: "\f14b";
}

.fa-pencil-square {
  --fa: "\f14b";
}

.fa-suitcase-rolling {
  --fa: "\f5c1";
}

.fa-person-circle-exclamation {
  --fa: "\e53f";
}

.fa-chevron-down {
  --fa: "\f078";
}

.fa-battery-full {
  --fa: "\f240";
}

.fa-battery {
  --fa: "\f240";
}

.fa-battery-5 {
  --fa: "\f240";
}

.fa-skull-crossbones {
  --fa: "\f714";
}

.fa-code-compare {
  --fa: "\e13a";
}

.fa-list-ul {
  --fa: "\f0ca";
}

.fa-list-dots {
  --fa: "\f0ca";
}

.fa-school-lock {
  --fa: "\e56f";
}

.fa-tower-cell {
  --fa: "\e585";
}

.fa-down-long {
  --fa: "\f309";
}

.fa-long-arrow-alt-down {
  --fa: "\f309";
}

.fa-ranking-star {
  --fa: "\e561";
}

.fa-chess-king {
  --fa: "\f43f";
}

.fa-person-harassing {
  --fa: "\e549";
}

.fa-brazilian-real-sign {
  --fa: "\e46c";
}

.fa-landmark-dome {
  --fa: "\f752";
}

.fa-landmark-alt {
  --fa: "\f752";
}

.fa-arrow-up {
  --fa: "\f062";
}

.fa-tv {
  --fa: "\f26c";
}

.fa-television {
  --fa: "\f26c";
}

.fa-tv-alt {
  --fa: "\f26c";
}

.fa-shrimp {
  --fa: "\e448";
}

.fa-list-check {
  --fa: "\f0ae";
}

.fa-tasks {
  --fa: "\f0ae";
}

.fa-jug-detergent {
  --fa: "\e519";
}

.fa-circle-user {
  --fa: "\f2bd";
}

.fa-user-circle {
  --fa: "\f2bd";
}

.fa-user-shield {
  --fa: "\f505";
}

.fa-wind {
  --fa: "\f72e";
}

.fa-car-burst {
  --fa: "\f5e1";
}

.fa-car-crash {
  --fa: "\f5e1";
}

.fa-y {
  --fa: "Y";
}

.fa-person-snowboarding {
  --fa: "\f7ce";
}

.fa-snowboarding {
  --fa: "\f7ce";
}

.fa-truck-fast {
  --fa: "\f48b";
}

.fa-shipping-fast {
  --fa: "\f48b";
}

.fa-fish {
  --fa: "\f578";
}

.fa-user-graduate {
  --fa: "\f501";
}

.fa-circle-half-stroke {
  --fa: "\f042";
}

.fa-adjust {
  --fa: "\f042";
}

.fa-clapperboard {
  --fa: "\e131";
}

.fa-circle-radiation {
  --fa: "\f7ba";
}

.fa-radiation-alt {
  --fa: "\f7ba";
}

.fa-baseball {
  --fa: "\f433";
}

.fa-baseball-ball {
  --fa: "\f433";
}

.fa-jet-fighter-up {
  --fa: "\e518";
}

.fa-diagram-project {
  --fa: "\f542";
}

.fa-project-diagram {
  --fa: "\f542";
}

.fa-copy {
  --fa: "\f0c5";
}

.fa-volume-xmark {
  --fa: "\f6a9";
}

.fa-volume-mute {
  --fa: "\f6a9";
}

.fa-volume-times {
  --fa: "\f6a9";
}

.fa-hand-sparkles {
  --fa: "\e05d";
}

.fa-grip {
  --fa: "\f58d";
}

.fa-grip-horizontal {
  --fa: "\f58d";
}

.fa-share-from-square {
  --fa: "\f14d";
}

.fa-share-square {
  --fa: "\f14d";
}

.fa-child-combatant {
  --fa: "\e4e0";
}

.fa-child-rifle {
  --fa: "\e4e0";
}

.fa-gun {
  --fa: "\e19b";
}

.fa-square-phone {
  --fa: "\f098";
}

.fa-phone-square {
  --fa: "\f098";
}

.fa-plus {
  --fa: "\+";
}

.fa-add {
  --fa: "\+";
}

.fa-expand {
  --fa: "\f065";
}

.fa-computer {
  --fa: "\e4e5";
}

.fa-xmark {
  --fa: "\f00d";
}

.fa-close {
  --fa: "\f00d";
}

.fa-multiply {
  --fa: "\f00d";
}

.fa-remove {
  --fa: "\f00d";
}

.fa-times {
  --fa: "\f00d";
}

.fa-arrows-up-down-left-right {
  --fa: "\f047";
}

.fa-arrows {
  --fa: "\f047";
}

.fa-chalkboard-user {
  --fa: "\f51c";
}

.fa-chalkboard-teacher {
  --fa: "\f51c";
}

.fa-peso-sign {
  --fa: "\e222";
}

.fa-building-shield {
  --fa: "\e4d8";
}

.fa-baby {
  --fa: "\f77c";
}

.fa-users-line {
  --fa: "\e592";
}

.fa-quote-left {
  --fa: "\f10d";
}

.fa-quote-left-alt {
  --fa: "\f10d";
}

.fa-tractor {
  --fa: "\f722";
}

.fa-trash-arrow-up {
  --fa: "\f829";
}

.fa-trash-restore {
  --fa: "\f829";
}

.fa-arrow-down-up-lock {
  --fa: "\e4b0";
}

.fa-lines-leaning {
  --fa: "\e51e";
}

.fa-ruler-combined {
  --fa: "\f546";
}

.fa-copyright {
  --fa: "\f1f9";
}

.fa-equals {
  --fa: "\=";
}

.fa-blender {
  --fa: "\f517";
}

.fa-teeth {
  --fa: "\f62e";
}

.fa-shekel-sign {
  --fa: "\f20b";
}

.fa-ils {
  --fa: "\f20b";
}

.fa-shekel {
  --fa: "\f20b";
}

.fa-sheqel {
  --fa: "\f20b";
}

.fa-sheqel-sign {
  --fa: "\f20b";
}

.fa-map {
  --fa: "\f279";
}

.fa-rocket {
  --fa: "\f135";
}

.fa-photo-film {
  --fa: "\f87c";
}

.fa-photo-video {
  --fa: "\f87c";
}

.fa-folder-minus {
  --fa: "\f65d";
}

.fa-hexagon-nodes-bolt {
  --fa: "\e69a";
}

.fa-store {
  --fa: "\f54e";
}

.fa-arrow-trend-up {
  --fa: "\e098";
}

.fa-plug-circle-minus {
  --fa: "\e55e";
}

.fa-sign-hanging {
  --fa: "\f4d9";
}

.fa-sign {
  --fa: "\f4d9";
}

.fa-bezier-curve {
  --fa: "\f55b";
}

.fa-bell-slash {
  --fa: "\f1f6";
}

.fa-tablet {
  --fa: "\f3fb";
}

.fa-tablet-android {
  --fa: "\f3fb";
}

.fa-school-flag {
  --fa: "\e56e";
}

.fa-fill {
  --fa: "\f575";
}

.fa-angle-up {
  --fa: "\f106";
}

.fa-drumstick-bite {
  --fa: "\f6d7";
}

.fa-holly-berry {
  --fa: "\f7aa";
}

.fa-chevron-left {
  --fa: "\f053";
}

.fa-bacteria {
  --fa: "\e059";
}

.fa-hand-lizard {
  --fa: "\f258";
}

.fa-notdef {
  --fa: "\e1fe";
}

.fa-disease {
  --fa: "\f7fa";
}

.fa-briefcase-medical {
  --fa: "\f469";
}

.fa-genderless {
  --fa: "\f22d";
}

.fa-chevron-right {
  --fa: "\f054";
}

.fa-retweet {
  --fa: "\f079";
}

.fa-car-rear {
  --fa: "\f5de";
}

.fa-car-alt {
  --fa: "\f5de";
}

.fa-pump-soap {
  --fa: "\e06b";
}

.fa-video-slash {
  --fa: "\f4e2";
}

.fa-battery-quarter {
  --fa: "\f243";
}

.fa-battery-2 {
  --fa: "\f243";
}

.fa-radio {
  --fa: "\f8d7";
}

.fa-baby-carriage {
  --fa: "\f77d";
}

.fa-carriage-baby {
  --fa: "\f77d";
}

.fa-traffic-light {
  --fa: "\f637";
}

.fa-thermometer {
  --fa: "\f491";
}

.fa-vr-cardboard {
  --fa: "\f729";
}

.fa-hand-middle-finger {
  --fa: "\f806";
}

.fa-percent {
  --fa: "\%";
}

.fa-percentage {
  --fa: "\%";
}

.fa-truck-moving {
  --fa: "\f4df";
}

.fa-glass-water-droplet {
  --fa: "\e4f5";
}

.fa-display {
  --fa: "\e163";
}

.fa-face-smile {
  --fa: "\f118";
}

.fa-smile {
  --fa: "\f118";
}

.fa-thumbtack {
  --fa: "\f08d";
}

.fa-thumb-tack {
  --fa: "\f08d";
}

.fa-trophy {
  --fa: "\f091";
}

.fa-person-praying {
  --fa: "\f683";
}

.fa-pray {
  --fa: "\f683";
}

.fa-hammer {
  --fa: "\f6e3";
}

.fa-hand-peace {
  --fa: "\f25b";
}

.fa-rotate {
  --fa: "\f2f1";
}

.fa-sync-alt {
  --fa: "\f2f1";
}

.fa-spinner {
  --fa: "\f110";
}

.fa-robot {
  --fa: "\f544";
}

.fa-peace {
  --fa: "\f67c";
}

.fa-gears {
  --fa: "\f085";
}

.fa-cogs {
  --fa: "\f085";
}

.fa-warehouse {
  --fa: "\f494";
}

.fa-arrow-up-right-dots {
  --fa: "\e4b7";
}

.fa-splotch {
  --fa: "\f5bc";
}

.fa-face-grin-hearts {
  --fa: "\f584";
}

.fa-grin-hearts {
  --fa: "\f584";
}

.fa-dice-four {
  --fa: "\f524";
}

.fa-sim-card {
  --fa: "\f7c4";
}

.fa-transgender {
  --fa: "\f225";
}

.fa-transgender-alt {
  --fa: "\f225";
}

.fa-mercury {
  --fa: "\f223";
}

.fa-arrow-turn-down {
  --fa: "\f149";
}

.fa-level-down {
  --fa: "\f149";
}

.fa-person-falling-burst {
  --fa: "\e547";
}

.fa-award {
  --fa: "\f559";
}

.fa-ticket-simple {
  --fa: "\f3ff";
}

.fa-ticket-alt {
  --fa: "\f3ff";
}

.fa-building {
  --fa: "\f1ad";
}

.fa-angles-left {
  --fa: "\f100";
}

.fa-angle-double-left {
  --fa: "\f100";
}

.fa-qrcode {
  --fa: "\f029";
}

.fa-clock-rotate-left {
  --fa: "\f1da";
}

.fa-history {
  --fa: "\f1da";
}

.fa-face-grin-beam-sweat {
  --fa: "\f583";
}

.fa-grin-beam-sweat {
  --fa: "\f583";
}

.fa-file-export {
  --fa: "\f56e";
}

.fa-arrow-right-from-file {
  --fa: "\f56e";
}

.fa-shield {
  --fa: "\f132";
}

.fa-shield-blank {
  --fa: "\f132";
}

.fa-arrow-up-short-wide {
  --fa: "\f885";
}

.fa-sort-amount-up-alt {
  --fa: "\f885";
}

.fa-comment-nodes {
  --fa: "\e696";
}

.fa-house-medical {
  --fa: "\e3b2";
}

.fa-golf-ball-tee {
  --fa: "\f450";
}

.fa-golf-ball {
  --fa: "\f450";
}

.fa-circle-chevron-left {
  --fa: "\f137";
}

.fa-chevron-circle-left {
  --fa: "\f137";
}

.fa-house-chimney-window {
  --fa: "\e00d";
}

.fa-pen-nib {
  --fa: "\f5ad";
}

.fa-tent-arrow-turn-left {
  --fa: "\e580";
}

.fa-tents {
  --fa: "\e582";
}

.fa-wand-magic {
  --fa: "\f0d0";
}

.fa-magic {
  --fa: "\f0d0";
}

.fa-dog {
  --fa: "\f6d3";
}

.fa-carrot {
  --fa: "\f787";
}

.fa-moon {
  --fa: "\f186";
}

.fa-wine-glass-empty {
  --fa: "\f5ce";
}

.fa-wine-glass-alt {
  --fa: "\f5ce";
}

.fa-cheese {
  --fa: "\f7ef";
}

.fa-yin-yang {
  --fa: "\f6ad";
}

.fa-music {
  --fa: "\f001";
}

.fa-code-commit {
  --fa: "\f386";
}

.fa-temperature-low {
  --fa: "\f76b";
}

.fa-person-biking {
  --fa: "\f84a";
}

.fa-biking {
  --fa: "\f84a";
}

.fa-broom {
  --fa: "\f51a";
}

.fa-shield-heart {
  --fa: "\e574";
}

.fa-gopuram {
  --fa: "\f664";
}

.fa-earth-oceania {
  --fa: "\e47b";
}

.fa-globe-oceania {
  --fa: "\e47b";
}

.fa-square-xmark {
  --fa: "\f2d3";
}

.fa-times-square {
  --fa: "\f2d3";
}

.fa-xmark-square {
  --fa: "\f2d3";
}

.fa-hashtag {
  --fa: "\#";
}

.fa-up-right-and-down-left-from-center {
  --fa: "\f424";
}

.fa-expand-alt {
  --fa: "\f424";
}

.fa-oil-can {
  --fa: "\f613";
}

.fa-t {
  --fa: "T";
}

.fa-hippo {
  --fa: "\f6ed";
}

.fa-chart-column {
  --fa: "\e0e3";
}

.fa-infinity {
  --fa: "\f534";
}

.fa-vial-circle-check {
  --fa: "\e596";
}

.fa-person-arrow-down-to-line {
  --fa: "\e538";
}

.fa-voicemail {
  --fa: "\f897";
}

.fa-fan {
  --fa: "\f863";
}

.fa-person-walking-luggage {
  --fa: "\e554";
}

.fa-up-down {
  --fa: "\f338";
}

.fa-arrows-alt-v {
  --fa: "\f338";
}

.fa-cloud-moon-rain {
  --fa: "\f73c";
}

.fa-calendar {
  --fa: "\f133";
}

.fa-trailer {
  --fa: "\e041";
}

.fa-bahai {
  --fa: "\f666";
}

.fa-haykal {
  --fa: "\f666";
}

.fa-sd-card {
  --fa: "\f7c2";
}

.fa-dragon {
  --fa: "\f6d5";
}

.fa-shoe-prints {
  --fa: "\f54b";
}

.fa-circle-plus {
  --fa: "\f055";
}

.fa-plus-circle {
  --fa: "\f055";
}

.fa-face-grin-tongue-wink {
  --fa: "\f58b";
}

.fa-grin-tongue-wink {
  --fa: "\f58b";
}

.fa-hand-holding {
  --fa: "\f4bd";
}

.fa-plug-circle-exclamation {
  --fa: "\e55d";
}

.fa-link-slash {
  --fa: "\f127";
}

.fa-chain-broken {
  --fa: "\f127";
}

.fa-chain-slash {
  --fa: "\f127";
}

.fa-unlink {
  --fa: "\f127";
}

.fa-clone {
  --fa: "\f24d";
}

.fa-person-walking-arrow-loop-left {
  --fa: "\e551";
}

.fa-arrow-up-z-a {
  --fa: "\f882";
}

.fa-sort-alpha-up-alt {
  --fa: "\f882";
}

.fa-fire-flame-curved {
  --fa: "\f7e4";
}

.fa-fire-alt {
  --fa: "\f7e4";
}

.fa-tornado {
  --fa: "\f76f";
}

.fa-file-circle-plus {
  --fa: "\e494";
}

.fa-book-quran {
  --fa: "\f687";
}

.fa-quran {
  --fa: "\f687";
}

.fa-anchor {
  --fa: "\f13d";
}

.fa-border-all {
  --fa: "\f84c";
}

.fa-face-angry {
  --fa: "\f556";
}

.fa-angry {
  --fa: "\f556";
}

.fa-cookie-bite {
  --fa: "\f564";
}

.fa-arrow-trend-down {
  --fa: "\e097";
}

.fa-rss {
  --fa: "\f09e";
}

.fa-feed {
  --fa: "\f09e";
}

.fa-draw-polygon {
  --fa: "\f5ee";
}

.fa-scale-balanced {
  --fa: "\f24e";
}

.fa-balance-scale {
  --fa: "\f24e";
}

.fa-gauge-simple-high {
  --fa: "\f62a";
}

.fa-tachometer {
  --fa: "\f62a";
}

.fa-tachometer-fast {
  --fa: "\f62a";
}

.fa-shower {
  --fa: "\f2cc";
}

.fa-desktop {
  --fa: "\f390";
}

.fa-desktop-alt {
  --fa: "\f390";
}

.fa-m {
  --fa: "M";
}

.fa-table-list {
  --fa: "\f00b";
}

.fa-th-list {
  --fa: "\f00b";
}

.fa-comment-sms {
  --fa: "\f7cd";
}

.fa-sms {
  --fa: "\f7cd";
}

.fa-book {
  --fa: "\f02d";
}

.fa-user-plus {
  --fa: "\f234";
}

.fa-check {
  --fa: "\f00c";
}

.fa-battery-three-quarters {
  --fa: "\f241";
}

.fa-battery-4 {
  --fa: "\f241";
}

.fa-house-circle-check {
  --fa: "\e509";
}

.fa-angle-left {
  --fa: "\f104";
}

.fa-diagram-successor {
  --fa: "\e47a";
}

.fa-truck-arrow-right {
  --fa: "\e58b";
}

.fa-arrows-split-up-and-left {
  --fa: "\e4bc";
}

.fa-hand-fist {
  --fa: "\f6de";
}

.fa-fist-raised {
  --fa: "\f6de";
}

.fa-cloud-moon {
  --fa: "\f6c3";
}

.fa-briefcase {
  --fa: "\f0b1";
}

.fa-person-falling {
  --fa: "\e546";
}

.fa-image-portrait {
  --fa: "\f3e0";
}

.fa-portrait {
  --fa: "\f3e0";
}

.fa-user-tag {
  --fa: "\f507";
}

.fa-rug {
  --fa: "\e569";
}

.fa-earth-europe {
  --fa: "\f7a2";
}

.fa-globe-europe {
  --fa: "\f7a2";
}

.fa-cart-flatbed-suitcase {
  --fa: "\f59d";
}

.fa-luggage-cart {
  --fa: "\f59d";
}

.fa-rectangle-xmark {
  --fa: "\f410";
}

.fa-rectangle-times {
  --fa: "\f410";
}

.fa-times-rectangle {
  --fa: "\f410";
}

.fa-window-close {
  --fa: "\f410";
}

.fa-baht-sign {
  --fa: "\e0ac";
}

.fa-book-open {
  --fa: "\f518";
}

.fa-book-journal-whills {
  --fa: "\f66a";
}

.fa-journal-whills {
  --fa: "\f66a";
}

.fa-handcuffs {
  --fa: "\e4f8";
}

.fa-triangle-exclamation {
  --fa: "\f071";
}

.fa-exclamation-triangle {
  --fa: "\f071";
}

.fa-warning {
  --fa: "\f071";
}

.fa-database {
  --fa: "\f1c0";
}

.fa-share {
  --fa: "\f064";
}

.fa-mail-forward {
  --fa: "\f064";
}

.fa-bottle-droplet {
  --fa: "\e4c4";
}

.fa-mask-face {
  --fa: "\e1d7";
}

.fa-hill-rockslide {
  --fa: "\e508";
}

.fa-right-left {
  --fa: "\f362";
}

.fa-exchange-alt {
  --fa: "\f362";
}

.fa-paper-plane {
  --fa: "\f1d8";
}

.fa-road-circle-exclamation {
  --fa: "\e565";
}

.fa-dungeon {
  --fa: "\f6d9";
}

.fa-align-right {
  --fa: "\f038";
}

.fa-money-bill-1-wave {
  --fa: "\f53b";
}

.fa-money-bill-wave-alt {
  --fa: "\f53b";
}

.fa-life-ring {
  --fa: "\f1cd";
}

.fa-hands {
  --fa: "\f2a7";
}

.fa-sign-language {
  --fa: "\f2a7";
}

.fa-signing {
  --fa: "\f2a7";
}

.fa-calendar-day {
  --fa: "\f783";
}

.fa-water-ladder {
  --fa: "\f5c5";
}

.fa-ladder-water {
  --fa: "\f5c5";
}

.fa-swimming-pool {
  --fa: "\f5c5";
}

.fa-arrows-up-down {
  --fa: "\f07d";
}

.fa-arrows-v {
  --fa: "\f07d";
}

.fa-face-grimace {
  --fa: "\f57f";
}

.fa-grimace {
  --fa: "\f57f";
}

.fa-wheelchair-move {
  --fa: "\e2ce";
}

.fa-wheelchair-alt {
  --fa: "\e2ce";
}

.fa-turn-down {
  --fa: "\f3be";
}

.fa-level-down-alt {
  --fa: "\f3be";
}

.fa-person-walking-arrow-right {
  --fa: "\e552";
}

.fa-square-envelope {
  --fa: "\f199";
}

.fa-envelope-square {
  --fa: "\f199";
}

.fa-dice {
  --fa: "\f522";
}

.fa-bowling-ball {
  --fa: "\f436";
}

.fa-brain {
  --fa: "\f5dc";
}

.fa-bandage {
  --fa: "\f462";
}

.fa-band-aid {
  --fa: "\f462";
}

.fa-calendar-minus {
  --fa: "\f272";
}

.fa-circle-xmark {
  --fa: "\f057";
}

.fa-times-circle {
  --fa: "\f057";
}

.fa-xmark-circle {
  --fa: "\f057";
}

.fa-gifts {
  --fa: "\f79c";
}

.fa-hotel {
  --fa: "\f594";
}

.fa-earth-asia {
  --fa: "\f57e";
}

.fa-globe-asia {
  --fa: "\f57e";
}

.fa-id-card-clip {
  --fa: "\f47f";
}

.fa-id-card-alt {
  --fa: "\f47f";
}

.fa-magnifying-glass-plus {
  --fa: "\f00e";
}

.fa-search-plus {
  --fa: "\f00e";
}

.fa-thumbs-up {
  --fa: "\f164";
}

.fa-user-clock {
  --fa: "\f4fd";
}

.fa-hand-dots {
  --fa: "\f461";
}

.fa-allergies {
  --fa: "\f461";
}

.fa-file-invoice {
  --fa: "\f570";
}

.fa-window-minimize {
  --fa: "\f2d1";
}

.fa-mug-saucer {
  --fa: "\f0f4";
}

.fa-coffee {
  --fa: "\f0f4";
}

.fa-brush {
  --fa: "\f55d";
}

.fa-file-half-dashed {
  --fa: "\e698";
}

.fa-mask {
  --fa: "\f6fa";
}

.fa-magnifying-glass-minus {
  --fa: "\f010";
}

.fa-search-minus {
  --fa: "\f010";
}

.fa-ruler-vertical {
  --fa: "\f548";
}

.fa-user-large {
  --fa: "\f406";
}

.fa-user-alt {
  --fa: "\f406";
}

.fa-train-tram {
  --fa: "\e5b4";
}

.fa-user-nurse {
  --fa: "\f82f";
}

.fa-syringe {
  --fa: "\f48e";
}

.fa-cloud-sun {
  --fa: "\f6c4";
}

.fa-stopwatch-20 {
  --fa: "\e06f";
}

.fa-square-full {
  --fa: "\f45c";
}

.fa-magnet {
  --fa: "\f076";
}

.fa-jar {
  --fa: "\e516";
}

.fa-note-sticky {
  --fa: "\f249";
}

.fa-sticky-note {
  --fa: "\f249";
}

.fa-bug-slash {
  --fa: "\e490";
}

.fa-arrow-up-from-water-pump {
  --fa: "\e4b6";
}

.fa-bone {
  --fa: "\f5d7";
}

.fa-table-cells-row-unlock {
  --fa: "\e691";
}

.fa-user-injured {
  --fa: "\f728";
}

.fa-face-sad-tear {
  --fa: "\f5b4";
}

.fa-sad-tear {
  --fa: "\f5b4";
}

.fa-plane {
  --fa: "\f072";
}

.fa-tent-arrows-down {
  --fa: "\e581";
}

.fa-exclamation {
  --fa: "\!";
}

.fa-arrows-spin {
  --fa: "\e4bb";
}

.fa-print {
  --fa: "\f02f";
}

.fa-turkish-lira-sign {
  --fa: "\e2bb";
}

.fa-try {
  --fa: "\e2bb";
}

.fa-turkish-lira {
  --fa: "\e2bb";
}

.fa-dollar-sign {
  --fa: "\$";
}

.fa-dollar {
  --fa: "\$";
}

.fa-usd {
  --fa: "\$";
}

.fa-x {
  --fa: "X";
}

.fa-magnifying-glass-dollar {
  --fa: "\f688";
}

.fa-search-dollar {
  --fa: "\f688";
}

.fa-users-gear {
  --fa: "\f509";
}

.fa-users-cog {
  --fa: "\f509";
}

.fa-person-military-pointing {
  --fa: "\e54a";
}

.fa-building-columns {
  --fa: "\f19c";
}

.fa-bank {
  --fa: "\f19c";
}

.fa-institution {
  --fa: "\f19c";
}

.fa-museum {
  --fa: "\f19c";
}

.fa-university {
  --fa: "\f19c";
}

.fa-umbrella {
  --fa: "\f0e9";
}

.fa-trowel {
  --fa: "\e589";
}

.fa-d {
  --fa: "D";
}

.fa-stapler {
  --fa: "\e5af";
}

.fa-masks-theater {
  --fa: "\f630";
}

.fa-theater-masks {
  --fa: "\f630";
}

.fa-kip-sign {
  --fa: "\e1c4";
}

.fa-hand-point-left {
  --fa: "\f0a5";
}

.fa-handshake-simple {
  --fa: "\f4c6";
}

.fa-handshake-alt {
  --fa: "\f4c6";
}

.fa-jet-fighter {
  --fa: "\f0fb";
}

.fa-fighter-jet {
  --fa: "\f0fb";
}

.fa-square-share-nodes {
  --fa: "\f1e1";
}

.fa-share-alt-square {
  --fa: "\f1e1";
}

.fa-barcode {
  --fa: "\f02a";
}

.fa-plus-minus {
  --fa: "\e43c";
}

.fa-video {
  --fa: "\f03d";
}

.fa-video-camera {
  --fa: "\f03d";
}

.fa-graduation-cap {
  --fa: "\f19d";
}

.fa-mortar-board {
  --fa: "\f19d";
}

.fa-hand-holding-medical {
  --fa: "\e05c";
}

.fa-person-circle-check {
  --fa: "\e53e";
}

.fa-turn-up {
  --fa: "\f3bf";
}

.fa-level-up-alt {
  --fa: "\f3bf";
}

.sr-only,
.fa-sr-only {
  position: absolute;
  width: 1px;
  height: 1px;
  padding: 0;
  margin: -1px;
  overflow: hidden;
  clip: rect(0, 0, 0, 0);
  white-space: nowrap;
  border-width: 0;
}

.sr-only-focusable:not(:focus),
.fa-sr-only-focusable:not(:focus) {
  position: absolute;
  width: 1px;
  height: 1px;
  padding: 0;
  margin: -1px;
  overflow: hidden;
  clip: rect(0, 0, 0, 0);
  white-space: nowrap;
  border-width: 0;
}

/*!
 * Bootstrap  v5.3.3 (https://getbootstrap.com/)
 * Copyright 2011-2024 The Bootstrap Authors
 * Licensed under MIT (https://github.com/twbs/bootstrap/blob/main/LICENSE)
 */
:root,
[data-bs-theme=light] {
  --bs-blue: #0f6cbf;
  --bs-indigo: #6610f2;
  --bs-purple: #613d7c;
  --bs-pink: #db1a74;
  --bs-red: #ca3120;
  --bs-orange: #ff7518;
  --bs-yellow: #f0ad4e;
  --bs-green: #357a32;
  --bs-teal: #20c997;
  --bs-cyan: #008196;
  --bs-black: #000;
  --bs-white: #fff;
  --bs-gray: #6a737b;
  --bs-gray-dark: #343a40;
  --bs-gray-100: #f8f9fa;
  --bs-gray-200: #e9ecef;
  --bs-gray-300: #dee2e6;
  --bs-gray-400: #ced4da;
  --bs-gray-500: #8f959e;
  --bs-gray-600: #6a737b;
  --bs-gray-700: #495057;
  --bs-gray-800: #343a40;
  --bs-gray-900: #1d2125;
  --bs-primary: #0f6cbf;
  --bs-secondary: #ced4da;
  --bs-success: #357a32;
  --bs-info: #008196;
  --bs-warning: #f0ad4e;
  --bs-danger: #ca3120;
  --bs-light: #f8f9fa;
  --bs-dark: #1d2125;
  --bs-primary-rgb: 15, 108, 191;
  --bs-secondary-rgb: 206, 212, 218;
  --bs-success-rgb: 53, 122, 50;
  --bs-info-rgb: 0, 129, 150;
  --bs-warning-rgb: 240, 173, 78;
  --bs-danger-rgb: 202, 49, 32;
  --bs-light-rgb: 248, 249, 250;
  --bs-dark-rgb: 29, 33, 37;
  --bs-primary-text-emphasis: #062b4c;
  --bs-secondary-text-emphasis: #525557;
  --bs-success-text-emphasis: #153114;
  --bs-info-text-emphasis: #00343c;
  --bs-warning-text-emphasis: #60451f;
  --bs-danger-text-emphasis: #51140d;
  --bs-light-text-emphasis: #495057;
  --bs-dark-text-emphasis: #495057;
  --bs-primary-bg-subtle: #cfe2f2;
  --bs-secondary-bg-subtle: #f5f6f8;
  --bs-success-bg-subtle: #d7e4d6;
  --bs-info-bg-subtle: #cce6ea;
  --bs-warning-bg-subtle: #fcefdc;
  --bs-danger-bg-subtle: #f4d6d2;
  --bs-light-bg-subtle: #fcfcfd;
  --bs-dark-bg-subtle: #ced4da;
  --bs-primary-border-subtle: #9fc4e5;
  --bs-secondary-border-subtle: #ebeef0;
  --bs-success-border-subtle: #aecaad;
  --bs-info-border-subtle: #99cdd5;
  --bs-warning-border-subtle: #f9deb8;
  --bs-danger-border-subtle: #eaada6;
  --bs-light-border-subtle: #e9ecef;
  --bs-dark-border-subtle: #8f959e;
  --bs-white-rgb: 255, 255, 255;
  --bs-black-rgb: 0, 0, 0;
  --bs-font-sans-serif: system-ui, -apple-system, "Segoe UI", Roboto, "Helvetica Neue", "Noto Sans", "Liberation Sans", Arial, sans-serif, "Apple Color Emoji", "Segoe UI Emoji", "Segoe UI Symbol", "Noto Color Emoji";
  --bs-font-monospace: SFMono-Regular, Menlo, Monaco, Consolas, "Liberation Mono", "Courier New", monospace;
  --bs-gradient: linear-gradient(180deg, rgba(255, 255, 255, 0.15), rgba(255, 255, 255, 0));
  --bs-body-font-family: var(--bs-font-sans-serif);
  --bs-body-font-size: 0.9375rem;
  --bs-body-font-weight: 400;
  --bs-body-line-height: 1.5;
  --bs-body-color: #1d2125;
  --bs-body-color-rgb: 29, 33, 37;
  --bs-body-bg: #fff;
  --bs-body-bg-rgb: 255, 255, 255;
  --bs-emphasis-color: #000;
  --bs-emphasis-color-rgb: 0, 0, 0;
  --bs-secondary-color: rgba(29, 33, 37, 0.75);
  --bs-secondary-color-rgb: 29, 33, 37;
  --bs-secondary-bg: #e9ecef;
  --bs-secondary-bg-rgb: 233, 236, 239;
  --bs-tertiary-color: rgba(29, 33, 37, 0.5);
  --bs-tertiary-color-rgb: 29, 33, 37;
  --bs-tertiary-bg: #f8f9fa;
  --bs-tertiary-bg-rgb: 248, 249, 250;
  --bs-heading-color: inherit;
  --bs-link-color: #0f6cbf;
  --bs-link-color-rgb: 15, 108, 191;
  --bs-link-decoration: none;
  --bs-link-hover-color: #0c5699;
  --bs-link-hover-color-rgb: 12, 86, 153;
  --bs-link-hover-decoration: underline;
  --bs-code-color: #db1a74;
  --bs-highlight-color: #1d2125;
  --bs-highlight-bg: #fcefdc;
  --bs-border-width: 1px;
  --bs-border-style: solid;
  --bs-border-color: #dee2e6;
  --bs-border-color-translucent: rgba(0, 0, 0, 0.175);
  --bs-border-radius: 0.5rem;
  --bs-border-radius-sm: 0.25rem;
  --bs-border-radius-lg: 0.6rem;
  --bs-border-radius-xl: 1rem;
  --bs-border-radius-xxl: 2rem;
  --bs-border-radius-2xl: var(--bs-border-radius-xxl);
  --bs-border-radius-pill: 50rem;
  --bs-box-shadow: 0 0.5rem 1rem rgba(0, 0, 0, 0.15);
  --bs-box-shadow-sm: 0 0.125rem 0.25rem rgba(0, 0, 0, 0.075);
  --bs-box-shadow-lg: 0 1rem 3rem rgba(0, 0, 0, 0.175);
  --bs-box-shadow-inset: inset 0 1px 2px rgba(0, 0, 0, 0.075);
  --bs-focus-ring-width: 0.25rem;
  --bs-focus-ring-opacity: 0.75;
  --bs-focus-ring-color: rgba(15, 108, 191, 0.75);
  --bs-form-valid-color: #357a32;
  --bs-form-valid-border-color: #357a32;
  --bs-form-invalid-color: #ca3120;
  --bs-form-invalid-border-color: #ca3120;
}

[data-bs-theme=dark] {
  color-scheme: dark;
  --bs-body-color: #dee2e6;
  --bs-body-color-rgb: 222, 226, 230;
  --bs-body-bg: #1d2125;
  --bs-body-bg-rgb: 29, 33, 37;
  --bs-emphasis-color: #fff;
  --bs-emphasis-color-rgb: 255, 255, 255;
  --bs-secondary-color: rgba(222, 226, 230, 0.75);
  --bs-secondary-color-rgb: 222, 226, 230;
  --bs-secondary-bg: #343a40;
  --bs-secondary-bg-rgb: 52, 58, 64;
  --bs-tertiary-color: rgba(222, 226, 230, 0.5);
  --bs-tertiary-color-rgb: 222, 226, 230;
  --bs-tertiary-bg: #292e33;
  --bs-tertiary-bg-rgb: 41, 46, 51;
  --bs-primary-text-emphasis: #6fa7d9;
  --bs-secondary-text-emphasis: #e2e5e9;
  --bs-success-text-emphasis: #86af84;
  --bs-info-text-emphasis: #66b3c0;
  --bs-warning-text-emphasis: #f6ce95;
  --bs-danger-text-emphasis: #df8379;
  --bs-light-text-emphasis: #f8f9fa;
  --bs-dark-text-emphasis: #dee2e6;
  --bs-primary-bg-subtle: #031626;
  --bs-secondary-bg-subtle: #292a2c;
  --bs-success-bg-subtle: #0b180a;
  --bs-info-bg-subtle: #001a1e;
  --bs-warning-bg-subtle: #302310;
  --bs-danger-bg-subtle: #280a06;
  --bs-light-bg-subtle: #343a40;
  --bs-dark-bg-subtle: #1a1d20;
  --bs-primary-border-subtle: #094173;
  --bs-secondary-border-subtle: #7c7f83;
  --bs-success-border-subtle: #20491e;
  --bs-info-border-subtle: #004d5a;
  --bs-warning-border-subtle: #90682f;
  --bs-danger-border-subtle: #791d13;
  --bs-light-border-subtle: #495057;
  --bs-dark-border-subtle: #343a40;
  --bs-heading-color: inherit;
  --bs-link-color: #6fa7d9;
  --bs-link-hover-color: #8cb9e1;
  --bs-link-color-rgb: 111, 167, 217;
  --bs-link-hover-color-rgb: 140, 185, 225;
  --bs-code-color: #e976ac;
  --bs-highlight-color: #dee2e6;
  --bs-highlight-bg: #60451f;
  --bs-border-color: #495057;
  --bs-border-color-translucent: rgba(255, 255, 255, 0.15);
  --bs-form-valid-color: #86af84;
  --bs-form-valid-border-color: #86af84;
  --bs-form-invalid-color: #df8379;
  --bs-form-invalid-border-color: #df8379;
}

*,
*::before,
*::after {
  box-sizing: border-box;
}

@media (prefers-reduced-motion: no-preference) {
  :root {
    scroll-behavior: smooth;
  }
}

body {
  margin: 0;
  font-family: var(--bs-body-font-family);
  font-size: var(--bs-body-font-size);
  font-weight: var(--bs-body-font-weight);
  line-height: var(--bs-body-line-height);
  color: var(--bs-body-color);
  text-align: var(--bs-body-text-align);
  background-color: var(--bs-body-bg);
  -webkit-text-size-adjust: 100%;
  -webkit-tap-highlight-color: rgba(0, 0, 0, 0);
}

hr {
  margin: 1rem 0;
  color: inherit;
  border: 0;
  border-top: var(--bs-border-width) solid;
  opacity: 0.15;
}

h6, .h6, h5, .h5, h4, .h4, h3, .h3, h2, .h2, h1, .h1 {
  margin-top: 0;
  margin-bottom: 0.5rem;
  font-weight: 500;
  line-height: 1.2;
  color: var(--bs-heading-color);
}

h1, .h1 {
  font-size: calc(1.359375rem + 1.3125vw);
}
@media (min-width: 1200px) {
  h1, .h1 {
    font-size: 2.34375rem;
  }
}

h2, .h2 {
  font-size: calc(1.3125rem + 0.75vw);
}
@media (min-width: 1200px) {
  h2, .h2 {
    font-size: 1.875rem;
  }
}

h3, .h3 {
  font-size: calc(1.2890625rem + 0.46875vw);
}
@media (min-width: 1200px) {
  h3, .h3 {
    font-size: 1.640625rem;
  }
}

h4, .h4 {
  font-size: calc(1.265625rem + 0.1875vw);
}
@media (min-width: 1200px) {
  h4, .h4 {
    font-size: 1.40625rem;
  }
}

h5, .h5 {
  font-size: 1.171875rem;
}

h6, .h6 {
  font-size: 0.9375rem;
}

p {
  margin-top: 0;
  margin-bottom: 1rem;
}

abbr[title] {
  text-decoration: underline dotted;
  cursor: help;
  text-decoration-skip-ink: none;
}

address {
  margin-bottom: 1rem;
  font-style: normal;
  line-height: inherit;
}

ol,
ul {
  padding-left: 2rem;
}

ol,
ul,
dl {
  margin-top: 0;
  margin-bottom: 1rem;
}

ol ol,
ul ul,
ol ul,
ul ol {
  margin-bottom: 0;
}

dt {
  font-weight: 700;
}

dd {
  margin-bottom: 0.5rem;
  margin-left: 0;
}

blockquote {
  margin: 0 0 1rem;
}

b,
strong {
  font-weight: bolder;
}

small, .small {
  font-size: 0.875em;
}

mark, .mark {
  padding: 0.1875em;
  color: var(--bs-highlight-color);
  background-color: var(--bs-highlight-bg);
}

sub,
sup {
  position: relative;
  font-size: 0.75em;
  line-height: 0;
  vertical-align: baseline;
}

sub {
  bottom: -0.25em;
}

sup {
  top: -0.5em;
}

a {
  color: rgba(var(--bs-link-color-rgb), var(--bs-link-opacity, 1));
  text-decoration: none;
}
a:hover {
  --bs-link-color-rgb: var(--bs-link-hover-color-rgb);
  text-decoration: underline;
}

a:not([href]):not([class]), a:not([href]):not([class]):hover {
  color: inherit;
  text-decoration: none;
}

pre,
code,
kbd,
samp {
  font-family: var(--bs-font-monospace);
  font-size: 1em;
}

pre {
  display: block;
  margin-top: 0;
  margin-bottom: 1rem;
  overflow: auto;
  font-size: 0.875em;
}
pre code {
  font-size: inherit;
  color: inherit;
  word-break: normal;
}

code {
  font-size: 0.875em;
  color: var(--bs-code-color);
  word-wrap: break-word;
}
a > code {
  color: inherit;
}

kbd {
  padding: 0.1875rem 0.375rem;
  font-size: 0.875em;
  color: var(--bs-body-bg);
  background-color: var(--bs-body-color);
  border-radius: 0.25rem;
}
kbd kbd {
  padding: 0;
  font-size: 1em;
}

figure {
  margin: 0 0 1rem;
}

img,
svg {
  vertical-align: middle;
}

table {
  caption-side: bottom;
  border-collapse: collapse;
}

caption {
  padding-top: 0.5rem;
  padding-bottom: 0.5rem;
  color: var(--bs-secondary-color);
  text-align: left;
}

th {
  text-align: inherit;
  text-align: -webkit-match-parent;
}

thead,
tbody,
tfoot,
tr,
td,
th {
  border-color: inherit;
  border-style: solid;
  border-width: 0;
}

label {
  display: inline-block;
}

button {
  border-radius: 0;
}

button:focus:not(:focus-visible) {
  outline: 0;
}

input,
button,
select,
optgroup,
textarea {
  margin: 0;
  font-family: inherit;
  font-size: inherit;
  line-height: inherit;
}

button,
select {
  text-transform: none;
}

[role=button] {
  cursor: pointer;
}

select {
  word-wrap: normal;
}
select:disabled {
  opacity: 1;
}

[list]:not([type=date]):not([type=datetime-local]):not([type=month]):not([type=week]):not([type=time])::-webkit-calendar-picker-indicator {
  display: none !important;
}

button,
[type=button],
[type=reset],
[type=submit] {
  -webkit-appearance: button;
}
button:not(:disabled),
[type=button]:not(:disabled),
[type=reset]:not(:disabled),
[type=submit]:not(:disabled) {
  cursor: pointer;
}

::-moz-focus-inner {
  padding: 0;
  border-style: none;
}

textarea {
  resize: vertical;
}

fieldset {
  min-width: 0;
  padding: 0;
  margin: 0;
  border: 0;
}

legend {
  float: left;
  width: 100%;
  padding: 0;
  margin-bottom: 0.5rem;
  font-size: calc(1.275rem + 0.3vw);
  line-height: inherit;
}
@media (min-width: 1200px) {
  legend {
    font-size: 1.5rem;
  }
}
legend + * {
  clear: left;
}

::-webkit-datetime-edit-fields-wrapper,
::-webkit-datetime-edit-text,
::-webkit-datetime-edit-minute,
::-webkit-datetime-edit-hour-field,
::-webkit-datetime-edit-day-field,
::-webkit-datetime-edit-month-field,
::-webkit-datetime-edit-year-field {
  padding: 0;
}

::-webkit-inner-spin-button {
  height: auto;
}

[type=search] {
  -webkit-appearance: textfield;
  outline-offset: -2px;
}

/* rtl:raw:
[type="tel"],
[type="url"],
[type="email"],
[type="number"] {
  direction: ltr;
}
*/
::-webkit-search-decoration {
  -webkit-appearance: none;
}

::-webkit-color-swatch-wrapper {
  padding: 0;
}

::file-selector-button {
  font: inherit;
  -webkit-appearance: button;
}

output {
  display: inline-block;
}

iframe {
  border: 0;
}

summary {
  display: list-item;
  cursor: pointer;
}

progress {
  vertical-align: baseline;
}

[hidden] {
  display: none !important;
}

.lead {
  font-size: 1.171875rem;
  font-weight: 300;
}

.display-1 {
  font-size: calc(1.625rem + 4.5vw);
  font-weight: 300;
  line-height: 1.2;
}
@media (min-width: 1200px) {
  .display-1 {
    font-size: 5rem;
  }
}

.display-2 {
  font-size: calc(1.575rem + 3.9vw);
  font-weight: 300;
  line-height: 1.2;
}
@media (min-width: 1200px) {
  .display-2 {
    font-size: 4.5rem;
  }
}

.display-3 {
  font-size: calc(1.525rem + 3.3vw);
  font-weight: 300;
  line-height: 1.2;
}
@media (min-width: 1200px) {
  .display-3 {
    font-size: 4rem;
  }
}

.display-4 {
  font-size: calc(1.475rem + 2.7vw);
  font-weight: 300;
  line-height: 1.2;
}
@media (min-width: 1200px) {
  .display-4 {
    font-size: 3.5rem;
  }
}

.display-5 {
  font-size: calc(1.425rem + 2.1vw);
  font-weight: 300;
  line-height: 1.2;
}
@media (min-width: 1200px) {
  .display-5 {
    font-size: 3rem;
  }
}

.display-6 {
  font-size: calc(1.375rem + 1.5vw);
  font-weight: 300;
  line-height: 1.2;
}
@media (min-width: 1200px) {
  .display-6 {
    font-size: 2.5rem;
  }
}

.list-unstyled {
  padding-left: 0;
  list-style: none;
}

.list-inline {
  padding-left: 0;
  list-style: none;
}

.list-inline-item {
  display: inline-block;
}
.list-inline-item:not(:last-child) {
  margin-right: 0.5rem;
}

.initialism {
  font-size: 0.875em;
  text-transform: uppercase;
}

.blockquote {
  margin-bottom: 1rem;
  font-size: 1.171875rem;
}
.blockquote > :last-child {
  margin-bottom: 0;
}

.blockquote-footer {
  margin-top: -1rem;
  margin-bottom: 1rem;
  font-size: 0.875em;
  color: #6a737b;
}
.blockquote-footer::before {
  content: "— ";
}

.img-fluid {
  max-width: 100%;
  height: auto;
}

.img-thumbnail {
  padding: 0.25rem;
  background-color: var(--bs-body-bg);
  border: var(--bs-border-width) solid var(--bs-border-color);
  border-radius: var(--bs-border-radius);
  max-width: 100%;
  height: auto;
}

.figure {
  display: inline-block;
}

.figure-img {
  margin-bottom: 0.5rem;
  line-height: 1;
}

.figure-caption {
  font-size: 0.875em;
  color: var(--bs-secondary-color);
}

.container,
.container-fluid,
.container-xxl,
.container-xl,
.container-lg,
.container-md,
.container-sm {
  --bs-gutter-x: 1.5rem;
  --bs-gutter-y: 0;
  width: 100%;
  padding-right: calc(var(--bs-gutter-x) * 0.5);
  padding-left: calc(var(--bs-gutter-x) * 0.5);
  margin-right: auto;
  margin-left: auto;
}

@media (min-width: 576px) {
  .container-sm, .container {
    max-width: 540px;
  }
}
@media (min-width: 768px) {
  .container-md, .container-sm, .container {
    max-width: 720px;
  }
}
@media (min-width: 992px) {
  .container-lg, .container-md, .container-sm, .container {
    max-width: 960px;
  }
}
@media (min-width: 1200px) {
  .container-xl, .container-lg, .container-md, .container-sm, .container {
    max-width: 1140px;
  }
}
@media (min-width: 1400px) {
  .container-xxl, .container-xl, .container-lg, .container-md, .container-sm, .container {
    max-width: 1320px;
  }
}
:root {
  --bs-breakpoint-xs: 0;
  --bs-breakpoint-sm: 576px;
  --bs-breakpoint-md: 768px;
  --bs-breakpoint-lg: 992px;
  --bs-breakpoint-xl: 1200px;
  --bs-breakpoint-xxl: 1400px;
}

.row {
  --bs-gutter-x: 1.5rem;
  --bs-gutter-y: 0;
  display: flex;
  flex-wrap: wrap;
  margin-top: calc(-1 * var(--bs-gutter-y));
  margin-right: calc(-0.5 * var(--bs-gutter-x));
  margin-left: calc(-0.5 * var(--bs-gutter-x));
}
.row > * {
  flex-shrink: 0;
  width: 100%;
  max-width: 100%;
  padding-right: calc(var(--bs-gutter-x) * 0.5);
  padding-left: calc(var(--bs-gutter-x) * 0.5);
  margin-top: var(--bs-gutter-y);
}

.col {
  flex: 1 0 0%;
}

.row-cols-auto > * {
  flex: 0 0 auto;
  width: auto;
}

.row-cols-1 > * {
  flex: 0 0 auto;
  width: 100%;
}

.row-cols-2 > * {
  flex: 0 0 auto;
  width: 50%;
}

.row-cols-3 > * {
  flex: 0 0 auto;
  width: 33.33333333%;
}

.row-cols-4 > * {
  flex: 0 0 auto;
  width: 25%;
}

.row-cols-5 > * {
  flex: 0 0 auto;
  width: 20%;
}

.row-cols-6 > * {
  flex: 0 0 auto;
  width: 16.66666667%;
}

.col-auto {
  flex: 0 0 auto;
  width: auto;
}

.col-1 {
  flex: 0 0 auto;
  width: 8.33333333%;
}

.col-2 {
  flex: 0 0 auto;
  width: 16.66666667%;
}

.col-3 {
  flex: 0 0 auto;
  width: 25%;
}

.col-4 {
  flex: 0 0 auto;
  width: 33.33333333%;
}

.col-5 {
  flex: 0 0 auto;
  width: 41.66666667%;
}

.col-6 {
  flex: 0 0 auto;
  width: 50%;
}

.col-7 {
  flex: 0 0 auto;
  width: 58.33333333%;
}

.col-8 {
  flex: 0 0 auto;
  width: 66.66666667%;
}

.col-9 {
  flex: 0 0 auto;
  width: 75%;
}

.col-10 {
  flex: 0 0 auto;
  width: 83.33333333%;
}

.col-11 {
  flex: 0 0 auto;
  width: 91.66666667%;
}

.col-12 {
  flex: 0 0 auto;
  width: 100%;
}

.offset-1 {
  margin-left: 8.33333333%;
}

.offset-2 {
  margin-left: 16.66666667%;
}

.offset-3 {
  margin-left: 25%;
}

.offset-4 {
  margin-left: 33.33333333%;
}

.offset-5 {
  margin-left: 41.66666667%;
}

.offset-6 {
  margin-left: 50%;
}

.offset-7 {
  margin-left: 58.33333333%;
}

.offset-8 {
  margin-left: 66.66666667%;
}

.offset-9 {
  margin-left: 75%;
}

.offset-10 {
  margin-left: 83.33333333%;
}

.offset-11 {
  margin-left: 91.66666667%;
}

.g-0,
.gx-0 {
  --bs-gutter-x: 0;
}

.g-0,
.gy-0 {
  --bs-gutter-y: 0;
}

.g-1,
.gx-1 {
  --bs-gutter-x: 0.25rem;
}

.g-1,
.gy-1 {
  --bs-gutter-y: 0.25rem;
}

.g-2,
.gx-2 {
  --bs-gutter-x: 0.5rem;
}

.g-2,
.gy-2 {
  --bs-gutter-y: 0.5rem;
}

.g-3,
.gx-3 {
  --bs-gutter-x: 1rem;
}

.g-3,
.gy-3 {
  --bs-gutter-y: 1rem;
}

.g-4,
.gx-4 {
  --bs-gutter-x: 1.5rem;
}

.g-4,
.gy-4 {
  --bs-gutter-y: 1.5rem;
}

.g-5,
.gx-5 {
  --bs-gutter-x: 2rem;
}

.g-5,
.gy-5 {
  --bs-gutter-y: 2rem;
}

.g-6,
.gx-6 {
  --bs-gutter-x: 3rem;
}

.g-6,
.gy-6 {
  --bs-gutter-y: 3rem;
}

@media (min-width: 576px) {
  .col-sm {
    flex: 1 0 0%;
  }
  .row-cols-sm-auto > * {
    flex: 0 0 auto;
    width: auto;
  }
  .row-cols-sm-1 > * {
    flex: 0 0 auto;
    width: 100%;
  }
  .row-cols-sm-2 > * {
    flex: 0 0 auto;
    width: 50%;
  }
  .row-cols-sm-3 > * {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .row-cols-sm-4 > * {
    flex: 0 0 auto;
    width: 25%;
  }
  .row-cols-sm-5 > * {
    flex: 0 0 auto;
    width: 20%;
  }
  .row-cols-sm-6 > * {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-sm-auto {
    flex: 0 0 auto;
    width: auto;
  }
  .col-sm-1 {
    flex: 0 0 auto;
    width: 8.33333333%;
  }
  .col-sm-2 {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-sm-3 {
    flex: 0 0 auto;
    width: 25%;
  }
  .col-sm-4 {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .col-sm-5 {
    flex: 0 0 auto;
    width: 41.66666667%;
  }
  .col-sm-6 {
    flex: 0 0 auto;
    width: 50%;
  }
  .col-sm-7 {
    flex: 0 0 auto;
    width: 58.33333333%;
  }
  .col-sm-8 {
    flex: 0 0 auto;
    width: 66.66666667%;
  }
  .col-sm-9 {
    flex: 0 0 auto;
    width: 75%;
  }
  .col-sm-10 {
    flex: 0 0 auto;
    width: 83.33333333%;
  }
  .col-sm-11 {
    flex: 0 0 auto;
    width: 91.66666667%;
  }
  .col-sm-12 {
    flex: 0 0 auto;
    width: 100%;
  }
  .offset-sm-0 {
    margin-left: 0;
  }
  .offset-sm-1 {
    margin-left: 8.33333333%;
  }
  .offset-sm-2 {
    margin-left: 16.66666667%;
  }
  .offset-sm-3 {
    margin-left: 25%;
  }
  .offset-sm-4 {
    margin-left: 33.33333333%;
  }
  .offset-sm-5 {
    margin-left: 41.66666667%;
  }
  .offset-sm-6 {
    margin-left: 50%;
  }
  .offset-sm-7 {
    margin-left: 58.33333333%;
  }
  .offset-sm-8 {
    margin-left: 66.66666667%;
  }
  .offset-sm-9 {
    margin-left: 75%;
  }
  .offset-sm-10 {
    margin-left: 83.33333333%;
  }
  .offset-sm-11 {
    margin-left: 91.66666667%;
  }
  .g-sm-0,
  .gx-sm-0 {
    --bs-gutter-x: 0;
  }
  .g-sm-0,
  .gy-sm-0 {
    --bs-gutter-y: 0;
  }
  .g-sm-1,
  .gx-sm-1 {
    --bs-gutter-x: 0.25rem;
  }
  .g-sm-1,
  .gy-sm-1 {
    --bs-gutter-y: 0.25rem;
  }
  .g-sm-2,
  .gx-sm-2 {
    --bs-gutter-x: 0.5rem;
  }
  .g-sm-2,
  .gy-sm-2 {
    --bs-gutter-y: 0.5rem;
  }
  .g-sm-3,
  .gx-sm-3 {
    --bs-gutter-x: 1rem;
  }
  .g-sm-3,
  .gy-sm-3 {
    --bs-gutter-y: 1rem;
  }
  .g-sm-4,
  .gx-sm-4 {
    --bs-gutter-x: 1.5rem;
  }
  .g-sm-4,
  .gy-sm-4 {
    --bs-gutter-y: 1.5rem;
  }
  .g-sm-5,
  .gx-sm-5 {
    --bs-gutter-x: 2rem;
  }
  .g-sm-5,
  .gy-sm-5 {
    --bs-gutter-y: 2rem;
  }
  .g-sm-6,
  .gx-sm-6 {
    --bs-gutter-x: 3rem;
  }
  .g-sm-6,
  .gy-sm-6 {
    --bs-gutter-y: 3rem;
  }
}
@media (min-width: 768px) {
  .col-md {
    flex: 1 0 0%;
  }
  .row-cols-md-auto > * {
    flex: 0 0 auto;
    width: auto;
  }
  .row-cols-md-1 > * {
    flex: 0 0 auto;
    width: 100%;
  }
  .row-cols-md-2 > * {
    flex: 0 0 auto;
    width: 50%;
  }
  .row-cols-md-3 > * {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .row-cols-md-4 > * {
    flex: 0 0 auto;
    width: 25%;
  }
  .row-cols-md-5 > * {
    flex: 0 0 auto;
    width: 20%;
  }
  .row-cols-md-6 > * {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-md-auto {
    flex: 0 0 auto;
    width: auto;
  }
  .col-md-1 {
    flex: 0 0 auto;
    width: 8.33333333%;
  }
  .col-md-2 {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-md-3 {
    flex: 0 0 auto;
    width: 25%;
  }
  .col-md-4 {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .col-md-5 {
    flex: 0 0 auto;
    width: 41.66666667%;
  }
  .col-md-6 {
    flex: 0 0 auto;
    width: 50%;
  }
  .col-md-7 {
    flex: 0 0 auto;
    width: 58.33333333%;
  }
  .col-md-8 {
    flex: 0 0 auto;
    width: 66.66666667%;
  }
  .col-md-9 {
    flex: 0 0 auto;
    width: 75%;
  }
  .col-md-10 {
    flex: 0 0 auto;
    width: 83.33333333%;
  }
  .col-md-11 {
    flex: 0 0 auto;
    width: 91.66666667%;
  }
  .col-md-12 {
    flex: 0 0 auto;
    width: 100%;
  }
  .offset-md-0 {
    margin-left: 0;
  }
  .offset-md-1 {
    margin-left: 8.33333333%;
  }
  .offset-md-2 {
    margin-left: 16.66666667%;
  }
  .offset-md-3 {
    margin-left: 25%;
  }
  .offset-md-4 {
    margin-left: 33.33333333%;
  }
  .offset-md-5 {
    margin-left: 41.66666667%;
  }
  .offset-md-6 {
    margin-left: 50%;
  }
  .offset-md-7 {
    margin-left: 58.33333333%;
  }
  .offset-md-8 {
    margin-left: 66.66666667%;
  }
  .offset-md-9 {
    margin-left: 75%;
  }
  .offset-md-10 {
    margin-left: 83.33333333%;
  }
  .offset-md-11 {
    margin-left: 91.66666667%;
  }
  .g-md-0,
  .gx-md-0 {
    --bs-gutter-x: 0;
  }
  .g-md-0,
  .gy-md-0 {
    --bs-gutter-y: 0;
  }
  .g-md-1,
  .gx-md-1 {
    --bs-gutter-x: 0.25rem;
  }
  .g-md-1,
  .gy-md-1 {
    --bs-gutter-y: 0.25rem;
  }
  .g-md-2,
  .gx-md-2 {
    --bs-gutter-x: 0.5rem;
  }
  .g-md-2,
  .gy-md-2 {
    --bs-gutter-y: 0.5rem;
  }
  .g-md-3,
  .gx-md-3 {
    --bs-gutter-x: 1rem;
  }
  .g-md-3,
  .gy-md-3 {
    --bs-gutter-y: 1rem;
  }
  .g-md-4,
  .gx-md-4 {
    --bs-gutter-x: 1.5rem;
  }
  .g-md-4,
  .gy-md-4 {
    --bs-gutter-y: 1.5rem;
  }
  .g-md-5,
  .gx-md-5 {
    --bs-gutter-x: 2rem;
  }
  .g-md-5,
  .gy-md-5 {
    --bs-gutter-y: 2rem;
  }
  .g-md-6,
  .gx-md-6 {
    --bs-gutter-x: 3rem;
  }
  .g-md-6,
  .gy-md-6 {
    --bs-gutter-y: 3rem;
  }
}
@media (min-width: 992px) {
  .col-lg {
    flex: 1 0 0%;
  }
  .row-cols-lg-auto > * {
    flex: 0 0 auto;
    width: auto;
  }
  .row-cols-lg-1 > * {
    flex: 0 0 auto;
    width: 100%;
  }
  .row-cols-lg-2 > * {
    flex: 0 0 auto;
    width: 50%;
  }
  .row-cols-lg-3 > * {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .row-cols-lg-4 > * {
    flex: 0 0 auto;
    width: 25%;
  }
  .row-cols-lg-5 > * {
    flex: 0 0 auto;
    width: 20%;
  }
  .row-cols-lg-6 > * {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-lg-auto {
    flex: 0 0 auto;
    width: auto;
  }
  .col-lg-1 {
    flex: 0 0 auto;
    width: 8.33333333%;
  }
  .col-lg-2 {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-lg-3 {
    flex: 0 0 auto;
    width: 25%;
  }
  .col-lg-4 {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .col-lg-5 {
    flex: 0 0 auto;
    width: 41.66666667%;
  }
  .col-lg-6 {
    flex: 0 0 auto;
    width: 50%;
  }
  .col-lg-7 {
    flex: 0 0 auto;
    width: 58.33333333%;
  }
  .col-lg-8 {
    flex: 0 0 auto;
    width: 66.66666667%;
  }
  .col-lg-9 {
    flex: 0 0 auto;
    width: 75%;
  }
  .col-lg-10 {
    flex: 0 0 auto;
    width: 83.33333333%;
  }
  .col-lg-11 {
    flex: 0 0 auto;
    width: 91.66666667%;
  }
  .col-lg-12 {
    flex: 0 0 auto;
    width: 100%;
  }
  .offset-lg-0 {
    margin-left: 0;
  }
  .offset-lg-1 {
    margin-left: 8.33333333%;
  }
  .offset-lg-2 {
    margin-left: 16.66666667%;
  }
  .offset-lg-3 {
    margin-left: 25%;
  }
  .offset-lg-4 {
    margin-left: 33.33333333%;
  }
  .offset-lg-5 {
    margin-left: 41.66666667%;
  }
  .offset-lg-6 {
    margin-left: 50%;
  }
  .offset-lg-7 {
    margin-left: 58.33333333%;
  }
  .offset-lg-8 {
    margin-left: 66.66666667%;
  }
  .offset-lg-9 {
    margin-left: 75%;
  }
  .offset-lg-10 {
    margin-left: 83.33333333%;
  }
  .offset-lg-11 {
    margin-left: 91.66666667%;
  }
  .g-lg-0,
  .gx-lg-0 {
    --bs-gutter-x: 0;
  }
  .g-lg-0,
  .gy-lg-0 {
    --bs-gutter-y: 0;
  }
  .g-lg-1,
  .gx-lg-1 {
    --bs-gutter-x: 0.25rem;
  }
  .g-lg-1,
  .gy-lg-1 {
    --bs-gutter-y: 0.25rem;
  }
  .g-lg-2,
  .gx-lg-2 {
    --bs-gutter-x: 0.5rem;
  }
  .g-lg-2,
  .gy-lg-2 {
    --bs-gutter-y: 0.5rem;
  }
  .g-lg-3,
  .gx-lg-3 {
    --bs-gutter-x: 1rem;
  }
  .g-lg-3,
  .gy-lg-3 {
    --bs-gutter-y: 1rem;
  }
  .g-lg-4,
  .gx-lg-4 {
    --bs-gutter-x: 1.5rem;
  }
  .g-lg-4,
  .gy-lg-4 {
    --bs-gutter-y: 1.5rem;
  }
  .g-lg-5,
  .gx-lg-5 {
    --bs-gutter-x: 2rem;
  }
  .g-lg-5,
  .gy-lg-5 {
    --bs-gutter-y: 2rem;
  }
  .g-lg-6,
  .gx-lg-6 {
    --bs-gutter-x: 3rem;
  }
  .g-lg-6,
  .gy-lg-6 {
    --bs-gutter-y: 3rem;
  }
}
@media (min-width: 1200px) {
  .col-xl {
    flex: 1 0 0%;
  }
  .row-cols-xl-auto > * {
    flex: 0 0 auto;
    width: auto;
  }
  .row-cols-xl-1 > * {
    flex: 0 0 auto;
    width: 100%;
  }
  .row-cols-xl-2 > * {
    flex: 0 0 auto;
    width: 50%;
  }
  .row-cols-xl-3 > * {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .row-cols-xl-4 > * {
    flex: 0 0 auto;
    width: 25%;
  }
  .row-cols-xl-5 > * {
    flex: 0 0 auto;
    width: 20%;
  }
  .row-cols-xl-6 > * {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-xl-auto {
    flex: 0 0 auto;
    width: auto;
  }
  .col-xl-1 {
    flex: 0 0 auto;
    width: 8.33333333%;
  }
  .col-xl-2 {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-xl-3 {
    flex: 0 0 auto;
    width: 25%;
  }
  .col-xl-4 {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .col-xl-5 {
    flex: 0 0 auto;
    width: 41.66666667%;
  }
  .col-xl-6 {
    flex: 0 0 auto;
    width: 50%;
  }
  .col-xl-7 {
    flex: 0 0 auto;
    width: 58.33333333%;
  }
  .col-xl-8 {
    flex: 0 0 auto;
    width: 66.66666667%;
  }
  .col-xl-9 {
    flex: 0 0 auto;
    width: 75%;
  }
  .col-xl-10 {
    flex: 0 0 auto;
    width: 83.33333333%;
  }
  .col-xl-11 {
    flex: 0 0 auto;
    width: 91.66666667%;
  }
  .col-xl-12 {
    flex: 0 0 auto;
    width: 100%;
  }
  .offset-xl-0 {
    margin-left: 0;
  }
  .offset-xl-1 {
    margin-left: 8.33333333%;
  }
  .offset-xl-2 {
    margin-left: 16.66666667%;
  }
  .offset-xl-3 {
    margin-left: 25%;
  }
  .offset-xl-4 {
    margin-left: 33.33333333%;
  }
  .offset-xl-5 {
    margin-left: 41.66666667%;
  }
  .offset-xl-6 {
    margin-left: 50%;
  }
  .offset-xl-7 {
    margin-left: 58.33333333%;
  }
  .offset-xl-8 {
    margin-left: 66.66666667%;
  }
  .offset-xl-9 {
    margin-left: 75%;
  }
  .offset-xl-10 {
    margin-left: 83.33333333%;
  }
  .offset-xl-11 {
    margin-left: 91.66666667%;
  }
  .g-xl-0,
  .gx-xl-0 {
    --bs-gutter-x: 0;
  }
  .g-xl-0,
  .gy-xl-0 {
    --bs-gutter-y: 0;
  }
  .g-xl-1,
  .gx-xl-1 {
    --bs-gutter-x: 0.25rem;
  }
  .g-xl-1,
  .gy-xl-1 {
    --bs-gutter-y: 0.25rem;
  }
  .g-xl-2,
  .gx-xl-2 {
    --bs-gutter-x: 0.5rem;
  }
  .g-xl-2,
  .gy-xl-2 {
    --bs-gutter-y: 0.5rem;
  }
  .g-xl-3,
  .gx-xl-3 {
    --bs-gutter-x: 1rem;
  }
  .g-xl-3,
  .gy-xl-3 {
    --bs-gutter-y: 1rem;
  }
  .g-xl-4,
  .gx-xl-4 {
    --bs-gutter-x: 1.5rem;
  }
  .g-xl-4,
  .gy-xl-4 {
    --bs-gutter-y: 1.5rem;
  }
  .g-xl-5,
  .gx-xl-5 {
    --bs-gutter-x: 2rem;
  }
  .g-xl-5,
  .gy-xl-5 {
    --bs-gutter-y: 2rem;
  }
  .g-xl-6,
  .gx-xl-6 {
    --bs-gutter-x: 3rem;
  }
  .g-xl-6,
  .gy-xl-6 {
    --bs-gutter-y: 3rem;
  }
}
@media (min-width: 1400px) {
  .col-xxl {
    flex: 1 0 0%;
  }
  .row-cols-xxl-auto > * {
    flex: 0 0 auto;
    width: auto;
  }
  .row-cols-xxl-1 > * {
    flex: 0 0 auto;
    width: 100%;
  }
  .row-cols-xxl-2 > * {
    flex: 0 0 auto;
    width: 50%;
  }
  .row-cols-xxl-3 > * {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .row-cols-xxl-4 > * {
    flex: 0 0 auto;
    width: 25%;
  }
  .row-cols-xxl-5 > * {
    flex: 0 0 auto;
    width: 20%;
  }
  .row-cols-xxl-6 > * {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-xxl-auto {
    flex: 0 0 auto;
    width: auto;
  }
  .col-xxl-1 {
    flex: 0 0 auto;
    width: 8.33333333%;
  }
  .col-xxl-2 {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-xxl-3 {
    flex: 0 0 auto;
    width: 25%;
  }
  .col-xxl-4 {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .col-xxl-5 {
    flex: 0 0 auto;
    width: 41.66666667%;
  }
  .col-xxl-6 {
    flex: 0 0 auto;
    width: 50%;
  }
  .col-xxl-7 {
    flex: 0 0 auto;
    width: 58.33333333%;
  }
  .col-xxl-8 {
    flex: 0 0 auto;
    width: 66.66666667%;
  }
  .col-xxl-9 {
    flex: 0 0 auto;
    width: 75%;
  }
  .col-xxl-10 {
    flex: 0 0 auto;
    width: 83.33333333%;
  }
  .col-xxl-11 {
    flex: 0 0 auto;
    width: 91.66666667%;
  }
  .col-xxl-12 {
    flex: 0 0 auto;
    width: 100%;
  }
  .offset-xxl-0 {
    margin-left: 0;
  }
  .offset-xxl-1 {
    margin-left: 8.33333333%;
  }
  .offset-xxl-2 {
    margin-left: 16.66666667%;
  }
  .offset-xxl-3 {
    margin-left: 25%;
  }
  .offset-xxl-4 {
    margin-left: 33.33333333%;
  }
  .offset-xxl-5 {
    margin-left: 41.66666667%;
  }
  .offset-xxl-6 {
    margin-left: 50%;
  }
  .offset-xxl-7 {
    margin-left: 58.33333333%;
  }
  .offset-xxl-8 {
    margin-left: 66.66666667%;
  }
  .offset-xxl-9 {
    margin-left: 75%;
  }
  .offset-xxl-10 {
    margin-left: 83.33333333%;
  }
  .offset-xxl-11 {
    margin-left: 91.66666667%;
  }
  .g-xxl-0,
  .gx-xxl-0 {
    --bs-gutter-x: 0;
  }
  .g-xxl-0,
  .gy-xxl-0 {
    --bs-gutter-y: 0;
  }
  .g-xxl-1,
  .gx-xxl-1 {
    --bs-gutter-x: 0.25rem;
  }
  .g-xxl-1,
  .gy-xxl-1 {
    --bs-gutter-y: 0.25rem;
  }
  .g-xxl-2,
  .gx-xxl-2 {
    --bs-gutter-x: 0.5rem;
  }
  .g-xxl-2,
  .gy-xxl-2 {
    --bs-gutter-y: 0.5rem;
  }
  .g-xxl-3,
  .gx-xxl-3 {
    --bs-gutter-x: 1rem;
  }
  .g-xxl-3,
  .gy-xxl-3 {
    --bs-gutter-y: 1rem;
  }
  .g-xxl-4,
  .gx-xxl-4 {
    --bs-gutter-x: 1.5rem;
  }
  .g-xxl-4,
  .gy-xxl-4 {
    --bs-gutter-y: 1.5rem;
  }
  .g-xxl-5,
  .gx-xxl-5 {
    --bs-gutter-x: 2rem;
  }
  .g-xxl-5,
  .gy-xxl-5 {
    --bs-gutter-y: 2rem;
  }
  .g-xxl-6,
  .gx-xxl-6 {
    --bs-gutter-x: 3rem;
  }
  .g-xxl-6,
  .gy-xxl-6 {
    --bs-gutter-y: 3rem;
  }
}
.table {
  --bs-table-color-type: initial;
  --bs-table-bg-type: initial;
  --bs-table-color-state: initial;
  --bs-table-bg-state: initial;
  --bs-table-color: var(--bs-emphasis-color);
  --bs-table-bg: var(--bs-body-bg);
  --bs-table-border-color: var(--bs-border-color);
  --bs-table-accent-bg: transparent;
  --bs-table-striped-color: var(--bs-emphasis-color);
  --bs-table-striped-bg: rgba(var(--bs-emphasis-color-rgb), 0.03);
  --bs-table-active-color: var(--bs-emphasis-color);
  --bs-table-active-bg: rgba(var(--bs-emphasis-color-rgb), 0.1);
  --bs-table-hover-color: var(--bs-emphasis-color);
  --bs-table-hover-bg: rgba(var(--bs-emphasis-color-rgb), 0.075);
  width: 100%;
  margin-bottom: 1rem;
  vertical-align: top;
  border-color: var(--bs-table-border-color);
}
.table > :not(caption) > * > * {
  padding: 0.5rem 0.5rem;
  color: var(--bs-table-color-state, var(--bs-table-color-type, var(--bs-table-color)));
  background-color: var(--bs-table-bg);
  border-bottom-width: var(--bs-border-width);
  box-shadow: inset 0 0 0 9999px var(--bs-table-bg-state, var(--bs-table-bg-type, var(--bs-table-accent-bg)));
}
.table > tbody {
  vertical-align: inherit;
}
.table > thead {
  vertical-align: bottom;
}

.table-group-divider {
  border-top: calc(var(--bs-border-width) * 2) solid currentcolor;
}

.caption-top {
  caption-side: top;
}

.table-sm > :not(caption) > * > * {
  padding: 0.25rem 0.25rem;
}

.table-bordered > :not(caption) > * {
  border-width: var(--bs-border-width) 0;
}
.table-bordered > :not(caption) > * > * {
  border-width: 0 var(--bs-border-width);
}

.table-borderless > :not(caption) > * > * {
  border-bottom-width: 0;
}
.table-borderless > :not(:first-child) {
  border-top-width: 0;
}

.table-striped > tbody > tr:nth-of-type(odd) > * {
  --bs-table-color-type: var(--bs-table-striped-color);
  --bs-table-bg-type: var(--bs-table-striped-bg);
}

.table-striped-columns > :not(caption) > tr > :nth-child(even) {
  --bs-table-color-type: var(--bs-table-striped-color);
  --bs-table-bg-type: var(--bs-table-striped-bg);
}

.table-active {
  --bs-table-color-state: var(--bs-table-active-color);
  --bs-table-bg-state: var(--bs-table-active-bg);
}

.table-hover > tbody > tr:hover > * {
  --bs-table-color-state: var(--bs-table-hover-color);
  --bs-table-bg-state: var(--bs-table-hover-bg);
}

.table-primary {
  --bs-table-color: #000;
  --bs-table-bg: #cfe2f2;
  --bs-table-border-color: #a6b5c2;
  --bs-table-striped-bg: #c9dbeb;
  --bs-table-striped-color: #000;
  --bs-table-active-bg: #bacbda;
  --bs-table-active-color: #000;
  --bs-table-hover-bg: #bfd1e0;
  --bs-table-hover-color: #000;
  color: var(--bs-table-color);
  border-color: var(--bs-table-border-color);
}

.table-secondary {
  --bs-table-color: #000;
  --bs-table-bg: #f5f6f8;
  --bs-table-border-color: #c4c5c6;
  --bs-table-striped-bg: #eeeff1;
  --bs-table-striped-color: #000;
  --bs-table-active-bg: #dddddf;
  --bs-table-active-color: #000;
  --bs-table-hover-bg: #e3e4e5;
  --bs-table-hover-color: #000;
  color: var(--bs-table-color);
  border-color: var(--bs-table-border-color);
}

.table-success {
  --bs-table-color: #000;
  --bs-table-bg: #d7e4d6;
  --bs-table-border-color: #acb6ab;
  --bs-table-striped-bg: #d1ddd0;
  --bs-table-striped-color: #000;
  --bs-table-active-bg: #c2cdc1;
  --bs-table-active-color: #000;
  --bs-table-hover-bg: #c7d3c6;
  --bs-table-hover-color: #000;
  color: var(--bs-table-color);
  border-color: var(--bs-table-border-color);
}

.table-info {
  --bs-table-color: #000;
  --bs-table-bg: #cce6ea;
  --bs-table-border-color: #a3b8bb;
  --bs-table-striped-bg: #c6dfe3;
  --bs-table-striped-color: #000;
  --bs-table-active-bg: #b8cfd3;
  --bs-table-active-color: #000;
  --bs-table-hover-bg: #bdd5d8;
  --bs-table-hover-color: #000;
  color: var(--bs-table-color);
  border-color: var(--bs-table-border-color);
}

.table-warning {
  --bs-table-color: #000;
  --bs-table-bg: #fcefdc;
  --bs-table-border-color: #cabfb0;
  --bs-table-striped-bg: #f4e8d5;
  --bs-table-striped-color: #000;
  --bs-table-active-bg: #e3d7c6;
  --bs-table-active-color: #000;
  --bs-table-hover-bg: #e9ddcc;
  --bs-table-hover-color: #000;
  color: var(--bs-table-color);
  border-color: var(--bs-table-border-color);
}

.table-danger {
  --bs-table-color: #000;
  --bs-table-bg: #f4d6d2;
  --bs-table-border-color: #c3aba8;
  --bs-table-striped-bg: #edd0cc;
  --bs-table-striped-color: #000;
  --bs-table-active-bg: #dcc1bd;
  --bs-table-active-color: #000;
  --bs-table-hover-bg: #e2c6c2;
  --bs-table-hover-color: #000;
  color: var(--bs-table-color);
  border-color: var(--bs-table-border-color);
}

.table-light {
  --bs-table-color: #000;
  --bs-table-bg: #f8f9fa;
  --bs-table-border-color: #c6c7c8;
  --bs-table-striped-bg: #f1f2f3;
  --bs-table-striped-color: #000;
  --bs-table-active-bg: #dfe0e1;
  --bs-table-active-color: #000;
  --bs-table-hover-bg: #e5e6e7;
  --bs-table-hover-color: #000;
  color: var(--bs-table-color);
  border-color: var(--bs-table-border-color);
}

.table-dark {
  --bs-table-color: #fff;
  --bs-table-bg: #1d2125;
  --bs-table-border-color: #4a4d51;
  --bs-table-striped-bg: #24282c;
  --bs-table-striped-color: #fff;
  --bs-table-active-bg: #34373b;
  --bs-table-active-color: #fff;
  --bs-table-hover-bg: #2e3235;
  --bs-table-hover-color: #fff;
  color: var(--bs-table-color);
  border-color: var(--bs-table-border-color);
}

.table-responsive {
  overflow-x: auto;
  -webkit-overflow-scrolling: touch;
}

@media (max-width: 575.98px) {
  .table-responsive-sm {
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
  }
}
@media (max-width: 767.98px) {
  .table-responsive-md {
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
  }
}
@media (max-width: 991.98px) {
  .table-responsive-lg {
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
  }
}
@media (max-width: 1199.98px) {
  .table-responsive-xl {
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
  }
}
@media (max-width: 1399.98px) {
  .table-responsive-xxl {
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
  }
}
.form-label {
  margin-bottom: 0.5rem;
}

.col-form-label {
  padding-top: calc(0.375rem + var(--bs-border-width));
  padding-bottom: calc(0.375rem + var(--bs-border-width));
  margin-bottom: 0;
  font-size: inherit;
  line-height: 1.5;
}

.col-form-label-lg {
  padding-top: calc(0.5rem + var(--bs-border-width));
  padding-bottom: calc(0.5rem + var(--bs-border-width));
  font-size: 1.171875rem;
}

.col-form-label-sm {
  padding-top: calc(0.25rem + var(--bs-border-width));
  padding-bottom: calc(0.25rem + var(--bs-border-width));
  font-size: 0.8203125rem;
}

.form-text {
  margin-top: 0.25rem;
  font-size: 0.875em;
  color: var(--bs-secondary-color);
}

.form-control {
  display: block;
  width: 100%;
  padding: 0.375rem 0.75rem;
  font-size: 0.9375rem;
  font-weight: 400;
  line-height: 1.5;
  color: var(--bs-body-color);
  appearance: none;
  background-color: var(--bs-body-bg);
  background-clip: padding-box;
  border: var(--bs-border-width) solid var(--bs-gray-500);
  border-radius: var(--bs-border-radius);
  transition: border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .form-control {
    transition: none;
  }
}
.form-control[type=file] {
  overflow: hidden;
}
.form-control[type=file]:not(:disabled):not([readonly]) {
  cursor: pointer;
}
.form-control:focus {
  color: var(--bs-body-color);
  background-color: var(--bs-body-bg);
  border-color: #87b6df;
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.form-control::-webkit-date-and-time-value {
  min-width: 85px;
  height: 1.5em;
  margin: 0;
}
.form-control::-webkit-datetime-edit {
  display: block;
  padding: 0;
}
.form-control::placeholder {
  color: var(--bs-secondary-color);
  opacity: 1;
}
.form-control:disabled {
  background-color: var(--bs-secondary-bg);
  opacity: 1;
}
.form-control::file-selector-button {
  padding: 0.375rem 0.75rem;
  margin: -0.375rem -0.75rem;
  margin-inline-end: 0.75rem;
  color: var(--bs-body-color);
  background-color: var(--bs-tertiary-bg);
  pointer-events: none;
  border-color: inherit;
  border-style: solid;
  border-width: 0;
  border-inline-end-width: var(--bs-border-width);
  border-radius: 0;
  transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .form-control::file-selector-button {
    transition: none;
  }
}
.form-control:hover:not(:disabled):not([readonly])::file-selector-button {
  background-color: var(--bs-secondary-bg);
}

.form-control-plaintext {
  display: block;
  width: 100%;
  padding: 0.375rem 0;
  margin-bottom: 0;
  line-height: 1.5;
  color: var(--bs-body-color);
  background-color: transparent;
  border: solid transparent;
  border-width: var(--bs-border-width) 0;
}
.form-control-plaintext:focus {
  outline: 0;
}
.form-control-plaintext.form-control-sm, .form-control-plaintext.form-control-lg {
  padding-right: 0;
  padding-left: 0;
}

.form-control-sm {
  min-height: calc(1.5em + 0.5rem + calc(var(--bs-border-width) * 2));
  padding: 0.25rem 0.5rem;
  font-size: 0.8203125rem;
  border-radius: var(--bs-border-radius-sm);
}
.form-control-sm::file-selector-button {
  padding: 0.25rem 0.5rem;
  margin: -0.25rem -0.5rem;
  margin-inline-end: 0.5rem;
}

.form-control-lg {
  min-height: calc(1.5em + 1rem + calc(var(--bs-border-width) * 2));
  padding: 0.5rem 1rem;
  font-size: 1.171875rem;
  border-radius: var(--bs-border-radius-lg);
}
.form-control-lg::file-selector-button {
  padding: 0.5rem 1rem;
  margin: -0.5rem -1rem;
  margin-inline-end: 1rem;
}

textarea.form-control {
  min-height: calc(1.5em + 0.75rem + calc(var(--bs-border-width) * 2));
}
textarea.form-control-sm {
  min-height: calc(1.5em + 0.5rem + calc(var(--bs-border-width) * 2));
}
textarea.form-control-lg {
  min-height: calc(1.5em + 1rem + calc(var(--bs-border-width) * 2));
}

.form-control-color {
  width: 3rem;
  height: calc(1.5em + 0.75rem + calc(var(--bs-border-width) * 2));
  padding: 0.375rem;
}
.form-control-color:not(:disabled):not([readonly]) {
  cursor: pointer;
}
.form-control-color::-moz-color-swatch {
  border: 0 !important;
  border-radius: var(--bs-border-radius);
}
.form-control-color::-webkit-color-swatch {
  border: 0 !important;
  border-radius: var(--bs-border-radius);
}
.form-control-color.form-control-sm {
  height: calc(1.5em + 0.5rem + calc(var(--bs-border-width) * 2));
}
.form-control-color.form-control-lg {
  height: calc(1.5em + 1rem + calc(var(--bs-border-width) * 2));
}

.form-select {
  --bs-form-select-bg-img: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16'%3e%3cpath fill='none' stroke='%23343a40' stroke-linecap='round' stroke-linejoin='round' stroke-width='2' d='m2 5 6 6 6-6'/%3e%3c/svg%3e");
  display: block;
  width: 100%;
  padding: 0.375rem 2.25rem 0.375rem 0.75rem;
  font-size: 0.9375rem;
  font-weight: 400;
  line-height: 1.5;
  color: var(--bs-body-color);
  appearance: none;
  background-color: var(--bs-body-bg);
  background-image: var(--bs-form-select-bg-img), var(--bs-form-select-bg-icon, none);
  background-repeat: no-repeat;
  background-position: right 0.75rem center;
  background-size: 16px 12px;
  border: var(--bs-border-width) solid var(--bs-gray-500);
  border-radius: var(--bs-border-radius);
  transition: border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .form-select {
    transition: none;
  }
}
.form-select:focus {
  border-color: #87b6df;
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.form-select[multiple], .form-select[size]:not([size="1"]) {
  padding-right: 0.75rem;
  background-image: none;
}
.form-select:disabled {
  background-color: var(--bs-secondary-bg);
}
.form-select:-moz-focusring {
  color: transparent;
  text-shadow: 0 0 0 var(--bs-body-color);
}

.form-select-sm {
  padding-top: 0.25rem;
  padding-bottom: 0.25rem;
  padding-left: 0.5rem;
  font-size: 0.8203125rem;
  border-radius: var(--bs-border-radius-sm);
}

.form-select-lg {
  padding-top: 0.5rem;
  padding-bottom: 0.5rem;
  padding-left: 1rem;
  font-size: 1.171875rem;
  border-radius: var(--bs-border-radius-lg);
}

[data-bs-theme=dark] .form-select {
  --bs-form-select-bg-img: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16'%3e%3cpath fill='none' stroke='%23dee2e6' stroke-linecap='round' stroke-linejoin='round' stroke-width='2' d='m2 5 6 6 6-6'/%3e%3c/svg%3e");
}

.form-check {
  display: block;
  min-height: 1.40625rem;
  padding-left: 1.5em;
  margin-bottom: 0.125rem;
}
.form-check .form-check-input {
  float: left;
  margin-left: -1.5em;
}

.form-check-reverse {
  padding-right: 1.5em;
  padding-left: 0;
  text-align: right;
}
.form-check-reverse .form-check-input {
  float: right;
  margin-right: -1.5em;
  margin-left: 0;
}

.form-check-input {
  --bs-form-check-bg: var(--bs-body-bg);
  flex-shrink: 0;
  width: 1em;
  height: 1em;
  margin-top: 0.25em;
  vertical-align: top;
  appearance: none;
  background-color: var(--bs-form-check-bg);
  background-image: var(--bs-form-check-bg-image);
  background-repeat: no-repeat;
  background-position: center;
  background-size: contain;
  border: var(--bs-border-width) solid var(--bs-gray-500);
  print-color-adjust: exact;
}
.form-check-input[type=checkbox] {
  border-radius: 0.25em;
}
.form-check-input[type=radio] {
  border-radius: 50%;
}
.form-check-input:active {
  filter: brightness(90%);
}
.form-check-input:focus {
  border-color: #87b6df;
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.form-check-input:checked {
  background-color: #0f6cbf;
  border-color: #0f6cbf;
}
.form-check-input:checked[type=checkbox] {
  --bs-form-check-bg-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 20 20'%3e%3cpath fill='none' stroke='%23fff' stroke-linecap='round' stroke-linejoin='round' stroke-width='3' d='m6 10 3 3 6-6'/%3e%3c/svg%3e");
}
.form-check-input:checked[type=radio] {
  --bs-form-check-bg-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='-4 -4 8 8'%3e%3ccircle r='2' fill='%23fff'/%3e%3c/svg%3e");
}
.form-check-input[type=checkbox]:indeterminate {
  background-color: #0f6cbf;
  border-color: #0f6cbf;
  --bs-form-check-bg-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 20 20'%3e%3cpath fill='none' stroke='%23fff' stroke-linecap='round' stroke-linejoin='round' stroke-width='3' d='M6 10h8'/%3e%3c/svg%3e");
}
.form-check-input:disabled {
  pointer-events: none;
  filter: none;
  opacity: 0.5;
}
.form-check-input[disabled] ~ .form-check-label, .form-check-input:disabled ~ .form-check-label {
  cursor: default;
  opacity: 0.5;
}

.form-switch {
  padding-left: 2.5em;
}
.form-switch .form-check-input {
  --bs-form-switch-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='-4 -4 8 8'%3e%3ccircle r='3' fill='rgba%280, 0, 0, 0.25%29'/%3e%3c/svg%3e");
  width: 2em;
  margin-left: -2.5em;
  background-image: var(--bs-form-switch-bg);
  background-position: left center;
  border-radius: 2em;
  transition: background-position 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .form-switch .form-check-input {
    transition: none;
  }
}
.form-switch .form-check-input:focus {
  --bs-form-switch-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='-4 -4 8 8'%3e%3ccircle r='3' fill='%2387b6df'/%3e%3c/svg%3e");
}
.form-switch .form-check-input:checked {
  background-position: right center;
  --bs-form-switch-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='-4 -4 8 8'%3e%3ccircle r='3' fill='%23fff'/%3e%3c/svg%3e");
}
.form-switch.form-check-reverse {
  padding-right: 2.5em;
  padding-left: 0;
}
.form-switch.form-check-reverse .form-check-input {
  margin-right: -2.5em;
  margin-left: 0;
}

.form-check-inline {
  display: inline-block;
  margin-right: 1rem;
}

.btn-check {
  position: absolute;
  clip: rect(0, 0, 0, 0);
  pointer-events: none;
}
.btn-check[disabled] + .btn, .btn-check:disabled + .btn {
  pointer-events: none;
  filter: none;
  opacity: 0.65;
}

[data-bs-theme=dark] .form-switch .form-check-input:not(:checked):not(:focus) {
  --bs-form-switch-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='-4 -4 8 8'%3e%3ccircle r='3' fill='rgba%28255, 255, 255, 0.25%29'/%3e%3c/svg%3e");
}

.form-range {
  width: 100%;
  height: 1.5rem;
  padding: 0;
  appearance: none;
  background-color: transparent;
}
.form-range:focus {
  outline: 0;
}
.form-range:focus::-webkit-slider-thumb {
  box-shadow: 0 0 0 1px #fff, 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.form-range:focus::-moz-range-thumb {
  box-shadow: 0 0 0 1px #fff, 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.form-range::-moz-focus-outer {
  border: 0;
}
.form-range::-webkit-slider-thumb {
  width: 1rem;
  height: 1rem;
  margin-top: -0.25rem;
  appearance: none;
  background-color: #0f6cbf;
  border: 0;
  border-radius: 1rem;
  transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .form-range::-webkit-slider-thumb {
    transition: none;
  }
}
.form-range::-webkit-slider-thumb:active {
  background-color: #b7d3ec;
}
.form-range::-webkit-slider-runnable-track {
  width: 100%;
  height: 0.5rem;
  color: transparent;
  cursor: pointer;
  background-color: var(--bs-secondary-bg);
  border-color: transparent;
  border-radius: 1rem;
}
.form-range::-moz-range-thumb {
  width: 1rem;
  height: 1rem;
  appearance: none;
  background-color: #0f6cbf;
  border: 0;
  border-radius: 1rem;
  transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .form-range::-moz-range-thumb {
    transition: none;
  }
}
.form-range::-moz-range-thumb:active {
  background-color: #b7d3ec;
}
.form-range::-moz-range-track {
  width: 100%;
  height: 0.5rem;
  color: transparent;
  cursor: pointer;
  background-color: var(--bs-secondary-bg);
  border-color: transparent;
  border-radius: 1rem;
}
.form-range:disabled {
  pointer-events: none;
}
.form-range:disabled::-webkit-slider-thumb {
  background-color: var(--bs-secondary-color);
}
.form-range:disabled::-moz-range-thumb {
  background-color: var(--bs-secondary-color);
}

.form-floating {
  position: relative;
}
.form-floating > .form-control,
.form-floating > .form-control-plaintext,
.form-floating > .form-select {
  height: calc(3.5rem + calc(var(--bs-border-width) * 2));
  min-height: calc(3.5rem + calc(var(--bs-border-width) * 2));
  line-height: 1.25;
}
.form-floating > label {
  position: absolute;
  top: 0;
  left: 0;
  z-index: 2;
  height: 100%;
  padding: 1rem 0.75rem;
  overflow: hidden;
  text-align: start;
  text-overflow: ellipsis;
  white-space: nowrap;
  pointer-events: none;
  border: var(--bs-border-width) solid transparent;
  transform-origin: 0 0;
  transition: opacity 0.1s ease-in-out, transform 0.1s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .form-floating > label {
    transition: none;
  }
}
.form-floating > .form-control,
.form-floating > .form-control-plaintext {
  padding: 1rem 0.75rem;
}
.form-floating > .form-control::placeholder,
.form-floating > .form-control-plaintext::placeholder {
  color: transparent;
}
.form-floating > .form-control:focus, .form-floating > .form-control:not(:placeholder-shown),
.form-floating > .form-control-plaintext:focus,
.form-floating > .form-control-plaintext:not(:placeholder-shown) {
  padding-top: 1.625rem;
  padding-bottom: 0.625rem;
}
.form-floating > .form-control:-webkit-autofill,
.form-floating > .form-control-plaintext:-webkit-autofill {
  padding-top: 1.625rem;
  padding-bottom: 0.625rem;
}
.form-floating > .form-select {
  padding-top: 1.625rem;
  padding-bottom: 0.625rem;
}
.form-floating > .form-control:focus ~ label,
.form-floating > .form-control:not(:placeholder-shown) ~ label,
.form-floating > .form-control-plaintext ~ label,
.form-floating > .form-select ~ label {
  color: rgba(var(--bs-body-color-rgb), 0.65);
  transform: scale(0.85) translateY(-0.5rem) translateX(0.15rem);
}
.form-floating > .form-control:focus ~ label::after,
.form-floating > .form-control:not(:placeholder-shown) ~ label::after,
.form-floating > .form-control-plaintext ~ label::after,
.form-floating > .form-select ~ label::after {
  position: absolute;
  inset: 1rem 0.375rem;
  z-index: -1;
  height: 1.5em;
  content: "";
  background-color: var(--bs-body-bg);
  border-radius: var(--bs-border-radius);
}
.form-floating > .form-control:-webkit-autofill ~ label {
  color: rgba(var(--bs-body-color-rgb), 0.65);
  transform: scale(0.85) translateY(-0.5rem) translateX(0.15rem);
}
.form-floating > .form-control-plaintext ~ label {
  border-width: var(--bs-border-width) 0;
}
.form-floating > :disabled ~ label,
.form-floating > .form-control:disabled ~ label {
  color: #6a737b;
}
.form-floating > :disabled ~ label::after,
.form-floating > .form-control:disabled ~ label::after {
  background-color: var(--bs-secondary-bg);
}

.input-group {
  position: relative;
  display: flex;
  flex-wrap: wrap;
  align-items: stretch;
  width: 100%;
}
.input-group > .form-control,
.input-group > .form-select,
.input-group > .form-floating {
  position: relative;
  flex: 1 1 auto;
  width: 1%;
  min-width: 0;
}
.input-group > .form-control:focus,
.input-group > .form-select:focus,
.input-group > .form-floating:focus-within {
  z-index: 5;
}
.input-group .btn {
  position: relative;
  z-index: 2;
}
.input-group .btn:focus {
  z-index: 5;
}

.input-group-text {
  display: flex;
  align-items: center;
  padding: 0.375rem 0.75rem;
  font-size: 0.9375rem;
  font-weight: 400;
  line-height: 1.5;
  color: var(--bs-body-color);
  text-align: center;
  white-space: nowrap;
  background-color: var(--bs-tertiary-bg);
  border: var(--bs-border-width) solid var(--bs-gray-500);
  border-radius: var(--bs-border-radius);
}

.input-group-lg > .form-control,
.input-group-lg > .form-select,
.input-group-lg > .input-group-text,
.input-group-lg > .btn {
  padding: 0.5rem 1rem;
  font-size: 1.171875rem;
  border-radius: var(--bs-border-radius-lg);
}

.input-group-sm > .form-control,
.input-group-sm > .form-select,
.input-group-sm > .input-group-text,
.input-group-sm > .btn {
  padding: 0.25rem 0.5rem;
  font-size: 0.8203125rem;
  border-radius: var(--bs-border-radius-sm);
}

.input-group-lg > .form-select,
.input-group-sm > .form-select {
  padding-right: 3rem;
}

.input-group:not(.has-validation) > :not(:last-child):not(.dropdown-toggle):not(.dropdown-menu):not(.form-floating),
.input-group:not(.has-validation) > .dropdown-toggle:nth-last-child(n+3),
.input-group:not(.has-validation) > .form-floating:not(:last-child) > .form-control,
.input-group:not(.has-validation) > .form-floating:not(:last-child) > .form-select {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}
.input-group.has-validation > :nth-last-child(n+3):not(.dropdown-toggle):not(.dropdown-menu):not(.form-floating),
.input-group.has-validation > .dropdown-toggle:nth-last-child(n+4),
.input-group.has-validation > .form-floating:nth-last-child(n+3) > .form-control,
.input-group.has-validation > .form-floating:nth-last-child(n+3) > .form-select {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}
.input-group > :not(:first-child):not(.dropdown-menu):not(.valid-tooltip):not(.valid-feedback):not(.invalid-tooltip):not(.invalid-feedback) {
  margin-left: calc(var(--bs-border-width) * -1);
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}
.input-group > .form-floating:not(:first-child) > .form-control,
.input-group > .form-floating:not(:first-child) > .form-select {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}

.valid-feedback {
  display: none;
  width: 100%;
  margin-top: 0.25rem;
  font-size: 0.875em;
  color: var(--bs-form-valid-color);
}

.valid-tooltip {
  position: absolute;
  top: 100%;
  z-index: 5;
  display: none;
  max-width: 100%;
  padding: 0.25rem 0.5rem;
  margin-top: 0.1rem;
  font-size: 0.8203125rem;
  color: #fff;
  background-color: var(--bs-success);
  border-radius: var(--bs-border-radius);
}

.was-validated :valid ~ .valid-feedback,
.was-validated :valid ~ .valid-tooltip,
.is-valid ~ .valid-feedback,
.is-valid ~ .valid-tooltip {
  display: block;
}

.was-validated .form-control:valid, .form-control.is-valid {
  border-color: var(--bs-form-valid-border-color);
  padding-right: calc(1.5em + 0.75rem);
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 8 8'%3e%3cpath fill='%23357a32' d='M2.3 6.73.6 4.53c-.4-1.04.46-1.4 1.1-.8l1.1 1.4 3.4-3.8c.6-.63 1.6-.27 1.2.7l-4 4.6c-.43.5-.8.4-1.1.1z'/%3e%3c/svg%3e");
  background-repeat: no-repeat;
  background-position: right calc(0.375em + 0.1875rem) center;
  background-size: calc(0.75em + 0.375rem) calc(0.75em + 0.375rem);
}
.was-validated .form-control:valid:focus, .form-control.is-valid:focus {
  border-color: var(--bs-form-valid-border-color);
  box-shadow: 0 0 0 0.25rem rgba(var(--bs-success-rgb), 0.75);
}

.was-validated textarea.form-control:valid, textarea.form-control.is-valid {
  padding-right: calc(1.5em + 0.75rem);
  background-position: top calc(0.375em + 0.1875rem) right calc(0.375em + 0.1875rem);
}

.was-validated .form-select:valid, .form-select.is-valid {
  border-color: var(--bs-form-valid-border-color);
}
.was-validated .form-select:valid:not([multiple]):not([size]), .was-validated .form-select:valid:not([multiple])[size="1"], .form-select.is-valid:not([multiple]):not([size]), .form-select.is-valid:not([multiple])[size="1"] {
  --bs-form-select-bg-icon: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 8 8'%3e%3cpath fill='%23357a32' d='M2.3 6.73.6 4.53c-.4-1.04.46-1.4 1.1-.8l1.1 1.4 3.4-3.8c.6-.63 1.6-.27 1.2.7l-4 4.6c-.43.5-.8.4-1.1.1z'/%3e%3c/svg%3e");
  padding-right: 4.125rem;
  background-position: right 0.75rem center, center right 2.25rem;
  background-size: 16px 12px, calc(0.75em + 0.375rem) calc(0.75em + 0.375rem);
}
.was-validated .form-select:valid:focus, .form-select.is-valid:focus {
  border-color: var(--bs-form-valid-border-color);
  box-shadow: 0 0 0 0.25rem rgba(var(--bs-success-rgb), 0.75);
}

.was-validated .form-control-color:valid, .form-control-color.is-valid {
  width: calc(3rem + calc(1.5em + 0.75rem));
}

.was-validated .form-check-input:valid, .form-check-input.is-valid {
  border-color: var(--bs-form-valid-border-color);
}
.was-validated .form-check-input:valid:checked, .form-check-input.is-valid:checked {
  background-color: var(--bs-form-valid-color);
}
.was-validated .form-check-input:valid:focus, .form-check-input.is-valid:focus {
  box-shadow: 0 0 0 0.25rem rgba(var(--bs-success-rgb), 0.75);
}
.was-validated .form-check-input:valid ~ .form-check-label, .form-check-input.is-valid ~ .form-check-label {
  color: var(--bs-form-valid-color);
}

.form-check-inline .form-check-input ~ .valid-feedback {
  margin-left: 0.5em;
}

.was-validated .input-group > .form-control:not(:focus):valid, .input-group > .form-control:not(:focus).is-valid,
.was-validated .input-group > .form-select:not(:focus):valid,
.input-group > .form-select:not(:focus).is-valid,
.was-validated .input-group > .form-floating:not(:focus-within):valid,
.input-group > .form-floating:not(:focus-within).is-valid {
  z-index: 3;
}

.invalid-feedback {
  display: none;
  width: 100%;
  margin-top: 0.25rem;
  font-size: 0.875em;
  color: var(--bs-form-invalid-color);
}

.invalid-tooltip {
  position: absolute;
  top: 100%;
  z-index: 5;
  display: none;
  max-width: 100%;
  padding: 0.25rem 0.5rem;
  margin-top: 0.1rem;
  font-size: 0.8203125rem;
  color: #fff;
  background-color: var(--bs-danger);
  border-radius: var(--bs-border-radius);
}

.was-validated :invalid ~ .invalid-feedback,
.was-validated :invalid ~ .invalid-tooltip,
.is-invalid ~ .invalid-feedback,
.is-invalid ~ .invalid-tooltip {
  display: block;
}

.was-validated .form-control:invalid, .form-control.is-invalid {
  border-color: var(--bs-form-invalid-border-color);
  padding-right: calc(1.5em + 0.75rem);
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 12 12' width='12' height='12' fill='none' stroke='%23ca3120'%3e%3ccircle cx='6' cy='6' r='4.5'/%3e%3cpath stroke-linejoin='round' d='M5.8 3.6h.4L6 6.5z'/%3e%3ccircle cx='6' cy='8.2' r='.6' fill='%23ca3120' stroke='none'/%3e%3c/svg%3e");
  background-repeat: no-repeat;
  background-position: right calc(0.375em + 0.1875rem) center;
  background-size: calc(0.75em + 0.375rem) calc(0.75em + 0.375rem);
}
.was-validated .form-control:invalid:focus, .form-control.is-invalid:focus {
  border-color: var(--bs-form-invalid-border-color);
  box-shadow: 0 0 0 0.25rem rgba(var(--bs-danger-rgb), 0.75);
}

.was-validated textarea.form-control:invalid, textarea.form-control.is-invalid {
  padding-right: calc(1.5em + 0.75rem);
  background-position: top calc(0.375em + 0.1875rem) right calc(0.375em + 0.1875rem);
}

.was-validated .form-select:invalid, .form-select.is-invalid {
  border-color: var(--bs-form-invalid-border-color);
}
.was-validated .form-select:invalid:not([multiple]):not([size]), .was-validated .form-select:invalid:not([multiple])[size="1"], .form-select.is-invalid:not([multiple]):not([size]), .form-select.is-invalid:not([multiple])[size="1"] {
  --bs-form-select-bg-icon: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 12 12' width='12' height='12' fill='none' stroke='%23ca3120'%3e%3ccircle cx='6' cy='6' r='4.5'/%3e%3cpath stroke-linejoin='round' d='M5.8 3.6h.4L6 6.5z'/%3e%3ccircle cx='6' cy='8.2' r='.6' fill='%23ca3120' stroke='none'/%3e%3c/svg%3e");
  padding-right: 4.125rem;
  background-position: right 0.75rem center, center right 2.25rem;
  background-size: 16px 12px, calc(0.75em + 0.375rem) calc(0.75em + 0.375rem);
}
.was-validated .form-select:invalid:focus, .form-select.is-invalid:focus {
  border-color: var(--bs-form-invalid-border-color);
  box-shadow: 0 0 0 0.25rem rgba(var(--bs-danger-rgb), 0.75);
}

.was-validated .form-control-color:invalid, .form-control-color.is-invalid {
  width: calc(3rem + calc(1.5em + 0.75rem));
}

.was-validated .form-check-input:invalid, .form-check-input.is-invalid {
  border-color: var(--bs-form-invalid-border-color);
}
.was-validated .form-check-input:invalid:checked, .form-check-input.is-invalid:checked {
  background-color: var(--bs-form-invalid-color);
}
.was-validated .form-check-input:invalid:focus, .form-check-input.is-invalid:focus {
  box-shadow: 0 0 0 0.25rem rgba(var(--bs-danger-rgb), 0.75);
}
.was-validated .form-check-input:invalid ~ .form-check-label, .form-check-input.is-invalid ~ .form-check-label {
  color: var(--bs-form-invalid-color);
}

.form-check-inline .form-check-input ~ .invalid-feedback {
  margin-left: 0.5em;
}

.was-validated .input-group > .form-control:not(:focus):invalid, .input-group > .form-control:not(:focus).is-invalid,
.was-validated .input-group > .form-select:not(:focus):invalid,
.input-group > .form-select:not(:focus).is-invalid,
.was-validated .input-group > .form-floating:not(:focus-within):invalid,
.input-group > .form-floating:not(:focus-within).is-invalid {
  z-index: 4;
}

.btn {
  --bs-btn-padding-x: 0.75rem;
  --bs-btn-padding-y: 0.375rem;
  --bs-btn-font-family: ;
  --bs-btn-font-size: 0.9375rem;
  --bs-btn-font-weight: 400;
  --bs-btn-line-height: 1.5;
  --bs-btn-color: var(--bs-body-color);
  --bs-btn-bg: transparent;
  --bs-btn-border-width: var(--bs-border-width);
  --bs-btn-border-color: transparent;
  --bs-btn-border-radius: var(--bs-border-radius);
  --bs-btn-hover-border-color: transparent;
  --bs-btn-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.15), 0 1px 1px rgba(0, 0, 0, 0.075);
  --bs-btn-disabled-opacity: 0.65;
  --bs-btn-focus-box-shadow: 0 0 0 0.25rem rgba(var(--bs-btn-focus-shadow-rgb), .5);
  display: inline-block;
  padding: var(--bs-btn-padding-y) var(--bs-btn-padding-x);
  font-family: var(--bs-btn-font-family);
  font-size: var(--bs-btn-font-size);
  font-weight: var(--bs-btn-font-weight);
  line-height: var(--bs-btn-line-height);
  color: var(--bs-btn-color);
  text-align: center;
  vertical-align: middle;
  cursor: pointer;
  user-select: none;
  border: var(--bs-btn-border-width) solid var(--bs-btn-border-color);
  border-radius: var(--bs-btn-border-radius);
  background-color: var(--bs-btn-bg);
  transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .btn {
    transition: none;
  }
}
.btn:hover {
  color: var(--bs-btn-hover-color);
  text-decoration: none;
  background-color: var(--bs-btn-hover-bg);
  border-color: var(--bs-btn-hover-border-color);
}
.btn-check + .btn:hover {
  color: var(--bs-btn-color);
  background-color: var(--bs-btn-bg);
  border-color: var(--bs-btn-border-color);
}
.btn:focus-visible {
  color: var(--bs-btn-hover-color);
  background-color: var(--bs-btn-hover-bg);
  border-color: var(--bs-btn-hover-border-color);
  outline: 0;
  box-shadow: var(--bs-btn-focus-box-shadow);
}
.btn-check:focus-visible + .btn {
  border-color: var(--bs-btn-hover-border-color);
  outline: 0;
  box-shadow: var(--bs-btn-focus-box-shadow);
}
.btn-check:checked + .btn, :not(.btn-check) + .btn:active, .btn:first-child:active, .btn.active, .btn.show {
  color: var(--bs-btn-active-color);
  background-color: var(--bs-btn-active-bg);
  border-color: var(--bs-btn-active-border-color);
}
.btn-check:checked + .btn:focus-visible, :not(.btn-check) + .btn:active:focus-visible, .btn:first-child:active:focus-visible, .btn.active:focus-visible, .btn.show:focus-visible {
  box-shadow: var(--bs-btn-focus-box-shadow);
}
.btn-check:checked:focus-visible + .btn {
  box-shadow: var(--bs-btn-focus-box-shadow);
}
.btn:disabled, .btn.disabled, fieldset:disabled .btn {
  color: var(--bs-btn-disabled-color);
  pointer-events: none;
  background-color: var(--bs-btn-disabled-bg);
  border-color: var(--bs-btn-disabled-border-color);
  opacity: var(--bs-btn-disabled-opacity);
}

.btn-primary {
  --bs-btn-color: #fff;
  --bs-btn-bg: #0f6cbf;
  --bs-btn-border-color: #0f6cbf;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #0d5ca2;
  --bs-btn-hover-border-color: #0c5699;
  --bs-btn-focus-shadow-rgb: 51, 130, 201;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #0c5699;
  --bs-btn-active-border-color: #0b518f;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #fff;
  --bs-btn-disabled-bg: #0f6cbf;
  --bs-btn-disabled-border-color: #0f6cbf;
}

.btn-secondary {
  --bs-btn-color: #000;
  --bs-btn-bg: #ced4da;
  --bs-btn-border-color: #ced4da;
  --bs-btn-hover-color: #000;
  --bs-btn-hover-bg: #d5dae0;
  --bs-btn-hover-border-color: #d3d8de;
  --bs-btn-focus-shadow-rgb: 175, 180, 185;
  --bs-btn-active-color: #000;
  --bs-btn-active-bg: #d8dde1;
  --bs-btn-active-border-color: #d3d8de;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #000;
  --bs-btn-disabled-bg: #ced4da;
  --bs-btn-disabled-border-color: #ced4da;
}

.btn-success {
  --bs-btn-color: #fff;
  --bs-btn-bg: #357a32;
  --bs-btn-border-color: #357a32;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #2d682b;
  --bs-btn-hover-border-color: #2a6228;
  --bs-btn-focus-shadow-rgb: 83, 142, 81;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #2a6228;
  --bs-btn-active-border-color: #285c26;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #fff;
  --bs-btn-disabled-bg: #357a32;
  --bs-btn-disabled-border-color: #357a32;
}

.btn-info {
  --bs-btn-color: #fff;
  --bs-btn-bg: #008196;
  --bs-btn-border-color: #008196;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #006e80;
  --bs-btn-hover-border-color: #006778;
  --bs-btn-focus-shadow-rgb: 38, 148, 166;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #006778;
  --bs-btn-active-border-color: #006171;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #fff;
  --bs-btn-disabled-bg: #008196;
  --bs-btn-disabled-border-color: #008196;
}

.btn-warning {
  --bs-btn-color: #000;
  --bs-btn-bg: #f0ad4e;
  --bs-btn-border-color: #f0ad4e;
  --bs-btn-hover-color: #000;
  --bs-btn-hover-bg: #f2b969;
  --bs-btn-hover-border-color: #f2b560;
  --bs-btn-focus-shadow-rgb: 204, 147, 66;
  --bs-btn-active-color: #000;
  --bs-btn-active-bg: #f3bd71;
  --bs-btn-active-border-color: #f2b560;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #000;
  --bs-btn-disabled-bg: #f0ad4e;
  --bs-btn-disabled-border-color: #f0ad4e;
}

.btn-danger {
  --bs-btn-color: #fff;
  --bs-btn-bg: #ca3120;
  --bs-btn-border-color: #ca3120;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #ac2a1b;
  --bs-btn-hover-border-color: #a2271a;
  --bs-btn-focus-shadow-rgb: 210, 80, 65;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #a2271a;
  --bs-btn-active-border-color: #982518;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #fff;
  --bs-btn-disabled-bg: #ca3120;
  --bs-btn-disabled-border-color: #ca3120;
}

.btn-light {
  --bs-btn-color: #000;
  --bs-btn-bg: #f8f9fa;
  --bs-btn-border-color: #f8f9fa;
  --bs-btn-hover-color: #000;
  --bs-btn-hover-bg: #d3d4d5;
  --bs-btn-hover-border-color: #c6c7c8;
  --bs-btn-focus-shadow-rgb: 211, 212, 213;
  --bs-btn-active-color: #000;
  --bs-btn-active-bg: #c6c7c8;
  --bs-btn-active-border-color: #babbbc;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #000;
  --bs-btn-disabled-bg: #f8f9fa;
  --bs-btn-disabled-border-color: #f8f9fa;
}

.btn-dark {
  --bs-btn-color: #fff;
  --bs-btn-bg: #1d2125;
  --bs-btn-border-color: #1d2125;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #3f4246;
  --bs-btn-hover-border-color: #34373b;
  --bs-btn-focus-shadow-rgb: 63, 66, 70;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #4a4d51;
  --bs-btn-active-border-color: #34373b;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #fff;
  --bs-btn-disabled-bg: #1d2125;
  --bs-btn-disabled-border-color: #1d2125;
}

.btn-outline-primary {
  --bs-btn-color: #0f6cbf;
  --bs-btn-border-color: #0f6cbf;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #0f6cbf;
  --bs-btn-hover-border-color: #0f6cbf;
  --bs-btn-focus-shadow-rgb: 15, 108, 191;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #0f6cbf;
  --bs-btn-active-border-color: #0f6cbf;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #0f6cbf;
  --bs-btn-disabled-bg: transparent;
  --bs-btn-disabled-border-color: #0f6cbf;
  --bs-gradient: none;
}

.btn-outline-secondary {
  --bs-btn-color: #ced4da;
  --bs-btn-border-color: #ced4da;
  --bs-btn-hover-color: #000;
  --bs-btn-hover-bg: #ced4da;
  --bs-btn-hover-border-color: #ced4da;
  --bs-btn-focus-shadow-rgb: 206, 212, 218;
  --bs-btn-active-color: #000;
  --bs-btn-active-bg: #ced4da;
  --bs-btn-active-border-color: #ced4da;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #ced4da;
  --bs-btn-disabled-bg: transparent;
  --bs-btn-disabled-border-color: #ced4da;
  --bs-gradient: none;
}

.btn-outline-success {
  --bs-btn-color: #357a32;
  --bs-btn-border-color: #357a32;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #357a32;
  --bs-btn-hover-border-color: #357a32;
  --bs-btn-focus-shadow-rgb: 53, 122, 50;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #357a32;
  --bs-btn-active-border-color: #357a32;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #357a32;
  --bs-btn-disabled-bg: transparent;
  --bs-btn-disabled-border-color: #357a32;
  --bs-gradient: none;
}

.btn-outline-info {
  --bs-btn-color: #008196;
  --bs-btn-border-color: #008196;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #008196;
  --bs-btn-hover-border-color: #008196;
  --bs-btn-focus-shadow-rgb: 0, 129, 150;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #008196;
  --bs-btn-active-border-color: #008196;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #008196;
  --bs-btn-disabled-bg: transparent;
  --bs-btn-disabled-border-color: #008196;
  --bs-gradient: none;
}

.btn-outline-warning {
  --bs-btn-color: #f0ad4e;
  --bs-btn-border-color: #f0ad4e;
  --bs-btn-hover-color: #000;
  --bs-btn-hover-bg: #f0ad4e;
  --bs-btn-hover-border-color: #f0ad4e;
  --bs-btn-focus-shadow-rgb: 240, 173, 78;
  --bs-btn-active-color: #000;
  --bs-btn-active-bg: #f0ad4e;
  --bs-btn-active-border-color: #f0ad4e;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #f0ad4e;
  --bs-btn-disabled-bg: transparent;
  --bs-btn-disabled-border-color: #f0ad4e;
  --bs-gradient: none;
}

.btn-outline-danger {
  --bs-btn-color: #ca3120;
  --bs-btn-border-color: #ca3120;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #ca3120;
  --bs-btn-hover-border-color: #ca3120;
  --bs-btn-focus-shadow-rgb: 202, 49, 32;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #ca3120;
  --bs-btn-active-border-color: #ca3120;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #ca3120;
  --bs-btn-disabled-bg: transparent;
  --bs-btn-disabled-border-color: #ca3120;
  --bs-gradient: none;
}

.btn-outline-light {
  --bs-btn-color: #f8f9fa;
  --bs-btn-border-color: #f8f9fa;
  --bs-btn-hover-color: #000;
  --bs-btn-hover-bg: #f8f9fa;
  --bs-btn-hover-border-color: #f8f9fa;
  --bs-btn-focus-shadow-rgb: 248, 249, 250;
  --bs-btn-active-color: #000;
  --bs-btn-active-bg: #f8f9fa;
  --bs-btn-active-border-color: #f8f9fa;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #f8f9fa;
  --bs-btn-disabled-bg: transparent;
  --bs-btn-disabled-border-color: #f8f9fa;
  --bs-gradient: none;
}

.btn-outline-dark {
  --bs-btn-color: #1d2125;
  --bs-btn-border-color: #1d2125;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #1d2125;
  --bs-btn-hover-border-color: #1d2125;
  --bs-btn-focus-shadow-rgb: 29, 33, 37;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #1d2125;
  --bs-btn-active-border-color: #1d2125;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #1d2125;
  --bs-btn-disabled-bg: transparent;
  --bs-btn-disabled-border-color: #1d2125;
  --bs-gradient: none;
}

.btn-link {
  --bs-btn-font-weight: 400;
  --bs-btn-color: var(--bs-link-color);
  --bs-btn-bg: transparent;
  --bs-btn-border-color: transparent;
  --bs-btn-hover-color: var(--bs-link-hover-color);
  --bs-btn-hover-border-color: transparent;
  --bs-btn-active-color: var(--bs-link-hover-color);
  --bs-btn-active-border-color: transparent;
  --bs-btn-disabled-color: #6a737b;
  --bs-btn-disabled-border-color: transparent;
  --bs-btn-box-shadow: 0 0 0 #000;
  --bs-btn-focus-shadow-rgb: 51, 130, 201;
  text-decoration: none;
}
.btn-link:hover, .btn-link:focus-visible {
  text-decoration: underline;
}
.btn-link:focus-visible {
  color: var(--bs-btn-color);
}
.btn-link:hover {
  color: var(--bs-btn-hover-color);
}

.btn-lg, .btn-group-lg > .btn {
  --bs-btn-padding-y: 0.5rem;
  --bs-btn-padding-x: 1rem;
  --bs-btn-font-size: 1.171875rem;
  --bs-btn-border-radius: var(--bs-border-radius-lg);
}

.btn-sm, .btn-group-sm > .btn {
  --bs-btn-padding-y: 0.25rem;
  --bs-btn-padding-x: 0.5rem;
  --bs-btn-font-size: 0.8203125rem;
  --bs-btn-border-radius: var(--bs-border-radius-sm);
}

.fade {
  transition: opacity 0.15s linear;
}
@media (prefers-reduced-motion: reduce) {
  .fade {
    transition: none;
  }
}
.fade:not(.show) {
  opacity: 0;
}

.collapse:not(.show) {
  display: none;
}

.collapsing {
  height: 0;
  overflow: hidden;
  transition: height 0.35s ease;
}
@media (prefers-reduced-motion: reduce) {
  .collapsing {
    transition: none;
  }
}
.collapsing.collapse-horizontal {
  width: 0;
  height: auto;
  transition: width 0.35s ease;
}
@media (prefers-reduced-motion: reduce) {
  .collapsing.collapse-horizontal {
    transition: none;
  }
}

.dropup,
.dropend,
.dropdown,
.dropstart,
.dropup-center,
.dropdown-center {
  position: relative;
}

.dropdown-toggle {
  white-space: nowrap;
}
.dropdown-toggle::after {
  display: inline-block;
  margin-left: 0.255em;
  vertical-align: 0.255em;
  content: "";
  border-top: 0.3em solid;
  border-right: 0.3em solid transparent;
  border-bottom: 0;
  border-left: 0.3em solid transparent;
}
.dropdown-toggle:empty::after {
  margin-left: 0;
}

.dropdown-menu {
  --bs-dropdown-zindex: 1000;
  --bs-dropdown-min-width: 10rem;
  --bs-dropdown-padding-x: 0;
  --bs-dropdown-padding-y: 0.5rem;
  --bs-dropdown-spacer: 0.125rem;
  --bs-dropdown-font-size: 0.9375rem;
  --bs-dropdown-color: var(--bs-body-color);
  --bs-dropdown-bg: var(--bs-body-bg);
  --bs-dropdown-border-color: var(--bs-border-color-translucent);
  --bs-dropdown-border-radius: var(--bs-border-radius);
  --bs-dropdown-border-width: var(--bs-border-width);
  --bs-dropdown-inner-border-radius: calc(var(--bs-border-radius) - var(--bs-border-width));
  --bs-dropdown-divider-bg: var(--bs-border-color-translucent);
  --bs-dropdown-divider-margin-y: 0.5rem;
  --bs-dropdown-box-shadow: var(--bs-box-shadow);
  --bs-dropdown-link-color: var(--bs-body-color);
  --bs-dropdown-link-hover-color: #fff;
  --bs-dropdown-link-hover-bg: #0f6cbf;
  --bs-dropdown-link-active-color: #fff;
  --bs-dropdown-link-active-bg: #0f6cbf;
  --bs-dropdown-link-disabled-color: var(--bs-tertiary-color);
  --bs-dropdown-item-padding-x: 1rem;
  --bs-dropdown-item-padding-y: 0.25rem;
  --bs-dropdown-header-color: #6a737b;
  --bs-dropdown-header-padding-x: 1rem;
  --bs-dropdown-header-padding-y: 0.5rem;
  position: absolute;
  z-index: var(--bs-dropdown-zindex);
  display: none;
  min-width: var(--bs-dropdown-min-width);
  padding: var(--bs-dropdown-padding-y) var(--bs-dropdown-padding-x);
  margin: 0;
  font-size: var(--bs-dropdown-font-size);
  color: var(--bs-dropdown-color);
  text-align: left;
  list-style: none;
  background-color: var(--bs-dropdown-bg);
  background-clip: padding-box;
  border: var(--bs-dropdown-border-width) solid var(--bs-dropdown-border-color);
  border-radius: var(--bs-dropdown-border-radius);
}
.dropdown-menu[data-bs-popper] {
  top: 100%;
  left: 0;
  margin-top: var(--bs-dropdown-spacer);
}

.dropdown-menu-start {
  --bs-position: start;
}
.dropdown-menu-start[data-bs-popper] {
  right: auto;
  left: 0;
}

.dropdown-menu-end {
  --bs-position: end;
}
.dropdown-menu-end[data-bs-popper] {
  right: 0;
  left: auto;
}

@media (min-width: 576px) {
  .dropdown-menu-sm-start {
    --bs-position: start;
  }
  .dropdown-menu-sm-start[data-bs-popper] {
    right: auto;
    left: 0;
  }
  .dropdown-menu-sm-end {
    --bs-position: end;
  }
  .dropdown-menu-sm-end[data-bs-popper] {
    right: 0;
    left: auto;
  }
}
@media (min-width: 768px) {
  .dropdown-menu-md-start {
    --bs-position: start;
  }
  .dropdown-menu-md-start[data-bs-popper] {
    right: auto;
    left: 0;
  }
  .dropdown-menu-md-end {
    --bs-position: end;
  }
  .dropdown-menu-md-end[data-bs-popper] {
    right: 0;
    left: auto;
  }
}
@media (min-width: 992px) {
  .dropdown-menu-lg-start {
    --bs-position: start;
  }
  .dropdown-menu-lg-start[data-bs-popper] {
    right: auto;
    left: 0;
  }
  .dropdown-menu-lg-end {
    --bs-position: end;
  }
  .dropdown-menu-lg-end[data-bs-popper] {
    right: 0;
    left: auto;
  }
}
@media (min-width: 1200px) {
  .dropdown-menu-xl-start {
    --bs-position: start;
  }
  .dropdown-menu-xl-start[data-bs-popper] {
    right: auto;
    left: 0;
  }
  .dropdown-menu-xl-end {
    --bs-position: end;
  }
  .dropdown-menu-xl-end[data-bs-popper] {
    right: 0;
    left: auto;
  }
}
@media (min-width: 1400px) {
  .dropdown-menu-xxl-start {
    --bs-position: start;
  }
  .dropdown-menu-xxl-start[data-bs-popper] {
    right: auto;
    left: 0;
  }
  .dropdown-menu-xxl-end {
    --bs-position: end;
  }
  .dropdown-menu-xxl-end[data-bs-popper] {
    right: 0;
    left: auto;
  }
}
.dropup .dropdown-menu[data-bs-popper] {
  top: auto;
  bottom: 100%;
  margin-top: 0;
  margin-bottom: var(--bs-dropdown-spacer);
}
.dropup .dropdown-toggle::after {
  display: inline-block;
  margin-left: 0.255em;
  vertical-align: 0.255em;
  content: "";
  border-top: 0;
  border-right: 0.3em solid transparent;
  border-bottom: 0.3em solid;
  border-left: 0.3em solid transparent;
}
.dropup .dropdown-toggle:empty::after {
  margin-left: 0;
}

.dropend .dropdown-menu[data-bs-popper] {
  top: 0;
  right: auto;
  left: 100%;
  margin-top: 0;
  margin-left: var(--bs-dropdown-spacer);
}
.dropend .dropdown-toggle::after {
  display: inline-block;
  margin-left: 0.255em;
  vertical-align: 0.255em;
  content: "";
  border-top: 0.3em solid transparent;
  border-right: 0;
  border-bottom: 0.3em solid transparent;
  border-left: 0.3em solid;
}
.dropend .dropdown-toggle:empty::after {
  margin-left: 0;
}
.dropend .dropdown-toggle::after {
  vertical-align: 0;
}

.dropstart .dropdown-menu[data-bs-popper] {
  top: 0;
  right: 100%;
  left: auto;
  margin-top: 0;
  margin-right: var(--bs-dropdown-spacer);
}
.dropstart .dropdown-toggle::after {
  display: inline-block;
  margin-left: 0.255em;
  vertical-align: 0.255em;
  content: "";
}
.dropstart .dropdown-toggle::after {
  display: none;
}
.dropstart .dropdown-toggle::before {
  display: inline-block;
  margin-right: 0.255em;
  vertical-align: 0.255em;
  content: "";
  border-top: 0.3em solid transparent;
  border-right: 0.3em solid;
  border-bottom: 0.3em solid transparent;
}
.dropstart .dropdown-toggle:empty::after {
  margin-left: 0;
}
.dropstart .dropdown-toggle::before {
  vertical-align: 0;
}

.dropdown-divider {
  height: 0;
  margin: var(--bs-dropdown-divider-margin-y) 0;
  overflow: hidden;
  border-top: 1px solid var(--bs-dropdown-divider-bg);
  opacity: 1;
}

.dropdown-item {
  display: block;
  width: 100%;
  padding: var(--bs-dropdown-item-padding-y) var(--bs-dropdown-item-padding-x);
  clear: both;
  font-weight: 400;
  color: var(--bs-dropdown-link-color);
  text-align: inherit;
  white-space: nowrap;
  background-color: transparent;
  border: 0;
  border-radius: var(--bs-dropdown-item-border-radius, 0);
}
.dropdown-item:hover, .dropdown-item:focus {
  color: var(--bs-dropdown-link-hover-color);
  text-decoration: none;
  background-color: var(--bs-dropdown-link-hover-bg);
}
.dropdown-item.active, .dropdown-item:active {
  color: var(--bs-dropdown-link-active-color);
  text-decoration: none;
  background-color: var(--bs-dropdown-link-active-bg);
}
.dropdown-item.disabled, .dropdown-item:disabled {
  color: var(--bs-dropdown-link-disabled-color);
  pointer-events: none;
  background-color: transparent;
}

.dropdown-menu.show {
  display: block;
}

.dropdown-header {
  display: block;
  padding: var(--bs-dropdown-header-padding-y) var(--bs-dropdown-header-padding-x);
  margin-bottom: 0;
  font-size: 0.8203125rem;
  color: var(--bs-dropdown-header-color);
  white-space: nowrap;
}

.dropdown-item-text {
  display: block;
  padding: var(--bs-dropdown-item-padding-y) var(--bs-dropdown-item-padding-x);
  color: var(--bs-dropdown-link-color);
}

.dropdown-menu-dark {
  --bs-dropdown-color: #dee2e6;
  --bs-dropdown-bg: #343a40;
  --bs-dropdown-border-color: var(--bs-border-color-translucent);
  --bs-dropdown-box-shadow: ;
  --bs-dropdown-link-color: #dee2e6;
  --bs-dropdown-link-hover-color: #fff;
  --bs-dropdown-divider-bg: var(--bs-border-color-translucent);
  --bs-dropdown-link-hover-bg: rgba(255, 255, 255, 0.15);
  --bs-dropdown-link-active-color: #fff;
  --bs-dropdown-link-active-bg: #0f6cbf;
  --bs-dropdown-link-disabled-color: #8f959e;
  --bs-dropdown-header-color: #8f959e;
}

.btn-group,
.btn-group-vertical {
  position: relative;
  display: inline-flex;
  vertical-align: middle;
}
.btn-group > .btn,
.btn-group-vertical > .btn {
  position: relative;
  flex: 1 1 auto;
}
.btn-group > .btn-check:checked + .btn,
.btn-group > .btn-check:focus + .btn,
.btn-group > .btn:hover,
.btn-group > .btn:focus,
.btn-group > .btn:active,
.btn-group > .btn.active,
.btn-group-vertical > .btn-check:checked + .btn,
.btn-group-vertical > .btn-check:focus + .btn,
.btn-group-vertical > .btn:hover,
.btn-group-vertical > .btn:focus,
.btn-group-vertical > .btn:active,
.btn-group-vertical > .btn.active {
  z-index: 1;
}

.btn-toolbar {
  display: flex;
  flex-wrap: wrap;
  justify-content: flex-start;
}
.btn-toolbar .input-group {
  width: auto;
}

.btn-group {
  border-radius: var(--bs-border-radius);
}
.btn-group > :not(.btn-check:first-child) + .btn,
.btn-group > .btn-group:not(:first-child) {
  margin-left: calc(var(--bs-border-width) * -1);
}
.btn-group > .btn:not(:last-child):not(.dropdown-toggle),
.btn-group > .btn.dropdown-toggle-split:first-child,
.btn-group > .btn-group:not(:last-child) > .btn {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}
.btn-group > .btn:nth-child(n+3),
.btn-group > :not(.btn-check) + .btn,
.btn-group > .btn-group:not(:first-child) > .btn {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}

.dropdown-toggle-split {
  padding-right: 0.5625rem;
  padding-left: 0.5625rem;
}
.dropdown-toggle-split::after, .dropup .dropdown-toggle-split::after, .dropend .dropdown-toggle-split::after {
  margin-left: 0;
}
.dropstart .dropdown-toggle-split::before {
  margin-right: 0;
}

.btn-sm + .dropdown-toggle-split, .btn-group-sm > .btn + .dropdown-toggle-split {
  padding-right: 0.375rem;
  padding-left: 0.375rem;
}

.btn-lg + .dropdown-toggle-split, .btn-group-lg > .btn + .dropdown-toggle-split {
  padding-right: 0.75rem;
  padding-left: 0.75rem;
}

.btn-group-vertical {
  flex-direction: column;
  align-items: flex-start;
  justify-content: center;
}
.btn-group-vertical > .btn,
.btn-group-vertical > .btn-group {
  width: 100%;
}
.btn-group-vertical > .btn:not(:first-child),
.btn-group-vertical > .btn-group:not(:first-child) {
  margin-top: calc(var(--bs-border-width) * -1);
}
.btn-group-vertical > .btn:not(:last-child):not(.dropdown-toggle),
.btn-group-vertical > .btn-group:not(:last-child) > .btn {
  border-bottom-right-radius: 0;
  border-bottom-left-radius: 0;
}
.btn-group-vertical > .btn ~ .btn,
.btn-group-vertical > .btn-group:not(:first-child) > .btn {
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}

.nav {
  --bs-nav-link-padding-x: 1rem;
  --bs-nav-link-padding-y: 0.5rem;
  --bs-nav-link-font-weight: ;
  --bs-nav-link-color: var(--bs-link-color);
  --bs-nav-link-hover-color: var(--bs-link-hover-color);
  --bs-nav-link-disabled-color: var(--bs-secondary-color);
  display: flex;
  flex-wrap: wrap;
  padding-left: 0;
  margin-bottom: 0;
  list-style: none;
}

.nav-link {
  display: block;
  padding: var(--bs-nav-link-padding-y) var(--bs-nav-link-padding-x);
  font-size: var(--bs-nav-link-font-size);
  font-weight: var(--bs-nav-link-font-weight);
  color: var(--bs-nav-link-color);
  background: none;
  border: 0;
  transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .nav-link {
    transition: none;
  }
}
.nav-link:hover, .nav-link:focus {
  color: var(--bs-nav-link-hover-color);
  text-decoration: none;
}
.nav-link:focus-visible {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.nav-link.disabled, .nav-link:disabled {
  color: var(--bs-nav-link-disabled-color);
  pointer-events: none;
  cursor: default;
}

.nav-tabs {
  --bs-nav-tabs-border-width: var(--bs-border-width);
  --bs-nav-tabs-border-color: var(--bs-border-color);
  --bs-nav-tabs-border-radius: var(--bs-border-radius);
  --bs-nav-tabs-link-hover-border-color: var(--bs-secondary-bg) var(--bs-secondary-bg) var(--bs-border-color);
  --bs-nav-tabs-link-active-color: var(--bs-emphasis-color);
  --bs-nav-tabs-link-active-bg: var(--bs-body-bg);
  --bs-nav-tabs-link-active-border-color: var(--bs-border-color) var(--bs-border-color) var(--bs-body-bg);
  border-bottom: var(--bs-nav-tabs-border-width) solid var(--bs-nav-tabs-border-color);
}
.nav-tabs .nav-link {
  margin-bottom: calc(-1 * var(--bs-nav-tabs-border-width));
  border: var(--bs-nav-tabs-border-width) solid transparent;
  border-top-left-radius: var(--bs-nav-tabs-border-radius);
  border-top-right-radius: var(--bs-nav-tabs-border-radius);
}
.nav-tabs .nav-link:hover, .nav-tabs .nav-link:focus {
  isolation: isolate;
  border-color: var(--bs-nav-tabs-link-hover-border-color);
}
.nav-tabs .nav-link.active,
.nav-tabs .nav-item.show .nav-link {
  color: var(--bs-nav-tabs-link-active-color);
  background-color: var(--bs-nav-tabs-link-active-bg);
  border-color: var(--bs-nav-tabs-link-active-border-color);
}
.nav-tabs .dropdown-menu {
  margin-top: calc(-1 * var(--bs-nav-tabs-border-width));
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}

.nav-pills {
  --bs-nav-pills-border-radius: var(--bs-border-radius);
  --bs-nav-pills-link-active-color: #fff;
  --bs-nav-pills-link-active-bg: #0f6cbf;
}
.nav-pills .nav-link {
  border-radius: var(--bs-nav-pills-border-radius);
}
.nav-pills .nav-link.active,
.nav-pills .show > .nav-link {
  color: var(--bs-nav-pills-link-active-color);
  background-color: var(--bs-nav-pills-link-active-bg);
}

.nav-underline {
  --bs-nav-underline-gap: 1rem;
  --bs-nav-underline-border-width: 0.125rem;
  --bs-nav-underline-link-active-color: var(--bs-emphasis-color);
  gap: var(--bs-nav-underline-gap);
}
.nav-underline .nav-link {
  padding-right: 0;
  padding-left: 0;
  border-bottom: var(--bs-nav-underline-border-width) solid transparent;
}
.nav-underline .nav-link:hover, .nav-underline .nav-link:focus {
  border-bottom-color: currentcolor;
}
.nav-underline .nav-link.active,
.nav-underline .show > .nav-link {
  font-weight: 700;
  color: var(--bs-nav-underline-link-active-color);
  border-bottom-color: currentcolor;
}

.nav-fill > .nav-link,
.nav-fill .nav-item {
  flex: 1 1 auto;
  text-align: center;
}

.nav-justified > .nav-link,
.nav-justified .nav-item {
  flex-basis: 0;
  flex-grow: 1;
  text-align: center;
}

.nav-fill .nav-item .nav-link,
.nav-justified .nav-item .nav-link {
  width: 100%;
}

.tab-content > .tab-pane {
  display: none;
}
.tab-content > .active {
  display: block;
}

.navbar {
  --bs-navbar-padding-x: 0;
  --bs-navbar-padding-y: 0.5rem;
  --bs-navbar-color: rgba(var(--bs-emphasis-color-rgb), 0.65);
  --bs-navbar-hover-color: rgba(var(--bs-emphasis-color-rgb), 0.8);
  --bs-navbar-disabled-color: rgba(var(--bs-emphasis-color-rgb), 0.3);
  --bs-navbar-active-color: rgba(var(--bs-emphasis-color-rgb), 1);
  --bs-navbar-brand-padding-y: 0.32421875rem;
  --bs-navbar-brand-margin-end: 1rem;
  --bs-navbar-brand-font-size: 1.171875rem;
  --bs-navbar-brand-color: rgba(var(--bs-emphasis-color-rgb), 1);
  --bs-navbar-brand-hover-color: rgba(var(--bs-emphasis-color-rgb), 1);
  --bs-navbar-nav-link-padding-x: 0.5rem;
  --bs-navbar-toggler-padding-y: 0.25rem;
  --bs-navbar-toggler-padding-x: 0.75rem;
  --bs-navbar-toggler-font-size: 1.171875rem;
  --bs-navbar-toggler-icon-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 30 30'%3e%3cpath stroke='rgba%2829, 33, 37, 0.75%29' stroke-linecap='round' stroke-miterlimit='10' stroke-width='2' d='M4 7h22M4 15h22M4 23h22'/%3e%3c/svg%3e");
  --bs-navbar-toggler-border-color: rgba(var(--bs-emphasis-color-rgb), 0.15);
  --bs-navbar-toggler-border-radius: var(--bs-border-radius);
  --bs-navbar-toggler-focus-width: 0.25rem;
  --bs-navbar-toggler-transition: box-shadow 0.15s ease-in-out;
  position: relative;
  display: flex;
  flex-wrap: wrap;
  align-items: center;
  justify-content: space-between;
  padding: var(--bs-navbar-padding-y) var(--bs-navbar-padding-x);
}
.navbar > .container,
.navbar > .container-fluid,
.navbar > .container-sm,
.navbar > .container-md,
.navbar > .container-lg,
.navbar > .container-xl,
.navbar > .container-xxl {
  display: flex;
  flex-wrap: inherit;
  align-items: center;
  justify-content: space-between;
}
.navbar-brand {
  padding-top: var(--bs-navbar-brand-padding-y);
  padding-bottom: var(--bs-navbar-brand-padding-y);
  margin-right: var(--bs-navbar-brand-margin-end);
  font-size: var(--bs-navbar-brand-font-size);
  color: var(--bs-navbar-brand-color);
  white-space: nowrap;
}
.navbar-brand:hover, .navbar-brand:focus {
  color: var(--bs-navbar-brand-hover-color);
  text-decoration: none;
}

.navbar-nav {
  --bs-nav-link-padding-x: 0;
  --bs-nav-link-padding-y: 0.5rem;
  --bs-nav-link-font-weight: ;
  --bs-nav-link-color: var(--bs-navbar-color);
  --bs-nav-link-hover-color: var(--bs-navbar-hover-color);
  --bs-nav-link-disabled-color: var(--bs-navbar-disabled-color);
  display: flex;
  flex-direction: column;
  padding-left: 0;
  margin-bottom: 0;
  list-style: none;
}
.navbar-nav .nav-link.active, .navbar-nav .nav-link.show {
  color: var(--bs-navbar-active-color);
}
.navbar-nav .dropdown-menu {
  position: static;
}

.navbar-text {
  padding-top: 0.5rem;
  padding-bottom: 0.5rem;
  color: var(--bs-navbar-color);
}
.navbar-text a,
.navbar-text a:hover,
.navbar-text a:focus {
  color: var(--bs-navbar-active-color);
}

.navbar-collapse {
  flex-basis: 100%;
  flex-grow: 1;
  align-items: center;
}

.navbar-toggler {
  padding: var(--bs-navbar-toggler-padding-y) var(--bs-navbar-toggler-padding-x);
  font-size: var(--bs-navbar-toggler-font-size);
  line-height: 1;
  color: var(--bs-navbar-color);
  background-color: transparent;
  border: var(--bs-border-width) solid var(--bs-navbar-toggler-border-color);
  border-radius: var(--bs-navbar-toggler-border-radius);
  transition: var(--bs-navbar-toggler-transition);
}
@media (prefers-reduced-motion: reduce) {
  .navbar-toggler {
    transition: none;
  }
}
.navbar-toggler:hover {
  text-decoration: none;
}
.navbar-toggler:focus {
  text-decoration: none;
  outline: 0;
  box-shadow: 0 0 0 var(--bs-navbar-toggler-focus-width);
}

.navbar-toggler-icon {
  display: inline-block;
  width: 1.5em;
  height: 1.5em;
  vertical-align: middle;
  background-image: var(--bs-navbar-toggler-icon-bg);
  background-repeat: no-repeat;
  background-position: center;
  background-size: 100%;
}

.navbar-nav-scroll {
  max-height: var(--bs-scroll-height, 75vh);
  overflow-y: auto;
}

@media (min-width: 576px) {
  .navbar-expand-sm {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }
  .navbar-expand-sm .navbar-nav {
    flex-direction: row;
  }
  .navbar-expand-sm .navbar-nav .dropdown-menu {
    position: absolute;
  }
  .navbar-expand-sm .navbar-nav .nav-link {
    padding-right: var(--bs-navbar-nav-link-padding-x);
    padding-left: var(--bs-navbar-nav-link-padding-x);
  }
  .navbar-expand-sm .navbar-nav-scroll {
    overflow: visible;
  }
  .navbar-expand-sm .navbar-collapse {
    display: flex !important;
    flex-basis: auto;
  }
  .navbar-expand-sm .navbar-toggler {
    display: none;
  }
  .navbar-expand-sm .offcanvas {
    position: static;
    z-index: auto;
    flex-grow: 1;
    width: auto !important;
    height: auto !important;
    visibility: visible !important;
    background-color: transparent !important;
    border: 0 !important;
    transform: none !important;
    transition: none;
  }
  .navbar-expand-sm .offcanvas .offcanvas-header {
    display: none;
  }
  .navbar-expand-sm .offcanvas .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
  }
}
@media (min-width: 768px) {
  .navbar-expand-md {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }
  .navbar-expand-md .navbar-nav {
    flex-direction: row;
  }
  .navbar-expand-md .navbar-nav .dropdown-menu {
    position: absolute;
  }
  .navbar-expand-md .navbar-nav .nav-link {
    padding-right: var(--bs-navbar-nav-link-padding-x);
    padding-left: var(--bs-navbar-nav-link-padding-x);
  }
  .navbar-expand-md .navbar-nav-scroll {
    overflow: visible;
  }
  .navbar-expand-md .navbar-collapse {
    display: flex !important;
    flex-basis: auto;
  }
  .navbar-expand-md .navbar-toggler {
    display: none;
  }
  .navbar-expand-md .offcanvas {
    position: static;
    z-index: auto;
    flex-grow: 1;
    width: auto !important;
    height: auto !important;
    visibility: visible !important;
    background-color: transparent !important;
    border: 0 !important;
    transform: none !important;
    transition: none;
  }
  .navbar-expand-md .offcanvas .offcanvas-header {
    display: none;
  }
  .navbar-expand-md .offcanvas .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
  }
}
@media (min-width: 992px) {
  .navbar-expand-lg {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }
  .navbar-expand-lg .navbar-nav {
    flex-direction: row;
  }
  .navbar-expand-lg .navbar-nav .dropdown-menu {
    position: absolute;
  }
  .navbar-expand-lg .navbar-nav .nav-link {
    padding-right: var(--bs-navbar-nav-link-padding-x);
    padding-left: var(--bs-navbar-nav-link-padding-x);
  }
  .navbar-expand-lg .navbar-nav-scroll {
    overflow: visible;
  }
  .navbar-expand-lg .navbar-collapse {
    display: flex !important;
    flex-basis: auto;
  }
  .navbar-expand-lg .navbar-toggler {
    display: none;
  }
  .navbar-expand-lg .offcanvas {
    position: static;
    z-index: auto;
    flex-grow: 1;
    width: auto !important;
    height: auto !important;
    visibility: visible !important;
    background-color: transparent !important;
    border: 0 !important;
    transform: none !important;
    transition: none;
  }
  .navbar-expand-lg .offcanvas .offcanvas-header {
    display: none;
  }
  .navbar-expand-lg .offcanvas .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
  }
}
@media (min-width: 1200px) {
  .navbar-expand-xl {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }
  .navbar-expand-xl .navbar-nav {
    flex-direction: row;
  }
  .navbar-expand-xl .navbar-nav .dropdown-menu {
    position: absolute;
  }
  .navbar-expand-xl .navbar-nav .nav-link {
    padding-right: var(--bs-navbar-nav-link-padding-x);
    padding-left: var(--bs-navbar-nav-link-padding-x);
  }
  .navbar-expand-xl .navbar-nav-scroll {
    overflow: visible;
  }
  .navbar-expand-xl .navbar-collapse {
    display: flex !important;
    flex-basis: auto;
  }
  .navbar-expand-xl .navbar-toggler {
    display: none;
  }
  .navbar-expand-xl .offcanvas {
    position: static;
    z-index: auto;
    flex-grow: 1;
    width: auto !important;
    height: auto !important;
    visibility: visible !important;
    background-color: transparent !important;
    border: 0 !important;
    transform: none !important;
    transition: none;
  }
  .navbar-expand-xl .offcanvas .offcanvas-header {
    display: none;
  }
  .navbar-expand-xl .offcanvas .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
  }
}
@media (min-width: 1400px) {
  .navbar-expand-xxl {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }
  .navbar-expand-xxl .navbar-nav {
    flex-direction: row;
  }
  .navbar-expand-xxl .navbar-nav .dropdown-menu {
    position: absolute;
  }
  .navbar-expand-xxl .navbar-nav .nav-link {
    padding-right: var(--bs-navbar-nav-link-padding-x);
    padding-left: var(--bs-navbar-nav-link-padding-x);
  }
  .navbar-expand-xxl .navbar-nav-scroll {
    overflow: visible;
  }
  .navbar-expand-xxl .navbar-collapse {
    display: flex !important;
    flex-basis: auto;
  }
  .navbar-expand-xxl .navbar-toggler {
    display: none;
  }
  .navbar-expand-xxl .offcanvas {
    position: static;
    z-index: auto;
    flex-grow: 1;
    width: auto !important;
    height: auto !important;
    visibility: visible !important;
    background-color: transparent !important;
    border: 0 !important;
    transform: none !important;
    transition: none;
  }
  .navbar-expand-xxl .offcanvas .offcanvas-header {
    display: none;
  }
  .navbar-expand-xxl .offcanvas .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
  }
}
.navbar-expand {
  flex-wrap: nowrap;
  justify-content: flex-start;
}
.navbar-expand .navbar-nav {
  flex-direction: row;
}
.navbar-expand .navbar-nav .dropdown-menu {
  position: absolute;
}
.navbar-expand .navbar-nav .nav-link {
  padding-right: var(--bs-navbar-nav-link-padding-x);
  padding-left: var(--bs-navbar-nav-link-padding-x);
}
.navbar-expand .navbar-nav-scroll {
  overflow: visible;
}
.navbar-expand .navbar-collapse {
  display: flex !important;
  flex-basis: auto;
}
.navbar-expand .navbar-toggler {
  display: none;
}
.navbar-expand .offcanvas {
  position: static;
  z-index: auto;
  flex-grow: 1;
  width: auto !important;
  height: auto !important;
  visibility: visible !important;
  background-color: transparent !important;
  border: 0 !important;
  transform: none !important;
  transition: none;
}
.navbar-expand .offcanvas .offcanvas-header {
  display: none;
}
.navbar-expand .offcanvas .offcanvas-body {
  display: flex;
  flex-grow: 0;
  padding: 0;
  overflow-y: visible;
}

.navbar-dark,
.navbar[data-bs-theme=dark] {
  --bs-navbar-color: rgba(255, 255, 255, 0.55);
  --bs-navbar-hover-color: rgba(255, 255, 255, 0.75);
  --bs-navbar-disabled-color: rgba(255, 255, 255, 0.25);
  --bs-navbar-active-color: #fff;
  --bs-navbar-brand-color: #fff;
  --bs-navbar-brand-hover-color: #fff;
  --bs-navbar-toggler-border-color: rgba(255, 255, 255, 0.1);
  --bs-navbar-toggler-icon-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 30 30'%3e%3cpath stroke='rgba%28255, 255, 255, 0.55%29' stroke-linecap='round' stroke-miterlimit='10' stroke-width='2' d='M4 7h22M4 15h22M4 23h22'/%3e%3c/svg%3e");
}

[data-bs-theme=dark] .navbar-toggler-icon {
  --bs-navbar-toggler-icon-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 30 30'%3e%3cpath stroke='rgba%28255, 255, 255, 0.55%29' stroke-linecap='round' stroke-miterlimit='10' stroke-width='2' d='M4 7h22M4 15h22M4 23h22'/%3e%3c/svg%3e");
}

.card {
  --bs-card-spacer-y: 1rem;
  --bs-card-spacer-x: 1rem;
  --bs-card-title-spacer-y: 0.5rem;
  --bs-card-title-color: ;
  --bs-card-subtitle-color: ;
  --bs-card-border-width: var(--bs-border-width);
  --bs-card-border-color: var(--bs-border-color-translucent);
  --bs-card-border-radius: var(--bs-border-radius);
  --bs-card-box-shadow: ;
  --bs-card-inner-border-radius: calc(var(--bs-border-radius) - (var(--bs-border-width)));
  --bs-card-cap-padding-y: 0.5rem;
  --bs-card-cap-padding-x: 1rem;
  --bs-card-cap-bg: rgba(var(--bs-body-color-rgb), 0.03);
  --bs-card-cap-color: ;
  --bs-card-height: ;
  --bs-card-color: ;
  --bs-card-bg: var(--bs-body-bg);
  --bs-card-img-overlay-padding: 1rem;
  --bs-card-group-margin: 0.25rem;
  position: relative;
  display: flex;
  flex-direction: column;
  min-width: 0;
  height: var(--bs-card-height);
  color: var(--bs-body-color);
  word-wrap: break-word;
  background-color: var(--bs-card-bg);
  background-clip: border-box;
  border: var(--bs-card-border-width) solid var(--bs-card-border-color);
  border-radius: var(--bs-card-border-radius);
}
.card > hr {
  margin-right: 0;
  margin-left: 0;
}
.card > .list-group {
  border-top: inherit;
  border-bottom: inherit;
}
.card > .list-group:first-child {
  border-top-width: 0;
  border-top-left-radius: var(--bs-card-inner-border-radius);
  border-top-right-radius: var(--bs-card-inner-border-radius);
}
.card > .list-group:last-child {
  border-bottom-width: 0;
  border-bottom-right-radius: var(--bs-card-inner-border-radius);
  border-bottom-left-radius: var(--bs-card-inner-border-radius);
}
.card > .card-header + .list-group,
.card > .list-group + .card-footer {
  border-top: 0;
}

.card-body {
  flex: 1 1 auto;
  padding: var(--bs-card-spacer-y) var(--bs-card-spacer-x);
  color: var(--bs-card-color);
}

.card-title {
  margin-bottom: var(--bs-card-title-spacer-y);
  color: var(--bs-card-title-color);
}

.card-subtitle {
  margin-top: calc(-0.5 * var(--bs-card-title-spacer-y));
  margin-bottom: 0;
  color: var(--bs-card-subtitle-color);
}

.card-text:last-child {
  margin-bottom: 0;
}

.card-link:hover {
  text-decoration: none;
}
.card-link + .card-link {
  margin-left: var(--bs-card-spacer-x);
}

.card-header {
  padding: var(--bs-card-cap-padding-y) var(--bs-card-cap-padding-x);
  margin-bottom: 0;
  color: var(--bs-card-cap-color);
  background-color: var(--bs-card-cap-bg);
  border-bottom: var(--bs-card-border-width) solid var(--bs-card-border-color);
}
.card-header:first-child {
  border-radius: var(--bs-card-inner-border-radius) var(--bs-card-inner-border-radius) 0 0;
}

.card-footer {
  padding: var(--bs-card-cap-padding-y) var(--bs-card-cap-padding-x);
  color: var(--bs-card-cap-color);
  background-color: var(--bs-card-cap-bg);
  border-top: var(--bs-card-border-width) solid var(--bs-card-border-color);
}
.card-footer:last-child {
  border-radius: 0 0 var(--bs-card-inner-border-radius) var(--bs-card-inner-border-radius);
}

.card-header-tabs {
  margin-right: calc(-0.5 * var(--bs-card-cap-padding-x));
  margin-bottom: calc(-1 * var(--bs-card-cap-padding-y));
  margin-left: calc(-0.5 * var(--bs-card-cap-padding-x));
  border-bottom: 0;
}
.card-header-tabs .nav-link.active {
  background-color: var(--bs-card-bg);
  border-bottom-color: var(--bs-card-bg);
}

.card-header-pills {
  margin-right: calc(-0.5 * var(--bs-card-cap-padding-x));
  margin-left: calc(-0.5 * var(--bs-card-cap-padding-x));
}

.card-img-overlay {
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
  padding: var(--bs-card-img-overlay-padding);
  border-radius: var(--bs-card-inner-border-radius);
}

.card-img,
.card-img-top,
.card-img-bottom {
  width: 100%;
}

.card-img,
.card-img-top {
  border-top-left-radius: var(--bs-card-inner-border-radius);
  border-top-right-radius: var(--bs-card-inner-border-radius);
}

.card-img,
.card-img-bottom {
  border-bottom-right-radius: var(--bs-card-inner-border-radius);
  border-bottom-left-radius: var(--bs-card-inner-border-radius);
}

.card-group > .card {
  margin-bottom: var(--bs-card-group-margin);
}
@media (min-width: 576px) {
  .card-group {
    display: flex;
    flex-flow: row wrap;
  }
  .card-group > .card {
    flex: 1 0 0%;
    margin-bottom: 0;
  }
  .card-group > .card + .card {
    margin-left: 0;
    border-left: 0;
  }
  .card-group > .card:not(:last-child) {
    border-top-right-radius: 0;
    border-bottom-right-radius: 0;
  }
  .card-group > .card:not(:last-child) .card-img-top,
  .card-group > .card:not(:last-child) .card-header {
    border-top-right-radius: 0;
  }
  .card-group > .card:not(:last-child) .card-img-bottom,
  .card-group > .card:not(:last-child) .card-footer {
    border-bottom-right-radius: 0;
  }
  .card-group > .card:not(:first-child) {
    border-top-left-radius: 0;
    border-bottom-left-radius: 0;
  }
  .card-group > .card:not(:first-child) .card-img-top,
  .card-group > .card:not(:first-child) .card-header {
    border-top-left-radius: 0;
  }
  .card-group > .card:not(:first-child) .card-img-bottom,
  .card-group > .card:not(:first-child) .card-footer {
    border-bottom-left-radius: 0;
  }
}

.accordion {
  --bs-accordion-color: var(--bs-body-color);
  --bs-accordion-bg: var(--bs-body-bg);
  --bs-accordion-transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out, border-radius 0.15s ease;
  --bs-accordion-border-color: var(--bs-border-color);
  --bs-accordion-border-width: var(--bs-border-width);
  --bs-accordion-border-radius: var(--bs-border-radius);
  --bs-accordion-inner-border-radius: calc(var(--bs-border-radius) - (var(--bs-border-width)));
  --bs-accordion-btn-padding-x: 1.25rem;
  --bs-accordion-btn-padding-y: 1rem;
  --bs-accordion-btn-color: var(--bs-body-color);
  --bs-accordion-btn-bg: var(--bs-accordion-bg);
  --bs-accordion-btn-icon: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='none' stroke='%231d2125' stroke-linecap='round' stroke-linejoin='round'%3e%3cpath d='M2 5L8 11L14 5'/%3e%3c/svg%3e");
  --bs-accordion-btn-icon-width: 1.25rem;
  --bs-accordion-btn-icon-transform: rotate(-180deg);
  --bs-accordion-btn-icon-transition: transform 0.2s ease-in-out;
  --bs-accordion-btn-active-icon: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='none' stroke='%23062b4c' stroke-linecap='round' stroke-linejoin='round'%3e%3cpath d='M2 5L8 11L14 5'/%3e%3c/svg%3e");
  --bs-accordion-btn-focus-box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
  --bs-accordion-body-padding-x: 1.25rem;
  --bs-accordion-body-padding-y: 1rem;
  --bs-accordion-active-color: var(--bs-primary-text-emphasis);
  --bs-accordion-active-bg: var(--bs-primary-bg-subtle);
}

.accordion-button {
  position: relative;
  display: flex;
  align-items: center;
  width: 100%;
  padding: var(--bs-accordion-btn-padding-y) var(--bs-accordion-btn-padding-x);
  font-size: 0.9375rem;
  color: var(--bs-accordion-btn-color);
  text-align: left;
  background-color: var(--bs-accordion-btn-bg);
  border: 0;
  border-radius: 0;
  overflow-anchor: none;
  transition: var(--bs-accordion-transition);
}
@media (prefers-reduced-motion: reduce) {
  .accordion-button {
    transition: none;
  }
}
.accordion-button:not(.collapsed) {
  color: var(--bs-accordion-active-color);
  background-color: var(--bs-accordion-active-bg);
  box-shadow: inset 0 calc(-1 * var(--bs-accordion-border-width)) 0 var(--bs-accordion-border-color);
}
.accordion-button:not(.collapsed)::after {
  background-image: var(--bs-accordion-btn-active-icon);
  transform: var(--bs-accordion-btn-icon-transform);
}
.accordion-button::after {
  flex-shrink: 0;
  width: var(--bs-accordion-btn-icon-width);
  height: var(--bs-accordion-btn-icon-width);
  margin-left: auto;
  content: "";
  background-image: var(--bs-accordion-btn-icon);
  background-repeat: no-repeat;
  background-size: var(--bs-accordion-btn-icon-width);
  transition: var(--bs-accordion-btn-icon-transition);
}
@media (prefers-reduced-motion: reduce) {
  .accordion-button::after {
    transition: none;
  }
}
.accordion-button:hover {
  z-index: 2;
}
.accordion-button:focus {
  z-index: 3;
  outline: 0;
  box-shadow: var(--bs-accordion-btn-focus-box-shadow);
}

.accordion-header {
  margin-bottom: 0;
}

.accordion-item {
  color: var(--bs-accordion-color);
  background-color: var(--bs-accordion-bg);
  border: var(--bs-accordion-border-width) solid var(--bs-accordion-border-color);
}
.accordion-item:first-of-type {
  border-top-left-radius: var(--bs-accordion-border-radius);
  border-top-right-radius: var(--bs-accordion-border-radius);
}
.accordion-item:first-of-type > .accordion-header .accordion-button {
  border-top-left-radius: var(--bs-accordion-inner-border-radius);
  border-top-right-radius: var(--bs-accordion-inner-border-radius);
}
.accordion-item:not(:first-of-type) {
  border-top: 0;
}
.accordion-item:last-of-type {
  border-bottom-right-radius: var(--bs-accordion-border-radius);
  border-bottom-left-radius: var(--bs-accordion-border-radius);
}
.accordion-item:last-of-type > .accordion-header .accordion-button.collapsed {
  border-bottom-right-radius: var(--bs-accordion-inner-border-radius);
  border-bottom-left-radius: var(--bs-accordion-inner-border-radius);
}
.accordion-item:last-of-type > .accordion-collapse {
  border-bottom-right-radius: var(--bs-accordion-border-radius);
  border-bottom-left-radius: var(--bs-accordion-border-radius);
}

.accordion-body {
  padding: var(--bs-accordion-body-padding-y) var(--bs-accordion-body-padding-x);
}

.accordion-flush > .accordion-item {
  border-right: 0;
  border-left: 0;
  border-radius: 0;
}
.accordion-flush > .accordion-item:first-child {
  border-top: 0;
}
.accordion-flush > .accordion-item:last-child {
  border-bottom: 0;
}
.accordion-flush > .accordion-item > .accordion-header .accordion-button, .accordion-flush > .accordion-item > .accordion-header .accordion-button.collapsed {
  border-radius: 0;
}
.accordion-flush > .accordion-item > .accordion-collapse {
  border-radius: 0;
}

[data-bs-theme=dark] .accordion-button::after {
  --bs-accordion-btn-icon: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='%236fa7d9'%3e%3cpath fill-rule='evenodd' d='M1.646 4.646a.5.5 0 0 1 .708 0L8 10.293l5.646-5.647a.5.5 0 0 1 .708.708l-6 6a.5.5 0 0 1-.708 0l-6-6a.5.5 0 0 1 0-.708z'/%3e%3c/svg%3e");
  --bs-accordion-btn-active-icon: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='%236fa7d9'%3e%3cpath fill-rule='evenodd' d='M1.646 4.646a.5.5 0 0 1 .708 0L8 10.293l5.646-5.647a.5.5 0 0 1 .708.708l-6 6a.5.5 0 0 1-.708 0l-6-6a.5.5 0 0 1 0-.708z'/%3e%3c/svg%3e");
}

.breadcrumb {
  --bs-breadcrumb-padding-x: 0;
  --bs-breadcrumb-padding-y: 0.5rem;
  --bs-breadcrumb-margin-bottom: 0;
  --bs-breadcrumb-bg: ;
  --bs-breadcrumb-border-radius: ;
  --bs-breadcrumb-divider-color: var(--bs-secondary-color);
  --bs-breadcrumb-item-padding-x: 0.5rem;
  --bs-breadcrumb-item-active-color: var(--bs-secondary-color);
  display: flex;
  flex-wrap: wrap;
  padding: var(--bs-breadcrumb-padding-y) var(--bs-breadcrumb-padding-x);
  margin-bottom: var(--bs-breadcrumb-margin-bottom);
  font-size: var(--bs-breadcrumb-font-size);
  list-style: none;
  background-color: var(--bs-breadcrumb-bg);
  border-radius: var(--bs-breadcrumb-border-radius);
}

.breadcrumb-item + .breadcrumb-item {
  padding-left: var(--bs-breadcrumb-item-padding-x);
}
.breadcrumb-item + .breadcrumb-item::before {
  float: left;
  padding-right: var(--bs-breadcrumb-item-padding-x);
  color: var(--bs-breadcrumb-divider-color);
  content: var(--bs-breadcrumb-divider, "/") /* rtl: var(--bs-breadcrumb-divider, "/") */;
}
.breadcrumb-item.active {
  color: var(--bs-breadcrumb-item-active-color);
}

.pagination {
  --bs-pagination-padding-x: 0.75rem;
  --bs-pagination-padding-y: 0.375rem;
  --bs-pagination-font-size: 0.9375rem;
  --bs-pagination-color: var(--bs-link-color);
  --bs-pagination-bg: var(--bs-body-bg);
  --bs-pagination-border-width: var(--bs-border-width);
  --bs-pagination-border-color: var(--bs-border-color);
  --bs-pagination-border-radius: var(--bs-border-radius);
  --bs-pagination-hover-color: var(--bs-link-hover-color);
  --bs-pagination-hover-bg: var(--bs-tertiary-bg);
  --bs-pagination-hover-border-color: var(--bs-border-color);
  --bs-pagination-focus-color: var(--bs-link-hover-color);
  --bs-pagination-focus-bg: var(--bs-secondary-bg);
  --bs-pagination-focus-box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
  --bs-pagination-active-color: #fff;
  --bs-pagination-active-bg: #0f6cbf;
  --bs-pagination-active-border-color: #0f6cbf;
  --bs-pagination-disabled-color: var(--bs-secondary-color);
  --bs-pagination-disabled-bg: var(--bs-secondary-bg);
  --bs-pagination-disabled-border-color: var(--bs-border-color);
  display: flex;
  padding-left: 0;
  list-style: none;
}

.page-link {
  position: relative;
  display: block;
  padding: var(--bs-pagination-padding-y) var(--bs-pagination-padding-x);
  font-size: var(--bs-pagination-font-size);
  color: var(--bs-pagination-color);
  background-color: var(--bs-pagination-bg);
  border: var(--bs-pagination-border-width) solid var(--bs-pagination-border-color);
  transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .page-link {
    transition: none;
  }
}
.page-link:hover {
  z-index: 2;
  color: var(--bs-pagination-hover-color);
  text-decoration: none;
  background-color: var(--bs-pagination-hover-bg);
  border-color: var(--bs-pagination-hover-border-color);
}
.page-link:focus {
  z-index: 3;
  color: var(--bs-pagination-focus-color);
  background-color: var(--bs-pagination-focus-bg);
  outline: 0;
  box-shadow: var(--bs-pagination-focus-box-shadow);
}
.page-link.active, .active > .page-link {
  z-index: 3;
  color: var(--bs-pagination-active-color);
  background-color: var(--bs-pagination-active-bg);
  border-color: var(--bs-pagination-active-border-color);
}
.page-link.disabled, .disabled > .page-link {
  color: var(--bs-pagination-disabled-color);
  pointer-events: none;
  background-color: var(--bs-pagination-disabled-bg);
  border-color: var(--bs-pagination-disabled-border-color);
}

.page-item:not(:first-child) .page-link {
  margin-left: calc(var(--bs-border-width) * -1);
}
.page-item:first-child .page-link {
  border-top-left-radius: var(--bs-pagination-border-radius);
  border-bottom-left-radius: var(--bs-pagination-border-radius);
}
.page-item:last-child .page-link {
  border-top-right-radius: var(--bs-pagination-border-radius);
  border-bottom-right-radius: var(--bs-pagination-border-radius);
}

.pagination-lg {
  --bs-pagination-padding-x: 1.5rem;
  --bs-pagination-padding-y: 0.75rem;
  --bs-pagination-font-size: 1.171875rem;
  --bs-pagination-border-radius: var(--bs-border-radius-lg);
}

.pagination-sm {
  --bs-pagination-padding-x: 0.5rem;
  --bs-pagination-padding-y: 0.25rem;
  --bs-pagination-font-size: 0.8203125rem;
  --bs-pagination-border-radius: var(--bs-border-radius-sm);
}

.badge {
  --bs-badge-padding-x: 0.65em;
  --bs-badge-padding-y: 0.35em;
  --bs-badge-font-size: 0.75em;
  --bs-badge-font-weight: 700;
  --bs-badge-color: #fff;
  --bs-badge-border-radius: var(--bs-border-radius);
  display: inline-block;
  padding: var(--bs-badge-padding-y) var(--bs-badge-padding-x);
  font-size: var(--bs-badge-font-size);
  font-weight: var(--bs-badge-font-weight);
  line-height: 1;
  color: var(--bs-badge-color);
  text-align: center;
  white-space: nowrap;
  vertical-align: baseline;
  border-radius: var(--bs-badge-border-radius);
}
.badge:empty {
  display: none;
}

.btn .badge {
  position: relative;
  top: -1px;
}

.alert {
  --bs-alert-bg: transparent;
  --bs-alert-padding-x: 1rem;
  --bs-alert-padding-y: 1rem;
  --bs-alert-margin-bottom: 1rem;
  --bs-alert-color: inherit;
  --bs-alert-border-color: transparent;
  --bs-alert-border: var(--bs-border-width) solid var(--bs-alert-border-color);
  --bs-alert-border-radius: var(--bs-border-radius);
  --bs-alert-link-color: inherit;
  position: relative;
  padding: var(--bs-alert-padding-y) var(--bs-alert-padding-x);
  margin-bottom: var(--bs-alert-margin-bottom);
  color: var(--bs-alert-color);
  background-color: var(--bs-alert-bg);
  border: var(--bs-alert-border);
  border-radius: var(--bs-alert-border-radius);
}

.alert-heading {
  color: inherit;
}

.alert-link {
  font-weight: 700;
  color: var(--bs-alert-link-color);
}

.alert-dismissible {
  padding-right: 3rem;
}
.alert-dismissible .btn-close {
  position: absolute;
  top: 0;
  right: 0;
  z-index: 2;
  padding: 1.25rem 1rem;
}

.alert-primary {
  --bs-alert-color: var(--bs-primary-text-emphasis);
  --bs-alert-bg: var(--bs-primary-bg-subtle);
  --bs-alert-border-color: var(--bs-primary-border-subtle);
  --bs-alert-link-color: var(--bs-primary-text-emphasis);
}

.alert-secondary {
  --bs-alert-color: var(--bs-secondary-text-emphasis);
  --bs-alert-bg: var(--bs-secondary-bg-subtle);
  --bs-alert-border-color: var(--bs-secondary-border-subtle);
  --bs-alert-link-color: var(--bs-secondary-text-emphasis);
}

.alert-success {
  --bs-alert-color: var(--bs-success-text-emphasis);
  --bs-alert-bg: var(--bs-success-bg-subtle);
  --bs-alert-border-color: var(--bs-success-border-subtle);
  --bs-alert-link-color: var(--bs-success-text-emphasis);
}

.alert-info {
  --bs-alert-color: var(--bs-info-text-emphasis);
  --bs-alert-bg: var(--bs-info-bg-subtle);
  --bs-alert-border-color: var(--bs-info-border-subtle);
  --bs-alert-link-color: var(--bs-info-text-emphasis);
}

.alert-warning {
  --bs-alert-color: var(--bs-warning-text-emphasis);
  --bs-alert-bg: var(--bs-warning-bg-subtle);
  --bs-alert-border-color: var(--bs-warning-border-subtle);
  --bs-alert-link-color: var(--bs-warning-text-emphasis);
}

.alert-danger {
  --bs-alert-color: var(--bs-danger-text-emphasis);
  --bs-alert-bg: var(--bs-danger-bg-subtle);
  --bs-alert-border-color: var(--bs-danger-border-subtle);
  --bs-alert-link-color: var(--bs-danger-text-emphasis);
}

.alert-light {
  --bs-alert-color: var(--bs-light-text-emphasis);
  --bs-alert-bg: var(--bs-light-bg-subtle);
  --bs-alert-border-color: var(--bs-light-border-subtle);
  --bs-alert-link-color: var(--bs-light-text-emphasis);
}

.alert-dark {
  --bs-alert-color: var(--bs-dark-text-emphasis);
  --bs-alert-bg: var(--bs-dark-bg-subtle);
  --bs-alert-border-color: var(--bs-dark-border-subtle);
  --bs-alert-link-color: var(--bs-dark-text-emphasis);
}

@keyframes progress-bar-stripes {
  0% {
    background-position-x: 1rem;
  }
}
.progress,
.progress-stacked {
  --bs-progress-height: 1rem;
  --bs-progress-font-size: 0.703125rem;
  --bs-progress-bg: var(--bs-secondary-bg);
  --bs-progress-border-radius: var(--bs-border-radius);
  --bs-progress-box-shadow: var(--bs-box-shadow-inset);
  --bs-progress-bar-color: #fff;
  --bs-progress-bar-bg: #0f6cbf;
  --bs-progress-bar-transition: width 0.6s ease;
  display: flex;
  height: var(--bs-progress-height);
  overflow: hidden;
  font-size: var(--bs-progress-font-size);
  background-color: var(--bs-progress-bg);
  border-radius: var(--bs-progress-border-radius);
}

.progress-bar {
  display: flex;
  flex-direction: column;
  justify-content: center;
  overflow: hidden;
  color: var(--bs-progress-bar-color);
  text-align: center;
  white-space: nowrap;
  background-color: var(--bs-progress-bar-bg);
  transition: var(--bs-progress-bar-transition);
}
@media (prefers-reduced-motion: reduce) {
  .progress-bar {
    transition: none;
  }
}

.progress-bar-striped {
  background-image: linear-gradient(45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-size: var(--bs-progress-height) var(--bs-progress-height);
}

.progress-stacked > .progress {
  overflow: visible;
}

.progress-stacked > .progress > .progress-bar {
  width: 100%;
}

.progress-bar-animated {
  animation: 1s linear infinite progress-bar-stripes;
}
@media (prefers-reduced-motion: reduce) {
  .progress-bar-animated {
    animation: none;
  }
}

.list-group {
  --bs-list-group-color: var(--bs-body-color);
  --bs-list-group-bg: var(--bs-body-bg);
  --bs-list-group-border-color: var(--bs-border-color);
  --bs-list-group-border-width: var(--bs-border-width);
  --bs-list-group-border-radius: var(--bs-border-radius);
  --bs-list-group-item-padding-x: 1rem;
  --bs-list-group-item-padding-y: 0.5rem;
  --bs-list-group-action-color: var(--bs-secondary-color);
  --bs-list-group-action-hover-color: var(--bs-emphasis-color);
  --bs-list-group-action-hover-bg: var(--bs-tertiary-bg);
  --bs-list-group-action-active-color: var(--bs-body-color);
  --bs-list-group-action-active-bg: var(--bs-secondary-bg);
  --bs-list-group-disabled-color: var(--bs-secondary-color);
  --bs-list-group-disabled-bg: var(--bs-body-bg);
  --bs-list-group-active-color: #fff;
  --bs-list-group-active-bg: #0f6cbf;
  --bs-list-group-active-border-color: #0f6cbf;
  display: flex;
  flex-direction: column;
  padding-left: 0;
  margin-bottom: 0;
  border-radius: var(--bs-list-group-border-radius);
}

.list-group-numbered {
  list-style-type: none;
  counter-reset: section;
}
.list-group-numbered > .list-group-item::before {
  content: counters(section, ".") ". ";
  counter-increment: section;
}

.list-group-item-action {
  width: 100%;
  color: var(--bs-list-group-action-color);
  text-align: inherit;
}
.list-group-item-action:hover, .list-group-item-action:focus {
  z-index: 1;
  color: var(--bs-list-group-action-hover-color);
  text-decoration: none;
  background-color: var(--bs-list-group-action-hover-bg);
}
.list-group-item-action:active {
  color: var(--bs-list-group-action-active-color);
  background-color: var(--bs-list-group-action-active-bg);
}

.list-group-item {
  position: relative;
  display: block;
  padding: var(--bs-list-group-item-padding-y) var(--bs-list-group-item-padding-x);
  color: var(--bs-list-group-color);
  background-color: var(--bs-list-group-bg);
  border: var(--bs-list-group-border-width) solid var(--bs-list-group-border-color);
}
.list-group-item:first-child {
  border-top-left-radius: inherit;
  border-top-right-radius: inherit;
}
.list-group-item:last-child {
  border-bottom-right-radius: inherit;
  border-bottom-left-radius: inherit;
}
.list-group-item.disabled, .list-group-item:disabled {
  color: var(--bs-list-group-disabled-color);
  pointer-events: none;
  background-color: var(--bs-list-group-disabled-bg);
}
.list-group-item.active {
  z-index: 2;
  color: var(--bs-list-group-active-color);
  background-color: var(--bs-list-group-active-bg);
  border-color: var(--bs-list-group-active-border-color);
}
.list-group-item + .list-group-item {
  border-top-width: 0;
}
.list-group-item + .list-group-item.active {
  margin-top: calc(-1 * var(--bs-list-group-border-width));
  border-top-width: var(--bs-list-group-border-width);
}

.list-group-horizontal {
  flex-direction: row;
}
.list-group-horizontal > .list-group-item:first-child:not(:last-child) {
  border-bottom-left-radius: var(--bs-list-group-border-radius);
  border-top-right-radius: 0;
}
.list-group-horizontal > .list-group-item:last-child:not(:first-child) {
  border-top-right-radius: var(--bs-list-group-border-radius);
  border-bottom-left-radius: 0;
}
.list-group-horizontal > .list-group-item.active {
  margin-top: 0;
}
.list-group-horizontal > .list-group-item + .list-group-item {
  border-top-width: var(--bs-list-group-border-width);
  border-left-width: 0;
}
.list-group-horizontal > .list-group-item + .list-group-item.active {
  margin-left: calc(-1 * var(--bs-list-group-border-width));
  border-left-width: var(--bs-list-group-border-width);
}

@media (min-width: 576px) {
  .list-group-horizontal-sm {
    flex-direction: row;
  }
  .list-group-horizontal-sm > .list-group-item:first-child:not(:last-child) {
    border-bottom-left-radius: var(--bs-list-group-border-radius);
    border-top-right-radius: 0;
  }
  .list-group-horizontal-sm > .list-group-item:last-child:not(:first-child) {
    border-top-right-radius: var(--bs-list-group-border-radius);
    border-bottom-left-radius: 0;
  }
  .list-group-horizontal-sm > .list-group-item.active {
    margin-top: 0;
  }
  .list-group-horizontal-sm > .list-group-item + .list-group-item {
    border-top-width: var(--bs-list-group-border-width);
    border-left-width: 0;
  }
  .list-group-horizontal-sm > .list-group-item + .list-group-item.active {
    margin-left: calc(-1 * var(--bs-list-group-border-width));
    border-left-width: var(--bs-list-group-border-width);
  }
}
@media (min-width: 768px) {
  .list-group-horizontal-md {
    flex-direction: row;
  }
  .list-group-horizontal-md > .list-group-item:first-child:not(:last-child) {
    border-bottom-left-radius: var(--bs-list-group-border-radius);
    border-top-right-radius: 0;
  }
  .list-group-horizontal-md > .list-group-item:last-child:not(:first-child) {
    border-top-right-radius: var(--bs-list-group-border-radius);
    border-bottom-left-radius: 0;
  }
  .list-group-horizontal-md > .list-group-item.active {
    margin-top: 0;
  }
  .list-group-horizontal-md > .list-group-item + .list-group-item {
    border-top-width: var(--bs-list-group-border-width);
    border-left-width: 0;
  }
  .list-group-horizontal-md > .list-group-item + .list-group-item.active {
    margin-left: calc(-1 * var(--bs-list-group-border-width));
    border-left-width: var(--bs-list-group-border-width);
  }
}
@media (min-width: 992px) {
  .list-group-horizontal-lg {
    flex-direction: row;
  }
  .list-group-horizontal-lg > .list-group-item:first-child:not(:last-child) {
    border-bottom-left-radius: var(--bs-list-group-border-radius);
    border-top-right-radius: 0;
  }
  .list-group-horizontal-lg > .list-group-item:last-child:not(:first-child) {
    border-top-right-radius: var(--bs-list-group-border-radius);
    border-bottom-left-radius: 0;
  }
  .list-group-horizontal-lg > .list-group-item.active {
    margin-top: 0;
  }
  .list-group-horizontal-lg > .list-group-item + .list-group-item {
    border-top-width: var(--bs-list-group-border-width);
    border-left-width: 0;
  }
  .list-group-horizontal-lg > .list-group-item + .list-group-item.active {
    margin-left: calc(-1 * var(--bs-list-group-border-width));
    border-left-width: var(--bs-list-group-border-width);
  }
}
@media (min-width: 1200px) {
  .list-group-horizontal-xl {
    flex-direction: row;
  }
  .list-group-horizontal-xl > .list-group-item:first-child:not(:last-child) {
    border-bottom-left-radius: var(--bs-list-group-border-radius);
    border-top-right-radius: 0;
  }
  .list-group-horizontal-xl > .list-group-item:last-child:not(:first-child) {
    border-top-right-radius: var(--bs-list-group-border-radius);
    border-bottom-left-radius: 0;
  }
  .list-group-horizontal-xl > .list-group-item.active {
    margin-top: 0;
  }
  .list-group-horizontal-xl > .list-group-item + .list-group-item {
    border-top-width: var(--bs-list-group-border-width);
    border-left-width: 0;
  }
  .list-group-horizontal-xl > .list-group-item + .list-group-item.active {
    margin-left: calc(-1 * var(--bs-list-group-border-width));
    border-left-width: var(--bs-list-group-border-width);
  }
}
@media (min-width: 1400px) {
  .list-group-horizontal-xxl {
    flex-direction: row;
  }
  .list-group-horizontal-xxl > .list-group-item:first-child:not(:last-child) {
    border-bottom-left-radius: var(--bs-list-group-border-radius);
    border-top-right-radius: 0;
  }
  .list-group-horizontal-xxl > .list-group-item:last-child:not(:first-child) {
    border-top-right-radius: var(--bs-list-group-border-radius);
    border-bottom-left-radius: 0;
  }
  .list-group-horizontal-xxl > .list-group-item.active {
    margin-top: 0;
  }
  .list-group-horizontal-xxl > .list-group-item + .list-group-item {
    border-top-width: var(--bs-list-group-border-width);
    border-left-width: 0;
  }
  .list-group-horizontal-xxl > .list-group-item + .list-group-item.active {
    margin-left: calc(-1 * var(--bs-list-group-border-width));
    border-left-width: var(--bs-list-group-border-width);
  }
}
.list-group-flush {
  border-radius: 0;
}
.list-group-flush > .list-group-item {
  border-width: 0 0 var(--bs-list-group-border-width);
}
.list-group-flush > .list-group-item:last-child {
  border-bottom-width: 0;
}

.list-group-item-primary {
  --bs-list-group-color: var(--bs-primary-text-emphasis);
  --bs-list-group-bg: var(--bs-primary-bg-subtle);
  --bs-list-group-border-color: var(--bs-primary-border-subtle);
  --bs-list-group-action-hover-color: var(--bs-emphasis-color);
  --bs-list-group-action-hover-bg: var(--bs-primary-border-subtle);
  --bs-list-group-action-active-color: var(--bs-emphasis-color);
  --bs-list-group-action-active-bg: var(--bs-primary-border-subtle);
  --bs-list-group-active-color: var(--bs-primary-bg-subtle);
  --bs-list-group-active-bg: var(--bs-primary-text-emphasis);
  --bs-list-group-active-border-color: var(--bs-primary-text-emphasis);
}

.list-group-item-secondary {
  --bs-list-group-color: var(--bs-secondary-text-emphasis);
  --bs-list-group-bg: var(--bs-secondary-bg-subtle);
  --bs-list-group-border-color: var(--bs-secondary-border-subtle);
  --bs-list-group-action-hover-color: var(--bs-emphasis-color);
  --bs-list-group-action-hover-bg: var(--bs-secondary-border-subtle);
  --bs-list-group-action-active-color: var(--bs-emphasis-color);
  --bs-list-group-action-active-bg: var(--bs-secondary-border-subtle);
  --bs-list-group-active-color: var(--bs-secondary-bg-subtle);
  --bs-list-group-active-bg: var(--bs-secondary-text-emphasis);
  --bs-list-group-active-border-color: var(--bs-secondary-text-emphasis);
}

.list-group-item-success {
  --bs-list-group-color: var(--bs-success-text-emphasis);
  --bs-list-group-bg: var(--bs-success-bg-subtle);
  --bs-list-group-border-color: var(--bs-success-border-subtle);
  --bs-list-group-action-hover-color: var(--bs-emphasis-color);
  --bs-list-group-action-hover-bg: var(--bs-success-border-subtle);
  --bs-list-group-action-active-color: var(--bs-emphasis-color);
  --bs-list-group-action-active-bg: var(--bs-success-border-subtle);
  --bs-list-group-active-color: var(--bs-success-bg-subtle);
  --bs-list-group-active-bg: var(--bs-success-text-emphasis);
  --bs-list-group-active-border-color: var(--bs-success-text-emphasis);
}

.list-group-item-info {
  --bs-list-group-color: var(--bs-info-text-emphasis);
  --bs-list-group-bg: var(--bs-info-bg-subtle);
  --bs-list-group-border-color: var(--bs-info-border-subtle);
  --bs-list-group-action-hover-color: var(--bs-emphasis-color);
  --bs-list-group-action-hover-bg: var(--bs-info-border-subtle);
  --bs-list-group-action-active-color: var(--bs-emphasis-color);
  --bs-list-group-action-active-bg: var(--bs-info-border-subtle);
  --bs-list-group-active-color: var(--bs-info-bg-subtle);
  --bs-list-group-active-bg: var(--bs-info-text-emphasis);
  --bs-list-group-active-border-color: var(--bs-info-text-emphasis);
}

.list-group-item-warning {
  --bs-list-group-color: var(--bs-warning-text-emphasis);
  --bs-list-group-bg: var(--bs-warning-bg-subtle);
  --bs-list-group-border-color: var(--bs-warning-border-subtle);
  --bs-list-group-action-hover-color: var(--bs-emphasis-color);
  --bs-list-group-action-hover-bg: var(--bs-warning-border-subtle);
  --bs-list-group-action-active-color: var(--bs-emphasis-color);
  --bs-list-group-action-active-bg: var(--bs-warning-border-subtle);
  --bs-list-group-active-color: var(--bs-warning-bg-subtle);
  --bs-list-group-active-bg: var(--bs-warning-text-emphasis);
  --bs-list-group-active-border-color: var(--bs-warning-text-emphasis);
}

.list-group-item-danger {
  --bs-list-group-color: var(--bs-danger-text-emphasis);
  --bs-list-group-bg: var(--bs-danger-bg-subtle);
  --bs-list-group-border-color: var(--bs-danger-border-subtle);
  --bs-list-group-action-hover-color: var(--bs-emphasis-color);
  --bs-list-group-action-hover-bg: var(--bs-danger-border-subtle);
  --bs-list-group-action-active-color: var(--bs-emphasis-color);
  --bs-list-group-action-active-bg: var(--bs-danger-border-subtle);
  --bs-list-group-active-color: var(--bs-danger-bg-subtle);
  --bs-list-group-active-bg: var(--bs-danger-text-emphasis);
  --bs-list-group-active-border-color: var(--bs-danger-text-emphasis);
}

.list-group-item-light {
  --bs-list-group-color: var(--bs-light-text-emphasis);
  --bs-list-group-bg: var(--bs-light-bg-subtle);
  --bs-list-group-border-color: var(--bs-light-border-subtle);
  --bs-list-group-action-hover-color: var(--bs-emphasis-color);
  --bs-list-group-action-hover-bg: var(--bs-light-border-subtle);
  --bs-list-group-action-active-color: var(--bs-emphasis-color);
  --bs-list-group-action-active-bg: var(--bs-light-border-subtle);
  --bs-list-group-active-color: var(--bs-light-bg-subtle);
  --bs-list-group-active-bg: var(--bs-light-text-emphasis);
  --bs-list-group-active-border-color: var(--bs-light-text-emphasis);
}

.list-group-item-dark {
  --bs-list-group-color: var(--bs-dark-text-emphasis);
  --bs-list-group-bg: var(--bs-dark-bg-subtle);
  --bs-list-group-border-color: var(--bs-dark-border-subtle);
  --bs-list-group-action-hover-color: var(--bs-emphasis-color);
  --bs-list-group-action-hover-bg: var(--bs-dark-border-subtle);
  --bs-list-group-action-active-color: var(--bs-emphasis-color);
  --bs-list-group-action-active-bg: var(--bs-dark-border-subtle);
  --bs-list-group-active-color: var(--bs-dark-bg-subtle);
  --bs-list-group-active-bg: var(--bs-dark-text-emphasis);
  --bs-list-group-active-border-color: var(--bs-dark-text-emphasis);
}

.btn-close {
  --bs-btn-close-color: #000;
  --bs-btn-close-bg: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='%23000'%3e%3cpath d='M.293.293a1 1 0 0 1 1.414 0L8 6.586 14.293.293a1 1 0 1 1 1.414 1.414L9.414 8l6.293 6.293a1 1 0 0 1-1.414 1.414L8 9.414l-6.293 6.293a1 1 0 0 1-1.414-1.414L6.586 8 .293 1.707a1 1 0 0 1 0-1.414z'/%3e%3c/svg%3e");
  --bs-btn-close-opacity: 0.5;
  --bs-btn-close-hover-opacity: 0.75;
  --bs-btn-close-focus-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
  --bs-btn-close-focus-opacity: 1;
  --bs-btn-close-disabled-opacity: 0.25;
  --bs-btn-close-white-filter: invert(1) grayscale(100%) brightness(200%);
  box-sizing: content-box;
  width: 1em;
  height: 1em;
  padding: 0.25em 0.25em;
  color: var(--bs-btn-close-color);
  background: transparent var(--bs-btn-close-bg) center/1em auto no-repeat;
  border: 0;
  border-radius: 0.5rem;
  opacity: var(--bs-btn-close-opacity);
}
.btn-close:hover {
  color: var(--bs-btn-close-color);
  text-decoration: none;
  opacity: var(--bs-btn-close-hover-opacity);
}
.btn-close:focus {
  outline: 0;
  box-shadow: var(--bs-btn-close-focus-shadow);
  opacity: var(--bs-btn-close-focus-opacity);
}
.btn-close:disabled, .btn-close.disabled {
  pointer-events: none;
  user-select: none;
  opacity: var(--bs-btn-close-disabled-opacity);
}

.btn-close-white {
  filter: var(--bs-btn-close-white-filter);
}

[data-bs-theme=dark] .btn-close {
  filter: var(--bs-btn-close-white-filter);
}

.toast {
  --bs-toast-zindex: 1090;
  --bs-toast-padding-x: 0.75rem;
  --bs-toast-padding-y: 0.5rem;
  --bs-toast-spacing: 1.5rem;
  --bs-toast-max-width: 350px;
  --bs-toast-font-size: 0.875rem;
  --bs-toast-color: ;
  --bs-toast-bg: rgba(var(--bs-body-bg-rgb), 0.85);
  --bs-toast-border-width: var(--bs-border-width);
  --bs-toast-border-color: var(--bs-border-color-translucent);
  --bs-toast-border-radius: var(--bs-border-radius);
  --bs-toast-box-shadow: var(--bs-box-shadow);
  --bs-toast-header-color: var(--bs-secondary-color);
  --bs-toast-header-bg: rgba(var(--bs-body-bg-rgb), 0.85);
  --bs-toast-header-border-color: var(--bs-border-color-translucent);
  width: var(--bs-toast-max-width);
  max-width: 100%;
  font-size: var(--bs-toast-font-size);
  color: var(--bs-toast-color);
  pointer-events: auto;
  background-color: var(--bs-toast-bg);
  background-clip: padding-box;
  border: var(--bs-toast-border-width) solid var(--bs-toast-border-color);
  box-shadow: var(--bs-toast-box-shadow);
  border-radius: var(--bs-toast-border-radius);
}
.toast.showing {
  opacity: 0;
}
.toast:not(.show) {
  display: none;
}

.toast-container {
  --bs-toast-zindex: 1090;
  position: absolute;
  z-index: var(--bs-toast-zindex);
  width: max-content;
  max-width: 100%;
  pointer-events: none;
}
.toast-container > :not(:last-child) {
  margin-bottom: var(--bs-toast-spacing);
}

.toast-header {
  display: flex;
  align-items: center;
  padding: var(--bs-toast-padding-y) var(--bs-toast-padding-x);
  color: var(--bs-toast-header-color);
  background-color: var(--bs-toast-header-bg);
  background-clip: padding-box;
  border-bottom: var(--bs-toast-border-width) solid var(--bs-toast-header-border-color);
  border-top-left-radius: calc(var(--bs-toast-border-radius) - var(--bs-toast-border-width));
  border-top-right-radius: calc(var(--bs-toast-border-radius) - var(--bs-toast-border-width));
}
.toast-header .btn-close {
  margin-right: calc(-0.5 * var(--bs-toast-padding-x));
  margin-left: var(--bs-toast-padding-x);
}

.toast-body {
  padding: var(--bs-toast-padding-x);
  word-wrap: break-word;
}

.modal {
  --bs-modal-zindex: 1055;
  --bs-modal-width: 500px;
  --bs-modal-padding: 1rem;
  --bs-modal-margin: 0.5rem;
  --bs-modal-color: ;
  --bs-modal-bg: var(--bs-body-bg);
  --bs-modal-border-color: var(--bs-border-color-translucent);
  --bs-modal-border-width: var(--bs-border-width);
  --bs-modal-border-radius: var(--bs-border-radius-lg);
  --bs-modal-box-shadow: var(--bs-box-shadow-sm);
  --bs-modal-inner-border-radius: calc(var(--bs-border-radius-lg) - (var(--bs-border-width)));
  --bs-modal-header-padding-x: 1rem;
  --bs-modal-header-padding-y: 1rem;
  --bs-modal-header-padding: 1rem 1rem;
  --bs-modal-header-border-color: var(--bs-border-color);
  --bs-modal-header-border-width: var(--bs-border-width);
  --bs-modal-title-line-height: 1.5;
  --bs-modal-footer-gap: 0.5rem;
  --bs-modal-footer-bg: ;
  --bs-modal-footer-border-color: var(--bs-border-color);
  --bs-modal-footer-border-width: var(--bs-border-width);
  position: fixed;
  top: 0;
  left: 0;
  z-index: var(--bs-modal-zindex);
  display: none;
  width: 100%;
  height: 100%;
  overflow-x: hidden;
  overflow-y: auto;
  outline: 0;
}

.modal-dialog {
  position: relative;
  width: auto;
  margin: var(--bs-modal-margin);
  pointer-events: none;
}
.modal.fade .modal-dialog {
  transition: transform 0.3s ease-out;
  transform: translate(0, -50px);
}
@media (prefers-reduced-motion: reduce) {
  .modal.fade .modal-dialog {
    transition: none;
  }
}
.modal.show .modal-dialog {
  transform: none;
}
.modal.modal-static .modal-dialog {
  transform: scale(1.02);
}

.modal-dialog-scrollable {
  height: calc(100% - var(--bs-modal-margin) * 2);
}
.modal-dialog-scrollable .modal-content {
  max-height: 100%;
  overflow: hidden;
}
.modal-dialog-scrollable .modal-body {
  overflow-y: auto;
}

.modal-dialog-centered {
  display: flex;
  align-items: center;
  min-height: calc(100% - var(--bs-modal-margin) * 2);
}

.modal-content {
  position: relative;
  display: flex;
  flex-direction: column;
  width: 100%;
  color: var(--bs-modal-color);
  pointer-events: auto;
  background-color: var(--bs-modal-bg);
  background-clip: padding-box;
  border: var(--bs-modal-border-width) solid var(--bs-modal-border-color);
  border-radius: var(--bs-modal-border-radius);
  outline: 0;
}

.modal-backdrop {
  --bs-backdrop-zindex: 1050;
  --bs-backdrop-bg: #000;
  --bs-backdrop-opacity: 0.5;
  position: fixed;
  top: 0;
  left: 0;
  z-index: var(--bs-backdrop-zindex);
  width: 100vw;
  height: 100vh;
  background-color: var(--bs-backdrop-bg);
}
.modal-backdrop.fade {
  opacity: 0;
}
.modal-backdrop.show {
  opacity: var(--bs-backdrop-opacity);
}

.modal-header {
  display: flex;
  flex-shrink: 0;
  align-items: center;
  padding: var(--bs-modal-header-padding);
  border-bottom: var(--bs-modal-header-border-width) solid var(--bs-modal-header-border-color);
  border-top-left-radius: var(--bs-modal-inner-border-radius);
  border-top-right-radius: var(--bs-modal-inner-border-radius);
}
.modal-header .btn-close {
  padding: calc(var(--bs-modal-header-padding-y) * 0.5) calc(var(--bs-modal-header-padding-x) * 0.5);
  margin: calc(-0.5 * var(--bs-modal-header-padding-y)) calc(-0.5 * var(--bs-modal-header-padding-x)) calc(-0.5 * var(--bs-modal-header-padding-y)) auto;
}

.modal-title {
  margin-bottom: 0;
  line-height: var(--bs-modal-title-line-height);
}

.modal-body {
  position: relative;
  flex: 1 1 auto;
  padding: var(--bs-modal-padding);
}

.modal-footer {
  display: flex;
  flex-shrink: 0;
  flex-wrap: wrap;
  align-items: center;
  justify-content: flex-end;
  padding: calc(var(--bs-modal-padding) - var(--bs-modal-footer-gap) * 0.5);
  background-color: var(--bs-modal-footer-bg);
  border-top: var(--bs-modal-footer-border-width) solid var(--bs-modal-footer-border-color);
  border-bottom-right-radius: var(--bs-modal-inner-border-radius);
  border-bottom-left-radius: var(--bs-modal-inner-border-radius);
}
.modal-footer > * {
  margin: calc(var(--bs-modal-footer-gap) * 0.5);
}

@media (min-width: 576px) {
  .modal {
    --bs-modal-margin: 1.75rem;
    --bs-modal-box-shadow: var(--bs-box-shadow);
  }
  .modal-dialog {
    max-width: var(--bs-modal-width);
    margin-right: auto;
    margin-left: auto;
  }
  .modal-sm {
    --bs-modal-width: 300px;
  }
}
@media (min-width: 992px) {
  .modal-lg,
  .modal-xl {
    --bs-modal-width: 800px;
  }
}
@media (min-width: 1200px) {
  .modal-xl {
    --bs-modal-width: 1140px;
  }
}
.modal-fullscreen {
  width: 100vw;
  max-width: none;
  height: 100%;
  margin: 0;
}
.modal-fullscreen .modal-content {
  height: 100%;
  border: 0;
  border-radius: 0;
}
.modal-fullscreen .modal-header,
.modal-fullscreen .modal-footer {
  border-radius: 0;
}
.modal-fullscreen .modal-body {
  overflow-y: auto;
}

@media (max-width: 575.98px) {
  .modal-fullscreen-sm-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }
  .modal-fullscreen-sm-down .modal-content {
    height: 100%;
    border: 0;
    border-radius: 0;
  }
  .modal-fullscreen-sm-down .modal-header,
  .modal-fullscreen-sm-down .modal-footer {
    border-radius: 0;
  }
  .modal-fullscreen-sm-down .modal-body {
    overflow-y: auto;
  }
}
@media (max-width: 767.98px) {
  .modal-fullscreen-md-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }
  .modal-fullscreen-md-down .modal-content {
    height: 100%;
    border: 0;
    border-radius: 0;
  }
  .modal-fullscreen-md-down .modal-header,
  .modal-fullscreen-md-down .modal-footer {
    border-radius: 0;
  }
  .modal-fullscreen-md-down .modal-body {
    overflow-y: auto;
  }
}
@media (max-width: 991.98px) {
  .modal-fullscreen-lg-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }
  .modal-fullscreen-lg-down .modal-content {
    height: 100%;
    border: 0;
    border-radius: 0;
  }
  .modal-fullscreen-lg-down .modal-header,
  .modal-fullscreen-lg-down .modal-footer {
    border-radius: 0;
  }
  .modal-fullscreen-lg-down .modal-body {
    overflow-y: auto;
  }
}
@media (max-width: 1199.98px) {
  .modal-fullscreen-xl-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }
  .modal-fullscreen-xl-down .modal-content {
    height: 100%;
    border: 0;
    border-radius: 0;
  }
  .modal-fullscreen-xl-down .modal-header,
  .modal-fullscreen-xl-down .modal-footer {
    border-radius: 0;
  }
  .modal-fullscreen-xl-down .modal-body {
    overflow-y: auto;
  }
}
@media (max-width: 1399.98px) {
  .modal-fullscreen-xxl-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }
  .modal-fullscreen-xxl-down .modal-content {
    height: 100%;
    border: 0;
    border-radius: 0;
  }
  .modal-fullscreen-xxl-down .modal-header,
  .modal-fullscreen-xxl-down .modal-footer {
    border-radius: 0;
  }
  .modal-fullscreen-xxl-down .modal-body {
    overflow-y: auto;
  }
}
.tooltip {
  --bs-tooltip-zindex: 1080;
  --bs-tooltip-max-width: 200px;
  --bs-tooltip-padding-x: 0.5rem;
  --bs-tooltip-padding-y: 0.25rem;
  --bs-tooltip-margin: ;
  --bs-tooltip-font-size: 0.8203125rem;
  --bs-tooltip-color: var(--bs-body-bg);
  --bs-tooltip-bg: var(--bs-emphasis-color);
  --bs-tooltip-border-radius: var(--bs-border-radius);
  --bs-tooltip-opacity: 0.9;
  --bs-tooltip-arrow-width: 0.8rem;
  --bs-tooltip-arrow-height: 0.4rem;
  z-index: var(--bs-tooltip-zindex);
  display: block;
  margin: var(--bs-tooltip-margin);
  font-family: var(--bs-font-sans-serif);
  font-style: normal;
  font-weight: 400;
  line-height: 1.5;
  text-align: left;
  text-align: start;
  text-decoration: none;
  text-shadow: none;
  text-transform: none;
  letter-spacing: normal;
  word-break: normal;
  white-space: normal;
  word-spacing: normal;
  line-break: auto;
  font-size: var(--bs-tooltip-font-size);
  word-wrap: break-word;
  opacity: 0;
}
.tooltip.show {
  opacity: var(--bs-tooltip-opacity);
}
.tooltip .tooltip-arrow {
  display: block;
  width: var(--bs-tooltip-arrow-width);
  height: var(--bs-tooltip-arrow-height);
}
.tooltip .tooltip-arrow::before {
  position: absolute;
  content: "";
  border-color: transparent;
  border-style: solid;
}

.bs-tooltip-top .tooltip-arrow, .bs-tooltip-auto[data-popper-placement^=top] .tooltip-arrow {
  bottom: calc(-1 * var(--bs-tooltip-arrow-height));
}
.bs-tooltip-top .tooltip-arrow::before, .bs-tooltip-auto[data-popper-placement^=top] .tooltip-arrow::before {
  top: -1px;
  border-width: var(--bs-tooltip-arrow-height) calc(var(--bs-tooltip-arrow-width) * 0.5) 0;
  border-top-color: var(--bs-tooltip-bg);
}

/* rtl:begin:ignore */
.bs-tooltip-end .tooltip-arrow, .bs-tooltip-auto[data-popper-placement^=right] .tooltip-arrow {
  left: calc(-1 * var(--bs-tooltip-arrow-height));
  width: var(--bs-tooltip-arrow-height);
  height: var(--bs-tooltip-arrow-width);
}
.bs-tooltip-end .tooltip-arrow::before, .bs-tooltip-auto[data-popper-placement^=right] .tooltip-arrow::before {
  right: -1px;
  border-width: calc(var(--bs-tooltip-arrow-width) * 0.5) var(--bs-tooltip-arrow-height) calc(var(--bs-tooltip-arrow-width) * 0.5) 0;
  border-right-color: var(--bs-tooltip-bg);
}

/* rtl:end:ignore */
.bs-tooltip-bottom .tooltip-arrow, .bs-tooltip-auto[data-popper-placement^=bottom] .tooltip-arrow {
  top: calc(-1 * var(--bs-tooltip-arrow-height));
}
.bs-tooltip-bottom .tooltip-arrow::before, .bs-tooltip-auto[data-popper-placement^=bottom] .tooltip-arrow::before {
  bottom: -1px;
  border-width: 0 calc(var(--bs-tooltip-arrow-width) * 0.5) var(--bs-tooltip-arrow-height);
  border-bottom-color: var(--bs-tooltip-bg);
}

/* rtl:begin:ignore */
.bs-tooltip-start .tooltip-arrow, .bs-tooltip-auto[data-popper-placement^=left] .tooltip-arrow {
  right: calc(-1 * var(--bs-tooltip-arrow-height));
  width: var(--bs-tooltip-arrow-height);
  height: var(--bs-tooltip-arrow-width);
}
.bs-tooltip-start .tooltip-arrow::before, .bs-tooltip-auto[data-popper-placement^=left] .tooltip-arrow::before {
  left: -1px;
  border-width: calc(var(--bs-tooltip-arrow-width) * 0.5) 0 calc(var(--bs-tooltip-arrow-width) * 0.5) var(--bs-tooltip-arrow-height);
  border-left-color: var(--bs-tooltip-bg);
}

/* rtl:end:ignore */
.tooltip-inner {
  max-width: var(--bs-tooltip-max-width);
  padding: var(--bs-tooltip-padding-y) var(--bs-tooltip-padding-x);
  color: var(--bs-tooltip-color);
  text-align: center;
  background-color: var(--bs-tooltip-bg);
  border-radius: var(--bs-tooltip-border-radius);
}

.popover {
  --bs-popover-zindex: 1070;
  --bs-popover-max-width: 300px;
  --bs-popover-font-size: 0.8203125rem;
  --bs-popover-bg: var(--bs-body-bg);
  --bs-popover-border-width: var(--bs-border-width);
  --bs-popover-border-color: var(--bs-border-color-translucent);
  --bs-popover-border-radius: var(--bs-border-radius-lg);
  --bs-popover-inner-border-radius: calc(var(--bs-border-radius-lg) - var(--bs-border-width));
  --bs-popover-box-shadow: var(--bs-box-shadow);
  --bs-popover-header-padding-x: 1rem;
  --bs-popover-header-padding-y: 0.5rem;
  --bs-popover-header-font-size: 0.9375rem;
  --bs-popover-header-color: inherit;
  --bs-popover-header-bg: var(--bs-secondary-bg);
  --bs-popover-body-padding-x: 1rem;
  --bs-popover-body-padding-y: 1rem;
  --bs-popover-body-color: var(--bs-body-color);
  --bs-popover-arrow-width: 1rem;
  --bs-popover-arrow-height: 0.5rem;
  --bs-popover-arrow-border: var(--bs-popover-border-color);
  z-index: var(--bs-popover-zindex);
  display: block;
  max-width: var(--bs-popover-max-width);
  font-family: var(--bs-font-sans-serif);
  font-style: normal;
  font-weight: 400;
  line-height: 1.5;
  text-align: left;
  text-align: start;
  text-decoration: none;
  text-shadow: none;
  text-transform: none;
  letter-spacing: normal;
  word-break: normal;
  white-space: normal;
  word-spacing: normal;
  line-break: auto;
  font-size: var(--bs-popover-font-size);
  word-wrap: break-word;
  background-color: var(--bs-popover-bg);
  background-clip: padding-box;
  border: var(--bs-popover-border-width) solid var(--bs-popover-border-color);
  border-radius: var(--bs-popover-border-radius);
}
.popover .popover-arrow {
  display: block;
  width: var(--bs-popover-arrow-width);
  height: var(--bs-popover-arrow-height);
}
.popover .popover-arrow::before, .popover .popover-arrow::after {
  position: absolute;
  display: block;
  content: "";
  border-color: transparent;
  border-style: solid;
  border-width: 0;
}

.bs-popover-top > .popover-arrow, .bs-popover-auto[data-popper-placement^=top] > .popover-arrow {
  bottom: calc(-1 * (var(--bs-popover-arrow-height)) - var(--bs-popover-border-width));
}
.bs-popover-top > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=top] > .popover-arrow::before, .bs-popover-top > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=top] > .popover-arrow::after {
  border-width: var(--bs-popover-arrow-height) calc(var(--bs-popover-arrow-width) * 0.5) 0;
}
.bs-popover-top > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=top] > .popover-arrow::before {
  bottom: 0;
  border-top-color: var(--bs-popover-arrow-border);
}
.bs-popover-top > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=top] > .popover-arrow::after {
  bottom: var(--bs-popover-border-width);
  border-top-color: var(--bs-popover-bg);
}

/* rtl:begin:ignore */
.bs-popover-end > .popover-arrow, .bs-popover-auto[data-popper-placement^=right] > .popover-arrow {
  left: calc(-1 * (var(--bs-popover-arrow-height)) - var(--bs-popover-border-width));
  width: var(--bs-popover-arrow-height);
  height: var(--bs-popover-arrow-width);
}
.bs-popover-end > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=right] > .popover-arrow::before, .bs-popover-end > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=right] > .popover-arrow::after {
  border-width: calc(var(--bs-popover-arrow-width) * 0.5) var(--bs-popover-arrow-height) calc(var(--bs-popover-arrow-width) * 0.5) 0;
}
.bs-popover-end > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=right] > .popover-arrow::before {
  left: 0;
  border-right-color: var(--bs-popover-arrow-border);
}
.bs-popover-end > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=right] > .popover-arrow::after {
  left: var(--bs-popover-border-width);
  border-right-color: var(--bs-popover-bg);
}

/* rtl:end:ignore */
.bs-popover-bottom > .popover-arrow, .bs-popover-auto[data-popper-placement^=bottom] > .popover-arrow {
  top: calc(-1 * (var(--bs-popover-arrow-height)) - var(--bs-popover-border-width));
}
.bs-popover-bottom > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=bottom] > .popover-arrow::before, .bs-popover-bottom > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=bottom] > .popover-arrow::after {
  border-width: 0 calc(var(--bs-popover-arrow-width) * 0.5) var(--bs-popover-arrow-height);
}
.bs-popover-bottom > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=bottom] > .popover-arrow::before {
  top: 0;
  border-bottom-color: var(--bs-popover-arrow-border);
}
.bs-popover-bottom > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=bottom] > .popover-arrow::after {
  top: var(--bs-popover-border-width);
  border-bottom-color: var(--bs-popover-bg);
}
.bs-popover-bottom .popover-header::before, .bs-popover-auto[data-popper-placement^=bottom] .popover-header::before {
  position: absolute;
  top: 0;
  left: 50%;
  display: block;
  width: var(--bs-popover-arrow-width);
  margin-left: calc(-0.5 * var(--bs-popover-arrow-width));
  content: "";
  border-bottom: var(--bs-popover-border-width) solid var(--bs-popover-header-bg);
}

/* rtl:begin:ignore */
.bs-popover-start > .popover-arrow, .bs-popover-auto[data-popper-placement^=left] > .popover-arrow {
  right: calc(-1 * (var(--bs-popover-arrow-height)) - var(--bs-popover-border-width));
  width: var(--bs-popover-arrow-height);
  height: var(--bs-popover-arrow-width);
}
.bs-popover-start > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=left] > .popover-arrow::before, .bs-popover-start > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=left] > .popover-arrow::after {
  border-width: calc(var(--bs-popover-arrow-width) * 0.5) 0 calc(var(--bs-popover-arrow-width) * 0.5) var(--bs-popover-arrow-height);
}
.bs-popover-start > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=left] > .popover-arrow::before {
  right: 0;
  border-left-color: var(--bs-popover-arrow-border);
}
.bs-popover-start > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=left] > .popover-arrow::after {
  right: var(--bs-popover-border-width);
  border-left-color: var(--bs-popover-bg);
}

/* rtl:end:ignore */
.popover-header {
  padding: var(--bs-popover-header-padding-y) var(--bs-popover-header-padding-x);
  margin-bottom: 0;
  font-size: var(--bs-popover-header-font-size);
  color: var(--bs-popover-header-color);
  background-color: var(--bs-popover-header-bg);
  border-bottom: var(--bs-popover-border-width) solid var(--bs-popover-border-color);
  border-top-left-radius: var(--bs-popover-inner-border-radius);
  border-top-right-radius: var(--bs-popover-inner-border-radius);
}
.popover-header:empty {
  display: none;
}

.popover-body {
  padding: var(--bs-popover-body-padding-y) var(--bs-popover-body-padding-x);
  color: var(--bs-popover-body-color);
}

.carousel {
  position: relative;
}

.carousel.pointer-event {
  touch-action: pan-y;
}

.carousel-inner {
  position: relative;
  width: 100%;
  overflow: hidden;
}
.carousel-inner::after {
  display: block;
  clear: both;
  content: "";
}

.carousel-item {
  position: relative;
  display: none;
  float: left;
  width: 100%;
  margin-right: -100%;
  backface-visibility: hidden;
  transition: transform 0.6s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .carousel-item {
    transition: none;
  }
}

.carousel-item.active,
.carousel-item-next,
.carousel-item-prev {
  display: block;
}

.carousel-item-next:not(.carousel-item-start),
.active.carousel-item-end {
  transform: translateX(100%);
}

.carousel-item-prev:not(.carousel-item-end),
.active.carousel-item-start {
  transform: translateX(-100%);
}

.carousel-fade .carousel-item {
  opacity: 0;
  transition-property: opacity;
  transform: none;
}
.carousel-fade .carousel-item.active,
.carousel-fade .carousel-item-next.carousel-item-start,
.carousel-fade .carousel-item-prev.carousel-item-end {
  z-index: 1;
  opacity: 1;
}
.carousel-fade .active.carousel-item-start,
.carousel-fade .active.carousel-item-end {
  z-index: 0;
  opacity: 0;
  transition: opacity 0s 0.6s;
}
@media (prefers-reduced-motion: reduce) {
  .carousel-fade .active.carousel-item-start,
  .carousel-fade .active.carousel-item-end {
    transition: none;
  }
}

.carousel-control-prev,
.carousel-control-next {
  position: absolute;
  top: 0;
  bottom: 0;
  z-index: 1;
  display: flex;
  align-items: center;
  justify-content: center;
  width: 15%;
  padding: 0;
  color: #fff;
  text-align: center;
  background: none;
  border: 0;
  opacity: 0.5;
  transition: opacity 0.15s ease;
}
@media (prefers-reduced-motion: reduce) {
  .carousel-control-prev,
  .carousel-control-next {
    transition: none;
  }
}
.carousel-control-prev:hover, .carousel-control-prev:focus,
.carousel-control-next:hover,
.carousel-control-next:focus {
  color: #fff;
  text-decoration: none;
  outline: 0;
  opacity: 0.9;
}

.carousel-control-prev {
  left: 0;
}

.carousel-control-next {
  right: 0;
}

.carousel-control-prev-icon,
.carousel-control-next-icon {
  display: inline-block;
  width: 2rem;
  height: 2rem;
  background-repeat: no-repeat;
  background-position: 50%;
  background-size: 100% 100%;
}

.carousel-control-prev-icon {
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='%23fff'%3e%3cpath d='M11.354 1.646a.5.5 0 0 1 0 .708L5.707 8l5.647 5.646a.5.5 0 0 1-.708.708l-6-6a.5.5 0 0 1 0-.708l6-6a.5.5 0 0 1 .708 0z'/%3e%3c/svg%3e") /*rtl:url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='%23fff'%3e%3cpath d='M4.646 1.646a.5.5 0 0 1 .708 0l6 6a.5.5 0 0 1 0 .708l-6 6a.5.5 0 0 1-.708-.708L10.293 8 4.646 2.354a.5.5 0 0 1 0-.708z'/%3e%3c/svg%3e")*/;
}

.carousel-control-next-icon {
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='%23fff'%3e%3cpath d='M4.646 1.646a.5.5 0 0 1 .708 0l6 6a.5.5 0 0 1 0 .708l-6 6a.5.5 0 0 1-.708-.708L10.293 8 4.646 2.354a.5.5 0 0 1 0-.708z'/%3e%3c/svg%3e") /*rtl:url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 16 16' fill='%23fff'%3e%3cpath d='M11.354 1.646a.5.5 0 0 1 0 .708L5.707 8l5.647 5.646a.5.5 0 0 1-.708.708l-6-6a.5.5 0 0 1 0-.708l6-6a.5.5 0 0 1 .708 0z'/%3e%3c/svg%3e")*/;
}

.carousel-indicators {
  position: absolute;
  right: 0;
  bottom: 0;
  left: 0;
  z-index: 2;
  display: flex;
  justify-content: center;
  padding: 0;
  margin-right: 15%;
  margin-bottom: 1rem;
  margin-left: 15%;
}
.carousel-indicators [data-bs-target] {
  box-sizing: content-box;
  flex: 0 1 auto;
  width: 30px;
  height: 3px;
  padding: 0;
  margin-right: 3px;
  margin-left: 3px;
  text-indent: -999px;
  cursor: pointer;
  background-color: #fff;
  background-clip: padding-box;
  border: 0;
  border-top: 10px solid transparent;
  border-bottom: 10px solid transparent;
  opacity: 0.5;
  transition: opacity 0.6s ease;
}
@media (prefers-reduced-motion: reduce) {
  .carousel-indicators [data-bs-target] {
    transition: none;
  }
}
.carousel-indicators .active {
  opacity: 1;
}

.carousel-caption {
  position: absolute;
  right: 15%;
  bottom: 1.25rem;
  left: 15%;
  padding-top: 1.25rem;
  padding-bottom: 1.25rem;
  color: #fff;
  text-align: center;
}

.carousel-dark .carousel-control-prev-icon,
.carousel-dark .carousel-control-next-icon {
  filter: invert(1) grayscale(100);
}
.carousel-dark .carousel-indicators [data-bs-target] {
  background-color: #000;
}
.carousel-dark .carousel-caption {
  color: #000;
}

[data-bs-theme=dark] .carousel .carousel-control-prev-icon,
[data-bs-theme=dark] .carousel .carousel-control-next-icon, [data-bs-theme=dark].carousel .carousel-control-prev-icon,
[data-bs-theme=dark].carousel .carousel-control-next-icon {
  filter: invert(1) grayscale(100);
}
[data-bs-theme=dark] .carousel .carousel-indicators [data-bs-target], [data-bs-theme=dark].carousel .carousel-indicators [data-bs-target] {
  background-color: #000;
}
[data-bs-theme=dark] .carousel .carousel-caption, [data-bs-theme=dark].carousel .carousel-caption {
  color: #000;
}

.spinner-grow,
.spinner-border {
  display: inline-block;
  width: var(--bs-spinner-width);
  height: var(--bs-spinner-height);
  vertical-align: var(--bs-spinner-vertical-align);
  border-radius: 50%;
  animation: var(--bs-spinner-animation-speed) linear infinite var(--bs-spinner-animation-name);
}

@keyframes spinner-border {
  to {
    transform: rotate(360deg) /* rtl:ignore */;
  }
}
.spinner-border {
  --bs-spinner-width: 2rem;
  --bs-spinner-height: 2rem;
  --bs-spinner-vertical-align: -0.125em;
  --bs-spinner-border-width: 0.25em;
  --bs-spinner-animation-speed: 0.75s;
  --bs-spinner-animation-name: spinner-border;
  border: var(--bs-spinner-border-width) solid currentcolor;
  border-right-color: transparent;
}

.spinner-border-sm {
  --bs-spinner-width: 1rem;
  --bs-spinner-height: 1rem;
  --bs-spinner-border-width: 0.2em;
}

@keyframes spinner-grow {
  0% {
    transform: scale(0);
  }
  50% {
    opacity: 1;
    transform: none;
  }
}
.spinner-grow {
  --bs-spinner-width: 2rem;
  --bs-spinner-height: 2rem;
  --bs-spinner-vertical-align: -0.125em;
  --bs-spinner-animation-speed: 0.75s;
  --bs-spinner-animation-name: spinner-grow;
  background-color: currentcolor;
  opacity: 0;
}

.spinner-grow-sm {
  --bs-spinner-width: 1rem;
  --bs-spinner-height: 1rem;
}

@media (prefers-reduced-motion: reduce) {
  .spinner-border,
  .spinner-grow {
    --bs-spinner-animation-speed: 1.5s;
  }
}
.offcanvas, .offcanvas-xxl, .offcanvas-xl, .offcanvas-lg, .offcanvas-md, .offcanvas-sm {
  --bs-offcanvas-zindex: 1045;
  --bs-offcanvas-width: 400px;
  --bs-offcanvas-height: 30vh;
  --bs-offcanvas-padding-x: 1rem;
  --bs-offcanvas-padding-y: 1rem;
  --bs-offcanvas-color: var(--bs-body-color);
  --bs-offcanvas-bg: var(--bs-body-bg);
  --bs-offcanvas-border-width: var(--bs-border-width);
  --bs-offcanvas-border-color: var(--bs-border-color-translucent);
  --bs-offcanvas-box-shadow: var(--bs-box-shadow-sm);
  --bs-offcanvas-transition: transform 0.3s ease-in-out;
  --bs-offcanvas-title-line-height: 1.5;
}

@media (max-width: 575.98px) {
  .offcanvas-sm {
    position: fixed;
    bottom: 0;
    z-index: var(--bs-offcanvas-zindex);
    display: flex;
    flex-direction: column;
    max-width: 100%;
    color: var(--bs-offcanvas-color);
    visibility: hidden;
    background-color: var(--bs-offcanvas-bg);
    background-clip: padding-box;
    outline: 0;
    transition: var(--bs-offcanvas-transition);
  }
}
@media (max-width: 575.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-sm {
    transition: none;
  }
}
@media (max-width: 575.98px) {
  .offcanvas-sm.offcanvas-start {
    top: 0;
    left: 0;
    width: var(--bs-offcanvas-width);
    border-right: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateX(-100%);
  }
}
@media (max-width: 575.98px) {
  .offcanvas-sm.offcanvas-end {
    top: 0;
    right: 0;
    width: var(--bs-offcanvas-width);
    border-left: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateX(100%);
  }
}
@media (max-width: 575.98px) {
  .offcanvas-sm.offcanvas-top {
    top: 0;
    right: 0;
    left: 0;
    height: var(--bs-offcanvas-height);
    max-height: 100%;
    border-bottom: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateY(-100%);
  }
}
@media (max-width: 575.98px) {
  .offcanvas-sm.offcanvas-bottom {
    right: 0;
    left: 0;
    height: var(--bs-offcanvas-height);
    max-height: 100%;
    border-top: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateY(100%);
  }
}
@media (max-width: 575.98px) {
  .offcanvas-sm.showing, .offcanvas-sm.show:not(.hiding) {
    transform: none;
  }
}
@media (max-width: 575.98px) {
  .offcanvas-sm.showing, .offcanvas-sm.hiding, .offcanvas-sm.show {
    visibility: visible;
  }
}
@media (min-width: 576px) {
  .offcanvas-sm {
    --bs-offcanvas-height: auto;
    --bs-offcanvas-border-width: 0;
    background-color: transparent !important;
  }
  .offcanvas-sm .offcanvas-header {
    display: none;
  }
  .offcanvas-sm .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
    background-color: transparent !important;
  }
}

@media (max-width: 767.98px) {
  .offcanvas-md {
    position: fixed;
    bottom: 0;
    z-index: var(--bs-offcanvas-zindex);
    display: flex;
    flex-direction: column;
    max-width: 100%;
    color: var(--bs-offcanvas-color);
    visibility: hidden;
    background-color: var(--bs-offcanvas-bg);
    background-clip: padding-box;
    outline: 0;
    transition: var(--bs-offcanvas-transition);
  }
}
@media (max-width: 767.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-md {
    transition: none;
  }
}
@media (max-width: 767.98px) {
  .offcanvas-md.offcanvas-start {
    top: 0;
    left: 0;
    width: var(--bs-offcanvas-width);
    border-right: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateX(-100%);
  }
}
@media (max-width: 767.98px) {
  .offcanvas-md.offcanvas-end {
    top: 0;
    right: 0;
    width: var(--bs-offcanvas-width);
    border-left: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateX(100%);
  }
}
@media (max-width: 767.98px) {
  .offcanvas-md.offcanvas-top {
    top: 0;
    right: 0;
    left: 0;
    height: var(--bs-offcanvas-height);
    max-height: 100%;
    border-bottom: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateY(-100%);
  }
}
@media (max-width: 767.98px) {
  .offcanvas-md.offcanvas-bottom {
    right: 0;
    left: 0;
    height: var(--bs-offcanvas-height);
    max-height: 100%;
    border-top: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateY(100%);
  }
}
@media (max-width: 767.98px) {
  .offcanvas-md.showing, .offcanvas-md.show:not(.hiding) {
    transform: none;
  }
}
@media (max-width: 767.98px) {
  .offcanvas-md.showing, .offcanvas-md.hiding, .offcanvas-md.show {
    visibility: visible;
  }
}
@media (min-width: 768px) {
  .offcanvas-md {
    --bs-offcanvas-height: auto;
    --bs-offcanvas-border-width: 0;
    background-color: transparent !important;
  }
  .offcanvas-md .offcanvas-header {
    display: none;
  }
  .offcanvas-md .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
    background-color: transparent !important;
  }
}

@media (max-width: 991.98px) {
  .offcanvas-lg {
    position: fixed;
    bottom: 0;
    z-index: var(--bs-offcanvas-zindex);
    display: flex;
    flex-direction: column;
    max-width: 100%;
    color: var(--bs-offcanvas-color);
    visibility: hidden;
    background-color: var(--bs-offcanvas-bg);
    background-clip: padding-box;
    outline: 0;
    transition: var(--bs-offcanvas-transition);
  }
}
@media (max-width: 991.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-lg {
    transition: none;
  }
}
@media (max-width: 991.98px) {
  .offcanvas-lg.offcanvas-start {
    top: 0;
    left: 0;
    width: var(--bs-offcanvas-width);
    border-right: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateX(-100%);
  }
}
@media (max-width: 991.98px) {
  .offcanvas-lg.offcanvas-end {
    top: 0;
    right: 0;
    width: var(--bs-offcanvas-width);
    border-left: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateX(100%);
  }
}
@media (max-width: 991.98px) {
  .offcanvas-lg.offcanvas-top {
    top: 0;
    right: 0;
    left: 0;
    height: var(--bs-offcanvas-height);
    max-height: 100%;
    border-bottom: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateY(-100%);
  }
}
@media (max-width: 991.98px) {
  .offcanvas-lg.offcanvas-bottom {
    right: 0;
    left: 0;
    height: var(--bs-offcanvas-height);
    max-height: 100%;
    border-top: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateY(100%);
  }
}
@media (max-width: 991.98px) {
  .offcanvas-lg.showing, .offcanvas-lg.show:not(.hiding) {
    transform: none;
  }
}
@media (max-width: 991.98px) {
  .offcanvas-lg.showing, .offcanvas-lg.hiding, .offcanvas-lg.show {
    visibility: visible;
  }
}
@media (min-width: 992px) {
  .offcanvas-lg {
    --bs-offcanvas-height: auto;
    --bs-offcanvas-border-width: 0;
    background-color: transparent !important;
  }
  .offcanvas-lg .offcanvas-header {
    display: none;
  }
  .offcanvas-lg .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
    background-color: transparent !important;
  }
}

@media (max-width: 1199.98px) {
  .offcanvas-xl {
    position: fixed;
    bottom: 0;
    z-index: var(--bs-offcanvas-zindex);
    display: flex;
    flex-direction: column;
    max-width: 100%;
    color: var(--bs-offcanvas-color);
    visibility: hidden;
    background-color: var(--bs-offcanvas-bg);
    background-clip: padding-box;
    outline: 0;
    transition: var(--bs-offcanvas-transition);
  }
}
@media (max-width: 1199.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-xl {
    transition: none;
  }
}
@media (max-width: 1199.98px) {
  .offcanvas-xl.offcanvas-start {
    top: 0;
    left: 0;
    width: var(--bs-offcanvas-width);
    border-right: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateX(-100%);
  }
}
@media (max-width: 1199.98px) {
  .offcanvas-xl.offcanvas-end {
    top: 0;
    right: 0;
    width: var(--bs-offcanvas-width);
    border-left: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateX(100%);
  }
}
@media (max-width: 1199.98px) {
  .offcanvas-xl.offcanvas-top {
    top: 0;
    right: 0;
    left: 0;
    height: var(--bs-offcanvas-height);
    max-height: 100%;
    border-bottom: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateY(-100%);
  }
}
@media (max-width: 1199.98px) {
  .offcanvas-xl.offcanvas-bottom {
    right: 0;
    left: 0;
    height: var(--bs-offcanvas-height);
    max-height: 100%;
    border-top: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateY(100%);
  }
}
@media (max-width: 1199.98px) {
  .offcanvas-xl.showing, .offcanvas-xl.show:not(.hiding) {
    transform: none;
  }
}
@media (max-width: 1199.98px) {
  .offcanvas-xl.showing, .offcanvas-xl.hiding, .offcanvas-xl.show {
    visibility: visible;
  }
}
@media (min-width: 1200px) {
  .offcanvas-xl {
    --bs-offcanvas-height: auto;
    --bs-offcanvas-border-width: 0;
    background-color: transparent !important;
  }
  .offcanvas-xl .offcanvas-header {
    display: none;
  }
  .offcanvas-xl .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
    background-color: transparent !important;
  }
}

@media (max-width: 1399.98px) {
  .offcanvas-xxl {
    position: fixed;
    bottom: 0;
    z-index: var(--bs-offcanvas-zindex);
    display: flex;
    flex-direction: column;
    max-width: 100%;
    color: var(--bs-offcanvas-color);
    visibility: hidden;
    background-color: var(--bs-offcanvas-bg);
    background-clip: padding-box;
    outline: 0;
    transition: var(--bs-offcanvas-transition);
  }
}
@media (max-width: 1399.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-xxl {
    transition: none;
  }
}
@media (max-width: 1399.98px) {
  .offcanvas-xxl.offcanvas-start {
    top: 0;
    left: 0;
    width: var(--bs-offcanvas-width);
    border-right: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateX(-100%);
  }
}
@media (max-width: 1399.98px) {
  .offcanvas-xxl.offcanvas-end {
    top: 0;
    right: 0;
    width: var(--bs-offcanvas-width);
    border-left: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateX(100%);
  }
}
@media (max-width: 1399.98px) {
  .offcanvas-xxl.offcanvas-top {
    top: 0;
    right: 0;
    left: 0;
    height: var(--bs-offcanvas-height);
    max-height: 100%;
    border-bottom: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateY(-100%);
  }
}
@media (max-width: 1399.98px) {
  .offcanvas-xxl.offcanvas-bottom {
    right: 0;
    left: 0;
    height: var(--bs-offcanvas-height);
    max-height: 100%;
    border-top: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
    transform: translateY(100%);
  }
}
@media (max-width: 1399.98px) {
  .offcanvas-xxl.showing, .offcanvas-xxl.show:not(.hiding) {
    transform: none;
  }
}
@media (max-width: 1399.98px) {
  .offcanvas-xxl.showing, .offcanvas-xxl.hiding, .offcanvas-xxl.show {
    visibility: visible;
  }
}
@media (min-width: 1400px) {
  .offcanvas-xxl {
    --bs-offcanvas-height: auto;
    --bs-offcanvas-border-width: 0;
    background-color: transparent !important;
  }
  .offcanvas-xxl .offcanvas-header {
    display: none;
  }
  .offcanvas-xxl .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
    background-color: transparent !important;
  }
}

.offcanvas {
  position: fixed;
  bottom: 0;
  z-index: var(--bs-offcanvas-zindex);
  display: flex;
  flex-direction: column;
  max-width: 100%;
  color: var(--bs-offcanvas-color);
  visibility: hidden;
  background-color: var(--bs-offcanvas-bg);
  background-clip: padding-box;
  outline: 0;
  transition: var(--bs-offcanvas-transition);
}
@media (prefers-reduced-motion: reduce) {
  .offcanvas {
    transition: none;
  }
}
.offcanvas.offcanvas-start {
  top: 0;
  left: 0;
  width: var(--bs-offcanvas-width);
  border-right: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
  transform: translateX(-100%);
}
.offcanvas.offcanvas-end {
  top: 0;
  right: 0;
  width: var(--bs-offcanvas-width);
  border-left: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
  transform: translateX(100%);
}
.offcanvas.offcanvas-top {
  top: 0;
  right: 0;
  left: 0;
  height: var(--bs-offcanvas-height);
  max-height: 100%;
  border-bottom: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
  transform: translateY(-100%);
}
.offcanvas.offcanvas-bottom {
  right: 0;
  left: 0;
  height: var(--bs-offcanvas-height);
  max-height: 100%;
  border-top: var(--bs-offcanvas-border-width) solid var(--bs-offcanvas-border-color);
  transform: translateY(100%);
}
.offcanvas.showing, .offcanvas.show:not(.hiding) {
  transform: none;
}
.offcanvas.showing, .offcanvas.hiding, .offcanvas.show {
  visibility: visible;
}

.offcanvas-backdrop {
  position: fixed;
  top: 0;
  left: 0;
  z-index: 1040;
  width: 100vw;
  height: 100vh;
  background-color: #000;
}
.offcanvas-backdrop.fade {
  opacity: 0;
}
.offcanvas-backdrop.show {
  opacity: 0.5;
}

.offcanvas-header {
  display: flex;
  align-items: center;
  padding: var(--bs-offcanvas-padding-y) var(--bs-offcanvas-padding-x);
}
.offcanvas-header .btn-close {
  padding: calc(var(--bs-offcanvas-padding-y) * 0.5) calc(var(--bs-offcanvas-padding-x) * 0.5);
  margin: calc(-0.5 * var(--bs-offcanvas-padding-y)) calc(-0.5 * var(--bs-offcanvas-padding-x)) calc(-0.5 * var(--bs-offcanvas-padding-y)) auto;
}

.offcanvas-title {
  margin-bottom: 0;
  line-height: var(--bs-offcanvas-title-line-height);
}

.offcanvas-body {
  flex-grow: 1;
  padding: var(--bs-offcanvas-padding-y) var(--bs-offcanvas-padding-x);
  overflow-y: auto;
}

.placeholder {
  display: inline-block;
  min-height: 1em;
  vertical-align: middle;
  cursor: wait;
  background-color: currentcolor;
  opacity: 0.5;
}
.placeholder.btn::before {
  display: inline-block;
  content: "";
}

.placeholder-xs {
  min-height: 0.6em;
}

.placeholder-sm {
  min-height: 0.8em;
}

.placeholder-lg {
  min-height: 1.2em;
}

.placeholder-glow .placeholder {
  animation: placeholder-glow 2s ease-in-out infinite;
}

@keyframes placeholder-glow {
  50% {
    opacity: 0.2;
  }
}
.placeholder-wave {
  mask-image: linear-gradient(130deg, #000 55%, rgba(0, 0, 0, 0.8) 75%, #000 95%);
  mask-size: 200% 100%;
  animation: placeholder-wave 2s linear infinite;
}

@keyframes placeholder-wave {
  100% {
    mask-position: -200% 0%;
  }
}
.clearfix::after {
  display: block;
  clear: both;
  content: "";
}

.text-bg-primary {
  color: #fff !important;
  background-color: RGBA(var(--bs-primary-rgb), var(--bs-bg-opacity, 1)) !important;
}

.text-bg-secondary {
  color: #000 !important;
  background-color: RGBA(var(--bs-secondary-rgb), var(--bs-bg-opacity, 1)) !important;
}

.text-bg-success {
  color: #fff !important;
  background-color: RGBA(var(--bs-success-rgb), var(--bs-bg-opacity, 1)) !important;
}

.text-bg-info {
  color: #fff !important;
  background-color: RGBA(var(--bs-info-rgb), var(--bs-bg-opacity, 1)) !important;
}

.text-bg-warning {
  color: #000 !important;
  background-color: RGBA(var(--bs-warning-rgb), var(--bs-bg-opacity, 1)) !important;
}

.text-bg-danger {
  color: #fff !important;
  background-color: RGBA(var(--bs-danger-rgb), var(--bs-bg-opacity, 1)) !important;
}

.text-bg-light {
  color: #000 !important;
  background-color: RGBA(var(--bs-light-rgb), var(--bs-bg-opacity, 1)) !important;
}

.text-bg-dark {
  color: #fff !important;
  background-color: RGBA(var(--bs-dark-rgb), var(--bs-bg-opacity, 1)) !important;
}

.link-primary {
  color: RGBA(var(--bs-primary-rgb), var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--bs-primary-rgb), var(--bs-link-underline-opacity, 1)) !important;
}
.link-primary:hover, .link-primary:focus {
  color: RGBA(12, 86, 153, var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(12, 86, 153, var(--bs-link-underline-opacity, 1)) !important;
}

.link-secondary {
  color: RGBA(var(--bs-secondary-rgb), var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--bs-secondary-rgb), var(--bs-link-underline-opacity, 1)) !important;
}
.link-secondary:hover, .link-secondary:focus {
  color: RGBA(216, 221, 225, var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(216, 221, 225, var(--bs-link-underline-opacity, 1)) !important;
}

.link-success {
  color: RGBA(var(--bs-success-rgb), var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--bs-success-rgb), var(--bs-link-underline-opacity, 1)) !important;
}
.link-success:hover, .link-success:focus {
  color: RGBA(42, 98, 40, var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(42, 98, 40, var(--bs-link-underline-opacity, 1)) !important;
}

.link-info {
  color: RGBA(var(--bs-info-rgb), var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--bs-info-rgb), var(--bs-link-underline-opacity, 1)) !important;
}
.link-info:hover, .link-info:focus {
  color: RGBA(0, 103, 120, var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(0, 103, 120, var(--bs-link-underline-opacity, 1)) !important;
}

.link-warning {
  color: RGBA(var(--bs-warning-rgb), var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--bs-warning-rgb), var(--bs-link-underline-opacity, 1)) !important;
}
.link-warning:hover, .link-warning:focus {
  color: RGBA(243, 189, 113, var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(243, 189, 113, var(--bs-link-underline-opacity, 1)) !important;
}

.link-danger {
  color: RGBA(var(--bs-danger-rgb), var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--bs-danger-rgb), var(--bs-link-underline-opacity, 1)) !important;
}
.link-danger:hover, .link-danger:focus {
  color: RGBA(162, 39, 26, var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(162, 39, 26, var(--bs-link-underline-opacity, 1)) !important;
}

.link-light {
  color: RGBA(var(--bs-light-rgb), var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--bs-light-rgb), var(--bs-link-underline-opacity, 1)) !important;
}
.link-light:hover, .link-light:focus {
  color: RGBA(249, 250, 251, var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(249, 250, 251, var(--bs-link-underline-opacity, 1)) !important;
}

.link-dark {
  color: RGBA(var(--bs-dark-rgb), var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--bs-dark-rgb), var(--bs-link-underline-opacity, 1)) !important;
}
.link-dark:hover, .link-dark:focus {
  color: RGBA(23, 26, 30, var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(23, 26, 30, var(--bs-link-underline-opacity, 1)) !important;
}

.link-body-emphasis {
  color: RGBA(var(--bs-emphasis-color-rgb), var(--bs-link-opacity, 1)) !important;
  text-decoration-color: RGBA(var(--bs-emphasis-color-rgb), var(--bs-link-underline-opacity, 1)) !important;
}
.link-body-emphasis:hover, .link-body-emphasis:focus {
  color: RGBA(var(--bs-emphasis-color-rgb), var(--bs-link-opacity, 0.75)) !important;
  text-decoration-color: RGBA(var(--bs-emphasis-color-rgb), var(--bs-link-underline-opacity, 0.75)) !important;
}

.focus-ring:focus {
  outline: 0;
  box-shadow: var(--bs-focus-ring-x, 0) var(--bs-focus-ring-y, 0) var(--bs-focus-ring-blur, 0) var(--bs-focus-ring-width) var(--bs-focus-ring-color);
}

.icon-link {
  display: inline-flex;
  gap: 0.375rem;
  align-items: center;
  text-decoration-color: rgba(var(--bs-link-color-rgb), var(--bs-link-opacity, 0.5));
  text-underline-offset: 0.25em;
  backface-visibility: hidden;
}
.icon-link > .bi {
  flex-shrink: 0;
  width: 1em;
  height: 1em;
  fill: currentcolor;
  transition: 0.2s ease-in-out transform;
}
@media (prefers-reduced-motion: reduce) {
  .icon-link > .bi {
    transition: none;
  }
}

.icon-link-hover:hover > .bi, .icon-link-hover:focus-visible > .bi {
  transform: var(--bs-icon-link-transform, translate3d(0.25em, 0, 0));
}

.ratio {
  position: relative;
  width: 100%;
}
.ratio::before {
  display: block;
  padding-top: var(--bs-aspect-ratio);
  content: "";
}
.ratio > * {
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
}

.ratio-1x1 {
  --bs-aspect-ratio: 100%;
}

.ratio-4x3 {
  --bs-aspect-ratio: 75%;
}

.ratio-16x9 {
  --bs-aspect-ratio: 56.25%;
}

.ratio-21x9 {
  --bs-aspect-ratio: 42.8571428571%;
}

.fixed-top {
  position: fixed;
  top: 0;
  right: 0;
  left: 0;
  z-index: 1030;
}

.fixed-bottom {
  position: fixed;
  right: 0;
  bottom: 0;
  left: 0;
  z-index: 1030;
}

.sticky-top {
  position: sticky;
  top: 0;
  z-index: 1020;
}

.sticky-bottom {
  position: sticky;
  bottom: 0;
  z-index: 1020;
}

@media (min-width: 576px) {
  .sticky-sm-top {
    position: sticky;
    top: 0;
    z-index: 1020;
  }
  .sticky-sm-bottom {
    position: sticky;
    bottom: 0;
    z-index: 1020;
  }
}
@media (min-width: 768px) {
  .sticky-md-top {
    position: sticky;
    top: 0;
    z-index: 1020;
  }
  .sticky-md-bottom {
    position: sticky;
    bottom: 0;
    z-index: 1020;
  }
}
@media (min-width: 992px) {
  .sticky-lg-top {
    position: sticky;
    top: 0;
    z-index: 1020;
  }
  .sticky-lg-bottom {
    position: sticky;
    bottom: 0;
    z-index: 1020;
  }
}
@media (min-width: 1200px) {
  .sticky-xl-top {
    position: sticky;
    top: 0;
    z-index: 1020;
  }
  .sticky-xl-bottom {
    position: sticky;
    bottom: 0;
    z-index: 1020;
  }
}
@media (min-width: 1400px) {
  .sticky-xxl-top {
    position: sticky;
    top: 0;
    z-index: 1020;
  }
  .sticky-xxl-bottom {
    position: sticky;
    bottom: 0;
    z-index: 1020;
  }
}
.hstack {
  display: flex;
  flex-direction: row;
  align-items: center;
  align-self: stretch;
}

.vstack {
  display: flex;
  flex: 1 1 auto;
  flex-direction: column;
  align-self: stretch;
}

.visually-hidden,
.visually-hidden-focusable:not(:focus):not(:focus-within) {
  width: 1px !important;
  height: 1px !important;
  padding: 0 !important;
  margin: -1px !important;
  overflow: hidden !important;
  clip: rect(0, 0, 0, 0) !important;
  white-space: nowrap !important;
  border: 0 !important;
}
.visually-hidden:not(caption),
.visually-hidden-focusable:not(:focus):not(:focus-within):not(caption) {
  position: absolute !important;
}

.stretched-link::after {
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
  z-index: 1;
  content: "";
}

.text-truncate {
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
}

.vr {
  display: inline-block;
  align-self: stretch;
  width: var(--bs-border-width);
  min-height: 1em;
  background-color: currentcolor;
  opacity: 0.15;
}

.align-baseline {
  vertical-align: baseline !important;
}

.align-top, [data-filterregion=value] div:first-of-type {
  vertical-align: top !important;
}

.align-middle, .reportbuilder-table td {
  vertical-align: middle !important;
}

.align-bottom {
  vertical-align: bottom !important;
}

.align-text-bottom {
  vertical-align: text-bottom !important;
}

.align-text-top {
  vertical-align: text-top !important;
}

.float-start {
  float: left !important;
}

.float-end {
  float: right !important;
}

.float-none {
  float: none !important;
}

.object-fit-contain {
  object-fit: contain !important;
}

.object-fit-cover {
  object-fit: cover !important;
}

.object-fit-fill {
  object-fit: fill !important;
}

.object-fit-scale {
  object-fit: scale-down !important;
}

.object-fit-none {
  object-fit: none !important;
}

.opacity-0 {
  opacity: 0 !important;
}

.opacity-25 {
  opacity: 0.25 !important;
}

.opacity-50 {
  opacity: 0.5 !important;
}

.opacity-75 {
  opacity: 0.75 !important;
}

.opacity-100 {
  opacity: 1 !important;
}

.overflow-auto {
  overflow: auto !important;
}

.overflow-hidden {
  overflow: hidden !important;
}

.overflow-visible {
  overflow: visible !important;
}

.overflow-scroll {
  overflow: scroll !important;
}

.overflow-x-auto {
  overflow-x: auto !important;
}

.overflow-x-hidden {
  overflow-x: hidden !important;
}

.overflow-x-visible {
  overflow-x: visible !important;
}

.overflow-x-scroll {
  overflow-x: scroll !important;
}

.overflow-y-auto {
  overflow-y: auto !important;
}

.overflow-y-hidden {
  overflow-y: hidden !important;
}

.overflow-y-visible {
  overflow-y: visible !important;
}

.overflow-y-scroll {
  overflow-y: scroll !important;
}

.d-inline {
  display: inline !important;
}

.d-inline-block {
  display: inline-block !important;
}

.d-block {
  display: block !important;
}

.d-grid {
  display: grid !important;
}

.d-inline-grid {
  display: inline-grid !important;
}

.d-table {
  display: table !important;
}

.d-table-row {
  display: table-row !important;
}

.d-table-cell {
  display: table-cell !important;
}

.d-flex {
  display: flex !important;
}

.d-inline-flex {
  display: inline-flex !important;
}

.d-none {
  display: none !important;
}

.shadow {
  box-shadow: var(--bs-box-shadow) !important;
}

.shadow-sm {
  box-shadow: var(--bs-box-shadow-sm) !important;
}

.shadow-lg {
  box-shadow: var(--bs-box-shadow-lg) !important;
}

.shadow-none {
  box-shadow: none !important;
}

.focus-ring-primary {
  --bs-focus-ring-color: rgba(var(--bs-primary-rgb), var(--bs-focus-ring-opacity));
}

.focus-ring-secondary {
  --bs-focus-ring-color: rgba(var(--bs-secondary-rgb), var(--bs-focus-ring-opacity));
}

.focus-ring-success {
  --bs-focus-ring-color: rgba(var(--bs-success-rgb), var(--bs-focus-ring-opacity));
}

.focus-ring-info {
  --bs-focus-ring-color: rgba(var(--bs-info-rgb), var(--bs-focus-ring-opacity));
}

.focus-ring-warning {
  --bs-focus-ring-color: rgba(var(--bs-warning-rgb), var(--bs-focus-ring-opacity));
}

.focus-ring-danger {
  --bs-focus-ring-color: rgba(var(--bs-danger-rgb), var(--bs-focus-ring-opacity));
}

.focus-ring-light {
  --bs-focus-ring-color: rgba(var(--bs-light-rgb), var(--bs-focus-ring-opacity));
}

.focus-ring-dark {
  --bs-focus-ring-color: rgba(var(--bs-dark-rgb), var(--bs-focus-ring-opacity));
}

.position-static {
  position: static !important;
}

.position-relative {
  position: relative !important;
}

.position-absolute {
  position: absolute !important;
}

.position-fixed {
  position: fixed !important;
}

.position-sticky {
  position: sticky !important;
}

.top-0 {
  top: 0 !important;
}

.top-50 {
  top: 50% !important;
}

.top-100 {
  top: 100% !important;
}

.bottom-0 {
  bottom: 0 !important;
}

.bottom-50 {
  bottom: 50% !important;
}

.bottom-100 {
  bottom: 100% !important;
}

.start-0 {
  left: 0 !important;
}

.start-50 {
  left: 50% !important;
}

.start-100 {
  left: 100% !important;
}

.end-0 {
  right: 0 !important;
}

.end-50 {
  right: 50% !important;
}

.end-100 {
  right: 100% !important;
}

.translate-middle {
  transform: translate(-50%, -50%) !important;
}

.translate-middle-x {
  transform: translateX(-50%) !important;
}

.translate-middle-y {
  transform: translateY(-50%) !important;
}

.border {
  border: var(--bs-border-width) var(--bs-border-style) var(--bs-border-color) !important;
}

.border-0 {
  border: 0 !important;
}

.border-top {
  border-top: var(--bs-border-width) var(--bs-border-style) var(--bs-border-color) !important;
}

.border-top-0 {
  border-top: 0 !important;
}

.border-end {
  border-right: var(--bs-border-width) var(--bs-border-style) var(--bs-border-color) !important;
}

.border-end-0 {
  border-right: 0 !important;
}

.border-bottom {
  border-bottom: var(--bs-border-width) var(--bs-border-style) var(--bs-border-color) !important;
}

.border-bottom-0 {
  border-bottom: 0 !important;
}

.border-start {
  border-left: var(--bs-border-width) var(--bs-border-style) var(--bs-border-color) !important;
}

.border-start-0 {
  border-left: 0 !important;
}

.border-primary {
  --bs-border-opacity: 1;
  border-color: rgba(var(--bs-primary-rgb), var(--bs-border-opacity)) !important;
}

.border-secondary {
  --bs-border-opacity: 1;
  border-color: rgba(var(--bs-secondary-rgb), var(--bs-border-opacity)) !important;
}

.border-success {
  --bs-border-opacity: 1;
  border-color: rgba(var(--bs-success-rgb), var(--bs-border-opacity)) !important;
}

.border-info {
  --bs-border-opacity: 1;
  border-color: rgba(var(--bs-info-rgb), var(--bs-border-opacity)) !important;
}

.border-warning {
  --bs-border-opacity: 1;
  border-color: rgba(var(--bs-warning-rgb), var(--bs-border-opacity)) !important;
}

.border-danger {
  --bs-border-opacity: 1;
  border-color: rgba(var(--bs-danger-rgb), var(--bs-border-opacity)) !important;
}

.border-light {
  --bs-border-opacity: 1;
  border-color: rgba(var(--bs-light-rgb), var(--bs-border-opacity)) !important;
}

.border-dark {
  --bs-border-opacity: 1;
  border-color: rgba(var(--bs-dark-rgb), var(--bs-border-opacity)) !important;
}

.border-black {
  --bs-border-opacity: 1;
  border-color: rgba(var(--bs-black-rgb), var(--bs-border-opacity)) !important;
}

.border-white {
  --bs-border-opacity: 1;
  border-color: rgba(var(--bs-white-rgb), var(--bs-border-opacity)) !important;
}

.border-primary-subtle {
  border-color: var(--bs-primary-border-subtle) !important;
}

.border-secondary-subtle {
  border-color: var(--bs-secondary-border-subtle) !important;
}

.border-success-subtle {
  border-color: var(--bs-success-border-subtle) !important;
}

.border-info-subtle {
  border-color: var(--bs-info-border-subtle) !important;
}

.border-warning-subtle {
  border-color: var(--bs-warning-border-subtle) !important;
}

.border-danger-subtle {
  border-color: var(--bs-danger-border-subtle) !important;
}

.border-light-subtle {
  border-color: var(--bs-light-border-subtle) !important;
}

.border-dark-subtle {
  border-color: var(--bs-dark-border-subtle) !important;
}

.border-1 {
  border-width: 1px !important;
}

.border-2 {
  border-width: 2px !important;
}

.border-3 {
  border-width: 3px !important;
}

.border-4 {
  border-width: 4px !important;
}

.border-5 {
  border-width: 5px !important;
}

.border-opacity-10 {
  --bs-border-opacity: 0.1;
}

.border-opacity-25 {
  --bs-border-opacity: 0.25;
}

.border-opacity-50 {
  --bs-border-opacity: 0.5;
}

.border-opacity-75 {
  --bs-border-opacity: 0.75;
}

.border-opacity-100 {
  --bs-border-opacity: 1;
}

.w-25 {
  width: 25% !important;
}

.w-50 {
  width: 50% !important;
}

.w-75 {
  width: 75% !important;
}

.w-100 {
  width: 100% !important;
}

.w-auto {
  width: auto !important;
}

.mw-100 {
  max-width: 100% !important;
}

.vw-100 {
  width: 100vw !important;
}

.min-vw-100 {
  min-width: 100vw !important;
}

.h-25 {
  height: 25% !important;
}

.h-50 {
  height: 50% !important;
}

.h-75 {
  height: 75% !important;
}

.h-100 {
  height: 100% !important;
}

.h-auto {
  height: auto !important;
}

.mh-100 {
  max-height: 100% !important;
}

.vh-100 {
  height: 100vh !important;
}

.min-vh-100 {
  min-height: 100vh !important;
}

.flex-fill, .reportbuilder-report-container {
  flex: 1 1 auto !important;
}

.flex-row {
  flex-direction: row !important;
}

.flex-column {
  flex-direction: column !important;
}

.flex-row-reverse {
  flex-direction: row-reverse !important;
}

.flex-column-reverse {
  flex-direction: column-reverse !important;
}

.flex-grow-0 {
  flex-grow: 0 !important;
}

.flex-grow-1 {
  flex-grow: 1 !important;
}

.flex-shrink-0 {
  flex-shrink: 0 !important;
}

.flex-shrink-1 {
  flex-shrink: 1 !important;
}

.flex-wrap {
  flex-wrap: wrap !important;
}

.flex-nowrap {
  flex-wrap: nowrap !important;
}

.flex-wrap-reverse {
  flex-wrap: wrap-reverse !important;
}

.justify-content-start {
  justify-content: flex-start !important;
}

.justify-content-end {
  justify-content: flex-end !important;
}

.justify-content-center {
  justify-content: center !important;
}

.justify-content-between {
  justify-content: space-between !important;
}

.justify-content-around {
  justify-content: space-around !important;
}

.justify-content-evenly {
  justify-content: space-evenly !important;
}

.align-items-start {
  align-items: flex-start !important;
}

.align-items-end {
  align-items: flex-end !important;
}

.align-items-center {
  align-items: center !important;
}

.align-items-baseline {
  align-items: baseline !important;
}

.align-items-stretch {
  align-items: stretch !important;
}

.align-content-start {
  align-content: flex-start !important;
}

.align-content-end {
  align-content: flex-end !important;
}

.align-content-center {
  align-content: center !important;
}

.align-content-between {
  align-content: space-between !important;
}

.align-content-around {
  align-content: space-around !important;
}

.align-content-stretch {
  align-content: stretch !important;
}

.align-self-auto {
  align-self: auto !important;
}

.align-self-start {
  align-self: flex-start !important;
}

.align-self-end {
  align-self: flex-end !important;
}

.align-self-center {
  align-self: center !important;
}

.align-self-baseline {
  align-self: baseline !important;
}

.align-self-stretch {
  align-self: stretch !important;
}

.order-first {
  order: -1 !important;
}

.order-0 {
  order: 0 !important;
}

.order-1 {
  order: 1 !important;
}

.order-2 {
  order: 2 !important;
}

.order-3 {
  order: 3 !important;
}

.order-4 {
  order: 4 !important;
}

.order-5 {
  order: 5 !important;
}

.order-last {
  order: 6 !important;
}

.m-0 {
  margin: 0 !important;
}

.m-1 {
  margin: 0.25rem !important;
}

.m-2 {
  margin: 0.5rem !important;
}

.m-3 {
  margin: 1rem !important;
}

.m-4 {
  margin: 1.5rem !important;
}

.m-5 {
  margin: 2rem !important;
}

.m-6 {
  margin: 3rem !important;
}

.m-auto {
  margin: auto !important;
}

.mx-0 {
  margin-right: 0 !important;
  margin-left: 0 !important;
}

.mx-1 {
  margin-right: 0.25rem !important;
  margin-left: 0.25rem !important;
}

.mx-2 {
  margin-right: 0.5rem !important;
  margin-left: 0.5rem !important;
}

.mx-3 {
  margin-right: 1rem !important;
  margin-left: 1rem !important;
}

.mx-4 {
  margin-right: 1.5rem !important;
  margin-left: 1.5rem !important;
}

.mx-5 {
  margin-right: 2rem !important;
  margin-left: 2rem !important;
}

.mx-6 {
  margin-right: 3rem !important;
  margin-left: 3rem !important;
}

.mx-auto {
  margin-right: auto !important;
  margin-left: auto !important;
}

.my-0 {
  margin-top: 0 !important;
  margin-bottom: 0 !important;
}

.my-1 {
  margin-top: 0.25rem !important;
  margin-bottom: 0.25rem !important;
}

.my-2 {
  margin-top: 0.5rem !important;
  margin-bottom: 0.5rem !important;
}

.my-3 {
  margin-top: 1rem !important;
  margin-bottom: 1rem !important;
}

.my-4 {
  margin-top: 1.5rem !important;
  margin-bottom: 1.5rem !important;
}

.my-5 {
  margin-top: 2rem !important;
  margin-bottom: 2rem !important;
}

.my-6 {
  margin-top: 3rem !important;
  margin-bottom: 3rem !important;
}

.my-auto {
  margin-top: auto !important;
  margin-bottom: auto !important;
}

.mt-0 {
  margin-top: 0 !important;
}

.mt-1 {
  margin-top: 0.25rem !important;
}

.mt-2 {
  margin-top: 0.5rem !important;
}

.mt-3 {
  margin-top: 1rem !important;
}

.mt-4 {
  margin-top: 1.5rem !important;
}

.mt-5 {
  margin-top: 2rem !important;
}

.mt-6 {
  margin-top: 3rem !important;
}

.mt-auto {
  margin-top: auto !important;
}

.me-0 {
  margin-right: 0 !important;
}

.me-1 {
  margin-right: 0.25rem !important;
}

.me-2 {
  margin-right: 0.5rem !important;
}

.me-3 {
  margin-right: 1rem !important;
}

.me-4 {
  margin-right: 1.5rem !important;
}

.me-5 {
  margin-right: 2rem !important;
}

.me-6 {
  margin-right: 3rem !important;
}

.me-auto {
  margin-right: auto !important;
}

.mb-0 {
  margin-bottom: 0 !important;
}

.mb-1 {
  margin-bottom: 0.25rem !important;
}

.mb-2 {
  margin-bottom: 0.5rem !important;
}

.mb-3 {
  margin-bottom: 1rem !important;
}

.mb-4 {
  margin-bottom: 1.5rem !important;
}

.mb-5 {
  margin-bottom: 2rem !important;
}

.mb-6 {
  margin-bottom: 3rem !important;
}

.mb-auto {
  margin-bottom: auto !important;
}

.ms-0 {
  margin-left: 0 !important;
}

.ms-1 {
  margin-left: 0.25rem !important;
}

.ms-2 {
  margin-left: 0.5rem !important;
}

.ms-3 {
  margin-left: 1rem !important;
}

.ms-4 {
  margin-left: 1.5rem !important;
}

.ms-5 {
  margin-left: 2rem !important;
}

.ms-6 {
  margin-left: 3rem !important;
}

.ms-auto {
  margin-left: auto !important;
}

.p-0 {
  padding: 0 !important;
}

.p-1 {
  padding: 0.25rem !important;
}

.p-2 {
  padding: 0.5rem !important;
}

.p-3 {
  padding: 1rem !important;
}

.p-4 {
  padding: 1.5rem !important;
}

.p-5 {
  padding: 2rem !important;
}

.p-6 {
  padding: 3rem !important;
}

.px-0 {
  padding-right: 0 !important;
  padding-left: 0 !important;
}

.px-1 {
  padding-right: 0.25rem !important;
  padding-left: 0.25rem !important;
}

.px-2 {
  padding-right: 0.5rem !important;
  padding-left: 0.5rem !important;
}

.px-3 {
  padding-right: 1rem !important;
  padding-left: 1rem !important;
}

.px-4 {
  padding-right: 1.5rem !important;
  padding-left: 1.5rem !important;
}

.px-5 {
  padding-right: 2rem !important;
  padding-left: 2rem !important;
}

.px-6 {
  padding-right: 3rem !important;
  padding-left: 3rem !important;
}

.py-0 {
  padding-top: 0 !important;
  padding-bottom: 0 !important;
}

.py-1 {
  padding-top: 0.25rem !important;
  padding-bottom: 0.25rem !important;
}

.py-2 {
  padding-top: 0.5rem !important;
  padding-bottom: 0.5rem !important;
}

.py-3 {
  padding-top: 1rem !important;
  padding-bottom: 1rem !important;
}

.py-4 {
  padding-top: 1.5rem !important;
  padding-bottom: 1.5rem !important;
}

.py-5 {
  padding-top: 2rem !important;
  padding-bottom: 2rem !important;
}

.py-6 {
  padding-top: 3rem !important;
  padding-bottom: 3rem !important;
}

.pt-0 {
  padding-top: 0 !important;
}

.pt-1 {
  padding-top: 0.25rem !important;
}

.pt-2 {
  padding-top: 0.5rem !important;
}

.pt-3 {
  padding-top: 1rem !important;
}

.pt-4 {
  padding-top: 1.5rem !important;
}

.pt-5 {
  padding-top: 2rem !important;
}

.pt-6 {
  padding-top: 3rem !important;
}

.pe-0 {
  padding-right: 0 !important;
}

.pe-1 {
  padding-right: 0.25rem !important;
}

.pe-2 {
  padding-right: 0.5rem !important;
}

.pe-3 {
  padding-right: 1rem !important;
}

.pe-4 {
  padding-right: 1.5rem !important;
}

.pe-5 {
  padding-right: 2rem !important;
}

.pe-6 {
  padding-right: 3rem !important;
}

.pb-0 {
  padding-bottom: 0 !important;
}

.pb-1 {
  padding-bottom: 0.25rem !important;
}

.pb-2 {
  padding-bottom: 0.5rem !important;
}

.pb-3 {
  padding-bottom: 1rem !important;
}

.pb-4 {
  padding-bottom: 1.5rem !important;
}

.pb-5 {
  padding-bottom: 2rem !important;
}

.pb-6 {
  padding-bottom: 3rem !important;
}

.ps-0 {
  padding-left: 0 !important;
}

.ps-1 {
  padding-left: 0.25rem !important;
}

.ps-2 {
  padding-left: 0.5rem !important;
}

.ps-3 {
  padding-left: 1rem !important;
}

.ps-4 {
  padding-left: 1.5rem !important;
}

.ps-5 {
  padding-left: 2rem !important;
}

.ps-6 {
  padding-left: 3rem !important;
}

.gap-0 {
  gap: 0 !important;
}

.gap-1 {
  gap: 0.25rem !important;
}

.gap-2 {
  gap: 0.5rem !important;
}

.gap-3 {
  gap: 1rem !important;
}

.gap-4 {
  gap: 1.5rem !important;
}

.gap-5 {
  gap: 2rem !important;
}

.gap-6 {
  gap: 3rem !important;
}

.row-gap-0 {
  row-gap: 0 !important;
}

.row-gap-1 {
  row-gap: 0.25rem !important;
}

.row-gap-2 {
  row-gap: 0.5rem !important;
}

.row-gap-3 {
  row-gap: 1rem !important;
}

.row-gap-4 {
  row-gap: 1.5rem !important;
}

.row-gap-5 {
  row-gap: 2rem !important;
}

.row-gap-6 {
  row-gap: 3rem !important;
}

.column-gap-0 {
  column-gap: 0 !important;
}

.column-gap-1 {
  column-gap: 0.25rem !important;
}

.column-gap-2 {
  column-gap: 0.5rem !important;
}

.column-gap-3 {
  column-gap: 1rem !important;
}

.column-gap-4 {
  column-gap: 1.5rem !important;
}

.column-gap-5 {
  column-gap: 2rem !important;
}

.column-gap-6 {
  column-gap: 3rem !important;
}

.font-monospace {
  font-family: var(--bs-font-monospace) !important;
}

.fs-1 {
  font-size: calc(1.359375rem + 1.3125vw) !important;
}

.fs-2 {
  font-size: calc(1.3125rem + 0.75vw) !important;
}

.fs-3 {
  font-size: calc(1.2890625rem + 0.46875vw) !important;
}

.fs-4 {
  font-size: calc(1.265625rem + 0.1875vw) !important;
}

.fs-5 {
  font-size: 1.171875rem !important;
}

.fs-6 {
  font-size: 0.9375rem !important;
}

.fst-italic {
  font-style: italic !important;
}

.fst-normal {
  font-style: normal !important;
}

.fw-lighter {
  font-weight: lighter !important;
}

.fw-light {
  font-weight: 300 !important;
}

.fw-normal {
  font-weight: 400 !important;
}

.fw-medium {
  font-weight: 500 !important;
}

.fw-semibold {
  font-weight: 600 !important;
}

.fw-bold {
  font-weight: 700 !important;
}

.fw-bolder {
  font-weight: bolder !important;
}

.lh-1 {
  line-height: 1 !important;
}

.lh-sm {
  line-height: 1.25 !important;
}

.lh-base {
  line-height: 1.5 !important;
}

.lh-lg {
  line-height: 2 !important;
}

.text-start {
  text-align: left !important;
}

.text-end {
  text-align: right !important;
}

.text-center {
  text-align: center !important;
}

.text-decoration-none {
  text-decoration: none !important;
}

.text-decoration-underline {
  text-decoration: underline !important;
}

.text-decoration-line-through {
  text-decoration: line-through !important;
}

.text-lowercase {
  text-transform: lowercase !important;
}

.text-uppercase {
  text-transform: uppercase !important;
}

.text-capitalize {
  text-transform: capitalize !important;
}

.text-wrap {
  white-space: normal !important;
}

.text-nowrap {
  white-space: nowrap !important;
}

/* rtl:begin:remove */
.text-break {
  word-wrap: break-word !important;
  word-break: break-word !important;
}

/* rtl:end:remove */
.text-primary {
  --bs-text-opacity: 1;
  color: rgba(var(--bs-primary-rgb), var(--bs-text-opacity)) !important;
}

.text-secondary {
  --bs-text-opacity: 1;
  color: rgba(var(--bs-secondary-rgb), var(--bs-text-opacity)) !important;
}

.text-success {
  --bs-text-opacity: 1;
  color: rgba(var(--bs-success-rgb), var(--bs-text-opacity)) !important;
}

.text-info {
  --bs-text-opacity: 1;
  color: rgba(var(--bs-info-rgb), var(--bs-text-opacity)) !important;
}

.text-warning {
  --bs-text-opacity: 1;
  color: rgba(var(--bs-warning-rgb), var(--bs-text-opacity)) !important;
}

.text-danger {
  --bs-text-opacity: 1;
  color: rgba(var(--bs-danger-rgb), var(--bs-text-opacity)) !important;
}

.text-light {
  --bs-text-opacity: 1;
  color: rgba(var(--bs-light-rgb), var(--bs-text-opacity)) !important;
}

.text-dark {
  --bs-text-opacity: 1;
  color: rgba(var(--bs-dark-rgb), var(--bs-text-opacity)) !important;
}

.text-black {
  --bs-text-opacity: 1;
  color: rgba(var(--bs-black-rgb), var(--bs-text-opacity)) !important;
}

.text-white {
  --bs-text-opacity: 1;
  color: rgba(var(--bs-white-rgb), var(--bs-text-opacity)) !important;
}

.text-body {
  --bs-text-opacity: 1;
  color: rgba(var(--bs-body-color-rgb), var(--bs-text-opacity)) !important;
}

.text-muted {
  --bs-text-opacity: 1;
  color: var(--bs-secondary-color) !important;
}

.text-black-50 {
  --bs-text-opacity: 1;
  color: rgba(0, 0, 0, 0.5) !important;
}

.text-white-50 {
  --bs-text-opacity: 1;
  color: rgba(255, 255, 255, 0.5) !important;
}

.text-body-secondary {
  --bs-text-opacity: 1;
  color: var(--bs-secondary-color) !important;
}

.text-body-tertiary {
  --bs-text-opacity: 1;
  color: var(--bs-tertiary-color) !important;
}

.text-body-emphasis {
  --bs-text-opacity: 1;
  color: var(--bs-emphasis-color) !important;
}

.text-reset {
  --bs-text-opacity: 1;
  color: inherit !important;
}

.text-opacity-25 {
  --bs-text-opacity: 0.25;
}

.text-opacity-50 {
  --bs-text-opacity: 0.5;
}

.text-opacity-75 {
  --bs-text-opacity: 0.75;
}

.text-opacity-100 {
  --bs-text-opacity: 1;
}

.text-primary-emphasis {
  color: var(--bs-primary-text-emphasis) !important;
}

.text-secondary-emphasis {
  color: var(--bs-secondary-text-emphasis) !important;
}

.text-success-emphasis {
  color: var(--bs-success-text-emphasis) !important;
}

.text-info-emphasis {
  color: var(--bs-info-text-emphasis) !important;
}

.text-warning-emphasis {
  color: var(--bs-warning-text-emphasis) !important;
}

.text-danger-emphasis {
  color: var(--bs-danger-text-emphasis) !important;
}

.text-light-emphasis {
  color: var(--bs-light-text-emphasis) !important;
}

.text-dark-emphasis {
  color: var(--bs-dark-text-emphasis) !important;
}

.link-opacity-10 {
  --bs-link-opacity: 0.1;
}

.link-opacity-10-hover:hover {
  --bs-link-opacity: 0.1;
}

.link-opacity-25 {
  --bs-link-opacity: 0.25;
}

.link-opacity-25-hover:hover {
  --bs-link-opacity: 0.25;
}

.link-opacity-50 {
  --bs-link-opacity: 0.5;
}

.link-opacity-50-hover:hover {
  --bs-link-opacity: 0.5;
}

.link-opacity-75 {
  --bs-link-opacity: 0.75;
}

.link-opacity-75-hover:hover {
  --bs-link-opacity: 0.75;
}

.link-opacity-100 {
  --bs-link-opacity: 1;
}

.link-opacity-100-hover:hover {
  --bs-link-opacity: 1;
}

.link-offset-1 {
  text-underline-offset: 0.125em !important;
}

.link-offset-1-hover:hover {
  text-underline-offset: 0.125em !important;
}

.link-offset-2 {
  text-underline-offset: 0.25em !important;
}

.link-offset-2-hover:hover {
  text-underline-offset: 0.25em !important;
}

.link-offset-3 {
  text-underline-offset: 0.375em !important;
}

.link-offset-3-hover:hover {
  text-underline-offset: 0.375em !important;
}

.link-underline-primary {
  --bs-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--bs-primary-rgb), var(--bs-link-underline-opacity)) !important;
}

.link-underline-secondary {
  --bs-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--bs-secondary-rgb), var(--bs-link-underline-opacity)) !important;
}

.link-underline-success {
  --bs-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--bs-success-rgb), var(--bs-link-underline-opacity)) !important;
}

.link-underline-info {
  --bs-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--bs-info-rgb), var(--bs-link-underline-opacity)) !important;
}

.link-underline-warning {
  --bs-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--bs-warning-rgb), var(--bs-link-underline-opacity)) !important;
}

.link-underline-danger {
  --bs-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--bs-danger-rgb), var(--bs-link-underline-opacity)) !important;
}

.link-underline-light {
  --bs-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--bs-light-rgb), var(--bs-link-underline-opacity)) !important;
}

.link-underline-dark {
  --bs-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--bs-dark-rgb), var(--bs-link-underline-opacity)) !important;
}

.link-underline {
  --bs-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--bs-link-color-rgb), var(--bs-link-underline-opacity, 1)) !important;
}

.link-underline-opacity-0 {
  --bs-link-underline-opacity: 0;
}

.link-underline-opacity-0-hover:hover {
  --bs-link-underline-opacity: 0;
}

.link-underline-opacity-10 {
  --bs-link-underline-opacity: 0.1;
}

.link-underline-opacity-10-hover:hover {
  --bs-link-underline-opacity: 0.1;
}

.link-underline-opacity-25 {
  --bs-link-underline-opacity: 0.25;
}

.link-underline-opacity-25-hover:hover {
  --bs-link-underline-opacity: 0.25;
}

.link-underline-opacity-50 {
  --bs-link-underline-opacity: 0.5;
}

.link-underline-opacity-50-hover:hover {
  --bs-link-underline-opacity: 0.5;
}

.link-underline-opacity-75 {
  --bs-link-underline-opacity: 0.75;
}

.link-underline-opacity-75-hover:hover {
  --bs-link-underline-opacity: 0.75;
}

.link-underline-opacity-100 {
  --bs-link-underline-opacity: 1;
}

.link-underline-opacity-100-hover:hover {
  --bs-link-underline-opacity: 1;
}

.bg-primary {
  --bs-bg-opacity: 1;
  background-color: rgba(var(--bs-primary-rgb), var(--bs-bg-opacity)) !important;
}

.bg-secondary {
  --bs-bg-opacity: 1;
  background-color: rgba(var(--bs-secondary-rgb), var(--bs-bg-opacity)) !important;
}

.bg-success {
  --bs-bg-opacity: 1;
  background-color: rgba(var(--bs-success-rgb), var(--bs-bg-opacity)) !important;
}

.bg-info {
  --bs-bg-opacity: 1;
  background-color: rgba(var(--bs-info-rgb), var(--bs-bg-opacity)) !important;
}

.bg-warning {
  --bs-bg-opacity: 1;
  background-color: rgba(var(--bs-warning-rgb), var(--bs-bg-opacity)) !important;
}

.bg-danger {
  --bs-bg-opacity: 1;
  background-color: rgba(var(--bs-danger-rgb), var(--bs-bg-opacity)) !important;
}

.bg-light {
  --bs-bg-opacity: 1;
  background-color: rgba(var(--bs-light-rgb), var(--bs-bg-opacity)) !important;
}

.bg-dark {
  --bs-bg-opacity: 1;
  background-color: rgba(var(--bs-dark-rgb), var(--bs-bg-opacity)) !important;
}

.bg-black {
  --bs-bg-opacity: 1;
  background-color: rgba(var(--bs-black-rgb), var(--bs-bg-opacity)) !important;
}

.bg-white {
  --bs-bg-opacity: 1;
  background-color: rgba(var(--bs-white-rgb), var(--bs-bg-opacity)) !important;
}

.bg-body {
  --bs-bg-opacity: 1;
  background-color: rgba(var(--bs-body-bg-rgb), var(--bs-bg-opacity)) !important;
}

.bg-transparent {
  --bs-bg-opacity: 1;
  background-color: transparent !important;
}

.bg-body-secondary {
  --bs-bg-opacity: 1;
  background-color: rgba(var(--bs-secondary-bg-rgb), var(--bs-bg-opacity)) !important;
}

.bg-body-tertiary {
  --bs-bg-opacity: 1;
  background-color: rgba(var(--bs-tertiary-bg-rgb), var(--bs-bg-opacity)) !important;
}

.bg-opacity-10 {
  --bs-bg-opacity: 0.1;
}

.bg-opacity-25 {
  --bs-bg-opacity: 0.25;
}

.bg-opacity-50 {
  --bs-bg-opacity: 0.5;
}

.bg-opacity-75 {
  --bs-bg-opacity: 0.75;
}

.bg-opacity-100 {
  --bs-bg-opacity: 1;
}

.bg-primary-subtle {
  background-color: var(--bs-primary-bg-subtle) !important;
}

.bg-secondary-subtle {
  background-color: var(--bs-secondary-bg-subtle) !important;
}

.bg-success-subtle {
  background-color: var(--bs-success-bg-subtle) !important;
}

.bg-info-subtle {
  background-color: var(--bs-info-bg-subtle) !important;
}

.bg-warning-subtle {
  background-color: var(--bs-warning-bg-subtle) !important;
}

.bg-danger-subtle {
  background-color: var(--bs-danger-bg-subtle) !important;
}

.bg-light-subtle {
  background-color: var(--bs-light-bg-subtle) !important;
}

.bg-dark-subtle {
  background-color: var(--bs-dark-bg-subtle) !important;
}

.bg-gradient {
  background-image: var(--bs-gradient) !important;
}

.user-select-all {
  user-select: all !important;
}

.user-select-auto {
  user-select: auto !important;
}

.user-select-none {
  user-select: none !important;
}

.pe-none {
  pointer-events: none !important;
}

.pe-auto {
  pointer-events: auto !important;
}

.rounded {
  border-radius: var(--bs-border-radius) !important;
}

.rounded-0 {
  border-radius: 0 !important;
}

.rounded-1 {
  border-radius: var(--bs-border-radius-sm) !important;
}

.rounded-2 {
  border-radius: var(--bs-border-radius) !important;
}

.rounded-3 {
  border-radius: var(--bs-border-radius-lg) !important;
}

.rounded-4 {
  border-radius: var(--bs-border-radius-xl) !important;
}

.rounded-5 {
  border-radius: var(--bs-border-radius-xxl) !important;
}

.rounded-circle {
  border-radius: 50% !important;
}

.rounded-pill {
  border-radius: var(--bs-border-radius-pill) !important;
}

.rounded-top {
  border-top-left-radius: var(--bs-border-radius) !important;
  border-top-right-radius: var(--bs-border-radius) !important;
}

.rounded-top-0 {
  border-top-left-radius: 0 !important;
  border-top-right-radius: 0 !important;
}

.rounded-top-1 {
  border-top-left-radius: var(--bs-border-radius-sm) !important;
  border-top-right-radius: var(--bs-border-radius-sm) !important;
}

.rounded-top-2 {
  border-top-left-radius: var(--bs-border-radius) !important;
  border-top-right-radius: var(--bs-border-radius) !important;
}

.rounded-top-3 {
  border-top-left-radius: var(--bs-border-radius-lg) !important;
  border-top-right-radius: var(--bs-border-radius-lg) !important;
}

.rounded-top-4 {
  border-top-left-radius: var(--bs-border-radius-xl) !important;
  border-top-right-radius: var(--bs-border-radius-xl) !important;
}

.rounded-top-5 {
  border-top-left-radius: var(--bs-border-radius-xxl) !important;
  border-top-right-radius: var(--bs-border-radius-xxl) !important;
}

.rounded-top-circle {
  border-top-left-radius: 50% !important;
  border-top-right-radius: 50% !important;
}

.rounded-top-pill {
  border-top-left-radius: var(--bs-border-radius-pill) !important;
  border-top-right-radius: var(--bs-border-radius-pill) !important;
}

.rounded-end {
  border-top-right-radius: var(--bs-border-radius) !important;
  border-bottom-right-radius: var(--bs-border-radius) !important;
}

.rounded-end-0 {
  border-top-right-radius: 0 !important;
  border-bottom-right-radius: 0 !important;
}

.rounded-end-1 {
  border-top-right-radius: var(--bs-border-radius-sm) !important;
  border-bottom-right-radius: var(--bs-border-radius-sm) !important;
}

.rounded-end-2 {
  border-top-right-radius: var(--bs-border-radius) !important;
  border-bottom-right-radius: var(--bs-border-radius) !important;
}

.rounded-end-3 {
  border-top-right-radius: var(--bs-border-radius-lg) !important;
  border-bottom-right-radius: var(--bs-border-radius-lg) !important;
}

.rounded-end-4 {
  border-top-right-radius: var(--bs-border-radius-xl) !important;
  border-bottom-right-radius: var(--bs-border-radius-xl) !important;
}

.rounded-end-5 {
  border-top-right-radius: var(--bs-border-radius-xxl) !important;
  border-bottom-right-radius: var(--bs-border-radius-xxl) !important;
}

.rounded-end-circle {
  border-top-right-radius: 50% !important;
  border-bottom-right-radius: 50% !important;
}

.rounded-end-pill {
  border-top-right-radius: var(--bs-border-radius-pill) !important;
  border-bottom-right-radius: var(--bs-border-radius-pill) !important;
}

.rounded-bottom {
  border-bottom-right-radius: var(--bs-border-radius) !important;
  border-bottom-left-radius: var(--bs-border-radius) !important;
}

.rounded-bottom-0 {
  border-bottom-right-radius: 0 !important;
  border-bottom-left-radius: 0 !important;
}

.rounded-bottom-1 {
  border-bottom-right-radius: var(--bs-border-radius-sm) !important;
  border-bottom-left-radius: var(--bs-border-radius-sm) !important;
}

.rounded-bottom-2 {
  border-bottom-right-radius: var(--bs-border-radius) !important;
  border-bottom-left-radius: var(--bs-border-radius) !important;
}

.rounded-bottom-3 {
  border-bottom-right-radius: var(--bs-border-radius-lg) !important;
  border-bottom-left-radius: var(--bs-border-radius-lg) !important;
}

.rounded-bottom-4 {
  border-bottom-right-radius: var(--bs-border-radius-xl) !important;
  border-bottom-left-radius: var(--bs-border-radius-xl) !important;
}

.rounded-bottom-5 {
  border-bottom-right-radius: var(--bs-border-radius-xxl) !important;
  border-bottom-left-radius: var(--bs-border-radius-xxl) !important;
}

.rounded-bottom-circle {
  border-bottom-right-radius: 50% !important;
  border-bottom-left-radius: 50% !important;
}

.rounded-bottom-pill {
  border-bottom-right-radius: var(--bs-border-radius-pill) !important;
  border-bottom-left-radius: var(--bs-border-radius-pill) !important;
}

.rounded-start {
  border-bottom-left-radius: var(--bs-border-radius) !important;
  border-top-left-radius: var(--bs-border-radius) !important;
}

.rounded-start-0 {
  border-bottom-left-radius: 0 !important;
  border-top-left-radius: 0 !important;
}

.rounded-start-1 {
  border-bottom-left-radius: var(--bs-border-radius-sm) !important;
  border-top-left-radius: var(--bs-border-radius-sm) !important;
}

.rounded-start-2 {
  border-bottom-left-radius: var(--bs-border-radius) !important;
  border-top-left-radius: var(--bs-border-radius) !important;
}

.rounded-start-3 {
  border-bottom-left-radius: var(--bs-border-radius-lg) !important;
  border-top-left-radius: var(--bs-border-radius-lg) !important;
}

.rounded-start-4 {
  border-bottom-left-radius: var(--bs-border-radius-xl) !important;
  border-top-left-radius: var(--bs-border-radius-xl) !important;
}

.rounded-start-5 {
  border-bottom-left-radius: var(--bs-border-radius-xxl) !important;
  border-top-left-radius: var(--bs-border-radius-xxl) !important;
}

.rounded-start-circle {
  border-bottom-left-radius: 50% !important;
  border-top-left-radius: 50% !important;
}

.rounded-start-pill {
  border-bottom-left-radius: var(--bs-border-radius-pill) !important;
  border-top-left-radius: var(--bs-border-radius-pill) !important;
}

.visible {
  visibility: visible !important;
}

.invisible {
  visibility: hidden !important;
}

.z-n1 {
  z-index: -1 !important;
}

.z-0 {
  z-index: 0 !important;
}

.z-1 {
  z-index: 1 !important;
}

.z-2 {
  z-index: 2 !important;
}

.z-3 {
  z-index: 3 !important;
}

@media (min-width: 576px) {
  .float-sm-start {
    float: left !important;
  }
  .float-sm-end {
    float: right !important;
  }
  .float-sm-none {
    float: none !important;
  }
  .object-fit-sm-contain {
    object-fit: contain !important;
  }
  .object-fit-sm-cover {
    object-fit: cover !important;
  }
  .object-fit-sm-fill {
    object-fit: fill !important;
  }
  .object-fit-sm-scale {
    object-fit: scale-down !important;
  }
  .object-fit-sm-none {
    object-fit: none !important;
  }
  .d-sm-inline {
    display: inline !important;
  }
  .d-sm-inline-block {
    display: inline-block !important;
  }
  .d-sm-block {
    display: block !important;
  }
  .d-sm-grid {
    display: grid !important;
  }
  .d-sm-inline-grid {
    display: inline-grid !important;
  }
  .d-sm-table {
    display: table !important;
  }
  .d-sm-table-row {
    display: table-row !important;
  }
  .d-sm-table-cell {
    display: table-cell !important;
  }
  .d-sm-flex {
    display: flex !important;
  }
  .d-sm-inline-flex {
    display: inline-flex !important;
  }
  .d-sm-none {
    display: none !important;
  }
  .flex-sm-fill {
    flex: 1 1 auto !important;
  }
  .flex-sm-row {
    flex-direction: row !important;
  }
  .flex-sm-column {
    flex-direction: column !important;
  }
  .flex-sm-row-reverse {
    flex-direction: row-reverse !important;
  }
  .flex-sm-column-reverse {
    flex-direction: column-reverse !important;
  }
  .flex-sm-grow-0 {
    flex-grow: 0 !important;
  }
  .flex-sm-grow-1 {
    flex-grow: 1 !important;
  }
  .flex-sm-shrink-0 {
    flex-shrink: 0 !important;
  }
  .flex-sm-shrink-1 {
    flex-shrink: 1 !important;
  }
  .flex-sm-wrap {
    flex-wrap: wrap !important;
  }
  .flex-sm-nowrap {
    flex-wrap: nowrap !important;
  }
  .flex-sm-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }
  .justify-content-sm-start {
    justify-content: flex-start !important;
  }
  .justify-content-sm-end {
    justify-content: flex-end !important;
  }
  .justify-content-sm-center {
    justify-content: center !important;
  }
  .justify-content-sm-between {
    justify-content: space-between !important;
  }
  .justify-content-sm-around {
    justify-content: space-around !important;
  }
  .justify-content-sm-evenly {
    justify-content: space-evenly !important;
  }
  .align-items-sm-start {
    align-items: flex-start !important;
  }
  .align-items-sm-end {
    align-items: flex-end !important;
  }
  .align-items-sm-center {
    align-items: center !important;
  }
  .align-items-sm-baseline {
    align-items: baseline !important;
  }
  .align-items-sm-stretch {
    align-items: stretch !important;
  }
  .align-content-sm-start {
    align-content: flex-start !important;
  }
  .align-content-sm-end {
    align-content: flex-end !important;
  }
  .align-content-sm-center {
    align-content: center !important;
  }
  .align-content-sm-between {
    align-content: space-between !important;
  }
  .align-content-sm-around {
    align-content: space-around !important;
  }
  .align-content-sm-stretch {
    align-content: stretch !important;
  }
  .align-self-sm-auto {
    align-self: auto !important;
  }
  .align-self-sm-start {
    align-self: flex-start !important;
  }
  .align-self-sm-end {
    align-self: flex-end !important;
  }
  .align-self-sm-center {
    align-self: center !important;
  }
  .align-self-sm-baseline {
    align-self: baseline !important;
  }
  .align-self-sm-stretch {
    align-self: stretch !important;
  }
  .order-sm-first {
    order: -1 !important;
  }
  .order-sm-0 {
    order: 0 !important;
  }
  .order-sm-1 {
    order: 1 !important;
  }
  .order-sm-2 {
    order: 2 !important;
  }
  .order-sm-3 {
    order: 3 !important;
  }
  .order-sm-4 {
    order: 4 !important;
  }
  .order-sm-5 {
    order: 5 !important;
  }
  .order-sm-last {
    order: 6 !important;
  }
  .m-sm-0 {
    margin: 0 !important;
  }
  .m-sm-1 {
    margin: 0.25rem !important;
  }
  .m-sm-2 {
    margin: 0.5rem !important;
  }
  .m-sm-3 {
    margin: 1rem !important;
  }
  .m-sm-4 {
    margin: 1.5rem !important;
  }
  .m-sm-5 {
    margin: 2rem !important;
  }
  .m-sm-6 {
    margin: 3rem !important;
  }
  .m-sm-auto {
    margin: auto !important;
  }
  .mx-sm-0 {
    margin-right: 0 !important;
    margin-left: 0 !important;
  }
  .mx-sm-1 {
    margin-right: 0.25rem !important;
    margin-left: 0.25rem !important;
  }
  .mx-sm-2 {
    margin-right: 0.5rem !important;
    margin-left: 0.5rem !important;
  }
  .mx-sm-3 {
    margin-right: 1rem !important;
    margin-left: 1rem !important;
  }
  .mx-sm-4 {
    margin-right: 1.5rem !important;
    margin-left: 1.5rem !important;
  }
  .mx-sm-5 {
    margin-right: 2rem !important;
    margin-left: 2rem !important;
  }
  .mx-sm-6 {
    margin-right: 3rem !important;
    margin-left: 3rem !important;
  }
  .mx-sm-auto {
    margin-right: auto !important;
    margin-left: auto !important;
  }
  .my-sm-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }
  .my-sm-1 {
    margin-top: 0.25rem !important;
    margin-bottom: 0.25rem !important;
  }
  .my-sm-2 {
    margin-top: 0.5rem !important;
    margin-bottom: 0.5rem !important;
  }
  .my-sm-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }
  .my-sm-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }
  .my-sm-5 {
    margin-top: 2rem !important;
    margin-bottom: 2rem !important;
  }
  .my-sm-6 {
    margin-top: 3rem !important;
    margin-bottom: 3rem !important;
  }
  .my-sm-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }
  .mt-sm-0 {
    margin-top: 0 !important;
  }
  .mt-sm-1 {
    margin-top: 0.25rem !important;
  }
  .mt-sm-2 {
    margin-top: 0.5rem !important;
  }
  .mt-sm-3 {
    margin-top: 1rem !important;
  }
  .mt-sm-4 {
    margin-top: 1.5rem !important;
  }
  .mt-sm-5 {
    margin-top: 2rem !important;
  }
  .mt-sm-6 {
    margin-top: 3rem !important;
  }
  .mt-sm-auto {
    margin-top: auto !important;
  }
  .me-sm-0 {
    margin-right: 0 !important;
  }
  .me-sm-1 {
    margin-right: 0.25rem !important;
  }
  .me-sm-2 {
    margin-right: 0.5rem !important;
  }
  .me-sm-3 {
    margin-right: 1rem !important;
  }
  .me-sm-4 {
    margin-right: 1.5rem !important;
  }
  .me-sm-5 {
    margin-right: 2rem !important;
  }
  .me-sm-6 {
    margin-right: 3rem !important;
  }
  .me-sm-auto {
    margin-right: auto !important;
  }
  .mb-sm-0 {
    margin-bottom: 0 !important;
  }
  .mb-sm-1 {
    margin-bottom: 0.25rem !important;
  }
  .mb-sm-2 {
    margin-bottom: 0.5rem !important;
  }
  .mb-sm-3 {
    margin-bottom: 1rem !important;
  }
  .mb-sm-4 {
    margin-bottom: 1.5rem !important;
  }
  .mb-sm-5 {
    margin-bottom: 2rem !important;
  }
  .mb-sm-6 {
    margin-bottom: 3rem !important;
  }
  .mb-sm-auto {
    margin-bottom: auto !important;
  }
  .ms-sm-0 {
    margin-left: 0 !important;
  }
  .ms-sm-1 {
    margin-left: 0.25rem !important;
  }
  .ms-sm-2 {
    margin-left: 0.5rem !important;
  }
  .ms-sm-3 {
    margin-left: 1rem !important;
  }
  .ms-sm-4 {
    margin-left: 1.5rem !important;
  }
  .ms-sm-5 {
    margin-left: 2rem !important;
  }
  .ms-sm-6 {
    margin-left: 3rem !important;
  }
  .ms-sm-auto {
    margin-left: auto !important;
  }
  .p-sm-0 {
    padding: 0 !important;
  }
  .p-sm-1 {
    padding: 0.25rem !important;
  }
  .p-sm-2 {
    padding: 0.5rem !important;
  }
  .p-sm-3 {
    padding: 1rem !important;
  }
  .p-sm-4 {
    padding: 1.5rem !important;
  }
  .p-sm-5 {
    padding: 2rem !important;
  }
  .p-sm-6 {
    padding: 3rem !important;
  }
  .px-sm-0 {
    padding-right: 0 !important;
    padding-left: 0 !important;
  }
  .px-sm-1 {
    padding-right: 0.25rem !important;
    padding-left: 0.25rem !important;
  }
  .px-sm-2 {
    padding-right: 0.5rem !important;
    padding-left: 0.5rem !important;
  }
  .px-sm-3 {
    padding-right: 1rem !important;
    padding-left: 1rem !important;
  }
  .px-sm-4 {
    padding-right: 1.5rem !important;
    padding-left: 1.5rem !important;
  }
  .px-sm-5 {
    padding-right: 2rem !important;
    padding-left: 2rem !important;
  }
  .px-sm-6 {
    padding-right: 3rem !important;
    padding-left: 3rem !important;
  }
  .py-sm-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }
  .py-sm-1 {
    padding-top: 0.25rem !important;
    padding-bottom: 0.25rem !important;
  }
  .py-sm-2 {
    padding-top: 0.5rem !important;
    padding-bottom: 0.5rem !important;
  }
  .py-sm-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }
  .py-sm-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }
  .py-sm-5 {
    padding-top: 2rem !important;
    padding-bottom: 2rem !important;
  }
  .py-sm-6 {
    padding-top: 3rem !important;
    padding-bottom: 3rem !important;
  }
  .pt-sm-0 {
    padding-top: 0 !important;
  }
  .pt-sm-1 {
    padding-top: 0.25rem !important;
  }
  .pt-sm-2 {
    padding-top: 0.5rem !important;
  }
  .pt-sm-3 {
    padding-top: 1rem !important;
  }
  .pt-sm-4 {
    padding-top: 1.5rem !important;
  }
  .pt-sm-5 {
    padding-top: 2rem !important;
  }
  .pt-sm-6 {
    padding-top: 3rem !important;
  }
  .pe-sm-0 {
    padding-right: 0 !important;
  }
  .pe-sm-1 {
    padding-right: 0.25rem !important;
  }
  .pe-sm-2 {
    padding-right: 0.5rem !important;
  }
  .pe-sm-3 {
    padding-right: 1rem !important;
  }
  .pe-sm-4 {
    padding-right: 1.5rem !important;
  }
  .pe-sm-5 {
    padding-right: 2rem !important;
  }
  .pe-sm-6 {
    padding-right: 3rem !important;
  }
  .pb-sm-0 {
    padding-bottom: 0 !important;
  }
  .pb-sm-1 {
    padding-bottom: 0.25rem !important;
  }
  .pb-sm-2 {
    padding-bottom: 0.5rem !important;
  }
  .pb-sm-3 {
    padding-bottom: 1rem !important;
  }
  .pb-sm-4 {
    padding-bottom: 1.5rem !important;
  }
  .pb-sm-5 {
    padding-bottom: 2rem !important;
  }
  .pb-sm-6 {
    padding-bottom: 3rem !important;
  }
  .ps-sm-0 {
    padding-left: 0 !important;
  }
  .ps-sm-1 {
    padding-left: 0.25rem !important;
  }
  .ps-sm-2 {
    padding-left: 0.5rem !important;
  }
  .ps-sm-3 {
    padding-left: 1rem !important;
  }
  .ps-sm-4 {
    padding-left: 1.5rem !important;
  }
  .ps-sm-5 {
    padding-left: 2rem !important;
  }
  .ps-sm-6 {
    padding-left: 3rem !important;
  }
  .gap-sm-0 {
    gap: 0 !important;
  }
  .gap-sm-1 {
    gap: 0.25rem !important;
  }
  .gap-sm-2 {
    gap: 0.5rem !important;
  }
  .gap-sm-3 {
    gap: 1rem !important;
  }
  .gap-sm-4 {
    gap: 1.5rem !important;
  }
  .gap-sm-5 {
    gap: 2rem !important;
  }
  .gap-sm-6 {
    gap: 3rem !important;
  }
  .row-gap-sm-0 {
    row-gap: 0 !important;
  }
  .row-gap-sm-1 {
    row-gap: 0.25rem !important;
  }
  .row-gap-sm-2 {
    row-gap: 0.5rem !important;
  }
  .row-gap-sm-3 {
    row-gap: 1rem !important;
  }
  .row-gap-sm-4 {
    row-gap: 1.5rem !important;
  }
  .row-gap-sm-5 {
    row-gap: 2rem !important;
  }
  .row-gap-sm-6 {
    row-gap: 3rem !important;
  }
  .column-gap-sm-0 {
    column-gap: 0 !important;
  }
  .column-gap-sm-1 {
    column-gap: 0.25rem !important;
  }
  .column-gap-sm-2 {
    column-gap: 0.5rem !important;
  }
  .column-gap-sm-3 {
    column-gap: 1rem !important;
  }
  .column-gap-sm-4 {
    column-gap: 1.5rem !important;
  }
  .column-gap-sm-5 {
    column-gap: 2rem !important;
  }
  .column-gap-sm-6 {
    column-gap: 3rem !important;
  }
  .text-sm-start {
    text-align: left !important;
  }
  .text-sm-end {
    text-align: right !important;
  }
  .text-sm-center {
    text-align: center !important;
  }
}
@media (min-width: 768px) {
  .float-md-start {
    float: left !important;
  }
  .float-md-end {
    float: right !important;
  }
  .float-md-none {
    float: none !important;
  }
  .object-fit-md-contain {
    object-fit: contain !important;
  }
  .object-fit-md-cover {
    object-fit: cover !important;
  }
  .object-fit-md-fill {
    object-fit: fill !important;
  }
  .object-fit-md-scale {
    object-fit: scale-down !important;
  }
  .object-fit-md-none {
    object-fit: none !important;
  }
  .d-md-inline {
    display: inline !important;
  }
  .d-md-inline-block {
    display: inline-block !important;
  }
  .d-md-block {
    display: block !important;
  }
  .d-md-grid {
    display: grid !important;
  }
  .d-md-inline-grid {
    display: inline-grid !important;
  }
  .d-md-table {
    display: table !important;
  }
  .d-md-table-row {
    display: table-row !important;
  }
  .d-md-table-cell {
    display: table-cell !important;
  }
  .d-md-flex {
    display: flex !important;
  }
  .d-md-inline-flex {
    display: inline-flex !important;
  }
  .d-md-none {
    display: none !important;
  }
  .flex-md-fill {
    flex: 1 1 auto !important;
  }
  .flex-md-row {
    flex-direction: row !important;
  }
  .flex-md-column {
    flex-direction: column !important;
  }
  .flex-md-row-reverse {
    flex-direction: row-reverse !important;
  }
  .flex-md-column-reverse {
    flex-direction: column-reverse !important;
  }
  .flex-md-grow-0 {
    flex-grow: 0 !important;
  }
  .flex-md-grow-1 {
    flex-grow: 1 !important;
  }
  .flex-md-shrink-0 {
    flex-shrink: 0 !important;
  }
  .flex-md-shrink-1 {
    flex-shrink: 1 !important;
  }
  .flex-md-wrap {
    flex-wrap: wrap !important;
  }
  .flex-md-nowrap {
    flex-wrap: nowrap !important;
  }
  .flex-md-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }
  .justify-content-md-start {
    justify-content: flex-start !important;
  }
  .justify-content-md-end {
    justify-content: flex-end !important;
  }
  .justify-content-md-center {
    justify-content: center !important;
  }
  .justify-content-md-between {
    justify-content: space-between !important;
  }
  .justify-content-md-around {
    justify-content: space-around !important;
  }
  .justify-content-md-evenly {
    justify-content: space-evenly !important;
  }
  .align-items-md-start {
    align-items: flex-start !important;
  }
  .align-items-md-end {
    align-items: flex-end !important;
  }
  .align-items-md-center {
    align-items: center !important;
  }
  .align-items-md-baseline {
    align-items: baseline !important;
  }
  .align-items-md-stretch {
    align-items: stretch !important;
  }
  .align-content-md-start {
    align-content: flex-start !important;
  }
  .align-content-md-end {
    align-content: flex-end !important;
  }
  .align-content-md-center {
    align-content: center !important;
  }
  .align-content-md-between {
    align-content: space-between !important;
  }
  .align-content-md-around {
    align-content: space-around !important;
  }
  .align-content-md-stretch {
    align-content: stretch !important;
  }
  .align-self-md-auto {
    align-self: auto !important;
  }
  .align-self-md-start {
    align-self: flex-start !important;
  }
  .align-self-md-end {
    align-self: flex-end !important;
  }
  .align-self-md-center {
    align-self: center !important;
  }
  .align-self-md-baseline {
    align-self: baseline !important;
  }
  .align-self-md-stretch {
    align-self: stretch !important;
  }
  .order-md-first {
    order: -1 !important;
  }
  .order-md-0 {
    order: 0 !important;
  }
  .order-md-1 {
    order: 1 !important;
  }
  .order-md-2 {
    order: 2 !important;
  }
  .order-md-3 {
    order: 3 !important;
  }
  .order-md-4 {
    order: 4 !important;
  }
  .order-md-5 {
    order: 5 !important;
  }
  .order-md-last {
    order: 6 !important;
  }
  .m-md-0 {
    margin: 0 !important;
  }
  .m-md-1 {
    margin: 0.25rem !important;
  }
  .m-md-2 {
    margin: 0.5rem !important;
  }
  .m-md-3 {
    margin: 1rem !important;
  }
  .m-md-4 {
    margin: 1.5rem !important;
  }
  .m-md-5 {
    margin: 2rem !important;
  }
  .m-md-6 {
    margin: 3rem !important;
  }
  .m-md-auto {
    margin: auto !important;
  }
  .mx-md-0 {
    margin-right: 0 !important;
    margin-left: 0 !important;
  }
  .mx-md-1 {
    margin-right: 0.25rem !important;
    margin-left: 0.25rem !important;
  }
  .mx-md-2 {
    margin-right: 0.5rem !important;
    margin-left: 0.5rem !important;
  }
  .mx-md-3 {
    margin-right: 1rem !important;
    margin-left: 1rem !important;
  }
  .mx-md-4 {
    margin-right: 1.5rem !important;
    margin-left: 1.5rem !important;
  }
  .mx-md-5 {
    margin-right: 2rem !important;
    margin-left: 2rem !important;
  }
  .mx-md-6 {
    margin-right: 3rem !important;
    margin-left: 3rem !important;
  }
  .mx-md-auto {
    margin-right: auto !important;
    margin-left: auto !important;
  }
  .my-md-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }
  .my-md-1 {
    margin-top: 0.25rem !important;
    margin-bottom: 0.25rem !important;
  }
  .my-md-2 {
    margin-top: 0.5rem !important;
    margin-bottom: 0.5rem !important;
  }
  .my-md-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }
  .my-md-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }
  .my-md-5 {
    margin-top: 2rem !important;
    margin-bottom: 2rem !important;
  }
  .my-md-6 {
    margin-top: 3rem !important;
    margin-bottom: 3rem !important;
  }
  .my-md-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }
  .mt-md-0 {
    margin-top: 0 !important;
  }
  .mt-md-1 {
    margin-top: 0.25rem !important;
  }
  .mt-md-2 {
    margin-top: 0.5rem !important;
  }
  .mt-md-3 {
    margin-top: 1rem !important;
  }
  .mt-md-4 {
    margin-top: 1.5rem !important;
  }
  .mt-md-5 {
    margin-top: 2rem !important;
  }
  .mt-md-6 {
    margin-top: 3rem !important;
  }
  .mt-md-auto {
    margin-top: auto !important;
  }
  .me-md-0 {
    margin-right: 0 !important;
  }
  .me-md-1 {
    margin-right: 0.25rem !important;
  }
  .me-md-2 {
    margin-right: 0.5rem !important;
  }
  .me-md-3 {
    margin-right: 1rem !important;
  }
  .me-md-4 {
    margin-right: 1.5rem !important;
  }
  .me-md-5 {
    margin-right: 2rem !important;
  }
  .me-md-6 {
    margin-right: 3rem !important;
  }
  .me-md-auto {
    margin-right: auto !important;
  }
  .mb-md-0 {
    margin-bottom: 0 !important;
  }
  .mb-md-1 {
    margin-bottom: 0.25rem !important;
  }
  .mb-md-2 {
    margin-bottom: 0.5rem !important;
  }
  .mb-md-3 {
    margin-bottom: 1rem !important;
  }
  .mb-md-4 {
    margin-bottom: 1.5rem !important;
  }
  .mb-md-5 {
    margin-bottom: 2rem !important;
  }
  .mb-md-6 {
    margin-bottom: 3rem !important;
  }
  .mb-md-auto {
    margin-bottom: auto !important;
  }
  .ms-md-0 {
    margin-left: 0 !important;
  }
  .ms-md-1 {
    margin-left: 0.25rem !important;
  }
  .ms-md-2 {
    margin-left: 0.5rem !important;
  }
  .ms-md-3 {
    margin-left: 1rem !important;
  }
  .ms-md-4 {
    margin-left: 1.5rem !important;
  }
  .ms-md-5 {
    margin-left: 2rem !important;
  }
  .ms-md-6 {
    margin-left: 3rem !important;
  }
  .ms-md-auto {
    margin-left: auto !important;
  }
  .p-md-0 {
    padding: 0 !important;
  }
  .p-md-1 {
    padding: 0.25rem !important;
  }
  .p-md-2 {
    padding: 0.5rem !important;
  }
  .p-md-3 {
    padding: 1rem !important;
  }
  .p-md-4 {
    padding: 1.5rem !important;
  }
  .p-md-5 {
    padding: 2rem !important;
  }
  .p-md-6 {
    padding: 3rem !important;
  }
  .px-md-0 {
    padding-right: 0 !important;
    padding-left: 0 !important;
  }
  .px-md-1 {
    padding-right: 0.25rem !important;
    padding-left: 0.25rem !important;
  }
  .px-md-2 {
    padding-right: 0.5rem !important;
    padding-left: 0.5rem !important;
  }
  .px-md-3 {
    padding-right: 1rem !important;
    padding-left: 1rem !important;
  }
  .px-md-4 {
    padding-right: 1.5rem !important;
    padding-left: 1.5rem !important;
  }
  .px-md-5 {
    padding-right: 2rem !important;
    padding-left: 2rem !important;
  }
  .px-md-6 {
    padding-right: 3rem !important;
    padding-left: 3rem !important;
  }
  .py-md-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }
  .py-md-1 {
    padding-top: 0.25rem !important;
    padding-bottom: 0.25rem !important;
  }
  .py-md-2 {
    padding-top: 0.5rem !important;
    padding-bottom: 0.5rem !important;
  }
  .py-md-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }
  .py-md-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }
  .py-md-5 {
    padding-top: 2rem !important;
    padding-bottom: 2rem !important;
  }
  .py-md-6 {
    padding-top: 3rem !important;
    padding-bottom: 3rem !important;
  }
  .pt-md-0 {
    padding-top: 0 !important;
  }
  .pt-md-1 {
    padding-top: 0.25rem !important;
  }
  .pt-md-2 {
    padding-top: 0.5rem !important;
  }
  .pt-md-3 {
    padding-top: 1rem !important;
  }
  .pt-md-4 {
    padding-top: 1.5rem !important;
  }
  .pt-md-5 {
    padding-top: 2rem !important;
  }
  .pt-md-6 {
    padding-top: 3rem !important;
  }
  .pe-md-0 {
    padding-right: 0 !important;
  }
  .pe-md-1 {
    padding-right: 0.25rem !important;
  }
  .pe-md-2 {
    padding-right: 0.5rem !important;
  }
  .pe-md-3 {
    padding-right: 1rem !important;
  }
  .pe-md-4 {
    padding-right: 1.5rem !important;
  }
  .pe-md-5 {
    padding-right: 2rem !important;
  }
  .pe-md-6 {
    padding-right: 3rem !important;
  }
  .pb-md-0 {
    padding-bottom: 0 !important;
  }
  .pb-md-1 {
    padding-bottom: 0.25rem !important;
  }
  .pb-md-2 {
    padding-bottom: 0.5rem !important;
  }
  .pb-md-3 {
    padding-bottom: 1rem !important;
  }
  .pb-md-4 {
    padding-bottom: 1.5rem !important;
  }
  .pb-md-5 {
    padding-bottom: 2rem !important;
  }
  .pb-md-6 {
    padding-bottom: 3rem !important;
  }
  .ps-md-0 {
    padding-left: 0 !important;
  }
  .ps-md-1 {
    padding-left: 0.25rem !important;
  }
  .ps-md-2 {
    padding-left: 0.5rem !important;
  }
  .ps-md-3 {
    padding-left: 1rem !important;
  }
  .ps-md-4 {
    padding-left: 1.5rem !important;
  }
  .ps-md-5 {
    padding-left: 2rem !important;
  }
  .ps-md-6 {
    padding-left: 3rem !important;
  }
  .gap-md-0 {
    gap: 0 !important;
  }
  .gap-md-1 {
    gap: 0.25rem !important;
  }
  .gap-md-2 {
    gap: 0.5rem !important;
  }
  .gap-md-3 {
    gap: 1rem !important;
  }
  .gap-md-4 {
    gap: 1.5rem !important;
  }
  .gap-md-5 {
    gap: 2rem !important;
  }
  .gap-md-6 {
    gap: 3rem !important;
  }
  .row-gap-md-0 {
    row-gap: 0 !important;
  }
  .row-gap-md-1 {
    row-gap: 0.25rem !important;
  }
  .row-gap-md-2 {
    row-gap: 0.5rem !important;
  }
  .row-gap-md-3 {
    row-gap: 1rem !important;
  }
  .row-gap-md-4 {
    row-gap: 1.5rem !important;
  }
  .row-gap-md-5 {
    row-gap: 2rem !important;
  }
  .row-gap-md-6 {
    row-gap: 3rem !important;
  }
  .column-gap-md-0 {
    column-gap: 0 !important;
  }
  .column-gap-md-1 {
    column-gap: 0.25rem !important;
  }
  .column-gap-md-2 {
    column-gap: 0.5rem !important;
  }
  .column-gap-md-3 {
    column-gap: 1rem !important;
  }
  .column-gap-md-4 {
    column-gap: 1.5rem !important;
  }
  .column-gap-md-5 {
    column-gap: 2rem !important;
  }
  .column-gap-md-6 {
    column-gap: 3rem !important;
  }
  .text-md-start {
    text-align: left !important;
  }
  .text-md-end {
    text-align: right !important;
  }
  .text-md-center {
    text-align: center !important;
  }
}
@media (min-width: 992px) {
  .float-lg-start {
    float: left !important;
  }
  .float-lg-end {
    float: right !important;
  }
  .float-lg-none {
    float: none !important;
  }
  .object-fit-lg-contain {
    object-fit: contain !important;
  }
  .object-fit-lg-cover {
    object-fit: cover !important;
  }
  .object-fit-lg-fill {
    object-fit: fill !important;
  }
  .object-fit-lg-scale {
    object-fit: scale-down !important;
  }
  .object-fit-lg-none {
    object-fit: none !important;
  }
  .d-lg-inline {
    display: inline !important;
  }
  .d-lg-inline-block {
    display: inline-block !important;
  }
  .d-lg-block {
    display: block !important;
  }
  .d-lg-grid {
    display: grid !important;
  }
  .d-lg-inline-grid {
    display: inline-grid !important;
  }
  .d-lg-table {
    display: table !important;
  }
  .d-lg-table-row {
    display: table-row !important;
  }
  .d-lg-table-cell {
    display: table-cell !important;
  }
  .d-lg-flex {
    display: flex !important;
  }
  .d-lg-inline-flex {
    display: inline-flex !important;
  }
  .d-lg-none {
    display: none !important;
  }
  .flex-lg-fill {
    flex: 1 1 auto !important;
  }
  .flex-lg-row {
    flex-direction: row !important;
  }
  .flex-lg-column {
    flex-direction: column !important;
  }
  .flex-lg-row-reverse {
    flex-direction: row-reverse !important;
  }
  .flex-lg-column-reverse {
    flex-direction: column-reverse !important;
  }
  .flex-lg-grow-0 {
    flex-grow: 0 !important;
  }
  .flex-lg-grow-1 {
    flex-grow: 1 !important;
  }
  .flex-lg-shrink-0 {
    flex-shrink: 0 !important;
  }
  .flex-lg-shrink-1 {
    flex-shrink: 1 !important;
  }
  .flex-lg-wrap {
    flex-wrap: wrap !important;
  }
  .flex-lg-nowrap {
    flex-wrap: nowrap !important;
  }
  .flex-lg-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }
  .justify-content-lg-start {
    justify-content: flex-start !important;
  }
  .justify-content-lg-end {
    justify-content: flex-end !important;
  }
  .justify-content-lg-center {
    justify-content: center !important;
  }
  .justify-content-lg-between {
    justify-content: space-between !important;
  }
  .justify-content-lg-around {
    justify-content: space-around !important;
  }
  .justify-content-lg-evenly {
    justify-content: space-evenly !important;
  }
  .align-items-lg-start {
    align-items: flex-start !important;
  }
  .align-items-lg-end {
    align-items: flex-end !important;
  }
  .align-items-lg-center {
    align-items: center !important;
  }
  .align-items-lg-baseline {
    align-items: baseline !important;
  }
  .align-items-lg-stretch {
    align-items: stretch !important;
  }
  .align-content-lg-start {
    align-content: flex-start !important;
  }
  .align-content-lg-end {
    align-content: flex-end !important;
  }
  .align-content-lg-center {
    align-content: center !important;
  }
  .align-content-lg-between {
    align-content: space-between !important;
  }
  .align-content-lg-around {
    align-content: space-around !important;
  }
  .align-content-lg-stretch {
    align-content: stretch !important;
  }
  .align-self-lg-auto {
    align-self: auto !important;
  }
  .align-self-lg-start {
    align-self: flex-start !important;
  }
  .align-self-lg-end {
    align-self: flex-end !important;
  }
  .align-self-lg-center {
    align-self: center !important;
  }
  .align-self-lg-baseline {
    align-self: baseline !important;
  }
  .align-self-lg-stretch {
    align-self: stretch !important;
  }
  .order-lg-first {
    order: -1 !important;
  }
  .order-lg-0 {
    order: 0 !important;
  }
  .order-lg-1 {
    order: 1 !important;
  }
  .order-lg-2 {
    order: 2 !important;
  }
  .order-lg-3 {
    order: 3 !important;
  }
  .order-lg-4 {
    order: 4 !important;
  }
  .order-lg-5 {
    order: 5 !important;
  }
  .order-lg-last {
    order: 6 !important;
  }
  .m-lg-0 {
    margin: 0 !important;
  }
  .m-lg-1 {
    margin: 0.25rem !important;
  }
  .m-lg-2 {
    margin: 0.5rem !important;
  }
  .m-lg-3 {
    margin: 1rem !important;
  }
  .m-lg-4 {
    margin: 1.5rem !important;
  }
  .m-lg-5 {
    margin: 2rem !important;
  }
  .m-lg-6 {
    margin: 3rem !important;
  }
  .m-lg-auto {
    margin: auto !important;
  }
  .mx-lg-0 {
    margin-right: 0 !important;
    margin-left: 0 !important;
  }
  .mx-lg-1 {
    margin-right: 0.25rem !important;
    margin-left: 0.25rem !important;
  }
  .mx-lg-2 {
    margin-right: 0.5rem !important;
    margin-left: 0.5rem !important;
  }
  .mx-lg-3 {
    margin-right: 1rem !important;
    margin-left: 1rem !important;
  }
  .mx-lg-4 {
    margin-right: 1.5rem !important;
    margin-left: 1.5rem !important;
  }
  .mx-lg-5 {
    margin-right: 2rem !important;
    margin-left: 2rem !important;
  }
  .mx-lg-6 {
    margin-right: 3rem !important;
    margin-left: 3rem !important;
  }
  .mx-lg-auto {
    margin-right: auto !important;
    margin-left: auto !important;
  }
  .my-lg-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }
  .my-lg-1 {
    margin-top: 0.25rem !important;
    margin-bottom: 0.25rem !important;
  }
  .my-lg-2 {
    margin-top: 0.5rem !important;
    margin-bottom: 0.5rem !important;
  }
  .my-lg-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }
  .my-lg-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }
  .my-lg-5 {
    margin-top: 2rem !important;
    margin-bottom: 2rem !important;
  }
  .my-lg-6 {
    margin-top: 3rem !important;
    margin-bottom: 3rem !important;
  }
  .my-lg-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }
  .mt-lg-0 {
    margin-top: 0 !important;
  }
  .mt-lg-1 {
    margin-top: 0.25rem !important;
  }
  .mt-lg-2 {
    margin-top: 0.5rem !important;
  }
  .mt-lg-3 {
    margin-top: 1rem !important;
  }
  .mt-lg-4 {
    margin-top: 1.5rem !important;
  }
  .mt-lg-5 {
    margin-top: 2rem !important;
  }
  .mt-lg-6 {
    margin-top: 3rem !important;
  }
  .mt-lg-auto {
    margin-top: auto !important;
  }
  .me-lg-0 {
    margin-right: 0 !important;
  }
  .me-lg-1 {
    margin-right: 0.25rem !important;
  }
  .me-lg-2 {
    margin-right: 0.5rem !important;
  }
  .me-lg-3 {
    margin-right: 1rem !important;
  }
  .me-lg-4 {
    margin-right: 1.5rem !important;
  }
  .me-lg-5 {
    margin-right: 2rem !important;
  }
  .me-lg-6 {
    margin-right: 3rem !important;
  }
  .me-lg-auto {
    margin-right: auto !important;
  }
  .mb-lg-0 {
    margin-bottom: 0 !important;
  }
  .mb-lg-1 {
    margin-bottom: 0.25rem !important;
  }
  .mb-lg-2 {
    margin-bottom: 0.5rem !important;
  }
  .mb-lg-3 {
    margin-bottom: 1rem !important;
  }
  .mb-lg-4 {
    margin-bottom: 1.5rem !important;
  }
  .mb-lg-5 {
    margin-bottom: 2rem !important;
  }
  .mb-lg-6 {
    margin-bottom: 3rem !important;
  }
  .mb-lg-auto {
    margin-bottom: auto !important;
  }
  .ms-lg-0 {
    margin-left: 0 !important;
  }
  .ms-lg-1 {
    margin-left: 0.25rem !important;
  }
  .ms-lg-2 {
    margin-left: 0.5rem !important;
  }
  .ms-lg-3 {
    margin-left: 1rem !important;
  }
  .ms-lg-4 {
    margin-left: 1.5rem !important;
  }
  .ms-lg-5 {
    margin-left: 2rem !important;
  }
  .ms-lg-6 {
    margin-left: 3rem !important;
  }
  .ms-lg-auto {
    margin-left: auto !important;
  }
  .p-lg-0 {
    padding: 0 !important;
  }
  .p-lg-1 {
    padding: 0.25rem !important;
  }
  .p-lg-2 {
    padding: 0.5rem !important;
  }
  .p-lg-3 {
    padding: 1rem !important;
  }
  .p-lg-4 {
    padding: 1.5rem !important;
  }
  .p-lg-5 {
    padding: 2rem !important;
  }
  .p-lg-6 {
    padding: 3rem !important;
  }
  .px-lg-0 {
    padding-right: 0 !important;
    padding-left: 0 !important;
  }
  .px-lg-1 {
    padding-right: 0.25rem !important;
    padding-left: 0.25rem !important;
  }
  .px-lg-2 {
    padding-right: 0.5rem !important;
    padding-left: 0.5rem !important;
  }
  .px-lg-3 {
    padding-right: 1rem !important;
    padding-left: 1rem !important;
  }
  .px-lg-4 {
    padding-right: 1.5rem !important;
    padding-left: 1.5rem !important;
  }
  .px-lg-5 {
    padding-right: 2rem !important;
    padding-left: 2rem !important;
  }
  .px-lg-6 {
    padding-right: 3rem !important;
    padding-left: 3rem !important;
  }
  .py-lg-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }
  .py-lg-1 {
    padding-top: 0.25rem !important;
    padding-bottom: 0.25rem !important;
  }
  .py-lg-2 {
    padding-top: 0.5rem !important;
    padding-bottom: 0.5rem !important;
  }
  .py-lg-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }
  .py-lg-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }
  .py-lg-5 {
    padding-top: 2rem !important;
    padding-bottom: 2rem !important;
  }
  .py-lg-6 {
    padding-top: 3rem !important;
    padding-bottom: 3rem !important;
  }
  .pt-lg-0 {
    padding-top: 0 !important;
  }
  .pt-lg-1 {
    padding-top: 0.25rem !important;
  }
  .pt-lg-2 {
    padding-top: 0.5rem !important;
  }
  .pt-lg-3 {
    padding-top: 1rem !important;
  }
  .pt-lg-4 {
    padding-top: 1.5rem !important;
  }
  .pt-lg-5 {
    padding-top: 2rem !important;
  }
  .pt-lg-6 {
    padding-top: 3rem !important;
  }
  .pe-lg-0 {
    padding-right: 0 !important;
  }
  .pe-lg-1 {
    padding-right: 0.25rem !important;
  }
  .pe-lg-2 {
    padding-right: 0.5rem !important;
  }
  .pe-lg-3 {
    padding-right: 1rem !important;
  }
  .pe-lg-4 {
    padding-right: 1.5rem !important;
  }
  .pe-lg-5 {
    padding-right: 2rem !important;
  }
  .pe-lg-6 {
    padding-right: 3rem !important;
  }
  .pb-lg-0 {
    padding-bottom: 0 !important;
  }
  .pb-lg-1 {
    padding-bottom: 0.25rem !important;
  }
  .pb-lg-2 {
    padding-bottom: 0.5rem !important;
  }
  .pb-lg-3 {
    padding-bottom: 1rem !important;
  }
  .pb-lg-4 {
    padding-bottom: 1.5rem !important;
  }
  .pb-lg-5 {
    padding-bottom: 2rem !important;
  }
  .pb-lg-6 {
    padding-bottom: 3rem !important;
  }
  .ps-lg-0 {
    padding-left: 0 !important;
  }
  .ps-lg-1 {
    padding-left: 0.25rem !important;
  }
  .ps-lg-2 {
    padding-left: 0.5rem !important;
  }
  .ps-lg-3 {
    padding-left: 1rem !important;
  }
  .ps-lg-4 {
    padding-left: 1.5rem !important;
  }
  .ps-lg-5 {
    padding-left: 2rem !important;
  }
  .ps-lg-6 {
    padding-left: 3rem !important;
  }
  .gap-lg-0 {
    gap: 0 !important;
  }
  .gap-lg-1 {
    gap: 0.25rem !important;
  }
  .gap-lg-2 {
    gap: 0.5rem !important;
  }
  .gap-lg-3 {
    gap: 1rem !important;
  }
  .gap-lg-4 {
    gap: 1.5rem !important;
  }
  .gap-lg-5 {
    gap: 2rem !important;
  }
  .gap-lg-6 {
    gap: 3rem !important;
  }
  .row-gap-lg-0 {
    row-gap: 0 !important;
  }
  .row-gap-lg-1 {
    row-gap: 0.25rem !important;
  }
  .row-gap-lg-2 {
    row-gap: 0.5rem !important;
  }
  .row-gap-lg-3 {
    row-gap: 1rem !important;
  }
  .row-gap-lg-4 {
    row-gap: 1.5rem !important;
  }
  .row-gap-lg-5 {
    row-gap: 2rem !important;
  }
  .row-gap-lg-6 {
    row-gap: 3rem !important;
  }
  .column-gap-lg-0 {
    column-gap: 0 !important;
  }
  .column-gap-lg-1 {
    column-gap: 0.25rem !important;
  }
  .column-gap-lg-2 {
    column-gap: 0.5rem !important;
  }
  .column-gap-lg-3 {
    column-gap: 1rem !important;
  }
  .column-gap-lg-4 {
    column-gap: 1.5rem !important;
  }
  .column-gap-lg-5 {
    column-gap: 2rem !important;
  }
  .column-gap-lg-6 {
    column-gap: 3rem !important;
  }
  .text-lg-start {
    text-align: left !important;
  }
  .text-lg-end {
    text-align: right !important;
  }
  .text-lg-center {
    text-align: center !important;
  }
}
@media (min-width: 1200px) {
  .float-xl-start {
    float: left !important;
  }
  .float-xl-end {
    float: right !important;
  }
  .float-xl-none {
    float: none !important;
  }
  .object-fit-xl-contain {
    object-fit: contain !important;
  }
  .object-fit-xl-cover {
    object-fit: cover !important;
  }
  .object-fit-xl-fill {
    object-fit: fill !important;
  }
  .object-fit-xl-scale {
    object-fit: scale-down !important;
  }
  .object-fit-xl-none {
    object-fit: none !important;
  }
  .d-xl-inline {
    display: inline !important;
  }
  .d-xl-inline-block {
    display: inline-block !important;
  }
  .d-xl-block {
    display: block !important;
  }
  .d-xl-grid {
    display: grid !important;
  }
  .d-xl-inline-grid {
    display: inline-grid !important;
  }
  .d-xl-table {
    display: table !important;
  }
  .d-xl-table-row {
    display: table-row !important;
  }
  .d-xl-table-cell {
    display: table-cell !important;
  }
  .d-xl-flex {
    display: flex !important;
  }
  .d-xl-inline-flex {
    display: inline-flex !important;
  }
  .d-xl-none {
    display: none !important;
  }
  .flex-xl-fill {
    flex: 1 1 auto !important;
  }
  .flex-xl-row {
    flex-direction: row !important;
  }
  .flex-xl-column {
    flex-direction: column !important;
  }
  .flex-xl-row-reverse {
    flex-direction: row-reverse !important;
  }
  .flex-xl-column-reverse {
    flex-direction: column-reverse !important;
  }
  .flex-xl-grow-0 {
    flex-grow: 0 !important;
  }
  .flex-xl-grow-1 {
    flex-grow: 1 !important;
  }
  .flex-xl-shrink-0 {
    flex-shrink: 0 !important;
  }
  .flex-xl-shrink-1 {
    flex-shrink: 1 !important;
  }
  .flex-xl-wrap {
    flex-wrap: wrap !important;
  }
  .flex-xl-nowrap {
    flex-wrap: nowrap !important;
  }
  .flex-xl-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }
  .justify-content-xl-start {
    justify-content: flex-start !important;
  }
  .justify-content-xl-end {
    justify-content: flex-end !important;
  }
  .justify-content-xl-center {
    justify-content: center !important;
  }
  .justify-content-xl-between {
    justify-content: space-between !important;
  }
  .justify-content-xl-around {
    justify-content: space-around !important;
  }
  .justify-content-xl-evenly {
    justify-content: space-evenly !important;
  }
  .align-items-xl-start {
    align-items: flex-start !important;
  }
  .align-items-xl-end {
    align-items: flex-end !important;
  }
  .align-items-xl-center {
    align-items: center !important;
  }
  .align-items-xl-baseline {
    align-items: baseline !important;
  }
  .align-items-xl-stretch {
    align-items: stretch !important;
  }
  .align-content-xl-start {
    align-content: flex-start !important;
  }
  .align-content-xl-end {
    align-content: flex-end !important;
  }
  .align-content-xl-center {
    align-content: center !important;
  }
  .align-content-xl-between {
    align-content: space-between !important;
  }
  .align-content-xl-around {
    align-content: space-around !important;
  }
  .align-content-xl-stretch {
    align-content: stretch !important;
  }
  .align-self-xl-auto {
    align-self: auto !important;
  }
  .align-self-xl-start {
    align-self: flex-start !important;
  }
  .align-self-xl-end {
    align-self: flex-end !important;
  }
  .align-self-xl-center {
    align-self: center !important;
  }
  .align-self-xl-baseline {
    align-self: baseline !important;
  }
  .align-self-xl-stretch {
    align-self: stretch !important;
  }
  .order-xl-first {
    order: -1 !important;
  }
  .order-xl-0 {
    order: 0 !important;
  }
  .order-xl-1 {
    order: 1 !important;
  }
  .order-xl-2 {
    order: 2 !important;
  }
  .order-xl-3 {
    order: 3 !important;
  }
  .order-xl-4 {
    order: 4 !important;
  }
  .order-xl-5 {
    order: 5 !important;
  }
  .order-xl-last {
    order: 6 !important;
  }
  .m-xl-0 {
    margin: 0 !important;
  }
  .m-xl-1 {
    margin: 0.25rem !important;
  }
  .m-xl-2 {
    margin: 0.5rem !important;
  }
  .m-xl-3 {
    margin: 1rem !important;
  }
  .m-xl-4 {
    margin: 1.5rem !important;
  }
  .m-xl-5 {
    margin: 2rem !important;
  }
  .m-xl-6 {
    margin: 3rem !important;
  }
  .m-xl-auto {
    margin: auto !important;
  }
  .mx-xl-0 {
    margin-right: 0 !important;
    margin-left: 0 !important;
  }
  .mx-xl-1 {
    margin-right: 0.25rem !important;
    margin-left: 0.25rem !important;
  }
  .mx-xl-2 {
    margin-right: 0.5rem !important;
    margin-left: 0.5rem !important;
  }
  .mx-xl-3 {
    margin-right: 1rem !important;
    margin-left: 1rem !important;
  }
  .mx-xl-4 {
    margin-right: 1.5rem !important;
    margin-left: 1.5rem !important;
  }
  .mx-xl-5 {
    margin-right: 2rem !important;
    margin-left: 2rem !important;
  }
  .mx-xl-6 {
    margin-right: 3rem !important;
    margin-left: 3rem !important;
  }
  .mx-xl-auto {
    margin-right: auto !important;
    margin-left: auto !important;
  }
  .my-xl-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }
  .my-xl-1 {
    margin-top: 0.25rem !important;
    margin-bottom: 0.25rem !important;
  }
  .my-xl-2 {
    margin-top: 0.5rem !important;
    margin-bottom: 0.5rem !important;
  }
  .my-xl-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }
  .my-xl-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }
  .my-xl-5 {
    margin-top: 2rem !important;
    margin-bottom: 2rem !important;
  }
  .my-xl-6 {
    margin-top: 3rem !important;
    margin-bottom: 3rem !important;
  }
  .my-xl-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }
  .mt-xl-0 {
    margin-top: 0 !important;
  }
  .mt-xl-1 {
    margin-top: 0.25rem !important;
  }
  .mt-xl-2 {
    margin-top: 0.5rem !important;
  }
  .mt-xl-3 {
    margin-top: 1rem !important;
  }
  .mt-xl-4 {
    margin-top: 1.5rem !important;
  }
  .mt-xl-5 {
    margin-top: 2rem !important;
  }
  .mt-xl-6 {
    margin-top: 3rem !important;
  }
  .mt-xl-auto {
    margin-top: auto !important;
  }
  .me-xl-0 {
    margin-right: 0 !important;
  }
  .me-xl-1 {
    margin-right: 0.25rem !important;
  }
  .me-xl-2 {
    margin-right: 0.5rem !important;
  }
  .me-xl-3 {
    margin-right: 1rem !important;
  }
  .me-xl-4 {
    margin-right: 1.5rem !important;
  }
  .me-xl-5 {
    margin-right: 2rem !important;
  }
  .me-xl-6 {
    margin-right: 3rem !important;
  }
  .me-xl-auto {
    margin-right: auto !important;
  }
  .mb-xl-0 {
    margin-bottom: 0 !important;
  }
  .mb-xl-1 {
    margin-bottom: 0.25rem !important;
  }
  .mb-xl-2 {
    margin-bottom: 0.5rem !important;
  }
  .mb-xl-3 {
    margin-bottom: 1rem !important;
  }
  .mb-xl-4 {
    margin-bottom: 1.5rem !important;
  }
  .mb-xl-5 {
    margin-bottom: 2rem !important;
  }
  .mb-xl-6 {
    margin-bottom: 3rem !important;
  }
  .mb-xl-auto {
    margin-bottom: auto !important;
  }
  .ms-xl-0 {
    margin-left: 0 !important;
  }
  .ms-xl-1 {
    margin-left: 0.25rem !important;
  }
  .ms-xl-2 {
    margin-left: 0.5rem !important;
  }
  .ms-xl-3 {
    margin-left: 1rem !important;
  }
  .ms-xl-4 {
    margin-left: 1.5rem !important;
  }
  .ms-xl-5 {
    margin-left: 2rem !important;
  }
  .ms-xl-6 {
    margin-left: 3rem !important;
  }
  .ms-xl-auto {
    margin-left: auto !important;
  }
  .p-xl-0 {
    padding: 0 !important;
  }
  .p-xl-1 {
    padding: 0.25rem !important;
  }
  .p-xl-2 {
    padding: 0.5rem !important;
  }
  .p-xl-3 {
    padding: 1rem !important;
  }
  .p-xl-4 {
    padding: 1.5rem !important;
  }
  .p-xl-5 {
    padding: 2rem !important;
  }
  .p-xl-6 {
    padding: 3rem !important;
  }
  .px-xl-0 {
    padding-right: 0 !important;
    padding-left: 0 !important;
  }
  .px-xl-1 {
    padding-right: 0.25rem !important;
    padding-left: 0.25rem !important;
  }
  .px-xl-2 {
    padding-right: 0.5rem !important;
    padding-left: 0.5rem !important;
  }
  .px-xl-3 {
    padding-right: 1rem !important;
    padding-left: 1rem !important;
  }
  .px-xl-4 {
    padding-right: 1.5rem !important;
    padding-left: 1.5rem !important;
  }
  .px-xl-5 {
    padding-right: 2rem !important;
    padding-left: 2rem !important;
  }
  .px-xl-6 {
    padding-right: 3rem !important;
    padding-left: 3rem !important;
  }
  .py-xl-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }
  .py-xl-1 {
    padding-top: 0.25rem !important;
    padding-bottom: 0.25rem !important;
  }
  .py-xl-2 {
    padding-top: 0.5rem !important;
    padding-bottom: 0.5rem !important;
  }
  .py-xl-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }
  .py-xl-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }
  .py-xl-5 {
    padding-top: 2rem !important;
    padding-bottom: 2rem !important;
  }
  .py-xl-6 {
    padding-top: 3rem !important;
    padding-bottom: 3rem !important;
  }
  .pt-xl-0 {
    padding-top: 0 !important;
  }
  .pt-xl-1 {
    padding-top: 0.25rem !important;
  }
  .pt-xl-2 {
    padding-top: 0.5rem !important;
  }
  .pt-xl-3 {
    padding-top: 1rem !important;
  }
  .pt-xl-4 {
    padding-top: 1.5rem !important;
  }
  .pt-xl-5 {
    padding-top: 2rem !important;
  }
  .pt-xl-6 {
    padding-top: 3rem !important;
  }
  .pe-xl-0 {
    padding-right: 0 !important;
  }
  .pe-xl-1 {
    padding-right: 0.25rem !important;
  }
  .pe-xl-2 {
    padding-right: 0.5rem !important;
  }
  .pe-xl-3 {
    padding-right: 1rem !important;
  }
  .pe-xl-4 {
    padding-right: 1.5rem !important;
  }
  .pe-xl-5 {
    padding-right: 2rem !important;
  }
  .pe-xl-6 {
    padding-right: 3rem !important;
  }
  .pb-xl-0 {
    padding-bottom: 0 !important;
  }
  .pb-xl-1 {
    padding-bottom: 0.25rem !important;
  }
  .pb-xl-2 {
    padding-bottom: 0.5rem !important;
  }
  .pb-xl-3 {
    padding-bottom: 1rem !important;
  }
  .pb-xl-4 {
    padding-bottom: 1.5rem !important;
  }
  .pb-xl-5 {
    padding-bottom: 2rem !important;
  }
  .pb-xl-6 {
    padding-bottom: 3rem !important;
  }
  .ps-xl-0 {
    padding-left: 0 !important;
  }
  .ps-xl-1 {
    padding-left: 0.25rem !important;
  }
  .ps-xl-2 {
    padding-left: 0.5rem !important;
  }
  .ps-xl-3 {
    padding-left: 1rem !important;
  }
  .ps-xl-4 {
    padding-left: 1.5rem !important;
  }
  .ps-xl-5 {
    padding-left: 2rem !important;
  }
  .ps-xl-6 {
    padding-left: 3rem !important;
  }
  .gap-xl-0 {
    gap: 0 !important;
  }
  .gap-xl-1 {
    gap: 0.25rem !important;
  }
  .gap-xl-2 {
    gap: 0.5rem !important;
  }
  .gap-xl-3 {
    gap: 1rem !important;
  }
  .gap-xl-4 {
    gap: 1.5rem !important;
  }
  .gap-xl-5 {
    gap: 2rem !important;
  }
  .gap-xl-6 {
    gap: 3rem !important;
  }
  .row-gap-xl-0 {
    row-gap: 0 !important;
  }
  .row-gap-xl-1 {
    row-gap: 0.25rem !important;
  }
  .row-gap-xl-2 {
    row-gap: 0.5rem !important;
  }
  .row-gap-xl-3 {
    row-gap: 1rem !important;
  }
  .row-gap-xl-4 {
    row-gap: 1.5rem !important;
  }
  .row-gap-xl-5 {
    row-gap: 2rem !important;
  }
  .row-gap-xl-6 {
    row-gap: 3rem !important;
  }
  .column-gap-xl-0 {
    column-gap: 0 !important;
  }
  .column-gap-xl-1 {
    column-gap: 0.25rem !important;
  }
  .column-gap-xl-2 {
    column-gap: 0.5rem !important;
  }
  .column-gap-xl-3 {
    column-gap: 1rem !important;
  }
  .column-gap-xl-4 {
    column-gap: 1.5rem !important;
  }
  .column-gap-xl-5 {
    column-gap: 2rem !important;
  }
  .column-gap-xl-6 {
    column-gap: 3rem !important;
  }
  .text-xl-start {
    text-align: left !important;
  }
  .text-xl-end {
    text-align: right !important;
  }
  .text-xl-center {
    text-align: center !important;
  }
}
@media (min-width: 1400px) {
  .float-xxl-start {
    float: left !important;
  }
  .float-xxl-end {
    float: right !important;
  }
  .float-xxl-none {
    float: none !important;
  }
  .object-fit-xxl-contain {
    object-fit: contain !important;
  }
  .object-fit-xxl-cover {
    object-fit: cover !important;
  }
  .object-fit-xxl-fill {
    object-fit: fill !important;
  }
  .object-fit-xxl-scale {
    object-fit: scale-down !important;
  }
  .object-fit-xxl-none {
    object-fit: none !important;
  }
  .d-xxl-inline {
    display: inline !important;
  }
  .d-xxl-inline-block {
    display: inline-block !important;
  }
  .d-xxl-block {
    display: block !important;
  }
  .d-xxl-grid {
    display: grid !important;
  }
  .d-xxl-inline-grid {
    display: inline-grid !important;
  }
  .d-xxl-table {
    display: table !important;
  }
  .d-xxl-table-row {
    display: table-row !important;
  }
  .d-xxl-table-cell {
    display: table-cell !important;
  }
  .d-xxl-flex {
    display: flex !important;
  }
  .d-xxl-inline-flex {
    display: inline-flex !important;
  }
  .d-xxl-none {
    display: none !important;
  }
  .flex-xxl-fill {
    flex: 1 1 auto !important;
  }
  .flex-xxl-row {
    flex-direction: row !important;
  }
  .flex-xxl-column {
    flex-direction: column !important;
  }
  .flex-xxl-row-reverse {
    flex-direction: row-reverse !important;
  }
  .flex-xxl-column-reverse {
    flex-direction: column-reverse !important;
  }
  .flex-xxl-grow-0 {
    flex-grow: 0 !important;
  }
  .flex-xxl-grow-1 {
    flex-grow: 1 !important;
  }
  .flex-xxl-shrink-0 {
    flex-shrink: 0 !important;
  }
  .flex-xxl-shrink-1 {
    flex-shrink: 1 !important;
  }
  .flex-xxl-wrap {
    flex-wrap: wrap !important;
  }
  .flex-xxl-nowrap {
    flex-wrap: nowrap !important;
  }
  .flex-xxl-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }
  .justify-content-xxl-start {
    justify-content: flex-start !important;
  }
  .justify-content-xxl-end {
    justify-content: flex-end !important;
  }
  .justify-content-xxl-center {
    justify-content: center !important;
  }
  .justify-content-xxl-between {
    justify-content: space-between !important;
  }
  .justify-content-xxl-around {
    justify-content: space-around !important;
  }
  .justify-content-xxl-evenly {
    justify-content: space-evenly !important;
  }
  .align-items-xxl-start {
    align-items: flex-start !important;
  }
  .align-items-xxl-end {
    align-items: flex-end !important;
  }
  .align-items-xxl-center {
    align-items: center !important;
  }
  .align-items-xxl-baseline {
    align-items: baseline !important;
  }
  .align-items-xxl-stretch {
    align-items: stretch !important;
  }
  .align-content-xxl-start {
    align-content: flex-start !important;
  }
  .align-content-xxl-end {
    align-content: flex-end !important;
  }
  .align-content-xxl-center {
    align-content: center !important;
  }
  .align-content-xxl-between {
    align-content: space-between !important;
  }
  .align-content-xxl-around {
    align-content: space-around !important;
  }
  .align-content-xxl-stretch {
    align-content: stretch !important;
  }
  .align-self-xxl-auto {
    align-self: auto !important;
  }
  .align-self-xxl-start {
    align-self: flex-start !important;
  }
  .align-self-xxl-end {
    align-self: flex-end !important;
  }
  .align-self-xxl-center {
    align-self: center !important;
  }
  .align-self-xxl-baseline {
    align-self: baseline !important;
  }
  .align-self-xxl-stretch {
    align-self: stretch !important;
  }
  .order-xxl-first {
    order: -1 !important;
  }
  .order-xxl-0 {
    order: 0 !important;
  }
  .order-xxl-1 {
    order: 1 !important;
  }
  .order-xxl-2 {
    order: 2 !important;
  }
  .order-xxl-3 {
    order: 3 !important;
  }
  .order-xxl-4 {
    order: 4 !important;
  }
  .order-xxl-5 {
    order: 5 !important;
  }
  .order-xxl-last {
    order: 6 !important;
  }
  .m-xxl-0 {
    margin: 0 !important;
  }
  .m-xxl-1 {
    margin: 0.25rem !important;
  }
  .m-xxl-2 {
    margin: 0.5rem !important;
  }
  .m-xxl-3 {
    margin: 1rem !important;
  }
  .m-xxl-4 {
    margin: 1.5rem !important;
  }
  .m-xxl-5 {
    margin: 2rem !important;
  }
  .m-xxl-6 {
    margin: 3rem !important;
  }
  .m-xxl-auto {
    margin: auto !important;
  }
  .mx-xxl-0 {
    margin-right: 0 !important;
    margin-left: 0 !important;
  }
  .mx-xxl-1 {
    margin-right: 0.25rem !important;
    margin-left: 0.25rem !important;
  }
  .mx-xxl-2 {
    margin-right: 0.5rem !important;
    margin-left: 0.5rem !important;
  }
  .mx-xxl-3 {
    margin-right: 1rem !important;
    margin-left: 1rem !important;
  }
  .mx-xxl-4 {
    margin-right: 1.5rem !important;
    margin-left: 1.5rem !important;
  }
  .mx-xxl-5 {
    margin-right: 2rem !important;
    margin-left: 2rem !important;
  }
  .mx-xxl-6 {
    margin-right: 3rem !important;
    margin-left: 3rem !important;
  }
  .mx-xxl-auto {
    margin-right: auto !important;
    margin-left: auto !important;
  }
  .my-xxl-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }
  .my-xxl-1 {
    margin-top: 0.25rem !important;
    margin-bottom: 0.25rem !important;
  }
  .my-xxl-2 {
    margin-top: 0.5rem !important;
    margin-bottom: 0.5rem !important;
  }
  .my-xxl-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }
  .my-xxl-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }
  .my-xxl-5 {
    margin-top: 2rem !important;
    margin-bottom: 2rem !important;
  }
  .my-xxl-6 {
    margin-top: 3rem !important;
    margin-bottom: 3rem !important;
  }
  .my-xxl-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }
  .mt-xxl-0 {
    margin-top: 0 !important;
  }
  .mt-xxl-1 {
    margin-top: 0.25rem !important;
  }
  .mt-xxl-2 {
    margin-top: 0.5rem !important;
  }
  .mt-xxl-3 {
    margin-top: 1rem !important;
  }
  .mt-xxl-4 {
    margin-top: 1.5rem !important;
  }
  .mt-xxl-5 {
    margin-top: 2rem !important;
  }
  .mt-xxl-6 {
    margin-top: 3rem !important;
  }
  .mt-xxl-auto {
    margin-top: auto !important;
  }
  .me-xxl-0 {
    margin-right: 0 !important;
  }
  .me-xxl-1 {
    margin-right: 0.25rem !important;
  }
  .me-xxl-2 {
    margin-right: 0.5rem !important;
  }
  .me-xxl-3 {
    margin-right: 1rem !important;
  }
  .me-xxl-4 {
    margin-right: 1.5rem !important;
  }
  .me-xxl-5 {
    margin-right: 2rem !important;
  }
  .me-xxl-6 {
    margin-right: 3rem !important;
  }
  .me-xxl-auto {
    margin-right: auto !important;
  }
  .mb-xxl-0 {
    margin-bottom: 0 !important;
  }
  .mb-xxl-1 {
    margin-bottom: 0.25rem !important;
  }
  .mb-xxl-2 {
    margin-bottom: 0.5rem !important;
  }
  .mb-xxl-3 {
    margin-bottom: 1rem !important;
  }
  .mb-xxl-4 {
    margin-bottom: 1.5rem !important;
  }
  .mb-xxl-5 {
    margin-bottom: 2rem !important;
  }
  .mb-xxl-6 {
    margin-bottom: 3rem !important;
  }
  .mb-xxl-auto {
    margin-bottom: auto !important;
  }
  .ms-xxl-0 {
    margin-left: 0 !important;
  }
  .ms-xxl-1 {
    margin-left: 0.25rem !important;
  }
  .ms-xxl-2 {
    margin-left: 0.5rem !important;
  }
  .ms-xxl-3 {
    margin-left: 1rem !important;
  }
  .ms-xxl-4 {
    margin-left: 1.5rem !important;
  }
  .ms-xxl-5 {
    margin-left: 2rem !important;
  }
  .ms-xxl-6 {
    margin-left: 3rem !important;
  }
  .ms-xxl-auto {
    margin-left: auto !important;
  }
  .p-xxl-0 {
    padding: 0 !important;
  }
  .p-xxl-1 {
    padding: 0.25rem !important;
  }
  .p-xxl-2 {
    padding: 0.5rem !important;
  }
  .p-xxl-3 {
    padding: 1rem !important;
  }
  .p-xxl-4 {
    padding: 1.5rem !important;
  }
  .p-xxl-5 {
    padding: 2rem !important;
  }
  .p-xxl-6 {
    padding: 3rem !important;
  }
  .px-xxl-0 {
    padding-right: 0 !important;
    padding-left: 0 !important;
  }
  .px-xxl-1 {
    padding-right: 0.25rem !important;
    padding-left: 0.25rem !important;
  }
  .px-xxl-2 {
    padding-right: 0.5rem !important;
    padding-left: 0.5rem !important;
  }
  .px-xxl-3 {
    padding-right: 1rem !important;
    padding-left: 1rem !important;
  }
  .px-xxl-4 {
    padding-right: 1.5rem !important;
    padding-left: 1.5rem !important;
  }
  .px-xxl-5 {
    padding-right: 2rem !important;
    padding-left: 2rem !important;
  }
  .px-xxl-6 {
    padding-right: 3rem !important;
    padding-left: 3rem !important;
  }
  .py-xxl-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }
  .py-xxl-1 {
    padding-top: 0.25rem !important;
    padding-bottom: 0.25rem !important;
  }
  .py-xxl-2 {
    padding-top: 0.5rem !important;
    padding-bottom: 0.5rem !important;
  }
  .py-xxl-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }
  .py-xxl-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }
  .py-xxl-5 {
    padding-top: 2rem !important;
    padding-bottom: 2rem !important;
  }
  .py-xxl-6 {
    padding-top: 3rem !important;
    padding-bottom: 3rem !important;
  }
  .pt-xxl-0 {
    padding-top: 0 !important;
  }
  .pt-xxl-1 {
    padding-top: 0.25rem !important;
  }
  .pt-xxl-2 {
    padding-top: 0.5rem !important;
  }
  .pt-xxl-3 {
    padding-top: 1rem !important;
  }
  .pt-xxl-4 {
    padding-top: 1.5rem !important;
  }
  .pt-xxl-5 {
    padding-top: 2rem !important;
  }
  .pt-xxl-6 {
    padding-top: 3rem !important;
  }
  .pe-xxl-0 {
    padding-right: 0 !important;
  }
  .pe-xxl-1 {
    padding-right: 0.25rem !important;
  }
  .pe-xxl-2 {
    padding-right: 0.5rem !important;
  }
  .pe-xxl-3 {
    padding-right: 1rem !important;
  }
  .pe-xxl-4 {
    padding-right: 1.5rem !important;
  }
  .pe-xxl-5 {
    padding-right: 2rem !important;
  }
  .pe-xxl-6 {
    padding-right: 3rem !important;
  }
  .pb-xxl-0 {
    padding-bottom: 0 !important;
  }
  .pb-xxl-1 {
    padding-bottom: 0.25rem !important;
  }
  .pb-xxl-2 {
    padding-bottom: 0.5rem !important;
  }
  .pb-xxl-3 {
    padding-bottom: 1rem !important;
  }
  .pb-xxl-4 {
    padding-bottom: 1.5rem !important;
  }
  .pb-xxl-5 {
    padding-bottom: 2rem !important;
  }
  .pb-xxl-6 {
    padding-bottom: 3rem !important;
  }
  .ps-xxl-0 {
    padding-left: 0 !important;
  }
  .ps-xxl-1 {
    padding-left: 0.25rem !important;
  }
  .ps-xxl-2 {
    padding-left: 0.5rem !important;
  }
  .ps-xxl-3 {
    padding-left: 1rem !important;
  }
  .ps-xxl-4 {
    padding-left: 1.5rem !important;
  }
  .ps-xxl-5 {
    padding-left: 2rem !important;
  }
  .ps-xxl-6 {
    padding-left: 3rem !important;
  }
  .gap-xxl-0 {
    gap: 0 !important;
  }
  .gap-xxl-1 {
    gap: 0.25rem !important;
  }
  .gap-xxl-2 {
    gap: 0.5rem !important;
  }
  .gap-xxl-3 {
    gap: 1rem !important;
  }
  .gap-xxl-4 {
    gap: 1.5rem !important;
  }
  .gap-xxl-5 {
    gap: 2rem !important;
  }
  .gap-xxl-6 {
    gap: 3rem !important;
  }
  .row-gap-xxl-0 {
    row-gap: 0 !important;
  }
  .row-gap-xxl-1 {
    row-gap: 0.25rem !important;
  }
  .row-gap-xxl-2 {
    row-gap: 0.5rem !important;
  }
  .row-gap-xxl-3 {
    row-gap: 1rem !important;
  }
  .row-gap-xxl-4 {
    row-gap: 1.5rem !important;
  }
  .row-gap-xxl-5 {
    row-gap: 2rem !important;
  }
  .row-gap-xxl-6 {
    row-gap: 3rem !important;
  }
  .column-gap-xxl-0 {
    column-gap: 0 !important;
  }
  .column-gap-xxl-1 {
    column-gap: 0.25rem !important;
  }
  .column-gap-xxl-2 {
    column-gap: 0.5rem !important;
  }
  .column-gap-xxl-3 {
    column-gap: 1rem !important;
  }
  .column-gap-xxl-4 {
    column-gap: 1.5rem !important;
  }
  .column-gap-xxl-5 {
    column-gap: 2rem !important;
  }
  .column-gap-xxl-6 {
    column-gap: 3rem !important;
  }
  .text-xxl-start {
    text-align: left !important;
  }
  .text-xxl-end {
    text-align: right !important;
  }
  .text-xxl-center {
    text-align: center !important;
  }
}
@media (min-width: 1200px) {
  .fs-1 {
    font-size: 2.34375rem !important;
  }
  .fs-2 {
    font-size: 1.875rem !important;
  }
  .fs-3 {
    font-size: 1.640625rem !important;
  }
  .fs-4 {
    font-size: 1.40625rem !important;
  }
}
@media print {
  .d-print-inline {
    display: inline !important;
  }
  .d-print-inline-block {
    display: inline-block !important;
  }
  .d-print-block {
    display: block !important;
  }
  .d-print-grid {
    display: grid !important;
  }
  .d-print-inline-grid {
    display: inline-grid !important;
  }
  .d-print-table {
    display: table !important;
  }
  .d-print-table-row {
    display: table-row !important;
  }
  .d-print-table-cell {
    display: table-cell !important;
  }
  .d-print-flex {
    display: flex !important;
  }
  .d-print-inline-flex {
    display: inline-flex !important;
  }
  .d-print-none {
    display: none !important;
  }
}
/**
 * Moodle variables
 *
 * Variables written for Moodle specific components
 *
 * Please do not override any Bootstrap variables here, custom Bootstrap variable should go in
 * preset files instead.
 */
/* core.less */
#region-main {
  overflow-y: visible;
  background-color: #fff;
}

@media (min-width: 576px) {
  .context-header-settings-menu,
  .region-main-settings-menu {
    float: right;
    width: auto;
    max-width: 4em;
    height: 2em;
    display: block;
    margin-top: 4px;
  }
}
@media (max-width: 767.98px) {
  .context-header-settings-menu,
  .region-main-settings-menu {
    display: flex;
    justify-content: flex-end;
  }
}
.context-header-settings-menu .dropdown-toggle > .icon,
#region-main-settings-menu .dropdown-toggle > .icon {
  height: 24px;
  font-size: 24px;
  width: auto;
}

/** Prevent user notifications overlapping with region main settings menu */
#user-notifications {
  display: block;
  overflow: hidden;
}

/** Page layout CSS starts **/
.layout-option-noheader #page-header,
.layout-option-nonavbar #page-navbar,
.layout-option-nofooter #page-footer,
.layout-option-nocourseheader .course-content-header,
.layout-option-nocoursefooter .course-content-footer {
  display: none;
}

/** Page layout CSS ends **/
.mdl-left {
  text-align: left;
}

.mdl-right {
  text-align: right;
}

/*rtl:ignore*/
.text-ltr {
  direction: ltr !important; /* stylelint-disable-line declaration-no-important */
}

#add,
#remove,
.centerpara,
.mdl-align {
  text-align: center;
}

a.dimmed,
a.dimmed:link,
a.dimmed:visited,
a.dimmed_text,
a.dimmed_text:link,
a.dimmed_text:visited,
.dimmed_text,
.dimmed_text a,
.dimmed_text a:link,
.dimmed_text a:visited,
.usersuspended,
.usersuspended a,
.usersuspended a:link,
.usersuspended a:visited,
.dimmed_category,
.dimmed_category a {
  color: var(--bs-secondary-color);
}

.aalink.focus, a.focus.autolink, .aalink:focus, a.autolink:focus,
#page-footer a:not([class]).focus,
#page-footer a:not([class]):focus,
.arrow_link.focus,
.arrow_link:focus,
a:not([class]).focus,
a:not([class]):focus,
.activityinstance > a.focus,
.activityinstance > a:focus {
  outline: 0.25rem solid transparent;
  color: #1d2125;
  background-color: #d2e8fb;
  box-shadow: 0 -0.25rem #d2e8fb, 0 0.25rem #343a40;
}
.aalink:focus:hover, a.autolink:focus:hover,
#page-footer a:not([class]):focus:hover,
.arrow_link:focus:hover,
a:not([class]):focus:hover,
.activityinstance > a:focus:hover {
  text-decoration: none;
}

.aabtn.focus, .aabtn:focus,
.btn-link.focus,
.btn-link:focus,
.nav-link.focus,
.nav-link:focus,
.editor_atto_toolbar button.focus,
.editor_atto_toolbar button:focus,
.editor_atto_toolbar .atto_toolbar_row.focus,
.editor_atto_toolbar .atto_toolbar_row:focus,
[role=button].focus,
[role=button]:focus,
.list-group-item-action.focus,
.list-group-item-action:focus,
input[type=checkbox].focus,
input[type=checkbox]:focus,
input[type=radio].focus,
input[type=radio]:focus,
input[type=file].focus,
input[type=file]:focus,
input[type=image].focus,
input[type=image]:focus,
.visually-hidden-focusable.focus,
.visually-hidden-focusable:focus,
a.dropdown-toggle.focus,
a.dropdown-toggle:focus,
.moodle-dialogue-base .closebutton.focus,
.moodle-dialogue-base .closebutton:focus,
button.btn-close.focus,
button.btn-close:focus,
.form-autocomplete-selection.focus,
.form-autocomplete-selection:focus,
[role=treeitem]:not([aria-expanded=true]).focus,
[role=treeitem]:not([aria-expanded=true]):focus {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.aabtn:focus:hover,
.btn-link:focus:hover,
.nav-link:focus:hover,
.editor_atto_toolbar button:focus:hover,
.editor_atto_toolbar .atto_toolbar_row:focus:hover,
[role=button]:focus:hover,
.list-group-item-action:focus:hover,
input[type=checkbox]:focus:hover,
input[type=radio]:focus:hover,
input[type=file]:focus:hover,
input[type=image]:focus:hover,
.visually-hidden-focusable:focus:hover,
a.dropdown-toggle:focus:hover,
.moodle-dialogue-base .closebutton:focus:hover,
button.btn-close:focus:hover,
.form-autocomplete-selection:focus:hover,
[role=treeitem]:not([aria-expanded=true]):focus:hover {
  text-decoration: none;
}

.modal-dialog[tabindex="0"].focus, .modal-dialog[tabindex="0"]:focus {
  outline: 0;
}
.modal-dialog[tabindex="0"].focus .modal-content, .modal-dialog[tabindex="0"]:focus .modal-content {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
  border-radius: var(--bs-border-radius-lg);
}

[role=treeitem][aria-expanded=true] {
  outline: 0;
}
[role=treeitem][aria-expanded=true].focus > *:first-child, [role=treeitem][aria-expanded=true]:focus > *:first-child {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
[role=treeitem][aria-expanded=true]:focus:hover {
  text-decoration: none;
}

.form-autocomplete-suggestions li[aria-selected=true] {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}

.safari input[type=checkbox].focus, .safari input[type=checkbox]:focus,
.safari input[type=radio].focus,
.safari input[type=radio]:focus {
  outline: auto;
}

.unlist,
.unlist li,
.inline-list,
.inline-list li,
.block .list,
.block .list li,
.section li.movehere,
.tabtree li {
  list-style: none;
  margin: 0;
  padding: 0;
}

.inline,
.inline-list li {
  display: inline;
}

.notifytiny {
  font-size: 0.703125rem;
}

.notifytiny li,
.notifytiny td {
  font-size: 100%;
}

.red,
.notifyproblem {
  color: #f0ad4e;
}

.green,
.notifysuccess {
  color: #357a32;
}

.highlight {
  color: #008196;
}

.bg-primary-light {
  background-color: #e7f0f9;
}

.fitem.advanced .text-info {
  font-weight: bold;
}

.reportlink {
  text-align: right;
}

a.autolink.glossary:hover {
  cursor: help;
}

/* Block which is hidden if javascript enabled, prevents fickering visible when JS from footer used! */
.collapsibleregioncaption {
  white-space: nowrap;
  min-height: 1.40625rem;
}

.pagelayout-mydashboard.jsenabled .collapsibleregioncaption {
  cursor: pointer;
}

.pagelayout-mydashboard #region-main {
  border: 0;
  padding: 0;
  background-color: transparent;
  margin-top: -1px;
}

@media (max-width: 767.98px) {
  .pagelayout-mydashboard #region-main-box,
  .pagelayout-login #region-main-box {
    padding-left: 0;
    padding-right: 0;
  }
}
.collapsibleregioncaption img {
  vertical-align: middle;
}

.jsenabled .hiddenifjs {
  display: none;
}

body:not(.jsenabled) .visibleifjs {
  display: none;
}

.jsenabled .collapsibleregion {
  overflow: hidden;
  box-sizing: content-box;
}

.jsenabled .collapsed .collapsibleregioninner {
  visibility: hidden;
}

.collapsible-actions {
  display: none;
  text-align: right;
}

.jsenabled .collapsible-actions {
  display: block;
}

.yui-overlay .yui-widget-bd {
  background-color: #ffee69;
  border: 1px solid #a6982b;
  border-top-color: #d4c237;
  color: #000;
  left: 0;
  padding: 2px 5px;
  position: relative;
  top: 0;
  z-index: 1;
}

.clearer {
  background: transparent;
  border-width: 0;
  clear: both;
  display: block;
  height: 1px;
  margin: 0;
  padding: 0;
}

.bold,
.warning,
.errorbox .title,
.pagingbar .title,
.pagingbar .thispage {
  font-weight: bold;
}

img.resize {
  height: 1em;
  width: 1em;
}

.block img.resize {
  height: 0.9em;
  width: 0.8em;
}

/* Icon styles */
img.activityicon {
  height: 32px;
  width: 32px;
  vertical-align: middle;
}

.headermain {
  font-weight: bold;
}

#maincontent {
  display: block;
  height: 1px;
  overflow: clip;
}

img.uihint {
  cursor: help;
}

#addmembersform table {
  margin-left: auto;
  margin-right: auto;
}

table.flexible .emptyrow {
  display: none;
}

form.popupform,
form.popupform div {
  display: inline;
}

.arrow_button input {
  overflow: hidden;
}

.no-overflow {
  overflow: auto;
}

.accesshide {
  width: 1px !important;
  height: 1px !important;
  padding: 0 !important;
  margin: -1px !important;
  overflow: hidden !important;
  clip: rect(0, 0, 0, 0) !important;
  white-space: nowrap !important;
  border: 0 !important;
}
.accesshide:not(caption) {
  position: absolute !important;
}

span.hide,
div.hide,
.hidden {
  display: none;
}

a.skip-block,
a.skip {
  position: absolute;
  top: -1000em;
  font-size: 0.85em;
  text-decoration: none;
}

a.skip-block:focus,
a.skip-block:active,
a.skip:focus,
a.skip:active {
  position: static;
  display: block;
}

.skip-block-to {
  display: block;
  height: 1px;
  overflow: hidden;
}

.addbloglink {
  text-align: center;
}

.blog_entry .audience {
  text-align: right;
  padding-right: 4px;
}

.blog_entry .tags {
  margin-top: 15px;
}

.blog_entry .content {
  margin-left: 43px;
}

#doc-contents h1, #doc-contents .h1 {
  margin: 1em 0 0 0;
}

#doc-contents ul {
  margin: 0;
  padding: 0;
  width: 90%;
}

#doc-contents ul li {
  list-style-type: none;
}

.groupmanagementtable td {
  vertical-align: top;
}

.groupmanagementtable #existingcell,
.groupmanagementtable #potentialcell {
  width: 42%;
}

.groupmanagementtable #buttonscell {
  width: 16%;
}

.groupmanagementtable #buttonscell p.arrow_button input {
  width: auto;
  min-width: 80%;
  margin: 0 auto;
  display: block;
}

.groupmanagementtable #removeselect_wrapper,
.groupmanagementtable #addselect_wrapper {
  width: 100%;
}

.groupmanagementtable #removeselect_wrapper label,
.groupmanagementtable #addselect_wrapper label {
  font-weight: normal;
}

#group-usersummary {
  width: 14em;
}

.notepost {
  margin-bottom: 1em;
}

.notepost .userpicture {
  float: left;
  margin-right: 5px;
}

.notepost .content,
.notepost .footer {
  clear: both;
}

.notesgroup {
  margin-left: 20px;
}

.path-my .coursebox {
  margin: 1rem 0;
  padding: 0;
}
.path-my .coursebox .overview {
  margin: 15px 30px 10px 30px;
}

.path-my .coursebox .info {
  float: none;
  margin: 0;
}

.mod_introbox {
  padding: 10px;
}

table.mod_index {
  width: 100%;
}

.comment-ctrl {
  font-size: 12px;
  display: none;
  margin: 0;
  padding: 0;
}

.comment-ctrl h5, .comment-ctrl .h5 {
  margin: 0;
  padding: 5px;
}

.comment-area {
  max-width: 400px;
  padding: 5px;
}

.comment-area textarea {
  width: 100%;
  overflow: auto;
}
.comment-area textarea.fullwidth {
  -webkit-box-sizing: border-box;
  -moz-box-sizing: border-box;
  box-sizing: border-box;
}

.comment-area .fd {
  text-align: right;
}

.comment-meta span {
  color: gray;
}

.comment-link img {
  vertical-align: text-bottom;
}

.comment-list {
  font-size: 11px;
  overflow: auto;
  list-style: none;
  padding: 0;
  margin: 0;
}

.comment-list li {
  margin: 2px;
  list-style: none;
  margin-bottom: 5px;
  clear: both;
  padding: 0.3em;
  position: relative;
}

.comment-list li.first {
  display: none;
}

.comment-paging {
  text-align: center;
}

.comment-paging .pageno {
  padding: 2px;
}

.comment-paging .curpage {
  border: 1px solid #ccc;
}

.comment-message .picture {
  float: left;
  margin-right: 0.25rem;
}

.comment-message .text {
  margin: 0;
  padding: 0;
}

.comment-message .text p {
  padding: 0;
  margin: 0 18px 0 0;
}

.comment-delete {
  position: absolute;
  top: 0;
  right: 0;
  margin: 0.3em;
}

.comment-report-selectall {
  display: none;
}

.comment-link {
  display: none;
}

.jsenabled .comment-link {
  display: block;
}

.jsenabled .showcommentsnonjs {
  display: none;
}

.jsenabled .comment-report-selectall {
  display: inline;
}

/**
* Completion progress report
*/
.completion-expired {
  color: #f0ad4e;
}

.completion-expected {
  font-size: 0.703125rem;
}

.completion-sortchoice,
.completion-identifyfield {
  font-size: 0.703125rem;
  vertical-align: bottom;
}

.completion-progresscell {
  text-align: right;
}

.completion-expired .completion-expected {
  font-weight: bold;
}

/**
* Tags
*/
span.flagged-tag,
tr.flagged-tag,
span.flagged-tag a,
tr.flagged-tag a {
  color: #f0ad4e;
}

.tag-management-table td,
.tag-management-table th {
  vertical-align: middle;
  padding: 4px;
}
.tag-management-table .inplaceeditable.inplaceeditingon input {
  width: 150px;
}

.path-admin-tag .addstandardtags {
  float: right;
}
.path-admin-tag .addstandardtags img {
  margin: 0 5px;
}

.path-tag .tag-relatedtags {
  padding-top: 10px;
}
.path-tag .tag-management-box {
  text-align: right;
}
.path-tag .tag-index-toc {
  padding: 10px;
  text-align: center;
}
.path-tag .tag-index-toc li {
  margin-left: 5px;
  margin-right: 5px;
}
.path-tag .tag-management-box li {
  margin-left: 5px;
  margin-right: 5px;
}
.path-tag .tag-management-box li a {
  background-repeat: no-repeat;
  background-position: left;
  padding-left: 17px;
}
.path-tag .tag-management-box li a.edittag {
  background-image: url([[pix:moodle|i/settings]]);
}
.path-tag .tag-management-box li a.flagasinappropriate {
  background-image: url([[pix:moodle|i/flagged]]);
}
.path-tag .tag-management-box li a.removefrommyinterests {
  background-image: url([[pix:moodle|t/delete]]);
}
.path-tag .tag-management-box li a.addtomyinterests {
  background-image: url([[pix:moodle|t/add]]);
}
.path-tag .tag-backtoallitems {
  text-align: center;
}
.path-tag .tag-index-items .tagarea {
  border: 1px solid #e3e3e3;
  border-radius: 4px;
  padding: 10px;
  margin-top: 10px;
}
.path-tag .tag-index-items .tagarea h3, .path-tag .tag-index-items .tagarea .h3 {
  display: block;
  padding: 3px 0 10px 0;
  margin: 0;
  font-size: 1.1em;
  font-weight: bold;
  line-height: 20px;
  color: #999;
  text-shadow: 0 1px 0 rgba(255, 255, 255, 0.5);
  text-transform: uppercase;
  word-wrap: break-word;
  border-bottom: solid 1px #e3e3e3;
  margin-bottom: 10px;
}
.path-tag .tag-index-items .tagarea .taggeditems::after {
  display: block;
  clear: both;
  content: "";
}
.path-tag .tag-index-items .tagarea .controls {
  text-align: center;
}
.path-tag .tag-index-items .tagarea .controls::after {
  display: block;
  clear: both;
  content: "";
}
.path-tag .tag-index-items .tagarea .controls.controls-bottom {
  margin-top: 5px;
}
.path-tag .tag-index-items .tagarea .controls .gotopage.nextpage {
  float: right;
}
.path-tag .tag-index-items .tagarea .controls .gotopage.prevpage {
  float: left;
}
.path-tag .tag-index-items .tagarea .controls .exclusivemode {
  display: inline-block;
}
.path-tag .tag-index-items .tagarea .controls .gotopage.prevpage::before {
  padding-right: 5px;
  padding-left: 5px;
  content: "«";
}
.path-tag .tag-index-items .tagarea .controls .gotopage.nextpage::after {
  padding-right: 5px;
  padding-left: 5px;
  content: "»";
}

.tag_feed img {
  max-height: 35px;
  max-width: 35px;
}
.tag_feed .muted a {
  color: var(--bs-secondary-color);
}

.tag_cloud {
  text-align: center;
}
.tag_cloud .inline-list li {
  padding: 0 0.2em;
}
.tag_cloud .tag_overflow {
  margin-top: 1em;
  font-style: italic;
}
.tag_cloud .s0 {
  font-size: 0.7em;
}
.tag_cloud .s1 {
  font-size: 0.8em;
}
.tag_cloud .s2 {
  font-size: 0.9em;
}
.tag_cloud .s3 {
  font-size: 1em;
}
.tag_cloud .s4 {
  font-size: 1.1em;
}
.tag_cloud .s5 {
  font-size: 1.2em;
}
.tag_cloud .s6 {
  font-size: 1.3em;
}
.tag_cloud .s7 {
  font-size: 1.4em;
}
.tag_cloud .s8 {
  font-size: 1.5em;
}
.tag_cloud .s9 {
  font-size: 1.6em;
}
.tag_cloud .s10 {
  font-size: 1.7em;
}
.tag_cloud .s11 {
  font-size: 1.8em;
}
.tag_cloud .s12 {
  font-size: 1.9em;
}
.tag_cloud .s13 {
  font-size: 2em;
}
.tag_cloud .s14 {
  font-size: 2.1em;
}
.tag_cloud .s15 {
  font-size: 2.2em;
}
.tag_cloud .s16 {
  font-size: 2.3em;
}
.tag_cloud .s17 {
  font-size: 2.4em;
}
.tag_cloud .s18 {
  font-size: 2.5em;
}
.tag_cloud .s19 {
  font-size: 2.6em;
}
.tag_cloud .s20 {
  font-size: 2.7em;
}

.tag_list ul {
  display: inline;
}
.tag_list .tagmorelink {
  display: none;
}
.tag_list.hideoverlimit .overlimit {
  display: none;
}
.tag_list.hideoverlimit .tagmorelink {
  display: inline;
}
.tag_list.hideoverlimit .taglesslink {
  display: none;
}

/**
* Web Service
*/
#webservice-doc-generator td {
  text-align: left;
  border: 0 solid #000;
}

/**
* Enrol
*/
.userenrolment {
  width: 100%;
  border-collapse: collapse;
}

.userenrolment tr {
  vertical-align: top;
}

.userenrolment td {
  padding: 0;
  height: 41px;
}

.userenrolment .subfield {
  margin-right: 5px;
}

.userenrolment .col_userdetails .subfield {
  margin-left: 40px;
}

.userenrolment .col_userdetails .subfield_picture {
  float: left;
  margin-left: 0;
}

.userenrolment .col_lastseen {
  width: 150px;
}

.userenrolment .col_role {
  width: 262px;
}

.userenrolment .col_role .roles,
.userenrolment .col_group .groups {
  margin-right: 30px;
}

.userenrolment .col_role .role {
  float: left;
  padding: 0 3px 3px;
  margin: 0 3px 3px;
  white-space: nowrap;
}

.userenrolment .col_group .group {
  float: left;
  padding: 3px;
  margin: 3px;
  white-space: nowrap;
}

.userenrolment .col_role .role a,
.userenrolment .col_group .group a {
  margin-left: 3px;
  cursor: pointer;
}

.userenrolment .col_role .addrole,
.userenrolment .col_group .addgroup {
  float: right;
  padding: 3px;
  margin: 3px;
}
.userenrolment .col_role .addrole > a:hover,
.userenrolment .col_group .addgroup > a:hover {
  border-bottom: 1px solid #666;
}

.userenrolment .col_role .addrole img,
.userenrolment .col_group .addgroup img {
  vertical-align: baseline;
}

.userenrolment .hasAllRoles .col_role .addrole {
  display: none;
}

.userenrolment .col_enrol .enrolment {
  float: left;
  padding: 0 3px 3px;
  margin: 0 3px 3px;
}

.userenrolment .col_enrol .enrolment a {
  float: right;
  margin-left: 3px;
}

#page-enrol-otherusers .userenrolment .col_role .role {
  float: none;
  margin: 3px 3px 3px 0;
  padding: 3px 3px 3px 0;
}

.corelightbox {
  background-color: #ccc;
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  text-align: center;
}

.corelightbox img {
  position: fixed;
  top: 50%;
  left: 50%;
}

/* Audio player size in 'block' mode (can only change width, height is hardcoded in JS) */
.resourcecontent .mediaplugin_mp3 object {
  height: 25px;
  width: 600px;
}

.resourcecontent audio.mediaplugin_html5audio {
  width: 600px;
}

/** Large resource images should avoid hidden overflow **/
.resourceimage {
  max-width: 100%;
}

/* Audio player size in 'inline' mode (can only change width, as above) */
.mediaplugin_mp3 object {
  height: 15px;
  width: 300px;
}

audio.mediaplugin_html5audio {
  width: 300px;
}

.core_media_preview.pagelayout-embedded #maincontent {
  height: 0;
}

.path-rating .ratingtable {
  width: 100%;
  margin-bottom: 1em;
}

.path-rating .ratingtable th.rating {
  width: 100%;
}

.path-rating .ratingtable td.rating,
.path-rating .ratingtable td.time {
  white-space: nowrap;
  text-align: center;
}

/* Moodle Dialogue Settings (moodle-core-dialogue)  */
.moodle-dialogue-base .moodle-dialogue-lightbox {
  background-color: #495057;
}

.pagelayout-popup .moodle-dialogue-base .moodle-dialogue-lightbox {
  background-color: transparent;
}
.pagelayout-popup .moodle-dialogue-base .moodle-dialogue {
  box-shadow: var(--bs-box-shadow);
}

.moodle-dialogue-base .hidden,
.moodle-dialogue-base .moodle-dialogue-hidden {
  display: none;
}

.no-scrolling {
  overflow: hidden;
}

.moodle-dialogue-base .moodle-dialogue-fullscreen {
  left: 0;
  top: 0;
  right: 0;
  bottom: -50px;
  position: fixed;
}

.moodle-dialogue-base .moodle-dialogue-fullscreen .moodle-dialogue-content {
  overflow: auto;
}

.moodle-dialogue-base .moodle-dialogue-wrap {
  background-color: #fff;
  border: 1px solid #ccc;
}

.modal.show {
  display: block;
}

.moodle-dialogue-base .moodle-dialogue-wrap .moodle-dialogue-hd {
  display: flex;
  padding: 1rem 1rem;
  border-bottom: 1px solid #dee2e6;
}

.moodle-dialogue-base .moodle-dialogue-wrap .moodle-dialogue-hd.yui3-widget-hd {
  min-height: 3rem;
  color: initial;
  background: initial;
  font-size: 1.5rem;
  line-height: 1.5;
}

.moodle-dialogue-base .moodle-dialogue-wrap .moodle-dialogue-hd h5, .moodle-dialogue-base .moodle-dialogue-wrap .moodle-dialogue-hd .h5 {
  font-size: 1.5rem;
  font-weight: 400;
  margin-bottom: 0;
  line-height: 1.5;
}

.moodle-dialogue-base .moodle-dialogue-wrap .moodle-dialogue-hd .yui3-widget-buttons {
  /*rtl:raw:
      left: 0;
      right: auto;
  */
  padding: 0;
  position: relative;
  margin-left: auto;
}

.moodle-dialogue-base .closebutton {
  padding: 1rem 1rem;
  margin: -1rem -1rem -1rem auto;
  position: relative;
  background-color: transparent;
  border: 0;
  background-image: none;
  box-shadow: none;
  opacity: 0.7;
}
.moodle-dialogue-base .closebutton:hover, .moodle-dialogue-base .closebutton:active {
  opacity: 1;
}
.moodle-dialogue-base .closebutton::after {
  content: "×";
}

.moodle-dialogue-base .moodle-dialogue .moodle-dialogue-bd {
  padding: 0.5rem;
}
.moodle-dialogue-base .moodle-dialogue .moodle-dialogue-bd body {
  background-color: #fff;
}

.moodle-dialogue-base .moodle-dialogue-fullscreen .moodle-dialogue-content {
  overflow: auto;
  position: absolute;
  top: 0;
  bottom: 50px;
  left: 0;
  right: 0;
  margin: 0;
  border: 0;
}

.moodle-dialogue-exception .moodle-exception-param label {
  font-weight: bold;
}

.moodle-dialogue-exception .param-stacktrace label {
  background-color: #eee;
  border: 1px solid #ccc;
  border-bottom-width: 0;
}

.moodle-dialogue-exception .param-stacktrace pre {
  border: 1px solid #ccc;
  background-color: #fff;
}

.moodle-dialogue-exception .param-stacktrace .stacktrace-file {
  color: navy;
  font-size: 0.8203125rem;
}

.moodle-dialogue-exception .param-stacktrace .stacktrace-line {
  color: #f0ad4e;
  font-size: 0.8203125rem;
}

.moodle-dialogue-exception .param-stacktrace .stacktrace-call {
  color: #333;
  font-size: 90%;
  border-bottom: 1px solid #eee;
}

.moodle-dialogue-base .moodle-dialogue .moodle-dialogue-content .moodle-dialogue-ft:empty {
  display: none;
}

.moodle-dialogue-base .moodle-dialogue .moodle-dialogue-content .moodle-dialogue-ft.yui3-widget-ft {
  background: initial;
}

.moodle-dialogue-confirm .confirmation-message {
  margin: 0.5rem 0;
}

.moodle-dialogue-confirm .confirmation-dialogue input {
  min-width: 80px;
}

.moodle-dialogue-exception .moodle-exception-message {
  margin: 1em;
}

.moodle-dialogue-exception .moodle-exception-param {
  margin-bottom: 0.5em;
}

.moodle-dialogue-exception .moodle-exception-param label {
  width: 150px;
}

.moodle-dialogue-exception .param-stacktrace label {
  display: block;
  margin: 0;
  padding: 4px 1em;
}

.moodle-dialogue-exception .param-stacktrace pre {
  display: block;
  height: 200px;
  overflow: auto;
}

.moodle-dialogue-exception .param-stacktrace .stacktrace-file {
  display: inline-block;
  margin: 4px 0;
}

.moodle-dialogue-exception .param-stacktrace .stacktrace-line {
  display: inline-block;
  width: 50px;
  margin: 4px 1em;
}

.moodle-dialogue-exception .param-stacktrace .stacktrace-call {
  padding-left: 25px;
  margin-bottom: 4px;
  padding-bottom: 4px;
}

.moodle-dialogue .moodle-dialogue-bd .content-lightbox {
  opacity: 0.75;
  width: 100%;
  height: 100%;
  top: 0;
  left: 0;
  background-color: #fff;
  text-align: center;
  padding: 10% 0;
}

/* Apply a default max-height on tooltip text */
.moodle-dialogue .tooltiptext {
  max-height: 300px;
}

.moodle-dialogue-base .moodle-dialogue.moodle-dialogue-tooltip {
  z-index: 3001;
}
.moodle-dialogue-base .moodle-dialogue.moodle-dialogue-tooltip .moodle-dialogue-bd {
  overflow: auto;
}

/**
 * Chooser Dialogues (moodle-core-chooserdialogue)
 *
 * This CSS belong to the chooser dialogue which should work both with, and
 * without javascript enabled
 */
/* Hide the dialog and it's title */
.chooserdialoguebody,
.choosertitle {
  display: none;
}

.moodle-dialogue.chooserdialogue .moodle-dialogue-content .moodle-dialogue-ft {
  margin: 0;
}

.chooserdialogue .moodle-dialogue-wrap .moodle-dialogue-bd {
  padding: 0;
  background: #f2f2f2;
  border-bottom-right-radius: 10px;
  border-bottom-left-radius: 10px;
}

/* Center the submit buttons within the area */
.choosercontainer #chooseform .submitbuttons {
  padding: 0.7em 0;
  text-align: right;
}

/* Fixed for safari browser on iPhone4S with ios7@mixin */
@media (max-height: 639px) {
  .ios .choosercontainer #chooseform .submitbuttons {
    padding: 45px 0;
  }
}
.choosercontainer #chooseform .submitbuttons input {
  min-width: 100px;
  margin: 0 0.5em;
}

/* Various settings for the options area */
.choosercontainer #chooseform .options {
  position: relative;
  border-bottom: 1px solid #bbb;
}

/* Only set these options if we're showing the js container */
.jschooser .choosercontainer #chooseform .alloptions {
  overflow-x: hidden;
  overflow-y: auto;
  max-width: 240px;
}
.jschooser .choosercontainer #chooseform .alloptions .option input[type=radio] {
  display: inline-block;
}
.jschooser .choosercontainer #chooseform .alloptions .option .typename {
  display: inline-block;
  width: 55%;
}

/* Settings for option rows and option subtypes */
.choosercontainer #chooseform .moduletypetitle,
.choosercontainer #chooseform .option,
.choosercontainer #chooseform .nonoption {
  margin-bottom: 0;
  padding: 0 1.6em 0 1.6em;
}

.choosercontainer #chooseform .moduletypetitle {
  text-transform: uppercase;
  padding-top: 1.2em;
  padding-bottom: 0.4em;
  margin-bottom: 0.5rem;
  font-size: 100%;
}

.choosercontainer #chooseform .option .typename,
.choosercontainer #chooseform .nonoption .typename {
  padding: 0 0 0 0.5em;
}

.choosercontainer #chooseform .modicon + .typename {
  padding-left: 0;
}

.choosercontainer #chooseform .option input[type=radio],
.choosercontainer #chooseform .option span.typename {
  vertical-align: middle;
}

.choosercontainer #chooseform .option label {
  display: block;
  margin: 0;
  padding: 0.5rem 0;
  border-bottom: 1px solid #fff;
}

.choosercontainer #chooseform .option .icon {
  margin: 0;
  padding: 0 1rem;
}

.choosercontainer #chooseform .nonoption {
  padding-left: 2.7em;
  padding-top: 0.3em;
  padding-bottom: 0.1em;
}

.choosercontainer #chooseform .subtype {
  margin-bottom: 0;
  padding: 0 1.6em 0 3.2em;
}

.choosercontainer #chooseform .subtype .typename {
  margin: 0 0 0 0.2em;
}

/* The instruction/help area */
.jschooser .choosercontainer #chooseform .instruction,
.jschooser .choosercontainer #chooseform .typesummary {
  display: none;
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  left: 240px;
  margin: 0;
  padding: 1.6em;
  background-color: #fff;
  overflow-x: hidden;
  overflow-y: auto;
  line-height: 2em;
}

/* Selected option settings */
.jschooser .choosercontainer #chooseform .instruction,
.choosercontainer #chooseform .selected .typesummary {
  display: block;
}

.choosercontainer #chooseform .selected {
  background-color: #fff;
  margin-top: -1px;
  padding-top: 1px;
}

@media (max-width: 575.98px) {
  .jsenabled .choosercontainer #chooseform .alloptions {
    max-width: 100%;
  }
  .jsenabled .choosercontainer #chooseform .instruction,
  .jsenabled .choosercontainer #chooseform .typesummary {
    position: static;
  }
}
/* Form element: listing */
.formlistingradio {
  padding-bottom: 25px;
  padding-right: 10px;
}

.formlistinginputradio {
  float: left;
}

.formlistingmain {
  min-height: 225px;
}

.formlisting {
  position: relative;
  margin: 15px 0;
  padding: 1px 19px 14px;
  background-color: #fff;
  border: 1px solid #ddd;
  border-radius: 4px;
}

.formlistingmore {
  position: absolute;
  cursor: pointer;
  bottom: -1px;
  right: -1px;
  padding: 3px 7px;
  font-size: 12px;
  font-weight: bold;
  background-color: whitesmoke;
  border: 1px solid #ddd;
  color: #9da0a4;
  border-radius: 4px 0 4px 0;
}

.formlistingall {
  margin: 15px 0;
  padding: 0;
  border-radius: 4px;
}

.formlistingrow {
  cursor: pointer;
  border-bottom: 1px solid;
  border-color: #e1e1e8;
  border-left: 1px solid #e1e1e8;
  border-right: 1px solid #e1e1e8;
  background-color: #f7f7f9;
  border-radius: 0 0 4px 4px;
  padding: 6px;
  top: 50%;
  left: 50%;
  min-height: 34px;
  float: left;
  width: 150px;
}

body.jsenabled .formlistingradio {
  display: none;
}

body.jsenabled .formlisting {
  display: block;
}

a.criteria-action {
  padding: 0 3px;
  float: right;
}

div.criteria-description {
  padding: 10px 15px;
  margin: 5px 0;
  background: none repeat scroll 0 0 #f9f9f9;
  border: 1px solid #eee;
}

ul.badges {
  margin: 0;
  list-style: none;
}

.badges li {
  position: relative;
  display: inline-block;
  padding-top: 1em;
  text-align: center;
  vertical-align: top;
  width: 150px;
}

.badges li .badge-name {
  display: block;
  padding: 5px;
}

.badges li > img {
  position: absolute;
}

.badges li .badge-image {
  width: 100px;
  height: 100px;
  left: 10px;
  top: 0;
  z-index: 1;
}

.badges li .badge-actions {
  position: relative;
}

.badges li .expireimage {
  background-image: url([[pix:i/expired]]);
  background-repeat: no-repeat;
  background-size: 100px 100px;
  width: 100px;
  height: 100px;
  left: 25px;
  top: 15px;
  position: absolute;
  z-index: 10;
  opacity: 0.85;
}

#badge-image {
  background-color: transparent;
  padding: 0;
  position: relative;
  min-width: 100px;
  width: 20%;
  display: inline-block;
  vertical-align: top;
  margin-top: 17px;
  margin-bottom: 20px;
}
#badge-image .expireimage {
  background-image: url([[pix:i/expired]]);
  background-repeat: no-repeat;
  background-size: 100px 100px;
  width: 100px;
  height: 100px;
  left: 0;
  top: 0;
  opacity: 0.85;
  position: absolute;
  z-index: 10;
}
#badge-image .singlebutton {
  padding-top: 5px;
  display: block;
}
#badge-image .singlebutton button {
  margin-left: 4px;
}

#badge-details {
  display: inline-block;
  width: 79%;
}

#badge-overview dl,
#badge-details dl {
  margin: 0;
}
#badge-overview dl dt,
#badge-overview dl dd,
#badge-details dl dt,
#badge-details dl dd {
  vertical-align: top;
  padding: 3px 0;
}
#badge-overview dl dt,
#badge-details dl dt {
  clear: both;
  display: inline-block;
  width: 20%;
  min-width: 100px;
}
#badge-overview dl dd,
#badge-details dl dd {
  display: inline-block;
  width: 79%;
  margin-left: 1%;
}

#badge-criteria li li {
  list-style-type: none;
}

#badge-image-col {
  flex: 0 0 400px;
}

.badge-profile {
  vertical-align: top;
}

.connected {
  color: #357a32;
}

.notconnected {
  color: #ca3120;
}

.connecting {
  color: #f0ad4e;
}

#page-badges-index #page .badges-heading {
  display: flex;
  justify-content: space-between;
  align-items: center;
}
#page-badges-index #page h2, #page-badges-index #page .h2 {
  margin: 0; /* Removes default margins */
}
#page-badges-index #page #action_bar {
  margin: 0; /* Removes any default margin that might affect alignment */
}
#page-badges-index #page .tertiary-navigation.container-fluid, #page-badges-index #page .tertiary-navigation.container-sm, #page-badges-index #page .tertiary-navigation.container-md, #page-badges-index #page .tertiary-navigation.container-lg, #page-badges-index #page .tertiary-navigation.container-xl, #page-badges-index #page .tertiary-navigation.container-xxl {
  width: auto;
  height: 50px;
  padding-top: 0;
}

#page-badges-award .recipienttable tr td {
  vertical-align: top;
}

#page-badges-award .recipienttable tr td.actions .actionbutton {
  margin: 0.3em 0;
  padding: 0.5em 0;
  width: 100%;
}

#page-badges-award .recipienttable tr td.existing,
#page-badges-award .recipienttable tr td.potential {
  width: 42%;
}

#issued-badge-table .activatebadge {
  display: inline-block;
}

.statusbox.active {
  background-color: #d7e4d6;
}

.statusbox.inactive {
  background-color: #fcefdc;
}

.statusbox {
  text-align: center;
  margin-bottom: 5px;
  padding: 5px;
}

.statusbox .activatebadge {
  display: inline-block;
}

.statusbox .activatebadge input[type=submit] {
  margin: 3px;
}

.activatebadge {
  margin: 0;
  text-align: left;
  vertical-align: middle;
}

img#persona_signin {
  cursor: pointer;
}

.addcourse {
  float: right;
}

.invisiblefieldset {
  display: inline;
  padding: 0;
  border-width: 0;
}

/** Page header */
#page-header h1.h2, #page-header .h2.h1 {
  font-weight: bold;
}
#page-header .logo {
  margin: 1rem 0;
}
#page-header .logo img {
  max-height: 75px;
}

/** Navbar logo. */
nav.navbar .logo img {
  max-height: 35px;
}

.nav.usernav .nav-item {
  display: flex;
}
.nav.usernav .usermenu .dropdown-toggle {
  padding: 0 0.5rem;
}

/** Header-bar styles **/
.page-context-header {
  overflow: hidden;
  padding: 0.25rem 0;
}
.page-context-header .page-header-image > a {
  display: inline-block;
}
.page-context-header .page-header-image .userinitials,
.page-context-header .page-header-image .userpicture {
  margin-right: 1rem;
}

ul.dragdrop-keyboard-drag li {
  list-style-type: none;
}
ul.dragdrop-keyboard-drag li a,
ul.dragdrop-keyboard-drag li a:hover {
  color: inherit;
  text-decoration: none;
}

a.disabled:hover,
a.disabled {
  text-decoration: none;
  cursor: default;
  font-style: italic;
  color: var(--bs-secondary-color);
}

a.btn.disabled {
  opacity: 1;
}

body.lockscroll {
  height: 100%;
  overflow: hidden;
}

.progressbar_container {
  max-width: 500px;
  margin: 0 auto;
}

/* IE10 only fix for calendar titling */
.ie10 .yui3-calendar-header-label {
  display: inline-block;
}

dd:before,
dd:after {
  display: block;
  content: " ";
}

dd:after {
  clear: both;
}

.nav-tabs > .active > a[href],
.nav-tabs > .active > a[href]:hover,
.nav-tabs > .active > a[href]:focus {
  cursor: pointer;
}

.inplaceeditable.inplaceeditingon {
  position: relative;
}
.inplaceeditable.inplaceeditingon .editinstructions {
  margin-top: -30px;
  font-weight: normal;
  margin-right: 0;
  margin-left: 0;
  left: 0;
  right: auto;
  white-space: nowrap;
  position: absolute;
  top: 0;
  font-size: 0.8203125rem;
  padding: 0.1em 0.4em;
  text-decoration: none;
  z-index: 9999;
  border: var(--bs-border-width) solid transparent;
  width: fit-content;
  color: #004d5a;
  border-color: #b3d9e0;
  background-color: #cce6ea;
}
@media (min-width: 576px) {
  .inplaceeditable.inplaceeditingon input {
    width: 330px;
    vertical-align: text-bottom;
    margin-bottom: 0;
  }
  .inplaceeditable.inplaceeditingon input[role=combobox] {
    width: auto;
  }
}
.inplaceeditable.inplaceeditingon select {
  margin-bottom: 0;
}
.inplaceeditable .quickediticon img {
  opacity: 0.2;
}
.inplaceeditable .quickeditlink {
  color: inherit;
  text-decoration: inherit;
}
.inplaceeditable:hover .quickeditlink .quickediticon img,
.inplaceeditable .quickeditlink:focus .quickediticon img {
  opacity: 1;
}
.inplaceeditable.inplaceeditable-toggle .quickediticon {
  display: none;
}
.inplaceeditable.inplaceeditable-autocomplete {
  display: block;
}

/** Chart area. */
@media (min-width: 992px) {
  .chart-area .chart-image {
    position: relative;
    margin: auto;
    height: 48vh;
    width: 46vw;
  }
}
.chart-area .chart-table-data {
  display: none;
}
.chart-area .chart-table {
  /** When accessible, we display the table only. */
}
.chart-area .chart-table .chart-output-htmltable caption {
  white-space: nowrap;
}
.chart-area .chart-table.accesshide .chart-table-expand {
  display: none;
}
.chart-area .chart-table.accesshide .chart-table-data {
  display: block;
}

/* YUI 2 Tree View */
/*rtl:raw:
.ygtvtn,
.ygtvtm,
.ygtvtmh,
.ygtvtmhh,
.ygtvtp,
.ygtvtph,
.ygtvtphh,
.ygtvln,
.ygtvlm,
.ygtvlmh,
.ygtvlmhh,
.ygtvlp,
.ygtvlph,
.ygtvlphh,
.ygtvdepthcell,
.ygtvok,
.ygtvok:hover,
.ygtvcancel,
.ygtvcancel:hover {
    background-image: url([[pix:theme|yui2-treeview-sprite-rtl]]);
}
*/
.hover-tooltip-container {
  position: relative;
}
.hover-tooltip-container .hover-tooltip {
  opacity: 0;
  visibility: hidden;
  position: absolute;
  /*rtl:ignore*/
  left: 50%;
  top: calc(-50% - 5px);
  transform: translate(-50%, -50%);
  background-color: #fff;
  border: 1px solid rgba(0, 0, 0, 0.2);
  border-radius: 0.3rem;
  box-sizing: border-box;
  padding: 5px;
  white-space: nowrap;
  transition: opacity 0.15s, visibility 0.15s;
  z-index: 1000;
}
.hover-tooltip-container .hover-tooltip:before {
  content: "";
  display: inline-block;
  border-left: 8px solid transparent;
  border-right: 8px solid transparent;
  border-top: 8px solid rgba(0, 0, 0, 0.2);
  position: absolute;
  bottom: -8px;
  left: calc(50% - 8px);
}
.hover-tooltip-container .hover-tooltip:after {
  content: "";
  display: inline-block;
  border-left: 7px solid transparent;
  border-right: 7px solid transparent;
  border-top: 7px solid #fff;
  position: absolute;
  bottom: -6px;
  left: calc(50% - 7px);
  z-index: 2;
}
.hover-tooltip-container:hover .hover-tooltip {
  opacity: 1;
  visibility: visible;
  transition: opacity 0.15s 0.5s, visibility 0.15s 0.5s;
}

#region-flat-nav {
  padding-right: 0;
  padding-left: 0;
}
#region-flat-nav .nav {
  margin-right: 0.75rem;
  background-color: var(--bs-body-bg);
}
@media (max-width: 767.98px) {
  #region-flat-nav .nav {
    margin-top: 1.5rem;
    margin-right: 0;
  }
}

.footer-dark a {
  color: #fff;
  text-decoration: underline;
  padding-top: 0.25rem;
  padding-bottom: 0.25rem;
}
.footer-dark a .icon {
  color: #fff;
}
.footer-dark a:focus .icon {
  color: #1d2125;
}

.footer-popover .btn-footer-popover {
  display: none;
  position: fixed;
  bottom: 2rem;
  right: 2rem;
}

.btn-footer-communication {
  display: none;
  position: fixed;
  bottom: 5rem;
  right: 2rem;
}

.hasstickyfooter .btn-footer-popover {
  bottom: calc(1rem + max(80px, 0.9375rem * 3));
}

.hasstickyfooter .btn-footer-communication {
  bottom: calc(4rem + max(80px, 0.9375rem * 3));
}

.hasstickyfooter .maintenancewarning {
  bottom: max(80px, 0.9375rem * 3);
}

.popover.footer .popover-body {
  padding: 0;
}
.popover.footer .popover-body .footer-section a {
  color: #1d2125;
  text-decoration: underline;
}
.popover.footer .popover-body .footer-section a .icon {
  color: #1d2125;
}
.popover.footer .popover-body .footer-section a:focus {
  text-decoration: none;
}

.footer-support-link {
  padding-bottom: 5px;
}

@media (min-width: 576px) {
  .jsenabled #page-footer .footer-content-popover {
    display: none;
  }
  .jsenabled .btn-footer-popover,
  .jsenabled .btn-footer-communication {
    display: block;
    z-index: 1000;
  }
  .jsenabled .maintenancewarning,
  .jsenabled.hasstickyfooter:has(.drawer-right-toggle) .maintenancewarning {
    right: 5rem;
  }
}
.bg-inverse a {
  color: #fff;
  text-decoration: underline;
}
.bg-inverse a .icon {
  color: #fff;
}

.sitelink img {
  width: 112px;
}

.competency-tree ul {
  padding-left: 1.5rem;
}

.visually-hidden-focusable:active, .visually-hidden-focusable:focus {
  z-index: 1031;
  position: fixed;
  background: #fff;
  padding: 7px;
  left: 0;
  top: 0;
}

[data-drag-type=move] {
  cursor: move;
  touch-action: none;
}

.clickable {
  cursor: pointer;
}

.overlay-icon-container {
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  background-color: rgba(255, 255, 255, 0.6);
}
.overlay-icon-container .loading-icon {
  position: absolute;
  top: 50%;
  left: 50%;
  transform: translate(-50%, -50%);
}
.overlay-icon-container .loading-icon .icon {
  max-height: 2em;
  max-width: 2em;
  font-size: 2em;
}

.w-auto {
  width: auto;
}

.bg-pulse-grey {
  animation: bg-pulse-grey 2s infinite linear;
}

@keyframes bg-pulse-grey {
  0% {
    background-color: #f8f9fa;
  }
  50% {
    background-color: #e9ecef;
  }
  100% {
    background-color: #f8f9fa;
  }
}
.line-height-0 {
  line-height: 0 !important; /* stylelint-disable-line declaration-no-important */
}

.line-height-1 {
  line-height: 0.25rem !important; /* stylelint-disable-line declaration-no-important */
}

.line-height-2 {
  line-height: 0.5rem !important; /* stylelint-disable-line declaration-no-important */
}

.line-height-3 {
  line-height: 1rem !important; /* stylelint-disable-line declaration-no-important */
}

.line-height-4 {
  line-height: 1.5rem !important; /* stylelint-disable-line declaration-no-important */
}

.line-height-5 {
  line-height: 2rem !important; /* stylelint-disable-line declaration-no-important */
}

.line-height-6 {
  line-height: 3rem !important; /* stylelint-disable-line declaration-no-important */
}

.dir-rtl .dir-rtl-hide {
  display: none;
}

.dir-ltr .dir-ltr-hide {
  display: none;
}

.paged-content-page-container {
  min-height: 3.125rem;
}

body.h5p-embed #page-content {
  display: inherit;
}
body.h5p-embed #maincontent {
  display: none;
}
body.h5p-embed .h5pmessages {
  min-height: 230px;
}

#h5pcontenttypes td {
  vertical-align: middle;
}

#page.drawers form#h5peditor,
#page.drawers form#coolh5peditor,
#page.drawers .core_contentbank_viewcontent {
  max-width: 960px;
  margin: 0 auto;
}

#page-contentbank-edit fieldset {
  margin-top: 1em;
}

.text-decoration-none {
  text-decoration: none !important; /* stylelint-disable-line declaration-no-important */
}

.colour-inherit {
  color: inherit !important; /* stylelint-disable-line declaration-no-important */
}

.position-right {
  right: 0 !important; /* stylelint-disable-line declaration-no-important */
}

.overflow-hidden {
  overflow: hidden !important; /* stylelint-disable-line declaration-no-important */
}

.text-break {
  overflow-wrap: break-word !important; /* stylelint-disable-line declaration-no-important */
}

.word-break {
  word-break: break-word !important; /* stylelint-disable-line declaration-no-important */
}

.z-index-0 {
  z-index: 0 !important; /* stylelint-disable-line declaration-no-important */
}

.z-index-1 {
  z-index: 1 !important; /* stylelint-disable-line declaration-no-important */
}

.img-responsive {
  max-width: 100%;
  height: auto;
}

input[disabled] {
  cursor: not-allowed;
}

.fade.in {
  opacity: 1;
}

.clamp-2 {
  display: -webkit-box;
  -webkit-box-orient: vertical;
  line-clamp: 2;
  -webkit-line-clamp: 2;
  overflow: hidden;
}

.word-break-all {
  word-break: break-all;
}

.matchtext {
  background-color: #a2cff8;
  color: #1d2125;
  height: 1.5rem;
}

.border-radius {
  border-radius: var(--bs-border-radius);
}

.v-hidden {
  visibility: hidden;
}

.dialog-big {
  max-width: 500px;
}

.dialog-small {
  max-width: 300px;
}

@media (min-width: 576px) {
  .dialog-big {
    width: 500px;
  }
  .dialog-small {
    width: 300px;
  }
}
/*
 * Helpers to show elements only when a parent element has focus or hover.
 */
.v-parent-focus {
  opacity: 0;
  visibility: hidden;
}

.focus-control:focus-within .v-parent-focus,
.focus-control:hover .v-parent-focus {
  opacity: 1;
  visibility: visible;
}

.emoji-picker {
  width: 350px;
  height: 400px;
}
.emoji-picker .category-button {
  padding: 0.375rem 0;
  height: 100%;
  width: 38.8888888889px;
  border-top: none;
  border-left: none;
  border-right: none;
  border-bottom: 2px solid transparent;
}
.emoji-picker .category-button.selected {
  border-bottom: 2px solid #0f6cbf;
}
.emoji-picker .emojis-container,
.emoji-picker .search-results-container {
  min-width: 280px;
}
.emoji-picker .picker-row {
  height: 40px;
}
.emoji-picker .picker-row .category-name {
  line-height: 40px;
}
.emoji-picker .picker-row .emoji-button {
  height: 40px;
  width: 40px;
  line-height: 40px;
  font-size: 24px;
  overflow: hidden;
}
.emoji-picker .emoji-preview {
  height: 40px;
  font-size: 40px;
  line-height: 40px;
}
.emoji-picker .emoji-short-name {
  line-height: 20px;
}
@media (max-width: 575.98px) {
  .emoji-picker {
    width: 320px;
  }
}

.emoji-auto-complete {
  height: 40px;
}
.emoji-auto-complete .btn.btn-icon.emoji-button {
  height: 40px;
  width: 40px;
  line-height: 40px;
  font-size: 24px;
}
.emoji-auto-complete .btn.btn-icon.emoji-button.active {
  background-color: #e9ecef;
}

.toast-wrapper {
  max-width: 350px;
  max-height: 0;
  z-index: 1051;
}
.toast-wrapper > :first-child {
  margin-top: 1rem;
}

.alert-primary a {
  color: #052644;
}
.alert-primary .btn-close {
  color: #020c14;
  opacity: 0.6;
}

.alert-secondary a {
  color: #636669;
}
.alert-secondary .btn-close {
  color: #4a4c4f;
  opacity: 0.6;
}

.alert-success a {
  color: #10250f;
}
.alert-success .btn-close {
  color: #000100;
  opacity: 0.6;
}

.alert-info a {
  color: #002127;
}
.alert-info .btn-close {
  color: black;
  opacity: 0.6;
}

.alert-warning a {
  color: #6a4c22;
}
.alert-warning .btn-close {
  color: #433016;
  opacity: 0.6;
}

.alert-danger a {
  color: #4d120c;
}
.alert-danger .btn-close {
  color: #210805;
  opacity: 0.6;
}

.alert-light a {
  color: #7b7b7d;
}
.alert-light .btn-close {
  color: #626263;
  opacity: 0.6;
}

.alert-dark a {
  color: black;
}
.alert-dark .btn-close {
  color: black;
  opacity: 0.6;
}

.alert a {
  font-weight: 700;
}

.breadcrumb:empty {
  padding: 0;
}

@media (max-width: 767.98px) {
  #page-navbar {
    width: 100%;
  }
  .breadcrumb:not(:empty) {
    width: 100%;
    flex-wrap: nowrap;
    margin-bottom: 0.5rem;
  }
  .breadcrumb:not(:empty) .breadcrumb-item {
    padding-top: 0.33333rem;
    padding-bottom: 0.33333rem;
    display: inline-flex;
    overflow: hidden;
  }
  .breadcrumb:not(:empty) .breadcrumb-item a,
  .breadcrumb:not(:empty) .breadcrumb-item span {
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap;
  }
  .mform {
    width: 100%;
    padding-right: 15px;
    padding-left: 15px;
  }
  .pagination {
    flex-wrap: wrap;
    justify-content: center;
  }
  .card .card-body {
    padding: 0.5rem;
  }
  #page-header .card {
    border: 0;
  }
  #page-header .card .card-body {
    padding: 0;
  }
  .nav-tabs:not(.more-nav),
  .nav-pills {
    --bs-nav-tabs-link-active-color: #fff;
    --bs-nav-tabs-link-active-bg: #0f6cbf;
    margin: 0;
    border: 0;
    padding: 0.125rem;
    background-color: #e9ecef;
  }
  .nav-tabs:not(.more-nav) .nav-item,
  .nav-pills .nav-item {
    flex: 1 1 auto;
    text-align: center;
  }
  .nav-tabs:not(.more-nav) .nav-link,
  .nav-pills .nav-link {
    border: 0;
    margin: 0.125rem;
  }
  .nav-tabs:not(.more-nav) .nav-link:not(.active),
  .nav-pills .nav-link:not(.active) {
    background-color: var(--bs-body-bg);
  }
}
@media (max-width: 576px) and (max-height: 320px) {
  div#page {
    margin-top: 0;
  }
  .navbar.fixed-top {
    position: relative;
    z-index: inherit;
  }
}
.link-underline {
  text-decoration: underline;
}
.link-underline:focus {
  text-decoration: none;
}

.alert.cta .cta-icon .icon {
  padding: 0.3rem;
}
.alert.cta .cta-icon .icon.fa {
  border-radius: 50%;
  border-style: solid;
  border-width: 0.125rem;
}

.core_payment_gateways_modal .form-check-label::before,
.core_payment_gateways_modal .form-check-label::after {
  top: 45%;
}

.visual-scroll-x {
  scrollbar-width: thin;
  scrollbar-color: #0f6cbf #a2cff8;
  -ms-overflow-style: -ms-autohiding-scrollbar;
}
.visual-scroll-x::-webkit-scrollbar {
  height: 8px;
  -webkit-appearance: none;
  appearance: none;
}
.visual-scroll-x::-webkit-scrollbar-thumb {
  background-color: #0f6cbf;
  border-right: 1px solid #fff;
}
.visual-scroll-x::-webkit-scrollbar-track {
  background-color: #a2cff8;
  border-right: 1px solid #fff;
}

body.dragging .drop-zone {
  border: 1px dashed #1d2125;
}
body.dragging .drop-up {
  border-top: 1px solid #1d2125;
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}
body.dragging .drop-down {
  border-bottom: 1px solid #1d2125;
  border-bottom-left-radius: 0;
  border-bottom-right-radius: 0;
}
body.dragging .dragging {
  opacity: 0.6;
}

.dragicon {
  visibility: hidden;
}

.draggable:hover .dragicon {
  visibility: visible;
  cursor: move;
}

.dropzone-container {
  cursor: pointer;
}
.dropzone-container .dropzone {
  display: flex;
  justify-content: center;
  align-items: center;
  flex-direction: column;
  border: 2px dashed #bbb;
  border-radius: 0.5rem;
}
.dropzone-container .dropzone.dragover {
  border: 2px dashed #6c8cd3;
}
.dropzone-container .dropzone-icon {
  color: #8f959e;
}
.dropzone-container .dropzone-icon .icon {
  font-size: 6em;
  width: auto;
  height: auto;
  max-width: initial;
  max-height: initial;
  margin-right: 0;
}
.dropzone-container .dropzone-visually-hidden-focusable:active, .dropzone-container .dropzone-visually-hidden-focusable:focus {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
  z-index: 1070;
  position: relative;
  background: #fff;
  padding: 7px;
}

.overlay-preview {
  background-color: rgba(255, 255, 255, 0.8);
  border: 2px dashed #0f6cbf;
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
}
.overlay-preview .overlay-preview-wrapper {
  position: absolute;
  top: 0;
  padding: 2rem;
  width: 100%;
}
.overlay-preview .overlay-preview-content {
  position: relative;
  top: 0;
  padding: 1rem;
  margin: 0 auto;
  width: 100%;
  max-width: 600px;
  background-color: #0f6cbf;
  color: #fff;
  text-align: center;
  font-size: 1.171875rem;
  border-radius: 0.5rem;
}

.overlay-preview-borders {
  outline: 2px dashed #0f6cbf;
}

.waitstate {
  display: none;
}

.stateready .waitstate {
  display: inherit;
}
.stateready .whilenostate {
  display: none;
}

.collapse-list .collapse-list-item {
  padding: 0.5rem 1rem;
}
.collapse-list .collapse-list-item:hover, .collapse-list .collapse-list-item:focus {
  background-color: #e6f2f5;
  border-color: #b3d9e0;
}
.collapse-list .collapse-list-item-content .collapse-list-item {
  padding-left: calc(1rem * 3);
}

.page-mycourses .block_myoverview .block-cards {
  border-top: 1px solid #dee2e6;
  padding-top: 0.5rem;
}
.page-mycourses .drawers .block_myoverview {
  border: 0;
}
.page-mycourses .drawers .block_myoverview > .card-body {
  padding-top: 0 !important; /* stylelint-disable-line declaration-no-important */
  padding-left: 0 !important; /* stylelint-disable-line declaration-no-important */
  padding-right: 0 !important; /* stylelint-disable-line declaration-no-important */
}

#page-my-index .my-action-buttons {
  display: flex;
  justify-content: center;
  align-items: center;
  gap: 10px;
}

.dropdown-toggle::after {
  font: var(--fa-font-solid);
  content: "\f078";
  margin-right: 0;
  margin-left: 4px;
  font-size: 9px;
  width: 9px;
  border: 0;
}

.listitem-category .dropdown-toggle::after {
  /* This can be removed when the .dropdown-toggle::after is updated to remove content. */
  content: none;
}

.dropleft .dropdown-toggle::before {
  border: 0;
  font: var(--fa-font-solid);
  content: "\f053";
  font-size: 9px;
  margin-left: 0;
  margin-right: 4px;
  width: 9px;
}

.dir-rtl .dropleft .dropdown-toggle::before {
  content: "\f054";
}

.dropright .dropdown-toggle::after {
  border: 0;
  font: var(--fa-font-solid);
  content: "\f054";
}

.dir-rtl .dropright .dropdown-toggle::after {
  content: "\f053";
}

.dropup .dropdown-toggle::after {
  border: 0;
  font: var(--fa-font-solid);
  content: "\f077";
}

.select-menu li:first-child ul[role=group] {
  padding: 0;
}
.select-menu ul[role=group] {
  padding: 0.3rem 0 0 0;
  margin: 0;
}
.select-menu ul[role=group] li:first-child {
  cursor: default;
  color: #6a737b;
  padding: 0.25rem 1.5rem;
  display: block;
}
.select-menu ul[role=group] .dropdown-item {
  padding-left: 2.5rem;
}
.select-menu .dropdown-item {
  padding: 0.25rem 1.5rem;
}
.select-menu .dropdown-item [aria-selected=true] {
  font-weight: bold;
}

[role=listbox] [role=option] {
  cursor: pointer;
}
[role=listbox] [role=option][aria-selected=true] {
  font-weight: bold;
}

.initialbargroups ul {
  -webkit-margin-start: 0; /* stylelint-disable-line */
  margin-right: -1px;
}

.initialbargroups .page-item:first-child .page-link {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}
.initialbargroups .pagination-lg:first-child .page-item:first-child .page-link {
  border-top-left-radius: var(--bs-border-radius-lg);
  border-bottom-left-radius: var(--bs-border-radius-lg);
}
.initialbargroups .pagination-sm:first-child .page-item:first-child .page-link {
  border-top-left-radius: var(--bs-border-radius-sm);
  border-bottom-left-radius: var(--bs-border-radius-sm);
}
.initialbargroups .page-item:last-child .page-link {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}
.initialbargroups .pagination-lg:last-child .page-item:last-child .page-link {
  border-top-right-radius: var(--bs-border-radius-lg);
  border-bottom-right-radius: var(--bs-border-radius-lg);
}
.initialbargroups .pagination-sm:last-child .page-item:last-child .page-link {
  border-top-right-radius: var(--bs-border-radius-sm);
  border-bottom-right-radius: var(--bs-border-radius-sm);
}

blockquote {
  margin: 0 0.5rem 1rem;
  padding-left: 1rem;
  color: #495057;
  border-left: 5px solid #8f959e;
}

.pagelayout-maintenance .page-header-headings,
.pagelayout-secure .page-header-headings {
  background-color: #fff;
}

/* Prevent long strings exceeding page width */
.page-header-headings:not(.text-truncate),
.coursename:not(.text-truncate),
.categoryname:not(.text-truncate),
.breadcrumb-item:not(.text-truncate) {
  word-break: normal;
  overflow-wrap: anywhere;
  white-space: normal;
}

/* Showmore component */
.showmore-container.collapsed .expanded-content {
  display: none;
}
.showmore-container:not(.collapsed) .collapsed-content {
  display: none;
}
.showmore-container .showmore-button {
  float: right;
  text-decoration: none;
}
.showmore-container .showmore-button .icon {
  font-size: 0.8203125rem;
  margin: 0;
}

/* Combobox search dropdowns */
.usersearchdropdown.dropdown-menu,
.gradesearchdropdown.dropdown-menu,
.groupsearchdropdown.dropdown-menu {
  top: 100%;
  width: 350px;
}
.usersearchdropdown.dropdown-menu .searchresultitemscontainer,
.gradesearchdropdown.dropdown-menu .searchresultitemscontainer,
.groupsearchdropdown.dropdown-menu .searchresultitemscontainer {
  max-height: 170px;
  overflow: auto;
  /* stylelint-disable declaration-no-important */
}
.usersearchdropdown.dropdown-menu .searchresultitemscontainer img,
.gradesearchdropdown.dropdown-menu .searchresultitemscontainer img,
.groupsearchdropdown.dropdown-menu .searchresultitemscontainer img {
  height: 48px !important;
  width: 48px !important;
}

/* Bulk actions in sticky footer. */
#sticky-footer [data-type=bulkactions] {
  display: flex;
  flex: 0 0 100%;
  align-items: center;
}

/* Choice list component. */
.choicelist {
  min-width: calc(300px - 25px);
}
.choicelist i.icon {
  vertical-align: middle;
}

/* General card customisations. */
.card.active {
  border-color: #8f959e;
}

.action-menu .dropdown-toggle {
  text-decoration: none;
  display: inline-block;
}

.action-menu {
  white-space: nowrap;
  display: inline;
}
.action-menu .dropdown-toggle.no-caret::after {
  display: none;
}
.action-menu .dropdown-toggle.no-caret::before {
  display: none;
}
.action-menu .dropdown.downleft .dropdown-subpanel-content {
  right: 0;
  left: auto;
}
.action-menu .dropdown-subpanel.content-displayed {
  background-color: #e9ecef;
}
.action-menu .dropdown-subpanel-content {
  max-width: 300px;
  box-shadow: 0 0 1rem rgba(0, 0, 0, 0.15);
}
.action-menu .dropdown-subpanel-content.show {
  animation: 0.15s animate-pop;
}
@media (prefers-reduced-motion: reduce) {
  .action-menu .dropdown-subpanel-content.show {
    animation: none;
  }
}
body.behat-site .action-menu .dropdown-subpanel-content.show {
  animation: none;
}
.action-menu .dropdown-subpanel .dropdown-item::after {
  border: 0;
  font: var(--fa-font-solid);
  content: "\f054";
}
.action-menu .dropdown-subpanel .dropdown-item::before {
  display: none;
}

@keyframes animate-pop {
  0% {
    transform: scale(0.9, 0.9);
  }
  100% {
    transform: scale(1, 1);
  }
}
.dir-rtl .action-menu .dropdown-subpanel .dropdown-item::after {
  border: 0;
  font: var(--fa-font-solid);
  content: "\f053";
}
.dir-rtl .action-menu .dropdown-subpanel .dropdown-item::before {
  display: none;
}

.dropdown-item a {
  display: block;
  width: 100%;
  color: #1d2125;
}
.dropdown-item.active a, .dropdown-item:active a, .dropdown-item:hover a, .dropdown-item:focus a, .dropdown-item:focus-within a {
  color: #fff;
}
.dropdown-item[aria-current=true], .dropdown-item[aria-selected=true] {
  position: relative;
  display: flex;
  align-items: center;
}
.dropdown-item[aria-current=true]:before, .dropdown-item[aria-selected=true]:before {
  font: var(--fa-font-solid);
  content: "\f00c";
  position: absolute;
  left: 0.4rem;
  font-size: 0.7rem;
}

.dropdown-item-outline:focus, .dropdown-item-outline:focus-within {
  outline: solid #0f6cbf;
}
.dropdown-item-outline a:focus,
.dropdown-item-outline a:focus-visible {
  outline: 0;
}

.icon {
  max-width: 30px;
  max-height: 24px;
  margin: 0 0.5rem 0 0;
  padding: 0;
  box-sizing: content-box;
}
.icon.spacer {
  margin-right: 0;
}
.icon.iconsize-medium {
  font-size: 32px;
  width: 40px;
  height: 32px;
  max-width: 40px;
  max-height: 32px;
}
.icon.iconsize-big {
  width: 80px;
  height: 64px;
  max-width: 80px;
  max-height: 64px;
  font-size: 64px;
}
.icon.emoticon {
  width: 16px;
  height: 16px;
}

img.icon {
  width: 20px;
  height: 16px;
}

.navbar-dark a .icon {
  color: rgba(255, 255, 255, 0.55) !important; /* stylelint-disable-line declaration-no-important */
}

.action-menu-item a:first-of-type > .icon {
  margin-left: 0.5rem;
}

.ygtvcell .icon {
  margin-left: 0 !important; /* stylelint-disable-line declaration-no-important */
}

.block_navigation .tree_item .icon,
.block_settings .tree_item .icon {
  margin-left: 0;
}

[data-action=toggle-drawer] .icon {
  margin: 0;
}

.icon-no-spacing a > .icon {
  margin: 0;
}

.icon-no-margin .icon {
  margin-left: 0;
  margin-right: 0;
  margin-top: 0;
  margin-bottom: 0;
}

.icon-large > .icon {
  width: 40px;
  height: 32px;
}

.icon-size-0 .icon {
  height: 0 !important; /* stylelint-disable-line declaration-no-important */
  width: 0 !important; /* stylelint-disable-line declaration-no-important */
  max-height: 0 !important; /* stylelint-disable-line declaration-no-important */
  max-width: 0 !important; /* stylelint-disable-line declaration-no-important */
  font-size: 0 !important; /* stylelint-disable-line declaration-no-important */
}

.icon-size-1 .icon {
  height: 4px !important; /* stylelint-disable-line declaration-no-important */
  width: 5px !important; /* stylelint-disable-line declaration-no-important */
  max-height: 4px !important; /* stylelint-disable-line declaration-no-important */
  max-width: 5px !important; /* stylelint-disable-line declaration-no-important */
  font-size: 4px !important; /* stylelint-disable-line declaration-no-important */
}

.icon-size-2 .icon {
  height: 8px !important; /* stylelint-disable-line declaration-no-important */
  width: 10px !important; /* stylelint-disable-line declaration-no-important */
  max-height: 8px !important; /* stylelint-disable-line declaration-no-important */
  max-width: 10px !important; /* stylelint-disable-line declaration-no-important */
  font-size: 8px !important; /* stylelint-disable-line declaration-no-important */
}

.icon-size-3 .icon {
  height: 16px !important; /* stylelint-disable-line declaration-no-important */
  width: 20px !important; /* stylelint-disable-line declaration-no-important */
  max-height: 16px !important; /* stylelint-disable-line declaration-no-important */
  max-width: 20px !important; /* stylelint-disable-line declaration-no-important */
  font-size: 16px !important; /* stylelint-disable-line declaration-no-important */
}

.icon-size-4 .icon {
  height: 24px !important; /* stylelint-disable-line declaration-no-important */
  width: 30px !important; /* stylelint-disable-line declaration-no-important */
  max-height: 24px !important; /* stylelint-disable-line declaration-no-important */
  max-width: 30px !important; /* stylelint-disable-line declaration-no-important */
  font-size: 24px !important; /* stylelint-disable-line declaration-no-important */
}

.icon-size-5 .icon {
  height: 32px !important; /* stylelint-disable-line declaration-no-important */
  width: 40px !important; /* stylelint-disable-line declaration-no-important */
  max-height: 32px !important; /* stylelint-disable-line declaration-no-important */
  max-width: 40px !important; /* stylelint-disable-line declaration-no-important */
  font-size: 32px !important; /* stylelint-disable-line declaration-no-important */
}

.icon-size-6 .icon {
  height: 40px !important; /* stylelint-disable-line declaration-no-important */
  width: 50px !important; /* stylelint-disable-line declaration-no-important */
  max-height: 40px !important; /* stylelint-disable-line declaration-no-important */
  max-width: 50px !important; /* stylelint-disable-line declaration-no-important */
  font-size: 40px !important; /* stylelint-disable-line declaration-no-important */
}

.icon-size-7 .icon {
  height: 48px !important; /* stylelint-disable-line declaration-no-important */
  width: 60px !important; /* stylelint-disable-line declaration-no-important */
  max-height: 48px !important; /* stylelint-disable-line declaration-no-important */
  max-width: 60px !important; /* stylelint-disable-line declaration-no-important */
  font-size: 48px !important; /* stylelint-disable-line declaration-no-important */
}

.helplink .icon {
  margin-left: 0.5rem;
}

.icons-collapse-expand {
  display: flex;
  align-items: center;
}
.icons-collapse-expand .expanded-icon {
  display: flex;
  align-items: center;
}
.icons-collapse-expand .collapsed-icon {
  display: none;
}
.icons-collapse-expand.collapsed .expanded-icon {
  display: none;
}
.icons-collapse-expand.collapsed .collapsed-icon {
  display: flex;
  align-items: center;
}

.icon-box {
  width: 60px;
  height: 48px;
  display: inline-flex;
  justify-content: center;
  align-items: center;
  background-color: #f8f9fa;
  border-radius: 12px;
  padding: 0.7rem;
}
.icon-box .icon {
  margin: 0;
  height: 40px;
  width: 32px;
}

.icon-circle {
  display: inline-block;
  background-color: #c2cad1;
  border-radius: 50%;
  padding: 1.3rem;
}
.icon-circle .icon {
  margin: 0;
  text-align: center;
  max-width: unset;
  max-height: unset;
  color: #000;
}
.icon-circle.reversed {
  background-color: #b7c0c8;
}
.icon-circle.reversed .icon {
  color: #fff;
}

/*
 * Activity icons
 */
.activityiconcontainer {
  width: 52px;
  height: 52px;
  max-width: 52px;
  max-height: 52px;
}
.activityiconcontainer.small {
  width: 42px;
  height: 42px;
  max-width: 42px;
  max-height: 42px;
}
.activityiconcontainer.smaller {
  width: 32px;
  height: 32px;
  max-width: 32px;
  max-height: 32px;
}
.activityiconcontainer .activityicon,
.activityiconcontainer .icon {
  margin: 0;
  font-size: inherit;
  width: inherit;
  height: inherit;
  max-width: inherit;
  max-height: inherit;
}

.activityiconcontainer.administration:not(.isbranded) .activityicon:not(.nofilter),
.activityiconcontainer.administration:not(.isbranded) .icon:not(.nofilter) {
  filter: url('data:image/svg+xml;utf8,<svg xmlns="http://www.w3.org/2000/svg"><filter id="recolor" color-interpolation-filters="sRGB"><feColorMatrix type="matrix" values="0 0 0 0 0.8549019608 0 0 0 0 0.3450980392 0 0 0 0 0.937254902 0 0 0 1 0 "/></filter></svg>#recolor');
}

.activityiconcontainer.assessment:not(.isbranded) .activityicon:not(.nofilter),
.activityiconcontainer.assessment:not(.isbranded) .icon:not(.nofilter) {
  filter: url('data:image/svg+xml;utf8,<svg xmlns="http://www.w3.org/2000/svg"><filter id="recolor" color-interpolation-filters="sRGB"><feColorMatrix type="matrix" values="0 0 0 0 0.9764705882 0 0 0 0 0 0 0 0 0 0.5254901961 0 0 0 1 0 "/></filter></svg>#recolor');
}

.activityiconcontainer.collaboration:not(.isbranded) .activityicon:not(.nofilter),
.activityiconcontainer.collaboration:not(.isbranded) .icon:not(.nofilter) {
  filter: url('data:image/svg+xml;utf8,<svg xmlns="http://www.w3.org/2000/svg"><filter id="recolor" color-interpolation-filters="sRGB"><feColorMatrix type="matrix" values="0 0 0 0 0.3568627451 0 0 0 0 0.2509803922 0 0 0 0 1 0 0 0 1 0 "/></filter></svg>#recolor');
}

.activityiconcontainer.communication:not(.isbranded) .activityicon:not(.nofilter),
.activityiconcontainer.communication:not(.isbranded) .icon:not(.nofilter) {
  filter: url('data:image/svg+xml;utf8,<svg xmlns="http://www.w3.org/2000/svg"><filter id="recolor" color-interpolation-filters="sRGB"><feColorMatrix type="matrix" values="0 0 0 0 0.9215686275 0 0 0 0 0.3843137255 0 0 0 0 0 0 0 0 1 0 "/></filter></svg>#recolor');
}

.activityiconcontainer.content:not(.isbranded) .activityicon:not(.nofilter),
.activityiconcontainer.content:not(.isbranded) .icon:not(.nofilter) {
  filter: url('data:image/svg+xml;utf8,<svg xmlns="http://www.w3.org/2000/svg"><filter id="recolor" color-interpolation-filters="sRGB"><feColorMatrix type="matrix" values="0 0 0 0 0 0 0 0 0 0.6 0 0 0 0 0.6784313725 0 0 0 1 0 "/></filter></svg>#recolor');
}

.activityiconcontainer.interactivecontent:not(.isbranded) .activityicon:not(.nofilter),
.activityiconcontainer.interactivecontent:not(.isbranded) .icon:not(.nofilter) {
  filter: url('data:image/svg+xml;utf8,<svg xmlns="http://www.w3.org/2000/svg"><filter id="recolor" color-interpolation-filters="sRGB"><feColorMatrix type="matrix" values="0 0 0 0 0.5529411765 0 0 0 0 0.2392156863 0 0 0 0 0.1058823529 0 0 0 1 0 "/></filter></svg>#recolor');
}

:root {
  --activityadministration: #da58ef;
  --activityassessment: #f90086;
  --activitycollaboration: #5b40ff;
  --activitycommunication: #eb6200;
  --activitycontent: #0099ad;
  --activityinteractivecontent: #8d3d1b;
}

/* admin.less */
.formtable tbody th {
  font-weight: normal;
  text-align: right;
}

.path-admin #assignrole {
  width: 60%;
  margin-left: auto;
  margin-right: auto;
}

.path-admin .admintable .leftalign {
  text-align: left;
}

.path-admin .admintable.environmenttable .name,
.path-admin .admintable.environmenttable .info,
.path-admin #assignrole .admintable .role,
.path-admin #assignrole .admintable .userrole,
.path-admin #assignrole .admintable .roleholder {
  white-space: nowrap;
}

.path-admin .incompatibleblockstable td.c0 {
  font-weight: bold;
}

#page-admin-course-category .addcategory {
  padding: 10px;
}

#page-admin-course-index .editcourse {
  margin: 20px auto;
}

#page-admin-course-index .editcourse th,
#page-admin-course-index .editcourse td {
  padding-left: 10px;
  padding-right: 10px;
}

.timewarninghidden {
  display: none;
}

#page-admin-qtypes #qtypes div,
#page-admin-qtypes #qtypes form,
#page-admin-qbehaviours #qbehaviours div,
#page-admin-qbehaviours #qbehaviours form {
  display: inline;
}

#page-admin-qtypes #qtypes img.spacer,
#page-admin-qbehaviours #qbehaviours img.spacer {
  width: 16px;
}

#page-admin-qbehaviours .cell.c3,
#page-admin-qtypes .cell.c3 {
  font-size: 0.8203125rem;
}

#page-admin-lang .generalbox,
#page-admin-course-index .singlebutton,
#page-admin-course-index .addcategory,
#page-course-index .buttons,
#page-course-index-category .buttons,
#page-admin-course-category .addcategory,
#page-admin-stickyblocks .generalbox,
#page-admin-maintenance .buttons,
#page-admin-course-index .buttons,
#page-admin-course-category .buttons,
#page-admin-index .copyright,
#page-admin-index .copyrightnotice,
#page-admin-index .adminerror .singlebutton,
#page-admin-index .adminwarning .singlebutton,
#page-admin-index #layout-table .singlebutton {
  text-align: center;
  margin-bottom: 1em;
}

.path-admin-roles .capabilitysearchui {
  text-align: left;
  margin-left: auto;
  margin-right: auto;
  margin-top: 1rem;
}

#page-admin-roles-define .topfields {
  margin: 1em 0 2em;
}

#page-admin-roles-override .capcurrent,
#page-admin-roles-define .capdefault {
  background-color: rgba(var(--bs-emphasis-color-rgb), 0.075);
}

#page-filter-manage .backlink,
.path-admin-roles .backlink {
  margin-top: 1em;
}

#page-admin-roles-explain #chooseuser h3, #page-admin-roles-explain #chooseuser .h3,
#page-admin-roles-usersroles .contextname {
  margin-top: 0;
}

#page-admin-roles-explain #chooseusersubmit {
  margin-top: 0;
  text-align: center;
}

#page-admin-roles-usersroles p {
  margin: 0;
}

#page-admin-roles-override .cell.c1,
#page-admin-roles-assign .cell.c3,
#page-admin-roles-assign .cell.c1 {
  padding-top: 0.75em;
}

#page-admin-roles-override .overridenotice,
#page-admin-roles-define .definenotice {
  margin: 1em 10% 2em 10%;
  text-align: left;
}

#page-admin-index .upgradekeyreq {
  min-width: 220px;
}

#page-admin-index .adminwarning.availableupdatesinfo .moodleupdateinfo span {
  display: block;
}

#page-admin-index .updateplugin div {
  margin-bottom: 0.5em;
}

#page-admin-user-user_bulk #users .fgroup {
  white-space: nowrap;
}

#page-admin-report-stats-index .graph {
  text-align: center;
  margin-bottom: 1em;
}

#page-admin-report-courseoverview-index .graph {
  text-align: center;
  margin-bottom: 1em;
}

#page-admin-lang .translator {
  border-width: 1px;
  border-style: solid;
}

.path-admin .roleassigntable {
  width: 100%;
}

.path-admin .roleassigntable td {
  vertical-align: top;
  padding: 0.2em 0.3em;
}

.path-admin .roleassigntable p {
  text-align: left;
  margin: 0.2em 0;
}

.path-admin .roleassigntable #existingcell,
.path-admin .roleassigntable #potentialcell {
  width: 42%;
}

.path-admin .roleassigntable #existingcell p > label:first-child,
.path-admin .roleassigntable #potentialcell p > label:first-child {
  font-weight: bold;
}

.path-admin .roleassigntable #buttonscell {
  width: 16%;
}

.path-admin .roleassigntable #buttonscell #assignoptions {
  font-size: 0.8203125rem;
}

.path-admin .roleassigntable #removeselect_wrapper,
.path-admin .roleassigntable #addselect_wrapper {
  width: 100%;
}

.path-admin table.rolecap tr.rolecap th {
  text-align: left;
  font-weight: normal;
}

.path-admin .rolecap .hiddenrow {
  display: none;
}

.path-admin #defineroletable .rolecap .inherit,
.path-admin #defineroletable .rolecap .allow,
.path-admin #defineroletable .rolecap .prevent,
.path-admin #defineroletable .rolecap .prohibit {
  text-align: center;
  padding: 0;
  min-width: 3.5em;
}

.path-admin .rolecap .cap-name,
.path-admin .rolecap .note {
  display: block;
  font-size: 0.8203125rem;
  white-space: nowrap;
  font-weight: normal;
}

.path-admin .rolecap label {
  display: block;
  text-align: center;
  padding: 0.5em;
  margin: 0;
}

.path-admin .header-maxwidth,
.path-admin .secondary-navigation .navigation .nav-tabs,
.format-site .header-maxwidth,
.format-site .secondary-navigation .navigation .nav-tabs {
  max-width: none;
}

.path-admin.path-admin-roles:not(.format-site) .header-maxwidth,
.path-admin.path-admin-roles:not(.format-site) .secondary-navigation .navigation .nav-tabs,
.path-admin.path-admin-tool-lp .header-maxwidth,
.path-admin.path-admin-tool-lp .secondary-navigation .navigation .nav-tabs {
  max-width: 830px;
}

.plugincheckwrapper {
  width: 100%;
}

.environmentbox {
  margin-top: 1em;
}

#mnetconfig table {
  margin-left: auto;
  margin-right: auto;
}

.environmenttable .cell {
  padding: 0.15em 0.5em;
}

#trustedhosts .table {
  margin-left: auto;
  margin-right: auto;
  width: 500px;
}

#trustedhosts .standard {
  width: auto;
}

#adminsettings legend {
  display: none;
}

#adminsettings fieldset.error {
  margin: 0.2em 0 0.5em 0;
}

#adminsettings fieldset.error legend {
  display: block;
}

#admin-spelllanguagelist textarea {
  /* rtl:ignore */
  text-align: left;
  /* rtl:ignore */
  direction: ltr;
}

/* Styles for flags on admin settings */
.adminsettingsflags {
  float: right;
}

.adminsettingsflags label {
  margin-right: 7px;
}

.form-description pre,
.formsettingheading pre {
  /*rtl:ignore*/
  direction: ltr;
}

.form-item .form-setting .form-htmlarea {
  display: inline;
}

.form-item .form-setting .form-htmlarea .htmlarea {
  width: 640px;
  display: block;
}

.form-item .form-setting .form-multicheckbox ul {
  list-style: none;
  padding: 0;
  margin: 7px 0 0 0;
}

.form-item .form-setting .defaultsnext {
  display: inline-block;
}

.form-item .form-setting .locked-checkbox {
  margin-right: 0.2em;
  margin-left: 0.5em;
  display: inline;
}

.form-item .form-setting .form-password .unmask,
.form-item .form-setting .form-defaultinfo {
  display: inline-block;
}

.form-item .form-setting .form-defaultinfo {
  max-width: 100%;
  word-wrap: break-word;
}

#admin-emoticons td input {
  width: 8em;
}

#admin-emoticons td.c0 input {
  width: 4em;
}

.admin_colourpicker,
.admin_colourpicker_preview {
  display: none;
}

.jsenabled .admin_colourpicker_preview {
  display: inline;
}

@media (min-width: 768px) {
  .jsenabled .admin_colourpicker {
    display: block;
    height: 102px;
    width: 410px;
    margin-bottom: 10px;
    box-sizing: content-box;
  }
  .admin_colourpicker .colourdialogue {
    float: left;
    border: 1px solid var(--bs-gray-500);
  }
  .admin_colourpicker .previewcolour {
    border: 1px solid var(--bs-gray-500);
    margin-left: 301px;
  }
  .admin_colourpicker .currentcolour {
    border: 1px solid var(--bs-gray-500);
    margin-left: 301px;
    border-top-width: 0;
  }
}
@media (max-width: 767.98px) {
  .jsenabled .admin_colourpicker {
    height: 150px;
    margin-bottom: 10px;
    display: block;
    position: relative;
  }
  .admin_colourpicker .previewcolour {
    display: none;
  }
  .admin_colourpicker .currentcolour {
    position: absolute;
    border: 1px solid #dee2e6;
    top: 100px;
    left: 0;
  }
}
.admin_colourpicker .loadingicon {
  vertical-align: middle;
  margin-left: auto;
}

#page-admin-index #notice .checkforupdates {
  text-align: center;
}

#page-admin-index #page-content .alert a {
  font-weight: normal;
  text-decoration: underline;
}

#page-admin-plugins #plugins-overview-panel .info {
  display: inline-block;
  margin-right: 1em;
}
#page-admin-plugins #plugins-overview-panel .info [data-filterby].active {
  font-weight: bold;
}
#page-admin-plugins #plugins-overview-panel .updateavailableinstallall {
  margin-right: 1em;
}
#page-admin-plugins .checkforupdates {
  margin: 10px 0;
}
#page-admin-plugins .checkforupdates .singlebutton {
  margin: 5px 0;
  padding: 0;
}
#page-admin-plugins .checkforupdates .singlebutton div,
#page-admin-plugins .checkforupdates .singlebutton input {
  margin: 0 3px 0 0;
}
#page-admin-plugins .updateavailableinstallall {
  margin: 5px 0;
  padding: 0;
}
#page-admin-plugins .updateavailableinstallall div,
#page-admin-plugins .updateavailableinstallall input {
  margin: 0 3px 5px 0;
}
#page-admin-plugins #plugins-control-panel .status-missing td,
#page-admin-plugins #plugins-control-panel .deprecatedtype td {
  background-color: #fcefdc;
}
#page-admin-plugins #plugins-control-panel .pluginname .componentname {
  font-size: 0.8203125rem;
  color: var(--bs-secondary-color);
  margin-left: 22px;
}
#page-admin-plugins #plugins-control-panel .version .versionnumber {
  font-size: 0.8203125rem;
  color: var(--bs-secondary-color);
}
#page-admin-plugins #plugins-control-panel .uninstall a {
  color: #ca3120;
}
#page-admin-plugins #plugins-control-panel .notes .label {
  margin-right: 3px;
}
#page-admin-plugins #plugins-control-panel .notes .requiredby {
  font-size: 0.8203125rem;
  color: var(--bs-secondary-color);
}

#plugins-check-page #plugins-check .status-missing td,
#plugins-check-page #plugins-check .deprecatedtype td,
#plugins-check-page #plugins-check .status-downgrade td {
  background-color: #f4d6d2;
}

#plugins-check-page .pluginupdateinfo,
#plugins-control-panel .pluginupdateinfo {
  background-color: #cce6ea;
  padding: 5px;
  margin: 10px 0;
  border-radius: 5px;
}
#plugins-check-page .pluginupdateinfo.maturity50,
#plugins-control-panel .pluginupdateinfo.maturity50 {
  background-color: #f4d6d2;
}
#plugins-check-page .pluginupdateinfo.maturity100, #plugins-check-page .pluginupdateinfo.maturity150,
#plugins-control-panel .pluginupdateinfo.maturity100,
#plugins-control-panel .pluginupdateinfo.maturity150 {
  background-color: #fcefdc;
}
#plugins-check-page .pluginupdateinfo .info,
#plugins-control-panel .pluginupdateinfo .info {
  display: inline-block;
}
#plugins-check-page .pluginupdateinfo .separator:after,
#plugins-control-panel .pluginupdateinfo .separator:after {
  content: " | ";
}
#plugins-check-page .pluginupdateinfo .singlebutton,
#plugins-control-panel .pluginupdateinfo .singlebutton {
  margin: 5px 0;
  padding: 0;
}
#plugins-check-page .pluginupdateinfo .singlebutton div,
#plugins-check-page .pluginupdateinfo .singlebutton input,
#plugins-control-panel .pluginupdateinfo .singlebutton div,
#plugins-control-panel .pluginupdateinfo .singlebutton input {
  margin: 0 3px 0 0;
}

.plugins-management-confirm-buttons > div {
  display: inline-block;
  margin: 1em 1em 1em 0;
}
.plugins-management-confirm-buttons .continue {
  padding: 0;
}
.plugins-management-confirm-buttons .continue div,
.plugins-management-confirm-buttons .continue input {
  margin: 0;
}

#page-admin-index .upgradepluginsinfo {
  text-align: center;
}

#page-admin-index .adminwarning.availableupdatesinfo .moodleupdateinfo .separator:after {
  content: " | ";
}

/** MNet networking */
#page-admin-mnet-peers .box.deletedhosts {
  margin-bottom: 1em;
  font-size: 0.8203125rem;
}

#core-cache-plugin-summaries table,
#core-cache-store-summaries table {
  width: 100%;
}

#core-cache-lock-summary table,
#core-cache-definition-summaries table,
#core-cache-mode-mappings table {
  margin: 0 auto;
}

#core-cache-store-summaries .default-store td {
  font-style: italic;
}

#core-cache-rescan-definitions,
#core-cache-mode-mappings .edit-link,
#core-cache-lock-additional-actions .new-instance {
  margin-top: 0.5em;
  text-align: center;
}

.maintenancewarning {
  position: fixed;
  bottom: 0;
  right: 0;
  margin: 1rem;
  overflow: hidden;
  z-index: 1000;
}

.jsenabled .maintenancewarning {
  transition: 0.2s;
}
@media (prefers-reduced-motion: reduce) {
  .jsenabled .maintenancewarning {
    transition: none;
  }
}

.modal.modal-in-page {
  z-index: 0;
}

#page-admin-search .adminpagetitle {
  margin-bottom: 0;
  border-bottom: none;
}
#page-admin-search .adminpagepath {
  display: flex;
  flex-wrap: wrap;
  list-style: none;
  padding: 0;
  margin: 0 0 1.5rem 0;
}
#page-admin-search .adminpagepath li + li:before {
  padding-right: 0.5rem;
  padding-left: 0.5rem;
  content: "/";
}
@media (min-width: 576px) {
  #page-admin-search .container {
    overflow-wrap: break-word;
  }
}

#page-admin-tasklogs .task-class {
  font-size: 0.8203125rem;
  color: #6a737b;
}

.path-admin-tool-uploaduser .uuwarning {
  background-color: #fcefdc;
}
.path-admin-tool-uploaduser .uuerror {
  background-color: #f4d6d2;
}
.path-admin-tool-uploaduser .uuinfo {
  background-color: #d7e4d6;
}

/**
 Blocks
 */
.blockmovetarget .accesshide {
  position: relative;
  left: initial;
}

.block:target {
  padding-top: 0 !important; /* stylelint-disable declaration-no-important */
  margin-top: 0 !important;
}

.block_search_forums .searchform {
  /* Override plugin's default. */
  text-align: left;
}

.block.block_navigation .block_tree ul,
.block_settings .block_tree ul {
  margin-left: 0;
}

.block .block-controls .dropdown-toggle {
  /* So that the icon takes the colour of the icon. */
  color: #1d2125;
}
.block .block-controls .dropdown-toggle::after {
  display: none;
}
.block .block-controls .dropdown-menu {
  max-width: 500px;
}
@media (max-width: 575.98px) {
  .block .block-controls .dropdown-menu {
    max-width: 85vw;
  }
}
.block .block-controls .dropdown-menu .dropdown-item {
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
}

[data-region=blocks-column] {
  width: 360px;
  float: right;
}

/* We put an absolutely positioned div in a relatively positioned div so it takes up no space */
@media (min-width: 576px) {
  #region-main-settings-menu {
    position: relative;
    float: left;
    width: 100%;
  }
  #region-main-settings-menu > div {
    position: absolute;
    right: 0;
    z-index: 100;
    margin: 1rem;
  }
  .region_main_settings_menu_proxy {
    width: 4rem;
    height: 2rem;
    background-color: #fff;
    margin-left: 0.5rem;
    margin-bottom: 0.5rem;
    border-bottom-left-radius: 0.5rem;
    float: right;
  }
}
@media (max-width: 767.98px) {
  #region-main-settings-menu .menubar {
    justify-content: flex-end;
  }
}
#region-main.has-blocks {
  display: inline-block;
  width: calc(100% - calc(360px + (1.5rem * 0.5)));
}
@media (max-width: 1199.98px) {
  #region-main.has-blocks {
    width: 100%;
    /* MDL-63102 - Remove extra space at bottom.
    If modifying make sure block-region is horizontally stacked when in full screen */
    display: block;
  }
}

.header-action #region-main-settings-menu {
  position: unset;
  float: none;
  width: auto;
}
.header-action #region-main-settings-menu > div {
  position: unset;
  right: auto;
  margin: 0;
}

@media (max-width: 1199.98px) {
  [data-region=blocks-column] {
    width: 100%;
  }
}

.block .empty-placeholder-image-lg {
  height: 5rem;
}

.block .searchbar .icon {
  margin-right: 0;
}

.block .block-cards .course-info-container {
  padding: 0.8rem;
}
.block .block-cards .progress {
  height: 0.5rem;
}
.block .block-cards .course-summaryitem {
  border: 1px solid #dee2e6;
  background-color: #fff;
}
.block .block-cards .icon {
  margin-right: 0;
}
.block .block-cards .card .coursemenubtn {
  margin-top: -0.5rem;
}
.block .block-cards span.categoryname,
.block .block-cards .btn-link {
  color: #1d2125;
}
.block .block-cards .progress-text {
  color: #6a737b;
}
.block .block-cards .multiline {
  white-space: normal;
}

.card-grid {
  display: flex;
}
.card-grid .card {
  flex: 1;
}
.drawercontent .card-grid .col, .blockcolumn .card-grid .col {
  flex: 0 0 auto;
  max-width: 100%;
}

.card-carousel {
  display: flex;
}
.card-carousel .card {
  flex: 1;
}
@media (min-width: 576px) {
  .card-carousel .card {
    flex: 0 1 auto;
    width: 240px;
    max-width: 100%;
  }
}

.course-card .card-img-top,
.theme-card .card-img-top {
  height: 7rem;
  background-position: center;
  background-size: cover;
}

.block_recentlyaccessedcourses .paging-bar-container {
  margin-top: -2.4rem;
  padding-right: 0.5rem;
  justify-content: flex-end;
}
@media (max-width: 575.98px) {
  .block_recentlyaccessedcourses .paging-bar-container {
    margin-top: 0;
  }
}

#block-region-side-pre .block_recentlyaccessedcourses .paging-bar-container {
  margin-top: 0;
}

.block_recentlyaccesseditems .activityiconcontainer {
  width: 40px;
  height: 40px;
}
aside[id^=block-region-side-] .block_recentlyaccesseditems .card:nth-of-type(n+4) {
  display: none;
}
#block-region-content .block_recentlyaccesseditems [data-region=more-items-button-container] {
  display: none;
}
.block_recentlyaccesseditems a.card:hover, .block_recentlyaccesseditems a.card:focus {
  text-decoration: none;
}
.block_recentlyaccesseditems a.card:hover h6, .block_recentlyaccesseditems a.card:hover .h6, .block_recentlyaccesseditems a.card:focus h6, .block_recentlyaccesseditems a.card:focus .h6 {
  text-decoration: underline;
}
.block_recentlyaccesseditems a.card small, .block_recentlyaccesseditems a.card .small {
  color: #1d2125;
}

.block_myoverview .content {
  min-height: 19.35rem;
}
.block_myoverview .paged-content-page-container {
  min-height: 13rem;
}
.block_myoverview .summary-image {
  height: 5rem;
  width: 5rem;
  background-position: center;
  background-size: cover;
}
.block_myoverview .list-image {
  height: 5rem;
  width: 20rem;
  background-position: center;
  background-size: cover;
}
@media (max-width: 1199.98px) {
  .block_myoverview .list-image {
    width: 100%;
  }
}

.block_timeline .paged-content-page-container {
  background-color: var(--bs-body-bg);
}

.block_timeline .event-action {
  padding-left: 5.55em;
}

.block_settings .block_tree [aria-expanded=true],
.block_settings .block_tree [aria-expanded=true].emptybranch,
.block_settings .block_tree [aria-expanded=false],
.block_navigation .block_tree [aria-expanded=true],
.block_navigation .block_tree [aria-expanded=true].emptybranch,
.block_navigation .block_tree [aria-expanded=false] {
  background-image: none;
}

.block_settings .block_tree [aria-expanded=true] > p:before,
.block_navigation .block_tree [aria-expanded=true] > p:before {
  font: var(--fa-font-solid);
  content: "\f107";
  margin-right: 0;
  font-size: 16px;
  width: 16px;
}

.block_settings .block_tree [aria-expanded=false] > p:before,
.block_navigation .block_tree [aria-expanded=false] > p:before {
  font: var(--fa-font-solid);
  content: "\f105";
  margin-right: 0;
  font-size: 16px;
  width: 16px;
}

.dir-rtl .block_settings .block_tree [aria-expanded=false] > p:before,
.dir-rtl .block_navigation .block_tree [aria-expanded=false] > p:before {
  font: var(--fa-font-solid);
  content: "\f104";
}

.block_navigation .block_tree p.hasicon,
.block_settings .block_tree p.hasicon {
  text-indent: -3px;
}
.block_navigation .block_tree p.hasicon .icon,
.block_settings .block_tree p.hasicon .icon {
  margin-right: 2px;
}

.block.invisibleblock .card-title {
  color: var(--bs-secondary-color);
}

.block_tree .tree_item.branch {
  margin-left: 8px;
}

.pagelayout-embedded .has-fake-blocks {
  padding: 1rem;
  display: flex;
}
.pagelayout-embedded .has-fake-blocks .embedded-main {
  order: 0;
  width: calc(100% - 360px);
  margin-right: 1rem;
}
.pagelayout-embedded .embedded-blocks {
  order: 1;
  width: 360px;
}
@media (max-width: 767.98px) {
  .pagelayout-embedded .has-fake-blocks {
    display: block;
  }
  .pagelayout-embedded .has-fake-blocks .embedded-main {
    width: 100%;
  }
  .pagelayout-embedded .embedded-blocks {
    width: 100%;
  }
}

.block-add {
  color: #0f6cbf;
  border-color: #3f89cc;
  background-color: #e7f0f9;
  border-radius: 0.5rem;
  width: 100%;
  border-width: 1px;
}
.block-add .pluscontainer {
  border: 1px solid #3f89cc;
  border-radius: 50%;
  width: 40px;
  height: 32px;
}
.block-add:hover {
  cursor: pointer;
  background-color: #e7f0f9;
}
.block-add:hover .activity-add-text {
  text-decoration: underline;
}

/* calendar.less */
.calendar_event_category {
  background-color: #e0cbe0;
}
.calendar_event_category .commands a {
  color: #0d5ca1;
}

.calendar_event_course {
  background-color: #ffd3bd;
}
.calendar_event_course .commands a {
  color: #0d5ca1;
}

.calendar_event_site {
  background-color: #d6f8cd;
}
.calendar_event_site .commands a {
  color: #0d5ca1;
}

.calendar_event_group {
  background-color: #fee7ae;
}
.calendar_event_group .commands a {
  color: #0d5ca1;
}

.calendar_event_user {
  background-color: #dce7ec;
}
.calendar_event_user .commands a {
  color: #0d5ca1;
}

.calendar_event_other {
  background-color: #ced4da;
}
.calendar_event_other .commands a {
  color: #0d5ca1;
}

.calendartable {
  width: 100%;
  table-layout: fixed;
}
.calendartable th,
.calendartable td {
  width: 14%;
  vertical-align: top;
  text-align: center;
  border: 0;
}

.calendar-controls .previous,
.calendar-controls .next,
.calendar-controls .current {
  display: block;
  float: left;
  width: 12%;
}
.calendar-controls .previous {
  text-align: left;
  border: 1px solid transparent;
  width: 25%;
}
.calendar-controls .current {
  text-align: center;
  width: 50%;
}
.calendar-controls .next {
  text-align: right;
  border: 1px solid transparent;
  width: 25%;
}
.calendar-controls .drop-target {
  box-sizing: border-box;
  border: 1px dashed #0f6cbf;
}

.filters table {
  border-collapse: separate;
  border-spacing: 2px;
  width: 100%;
}

#region-main .maincalendar .calendarwrapper td > div {
  height: 11.5em;
  overflow: hidden;
}

.maincalendar {
  vertical-align: top;
  padding: 0;
}
.maincalendar .bottom {
  text-align: left;
  width: 98%;
  margin: 10px auto;
}
.maincalendar .bottom span.footer-link:after {
  content: "•";
  color: #0f6cbf;
}
.maincalendar .bottom span.footer-link:last-child:after {
  content: none;
}
.maincalendar .heightcontainer {
  height: 100%;
  position: relative;
}
.maincalendar .calendarmonth {
  width: 98%;
  margin: 10px auto;
}
.maincalendar .calendarmonth ul {
  margin: 0;
  padding: 0;
}
.maincalendar .calendarmonth ul li[data-event-folded=true] {
  display: none;
}
.maincalendar .calendarmonth ul li {
  list-style-type: none;
  line-height: 1.2em;
}
.maincalendar .calendarmonth ul li > a {
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
  max-width: 100%;
  display: inline-block;
}
.maincalendar .calendarmonth ul li > a:hover {
  text-decoration: none;
}
.maincalendar .calendarmonth ul li > a:hover .eventname {
  text-decoration: underline;
}
.maincalendar .calendarmonth ul li a[data-action=view-day-link] {
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
}
.maincalendar .calendarmonth ul li .icon {
  margin-left: 0.25em;
  margin-right: 0.25em;
  vertical-align: initial;
}
.maincalendar .calendarmonth ul li .calendar-circle {
  width: 12px;
  height: 12px;
  border-radius: 6px;
  vertical-align: middle;
  display: inline-block;
}
.maincalendar .calendarmonth ul li .calendar-circle.calendar_event_category {
  background-color: #e0cbe0;
  border: 2px solid #9e619f;
}
.maincalendar .calendarmonth ul li .calendar-circle.calendar_event_course {
  background-color: #ffd3bd;
  border: 2px solid #d34600;
}
.maincalendar .calendarmonth ul li .calendar-circle.calendar_event_site {
  background-color: #d6f8cd;
  border: 2px solid #2b8713;
}
.maincalendar .calendarmonth ul li .calendar-circle.calendar_event_group {
  background-color: #fee7ae;
  border: 2px solid #9a6e02;
}
.maincalendar .calendarmonth ul li .calendar-circle.calendar_event_user {
  background-color: #dce7ec;
  border: 2px solid #4e7c91;
}
.maincalendar .calendarmonth ul li .calendar-circle.calendar_event_other {
  background-color: #ced4da;
  border: 2px solid #687889;
}
.maincalendar .calendarmonth th {
  text-align: left;
  padding-left: 16px;
}
.maincalendar .calendarmonth td a.day:focus {
  display: inline-block;
  border-radius: 50%;
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.maincalendar .calendarmonth td .day-number-circle {
  display: inline-block;
  line-height: 0;
  width: 30px;
  height: 30px;
}
.maincalendar .calendarmonth td .day-number-circle .day-number {
  display: inline-block;
  padding: 50% 4px;
  width: 100%;
  text-align: center;
}
.maincalendar .calendarmonth td.today .day-number-circle {
  border-radius: 50%;
  color: #fff;
  background-color: #0f6cbf;
}
.maincalendar .calendarmonth .clickable:hover {
  background-color: #ededed;
}
.maincalendar .controls {
  width: 98%;
  margin: 10px auto;
}
.maincalendar .calendar_event_category a:has(> .icon):hover,
.maincalendar .calendar_event_course a:has(> .icon):hover,
.maincalendar .calendar_event_site a:has(> .icon):hover,
.maincalendar .calendar_event_group a:has(> .icon):hover,
.maincalendar .calendar_event_user a:has(> .icon):hover {
  color: #07345a;
  text-decoration: none;
}
.maincalendar .calendar_event_category {
  border-color: #e0cbe0;
}
.maincalendar .calendar_event_course {
  border-color: #ffd3bd;
}
.maincalendar .calendar_event_site {
  border-color: #d6f8cd;
}
.maincalendar .calendar_event_group {
  border-color: #fee7ae;
}
.maincalendar .calendar_event_user {
  border-color: #dce7ec;
}
.maincalendar .calendar_event_other {
  border-color: #ced4da;
}
.maincalendar .calendartable td,
.maincalendar .calendartable li {
  padding: 4px;
}
.maincalendar .calendartable li {
  text-align: left;
}
.maincalendar .header {
  overflow: hidden;
}
.maincalendar .header .buttons {
  float: right;
}
.maincalendar .event .card-header img {
  vertical-align: baseline;
}
.maincalendar .event .location {
  word-break: break-all;
  overflow-wrap: break-word;
}
.maincalendar table#subscription_details_table td {
  vertical-align: middle;
}
.maincalendar table#subscription_details_table td > .btn-group button {
  padding-left: 0;
}

#page-calendar-export .indent {
  padding-left: 20px;
}

.block .bottom {
  width: 98%;
  margin: 10px auto;
}
.block .bottom span.footer-link:after {
  content: "•";
  color: #0f6cbf;
}
.block .bottom span.footer-link:last-child:after {
  content: none;
}
.block .minicalendar {
  max-width: 280px;
  margin: 0 auto;
  width: 100%;
}
.block .minicalendar th,
.block .minicalendar td {
  padding: 2px;
  font-size: 0.8em;
  text-align: center;
}
.block .minicalendar td.weekend {
  color: var(--bs-secondary-color);
}
.block .minicalendar td a {
  width: 100%;
  height: 100%;
  display: block;
  color: #0d5ca1;
}
.block .minicalendar td.duration_global {
  border-top: 1px solid #d6f8cd;
  border-bottom: 1px solid #d6f8cd;
}
.block .minicalendar td.duration_global.duration_finish {
  background-color: #d6f8cd;
}
.block .minicalendar td.duration_category {
  border-top: 1px solid #e0cbe0;
  border-bottom: 1px solid #e0cbe0;
}
.block .minicalendar td.duration_category.duration_finish {
  background-color: #e0cbe0;
}
.block .minicalendar td.duration_course {
  border-top: 1px solid #ffd3bd;
  border-bottom: 1px solid #ffd3bd;
}
.block .minicalendar td.duration_course.duration_finish {
  background-color: #ffd3bd;
}
.block .minicalendar td.duration_group {
  border-top: 1px solid #fee7ae;
  border-bottom: 1px solid #fee7ae;
}
.block .minicalendar td.duration_group.duration_finish {
  background-color: #fee7ae;
}
.block .minicalendar td.duration_user {
  border-top: 1px solid #dce7ec;
  border-bottom: 1px solid #dce7ec;
}
.block .minicalendar td.duration_user.duration_finish {
  background-color: #dce7ec;
}
.block .minicalendar td.duration_other {
  border-top: 1px solid #ced4da;
  border-bottom: 1px solid #ced4da;
}
.block .minicalendar td.duration_other.duration_finish {
  background-color: #ced4da;
}
.block .minicalendar caption {
  font-size: inherit;
  font-weight: inherit;
  line-height: inherit;
  text-align: center;
}
.block .calendar_filters ul {
  list-style: none;
  margin: 0;
  padding: 0;
}
.block .calendar_filters li {
  margin-bottom: 0.2em;
}
.block .calendar_filters li span.calendar_event_category i {
  color: #0d5ca1;
}
.block .calendar_filters li span.calendar_event_course i {
  color: #0d5ca1;
}
.block .calendar_filters li span.calendar_event_site i {
  color: #0d5ca1;
}
.block .calendar_filters li span.calendar_event_group i {
  color: #0d5ca1;
}
.block .calendar_filters li span.calendar_event_user i {
  color: #0d5ca1;
}
.block .calendar_filters li span.calendar_event_other i {
  color: #0d5ca1;
}
.block .calendar_filters li span img {
  padding: 0 0.2em;
  margin: 0;
}
.block .calendar_filters li .icon {
  vertical-align: initial;
  margin: 0 0.1rem 0 0.4rem;
}
.block .calendar_filters li > a:hover {
  text-decoration: none;
}
.block .calendar_filters li > a:hover .eventname {
  text-decoration: underline;
}
.block .content h3.eventskey, .block .content .eventskey.h3 {
  margin-top: 0.5em;
}

.path-course-view .block.block_calendar_month .maincalendar div.header {
  visibility: hidden;
  height: 0;
}
.path-course-view .block.block_calendar_month .maincalendar .calendarwrapper .arrow_text {
  width: 1px !important;
  height: 1px !important;
  padding: 0 !important;
  margin: -1px !important;
  overflow: hidden !important;
  clip: rect(0, 0, 0, 0) !important;
  white-space: nowrap !important;
  border: 0 !important;
}
.path-course-view .block.block_calendar_month .maincalendar .calendarwrapper .arrow_text:not(caption) {
  position: absolute !important;
}
.path-course-view .block.block_calendar_month .footer .bottom .footer-link {
  display: block;
}
.path-course-view .block.block_calendar_month .footer .bottom .footer-link:after {
  content: none;
}

/* Display month name above the calendar */
table.calendartable caption {
  caption-side: top;
}

@media (min-width: 768px) {
  #page-calender-view .container-fluid, #page-calender-view .container-sm, #page-calender-view .container-md, #page-calender-view .container-lg, #page-calender-view .container-xl, #page-calender-view .container-xxl {
    min-width: 1024px;
  }
}
@media (min-width: 768px) {
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar div.header {
    visibility: hidden;
    height: 0;
  }
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar .calendarwrapper .current {
    width: 40%;
    font-size: inherit;
    line-height: inherit;
  }
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar .calendarwrapper .previous,
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar .calendarwrapper .next {
    width: 30%;
  }
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar .calendarwrapper .previous .arrow_text,
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar .calendarwrapper .next .arrow_text {
    display: none;
  }
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar .calendartable.calendarmonth th,
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar .calendartable.calendarmonth td {
    border: none;
    text-align: center !important;
    padding: 0;
  }
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar .calendartable.calendarmonth td {
    height: auto;
    font-size: 0.8em;
  }
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar .calendartable.calendarmonth td.hasevent [data-region=day-content] {
    display: none;
  }
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar .calendartable.calendarmonth td.hasevent .day-number {
    display: inline-block;
    position: relative;
  }
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar .calendartable.calendarmonth td.hasevent .day-number:before {
    content: ".";
    display: inline-block;
    position: absolute;
    bottom: 0.4em;
    left: 0;
    text-align: center;
    width: 100%;
    font-size: 3em;
    color: inherit;
  }
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar .calendartable.calendarmonth td:after {
    content: "";
    display: block;
    margin-top: calc(100% - 26px);
  }
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar .calendartable.calendarmonth td.clickable:hover {
    background-color: inherit;
  }
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .maincalendar .calendartable.calendarmonth td.clickable:not(.today):hover .day-number-circle {
    border-radius: 50%;
    background-color: #ededed;
  }
  [data-blockregion]:not([data-blockregion=content]) .block.block_calendar_month .bottom {
    border-top: 1px solid var(--bs-border-color-translucent);
    padding-top: 0.5rem;
  }
}
@media (max-width: 768px) {
  .maincalendar .calendartable.calendarmonth th,
  .maincalendar .calendartable.calendarmonth td {
    border: none;
    text-align: center !important;
    padding: 0;
  }
  .maincalendar .calendartable.calendarmonth td {
    height: auto;
    font-size: inherit;
    padding: 0;
  }
  .maincalendar .calendartable.calendarmonth td.hasevent [data-region=day-content] {
    display: none;
  }
  .maincalendar .calendartable.calendarmonth td.hasevent .day-number {
    display: inline-block;
    position: relative;
  }
  .maincalendar .calendartable.calendarmonth td.hasevent .day-number:before {
    content: ".";
    display: inline-block;
    position: absolute;
    bottom: 0.4em;
    left: 0;
    text-align: center;
    width: 100%;
    font-size: 3em;
    color: inherit;
  }
  .maincalendar .calendartable.calendarmonth td:after {
    content: "";
    display: block;
    margin-top: calc(100% - 26px);
  }
  .maincalendar .calendartable.calendarmonth td > div {
    height: auto !important;
  }
}
.calendarwrapper {
  position: relative;
}

.day-popover-content:empty + .day-popover-alternate {
  display: block;
}

.location-content {
  overflow-wrap: break-word;
}

.description-content {
  overflow-wrap: break-word;
}
.description-content > p {
  margin: 0;
}

.cal_courses_flt {
  color: #6a737b;
  max-width: 75%;
}

.content-bank-container .cb-content-wrapper {
  padding: 0.5rem;
  min-height: 140px;
  max-height: 500px;
  overflow-x: auto;
  flex-wrap: wrap;
}
.content-bank-container .cb-thumbnail {
  width: 24px;
  height: 24px;
  background-repeat: no-repeat;
  background-position: center;
  background-size: cover;
}
.content-bank-container.view-grid {
  /* Display a centered eye slash on top of unlisted content icons. */
}
.content-bank-container.view-grid .cb-listitem {
  margin-bottom: 0.5rem;
}
.content-bank-container.view-grid .cb-listitem.cb-unlisted {
  position: relative;
}
@media (max-width: 767.98px) {
  .content-bank-container.view-grid .cb-listitem {
    flex-basis: 50%;
  }
}
@media (min-width: 576px) {
  .content-bank-container.view-grid .cb-listitem {
    max-width: 120px;
    min-width: 120px;
  }
}
.content-bank-container.view-grid .cb-name {
  text-align: center;
}
.content-bank-container.view-grid .cb-file {
  padding: 0.5rem;
}
.content-bank-container.view-grid .cb-thumbnail {
  width: 64px;
  height: 64px;
  margin-left: auto;
  margin-right: auto;
  margin-bottom: 0.5rem;
}
.content-bank-container.view-grid .cb-unlisted .cb-thumbnail {
  opacity: 0.15;
}
.content-bank-container.view-grid .cb-unlisted::after {
  font: var(--fa-font-solid);
  content: "\f070";
  position: absolute;
  top: 20px;
  left: 0;
  width: 100%;
  font-size: 26px;
  text-align: center;
  opacity: 0.9;
  text-shadow: 0 0 10px #fff;
}
.content-bank-container.view-grid .cb-heading,
.content-bank-container.view-grid .cb-uses,
.content-bank-container.view-grid .cb-date,
.content-bank-container.view-grid .cb-size,
.content-bank-container.view-grid .cb-type,
.content-bank-container.view-grid .cb-author {
  display: none;
}
.content-bank-container.view-list .cb-content-wrapper {
  padding: 0 0.5rem;
  flex-direction: column;
  flex-wrap: nowrap;
}
.content-bank-container.view-list .cb-thumbnail {
  margin-right: 0.5rem;
}
.content-bank-container.view-list .cb-listitem,
.content-bank-container.view-list .cb-heading {
  display: flex;
  flex-wrap: wrap;
  width: 100%;
  border-bottom: 1px solid #dee2e6;
}
.content-bank-container.view-list .cb-column {
  display: flex;
  padding: 0.25rem;
}
.content-bank-container.view-list .cb-column {
  border-right: 1px solid #dee2e6;
}
.content-bank-container.view-list .cb-listitem.cb-unlisted .cb-thumbnail {
  opacity: 0.3;
}
.content-bank-container.view-list .cb-listitem.cb-unlisted .cb-column,
.content-bank-container.view-list .cb-listitem.cb-unlisted .cb-column a {
  color: var(--bs-secondary-color);
}
@media (max-width: 767.98px) {
  .content-bank-container.view-list .cb-column {
    flex: 0 0 50%;
    max-width: 50%;
  }
}
@media (min-width: 576px) {
  .content-bank-container.view-list .cb-heading {
    position: sticky;
    top: 0;
    z-index: 1;
  }
  .content-bank-container.view-list .cb-file {
    flex: 0 0 40%;
    max-width: 40%;
  }
  .content-bank-container.view-list .cb-uses,
  .content-bank-container.view-list .cb-date,
  .content-bank-container.view-list .cb-size,
  .content-bank-container.view-list .cb-type,
  .content-bank-container.view-list .cb-author {
    flex: 0 0 12%;
    max-width: 12%;
  }
  .content-bank-container.view-list .cb-column.last {
    border-right: 0;
  }
}
.content-bank-container.view-list .cb-btnsort span {
  display: none;
}
.content-bank-container.view-list .cb-btnsort .title {
  display: inline;
}
.content-bank-container.view-list .cb-btnsort.dir-none .default, .content-bank-container.view-list .cb-btnsort.dir-asc .asc, .content-bank-container.view-list .cb-btnsort.dir-desc .desc {
  display: inline;
}

.cb-toolbar-container .dropdown-scrollable {
  max-height: 190px;
  overflow-y: auto;
}

.cb-navigation-container .singleselect,
.cb-navigation-container .singleselect .form-select {
  width: 100%;
}

/*
 * Course page styles.
 */
/* Course content */
.course-content ul.section-list {
  padding: 0;
  margin: 0;
}

.course-content .section-item ul,
.course-content .section-item .activity-item ul {
  list-style: disc;
}
.course-content .section-item ul ul,
.course-content .section-item .activity-item ul ul {
  list-style: circle;
}
.course-content .section-item ul ul ul,
.course-content .section-item .activity-item ul ul ul {
  list-style: square;
}

.section-collapsemenu .collapseall {
  display: block;
}
.section-collapsemenu .expandall {
  display: none;
}
.section-collapsemenu.collapsed .collapseall {
  display: none;
}
.section-collapsemenu.collapsed .expandall {
  display: block;
}

.max-section-alert {
  border-top: 2px dashed #dee2e6;
  font-size: 0.8203125rem;
  font-weight: normal;
  color: #6a737b;
}

/* Sections */
.course-section {
  list-style: none;
  margin-top: 1rem;
}
.editing .course-section {
  margin-top: 0.5rem;
}
.course-section .section-item {
  padding: 1rem;
  border: 1px solid #dee2e6;
  border-radius: 1rem;
}
.course-section.hidden .section-item, .course-section.orphaned .section-item {
  background-color: #f8f9fa;
}
.course-section.current > .section-item {
  position: relative;
}
.course-section.current > .section-item::before {
  border-left: #0f6cbf 3px solid;
  bottom: 0;
  content: "";
  left: -8px;
  position: absolute;
  top: 0;
}
.course-section .sectionname > a {
  color: #1d2125;
}
.course-section .sectionname > a:hover {
  text-decoration: none;
}
.course-section .sectionbadges .badge {
  margin-left: 0.5rem;
  font-weight: normal;
}
.course-section .sectionbadges .badge .icon {
  font-size: 12px;
  width: 12px;
  height: 12px;
}
.course-section .course-section-header.draggable {
  cursor: move;
}
.course-section .section_action_menu .dropdown-toggle::after {
  display: none;
}
.course-section .summarytext img {
  border-radius: 1rem;
}
.course-section .availabilityinfo {
  margin-top: 0.5rem;
  padding: 0.25rem 1rem;
  background-color: #e9ecef;
  font-size: 0.875em;
  border-radius: 1rem;
}
.course-section .availabilityinfo .editavailability a {
  border-radius: 0.5rem;
  font-weight: bold;
}
.course-section .availabilityinfo .editavailability a:hover {
  background-color: #ced4da;
}
.course-section .availabilityinfo .editavailability a .icon {
  font-size: inherit;
  margin-right: 0.25rem;
}
.course-section .section_action_menu > .action-menu .btn-icon {
  font-size: 1.171875rem;
}
.course-section .section-summary-activities .icon {
  width: inherit;
  color: #0f6cbf;
}
.course-section .section-summary-activities + .section {
  border-top: 1px solid #dee2e6;
  margin-top: 1rem !important;
  padding-top: 1rem !important;
}
.course-section .section_goto .icon {
  font-size: 1.171875rem;
  color: #0f6cbf;
}
.course-section .overlay-preview {
  z-index: 5;
}

.section li.movehere a {
  display: block;
  width: 100%;
  height: 2rem;
  border: 2px dashed #343a40;
}

/* Activities */
.activity {
  list-style: none;
  padding-top: 0.25rem;
  margin-top: 0.25rem;
  border-top: 1px solid #dee2e6;
}
.activity.indented .activity-item {
  margin-left: 1rem;
}
.editing .activity, .section .activity:first-child {
  padding-top: 0;
  margin-top: 0;
  border-top: none;
}

.activity-item {
  padding: 0.75rem;
  border-radius: 1rem;
}
.activity-item.activityinline {
  padding: 0.75rem 0;
}
.activity-item.hiddenactivity {
  background-color: #f8f9fa;
}
.activity-item.hiddenactivity .activityiconcontainer,
.activity-item.hiddenactivity .badge {
  mix-blend-mode: multiply;
}
.editing .activity-item {
  cursor: move;
}
.editing .activity-item .a {
  cursor: pointer;
}
.editing .activity-item:hover:not(:has(.activity:hover)), .editing .activity-item.selected {
  outline: 2px solid #0f6cbf;
  box-shadow: 0 0.125rem 0.25rem rgba(0, 0, 0, 0.075);
}
.editing .activity-item:hover:not(:has(.activity:hover)) .activityiconcontainer,
.editing .activity-item:hover:not(:has(.activity:hover)) .badge, .editing .activity-item.selected .activityiconcontainer,
.editing .activity-item.selected .badge {
  mix-blend-mode: multiply;
}
.activity.dragging .activity-item {
  border: 2px solid #0f6cbf;
}
.editing .activity.dragging .activity-item:hover {
  outline: none;
}
.activity-item .activity-grid {
  display: grid;
  align-items: center;
  grid-template-columns: min-content 1fr min-content min-content min-content;
  grid-template-rows: 1fr repeat(5, min-content);
  grid-template-areas: "icon   name          groupmode      completion    actions" "icon   visibility    groupmode      completion    actions" "icon   dates         groupmode      completion    actions" "icon   altcontent    altcontent     altcontent    altcontent" "icon   afterlink     afterlink      afterlink     afterlink" "icon   availability  availability   availability  availability";
}
@media (max-width: 575.98px) {
  .activity-item .activity-grid {
    grid-template-columns: min-content 1fr min-content min-content min-content;
    grid-template-rows: 1fr repeat(4, min-content);
    grid-template-areas: "icon          name          actions" "icon          visibility    actions" "dates         dates         dates" "groupmode     groupmode     groupmode" "completion    completion    completion" "altcontent    altcontent    altcontent" "afterlink     afterlink     afterlink" "availability  availability  availability";
  }
}
.activity-item .activity-grid.noname-grid {
  grid-template-columns: min-content 1fr min-content min-content;
  grid-template-areas: "visibility    groupmode        completion    actions" "altcontent    altcontent       altcontent    altcontent" "afterlink     afterlink        afterlink     afterlink" "availability  availability     availability  availability";
}
@media (max-width: 575.98px) {
  .activity-item .activity-grid.noname-grid {
    grid-template-columns: 1fr min-content;
    grid-template-areas: "visibility    actions" "altcontent    altcontent" "groupmode     groupmode" "afterlink     afterlink" "completion    completion" "availability  availability";
  }
}
.activity-item .activity-actions {
  grid-area: actions;
}
.activity-item .activity-actions .actions {
  position: relative;
}
.activity-item .activity-icon {
  grid-area: icon;
}
.activity-item .activity-dates {
  grid-area: dates;
  font-size: 0.875em;
  color: #495057;
  display: flex;
  flex-wrap: wrap;
  column-gap: 0.75rem;
}
@media (max-width: 575.98px) {
  .activity-item .activity-dates {
    margin-top: 0.5rem;
  }
}
.activity-item .activity-name-area {
  grid-area: name;
}
.activity-item .activity-name-area .activityname .afterlink {
  margin-left: 0.5rem;
}
.activity-item .activity-name-area .activityname .inplaceeditable .quickeditlink {
  position: relative;
  z-index: 2;
  margin-left: 0.5rem;
}
.activity-item .activity-name-area .activitybadge.badge-none {
  font-weight: normal;
  --bs-badge-color: #1d2125;
  font-size: 0.875em;
  padding: 0;
}
.activity-item .activity-completion {
  grid-area: completion;
  justify-self: end;
}
.activity-item .activity-completion button,
.activity-item .activity-completion a[role=button] {
  min-height: 32px;
}
@media (max-width: 575.98px) {
  .activity-item .activity-completion {
    width: 100%;
    margin-top: 0.5rem;
  }
  .activity-item .activity-completion button {
    width: 100%;
  }
}
.activity-item .activity-completion .completion-dialog {
  color: #495057;
  font-size: 0.8203125rem;
  min-width: 12rem;
}
.activity-item .activity-completion .completion-dialog .icon {
  font-size: 0.8203125rem;
  width: 0.8203125rem;
  height: 0.8203125rem;
  margin-right: 0.25rem;
}
.activity-item .activity-completion .completion-dialog .editcompletion a {
  border-radius: 0.5rem;
  color: #495057;
  font-weight: bold;
  text-decoration: none;
}
.activity-item .activity-completion .completion-dialog .editcompletion a:hover {
  background-color: #e9ecef;
}
.activity-item .activity-groupmode-info {
  grid-area: groupmode;
  justify-self: end;
}
.activity-item .activity-groupmode-info .groupmode-information {
  height: 32px;
  width: 32px;
  border-radius: 0.5rem;
}
.activity-item .activity-groupmode-info .groupmode-information .icon {
  width: 16px;
  height: 16px;
}
.activity-item .activity-groupmode-info .groupmode-icon-info {
  display: none;
}
@media (max-width: 575.98px) {
  .activity-item .activity-groupmode-info {
    width: 100%;
    margin-top: 0.5rem;
    padding-top: 0.5rem;
    border-top: 1px solid #dee2e6;
  }
  .activity-item .activity-groupmode-info .groupmode-information {
    width: auto;
    font-size: inherit;
    padding: 0 0.5rem;
  }
  .activity-item .activity-groupmode-info .groupmode-icon-info {
    display: inline;
  }
  .activity-item .activity-groupmode-info .v-parent-focus {
    opacity: 1;
    visibility: visible;
  }
}
.activity-item .activity-badges {
  grid-area: visibility;
}
.activity-item .activity-badges .badge {
  font-weight: normal;
}
.activity-item .activity-badges .badge .icon {
  font-size: 12px;
  width: 12px;
  height: 12px;
}
.activity-item .activity-altcontent {
  grid-area: altcontent;
  margin-top: 0.25rem;
}
.activity-item .activity-altcontent.activity-description {
  margin-top: 0.5rem;
  padding-top: 0.5rem;
  border-top: 1px solid #dee2e6;
  font-size: 0.875em;
}
.activity-item .activity-altcontent.activity-description ul:not(.activity-description ul ul) {
  margin-bottom: 1rem;
}
.activity-item .activity-altcontent img {
  border-radius: 1rem;
}
.activity-item .activity-availability {
  grid-area: availability;
}
.activity-item .activity-afterlink {
  grid-area: afterlink;
  margin-top: 0.5rem;
  padding-top: 0.5rem;
  border-top: 1px solid #dee2e6;
}
.activity-item .no-overflow {
  width: 100%;
}

.section .draggable .activity-item .dragicon {
  display: none;
}

/* Dividers */
.course-content .divider {
  position: relative;
}
.course-content .divider hr {
  width: 100%;
  margin: 0.5rem 0.25rem;
  border-top: 2px dashed #dee2e6;
  opacity: 1;
}
.changenumsections.disabled .course-content .divider hr {
  border-top: 2px dashed #dee2e6;
}
.course-content .divider .divider-content {
  opacity: 0;
  visibility: hidden;
  transition: visibility 0.1s;
  position: absolute;
  background: linear-gradient(transparent 40%, #fff 40%, #fff 60%, transparent 60%);
}
.section.hidden .course-content .divider .divider-content {
  background: linear-gradient(transparent 40%, #f8f9fa 40%, #f8f9fa 60%, transparent 60%);
}
.course-content .divider.always-visible .divider-content {
  opacity: 1;
  visibility: visible;
}
.course-content .divider.always-hidden hr {
  opacity: 0;
  visibility: hidden;
}
.course-content .divider:hover .divider-content, .course-content .divider:focus .divider-content, .course-content .divider:focus-within .divider-content {
  opacity: 1;
  visibility: visible;
}
.course-content .divider:hover hr, .course-content .divider:focus hr, .course-content .divider:focus-within hr {
  opacity: 1;
  visibility: visible;
}
.course-content .divider:has(.btn.add-content:hover) hr {
  border-color: #0f6cbf;
}
.changenumsections.disabled .course-content .divider:has(.btn.add-content:hover) hr {
  border-color: #e9ecef;
}

.activity:focus-within + .activity .divider .divider-content,
.course-section-header:focus-within + .content .section .activity:first-child .divider .divider-content,
.content .section .activity:focus-within .divider .divider-content,
.course-content:focus-within .changenumsections .divider .divider-content {
  visibility: visible;
}

.course-content ul.topics > li:last-child .changenumsections {
  display: none;
}

.course-content .divider .btn.add-content {
  position: relative;
  z-index: 1;
  border-radius: 0.5rem;
  font-size: 0.8203125rem;
  font-weight: bold;
  color: #094173;
  background-color: #cfe2f2;
}
.course-content .divider .btn.add-content:hover, .course-content .divider .btn.add-content:focus {
  color: #fff;
  background-color: #0f6cbf;
}
.course-content .divider .btn.add-content .icon {
  width: 14px;
  height: 14px;
  font-size: 14px;
}
.changenumsections.disabled .course-content .divider .btn.add-content {
  color: #8f959e;
  background-color: #e9ecef;
  outline: none;
  box-shadow: none;
  pointer-events: auto;
}
.changenumsections.disabled .course-content .divider .btn.add-content:hover, .changenumsections.disabled .course-content .divider .btn.add-content:focus {
  color: #8f959e;
  background-color: #e9ecef;
  outline: none;
  box-shadow: none;
}

/* Bulk editing */
.bulkenabled .bulk-hidden {
  display: none !important;
}
.bulkenabled .section {
  margin-left: 1rem;
}
.bulkenabled .section:not(:first-child) {
  margin-top: 1.5rem;
}
.bulkenabled .activity {
  margin-top: 0.5rem;
  margin-left: 2rem;
  padding-top: 0.5rem;
  border-top: 2px dashed #dee2e6;
}
.bulkenabled .activity:first-child {
  margin-top: 1.5rem;
}
.bulkenabled .activity-item .bulkselect {
  float: left;
  margin-left: -2rem;
}
.bulkenabled .activity-item:has(.delegated-section):hover {
  outline: none !important;
  box-shadow: none !important;
}
.bulkenabled .course-section-header .bulkselect {
  left: -2.75rem;
  position: relative;
  width: 0;
}
@media (max-width: 767.98px) {
  .bulkenabled .course-content {
    margin-left: 2rem;
  }
}
.bulkenabled .sticky-footer-content.bulkactions .btn {
  --bs-btn-disabled-border-color: transparent;
}
.bulkenabled .sticky-footer-content.bulkactions .btn:focus {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}

@media (max-width: 767.98px) {
  .sticky-footer-content.bulkactions {
    position: relative;
  }
  .sticky-footer-content.bulkactions .form-check,
  .sticky-footer-content.bulkactions .bulkaction-name,
  .sticky-footer-content.bulkactions .bulkcount {
    display: none;
  }
  .sticky-footer-content.bulkactions .actions button {
    padding: 0 0.5rem;
  }
  .sticky-footer-content.bulkactions .bulkcancel {
    position: absolute;
    top: 0;
    right: 0;
  }
}

/* Single section page */
.single-section > ul > .course-section.hidden .section-item {
  background-color: inherit;
}
.single-section > ul > .course-section .section-item {
  padding: 0;
  border: none;
}

.single-section-page .header-action {
  display: inline-block;
}
.single-section-page .page-context-header {
  overflow: visible;
}

/* Default activity completion page */
.defaultactivitycompletion-item a {
  color: #000;
  text-decoration: none;
}
.defaultactivitycompletion-item .activityicon {
  width: 32px;
  height: 32px;
}

/* Home page course */
.sitetopic .section-item {
  margin-bottom: 1.5rem;
  border-radius: 0.5rem;
}

/* Move activity and Move section modals */
.move-activity-tree .collapse-list-item {
  border-radius: 0.5rem;
}
.move-activity-tree .collapse-list-item:hover, .move-activity-tree .collapse-list-item:focus {
  background-color: #cfe2f2;
}
.move-activity-tree .collapse-list-item:hover a, .move-activity-tree .collapse-list-item:focus a {
  color: #094173;
}
.move-activity-tree .collapse-list-item a {
  color: var(--bs-body-color);
}
.move-activity-tree .collapse-list-item a:hover, .move-activity-tree .collapse-list-item a:focus {
  text-decoration: none;
}
.move-activity-tree .collapse-list-item a.disabled {
  color: var(--bs-secondary-color);
}
.move-activity-tree .collapse-list-item-content .collapse-list-item {
  padding: 0.5rem 1rem;
}
.move-activity-tree ul {
  margin-left: 1.5rem;
}
.move-activity-tree .collapse-list-link {
  font-weight: bold;
}

/* Subsections */
.activity.subsection {
  border-top: none;
}
.activity.subsection > .activity-item {
  border: 1px solid #dee2e6;
  padding: 0;
  margin: 0.5rem 0;
}
.activity.subsection > .activity-item > .activity-grid > .activity-altcontent {
  margin-top: 0;
}
.activity.subsection .section {
  margin-top: 0;
}
.activity.subsection .section .section-item {
  border: none;
  padding: 0.75rem;
}
.activity.subsection .section .icons-collapse-expand:has(+ h4, + .h4) {
  height: 24px;
  width: 24px;
  font-size: 0.8203125rem;
}
.activity.subsection .section h4, .activity.subsection .section .h4 {
  font-size: 1.171875rem;
}
.activity.subsection .section .section_action_menu > .action-menu .btn-icon {
  font-size: inherit;
}
.activity.subsection + .activity {
  border-top: none;
}
.activity.subsection .focus-control:not(:has(.focus-control)):focus-within .v-parent-focus, .activity.subsection .focus-control:not(:has(.focus-control)):hover .v-parent-focus {
  opacity: 1 !important;
  visibility: visible !important;
}
.activity.subsection .focus-control:focus-within .focus-control .v-parent-focus,
.activity.subsection .focus-control:hover .focus-control .v-parent-focus {
  opacity: 0;
  visibility: hidden;
}

/* Spinners */
@keyframes editinprogress-rotation {
  0% {
    opacity: 0;
    transform: rotate(0deg);
  }
  50% {
    opacity: 1;
  }
  100% {
    opacity: 0;
    transform: rotate(359deg);
  }
}
.editing .editinprogress {
  position: relative;
}
.editing .editinprogress > * {
  opacity: 0.4;
}
.editing .editinprogress .corelightbox,
.editing .editinprogress .lightbox {
  display: none;
}
.editing .editinprogress:after {
  font: var(--fa-font-solid);
  position: absolute;
  font-size: 20px;
  color: #6a737b;
  content: "\f110";
  display: flex;
  justify-content: center;
  align-items: center;
  width: 30px;
  height: 30px;
  left: calc(50% - 15px);
  top: calc(50% - 15px);
  animation: editinprogress-rotation 2s infinite linear;
}
.editing .editinprogress .editinprogress:after {
  display: none;
}

/* Drag and drop */
.course-content .section.dropready.main.drop-down {
  border-bottom: 1px solid #1d2125;
}
.course-content .section.dropready .course-section-header.dropready.drop-zone {
  margin-top: -2px;
}
.course-content .section.dropready li.activity.dropready.drop-down {
  border-bottom: 1px solid #1d2125;
  margin-bottom: -1px;
}
.course-content .section.dropready li.activity.dropready.drop-up {
  border-top: 1px solid #1d2125;
  margin-top: -1px;
}
.course-content .section.dropready [data-for=sectioninfo] {
  min-height: 1px;
}
.course-content .section.dropready [data-for=sectioninfo].drop-down {
  margin-top: -1px;
}

/* Old indentation styles */
.mod-indent-outer {
  display: table;
}

.mod-indent {
  display: table-cell;
}

.label .mod-indent {
  float: left;
  padding-top: 20px;
}

.activity.label.modtype_label .mod-indent {
  float: none;
}

@media (min-width: 576px) {
  /* Creates a series of .mod-indent-# rule declarations based on indent size and number of indent levels. */
  .mod-indent-1 {
    width: 30px;
  }
  .mod-indent-2 {
    width: 60px;
  }
  .mod-indent-3 {
    width: 90px;
  }
  .mod-indent-4 {
    width: 120px;
  }
  .mod-indent-5 {
    width: 150px;
  }
  .mod-indent-6 {
    width: 180px;
  }
  .mod-indent-7 {
    width: 210px;
  }
  .mod-indent-8 {
    width: 240px;
  }
  .mod-indent-9 {
    width: 270px;
  }
  .mod-indent-10 {
    width: 300px;
  }
  .mod-indent-11 {
    width: 330px;
  }
  .mod-indent-12 {
    width: 360px;
  }
  .mod-indent-13 {
    width: 390px;
  }
  .mod-indent-14 {
    width: 420px;
  }
  .mod-indent-15 {
    width: 450px;
  }
  .mod-indent-16 {
    width: 480px;
  }
  .mod-indent-huge {
    width: 480px;
  }
}
/*
 * Course management page styles.
 */
#course-category-listings {
  margin-bottom: 0;
  /** Two column layout */
  /** Three column layout */
}
#course-category-listings.columns-2 > #course-listing > div {
  position: relative;
  left: -1px;
}
#course-category-listings.columns-3 > #course-listing > div {
  height: 100%;
}
#course-category-listings > div > div {
  min-height: 300px;
}
#course-category-listings > div > div > ul.ml > li:first-child > div {
  border-top: 0;
}
#course-category-listings h3, #course-category-listings .h3 {
  margin: 0;
  padding: 0.4rem 0.6rem 0.3rem;
}
#course-category-listings h4, #course-category-listings .h4 {
  margin: 1rem 0 0;
  padding: 0.6rem 1rem 0.5rem;
}
#course-category-listings .moodle-actionmenu {
  white-space: nowrap;
}
#course-category-listings .listing-actions {
  text-align: center;
}
#course-category-listings .listing-actions > .moodle-actionmenu {
  display: inline-block;
}
#course-category-listings ul.ml {
  list-style: none;
  margin: 1rem 0;
}
#course-category-listings ul.ml ul.ml {
  margin: 0;
}
#course-category-listings .listitem[data-selected="1"] {
  border-left: calc(var(--bs-border-width) + 5px) solid #0f6cbf;
  padding-left: calc(1rem - 5px);
}
#course-category-listings .listitem:hover {
  z-index: 2;
}
#course-category-listings .item-actions {
  margin-right: 1em;
  display: inline-block;
}
#course-category-listings .item-actions.show .menu img {
  width: 12px;
  max-width: none;
}
#course-category-listings .item-actions .menu-action-text {
  vertical-align: inherit;
}
#course-category-listings .listitem > div > .float-start {
  float: left;
}
#course-category-listings .listitem > div > .float-end {
  float: right;
  text-align: right;
}
#course-category-listings .listitem > div .item-actions .action-show {
  display: none;
}
#course-category-listings .listitem > div .item-actions .action-hide {
  display: inline;
}
#course-category-listings .listitem > div .without-actions {
  color: #333;
}
#course-category-listings .listitem > div .idnumber {
  margin-right: 2em;
}
#course-category-listings .listitem[data-visible="0"] {
  color: var(--bs-secondary-color);
}
#course-category-listings .listitem[data-visible="0"] > div > a {
  color: var(--bs-secondary-color);
}
#course-category-listings .listitem[data-visible="0"] > div .item-actions .action-show {
  display: inline;
}
#course-category-listings .listitem[data-visible="0"] > div .item-actions .action-hide {
  display: none;
}
#course-category-listings .listitem.highlight {
  background-color: #fff;
}
#course-category-listings .listitem.highlight > div,
#course-category-listings .listitem.highlight > div:hover, #course-category-listings .listitem.highlight[data-selected="1"] > div {
  background-color: rgba(var(--bs-emphasis-color-rgb), 0.075);
}
#course-category-listings #course-listing .listitem .categoryname {
  display: inline-block;
  margin-left: 1em;
  color: #a1a1a8;
}
#course-category-listings #course-listing .listitem .coursename {
  display: inline-block;
  flex-basis: 10rem;
}
#course-category-listings #course-listing > .firstpage .listitem:first-child > div .item-actions .action-moveup,
#course-category-listings #course-listing > .lastpage .listitem:last-child > div .item-actions .action-movedown {
  display: none;
}
#course-category-listings #category-listing .listitem.collapsed > ul.ml {
  display: none;
}
#course-category-listings #category-listing .listitem:first-child > div .item-actions .action-moveup, #course-category-listings #category-listing .listitem:last-child > div .item-actions .action-movedown {
  display: none;
}
#course-category-listings #category-listing .course-count {
  color: #a1a1a8;
  margin-right: 2rem;
  min-width: 3.5em;
  display: inline-block;
}
#course-category-listings #category-listing .category-listing > ul > .listitem:first-child {
  position: relative;
}
#course-category-listings #category-listing .category-bulk-actions {
  margin: 0 0.5em 0.5em;
  position: relative;
}
#course-category-listings .detail-pair > * {
  display: inline-block;
}
#course-category-listings .detail-pair .pair-key {
  font-weight: bold;
  vertical-align: top;
}
#course-category-listings .detail-pair .pair-key span {
  margin-right: 1rem;
  display: block;
}
#course-category-listings .detail-pair .pair-value select {
  max-width: 100%;
}
#course-category-listings .listing-pagination {
  text-align: center;
}
#course-category-listings .listing-pagination .yui3-button {
  --bs-btn-color: #fff;
  --bs-btn-bg: #008196;
  --bs-btn-border-color: #008196;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #006e80;
  --bs-btn-hover-border-color: #006778;
  --bs-btn-focus-shadow-rgb: 38, 148, 166;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #006778;
  --bs-btn-active-border-color: #006171;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #fff;
  --bs-btn-disabled-bg: #008196;
  --bs-btn-disabled-border-color: #008196;
  border: 0;
  margin: 0.4rem 0.2rem 0.45rem;
  font-size: 10.4px;
}
#course-category-listings .listing-pagination .yui3-button.active-page {
  --bs-btn-color: #fff;
  --bs-btn-bg: #0f6cbf;
  --bs-btn-border-color: #0f6cbf;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #0d5ca2;
  --bs-btn-hover-border-color: #0c5699;
  --bs-btn-focus-shadow-rgb: 51, 130, 201;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #0c5699;
  --bs-btn-active-border-color: #0b518f;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #fff;
  --bs-btn-disabled-bg: #0f6cbf;
  --bs-btn-disabled-border-color: #0f6cbf;
}
#course-category-listings .listing-pagination-totals {
  text-align: center;
}
#course-category-listings .listing-pagination-totals.dimmed {
  color: var(--bs-secondary-color);
  margin: 0.4rem 1rem 0.45rem;
}
#course-category-listings .select-a-category .notifymessage,
#course-category-listings .select-a-category .alert {
  margin: 1em;
}

#course-category-listings #course-listing .listitem .drag-handle {
  display: none;
}

.jsenabled #course-category-listings #course-listing .listitem .drag-handle {
  display: inline-block;
  margin: 0 6px 0 0;
  cursor: pointer;
}

/** Management header styling **/
.course-being-dragged-proxy {
  border: 0;
  color: #0f6cbf;
  vertical-align: middle;
  padding: 0 0 0 4em;
}

.course-being-dragged {
  opacity: 0.5;
}

/**
 * Display sizes:
 * Large displays                   1200        +
 * Default displays                  980     1199
 * Tablets                           768      979
 * Small tablets and large phones    481      767
 * Phones                              0      480
 */
@media (min-width: 1200px) and (max-width: 1600px) {
  #course-category-listings.columns-3 {
    background-color: #fff;
    border: 0;
  }
  #course-category-listings.columns-3 #category-listing,
  #course-category-listings.columns-3 #course-listing {
    width: 50%;
  }
  #course-category-listings.columns-3 #category-listing > div,
  #course-category-listings.columns-3 #course-listing > div,
  #course-category-listings.columns-3 #course-detail > div {
    background-color: #fff;
  }
  #course-category-listings.columns-3 #course-detail {
    width: 100%;
    margin-top: 1em;
  }
}
@media (max-width: 1199px) {
  #course-category-listings.columns-2,
  #course-category-listings.columns-3 {
    border: 0;
  }
  #course-category-listings.columns-2 #category-listing,
  #course-category-listings.columns-2 #course-listing,
  #course-category-listings.columns-2 #course-detail,
  #course-category-listings.columns-3 #category-listing,
  #course-category-listings.columns-3 #course-listing,
  #course-category-listings.columns-3 #course-detail {
    width: 100%;
    margin: 0 0 1em;
  }
}
/*
 * Course listings and course summary styles.
 */
#page-course-pending .singlebutton,
#page-course-index .singlebutton,
#page-course-index-category .singlebutton,
#page-course-editsection .singlebutton {
  text-align: center;
}

#page-admin-course-manage #movecourses td img {
  margin: 0 0.22em;
  vertical-align: text-bottom;
}

#page-course-pending .pendingcourserequests {
  margin-bottom: 1em;
}

#page-course-pending .pendingcourserequests .singlebutton {
  display: inline;
}

#page-course-pending .pendingcourserequests .cell {
  padding: 0 5px;
}

#page-course-pending .pendingcourserequests .cell.c6 {
  white-space: nowrap;
}

.coursebox {
  display: flex;
  flex-direction: column;
}
.coursebox .info {
  display: flex;
  align-items: center;
}

#frontpage-available-course-list,
#frontpage-course-list,
.course-search-result {
  margin-top: 0.5rem;
}
#frontpage-available-course-list .coursebox,
#frontpage-course-list .coursebox,
.course-search-result .coursebox {
  padding: 0.5rem;
  border: 1px solid #dee2e6;
  margin-bottom: 0.5rem;
  border-radius: 0.5rem;
}

.coursebox > .info > .coursename {
  font-size: 0.9375rem;
  font-weight: normal;
  margin: 5px;
  padding: 0;
}

.coursebox .content .teachers li {
  list-style-type: none;
  padding: 0;
  margin: 0;
}

.coursebox .customfieldname,
.coursebox .customfieldseparator {
  font-weight: 700;
}

.coursebox .content .coursefile {
  max-width: 100px;
}

.coursebox .content .courseimage img {
  max-width: 100px;
  max-height: 100px;
}

.coursebox .content .coursecat,
.coursebox .content .summary,
.coursebox .content .courseimage,
.coursebox .content .coursefile,
.coursebox .content .teachers,
.coursebox.remotecoursebox .remotecourseinfo,
.coursebox .content .customfields-container {
  margin: 15px 5px 5px;
  padding: 0;
}

.category-browse .coursebox .content .coursecat,
.category-browse .coursebox .content .summary,
.category-browse .coursebox .content .courseimage,
.category-browse .coursebox .content .coursefile,
.category-browse .coursebox .content .teachers,
.category-browse .coursebox.remotecoursebox .remotecourseinfo,
.category-browse .coursebox .content .customfields-container {
  margin-top: 0;
}

.coursebox.collapsed > .content {
  display: none;
}

.courses > .paging.paging-morelink {
  text-align: center;
  padding: 1rem;
}

.course_category_tree .category .numberofcourse {
  font-size: 0.8203125rem;
}

.course_category_tree .category > .info > .categoryname {
  margin: 5px;
  font-size: 0.9375rem;
  font-weight: normal;
  padding: 2px 18px;
}

.course_category_tree .category.with_children > .info > .categoryname {
  background-image: url([[pix:moodle|t/expanded]]);
  background-size: 20px 16px;
  background-repeat: no-repeat;
  background-position: center left;
}

.course_category_tree .category.with_children.collapsed > .info > .categoryname {
  background-image: url([[pix:moodle|t/collapsed]]);
}

/* rtl:raw:
.course_category_tree .category.with_children.collapsed > .info > .categoryname {
    background-image:url([[pix:moodle|t/collapsed_rtl]]);
}
*/
.course_category_tree .category.collapsed > .content {
  display: none;
}

.course_category_tree .category > .content {
  padding-left: 16px;
}

#page-course-index-category .categorypicker {
  margin: 10px 0 20px;
}

/*
 * Course Overview page styles.
 */
#course-overview-page .table {
  margin-bottom: 0;
  border-top: none;
}
#course-overview-page .table td,
#course-overview-page .table th {
  padding: 0.5rem 1.5rem;
}
#course-overview-page .table tr:last-child td {
  border-bottom: none;
}
#course-overview-page table tbody td {
  align-content: center;
}
#course-overview-page table tbody td .btn {
  --bs-btn-padding-y: 0.25rem;
  --bs-btn-padding-x: 0.5rem;
  --bs-btn-font-size: 0.8203125rem;
  --bs-btn-border-radius: var(--bs-border-radius);
}
#course-overview-page table tbody td .btn:not(.dropdown-toggle) {
  min-width: 6rem;
}
#course-overview-page table tbody td .btn:hover .badge.text-bg-primary {
  color: var(--bs-body-color) !important; /* stylelint-disable-line declaration-no-important */
  background-color: var(--bs-body-bg) !important; /* stylelint-disable-line declaration-no-important */
  transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  #course-overview-page table tbody td .btn:hover .badge.text-bg-primary {
    transition: none;
  }
}
#course-overview-page table tbody td .dropdown:not(.completion-dropdown) .dropdown-toggle {
  display: inline-flex;
  justify-content: space-between;
  align-items: center;
  min-width: 5rem;
  --bs-btn-font-weight: 400;
}
#course-overview-page table tbody td .dropdown:not(.completion-dropdown) .dropdown-menu {
  max-width: 20rem;
}
#course-overview-page .inplaceeditable-select {
  display: inline-block;
}
#course-overview-page .form-select {
  padding-top: 0.25rem;
  padding-bottom: 0.25rem;
}
#course-overview-page .activityiconcontainer {
  width: 40px;
  height: 40px;
  max-height: 40px;
  max-width: 40px;
  margin-right: 0.5rem;
}

/**
 * File modchooser.scss.
 * Contains styles for the module chooser.
 */
/* Main modal dialogue style customization. */
.modchooser.modal-dialog {
  --bs-modal-width: 1140px;
  --bs-modal-header-border-width: 0;
}
.modchooser.modal-dialog .modal-header .modal-title {
  flex-grow: 1;
  text-align: center;
  font-weight: 700;
  font-size: 1.171875rem;
}
@media (min-width: 576px) {
  .modchooser.modal-dialog .modal-header .modal-title {
    font-size: 1.40625rem;
  }
}
.modchooser.modal-dialog .modal-body {
  display: flex;
  flex-direction: column;
  overflow-y: auto;
  padding: 0;
}
.modchooser.modal-dialog .modal-body .carousel-item {
  background-color: var(--bs-body-bg);
  transition: none !important; /* stylelint-disable-line declaration-no-important */
}
.modchooser.modal-dialog .modal-body .carousel-item.active {
  display: flex;
}
@media (min-width: 576px) {
  .modchooser.modal-dialog .modal-body .carousel-item {
    height: 640px;
    max-height: calc(100vh - var(--bs-modal-margin) * 2 - 9rem);
  }
}
.modchooser.modal-dialog .modal-body .loading-icon {
  opacity: 1;
}
.modchooser.modal-dialog .modal-body .loading-icon .icon {
  display: block;
  max-height: 3rem;
  max-width: 3rem;
  font-size: 3rem;
}
.carousel-item .modchooser.modal-dialog .modal-body .loading-icon .icon {
  margin: 1rem auto;
}
.modchooser.modal-dialog .modal-footer {
  height: 70px;
  background: var(--bs-body-bg);
}
.modchooser.modal-dialog .modal-footer .moodlenet-logo .icon {
  height: 2.5rem;
  width: 6rem;
  max-height: 2.5rem;
  max-width: 6rem;
  margin-bottom: 0.6rem;
}

/* Main container layout styles. */
.modchoosercontainer {
  display: grid;
  width: 100%;
  grid-template-areas: "nav filters" "nav content";
  grid-template-columns: auto 1fr;
  grid-template-rows: auto 1fr;
  gap: 1rem;
}
.modchoosercontainer .modchooserfilters {
  grid-area: filters;
}
.modchoosercontainer .modchoosernav {
  grid-area: nav;
  min-width: 150px;
}
.modchoosercontainer .modchoosercontent {
  grid-area: content;
  overflow: auto;
}
.modchoosercontainer .searchresultscontainer {
  overflow: auto;
}
@media (max-width: 767.98px) {
  .modchoosercontainer {
    grid-template-areas: "filters" "nav" "content";
    grid-template-columns: 1fr;
    grid-template-rows: auto auto 1fr;
  }
}

/* Main container Filters styles. */
@media (min-width: 992px) {
  .modchooserfilters .simplesearchform {
    justify-content: center;
    position: relative;
    left: calc(-1 * (75px + 2.5rem));
  }
  .modchooserfilters .searchbar {
    max-width: 25rem !important; /* stylelint-disable-line declaration-no-important */
  }
}

/* Main container Navigation styles. */
.modchoosernav {
  --bs-nav-pills-border-radius: 50rem;
  --bs-nav-pills-link-active-bg: #1d2125;
  --bs-nav-link-font-size: 0.9375rem;
}
.modchoosernav .nav-link {
  --bs-nav-link-color: #1d2125;
  --bs-nav-link-hover-color: #1d2125;
  --bs-nav-link-font-weight: 500;
  --bs-nav-link-padding-y: .3rem;
  width: fit-content;
  border: 1px solid transparent;
}
@media (min-width: 768px) {
  .modchoosernav .nav-link {
    max-width: 200px;
    overflow: hidden;
    text-overflow: ellipsis;
    white-space: nowrap;
  }
}
.modchoosernav .nav-link:not(.active):hover {
  border: 1px solid #dee2e6;
  background-color: #f8f9fa;
}
@media (max-width: 767.98px) {
  .modchoosernav {
    overflow-x: auto;
    white-space: nowrap;
    background-color: #fff !important; /* stylelint-disable-line declaration-no-important */
  }
  .modchoosernav .nav-link {
    border: 1px solid #dee2e6 !important; /* stylelint-disable-line declaration-no-important */
  }
}

/* Main container Content styles. */
.modchoosercontent .optionscontainer {
  display: grid;
  grid-template-columns: repeat(4, 1fr);
  gap: 0.5rem;
}
@media (max-width: 991.98px) {
  .modchoosercontent .optionscontainer {
    grid-template-columns: repeat(3, 1fr);
  }
}
@media (max-width: 575.98px) {
  .modchoosercontent .optionscontainer {
    grid-template-columns: repeat(2, 1fr);
  }
}
.modchoosercontent .optionscontainer .option {
  --bs-btn-border-radius: 0.6rem;
  height: fit-content;
  border: 2px solid transparent;
}
.modchoosercontent .optionscontainer .option.selected {
  border: 2px solid #0f6cbf;
  background-color: #e7f0f9;
}
.modchoosercontent .optionscontainer .option:not(.selected):hover {
  background-color: #f8f9fa;
}
.modchoosercontent .optionscontainer .option .optioninfo {
  display: block;
  text-align: initial;
}
.modchoosercontent .optionscontainer .option .optionactions {
  position: relative;
  z-index: 10;
}

/* Option summary styles. */
.modchooser .optionsummary {
  overflow-x: hidden;
  overflow-y: auto;
  width: 100%;
  background-color: #fff;
}
.modchooser .optionsummary .activitychooser-option-details {
  min-width: 15rem;
}
.modchooser .optionsummary .activitychooser-option-details .modchooser-detail:not(:last-child):not(:only-child) {
  margin-bottom: 1.5rem;
  padding-bottom: 1.5rem;
  border-bottom: 1px solid #dee2e6;
}

/* Make modchooser modal full height and scrollable on small screens. */
@media (max-width: 575.98px) {
  .path-course-view .modal-dialog.modal-lg,
  .path-course-view .modal-content {
    height: 100%;
    overflow-y: auto;
    margin: 0;
  }
  .modchooser .modal-body,
  .modchooser .modal-body .carousel,
  .modchooser .modal-body .carousel-inner,
  .modchooser .modal-body .carousel-item,
  .modchooser .modal-body .optionsummary,
  .modchooser .modal-body .searchresultitemscontainer-wrapper {
    height: 100%;
    overflow-y: auto;
  }
  .modchoosercontainer,
  .optionscontainer,
  .searchresultitemscontainer {
    height: 100%;
    overflow-y: auto;
  }
}
/* Anchor link offset fix. This makes hash links scroll 60px down to account for the fixed header. */
:target,
:focus {
  scroll-margin-top: 70px;
}

.pagelayout-embedded :target {
  padding-top: initial;
  margin-top: initial;
}

#nav-drawer.closed {
  left: -305px;
}

#nav-drawer[aria-hidden=true] .list-group-item {
  display: none;
}

/* Use a variable for the drawer background colors. */
[data-region=drawer] {
  position: fixed;
  width: 285px;
  top: 60px;
  height: calc(100% - 60px);
  overflow-y: auto;
  -webkit-overflow-scrolling: touch;
  z-index: 999;
  background-color: #f2f2f2;
  transition: right 0.5s ease, left 0.5s ease;
}
@media (prefers-reduced-motion: reduce) {
  [data-region=drawer] {
    transition: none;
  }
}

@media (min-width: 576px) {
  [data-region=drawer] {
    padding: 20px 20px;
  }
  .jsenabled .popover-process-monitor,
  .jsenabled .btn-footer-popover,
  .jsenabled .btn-footer-communication {
    transition: 0.2s;
  }
}
@media (min-width: 576px) and (prefers-reduced-motion: reduce) {
  .jsenabled .popover-process-monitor,
  .jsenabled .btn-footer-popover,
  .jsenabled .btn-footer-communication {
    transition: none;
  }
}
#nav-drawer {
  right: auto;
  left: 0;
  /* Override the z-indexes defined in bootstrap/_list-group.scss that
     lead to side effects on the user tours positioning. */
}
#nav-drawer .list-group-item-action.active,
#nav-drawer .list-group-item.active {
  z-index: inherit;
}
#nav-drawer .list-group-item-action.active + .list-group-item,
#nav-drawer .list-group-item.active + .list-group-item {
  border-top: none;
}
#nav-drawer .list-group ul {
  list-style: none;
  padding: 0;
  margin: 0;
}
#nav-drawer .list-group li {
  margin-bottom: -1px;
}
#nav-drawer .list-group li:last-child {
  margin-bottom: 0;
}

body.drawer-ease {
  transition: margin-left 0.5s ease, margin-right 0.5s ease;
}
@media (prefers-reduced-motion: reduce) {
  body.drawer-ease {
    transition: none;
  }
}

@media (min-width: 768px) {
  body:not(.uses-drawers).drawer-open-left {
    margin-left: 285px;
  }
}

@media (min-width: 768px) {
  body.drawer-open-left #page.drawers {
    margin-left: 285px;
    padding-left: 1rem;
  }
}

@media (min-width: 768px) {
  body.drawer-open-right {
    margin-right: 315px;
  }
}

[data-region=right-hand-drawer] {
  display: flex;
  flex-direction: column;
  transition: right 0.2s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  [data-region=right-hand-drawer] {
    transition: none;
  }
}
[data-region=right-hand-drawer].drawer {
  z-index: 1021;
  position: fixed;
  top: 60px;
  right: 0;
  height: calc(100% - 60px);
  width: 320px;
  box-shadow: -2px 2px 4px rgba(0, 0, 0, 0.25);
  padding: 0;
  visibility: visible;
  opacity: 1;
}
[data-region=right-hand-drawer].hidden {
  display: block;
  right: -320px;
  visibility: hidden;
  opacity: 0;
  transition: right 0.2s ease-in-out, visibility 0s ease-in-out 0.2s, opacity 0s ease-in-out 0.2s;
}
@media (prefers-reduced-motion: reduce) {
  [data-region=right-hand-drawer].hidden {
    transition: none;
  }
}

@media (max-width: 767.98px) {
  [data-region=right-hand-drawer].drawer {
    top: 0;
    height: 100%;
    z-index: 1031;
  }
  body.drawer-open-left,
  body.drawer-open-right {
    overflow: hidden;
  }
}
.dir-rtl [data-region=right-hand-drawer] {
  box-shadow: 2px 2px 4px rgba(0, 0, 0, 0.25);
}

.drawer {
  background-color: #f8f9fa;
  z-index: 1015;
  position: fixed;
  height: 100vh;
  top: 0;
}
@media (max-width: 991.98px) {
  .drawer {
    z-index: 1035;
  }
}
.drawer.not-initialized {
  display: none;
}
.drawer.drawer-right {
  transition: right 0.2s ease, top 0.2s ease, bottom 0.2s ease, visibility 0.2s ease, transform 0.5s ease;
  width: 315px;
  max-width: 315px;
  right: calc(-315px + -10px);
  visibility: hidden;
}
@media (prefers-reduced-motion: reduce) {
  .drawer.drawer-right {
    transition: none;
  }
}
.drawer.drawer-right.show {
  right: 0;
  visibility: visible;
}
.drawer.drawer-right .drawertoggle {
  margin-left: auto;
  margin-right: 5px;
}
.drawer.drawer-left {
  transition: left 0.2s ease, top 0.2s ease, bottom 0.2s ease, visibility 0.2s ease;
  width: 285px;
  max-width: 285px;
  left: calc(-285px + -10px);
  visibility: hidden;
}
@media (prefers-reduced-motion: reduce) {
  .drawer.drawer-left {
    transition: none;
  }
}
.drawer.drawer-left.show {
  left: 0;
  visibility: visible;
}
.drawer.drawer-left .drawertoggle {
  margin-right: auto;
  margin-left: 5px;
}
.drawer.drawer-bottom {
  bottom: -110%;
}
.drawer.drawer-bottom.show {
  bottom: 0;
}
.drawer.drawer-primary .drawertoggle {
  margin-right: 16px;
  margin-left: 5px;
}

@media (min-width: 992px) {
  .drawer#theme_boost-drawers-blocks:focus-within {
    z-index: 1031;
  }
  .drawer.not-initialized {
    display: block;
  }
}
.drawer-md,
.drawer-sm {
  display: none;
}

.drawerheader {
  padding: 0;
  height: 60px;
  display: flex;
  align-items: center;
}
.drawerheader .sitename {
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
  margin-right: 16px;
}

.drawer.scrolled .drawerheader {
  box-shadow: 0 8px 11px -7px rgba(0, 0, 0, 0.25);
}

@media (max-width: 991.98px) {
  .drawer-md {
    display: block;
    background-color: #f8f9fa;
    z-index: 1015;
    position: fixed;
    height: 100vh;
    top: 0;
  }
}
@media (max-width: 991.98px) and (max-width: 991.98px) {
  .drawer-md {
    z-index: 1035;
  }
}
@media (max-width: 991.98px) {
  .drawer-md.not-initialized {
    display: none;
  }
}
@media (max-width: 991.98px) {
  .drawer-md.drawer-right {
    transition: right 0.2s ease, top 0.2s ease, bottom 0.2s ease, visibility 0.2s ease, transform 0.5s ease;
    width: 315px;
    max-width: 315px;
    right: calc(-315px + -10px);
    visibility: hidden;
  }
}
@media (max-width: 991.98px) and (prefers-reduced-motion: reduce) {
  .drawer-md.drawer-right {
    transition: none;
  }
}
@media (max-width: 991.98px) {
  .drawer-md.drawer-right.show {
    right: 0;
    visibility: visible;
  }
}
@media (max-width: 991.98px) {
  .drawer-md.drawer-right .drawertoggle {
    margin-left: auto;
    margin-right: 5px;
  }
}
@media (max-width: 991.98px) {
  .drawer-md.drawer-left {
    transition: left 0.2s ease, top 0.2s ease, bottom 0.2s ease, visibility 0.2s ease;
    width: 285px;
    max-width: 285px;
    left: calc(-285px + -10px);
    visibility: hidden;
  }
}
@media (max-width: 991.98px) and (prefers-reduced-motion: reduce) {
  .drawer-md.drawer-left {
    transition: none;
  }
}
@media (max-width: 991.98px) {
  .drawer-md.drawer-left.show {
    left: 0;
    visibility: visible;
  }
}
@media (max-width: 991.98px) {
  .drawer-md.drawer-left .drawertoggle {
    margin-right: auto;
    margin-left: 5px;
  }
}
@media (max-width: 991.98px) {
  .drawer-md.drawer-bottom {
    bottom: -110%;
  }
  .drawer-md.drawer-bottom.show {
    bottom: 0;
  }
}
@media (max-width: 991.98px) {
  .drawer-md.drawer-primary .drawertoggle {
    margin-right: 16px;
    margin-left: 5px;
  }
}
@media (max-width: 767.98px) {
  .drawer-sm {
    display: block;
    background-color: #f8f9fa;
    z-index: 1015;
    position: fixed;
    height: 100vh;
    top: 0;
  }
}
@media (max-width: 767.98px) and (max-width: 991.98px) {
  .drawer-sm {
    z-index: 1035;
  }
}
@media (max-width: 767.98px) {
  .drawer-sm.not-initialized {
    display: none;
  }
}
@media (max-width: 767.98px) {
  .drawer-sm.drawer-right {
    transition: right 0.2s ease, top 0.2s ease, bottom 0.2s ease, visibility 0.2s ease, transform 0.5s ease;
    width: 315px;
    max-width: 315px;
    right: calc(-315px + -10px);
    visibility: hidden;
  }
}
@media (max-width: 767.98px) and (prefers-reduced-motion: reduce) {
  .drawer-sm.drawer-right {
    transition: none;
  }
}
@media (max-width: 767.98px) {
  .drawer-sm.drawer-right.show {
    right: 0;
    visibility: visible;
  }
}
@media (max-width: 767.98px) {
  .drawer-sm.drawer-right .drawertoggle {
    margin-left: auto;
    margin-right: 5px;
  }
}
@media (max-width: 767.98px) {
  .drawer-sm.drawer-left {
    transition: left 0.2s ease, top 0.2s ease, bottom 0.2s ease, visibility 0.2s ease;
    width: 285px;
    max-width: 285px;
    left: calc(-285px + -10px);
    visibility: hidden;
  }
}
@media (max-width: 767.98px) and (prefers-reduced-motion: reduce) {
  .drawer-sm.drawer-left {
    transition: none;
  }
}
@media (max-width: 767.98px) {
  .drawer-sm.drawer-left.show {
    left: 0;
    visibility: visible;
  }
}
@media (max-width: 767.98px) {
  .drawer-sm.drawer-left .drawertoggle {
    margin-right: auto;
    margin-left: 5px;
  }
}
@media (max-width: 767.98px) {
  .drawer-sm.drawer-bottom {
    bottom: -110%;
  }
  .drawer-sm.drawer-bottom.show {
    bottom: 0;
  }
}
@media (max-width: 767.98px) {
  .drawer-sm.drawer-primary .drawertoggle {
    margin-right: 16px;
    margin-left: 5px;
  }
}
.drawercontent {
  position: relative;
  z-index: -1;
  height: calc(100% - 60px);
  display: flex;
  flex-direction: column;
  flex-wrap: nowrap;
  overflow-y: auto;
  padding: 0.4rem;
  scrollbar-width: thin;
  scrollbar-color: #6a737b #f8f9fa;
}
.drawercontent .dropdown-menu .dropdown-item {
  width: 220px;
  white-space: normal;
}
.drawercontent::-webkit-scrollbar {
  width: 12px;
}
.drawercontent::-webkit-scrollbar-track {
  background: #f8f9fa;
}
.drawercontent::-webkit-scrollbar-thumb {
  background-color: #6a737b;
  border-radius: 20px;
  border: 3px solid #f8f9fa;
}
.drawercontent::-webkit-scrollbar-thumb:hover {
  background-color: #495057;
}

.fp-content-center {
  height: 100%;
  width: 100%;
  display: table-cell;
  vertical-align: middle;
}

.fp-content-hidden {
  visibility: hidden;
}

.yui3-panel-focused {
  outline: none;
}

.fp-panel-button {
  background: #fff;
  padding: 3px 20px 2px 20px;
  text-align: center;
  margin: 10px;
  border-radius: 10px;
  display: inline-block;
}

.filepicker .yui3-widget-content-expanded {
  height: auto;
}

/* The javascript is adding a style="height: 0px;" to this element - we need to set the min-height so the height is ignored. */
.filepicker .moodle-dialogue-bd {
  min-height: 520px;
}

.fp-navbar {
  border-color: var(--bs-gray-500);
  border-bottom: 0;
  border-bottom-left-radius: 0;
  border-bottom-right-radius: 0;
}

.file-picker .fp-content {
  border-top: 0;
  background: #fff;
  clear: none;
  overflow: auto;
  height: 452px;
}

.filepicker.moodle-dialogue-fullscreen .file-picker .fp-content {
  width: 100%;
}

.file-picker .fp-content-loading {
  height: 100%;
  width: 100%;
  display: table;
  text-align: center;
}

.file-picker .fp-content .fp-object-container {
  width: 98%;
  height: 98%;
}

.file-picker .fp-def-search {
  margin-top: 0;
}

.file-picker .fp-list {
  list-style-type: none;
  padding: 0;
  float: left;
  width: 100%;
  margin: 0;
}

.file-picker .fp-list .fp-repo a {
  display: block;
  padding: 0.5em 0.7em;
}

.file-picker .fp-list .fp-repo.active {
  background: #f2f2f2;
}

.file-picker .fp-list .fp-repo-icon {
  padding: 0 7px 0 5px;
  width: 16px;
  height: 16px;
}

.fp-toolbar {
  float: left;
}

.fp-toolbar.empty {
  display: none;
}

.fp-toolbar .disabled {
  display: none;
}

.fp-toolbar div {
  display: block;
  float: left;
  margin-right: 4px;
}

.fp-toolbar img {
  vertical-align: -15%;
  margin-right: 5px;
}

.fp-viewbar:not(.disabled) a.checked {
  background-color: #b1bbc4;
  color: #000;
  border-color: #aab4bf;
}

.fp-viewbar.disabled a {
  pointer-events: none;
  opacity: 0.65;
}

.file-picker .fp-clear-left {
  clear: left;
}

.fp-pathbar.empty {
  display: none;
}

.fp-pathbar .fp-path-folder {
  background: url("[[pix:theme|fp/path_folder]]") left 3px no-repeat;
  background-size: 12px 12px;
  height: 12px;
  margin-left: 12px;
}

/*rtl:raw:
.fp-pathbar .fp-path-folder {
    background-image: url('[[pix:theme|fp/path_folder_rtl]]');
}
*/
.fp-pathbar .fp-path-folder-name {
  margin-left: 24px;
}

.fp-iconview .fp-file {
  float: left;
  text-align: center;
  position: relative;
  margin: 10px 10px 35px;
}

.fp-iconview .fp-thumbnail {
  min-width: 110px;
  min-height: 110px;
  line-height: 110px;
  text-align: center;
  border: 1px solid #fff;
  display: block;
}

.fp-iconview .fp-thumbnail img {
  border: 1px solid #ddd;
  padding: 3px;
  vertical-align: middle;
}

.fp-iconview .fp-thumbnail:hover {
  background: #fff;
  border: 1px solid #ddd;
}

.fp-iconview .fp-filename-field {
  height: 33px;
  margin-top: 3px;
  word-wrap: break-word;
  overflow: hidden;
  position: absolute;
}

.fp-iconview .fp-file:focus .fp-filename-field,
.fp-iconview .fp-file:hover .fp-filename-field {
  overflow: visible;
  z-index: 1000;
}
.fp-iconview .fp-file:focus .fp-filename,
.fp-iconview .fp-file:hover .fp-filename {
  overflow: inherit;
  white-space: normal;
  text-overflow: inherit;
}

.fp-iconview .fp-filename-field .fp-filename {
  background: #fff;
  padding-top: 5px;
  padding-bottom: 12px;
  min-width: 112px;
}

.file-picker .yui3-datatable table {
  border: 0 solid #bbb;
  width: 100%;
}

.file-picker .ygtvtn,
.filemanager .ygtvtn {
  /*rtl:remove*/
  background: url("[[pix:moodle|y/tn]]") 0 0 no-repeat;
  /*rtl:raw:
  background: url('[[pix:moodle|y/tn_rtl]]') 0 0 no-repeat;
  */
  width: 19px;
  height: 32px;
}

.file-picker .ygtvtm,
.filemanager .ygtvtm {
  background: url("[[pix:moodle|y/tm]]") 0 10px no-repeat;
  /*rtl:raw:
      background-position: 2px 10px;
  */
  width: 13px;
  height: 12px;
  cursor: pointer;
}

.file-picker .ygtvtmh,
.filemanager .ygtvtmh {
  background: url("[[pix:moodle|y/tm]]") 0 10px no-repeat;
  /*rtl:raw:
      background-position: 2px 10px;
  */
  width: 13px;
  height: 12px;
  cursor: pointer;
}

.file-picker .ygtvtp,
.filemanager .ygtvtp {
  /*rtl:remove*/
  background: url("[[pix:moodle|y/tp]]") 0 10px no-repeat;
  /*rtl:raw:
  background: url('[[pix:moodle|y/tp_rtl]]') 2px 10px no-repeat;
  */
  width: 13px;
  height: 12px;
  cursor: pointer;
}

.file-picker .ygtvtph,
.filemanager .ygtvtph {
  /*rtl:remove*/
  background: url("[[pix:moodle|y/tp]]") 0 10px no-repeat;
  /*rtl:raw:
  background: url('[[pix:moodle|y/tp_rtl]]') 2px 10px no-repeat;
  */
  width: 13px;
  height: 22px;
  cursor: pointer;
}

.file-picker .ygtvln,
.filemanager .ygtvln {
  /*rtl:remove*/
  background: url("[[pix:moodle|y/ln]]") 0 0 no-repeat;
  /*rtl:raw:
  background: url('[[pix:moodle|y/ln_rtl]]') 0 0 no-repeat;
  */
  width: 19px;
  height: 32px;
}

.file-picker .ygtvlm,
.filemanager .ygtvlm {
  background: url("[[pix:moodle|y/lm]]") 0 10px no-repeat;
  /*rtl:raw:
      background-position: 2px 10px;
  */
  width: 13px;
  height: 12px;
  cursor: pointer;
}

.file-picker .ygtvlmh,
.filemanager .ygtvlmh {
  background: url("[[pix:moodle|y/lm]]") 0 10px no-repeat;
  /*rtl:raw:
      background-position: 2px 10px;
  */
  width: 13px;
  height: 12px;
  cursor: pointer;
}

.file-picker .ygtvlp,
.filemanager .ygtvlp {
  /*rtl:remove*/
  background: url("[[pix:moodle|y/lp]]") 0 10px no-repeat;
  /*rtl:raw:
  background: url('[[pix:moodle|y/lp_rtl]]') 2px 10px no-repeat;
  */
  width: 13px;
  height: 12px;
  cursor: pointer;
}

.file-picker .ygtvlph,
.filemanager .ygtvlph {
  /*rtl:remove*/
  background: url("[[pix:moodle|y/lp]]") 0 10px no-repeat;
  /*rtl:raw:
  background: url('[[pix:moodle|y/lp_rtl]]') 2px 10px no-repeat;
  */
  width: 13px;
  height: 12px;
  cursor: pointer;
}

.file-picker .ygtvloading,
.filemanager .ygtvloading {
  background: transparent url("[[pix:moodle|y/loading]]") 0 0 no-repeat;
  width: 16px;
  height: 22px;
}

.file-picker .ygtvdepthcell,
.filemanager .ygtvdepthcell {
  background: url("[[pix:moodle|y/vline]]") 0 0 no-repeat;
  /*rtl:raw:
  background-position: 0 0;
  */
  width: 17px;
  height: 32px;
}

.file-picker .ygtvblankdepthcell,
.filemanager .ygtvblankdepthcell {
  width: 17px;
  height: 22px;
}

a.ygtvspacer:hover {
  color: transparent;
  text-decoration: none;
}

.ygtvlabel,
.ygtvlabel:link,
.ygtvlabel:visited,
.ygtvlabel:hover {
  background-color: transparent;
  cursor: pointer;
  margin-left: 2px;
  text-decoration: none;
}

.file-picker .ygtvfocus,
.filemanager .ygtvfocus {
  background-color: #eee;
}

.fp-filename-icon {
  margin-top: 10px;
  display: block;
  position: relative;
}

.fp-icon {
  float: left;
  margin-top: -7px;
  width: 24px;
  height: 24px;
  margin-right: 10px;
  text-align: center;
  line-height: 24px;
}

.fp-icon img {
  max-height: 24px;
  max-width: 24px;
  vertical-align: middle;
}

.fp-filename {
  padding-right: 10px;
}

.file-picker .fp-login-form {
  height: 100%;
  width: 100%;
  display: table;
}

.file-picker .fp-upload-form {
  height: 100%;
  width: 100%;
  display: table;
}

.file-picker .fp-upload-form table {
  margin: 0 auto;
}

.file-picker.fp-dlg {
  text-align: center;
}

.file-picker.fp-dlg .fp-dlg-buttons {
  margin: 0 20px;
}

.file-picker.fp-msg {
  text-align: center;
}

.file-picker .fp-content-error {
  height: 100%;
  width: 100%;
  display: table;
  text-align: center;
}

.file-picker .fp-nextpage {
  clear: both;
}

.file-picker .fp-nextpage .fp-nextpage-loading {
  display: none;
}

.file-picker .fp-nextpage.loading .fp-nextpage-link {
  display: none;
}

.file-picker .fp-nextpage.loading .fp-nextpage-loading {
  display: block;
  text-align: center;
  height: 100px;
  padding-top: 50px;
}

.fp-select .fp-select-loading {
  text-align: center;
  margin-top: 20px;
}

.fp-select table {
  padding: 0 0 10px;
}

.fp-select table .mdl-right {
  min-width: 84px;
}

.fp-select .fp-reflist .mdl-right {
  vertical-align: top;
}

.fp-select .fp-select-buttons {
  float: right;
}

.fp-select .fp-info {
  font-size: 0.703125rem;
}

.fp-select .fp-thumbnail {
  float: left;
  min-width: 110px;
  min-height: 110px;
  line-height: 110px;
  text-align: center;
  margin: 10px 20px 0 0;
  background: #fff;
  border: 1px solid #ddd;
}

.fp-select .fp-thumbnail img {
  border: 1px solid #ddd;
  padding: 3px;
  vertical-align: middle;
  margin: 10px;
}

.fp-select .fp-fileinfo {
  display: inline-block;
  margin-top: 10px;
}

.file-picker.fp-select .fp-fileinfo {
  max-width: 240px;
}

.fp-select .fp-fileinfo div {
  padding-bottom: 5px;
}

.file-picker.fp-select .uneditable {
  display: none;
}

.file-picker.fp-select .fp-select-loading {
  display: none;
}

.file-picker.fp-select.loading .fp-select-loading {
  display: block;
}

.file-picker.fp-select.loading form {
  display: none;
}

.fp-select .fp-dimensions.fp-unknown {
  display: none;
}

.fp-select .fp-size.fp-unknown {
  display: none;
}

.filemanager-loading {
  display: none;
}

.jsenabled .filemanager-loading {
  display: block;
  margin-top: 100px;
}

.filemanager.fm-loading .filemanager-toolbar,
.filemanager.fm-loading .fp-pathbar,
.filemanager.fm-loading .filemanager-container,
.filemanager.fm-loaded .filemanager-loading,
.filemanager.fm-maxfiles .fp-btn-add,
.filemanager.fm-maxfiles .dndupload-message,
.filemanager.fm-noitems .fp-btn-download,
.filemanager.fm-noitems .fp-btn-delete,
.filemanager .fm-empty-container,
.filemanager.fm-noitems .filemanager-container .fp-content {
  display: none;
}

.filemanager .fp-img-downloading {
  display: none;
  padding-top: 7px;
}

.filemanager .filemanager-updating {
  display: none;
  text-align: center;
}

.filemanager.fm-updating .filemanager-updating {
  display: block;
  margin-top: 37px;
}

.filemanager.fm-updating .fm-content-wrapper,
.filemanager.fm-nomkdir .fp-btn-mkdir,
.fitem.disabled .filemanager .filemanager-toolbar,
.fitem.disabled .filemanager .fp-pathbar,
.fitem.disabled .filemanager .fp-restrictions,
.fitem.disabled .filemanager .fm-content-wrapper {
  display: none;
}

.filemanager .fp-restrictions {
  text-align: right;
}

.filemanager-toolbar {
  padding: 4px;
  overflow: hidden;
}

.filemanager .fp-pathbar.empty {
  display: none;
}

.filepicker-filelist,
.filemanager-container {
  min-height: 140px;
  border: 1px solid var(--bs-gray-500);
  border-radius: 0.5rem;
}

.filemanager .fp-content {
  overflow: auto;
  max-height: 472px;
  min-height: 157px;
}

.filemanager-container,
.filepicker-filelist {
  overflow: hidden;
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}

.file-picker .yui3-datatable-header {
  /*rtl:raw:
  text-align: right;
  */
  background: initial;
}

.fitem.disabled .filepicker-filelist,
.fitem.disabled .filemanager-container {
  background-color: #ebebe4;
}

.fitem.disabled .fp-btn-choose {
  color: var(--bs-secondary-color);
}

.fitem.disabled .filepicker-filelist .filepicker-filename {
  display: none;
}

.fp-iconview .fp-reficons1 {
  position: absolute;
  height: 100%;
  width: 100%;
  top: 0;
  left: 0;
}

.fp-iconview .fp-reficons2 {
  position: absolute;
  height: 100%;
  width: 100%;
  top: 0;
  left: 0;
}

.fp-iconview .fp-file.fp-hasreferences .fp-reficons1 {
  background: url("[[pix:theme|fp/link]]") no-repeat;
  /*rtl:raw:
  transform: scaleX(-1);
  */
  /*rtl:ignore*/
  background-position: bottom right;
  background-size: 16px 16px;
}

.fp-iconview .fp-file.fp-isreference .fp-reficons2 {
  background: url("[[pix:theme|fp/alias]]") no-repeat;
  /*rtl:raw:
  transform: scaleX(-1);
  */
  /*rtl:ignore*/
  background-position: bottom left;
  background-size: 16px 16px;
}

.filemanager .fp-iconview .fp-file.fp-originalmissing .fp-thumbnail img {
  display: none;
}

.filemanager .fp-iconview .fp-file.fp-originalmissing .fp-thumbnail {
  background: url([[pix:s/dead]]) no-repeat;
  background-position: center center;
}

.filemanager .yui3-datatable table {
  border: 0 solid #bbb;
  width: 100%;
}

/* Override YUI default styling */
/* stylelint-disable declaration-no-important */
.filemanager .yui3-datatable-header {
  /*rtl:raw:
  text-align: right;
  */
  background: #fff !important;
  border-bottom: 1px solid #ccc !important;
  border-left: 0 solid #fff !important;
  color: #555 !important;
}
.filemanager .yui3-datatable-odd .yui3-datatable-cell {
  background-color: #f6f6f6 !important;
  border-left: 0 solid #f6f6f6;
}
.filemanager .yui3-datatable-even .yui3-datatable-cell {
  background-color: #fff !important;
  border-left: 0 solid #fff;
}

/* stylelint-enable */
.filemanager .fp-filename-icon.fp-hasreferences .fp-reficons1 {
  background: url("[[pix:theme|fp/link_sm]]") no-repeat 0 0;
  height: 100%;
  width: 100%;
  /*rtl:raw:
  transform: scaleX(-1);
  */
  position: absolute;
  top: 8px;
  left: 17px;
  background-size: 16px 16px;
}

.filemanager .fp-filename-icon.fp-isreference .fp-reficons2 {
  background: url("[[pix:theme|fp/alias_sm]]") no-repeat 0 0;
  height: 100%;
  width: 100%;
  /*rtl:raw:
  transform: scaleX(-1);
  */
  position: absolute;
  top: 9px;
  left: -6px;
  background-size: 16px 16px;
}

.filemanager .fp-contextmenu {
  display: none;
}

.filemanager .fp-iconview .fp-folder.fp-hascontextmenu .fp-contextmenu {
  position: absolute;
  right: 0;
  bottom: 0;
  display: flex;
  align-items: center;
  justify-content: center;
}

.filemanager .fp-treeview .fp-folder.fp-hascontextmenu .fp-contextmenu,
.filemanager .fp-tableview .fp-folder.fp-hascontextmenu .fp-contextmenu {
  display: inline;
  position: absolute;
  left: 14px;
  margin-right: -20px;
  top: 6px;
}

.filepicker-filelist .filepicker-container,
.filemanager.fm-noitems .fm-empty-container {
  display: block;
  position: absolute;
  top: 10px;
  bottom: 10px;
  left: 10px;
  right: 10px;
  border: 2px dashed #bbb;
  padding-top: 85px;
  text-align: center;
}

.filepicker-filelist .dndupload-target,
.filemanager-container .dndupload-target {
  background: #fff;
  position: absolute;
  top: 10px;
  bottom: 10px;
  left: 10px;
  right: 10px;
  border: 2px dashed #fb7979;
  padding-top: 85px;
  text-align: center;
}

.filepicker-filelist.dndupload-over .dndupload-target,
.filemanager-container.dndupload-over .dndupload-target {
  background: #fff;
  position: absolute;
  top: 10px;
  bottom: 10px;
  left: 10px;
  right: 10px;
  border: 2px dashed #6c8cd3;
  padding-top: 85px;
  text-align: center;
}

.dndupload-message {
  display: none;
}

.dndsupported .dndupload-message {
  display: inline;
}

.dnduploadnotsupported-message {
  display: none;
}

.dndnotsupported .dnduploadnotsupported-message {
  display: inline;
}

.dndupload-target {
  display: none;
}

.dndsupported .dndupload-ready .dndupload-target {
  display: block;
}

.dndupload-uploadinprogress {
  display: none;
  text-align: center;
}

.dndupload-uploading .dndupload-uploadinprogress {
  display: block;
}

.dndupload-arrow {
  width: 100%;
  height: 80px;
  position: absolute;
  top: 5px;
  color: #8f959e;
}

.fitem.disabled .filepicker-container,
.fitem.disabled .fm-empty-container {
  display: none;
}

.dndupload-progressbars {
  padding: 10px;
  display: none;
}

.dndupload-inprogress .dndupload-progressbars {
  display: block;
}

.dndupload-inprogress .fp-content {
  display: none;
}

.filemanager.fm-noitems .dndupload-inprogress .fm-empty-container {
  display: none;
}

.filepicker-filelist.dndupload-inprogress .filepicker-container {
  display: none;
}

.filepicker-filelist.dndupload-inprogress a {
  display: none;
}

.filemanager.fp-select .fp-select-loading {
  display: none;
}

.filemanager.fp-select.loading .fp-select-loading {
  display: block;
}

.filemanager.fp-select.loading form {
  display: none;
}

.filemanager.fp-select.fp-folder .fp-license,
.filemanager.fp-select.fp-folder .fp-author,
.filemanager.fp-select.fp-file .fp-file-unzip,
.filemanager.fp-select.fp-folder .fp-file-unzip,
.filemanager.fp-select.fp-file .fp-file-zip,
.filemanager.fp-select.fp-zip .fp-file-zip {
  display: none;
}

.filemanager.fp-select .fp-file-setmain,
.filemanager.fp-select .fp-file-setmain-help {
  display: none;
}

.filemanager.fp-select.fp-cansetmain .fp-file-setmain,
.filemanager.fp-select.fp-cansetmain .fp-file-setmain-help {
  display: inline-block;
}

.filemanager .fp-mainfile .fp-filename {
  font-weight: bold;
}

.filemanager.fp-select.fp-folder .fp-file-download {
  display: none;
}

.fm-operation {
  font-weight: bold;
}

.filemanager.fp-select .fp-original.fp-unknown,
.filemanager.fp-select .fp-original .fp-originloading {
  display: none;
}

.filemanager.fp-select .fp-original.fp-loading .fp-originloading {
  display: inline;
}

.filemanager.fp-select .fp-reflist.fp-unknown,
.filemanager.fp-select .fp-reflist .fp-reflistloading {
  display: none;
}

.filemanager.fp-select .fp-reflist.fp-loading .fp-reflistloading {
  display: inline;
}

.filemanager.fp-select .fp-reflist .fp-value {
  background: #f9f9f9;
  border: 1px solid #bbb;
  padding: 8px 7px;
  margin: 0;
  max-height: 75px;
  overflow: auto;
}

.filemanager.fp-select .fp-reflist .fp-value li {
  padding-bottom: 7px;
}

.filemanager.fp-mkdir-dlg {
  text-align: center;
}

.filemanager.fp-mkdir-dlg .fp-mkdir-dlg-text {
  text-align: left;
  margin: 20px;
}

.filemanager.fp-dlg {
  text-align: center;
}

.file-picker div.bd {
  text-align: left;
}

.fp-formset {
  padding: 10px;
}
.fp-formset input[type=file] {
  line-height: inherit;
}

.fp-forminset {
  padding: 0 10px;
}

.fp-fileinfo .fp-value {
  display: inline-block;
  padding-left: 5px;
}

/** The message area **/
.hidden {
  display: none;
}

.preferences-container .container-fluid, .preferences-container .container-sm, .preferences-container .container-md, .preferences-container .container-lg, .preferences-container .container-xl, .preferences-container .container-xxl {
  padding: 0;
}
.preferences-container .container-fluid .col-md-6, .preferences-container .container-sm .col-md-6, .preferences-container .container-md .col-md-6, .preferences-container .container-lg .col-md-6, .preferences-container .container-xl .col-md-6, .preferences-container .container-xxl .col-md-6 {
  min-height: 20px;
}
.preferences-container .align-bottom {
  vertical-align: bottom;
}
.preferences-container .preference-table {
  border: 1px solid #ddd;
}
.preferences-container .preference-table thead th {
  text-align: center;
}
.preferences-container .preference-table thead th .config-warning {
  display: none;
}
.preferences-container .preference-table thead th.unconfigured .config-warning {
  display: inline-block;
}
.preferences-container .preference-table tr th {
  border-left: 1px solid #dee2e6;
}
.preferences-container .preference-table tr td:not(:first-child) {
  width: 150px;
  text-align: center;
}
.preferences-container .preference-table tr td:nth-child(even) {
  border: 1px solid #dee2e6;
}
.preferences-container .preference-table .preference-row .hover-tooltip-container {
  display: inline-block;
}
.preferences-container .preference-table .preference-row .preference-name {
  vertical-align: middle;
}
.preferences-container .preference-table .preference-row .disabled-message {
  text-align: center;
  height: 30px;
  line-height: 30px;
}
.preferences-container .preference-table .preference-row.loading .preference-name .loading-icon {
  display: block;
}

.disabled-message {
  display: none;
}

.disabled .disabled-message {
  display: block;
}
.disabled .disabled-message + form {
  display: none;
}

.general-settings-container .loading-icon {
  display: none;
}
.general-settings-container .loading .loading-icon {
  display: inline-block;
}
.general-settings-container label {
  display: inline-block;
}

.processor-container {
  position: relative;
}
.processor-container .loading-container {
  display: none;
  position: absolute;
  width: 100%;
  height: 100%;
  text-align: center;
  background-color: rgba(255, 255, 255, 0.5);
}
.processor-container .loading-container .vertical-align {
  height: 100%;
  width: 0%;
  display: inline-block;
  vertical-align: middle;
}
.processor-container.loading .loading-container {
  display: block;
}

.preferences-page-container .checkbox-container {
  margin: 30px 5px;
  line-height: 20px;
}
.preferences-page-container .checkbox-container input {
  line-height: 20px;
  margin: 0;
}
.preferences-page-container .checkbox-container .loading-icon {
  display: none;
}
.preferences-page-container .checkbox-container.loading .loading-icon {
  display: inline-block;
}

.notification-area {
  height: 600px;
  box-sizing: border-box;
  border-radius: 4px;
  margin-bottom: 30px;
  border: 1px solid #e3e3e3;
}
@media (max-height: 670px) {
  .notification-area {
    height: 500px;
  }
}
.notification-area .control-area {
  box-sizing: border-box;
  display: inline-block;
  width: 300px;
  height: 100%;
  overflow: auto;
  -webkit-overflow-scrolling: touch;
  border-right: 1px solid #e3e3e3;
}
.notification-area .control-area .content {
  position: relative;
}
.notification-area .control-area .content .content-item-container {
  cursor: pointer;
}
.notification-area .control-area .content:empty + .empty-text {
  display: block;
}
.notification-area .control-area .loading-icon {
  display: none;
}
.notification-area .control-area .empty-text {
  display: none;
  text-align: center;
  padding-top: 20px;
}
.notification-area .control-area.loading .loading-icon {
  display: block;
  text-align: center;
  box-sizing: border-box;
  padding: 5px;
}
.notification-area .control-area.loading .content:empty + .empty-text {
  display: none;
}
.notification-area .content-area {
  box-sizing: border-box;
  display: inline-block;
  width: calc(100% - 300px);
  float: right;
}
.notification-area .content-area .toggle-mode {
  display: none;
}
.notification-area .content-area .header {
  height: 50px;
  box-sizing: border-box;
  border-bottom: 1px solid #e3e3e3;
  padding: 5px;
}
.notification-area .content-area .header .image-container {
  display: inline-block;
  height: 25px;
  width: 24px;
  float: left;
}
.notification-area .content-area .header .subject-container {
  display: inline-block;
  max-width: calc(100% - 24px);
  white-space: nowrap;
  overflow: hidden;
  text-overflow: ellipsis;
  height: 25px;
  padding-left: 5px;
  box-sizing: border-box;
}
.notification-area .content-area .header .timestamp {
  font-size: 10px;
  line-height: 10px;
  margin: 0;
  color: #666;
  padding-left: 5px;
}
.notification-area .content-area .header:empty {
  display: none;
}
.notification-area .content-area > .content {
  height: 500px;
  box-sizing: border-box;
  overflow: auto;
  -webkit-overflow-scrolling: touch;
  padding: 15px;
}
@media (max-height: 670px) {
  .notification-area .content-area > .content {
    height: 400px;
  }
}
.notification-area .content-area > .content:empty {
  display: none;
}
.notification-area .content-area > .content:empty + .empty-text {
  display: block;
  text-align: center;
  padding-top: 100px;
}
.notification-area .content-area .empty-text {
  display: none;
}
.notification-area .content-area .footer {
  height: 50px;
  box-sizing: border-box;
  text-align: center;
}
.notification-area .content-area .footer a {
  line-height: 50px;
}
.notification-area .content-area .footer:empty {
  display: none;
}

@media (max-width: 979px) {
  .notification-area {
    position: relative;
    overflow: hidden;
  }
  .notification-area .control-area {
    border-right: none;
    width: 100%;
    position: absolute;
    top: 0;
    left: 0;
    opacity: 1;
    visibility: visible;
    transition: left 0.25s;
  }
  .notification-area .content-area {
    width: 100%;
    position: absolute;
    top: 0;
    right: -100%;
    opacity: 0;
    visibility: hidden;
    transition: right 0.25s, opacity 0.25s, visibility 0.25s;
  }
  .notification-area .content-area .toggle-mode {
    display: inline-block;
    float: left;
    width: 70px;
    height: 50px;
    line-height: 50px;
    box-sizing: border-box;
    border-right: 1px solid #e3e3e3;
    border-bottom: 1px solid #e3e3e3;
  }
  .notification-area .content-area .header {
    display: inline-block;
    width: calc(100% - 70px);
  }
  .notification-area.show-content-area .control-area {
    left: -100%;
    opacity: 0;
    visibility: hidden;
    transition: left 0.25s, opacity 0.25s, visibility 0.25s;
  }
  .notification-area.show-content-area .content-area {
    right: 0;
    opacity: 1;
    visibility: visible;
    transition: right 0.25s;
  }
}
.drawer .message-app {
  height: 100%;
}
.drawer .message-app .icon-back-in-app {
  display: none;
}
.drawer .message-app .icon-back-in-drawer {
  display: inherit;
}

.message-app {
  display: flex;
  flex-direction: column;
  background-color: #eff1f3;
}
.message-app .icon-back-in-drawer {
  display: none;
}
.message-app.main {
  min-height: 400px;
}
.message-app .header-container {
  flex-shrink: 0;
}
.message-app .overflow-y {
  overflow-y: auto;
}
@media (max-height: 320px) {
  .message-app .header-container [data-region=view-overview]:not(.hidden) {
    display: flex;
    align-items: center;
  }
  .message-app .overflow-y {
    overflow-y: unset;
  }
}
.message-app .body-container {
  flex: 1;
  overflow: hidden;
}
.message-app .body-container > * {
  position: absolute;
  right: 0;
  left: 0;
  top: 0;
  bottom: 0;
  overflow: auto;
}
.message-app .footer-container {
  flex-shrink: 0;
}
.message-app .footer-container textarea {
  direction: ltr;
}
.message-app .contact-status {
  position: absolute;
  left: 39px;
  top: 34px;
}
.message-app .contact-status.online .icon {
  color: #357a32;
}
.message-app .message p {
  margin: 0;
}
.message-app .clickable {
  cursor: pointer;
}
.message-app .clickable:hover {
  filter: drop-shadow(2px 2px 2px rgba(0, 0, 0, 0.3));
}
.message-app a,
.message-app .btn-link {
  color: inherit;
}
.message-app .btn-link:hover, .message-app .btn-link:focus {
  background-color: rgba(0, 0, 0, 0.035);
  text-decoration: none;
}
.message-app .icon {
  margin-right: 0;
}
.message-app .overview-section-toggle .collapsed-icon-container {
  display: none;
}
.message-app .overview-section-toggle .expanded-icon-container {
  display: inline-block;
}
.message-app .overview-section-toggle.collapsed .collapsed-icon-container {
  display: inline-block;
}
.message-app .overview-section-toggle.collapsed .expanded-icon-container {
  display: none;
}
.message-app .view-overview-body .section {
  display: block;
}
.message-app .view-overview-body .section.expanded {
  display: flex;
}
.message-app .view-overview-body .section div[data-region=toggle] {
  padding: 0.1rem;
}
.message-app .view-conversation .content-message-container img {
  max-width: 100%;
}
.message-app .list-group {
  border-radius: 0;
}
.message-app .list-group .list-group-item {
  border-left: 0;
  border-right: 0;
}
.message-app .list-group .list-group-item:hover {
  color: #fff;
  background-color: #0f6cbf;
}
.message-app .list-group .list-group-item:first-child {
  border-top: 0;
}
.message-app .list-group .list-group-item:last-child {
  border-bottom: 0;
}
.message-app .list-group .list-group-item.list-group-item-action {
  margin: 0.1rem;
  width: auto;
  text-align: inherit;
}
.message-app .last-message {
  min-height: 1.5rem;
}
.message-app .section .collapsing {
  overflow: hidden;
}
.message-app .message.send {
  background-color: #dee2e6;
  color: #000;
}
.message-app .message.send .time {
  color: black;
}
.message-app .message.send .tail {
  right: 0;
  margin-right: -0.5rem;
  border-bottom-color: #dee2e6;
}
.message-app .message.received {
  background-color: #fff;
  color: #000;
}
.message-app .message.received .time {
  color: #4d4d4d;
}
.message-app .message.received .tail {
  left: 0;
  margin-left: -0.5rem;
  border-bottom-color: #fff;
}
.message-app .message .tail {
  content: "";
  bottom: 0;
  width: 0;
  height: 0;
  border: 0.5rem solid transparent;
  position: relative;
}
.message-app .day {
  color: #000;
}
.message-app .lazy-load-list {
  overflow-y: inherit;
}

#page-message-index #page-header {
  display: none;
}
#page-message-index #region-main {
  height: 100%;
  margin-top: 0;
}
#page-message-index #region-main .conversationcontainer .section {
  max-height: calc(100vh - 50px);
}
#page-message-index #region-main div[role=main] {
  height: 100%;
}
#page-message-index #region-main div[role=main] #maincontent {
  margin-top: -1px;
}
#page-message-index #region-main div[role=main] .message-app.main {
  height: 100%;
}

.dir-rtl .message-drawer {
  box-shadow: 2px 2px 4px rgba(0, 0, 0, 0.08);
}

.message-app .emoji-picker-container {
  position: absolute;
  top: -5px;
  right: 5px;
  transform: translateY(-100%);
}
.message-app .emoji-picker-container .emoji-picker .picker-row .emoji-button {
  height: 40px;
  width: 40px;
}
@media (max-width: 575.98px) {
  .message-app .emoji-picker-container {
    right: -0.5rem;
  }
}
@media (max-height: 495px) {
  .message-app .emoji-picker-container {
    position: fixed;
    top: 0;
    transform: none;
  }
}
.message-app .emoji-auto-complete-container {
  overflow: auto;
  max-height: 90px;
  transition: max-height 0.15s ease-in-out;
  visibility: visible;
}
.message-app .emoji-auto-complete-container.hidden {
  display: block;
  max-height: 0;
  visibility: hidden;
  overflow: hidden;
  transition: max-height 0.15s ease-in-out, visibility 0s linear 0.15s, overflow 0s linear 0.15s;
}

/* Question */
.questionbank h2, .questionbank .h2 {
  margin-top: 0;
}

.questioncategories h3, .questioncategories .h3 {
  margin-top: 0;
}

#chooseqtypebox {
  margin-top: 1em;
}

#chooseqtype h3, #chooseqtype .h3 {
  margin: 0 0 0.3em;
}

#chooseqtype .instruction {
  display: none;
}

#chooseqtype .fakeqtypes {
  border-top: 1px solid silver;
}

#chooseqtype .qtypeoption {
  margin-bottom: 0.5em;
}

#chooseqtype label {
  display: block;
}

#chooseqtype .qtypename img {
  padding: 0 0.3em;
}

#chooseqtype .qtypename {
  display: inline-table;
  width: 16em;
}

#chooseqtype .qtypesummary {
  display: block;
  margin: 0 2em;
}

#chooseqtype .submitbuttons {
  margin: 0.7em 0;
  text-align: center;
}

#qtypechoicecontainer {
  display: none;
}

#qtypechoicecontainer_c.yui-panel-container.shadow .underlay {
  background: none;
}

#qtypechoicecontainer.yui-panel .hd {
  color: #333;
  letter-spacing: 1px;
  text-shadow: 1px 1px 1px #fff;
  border-top-left-radius: 10px;
  border-top-right-radius: 10px;
  border: 1px solid #ccc;
  border-bottom: 1px solid #bbb;
  background-image: linear-gradient(to bottom, #fff, #ccc);
}

#qtypechoicecontainer {
  font-size: 12px;
  color: #333;
  background: #f2f2f2;
  border-radius: 10px;
  border: 1px solid #ccc;
  border-top: 0 none;
}

#qtypechoicecontainer #chooseqtype {
  width: 40em;
}

#chooseqtypehead h3, #chooseqtypehead .h3 {
  margin: 0;
  font-weight: normal;
}

#chooseqtype .qtypes {
  position: relative;
  border-bottom: 1px solid #bbb;
  padding: 0.24em 0;
}

#chooseqtype .alloptions {
  overflow-x: hidden;
  overflow-y: auto;
  max-height: calc(100vh - 15em);
  width: 60%;
}

#chooseqtype .qtypeoption {
  margin-bottom: 0;
  padding: 0.3em 0.3em 0.3em 1.6em;
}

#chooseqtype .qtypeoption img {
  vertical-align: text-bottom;
  padding-left: 1em;
  padding-right: 0.5em;
}

#chooseqtype .selected {
  background-color: #fff;
}

#chooseqtype .instruction,
#chooseqtype .qtypesummary {
  display: none;
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  left: 60%;
  margin: 0;
  overflow-x: hidden;
  overflow-y: auto;
  padding: 1.5em 1.6em;
  background-color: #fff;
}

#chooseqtype .instruction,
#chooseqtype .selected .qtypesummary {
  display: block;
}

.chooserdialogue-questionchooser .modicon img.icon,
#categoryquestions .qtype img.icon,
.choosercontainer .modicon img.icon {
  /* Override default icon width set for fontawesome icons */
  width: 16px;
}

table.question-bank-table {
  margin: 0;
  background-color: #fff;
  table-layout: fixed;
  overflow-x: scroll;
  width: min-content;
  max-width: 100%;
}
table.question-bank-table th {
  text-align: left;
}
table.question-bank-table > tbody > tr.r1 {
  background-color: rgba(var(--bs-emphasis-color-rgb), 0.03);
}
table.question-bank-table.generaltable > tbody > tr.highlight.firstrow td {
  border-top: 3px solid #008196;
}
table.question-bank-table > tbody > tr.highlight.firstrow {
  border-right: 2px solid #008196;
  border-left: 2px solid #008196;
}
table.question-bank-table.generaltable > tbody > tr.highlight.extrarow td, table.question-bank-table.generaltable > tbody > tr.highlight.firstrow.onlyhighlightrow td {
  border-bottom: 2px solid #008196;
}
table.question-bank-table > tbody > tr.highlight.extrarow {
  border-top: 0;
  border-right: 2px solid #008196;
  border-left: 2px solid #008196;
}
table.question-bank-table .checkbox input[type=checkbox] {
  margin-left: 0;
  float: none;
}
table.question-bank-table .iconcol {
  padding: 3px;
  box-sizing: content-box;
}
table.question-bank-table .iconcol .icon {
  margin: 0;
  width: 12px;
  height: 12px;
}
table.question-bank-table label {
  margin: 0;
  display: block;
}
table.question-bank-table .header {
  text-align: left;
}
table.question-bank-table .header.sortable-list-current-position {
  background-color: #a2cff8;
}
table.question-bank-table .header.sortable-list-is-dragged {
  background-color: #fff;
  opacity: 0.85;
}
table.question-bank-table .header .header-text > div {
  display: inline-block;
}
table.question-bank-table .header .dropdown-toggle::after {
  margin-left: 0;
}
table.question-bank-table .header.checkbox .form-check {
  padding-left: 0;
}

#page-mod-quiz-edit div.questionbankwindow div.header {
  margin: 0;
}
#page-mod-quiz-edit div.questionbankwindow.block {
  padding: 0;
}

.questionbank .singleselect {
  margin: 0;
}

/* Question editing form */
#combinedfeedbackhdr div.fhtmleditor {
  padding: 0;
}

#combinedfeedbackhdr div.fcheckbox {
  margin-bottom: 1em;
}

#multitriesheader div.fitem_feditor {
  margin-top: 1em;
}

#multitriesheader div.fitem_fgroup {
  margin-bottom: 1em;
}

#multitriesheader div.fitem_fgroup fieldset.felement label {
  margin-left: 0.3em;
  margin-right: 0.3em;
}

body.path-question-type {
  /* Hacks to display the labels within a form group. */
}
body.path-question-type .fitem .col-form-label.visually-hidden:not(legend):not([for=id_category], [for=id_filetypeslist]) {
  /* stylelint-disable declaration-no-important */
  position: static !important;
  width: auto !important;
  overflow: visible !important;
  /* stylelint-enable declaration-no-important */
}

.que {
  text-align: left;
  margin: 0 auto 1.8em auto;
  display: flex;
  align-items: start;
  gap: 1rem;
}
@media (max-width: 767.98px) {
  .que {
    flex-direction: column;
  }
}

.que .info {
  flex-shrink: 0;
  width: 7em;
  padding: 0.5em;
  background-color: #f8f9fa;
  border: 1px solid #cad0d7;
  border-radius: 2px;
}
@media (max-width: 767.98px) {
  .que .info {
    width: 100%;
  }
}

.que h3.no, .que .no.h3 {
  margin: 0;
  font-size: 0.8em;
  line-height: 1;
}

.que span.qno {
  font-size: 1.5em;
  font-weight: bold;
  word-break: break-word;
}

.que .info > div {
  font-size: 0.8em;
  margin-top: 0.7em;
}

.que .info .questionflag.editable {
  cursor: pointer;
}

.que .info .editquestion img,
.que .info .questionflag img,
.que .info .questionflag input {
  vertical-align: bottom;
}

.que .content {
  flex-grow: 1;
}
@media (max-width: 767.98px) {
  .que .content {
    width: 100%;
  }
}

.que .formulation,
.que .outcome,
.que .comment {
  position: relative;
  padding: 1rem 1rem;
  margin-bottom: 1rem;
  border: var(--bs-border-width) solid transparent;
  border-radius: var(--bs-border-radius);
}

.que .outcome,
.que .comment {
  color: #a67736;
  border-color: #fbe6ca;
  background-color: #fcefdc;
}
.que .outcome a,
.que .comment a {
  color: #041c31;
}

.que .formulation {
  color: #001a1e;
  border-color: #b3d9e0;
  background-color: #cce6ea;
}

.que.multichoice .answer div.r0 .icon.fa-check,
.que.multichoice .answer div.r1 .icon.fa-check,
.que.multichoice .answer div.r0 .icon.fa-remove,
.que.multichoice .answer div.r1 .icon.fa-remove {
  text-indent: 0;
}

.formulation input[type=text],
.formulation select {
  width: auto;
  vertical-align: baseline;
}

.que.multianswer .formulation .yui3-widget-positioned {
  box-sizing: content-box;
}
.que.multianswer .formulation .yui3-widget-positioned .feedbackspan {
  width: inherit;
  max-width: inherit;
}

.que input[size],
.que textarea[cols] {
  width: auto;
  max-width: 100%;
}

.que .comment {
  color: #20491e;
  border-color: #c2d7c2;
  background-color: #d7e4d6;
}

.que .ablock {
  margin: 0.7em 0 0.3em 0;
}

.que .im-controls {
  margin-top: 0.5em;
  text-align: left;
}

.que .specificfeedback,
.que .generalfeedback,
.que .numpartscorrect .que .rightanswer,
.que .im-feedback,
.que .feedback,
.que p {
  margin: 0 0 0.5em;
}

.que .correctness.correct {
  color: #fff;
  background-color: #357a32;
}
.que .correctness.partiallycorrect {
  color: #000;
  background-color: #f0ad4e;
}
.que .correctness.notanswered, .que .correctness.incorrect {
  color: #fff;
  background-color: #ca3120;
}

.que .qtext {
  margin-bottom: 1.5em;
}

.que .validationerror {
  color: #ca3120;
}

.que .grading,
.que .comment,
.que .commentlink,
.que .history {
  margin-top: 0.5em;
}

.que .history h3, .que .history .h3 {
  margin: 0 0 0.2em;
  font-size: 1em;
}

.que .history table {
  width: 100%;
  margin: 0;
}

.que .history .current {
  font-weight: bold;
}

.que .questioncorrectnessicon {
  vertical-align: text-bottom;
}

body.jsenabled .questionflag input[type=checkbox] {
  display: none;
}

.que .questionflagimage {
  padding-right: 3px;
  height: 16px;
  width: 16px;
}

.importerror {
  margin-top: 10px;
  border-bottom: 1px solid #555;
}

.mform .que.comment .fitemtitle {
  width: 20%;
}

#page-question-preview #techinfo {
  margin: 1em 0;
}

#page-question-preview .collapsibleregion .collapsibleregioncaption,
#page-question-preview .collapsibleregion .collapsibleregionextracontent {
  display: inline-block;
}

#page-mod-quiz-edit ul.slots .activityinstance > a {
  display: flex;
  max-width: 100%;
  align-items: center;
  text-indent: 0;
  padding-left: 0;
}
#page-mod-quiz-edit ul.slots .activityinstance img.activityicon {
  margin-left: 0;
  width: 16px;
  height: 16px;
  padding-right: 4px;
}

#page-mod-quiz-edit .activity img.activityicon {
  vertical-align: text-top;
}

#page-mod-quiz-edit .box.generalbox.questionbank {
  padding: 0.5em;
}

#page-mod-quiz-edit .questionbank .categorypagingbarcontainer,
#page-mod-quiz-edit .questionbank .categoryquestionscontainer,
#page-mod-quiz-edit .questionbank .choosecategory {
  padding: 0;
}

#page-mod-quiz-edit .questionbank .choosecategory select {
  width: 100%;
}

#page-mod-quiz-edit div.questionbank .categoryquestionscontainer {
  background: transparent;
}

#page-mod-quiz-edit .questionbankwindow div.header {
  color: #444;
  text-shadow: none;
  border-top-left-radius: 4px;
  border-top-right-radius: 4px;
  margin: 0 -10px 0 -10px;
  padding: 2px 10px 2px 10px;
  background: transparent; /* Old browsers */
}

#page-mod-quiz-edit .questionbankwindow div.header a:link,
#page-mod-quiz-edit .questionbankwindow div.header a:visited {
  color: #0f6cbf;
}

#page-mod-quiz-edit .questionbankwindow div.header a:hover {
  color: #0c5699;
}

#page-mod-quiz-edit .createnewquestion {
  padding: 0.3em 0;
}
#page-mod-quiz-edit .createnewquestion div,
#page-mod-quiz-edit .createnewquestion input {
  margin: 0;
}

#page-mod-quiz-edit .questionbankwindow div.header .title {
  color: #1d2125;
}

#page-mod-quiz-edit div.container div.generalbox {
  background-color: transparent;
  padding: 1.5em;
}

#page-mod-quiz-edit .categoryinfo {
  background-color: transparent;
  border-bottom: none;
}

#page-mod-quiz-edit .createnewquestion .singlebutton input {
  margin-bottom: 0;
}

#page-mod-quiz-edit div.questionbank .categorysortopotionscontainer,
#page-mod-quiz-edit div.questionbank .categoryselectallcontainer {
  padding: 0 0 1.5em 0;
}

#page-mod-quiz-edit div.questionbank .categorypagingbarcontainer {
  background-color: transparent;
  margin: 0;
  border-top: 0;
  border-bottom: 0;
}
#page-mod-quiz-edit div.questionbank .categorypagingbarcontainer .paging {
  padding: 0 0.3em;
}

#page-mod-quiz-edit div.question div.content div.questioncontrols {
  background-color: #fff;
}

#page-mod-quiz-edit div.question div.content div.points {
  margin-top: -0.5em;
  padding-bottom: 0;
  border: none;
  background-color: #fff;
  position: static;
  width: 12.1em;
  float: right;
  margin-right: 60px;
}

#page-mod-quiz-edit div.question div.content div.points br {
  display: none;
}

#page-mod-quiz-edit div.question div.content div.points label {
  display: inline-block;
}

#page-mod-quiz-edit div.quizpage .pagecontent .pagestatus {
  background-color: #fff;
}

#page-mod-quiz-edit .quizpagedelete,
#page-mod-quiz-edit .quizpagedelete img {
  background-color: transparent;
}

#page-mod-quiz-edit div.quizpage .pagecontent {
  border: 1px solid #ddd;
  border-radius: 2px;
  overflow: hidden;
}

#page-mod-quiz-edit div.questionbank .categoryinfo {
  padding: 0.3em 0;
}

.questionbankwindow .module {
  width: auto;
}

.questionbankwindow .form-autocomplete-selection {
  margin-left: 0;
}

#page-mod-quiz-edit div.editq div.question div.content {
  background-color: #fff;
  border: 1px solid #ddd;
  border-radius: 2px;
  overflow: hidden;
}

#page-mod-quiz-edit ul.slots .activityinstance img.activityicon {
  margin-top: 0;
  padding-right: 4px;
}

.path-mod-quiz .statedetails {
  display: block;
  font-size: 0.9em;
}

a#hidebankcmd {
  color: #0f6cbf;
}

.que.shortanswer .answer {
  padding: 0;
}

.que label {
  display: inline;
}

.que .content .answer div[data-region=answer-label] .mediaplugin {
  width: 400px;
}

body.path-question-type .mform fieldset.hidden {
  padding: 0;
  margin: 0.7em 0 0;
}

.que.ddwtos,
.que.ddwtos .drop {
  box-sizing: content-box;
}

.tag-condition-container {
  position: relative;
}

.qbank_managecategories-item.qbank_managecategories-droptarget-before,
.qbank_managecategories-item .qbank_managecategories-item.qbank_managecategories-droptarget-before,
.qbank_managecategories-categorylist.qbank_managecategories-droptarget-before > .qbank_managecategories-item:first-child {
  border-top-color: #0f6cbf;
}

.qbank_managecategories-item.qbank_managecategories-droptarget,
.qbank_managecategories-categorylist.qbank_managecategories-droptarget > .qbank_managecategories-item:last-child {
  border-bottom-color: #0f6cbf;
}

.qbank_managecategories-newchild.qbank_managecategories-droptarget {
  border-color: #0f6cbf;
  color: #0f6cbf;
}

/* user.less */
.userprofile .fullprofilelink {
  text-align: center;
  margin: 10px;
}

.userprofile .page-context-header {
  margin-bottom: 10px;
  column-count: 1;
}

.userprofile .description {
  margin-top: 10px;
  margin-bottom: 30px;
}

.userprofile .profile_tree {
  column-count: 2;
}

#participantsform .no-overflow {
  overflow: visible;
}

.userprofile dl.list > dd + dt {
  clear: left;
}

.user-box {
  margin: 8px;
  width: 115px;
  height: 160px;
  text-align: center;
  float: left;
  clear: none;
}

#page-user-profile .node_category ul,
.path-user .node_category ul {
  margin: 0;
  list-style: none;
  padding-left: 0;
}
#page-user-profile .node_category li,
.path-user .node_category li {
  margin-top: 5px;
}
#page-user-profile .node_category .editprofile,
#page-user-profile .node_category .viewmore,
.path-user .node_category .editprofile,
.path-user .node_category .viewmore {
  text-align: right;
}

.ajax-contact-button {
  box-sizing: border-box;
  position: relative;
}
.ajax-contact-button.loading .loading-icon {
  display: block;
}
.ajax-contact-button .loading-icon {
  display: none;
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
  background-color: rgba(255, 255, 255, 0.7);
}
.ajax-contact-button .loading-icon .icon {
  position: absolute;
  left: 50%;
  top: 50%;
  transform: translate(-50%, -50%);
}

@media (max-width: 480px) {
  .userprofile .profile_tree {
    /** Display the profile on one column on phones@mixin  */
    column-count: 1;
  }
}
.userlist #showall {
  margin: 10px 0;
}

.userlist .buttons {
  text-align: center;
}

.userlist .buttons label {
  padding: 0 3px;
}

.userlist table#participants {
  text-align: center;
}

.userlist table#participants td {
  text-align: left;
  padding: 4px;
  vertical-align: middle;
}

.userlist table#participants th {
  text-align: left;
  padding: 4px;
}

.userlist table.controls {
  width: 100%;
}
.userlist table.controls tr {
  vertical-align: top;
}
.userlist table.controls .right {
  text-align: right;
}
.userlist table.controls .groupselector {
  margin-bottom: 0;
  margin-top: 0;
}
.userlist table.controls .groupselector label {
  display: block;
}

.userinfobox {
  width: 100%;
  border: 1px solid;
  border-collapse: separate;
  padding: 10px;
}

.userinfobox .left,
.userinfobox .side {
  width: 100px;
  vertical-align: top;
}

.userinfobox .userpicture {
  width: 100px;
  height: 100px;
}

.userinfobox .content {
  vertical-align: top;
}

.userinfobox .links {
  width: 100px;
  padding: 5px;
  vertical-align: bottom;
}

.userinfobox .links a {
  display: block;
}

.userinfobox .list td {
  padding: 3px;
}

.userinfobox .username {
  padding-bottom: 20px;
  font-weight: bold;
}

.userinfobox td.label {
  text-align: right;
  white-space: nowrap;
  vertical-align: top;
  font-weight: bold;
}

.group-edit {
  position: absolute;
  right: 0;
  margin-right: 0.6em;
}

.group-image {
  display: block;
  float: left;
  margin-right: 1em;
}
.group-image .grouppicture {
  border-radius: 50%;
}

.groupinfobox .left {
  padding: 10px;
  width: 100px;
  vertical-align: top;
}

.course-participation #showall {
  text-align: center;
  margin: 10px 0;
}

#user-policy .noticebox {
  text-align: center;
  margin-left: auto;
  margin-right: auto;
  margin-bottom: 10px;
  width: 80%;
  height: 250px;
}

#user-policy #policyframe {
  width: 100%;
  height: 100%;
}

.iplookup #map {
  margin: auto;
}

.userselector select {
  width: 100%;
}

.userselector div {
  margin-top: 0.2em;
}

.userselector div label {
  margin-right: 0.3em;
}

/* Next style does not work in all browsers but looks nicer when it does */
.userselector .userselector-infobelow {
  font-size: 0.8em;
}

#userselector_options .collapsibleregioncaption {
  font-weight: bold;
}
#userselector_options .collapsibleregioncaption img {
  width: 16px;
  height: 16px;
}

#userselector_options p {
  margin: 0.2em 0;
  text-align: left;
}

/** user full profile */
#page-user-profile .messagebox {
  text-align: center;
  margin-left: auto;
  margin-right: auto;
}

/** user course profile */
#page-course-view-weeks .messagebox {
  text-align: center;
  margin-left: auto;
  margin-right: auto;
}

.profileeditor > .singleselect {
  margin: 0 0.5em 0 0;
}
.profileeditor > .singlebutton {
  display: inline-block;
  margin: 0 0 0 0.5em;
}
.profileeditor > .singlebutton div,
.profileeditor > .singlebutton input {
  margin: 0;
}

.userlist h3 .action-icon, .userlist .h3 .action-icon {
  display: none;
}

#page-enrol-users .popover {
  max-width: none;
}

.user-enroller-panel {
  width: 600px;
}

[data-filterverbfor],
[data-filterregion=filter]:last-child [data-filterregion=joinadverb] {
  display: none;
}

[data-filterverb="0"] [data-filterverbfor="0"],
[data-filterverb="1"] [data-filterverbfor="1"],
[data-filterverb="2"] [data-filterverbfor="2"] {
  display: block;
}

#page-user-contactsitesupport .supporticon i {
  font-size: 35px;
}

.search-results .result {
  margin-left: 0;
  margin-right: 0;
}

.search-results .result .result-content {
  margin: 7px 0;
}

.search-results .result .filename {
  font-style: italic;
}

.simplesearchform .input-group input.form-control {
  border-top-left-radius: 0.5rem;
  border-bottom-left-radius: 0.5rem;
}
.simplesearchform .btn {
  padding-left: 0.5rem;
  padding-right: 0.5rem;
}
.simplesearchform .btn .icon {
  margin: 0;
}
.simplesearchform .btn-submit {
  border-color: var(--bs-gray-500);
  color: #6a737b;
}
.simplesearchform .btn-close,
.simplesearchform .btn-clear {
  position: absolute;
  top: 0;
  right: 0;
  color: #6a737b;
  z-index: 6;
}
.simplesearchform .btn-close {
  right: 2.5rem;
  top: 0.4rem;
  font-size: inherit;
  line-height: inherit;
}
.simplesearchform .btn-submit {
  background-color: #f8f9fa;
}
.simplesearchform .withclear {
  padding-right: 2rem;
}
.simplesearchform .searchinput {
  display: flex;
  flex: 1 1 auto;
}
.simplesearchform .collapsing {
  height: inherit;
  transition: none;
  width: inherit;
}

.simplesearchform .collapse.show,
.simplesearchform .collapsing {
  position: absolute;
  left: 0;
  top: 0;
  width: 100%;
  display: flex;
  background-color: #fff;
  z-index: 1070;
  height: 60px;
}
.simplesearchform .collapse.show .searchform-navbar,
.simplesearchform .collapsing .searchform-navbar {
  width: auto;
  margin-left: auto;
  margin-right: auto;
}

.search-areas-actions {
  margin-bottom: 1rem;
}

.search-areas-actions > div {
  margin-right: 1rem;
  display: inline-block;
}

#core-search-areas .lastcol li {
  margin-left: 24px;
  text-indent: -24px;
}

#core-search-areas .lastcol li > i {
  text-indent: 0;
}

/**
 * Moodle forms HTML isn't changeable via renderers (yet?) so this
 * .less file imports styles from the bootstrap $variables file and
 * adds them to the existing Moodle form CSS ids and classes.
 *
 */
.jsenabled .mform .containsadvancedelements .advanced {
  display: none;
}

.mform .containsadvancedelements .advanced.show {
  display: flex;
}

#adminsettings span.error {
  display: inline-block;
  border: 1px solid #efc1bc;
  border-radius: 4px;
  background-color: #f4d6d2;
  padding: 4px;
  margin-bottom: 4px;
}

.mform .d-flex .fitem {
  margin: 0.1rem 0.25rem 0.1rem 0 !important; /* stylelint-disable-line declaration-no-important */
}
.mform .d-flex br + label {
  justify-content: flex-start;
  width: 100%;
  margin-right: 0;
}

.d-flex > .form-control {
  width: auto;
  max-width: 100%;
}
.d-flex > textarea.form-control {
  width: 100%;
}

.form-select {
  width: auto;
  max-width: 100%;
}
.form-select.dropdown-toggle::after {
  content: none;
}

#jump-to-activity.form-select {
  width: 100%;
}

.mform fieldset {
  margin-bottom: 0.5rem;
  border-bottom: 1px solid var(--bs-border-color);
}

#adminsettings .form-control[size] {
  width: auto;
}

#adminsettings .error {
  color: #ca3120;
}

.mform ul.file-list {
  padding: 0;
  margin: 0;
  list-style: none;
}

.mform label .req,
.mform label .adv {
  cursor: help;
}

/*rtl:ignore*/
input#id_externalurl {
  direction: ltr;
}

#portfolio-add-button {
  display: inline;
}

.form-defaultinfo,
.form-label .form-shortname {
  color: var(--bs-secondary-color);
}

.form-label .form-shortname {
  font-size: 0.703125rem;
  display: block;
}

.formsettingheading .form-horizontal {
  color: var(--bs-secondary-color);
}

.no-felement.fstatic {
  color: var(--bs-secondary-color);
  padding-top: 5px;
}

.no-fitem .fstaticlabel {
  font-weight: bold;
}

.form-item .form-setting .defaultsnext > input {
  display: inline-block;
}

.form-item .form-setting .form-checkbox.defaultsnext {
  margin-top: 5px;
  display: inline-block;
}

#adminsettings h3, #adminsettings .h3 {
  display: block;
  width: 100%;
  padding: 0;
  margin-bottom: 1.5;
  font-size: 1.171875rem;
  line-height: 3;
  border: 0;
  border-bottom: 1px solid #e5e5e5;
}

/* rtl:ignore */
.mform .fitem .felement input[name=email],
.mform .fitem .felement input[name=email2],
.mform .fitem .felement input[name=url],
.mform .fitem .felement input[name=idnumber],
.mform .fitem .felement input[name=phone1],
.mform .fitem .felement input[name=phone2] {
  text-align: left;
  direction: ltr;
}

.que.match .mediaplugin {
  width: 50vw;
}

/* rtl:ignore */
#page-admin-grade-edit-scale-edit .error input#id_name {
  margin-right: 170px;
}

#page-grade-edit-outcome-course .courseoutcomes {
  margin-left: auto;
  margin-right: auto;
  width: 100%;
}

#page-grade-edit-outcome-course .courseoutcomes td {
  text-align: center;
}

/* Install Process' text fields Forms, should always be justified to the left */
/* rtl:ignore */
#installform #id_wwwroot,
#installform #id_dirroot,
#installform #id_dataroot,
#installform #id_dbhost,
#installform #id_dbname,
#installform #id_dbuser,
#installform #id_dbpass,
#installform #id_prefix {
  direction: ltr;
}

.mdl-right > label {
  display: inline-block;
}

.singleselect {
  max-width: 100%;
}

.form-item .form-label label {
  margin-bottom: 0;
}

div#dateselector-calendar-panel {
  z-index: 3100; /* Set higher than the z-index of the filemanager - see MDL-39047. */
}

/**
 * Show the labels above text editors and file managers except on wide screens.
 */
/* Section and module editing forms contain special JS components for the
   availability system (if enabled). */
#id_availabilityconditionsjson[aria-hidden=true],
.availability-field [aria-hidden=true] {
  display: none;
}

.availability-field img {
  width: 16px;
  height: 16px;
}

.availability-eye {
  clear: left;
  float: left;
}

.availability-inner,
.availability-plugincontrols {
  float: left;
  border-radius: var(--bs-border-radius);
  border: var(--bs-border-width) solid var(--bs-border-color-translucent);
  padding: 1rem;
  margin-top: 0.5rem;
}

.availability-plugincontrols,
.availability-childlist .availability-inner {
  margin-left: 0.625rem;
}

.availability-field .availability-plugincontrols .availability-group select {
  max-width: 12rem;
}

/* Custom styles for autocomplete form element */
/* These styles reserve a standard amount of space in the DOM to avoid flicker when the original select element is replaced */
[data-fieldtype=autocomplete] select,
[data-fieldtype=tags] select,
.form-autocomplete-original-select {
  visibility: hidden;
  overflow: hidden;
  width: 15rem;
  height: 44px;
  margin: 0;
  padding: 0;
  border: 0;
  margin-top: 1.65625rem;
  vertical-align: bottom;
}

.form-autocomplete-selection {
  margin: 0.25rem 0;
  min-height: 2.375rem;
}

.form-autocomplete-selection [role=option] {
  cursor: pointer;
  white-space: inherit;
  word-break: break-word;
  line-height: 1.4;
  text-align: left;
}

.form-autocomplete-suggestions {
  position: absolute;
  background-color: #fff;
  border: 1px solid var(--bs-gray-500);
  min-width: 206px;
  max-height: 20em;
  overflow: auto;
  margin: 0.125rem 0 0;
  padding: 0.5rem 0;
  z-index: 3;
}

.form-autocomplete-suggestions li {
  list-style-type: none;
  padding: 0.25rem 1rem;
  margin: 0;
  cursor: pointer;
  color: #1d2125;
  word-break: normal;
  overflow-wrap: anywhere;
  white-space: normal;
}
.form-autocomplete-suggestions li:hover, .form-autocomplete-suggestions li:focus, .form-autocomplete-suggestions li[aria-selected=true] {
  background-color: #0f6cbf;
  color: #fff;
}
.form-autocomplete-suggestions li[aria-disabled=true] {
  pointer-events: none;
  color: #6a737b;
  background-color: var(--bs-secondary-bg);
}
.form-autocomplete-suggestions li.suggestions-heading {
  pointer-events: none;
  font-weight: bold;
  color: #1d2125;
  background-color: #fff;
  padding-left: calc(1rem / 2);
}
.form-autocomplete-suggestions li::before {
  content: "​";
}

.form-autocomplete-downarrow {
  color: #1d2125;
  top: 0.2rem;
  right: 0.5rem;
  cursor: pointer;
}
.form-autocomplete-downarrow .loading-icon {
  position: absolute;
  top: 0;
  left: 0;
  background-color: #fff;
}

/** Undo some bootstrap things */
.form-autocomplete-selection + input.form-control {
  width: auto;
  display: inline-block;
  vertical-align: middle;
}

.form-autocomplete-selection [data-active-selection=true] {
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}

select.form-control[size], select.form-control[multiple] {
  padding-right: 0;
}
select.form-control[size] option, select.form-control[multiple] option {
  width: fit-content;
}

/* Non-bootstrap selects with a size show their contents outside of the element.
 * Remove when we update to stable bootstrap 4. (MDL-56511) */
select[size],
select[multiple] {
  overflow: auto;
}

select[size="1"] {
  overflow: visible;
}

textarea[data-auto-rows] {
  overflow-x: hidden;
  resize: none;
}

/** Display elements under labels in vertical forms regardless of the screen size. */
.mform.full-width-labels .fitem.row {
  margin-left: 0;
  margin-right: 0;
}
.mform.full-width-labels .fitem.row > .col-md-3, .mform.full-width-labels .fitem.row > .col-md-9 {
  flex: 0 0 100%;
  max-width: 100%;
  width: inherit;
  padding-right: 0;
  padding-left: 0;
}
.mform.full-width-labels .fitem.row.femptylabel > .col-md-3 {
  display: none;
}
.mform.full-width-labels .fitem.row .form-control {
  width: 100%;
}

.mform .col-form-label .form-label-addon {
  margin-left: 0.25rem;
}

@media (min-width: 576px) {
  .mform:not(.full-width-labels) .col-form-label .form-label-addon {
    margin-left: auto;
  }
}
[data-fieldtype=modgrade] .fitem {
  padding-bottom: 0.375rem;
}

[data-fieldtype=modgrade] .form-select {
  width: 100%;
}

[data-fieldtype=modgrade] {
  background-color: var(--bs-body-bg);
  border-radius: var(--bs-border-radius);
  border: var(--bs-border-width) solid var(--bs-border-color-translucent);
  padding: 1rem;
  margin-left: 0.75rem;
  max-width: 30rem;
}

[data-filetypesbrowserbody] [aria-expanded=false] > [role=group],
[data-filetypesbrowserbody] [aria-expanded=false] [data-filetypesbrowserfeature=hideifcollapsed],
[data-filetypesbrowserbody] [aria-expanded=true] [data-filetypesbrowserfeature=hideifexpanded] {
  display: none;
}

.felement[data-fieldtype=autocomplete],
.felement[data-fieldtype=tags] {
  display: block !important; /* stylelint-disable-line declaration-no-important */
  position: static;
}

[data-fieldtype=editor] > div {
  flex-grow: 1;
}

@media (min-width: 768px) {
  .mform fieldset .fcontainer.collapseable .col-form-label {
    padding-left: 2.5rem;
  }
}
.collapsemenu .collapseall {
  display: block;
}
.collapsemenu .expandall {
  display: none;
}
.collapsemenu.collapsed .collapseall {
  display: none;
}
.collapsemenu.collapsed .expandall {
  display: block;
}

.input-group.form-inset .form-inset-item {
  position: absolute;
  padding-top: calc(0.375rem + var(--bs-border-width));
  z-index: 3;
}
.input-group.form-inset.form-inset-left .form-control {
  padding-left: 1.5rem;
}
.input-group.form-inset.form-inset-right .form-control {
  padding-right: 1.5rem;
}
.input-group.form-inset.form-inset-right .form-inset-item {
  right: 0;
}

.form-check.left-indented {
  padding-left: 0;
}

.toggle-sensitive-btn .icon {
  margin-right: 0;
}

@media (min-width: 576px) {
  .toggle-sensitive-wrapper.small-screens-only.input-group:not(.has-validation) > .form-control:not(:last-child) {
    border-radius: var(--bs-border-radius);
  }
  .toggle-sensitive-wrapper.small-screens-only.input-group:not(.has-validation) > .form-control-lg:not(:last-child) {
    border-radius: var(--bs-border-radius-lg);
  }
  .toggle-sensitive-wrapper.small-screens-only.input-group:not(.has-validation) > .toggle-sensitive-btn {
    display: none;
  }
}
.form-control[readonly] {
  background-color: var(--bs-secondary-bg);
  opacity: 1;
}

.pagelayout-login #region-main {
  border: 0;
  background-color: inherit;
}

.pagelayout-login #page {
  background: #f8f9fa;
  background-image: linear-gradient(to right, #f8f9fa 0%, #dee2e6 100%);
}
.pagelayout-login #page div[role=main] {
  height: 100%;
}

.login-wrapper {
  display: flex;
  align-items: center;
  justify-content: center;
  height: 100%;
}

.login-container {
  background-color: #fff;
  padding: 3rem;
  box-shadow: 0 0.5rem 1rem rgba(0, 0, 0, 0.15);
  margin-bottom: 2rem;
  border-radius: 0.5rem;
}
.login-container .login-languagemenu {
  display: flex;
  justify-content: flex-start;
}
.login-container .login-languagemenu .dropdown-menu {
  max-height: 300px;
  overflow-y: auto;
}
.login-container .login-logo {
  display: flex;
  justify-content: center;
  margin-bottom: 1rem;
}
.login-container .login-divider {
  margin-top: 1.5rem;
  margin-bottom: 1.5rem;
  border-top: 1px solid #dee2e6;
}
.login-container h1.login-heading, .login-container .login-heading.h1 {
  font-size: 1.875rem;
}
.login-container h2.login-heading, .login-container .login-heading.h2 {
  font-size: 1.40625rem;
}
.login-container .login-identityproviders .login-identityprovider-btn {
  border: 1px solid #dee2e6;
}
.login-container .divider {
  width: 1px;
  background-color: #dee2e6;
  height: 1.875rem;
}
.login-container .action-menu-trigger a {
  margin: 0.5rem 0;
}

@media (min-width: 768px) {
  .login-container {
    width: 500px !important; /* stylelint-disable-line declaration-no-important */
  }
}
/* modules.less */
select {
  width: auto;
}

.path-mod .activity-header:not(:empty) {
  background-color: #f8f9fa;
  margin-bottom: 1rem;
  padding-left: 1rem;
  padding-right: 1rem;
  border-radius: 0.5rem;
}
.path-mod .activity-header:not(:empty) > div:last-child > div:last-child {
  border-bottom: 0;
}
.path-mod .activity-information .activity-dates {
  padding-top: 1rem;
  padding-bottom: 1rem;
  border-bottom: 1px solid #dee2e6;
}
.path-mod .activity-information .completion-info {
  padding-top: 1rem;
  padding-bottom: 1rem;
  border-bottom: 1px solid #dee2e6;
}
.path-mod .activity-description {
  padding-top: 1rem;
  padding-bottom: 1rem;
}
.path-mod .activity-description > .box.py-3 {
  padding-top: 0 !important; /* stylelint-disable-line declaration-no-important */
  padding-bottom: 0 !important; /* stylelint-disable-line declaration-no-important */
}
.path-mod .activity-description > .box.py-3:empty {
  display: none;
}
.path-mod .automatic-completion-conditions .badge {
  font-size: 80%;
  padding: 0.5rem;
  margin-top: 0.25rem;
  mix-blend-mode: multiply;
}
.path-mod .automatic-completion-conditions .badge.badge-light {
  background-color: #e9ecef !important; /* stylelint-disable-line declaration-no-important */
}
.path-mod .automatic-completion-conditions .badge .icon {
  width: 0.7rem;
  height: 0.7rem;
  font-size: 0.7rem;
}
.path-mod .automatic-completion-conditions .badge:first-child {
  margin-top: 0;
}
.path-mod .activity-description .no-overflow p:last-child {
  padding-bottom: 0;
  margin-bottom: 0;
}

.path-mod-choice .horizontal .choices .option {
  display: inline-block;
}

.path-mod-choice .choices .option label {
  vertical-align: top;
}

.path-mod-forum .forumsearch input,
.path-mod-forum .forumsearch .helptooltip {
  margin: 0 3px;
}

.path-mod-forum .forumheaderlist,
.path-mod-forum .forumheaderlist td {
  border: none;
}

.path-mod-forum .forumheaderlist thead .header,
.path-mod-forum .forumheaderlist tbody .discussion td {
  white-space: normal;
  vertical-align: top;
  padding-left: 0.5em;
  padding-right: 0.5em;
}
.path-mod-forum .forumheaderlist thead .header {
  white-space: normal;
  vertical-align: top;
}
.path-mod-forum .forumheaderlist thead .header.replies {
  text-align: center;
}
.path-mod-forum .forumheaderlist thead .header.lastpost {
  text-align: right;
}
.path-mod-forum .forumheaderlist thead .header th.discussionsubscription,
.path-mod-forum .forumheaderlist tbody .discussion td.discussionsubscription {
  width: 16px;
  padding-left: 0.5em;
  padding-right: 0.5em;
}
.path-mod-forum .forumheaderlist .discussion .replies,
.path-mod-forum .forumheaderlist .discussion .lastpost {
  white-space: normal;
}
.path-mod-forum .forumheaderlist .discussion .discussionsubscription,
.path-mod-forum .forumheaderlist .discussion .replies {
  text-align: center;
}
.path-mod-forum .forumheaderlist .discussion .topic,
.path-mod-forum .forumheaderlist .discussion .discussionsubscription,
.path-mod-forum .forumheaderlist .discussion .topic.starter,
.path-mod-forum .forumheaderlist .discussion .replies,
.path-mod-forum .forumheaderlist .discussion .lastpost {
  vertical-align: top;
}
.path-mod-forum .discussion-list .topic {
  font-weight: inherit;
}

.discussion-settings-container .form-select {
  width: 100%;
}
.discussion-settings-container input {
  max-width: 100%;
}

.forumpost {
  border: 1px solid #dee2e6;
  display: block;
  padding: 6px;
}
.forumpost .header {
  margin-bottom: 3px;
}
.forumpost .picture img {
  margin: 3px;
}
.forumpost .picture img.userpicture {
  margin-left: 3px;
  margin-right: 10px;
}
.forumpost .content .posting.fullpost {
  margin-top: 8px;
}
.forumpost .row {
  display: block;
}
.forumpost .row .topic,
.forumpost .row .content-mask,
.forumpost .row .options {
  margin-left: 48px;
}
.forumpost .row.side {
  clear: both;
}

.forumpost .row .left {
  width: 48px;
}

.forumpost .options .commands {
  margin-left: 0;
}

.forumpost .subject {
  font-weight: bold;
}

.forumsearch input[type=text] {
  margin-bottom: 0;
}

#page-mod-forum-view table .fit-content {
  width: 1px;
  white-space: nowrap;
}
#page-mod-forum-view table .limit-width {
  max-width: 200px;
  white-space: nowrap;
  text-overflow: ellipsis;
  overflow: hidden;
}
#page-mod-forum-view table .limit-width .author-info {
  max-width: calc(100% - 35px - 0.5rem);
}

/** Gently highlight the selected post by changing it's background to blue and then fading it out. */
@keyframes background-highlight {
  from {
    background-color: rgba(0, 123, 255, 0.5);
  }
  to {
    background-color: inherit;
  }
}
.path-mod-forum .nested-v2-display-mode,
.path-mod-forum.nested-v2-display-mode {
  /** Reset the badge styling back to pill style. */
  /** Style the ratings like a badge. */
  /** Don't show the discussion locked alert in this mode because it's already indicated with a badge. */
  /** Fix muted text contrast ratios for accessibility. */
  /** Make the tag list text screen reader visible only */
}
.path-mod-forum .nested-v2-display-mode .discussionsubscription,
.path-mod-forum.nested-v2-display-mode .discussionsubscription {
  margin-top: 0;
  text-align: inherit;
  margin-bottom: 0;
}
.path-mod-forum .nested-v2-display-mode .preload-subscribe,
.path-mod-forum .nested-v2-display-mode .preload-unsubscribe,
.path-mod-forum.nested-v2-display-mode .preload-subscribe,
.path-mod-forum.nested-v2-display-mode .preload-unsubscribe {
  display: none;
}
.path-mod-forum .nested-v2-display-mode .post-message,
.path-mod-forum.nested-v2-display-mode .post-message {
  line-height: 1.6;
}
.path-mod-forum .nested-v2-display-mode .indent,
.path-mod-forum.nested-v2-display-mode .indent {
  margin-left: 0;
}
.path-mod-forum .nested-v2-display-mode .badge,
.path-mod-forum.nested-v2-display-mode .badge {
  font-size: inherit;
  font-weight: inherit;
  padding-left: 0.5rem;
  padding-right: 0.5rem;
  border-radius: 10rem;
}
.path-mod-forum .nested-v2-display-mode .badge-light,
.path-mod-forum.nested-v2-display-mode .badge-light {
  background-color: #f6f6f6;
  color: #5b5b5b;
}
.path-mod-forum .nested-v2-display-mode .rating-aggregate-container,
.path-mod-forum.nested-v2-display-mode .rating-aggregate-container {
  background-color: #f6f6f6;
  color: #5b5b5b;
  padding: 0.25em 0.5em;
  line-height: 1;
  margin-right: 0.5rem;
  vertical-align: middle;
  border-radius: 10rem;
  text-align: center;
}
.path-mod-forum .nested-v2-display-mode .ratinginput,
.path-mod-forum.nested-v2-display-mode .ratinginput {
  padding: 0.25em 1.75rem 0.25em 0.75em;
  line-height: 1;
  height: auto;
  border-radius: 10rem;
}
@media (max-width: 767.98px) {
  .path-mod-forum .nested-v2-display-mode .ratinginput,
  .path-mod-forum.nested-v2-display-mode .ratinginput {
    margin-top: 0.5rem;
  }
}
.path-mod-forum .nested-v2-display-mode .group-image,
.path-mod-forum.nested-v2-display-mode .group-image {
  width: 35px;
  height: 35px;
  margin-right: 0;
  float: none;
  display: inline-block;
}
.path-mod-forum .nested-v2-display-mode .alert.discussionlocked,
.path-mod-forum.nested-v2-display-mode .alert.discussionlocked {
  width: 1px !important;
  height: 1px !important;
  padding: 0 !important;
  margin: -1px !important;
  overflow: hidden !important;
  clip: rect(0, 0, 0, 0) !important;
  white-space: nowrap !important;
  border: 0 !important;
}
.path-mod-forum .nested-v2-display-mode .alert.discussionlocked:not(caption),
.path-mod-forum.nested-v2-display-mode .alert.discussionlocked:not(caption) {
  position: absolute !important;
}
.path-mod-forum .nested-v2-display-mode .text-muted,
.path-mod-forum .nested-v2-display-mode .dimmed_text,
.path-mod-forum.nested-v2-display-mode .text-muted,
.path-mod-forum.nested-v2-display-mode .dimmed_text {
  color: #707070 !important; /* stylelint-disable-line declaration-no-important */
}
.path-mod-forum .nested-v2-display-mode .author-header,
.path-mod-forum.nested-v2-display-mode .author-header {
  font-style: italic;
}
.path-mod-forum .nested-v2-display-mode .author-header .author-name,
.path-mod-forum.nested-v2-display-mode .author-header .author-name {
  font-style: normal;
}
.path-mod-forum .nested-v2-display-mode .tag_list > b,
.path-mod-forum.nested-v2-display-mode .tag_list > b {
  width: 1px !important;
  height: 1px !important;
  padding: 0 !important;
  margin: -1px !important;
  overflow: hidden !important;
  clip: rect(0, 0, 0, 0) !important;
  white-space: nowrap !important;
  border: 0 !important;
}
.path-mod-forum .nested-v2-display-mode .tag_list > b:not(caption),
.path-mod-forum.nested-v2-display-mode .tag_list > b:not(caption) {
  position: absolute !important;
}
.path-mod-forum .nested-v2-display-mode :target > .focus-target,
.path-mod-forum.nested-v2-display-mode :target > .focus-target {
  animation-name: background-highlight;
  animation-duration: 1s;
  animation-timing-function: ease-in-out;
  animation-iteration-count: 1;
}
.path-mod-forum .nested-v2-display-mode .forum-post-container .replies-container .forum-post-container,
.path-mod-forum.nested-v2-display-mode .forum-post-container .replies-container .forum-post-container {
  border-top: 1px solid #dee2e6;
  padding-top: 1.5rem;
}
.path-mod-forum .nested-v2-display-mode .forum-post-container .replies-container .forum-post-container .replies-container .forum-post-container,
.path-mod-forum.nested-v2-display-mode .forum-post-container .replies-container .forum-post-container .replies-container .forum-post-container {
  border-top: none;
  padding-top: 0;
}
.path-mod-forum .nested-v2-display-mode .forum-post-container .replies-container .inline-reply-container .reply-author,
.path-mod-forum.nested-v2-display-mode .forum-post-container .replies-container .inline-reply-container .reply-author {
  display: none;
}
.path-mod-forum .nested-v2-display-mode .forum-post-container .post-message p:last-of-type,
.path-mod-forum.nested-v2-display-mode .forum-post-container .post-message p:last-of-type {
  margin-bottom: 0;
}
.path-mod-forum .nested-v2-display-mode .forum-post-container .author-image-container,
.path-mod-forum.nested-v2-display-mode .forum-post-container .author-image-container {
  width: 70px;
  margin-right: 24px;
  flex-shrink: 0;
}
.path-mod-forum .nested-v2-display-mode .forum-post-container .inline-reply-container textarea,
.path-mod-forum.nested-v2-display-mode .forum-post-container .inline-reply-container textarea {
  border: 0;
  resize: none;
}
.path-mod-forum .nested-v2-display-mode .forum-post-container .indent,
.path-mod-forum.nested-v2-display-mode .forum-post-container .indent {
  /**
   * The first post and first set of replies have a larger author image so offset the 2nd
   * set of replies by the image width + margin to ensure they align.
   */
}
.path-mod-forum .nested-v2-display-mode .forum-post-container .indent .indent,
.path-mod-forum.nested-v2-display-mode .forum-post-container .indent .indent {
  padding-left: 94px;
  /**
   * Reduce the size of the the author image for all second level replies (and below).
   */
  /**
   * Adjust the indentation offset for all 3rd level replies and below for the smaller author image.
   */
}
.path-mod-forum .nested-v2-display-mode .forum-post-container .indent .indent .author-image-container,
.path-mod-forum.nested-v2-display-mode .forum-post-container .indent .indent .author-image-container {
  width: 30px;
  margin-right: 8px;
  padding-top: 3px;
}
.path-mod-forum .nested-v2-display-mode .forum-post-container .indent .indent .indent,
.path-mod-forum.nested-v2-display-mode .forum-post-container .indent .indent .indent {
  padding-left: 38px;
  /**
   * Stop indenting the replies after the 5th reply.
   */
}
.path-mod-forum .nested-v2-display-mode .forum-post-container .indent .indent .indent .indent .indent .indent,
.path-mod-forum.nested-v2-display-mode .forum-post-container .indent .indent .indent .indent .indent .indent {
  padding-left: 0;
}

/** Extra small devices (portrait phones, less than 576px). */
@media (max-width: 767.98px) {
  #page-mod-forum-discuss.nested-v2-display-mode .forum-post-container .author-image-container {
    width: 30px;
    margin-right: 8px;
  }
  #page-mod-forum-discuss.nested-v2-display-mode .forum-post-container .indent .indent {
    padding-left: 38px;
  }
  #page-mod-forum-discuss.nested-v2-display-mode .forum-post-container .indent .indent .indent .indent {
    padding-left: 0;
  }
  #page-mod-forum-discuss.nested-v2-display-mode .group-image {
    width: 30px;
    height: 30px;
  }
}
.filter-scrollable {
  overflow-y: auto;
  max-height: 25em;
  margin-bottom: 1em;
}

.filter-dates-popover {
  width: 100%;
  max-width: 41.5em;
}

/* stylelint-disable-line max-line-length */
@keyframes expandSearchButton {
  from {
    height: 36px;
    width: 36px;
    border-radius: 18px;
    background-color: #e9ecef;
  }
  to {
    width: 100%;
    height: calc(1.5em + 1rem + calc(var(--bs-border-width) * 2));
    border-radius: 0;
    background-color: var(--bs-body-bg);
    border-color: var(--bs-gray-500);
    padding-left: calc(0.5rem + 8px);
    padding-top: 0.5rem;
    padding-bottom: 0.5rem;
    font-size: 1.171875rem;
    line-height: 1.5;
    right: 0;
  }
}
@keyframes collapseSearchButton {
  from {
    width: 100%;
    height: calc(1.5em + 1rem + calc(var(--bs-border-width) * 2));
    border-radius: 0;
    background-color: var(--bs-body-bg);
    border-color: var(--bs-gray-500);
    padding-left: calc(0.5rem + 8px);
    padding-top: 0.5rem;
    padding-bottom: 0.5rem;
    font-size: 1.171875rem;
    line-height: 1.5;
    right: 0;
  }
  to {
    height: 36px;
    width: 36px;
    border-radius: 18px;
    background-color: #e9ecef;
  }
}
.path-mod-forum .unified-grader .navbar {
  max-height: none;
  z-index: 1;
}
.path-mod-forum .unified-grader .body-container {
  overflow: auto;
}
.path-mod-forum .unified-grader .body-container.hidden {
  display: none !important; /* stylelint-disable-line declaration-no-important */
}
.path-mod-forum .unified-grader .userpicture {
  height: 60px;
  width: 60px;
}
.path-mod-forum .unified-grader .grader-grading-panel {
  top: 0;
  position: absolute;
  height: 100%;
  z-index: 0;
  width: 430px;
}
.path-mod-forum .unified-grader .grader-grading-panel.hidden {
  right: -430px;
}
.path-mod-forum .unified-grader .grader-grading-panel .grading-icon {
  width: 36px;
}
.path-mod-forum .unified-grader .grader-grading-panel .user-picker-container .user-full-name {
  max-width: 240px;
}
.path-mod-forum .unified-grader .grader-grading-panel .user-picker-container .page-link {
  width: 36px;
  height: 36px;
  display: flex;
  text-align: center;
  align-items: center;
  justify-content: center;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container {
  height: 65px;
  position: relative;
  overflow: hidden;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .info-container {
  position: absolute;
  top: 50%;
  left: 0;
  transform: translateY(-50%);
  width: 100%;
  height: 100%;
  padding: 0.5rem;
  padding-right: calc(36px + 0.5rem);
  opacity: 1;
  visibility: visible;
  transition: left 0.3s ease-in-out;
  z-index: 1;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .toggle-search-button.expand {
  animation-name: expandSearchButton;
  animation-duration: 0.3s;
  animation-timing-function: ease-in-out;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .toggle-search-button.collapse {
  display: block;
  animation-name: collapseSearchButton;
  animation-duration: 0.3s;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container {
  overflow: hidden;
  position: absolute;
  top: 50%;
  right: 0;
  transform: translateY(-50%);
  z-index: 2;
  width: 100%;
  height: 100% !important; /* stylelint-disable-line declaration-no-important */
  padding: 0.5rem;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container .search-input-container {
  position: relative;
  overflow: visible;
  flex-wrap: nowrap;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container .search-input-container input {
  padding-left: calc(0.5rem + 0.5rem + 36px - calc(var(--bs-border-width) * 2));
  padding-right: calc(0.5rem + 36px);
  opacity: 1;
  visibility: visible;
  transition: opacity 0s linear 0.3s, visibility 0s linear;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container .search-input-container .search-icon {
  position: absolute;
  top: 50%;
  left: 0.5rem;
  transform: translateY(-50%);
  color: var(--bs-body-color);
  height: 36px;
  width: calc(36px - calc(var(--bs-border-width) * 2));
  background-color: var(--bs-body-bg);
  opacity: 1;
  visibility: visible;
  transition: opacity 0s linear 0.3s, visibility 0s linear 0.3s;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container .search-input-container .toggle-search-button {
  position: absolute;
  top: 50%;
  right: 0.5rem;
  transform: translateY(-50%);
  z-index: 1;
  color: inherit;
  text-align: left;
  padding-left: 9px;
  transition: right 0s linear 0.3s;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container .search-input-container .toggle-search-button .expanded-icon {
  opacity: 1;
  visibility: visible;
  max-width: 50px;
  max-height: 50px;
  transition: opacity 0s linear 0.3s, max-height 0s linear 0.3s, max-width 0s linear 0.3s, visibility 0s linear 0.3s;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container .search-input-container .toggle-search-button .collapsed-icon {
  opacity: 0;
  visibility: hidden;
  max-height: 0;
  max-width: 0;
  overflow: hidden;
  transition: opacity 0s linear 0.3s, max-height 0s linear 0.3s, max-width 0s linear 0.3s, visibility 0s linear 0.3s;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container.collapsed {
  width: calc(36px + 0.5rem + 0.5rem);
  transition: width 0.3s ease-in-out;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container.collapsed .search-input-container {
  flex-wrap: nowrap;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container.collapsed .search-input-container input,
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container.collapsed .search-input-container .search-icon {
  opacity: 0;
  visibility: hidden;
  transition: opacity 0s linear, visibility 0s linear;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container.collapsed .search-input-container input {
  padding-left: 0;
  padding-right: 0;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container.collapsed .search-input-container .toggle-search-button .expanded-icon {
  opacity: 0;
  visibility: hidden;
  max-height: 0;
  max-width: 0;
  overflow: hidden;
  transition: opacity 0s linear, max-height 0s linear, max-width 0s linear, visibility 0s linear;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container.collapsed .search-input-container .toggle-search-button .collapsed-icon {
  opacity: 1;
  visibility: visible;
  max-width: 50px;
  max-height: 50px;
  transition: opacity 0s linear, max-height 0s linear, max-width 0s linear, visibility 0s linear;
}
.path-mod-forum .unified-grader .grader-grading-panel .header-container .user-search-container:not(.collapsed) + .info-container {
  opacity: 0;
  visibility: hidden;
  left: -100%;
  transition: left 0.3s ease-in-out, opacity 0s linear 0.3s, visibility 0s linear 0.3s, padding 0s linear 0.3s;
}
.path-mod-forum .unified-grader .grader-module-content {
  overflow-y: auto;
  margin-right: 430px;
  transition: margin-right 0.2s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .path-mod-forum .unified-grader .grader-module-content {
    transition: none;
  }
}
.path-mod-forum .unified-grader .drawer-button {
  position: relative;
}
.path-mod-forum .unified-grader .drawer-button.active::after {
  content: "";
  position: absolute;
  bottom: calc(-0.5rem - 1px);
  left: 0;
  width: 100%;
  height: 3px;
  background-color: #0f6cbf;
}
.path-mod-forum .unified-grader .drawer-button .icon {
  font-size: 20px;
  height: 20px;
  width: 20px;
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container:last-of-type > hr {
  display: none;
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container:last-of-type > hr {
  display: none;
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container {
  position: relative;
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container .show-content-button {
  position: absolute;
  height: 100%;
  width: 100%;
  left: 0;
  top: 0;
  padding-left: calc(1rem + 45px);
  text-align: left;
  z-index: 1;
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container .show-content-button:not(.collapsed) {
  display: none;
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container .content {
  display: block;
  height: auto !important; /* stylelint-disable-line declaration-no-important */
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container .content .header {
  transition: margin-bottom 0.3s ease-in-out;
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container .content .header div + div {
  opacity: 1;
  visibility: visible;
  max-height: none;
  transition: opacity 0.3s linear, visibility 0s linear;
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container .content .body-content-container {
  opacity: 1;
  visibility: visible;
  max-height: none;
  transition: opacity 0.3s linear, visibility 0s linear;
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container .content .forum-post-core {
  opacity: 1;
  visibility: visible;
  max-height: none;
  transition: opacity 0.3s linear, visibility 0s linear;
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container .show-content-button.collapsed + .content {
  opacity: 0.3;
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container .show-content-button.collapsed + .content .header {
  margin-bottom: 0 !important; /* stylelint-disable-line declaration-no-important */
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container .show-content-button.collapsed + .content .header div + div {
  opacity: 0;
  visibility: hidden;
  max-height: 0;
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container .show-content-button.collapsed + .content .body-content-container {
  opacity: 0;
  visibility: hidden;
  max-height: 0;
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container .show-content-button.collapsed + .content .forum-post-core {
  opacity: 0;
  visibility: hidden;
  max-height: 0;
}
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container .show-content-button.collapsed:hover + .content,
.path-mod-forum .unified-grader .grader-module-content-display .discussion-container .posts-container .parent-container .show-content-button.collapsed:focus + .content {
  opacity: 1;
}
.path-mod-forum .unified-grader .grader-module-content-display .no-post-container .icon {
  height: 250px;
  width: 250px;
  margin-right: 0;
}
.path-mod-forum .unified-grader .grader-module-content-display .nested-v2-display-mode .discussion-container .posts-container .parent-container .show-content-button {
  padding-left: 94px;
}
.path-mod-forum .unified-grader .no-search-results-container .icon {
  height: 250px;
  width: 250px;
  margin-right: 0;
}
.path-mod-forum .unified-grader .nested-v2-display-mode .view-context-button {
  margin-left: 94px;
  border-radius: 0.6rem;
}
.path-mod-forum .unified-grader .nested-v2-display-mode .parent-container .author-image-container {
  position: relative;
}
.path-mod-forum .unified-grader .nested-v2-display-mode .parent-container .author-image-container:after {
  position: absolute;
  top: calc(70px + 0.5rem);
  content: "";
  background-color: #e9ecef;
  width: 2px;
  height: calc(100% - 70px + 0.5rem);
}
.path-mod-forum .unified-grader .nested-v2-display-mode .parent-container + .post-container .author-image-container img {
  width: 30px !important; /* stylelint-disable-line declaration-no-important */
}

.path-mod-forum .unified-grader .nested-v2-display-mode .post-subject,
.path-mod-forum .modal .nested-v2-display-mode .post-subject {
  display: none;
}

@media (max-width: 575.98px) {
  .path-mod-forum .unified-grader .grader-grading-panel {
    width: 100%;
    position: fixed;
    height: calc(100vh - 50px);
    overflow: scroll;
    top: 50px;
  }
  .path-mod-forum .unified-grader .body-container {
    overflow: visible;
  }
}
.maincalendar .calendarmonth td,
.maincalendar .calendarmonth th {
  border: 1px dotted var(--bs-border-color);
}

.path-grade-report-grader h1, .path-grade-report-grader .h1 {
  text-align: inherit;
}

#page-mod-data-view #singleimage {
  width: auto;
}

.template_heading {
  margin-top: 10px;
}

.breadcrumb-button {
  margin-top: 4px;
}

.breadcrumb-button .singlebutton {
  float: left;
  margin-left: 4px;
}

.langmenu form {
  margin: 0;
}

canvas {
  -ms-touch-action: auto;
}

div#dock {
  display: none;
}

/** General styles (scope: all of lesson) **/
.path-mod-lesson .invisiblefieldset.fieldsetfix {
  display: block;
}

.path-mod-lesson .answeroption .checkbox label p {
  display: inline;
}

.path-mod-lesson .slideshow {
  overflow: auto;
  padding: 15px;
}

#page-mod-lesson-view .branchbuttoncontainer .singlebutton button[type=submit] {
  white-space: normal;
}

#page-mod-lesson-view .vertical .singlebutton {
  display: block;
}
#page-mod-lesson-view .vertical .singlebutton + .singlebutton {
  margin-left: 0;
  margin-top: 1rem;
}
#page-mod-lesson-view .fitem .felement .form-select {
  align-self: flex-start;
}

.path-mod-lesson .table td {
  vertical-align: middle;
}
.path-mod-lesson .table td label {
  margin-bottom: 0;
}
.path-mod-lesson .table td .highlight {
  display: inline-block;
  margin-left: 0.25rem;
}
.path-mod-lesson .table td input[type=checkbox] {
  display: block;
}

.path-mod-wiki .wiki_headingtitle,
.path-mod-wiki .midpad,
.path-mod-wiki .wiki_headingtime {
  text-align: inherit;
}

.path-mod-wiki .wiki_contentbox {
  width: 100%;
}

.nav .caret {
  margin-left: 4px;
}

.nav .divider {
  overflow: hidden;
  width: 0;
}

.userloggedinas .usermenu .usertext,
.userswitchedrole .usermenu .usertext,
.loginfailures .usermenu .usertext {
  float: left;
  text-align: right;
  margin-right: 0.5rem;
  height: 35px;
}
.userloggedinas .usermenu .usertext .meta,
.userswitchedrole .usermenu .usertext .meta,
.loginfailures .usermenu .usertext .meta {
  font-size: 0.8203125rem;
  align-items: center;
}
.userloggedinas .usermenu .avatar img,
.userswitchedrole .usermenu .avatar img,
.loginfailures .usermenu .avatar img {
  margin: 0;
}

.userloggedinas .usermenu .userbutton .avatars {
  position: relative;
  display: inline-block;
}
.userloggedinas .usermenu .userbutton .avatars .avatar.current {
  display: inline-block;
  position: absolute;
  bottom: 0;
  right: 0;
  width: 20px;
  height: 20px;
  border-radius: 50%;
}
.userloggedinas .usermenu .userbutton .avatars .avatar.current img {
  vertical-align: baseline;
}
.userloggedinas .usermenu .userbutton .avatars .avatar.current .userinitials.size-35 {
  width: 20px;
  height: 20px;
  border: 1px solid #dee2e6;
  background-color: #fff;
  font-size: 0.5625rem;
}
.userloggedinas .usermenu .userbutton .avatars .avatar img {
  width: inherit;
  height: inherit;
}
.userloggedinas .usermenu .userbutton .avatars .realuser {
  width: 35px;
  height: 35px;
  display: inline-block;
}

.userinitials {
  background-color: #e9ecef;
  vertical-align: middle;
  display: inline-flex;
  align-items: center;
  justify-content: center;
  border-radius: 50%;
  color: #343a40;
  font-weight: normal;
  margin-right: 0.25rem;
}
.userinitials.size-16, .userinitials.size-30 {
  font-size: 0.7rem;
  width: 30px;
  height: 30px;
}
.userinitials.size-35 {
  width: 35px;
  height: 35px;
}
.userinitials.size-50 {
  width: 50px;
  height: 50px;
}
.userinitials.size-64 {
  width: 64px;
  height: 64px;
}
.userinitials.size-100 {
  width: 100px;
  height: 100px;
  font-size: 1.875rem;
}

img.userpicture {
  margin-right: 0.25rem;
}

@media (max-width: 767.98px) {
  .usertext {
    display: none;
  }
}
#page-mod-quiz-mod #id_reviewoptionshdr .col-md-3,
#page-mod-quiz-mod #id_reviewoptionshdr .col-md-9 {
  width: auto;
  max-width: none;
}

#page-mod-quiz-mod #id_reviewoptionshdr .fcontainer > .fitem {
  float: left;
  width: 20rem;
  display: inline-block;
  min-height: 12rem;
}

#page-mod-quiz-mod #id_reviewoptionshdr .btn-link {
  line-height: 1.5;
  vertical-align: bottom;
}

#page-mod-quiz-mod #id_reviewoptionshdr .form-check {
  width: auto;
  height: 22px;
  justify-content: flex-start;
}

#page-mod-quiz-mod #id_reviewoptionshdr .review_option_item {
  width: 90%;
  height: 22px;
}

.path-mod-quiz #mod_quiz_navblock .qnbutton {
  text-decoration: none;
  font-size: 14px;
  line-height: 20px;
  font-weight: normal;
  background-color: var(--bs-body-bg);
  background-image: none;
  height: 40px;
  width: 30px;
  border-radius: 3px;
  border: 0;
  overflow: hidden;
  white-space: nowrap;
  margin: 0 6px 6px 0;
}
.path-mod-quiz #mod_quiz_navblock span.qnbutton {
  cursor: default;
  background-color: var(--bs-secondary-bg);
  color: #495057;
}
.path-mod-quiz #mod_quiz_navblock a.qnbutton:hover,
.path-mod-quiz #mod_quiz_navblock a.qnbutton:active,
.path-mod-quiz #mod_quiz_navblock a.qnbutton:focus {
  text-decoration: underline;
}
.path-mod-quiz #mod_quiz_navblock .qnbutton .thispageholder {
  border: 1px solid;
  border-radius: 3px;
  z-index: 1;
}
.path-mod-quiz #mod_quiz_navblock .qnbutton.thispage .thispageholder {
  border-width: 3px;
}
.path-mod-quiz #mod_quiz_navblock .allquestionsononepage .qnbutton.thispage .thispageholder {
  border-width: 1px;
}
.path-mod-quiz #mod_quiz_navblock .qnbutton .trafficlight {
  border: 0;
  background: var(--bs-body-bg) none center/10px no-repeat scroll;
  height: 20px;
  margin-top: 20px;
  border-radius: 0 0 3px 3px;
}
.path-mod-quiz #mod_quiz_navblock .qnbutton.correct .thispageholder {
  border-color: #306e2d;
}
.path-mod-quiz #mod_quiz_navblock .qnbutton.incorrect .thispageholder {
  border-color: #b62c1d;
}
.path-mod-quiz #mod_quiz_navblock .qnbutton.partiallycorrect .thispageholder {
  border-color: #90682f;
}
.path-mod-quiz #mod_quiz_navblock .qnbutton.correct,
.path-mod-quiz #mod_quiz_navblock .qnbutton.correct .trafficlight {
  background-color: #dff4d8; /* light green */
}
.path-mod-quiz #mod_quiz_navblock .qnbutton.incorrect,
.path-mod-quiz #mod_quiz_navblock .qnbutton.incorrect .trafficlight {
  background-color: #fdd; /* light red */
}
.path-mod-quiz #mod_quiz_navblock .qnbutton.partiallycorrect,
.path-mod-quiz #mod_quiz_navblock .qnbutton.partiallycorrect .trafficlight {
  background-color: #fcefdc; /* light yellow */
}
.path-mod-quiz #mod_quiz_navblock .qnbutton.blocked,
.path-mod-quiz #mod_quiz_navblock .qnbutton.blocked .trafficlight,
.path-mod-quiz #mod_quiz_navblock .qnbutton.complete,
.path-mod-quiz #mod_quiz_navblock .qnbutton.complete .trafficlight,
.path-mod-quiz #mod_quiz_navblock .qnbutton.answersaved,
.path-mod-quiz #mod_quiz_navblock .qnbutton.answersaved .trafficlight,
.path-mod-quiz #mod_quiz_navblock .qnbutton.requiresgrading,
.path-mod-quiz #mod_quiz_navblock .qnbutton.requiresgrading .trafficlight {
  background-color: #dee2e6;
}
.path-mod-quiz #mod_quiz_navblock .qnbutton.notyetanswered .trafficlight,
.path-mod-quiz #mod_quiz_navblock .qnbutton.invalidanswer .trafficlight {
  background-color: var(--bs-body-bg);
}
.path-mod-quiz #mod_quiz_navblock .qnbutton.correct .trafficlight {
  background-image: url([[pix:core|i/grade_correct]]);
  color: #357a32;
  background-size: 14px;
}
.path-mod-quiz #mod_quiz_navblock .qnbutton.blocked .trafficlight {
  background-image: url([[pix:core|t/locked]]);
}
.path-mod-quiz #mod_quiz_navblock .qnbutton.notanswered .trafficlight,
.path-mod-quiz #mod_quiz_navblock .qnbutton.incorrect .trafficlight {
  background-image: url([[pix:core|i/grade_incorrect]]);
  color: #ca3120;
  background-size: 14px;
}
.path-mod-quiz #mod_quiz_navblock .qnbutton.partiallycorrect .trafficlight {
  background-image: url([[pix:core|i/grade_partiallycorrect]]);
  color: #f0ad4e;
  background-size: 14px;
}
.path-mod-quiz #mod_quiz_navblock .qnbutton.complete .trafficlight,
.path-mod-quiz #mod_quiz_navblock .qnbutton.answersaved .trafficlight,
.path-mod-quiz #mod_quiz_navblock .qnbutton.requiresgrading .trafficlight {
  background-color: #dee2e6;
}

#page-mod-quiz-edit ul.slots li.section li.activity .instancemaxmarkcontainer form input {
  height: 1.4em;
  vertical-align: middle;
}

#page-mod-quiz-edit ul.slots li.section li.activity .instancemaxmarkcontainer {
  padding: 0.5em 0 0.5em 0.1em;
  margin: 2px;
}

/* Override the color of the quiz attempt icons for accessibility. */
#attemptsform .icon.text-success,
.formulation .answer .icon.text-success {
  color: #306e2d !important; /* stylelint-disable-line declaration-no-important */
}
#attemptsform .icon.text-danger,
.formulation .answer .icon.text-danger {
  color: #b62c1d !important; /* stylelint-disable-line declaration-no-important */
}
#attemptsform .icon.text-warning,
.formulation .answer .icon.text-warning {
  color: #90682f !important; /* stylelint-disable-line declaration-no-important */
}

/* Countdown timer. */
#page-mod-quiz-attempt #region-main {
  overflow-x: inherit;
}

#quiz-timer-wrapper {
  display: none;
  position: sticky;
  justify-content: end;
  top: 65px;
  z-index: 1020;
}
#quiz-timer-wrapper #quiz-timer {
  border: 1px solid #ca3120;
  background-color: #fff;
}

.pagelayout-embedded #quiz-timer-wrapper {
  top: 5px;
}

#quiz-timer-wrapper #quiz-timer.timeleft0 {
  background-color: #ca3120;
  color: #fff;
}

#quiz-timer-wrapper #quiz-timer.timeleft1 {
  background-color: #d73422;
  color: #fff;
}

#quiz-timer-wrapper #quiz-timer.timeleft2 {
  background-color: #dd3d2b;
  color: #000;
}

#quiz-timer-wrapper #quiz-timer.timeleft3 {
  background-color: #e04938;
  color: #000;
}

#quiz-timer-wrapper #quiz-timer.timeleft4 {
  background-color: #e25546;
  color: #000;
}

#quiz-timer-wrapper #quiz-timer.timeleft5 {
  background-color: #e46153;
  color: #000;
}

#quiz-timer-wrapper #quiz-timer.timeleft6 {
  background-color: #e66d60;
  color: #000;
}

#quiz-timer-wrapper #quiz-timer.timeleft7 {
  background-color: #e8796d;
  color: #000;
}

#quiz-timer-wrapper #quiz-timer.timeleft8 {
  background-color: #ea867a;
  color: #000;
}

#quiz-timer-wrapper #quiz-timer.timeleft9 {
  background-color: #ec9288;
  color: #000;
}

#quiz-timer-wrapper #quiz-timer.timeleft10 {
  background-color: #ee9e95;
  color: #000;
}

#quiz-timer-wrapper #quiz-timer.timeleft11 {
  background-color: #f0aaa2;
  color: #000;
}

#quiz-timer-wrapper #quiz-timer.timeleft12 {
  background-color: #f2b6af;
  color: #000;
}

#quiz-timer-wrapper #quiz-timer.timeleft13 {
  background-color: #f4c2bc;
  color: #000;
}

#quiz-timer-wrapper #quiz-timer.timeleft14 {
  background-color: #f7ceca;
  color: #000;
}

#quiz-timer-wrapper #quiz-timer.timeleft15 {
  background-color: #f9dad7;
  color: #000;
}

#quiz-timer-wrapper #quiz-timer.timeleft16 {
  background-color: #fbe6e4;
  color: #000;
}

.path-mod-assign [data-region=grade-actions-panel] [data-region=grade-actions] .collapse-buttons {
  top: auto;
}

.path-mod-assign #page-content [data-region=grade-panel] .mform:not(.unresponsive) .fcontainer .fitem.popout .felement {
  height: calc(100% - 4rem);
}

.path-mod-assign [data-region=grade-panel] {
  padding-top: 1rem;
}

.path-mod-assign [data-region=grade-panel] .fitem > .col-md-3,
.path-mod-assign [data-region=grade-panel] .fitem > .col-md-9 {
  width: 100%;
  padding: 0;
  max-width: 100%;
  flex: none;
}

.path-mod-assign [data-region=grade-panel] fieldset,
.path-mod-assign [data-region=grade-panel] .fitem.row {
  margin: 0;
}

.path-mod-assign [data-region=grade-panel] .mform .fitem.has-popout .felement {
  width: 100%;
  overflow: auto;
  height: calc(100% - 4rem);
}

.path-mod-assign [data-region=grade-panel] .mform .fitem .felement {
  width: auto;
}

.path-mod-assign [data-region=grade-panel] .popout {
  background-color: var(--bs-body-bg);
}

.path-mod-assign [data-region=grade-panel] .fitem.has-popout {
  background-color: var(--bs-body-bg);
  border-radius: var(--bs-border-radius);
  border: var(--bs-border-width) solid var(--bs-border-color-translucent);
  padding: 1rem;
  margin-bottom: 1rem;
}

.path-mod-assign [data-region=grade-panel] .has-popout .col-md-3 {
  border-bottom: var(--bs-border-width) solid;
  margin-bottom: 1rem;
}

.path-mod-assign [data-region=grade-panel] .popout > .col-md-3 {
  display: flex;
  align-items: flex-start;
  justify-content: space-between;
  font-size: 1.40625rem;
}

.path-mod-assign [data-region=grade-panel] .popout [data-region=popout-button] {
  margin-top: 0;
}

.path-mod-assign [data-region=assignment-info] {
  overflow-y: hidden;
}

.path-mod-assign [data-region=grading-navigation] {
  padding: 6px;
}

.path-mod-assign [data-region=grade-actions] {
  padding: 10px;
}

.path-mod-assign [data-region=user-info] .img-rounded {
  margin-top: 0;
}

.path-mod-assign [data-region=grading-navigation-panel] {
  height: 85px;
}

@media (max-width: 767px) {
  .path-mod-assign [data-region=grading-navigation-panel] {
    height: auto;
  }
  .path-mod-assign [data-region=user-info] {
    margin-top: 1rem;
  }
}
.path-mod-assign [data-region=grading-navigation] [data-region=input-field] input {
  width: auto;
  display: inline-block;
}

/**
 * Assign feedback.
 */
.assignfeedback_editpdf_widget * {
  box-sizing: content-box;
}

.assignfeedback_editpdf_widget button {
  box-sizing: border-box;
}

.assignfeedback_editpdf_widget .commentcolourbutton img {
  border-width: 0;
}

.assignfeedback_editpdf_widget .label {
  position: relative;
  padding: 1rem 1rem;
  margin-bottom: 1rem;
  border: var(--bs-border-width) solid transparent;
  color: #004d5a;
  border-color: #b3d9e0;
  border-radius: var(--bs-border-radius);
  background-color: #cce6ea;
}

.assignfeedback_editpdf_menu {
  padding: 0;
}

.path-mod-assign [data-region=grade-panel] .gradingform_guide .remark .commentchooser {
  float: none;
}

.path-mod-assign [data-region=grade-panel] .gradingform_guide .markingguideremark {
  width: 100%;
}

.path-mod-assign [data-region=grade-panel] .mform .fitem .felement[data-fieldtype=grading] {
  padding-left: 1rem;
  padding-right: 1rem;
}

.path-mod-assign [data-region=grade-panel] .showmarkerdesc,
.path-mod-assign [data-region=grade-panel] .showstudentdesc {
  background-color: var(--bs-body-bg);
}

.path-mod-assign .gradingtable thead tr {
  position: sticky;
  top: 60px;
  z-index: 1;
}

/**
 * Mod LTI.
 */
.path-admin-mod-lti .btn .loader img,
.path-admin-mod-lti #tool-list-loader-container .loader img {
  height: auto;
}

/**
 * Mod Feedback.
 */
.path-mod-feedback span.feedback_info {
  font-weight: bold;
}
.path-mod-feedback span.feedback_depend {
  color: #ca3120;
}
.path-mod-feedback hr.feedback_pagebreak {
  height: 4px;
  color: #6a737b;
  background-color: #6a737b;
  border: 0;
  margin: 0;
}
.path-mod-feedback .feedback_bar_image {
  height: 10px;
}
.path-mod-feedback #analysis-form label {
  display: inline;
}
.path-mod-feedback .templateslist td.cell.action {
  width: 10%;
}
.path-mod-feedback .templateslist th.header.action {
  width: 10%;
}
.path-mod-feedback .feedback-item-label {
  width: 100%;
}
.path-mod-feedback #feedback_edit_form [id*=_feedback_item_].feedback_itemlist,
.path-mod-feedback #feedback_edit_form .feedback_itemlist.sortable-list-is-dragged {
  padding: 1rem;
  border: 1px solid #dee2e6;
  background-color: #fff;
  position: relative;
  border-radius: 0.5rem;
}
.path-mod-feedback #feedback_edit_form [id*=_feedback_item_].feedback_itemlist .itemname,
.path-mod-feedback #feedback_edit_form .feedback_itemlist.sortable-list-is-dragged .itemname {
  margin-right: 2rem;
}
.path-mod-feedback #feedback_edit_form [id*=_feedback_item_].feedback_itemlist .itemactions,
.path-mod-feedback #feedback_edit_form .feedback_itemlist.sortable-list-is-dragged .itemactions {
  position: absolute;
  top: 1rem;
  right: 1rem;
}
.path-mod-feedback #feedback_edit_form [id*=_feedback_item_].feedback_itemlist.sortable-list-current-position,
.path-mod-feedback #feedback_edit_form .feedback_itemlist.sortable-list-is-dragged.sortable-list-current-position {
  background-color: #f8f9fa;
}
.path-mod-feedback #feedback_edit_form .sortable-list-is-dragged {
  opacity: 0.75;
  max-width: 830px;
}
.path-mod-feedback #feedback_edit_form .loading-icon {
  position: absolute;
  left: 50%;
  top: calc(50% - 0.5rem);
}
.path-mod-feedback#page-mod-feedback-manage_templates .coursetemplates .no-overflow,
.path-mod-feedback#page-mod-feedback-manage_templates .publictemplates .no-overflow {
  overflow: visible;
}
.path-mod-feedback#page-mod-feedback-manage_templates .coursetemplates .action-menu .menubar,
.path-mod-feedback#page-mod-feedback-manage_templates .publictemplates .action-menu .menubar {
  justify-content: end;
}
.path-mod-feedback table.analysis {
  width: 100%;
  border-top: 1px solid #dee2e6;
  margin-top: 10px;
}
.path-mod-feedback table.analysis.itemtype_textarea td {
  padding: 4px 0;
}
.path-mod-feedback table.analysis tr:first-child th {
  padding-top: 10px;
}
.path-mod-feedback table.analysis tr:hover {
  background: #dee2e6;
}
.path-mod-feedback table.analysis td.singlevalue:before,
.path-mod-feedback table.analysis td.optionname:before {
  content: "- ";
}
.path-mod-feedback table.analysis tr.isempty {
  display: none;
}
.path-mod-feedback #showentrytable td.cell.completed_timemodified,
.path-mod-feedback #showentryanontable td.cell.random_response {
  font-weight: bold;
}
.path-mod-feedback #showentrytable td.cell.userpic,
.path-mod-feedback #showentrytable td.cell.deleteentry,
.path-mod-feedback #showentryanontable td.cell.deleteentry {
  width: 10px;
}
.path-mod-feedback .response_navigation {
  margin: 0.5rem 0;
}
.path-mod-feedback .response_navigation a {
  display: block;
}
.path-mod-feedback .response_navigation a.back_to_list {
  text-align: center;
}
.path-mod-feedback .response_navigation .prev_response {
  text-align: left;
}
.path-mod-feedback .response_navigation .prev_response:before {
  /*rtl:raw:
  content: ' ► ';
  */
  /*rtl:remove*/
  content: " ◄ ";
}
.path-mod-feedback .response_navigation .next_response {
  text-align: right;
}
.path-mod-feedback .response_navigation .next_response:after {
  /*rtl:raw:
  content: ' ◄ ';
  */
  /*rtl:remove*/
  content: " ► ";
}

/* reports.less */
#page-report-participation-index .participationselectform div label {
  display: inline-block;
  margin: 0 5px;
}

#page-report-participation-index .participationselectform div label[for=menuinstanceid] {
  margin-left: 0;
}

#page-report-outline-index .font-lg {
  font-size: 1.03125rem;
}

#page-report-outline-index #outlinereport tbody tr {
  background-color: #fff;
}
#page-report-outline-index #outlinereport tbody tr.section {
  padding-left: 2rem;
}
#page-report-outline-index #outlinereport tbody tr.section h3, #page-report-outline-index #outlinereport tbody tr.section .h3 {
  font-size: 1.171875rem;
}
#page-report-outline-index #outlinereport tbody tr.section h4, #page-report-outline-index #outlinereport tbody tr.section .h4 {
  font-size: 1.03125rem;
}
#page-report-outline-index #outlinereport tbody td.delegated {
  padding-left: 3.5rem;
}

#page-report-outline-user .section {
  border: 1px solid #dee2e6;
  border-radius: 1rem;
}

#page-report-outline-user .font-lg {
  font-size: 1.03125rem;
}

#page-report-log-index #menumodid option:disabled {
  color: #1f2428;
  font-weight: bolder;
}

.path-backup .mform {
  /* These are long labels with checkboxes on the right. */
}
.path-backup .mform .grouped_settings {
  clear: both;
  overflow: hidden;
  /* Use card styles but avoid extend because that brings in too much. */
}
.path-backup .mform .grouped_settings.section_level {
  background-color: var(--bs-body-bg);
  border-radius: var(--bs-border-radius);
  border: var(--bs-border-width) solid var(--bs-border-color-translucent);
  padding: 1rem;
  margin-bottom: 1rem;
}
.path-backup .mform .grouped_settings.section_level::after {
  display: block;
  clear: both;
  content: "";
}
.path-backup .mform .grouped_settings.subsection_level {
  background-color: var(--bs-body-bg);
  border-radius: var(--bs-border-radius);
  border: var(--bs-border-width) solid var(--bs-border-color-translucent);
  padding: 1rem;
  margin: 0 1rem 1rem 1rem;
}
.path-backup .mform .grouped_settings.subsection_level::after {
  display: block;
  clear: both;
  content: "";
}
.path-backup .mform .include_setting {
  width: 50%;
  display: inline-block;
  float: left;
  padding: 0.3rem;
}
.path-backup .mform .normal_setting {
  width: 50%;
  display: inline-block;
  float: left;
  padding: 0.3rem;
}

.path-backup {
  /* Bold section labels */
}
.path-backup .section_level {
  font-weight: bold;
}
.path-backup .section_level .activity_level {
  font-weight: normal;
}
.path-backup .proceedbutton {
  margin-left: auto;
}

/* Override the columns width to leave more room for the labels. */
.path-backup .mform fieldset {
  padding: 1rem 0;
}
.path-backup .mform .grouped_settings.activity_level {
  /* Striped rows like a table */
}
.path-backup .mform .grouped_settings.activity_level:nth-of-type(odd) {
  background-color: rgba(var(--bs-emphasis-color-rgb), 0.03);
}
.path-backup .mform .grouped_settings.activity_level:nth-of-type(even) {
  background-color: transparent;
}
.path-backup .mform .fcontainer .grouped_settings .fitem,
.path-backup .mform .fcontainer .root_setting .fitem {
  margin: 0.25rem 0 !important; /* stylelint-disable declaration-no-important */
  /* These checkboxes with no label on the left. */
}
.path-backup .mform .fcontainer .grouped_settings .fitem .col-form-label,
.path-backup .mform .fcontainer .root_setting .fitem .col-form-label {
  padding-top: 0;
}
.path-backup .mform .fcontainer .grouped_settings .fitem .col-md-3.checkbox,
.path-backup .mform .fcontainer .root_setting .fitem .col-md-3.checkbox {
  width: 0%;
}
.path-backup .mform .fcontainer .grouped_settings .fitem .col-md-9.checkbox,
.path-backup .mform .fcontainer .root_setting .fitem .col-md-9.checkbox {
  width: 100%;
  left: 0;
}

/* Detail pair is (usually) some short label with a longer value */
.path-backup .detail-pair .detail-pair-label {
  width: 25%;
  float: left;
  clear: left;
}
.path-backup .detail-pair .detail-pair-value {
  width: 75%;
  float: left;
}

.path-backup .backup-restore .singlebutton {
  float: right;
}

/* Make these bits full width and work with the detail-pair */
.path-backup .backup-section {
  /* Fix for nested table headers */
  /* Add card styles to backup sections */
  background-color: var(--bs-body-bg);
  border-radius: var(--bs-border-radius);
  border: var(--bs-border-width) solid var(--bs-border-color-translucent);
  padding: 1rem;
  margin-bottom: 1rem;
}
.path-backup .backup-section .sub-header,
.path-backup .backup-section .backup-sub-section,
.path-backup .backup-section .singlebutton,
.path-backup .backup-section .header {
  width: 100%;
  float: left;
  clear: both;
}
.path-backup .backup-section th.header {
  width: auto;
  float: none;
}
.path-backup .backup-section ::after {
  content: "";
  display: table;
  clear: both;
}
.path-backup .backup-section::after {
  display: block;
  clear: both;
  content: "";
}

.path-backup .notification.dependencies_enforced {
  color: #ca3120;
  font-weight: bold;
}

.path-backup .backup_progress {
  margin-top: 1rem;
  margin-bottom: 1rem;
}
.path-backup .backup_progress .backup_stage {
  color: var(--bs-secondary-color);
}
.path-backup .backup_progress .backup_stage.backup_stage_current {
  font-weight: bold;
  color: inherit;
}

.path-backup .backup_progress span.backup_stage.backup_stage_complete {
  color: inherit;
}

#page-backup-restore .filealiasesfailures {
  background-color: #f4d6d2;
}
#page-backup-restore .filealiasesfailures .aliaseslist {
  background-color: #fff;
}

.path-backup .wibbler {
  width: 500px;
  margin: 0 auto 10px;
  border-bottom: 1px solid #000;
  border-right: 1px solid #000;
  border-left: 1px solid #000;
  position: relative;
  min-height: 4px;
}
.path-backup .wibbler .wibble {
  position: absolute;
  left: 0;
  right: 0;
  top: 0;
  height: 4px;
}
.path-backup .wibbler .state0 {
  background: #eee;
}
.path-backup .wibbler .state1 {
  background: #ddd;
}
.path-backup .wibbler .state2 {
  background: #ccc;
}
.path-backup .wibbler .state3 {
  background: #bbb;
}
.path-backup .wibbler .state4 {
  background: #aaa;
}
.path-backup .wibbler .state5 {
  background: #999;
}
.path-backup .wibbler .state6 {
  background: #888;
}
.path-backup .wibbler .state7 {
  background: #777;
}
.path-backup .wibbler .state8 {
  background: #666;
}
.path-backup .wibbler .state9 {
  background: #555;
}
.path-backup .wibbler .state10 {
  background: #444;
}
.path-backup .wibbler .state11 {
  background: #333;
}
.path-backup .wibbler .state12 {
  background: #222;
}

/**
 * File tables.scss.
 * Contains styles for tables.
 */
.generaltable {
  border-top: var(--bs-border-width) solid var(--bs-border-color);
}
.generaltable th,
.generaltable td {
  padding: 0.75rem;
  border-top: var(--bs-border-width) solid var(--bs-border-color);
}

table {
  caption-side: top;
}
table .sticky-column {
  position: sticky;
  left: 0;
  background-color: inherit;
}

.table-dynamic > .loading-icon {
  position: absolute;
  left: calc(50% - 1.5rem);
  top: 200px;
}
.table-dynamic > .loading-icon .icon {
  max-height: 3rem;
  max-width: 3rem;
  font-size: 3rem;
}

/**
 * Custom styles for Bootstrap table classes.
 */
.table {
  border-top: var(--bs-border-width) solid var(--bs-border-color);
}
.table tbody td,
.table tbody th {
  color: inherit;
  background-color: inherit;
}
.table tbody tr:hover.dimmed_text a:not(.menu-action), .table tbody tr:hover.text-muted a:not(.menu-action) {
  color: var(--bs-emphasis-color);
}
.table tbody tr:hover td.sticky-column {
  background-color: rgba(var(--bs-emphasis-color-rgb), 0.075);
}

.table-responsive {
  scrollbar-width: thin;
  scrollbar-color: #6a737b #f8f9fa;
}
.table-responsive::-webkit-scrollbar {
  width: 12px;
}
.table-responsive::-webkit-scrollbar-track {
  background: #f8f9fa;
}
.table-responsive::-webkit-scrollbar-thumb {
  background-color: #6a737b;
  border-radius: 20px;
  border: 3px solid #f8f9fa;
}
.table-responsive::-webkit-scrollbar-thumb:hover {
  background-color: #495057;
}
.table-responsive .dropdown {
  position: static !important; /* stylelint-disable-line declaration-no-important */
}
.table-responsive .table {
  margin-bottom: 0;
}
.table-responsive .table .visually-hidden,
.table-responsive .table .accesshide {
  position: relative !important; /* stylelint-disable-line declaration-no-important */
  display: block;
}

/**
 * File buttons.scss.
 * Contains styles for buttons.
 */
.singlebutton {
  display: inline-block;
}
.singlebutton + .singlebutton {
  margin-left: 0.5rem;
}

.continuebutton {
  text-align: center;
}

p.arrow_button {
  margin-top: 5em;
  text-align: center;
}

#addcontrols {
  margin-top: 11.25rem;
  text-align: center;
  margin-bottom: 3em;
}
#addcontrols label {
  display: inline;
}

#addcontrols input,
#removecontrols input {
  width: 100%;
  margin: auto;
}

.btn-lineup {
  margin: 0 0 10px 5px;
}

/* Small buttons should have the same border-radius as the normal buttons. */
.btn-sm,
.btn-group-sm > .btn {
  --bs-btn-border-radius: var(--bs-border-radius);
}

/**
 * Focus styles to improve Bootstrap default contrast.
 */
.btn:focus-visible, .btn.focus {
  outline: 0.25rem solid #5f6e7d;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-primary:focus-visible, .btn-primary.focus {
  outline: 0.25rem solid #000102;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-secondary:focus-visible, .btn-secondary.focus {
  outline: 0.25rem solid #5f6e7d;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-success:focus-visible, .btn-success.focus {
  outline: 0.25rem solid black;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-info:focus-visible, .btn-info.focus {
  outline: 0.25rem solid black;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-warning:focus-visible, .btn-warning.focus {
  outline: 0.25rem solid #694109;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-danger:focus-visible, .btn-danger.focus {
  outline: 0.25rem solid #1a0604;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-light:focus-visible, .btn-light.focus {
  outline: 0.25rem solid #8193a5;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-dark:focus-visible, .btn-dark.focus {
  outline: 0.25rem solid black;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-outline-primary:focus-visible, .btn-outline-primary.focus {
  outline: 0.25rem solid #000102;
  box-shadow: inset 0 0 0 2px #343a40;
}

.btn-outline-secondary:focus-visible, .btn-outline-secondary.focus {
  outline: 0.25rem solid #5f6e7d;
  box-shadow: inset 0 0 0 2px #343a40;
}

.btn-outline-success:focus-visible, .btn-outline-success.focus {
  outline: 0.25rem solid black;
  box-shadow: inset 0 0 0 2px #343a40;
}

.btn-outline-info:focus-visible, .btn-outline-info.focus {
  outline: 0.25rem solid black;
  box-shadow: inset 0 0 0 2px #343a40;
}

.btn-outline-warning:focus-visible, .btn-outline-warning.focus {
  outline: 0.25rem solid #694109;
  box-shadow: inset 0 0 0 2px #343a40;
}

.btn-outline-danger:focus-visible, .btn-outline-danger.focus {
  outline: 0.25rem solid #1a0604;
  box-shadow: inset 0 0 0 2px #343a40;
}

.btn-outline-light:focus-visible, .btn-outline-light.focus {
  outline: 0.25rem solid #8193a5;
  box-shadow: inset 0 0 0 2px #343a40;
}

.btn-outline-dark:focus-visible, .btn-outline-dark.focus {
  outline: 0.25rem solid black;
  box-shadow: inset 0 0 0 2px #343a40;
}

/**
 * Icon buttons.
 */
.btn-icon {
  --bs-btn-hover-bg: var(--bs-secondary-bg);
  --bs-btn-border-radius: var(--bs-border-radius);
  --bs-btn-padding-x: 0;
  --bs-btn-padding-y: 0;
  --bs-btn-font-size: 16px;
  height: calc(16px + 1rem);
  width: calc(16px + 1rem);
  display: inline-flex;
  align-items: center;
  justify-content: center;
}
.btn-icon:focus-visible, .btn-icon.focus {
  outline: 0.25rem solid #5f6e7d;
  box-shadow: inset 0 0 0 2px #fff;
}
.btn-icon .icon {
  margin: 0;
}
.btn-icon.icon-size-0 {
  height: calc(0 + 1rem);
  width: calc(0 + 1rem);
  --bs-btn-font-size: 0;
  --bs-btn-line-height: 0;
}
.btn-icon.icon-size-1 {
  height: calc(4px + 1rem);
  width: calc(4px + 1rem);
  --bs-btn-font-size: 4px;
  --bs-btn-line-height: 4px;
}
.btn-icon.icon-size-2 {
  height: calc(8px + 1rem);
  width: calc(8px + 1rem);
  --bs-btn-font-size: 8px;
  --bs-btn-line-height: 8px;
}
.btn-icon.icon-size-3 {
  height: calc(16px + 1rem);
  width: calc(16px + 1rem);
  --bs-btn-font-size: 16px;
  --bs-btn-line-height: 16px;
}
.btn-icon.icon-size-4 {
  height: calc(24px + 1rem);
  width: calc(24px + 1rem);
  --bs-btn-font-size: 24px;
  --bs-btn-line-height: 24px;
}
.btn-icon.icon-size-5 {
  height: calc(32px + 1rem);
  width: calc(32px + 1rem);
  --bs-btn-font-size: 32px;
  --bs-btn-line-height: 32px;
}
.btn-icon.icon-size-6 {
  height: calc(40px + 1rem);
  width: calc(40px + 1rem);
  --bs-btn-font-size: 40px;
  --bs-btn-line-height: 40px;
}
.btn-icon.icon-size-7 {
  height: calc(48px + 1rem);
  width: calc(48px + 1rem);
  --bs-btn-font-size: 48px;
  --bs-btn-line-height: 48px;
}

/* Specific styles for the collapse/expand buttons. */
.btn-icon.icons-collapse-expand {
  color: #0f6cbf;
  background-color: #e7f0f9;
}
.btn-icon.icons-collapse-expand:hover {
  outline: 2px solid #0f6cbf;
}
.btn-icon.icons-collapse-expand:focus-visible, .btn-icon.icons-collapse-expand.focus {
  outline: 0.25rem solid #3481c1;
  box-shadow: inset 0 0 0 2px #fff;
}

/**
 * Subtle buttons.
 */
.btn-subtle-primary {
  --bs-btn-font-weight: 500;
  --bs-btn-color: var(--bs-primary-text-emphasis);
  --bs-btn-bg: var(--bs-primary-bg-subtle);
  --bs-btn-active-color: var(--bs-primary-text-emphasis);
  --bs-btn-active-bg: var(--bs-primary-bg-subtle);
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: var(--bs-primary);
  --bs-btn-focus-shadow-rgb: 15, 108, 191;
}
.btn-subtle-primary:focus-visible, .btn-subtle-primary.focus {
  outline: 0.25rem solid #000102;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-subtle-secondary {
  --bs-btn-font-weight: 500;
  --bs-btn-color: var(--bs-secondary-text-emphasis);
  --bs-btn-bg: var(--bs-secondary-bg-subtle);
  --bs-btn-active-color: var(--bs-secondary-text-emphasis);
  --bs-btn-active-bg: var(--bs-secondary-bg-subtle);
  --bs-btn-hover-color: #000;
  --bs-btn-hover-bg: var(--bs-secondary);
  --bs-btn-focus-shadow-rgb: 206, 212, 218;
}
.btn-subtle-secondary:focus-visible, .btn-subtle-secondary.focus {
  outline: 0.25rem solid #5f6e7d;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-subtle-success {
  --bs-btn-font-weight: 500;
  --bs-btn-color: var(--bs-success-text-emphasis);
  --bs-btn-bg: var(--bs-success-bg-subtle);
  --bs-btn-active-color: var(--bs-success-text-emphasis);
  --bs-btn-active-bg: var(--bs-success-bg-subtle);
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: var(--bs-success);
  --bs-btn-focus-shadow-rgb: 53, 122, 50;
}
.btn-subtle-success:focus-visible, .btn-subtle-success.focus {
  outline: 0.25rem solid black;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-subtle-info {
  --bs-btn-font-weight: 500;
  --bs-btn-color: var(--bs-info-text-emphasis);
  --bs-btn-bg: var(--bs-info-bg-subtle);
  --bs-btn-active-color: var(--bs-info-text-emphasis);
  --bs-btn-active-bg: var(--bs-info-bg-subtle);
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: var(--bs-info);
  --bs-btn-focus-shadow-rgb: 0, 129, 150;
}
.btn-subtle-info:focus-visible, .btn-subtle-info.focus {
  outline: 0.25rem solid black;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-subtle-warning {
  --bs-btn-font-weight: 500;
  --bs-btn-color: var(--bs-warning-text-emphasis);
  --bs-btn-bg: var(--bs-warning-bg-subtle);
  --bs-btn-active-color: var(--bs-warning-text-emphasis);
  --bs-btn-active-bg: var(--bs-warning-bg-subtle);
  --bs-btn-hover-color: #000;
  --bs-btn-hover-bg: var(--bs-warning);
  --bs-btn-focus-shadow-rgb: 240, 173, 78;
}
.btn-subtle-warning:focus-visible, .btn-subtle-warning.focus {
  outline: 0.25rem solid #694109;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-subtle-danger {
  --bs-btn-font-weight: 500;
  --bs-btn-color: var(--bs-danger-text-emphasis);
  --bs-btn-bg: var(--bs-danger-bg-subtle);
  --bs-btn-active-color: var(--bs-danger-text-emphasis);
  --bs-btn-active-bg: var(--bs-danger-bg-subtle);
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: var(--bs-danger);
  --bs-btn-focus-shadow-rgb: 202, 49, 32;
}
.btn-subtle-danger:focus-visible, .btn-subtle-danger.focus {
  outline: 0.25rem solid #1a0604;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-subtle-light {
  --bs-btn-font-weight: 500;
  --bs-btn-color: var(--bs-light-text-emphasis);
  --bs-btn-bg: var(--bs-light-bg-subtle);
  --bs-btn-active-color: var(--bs-light-text-emphasis);
  --bs-btn-active-bg: var(--bs-light-bg-subtle);
  --bs-btn-hover-color: #000;
  --bs-btn-hover-bg: var(--bs-light);
  --bs-btn-focus-shadow-rgb: 248, 249, 250;
}
.btn-subtle-light:focus-visible, .btn-subtle-light.focus {
  outline: 0.25rem solid #8193a5;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-subtle-dark {
  --bs-btn-font-weight: 500;
  --bs-btn-color: var(--bs-dark-text-emphasis);
  --bs-btn-bg: var(--bs-dark-bg-subtle);
  --bs-btn-active-color: var(--bs-dark-text-emphasis);
  --bs-btn-active-bg: var(--bs-dark-bg-subtle);
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: var(--bs-dark);
  --bs-btn-focus-shadow-rgb: 29, 33, 37;
}
.btn-subtle-dark:focus-visible, .btn-subtle-dark.focus {
  outline: 0.25rem solid black;
  box-shadow: inset 0 0 0 2px #fff;
}

.btn-subtle-body {
  --bs-btn-font-weight: 500;
  --bs-btn-color: var(--bs-body-color);
  --bs-btn-bg: transparent;
  --bs-btn-border-color: var(--bs-border-color);
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: var(--bs-gray-600);
  --bs-btn-focus-shadow-rgb: 106, 115, 123;
}
.btn-subtle-body:focus-visible, .btn-subtle-body.focus {
  outline: 0.25rem solid #0c0d0d;
  box-shadow: inset 0 0 0 2px #fff;
}

/*
 * Add content buttons.
 */
.btn-add-content {
  border-radius: 0.5rem;
  border: 2px dashed #dee2e6;
  color: #0f6cbf;
  background-color: #fff;
  font-size: 0.8203125rem;
  font-weight: bold;
}
.btn-add-content:hover, .btn-add-content:focus {
  background-color: #e7f0f9;
  border-style: solid;
  border-color: #0f6cbf;
  color: #0f6cbf;
}

.gradetreebox {
  margin: 20px 0 30px 0;
}
.gradetreebox h4, .gradetreebox .h4 {
  font-size: 0.9375rem;
}
.gradetreebox th.cell,
.gradetreebox input[type=text] {
  width: auto;
}
.gradetreebox input[type=text],
.gradetreebox select {
  margin-bottom: 0;
}

.core_grades_notices .singlebutton {
  display: inline-block;
}

.path-grade-report #maincontent + .urlselect {
  position: absolute;
  left: 40vw;
}

.path-grade-report-grader #region-main {
  min-width: 100%;
  width: auto;
  display: flex;
  flex-direction: column;
}
.path-grade-report-grader #region-main > .card {
  width: auto;
  overflow-x: initial;
}
.path-grade-report-grader #region-main div[role=main] {
  flex: 1 1 auto;
}
.path-grade-report-grader [data-region=blocks-column] {
  width: 100%;
  clear: both;
}

.path-grade-report-grader .gradepass,
.path-grade-report-user .gradepass {
  color: #357a32;
}
.path-grade-report-grader .gradefail,
.path-grade-report-user .gradefail {
  color: #ca3120;
}

.path-grade #region-main {
  overflow-x: visible;
}
.path-grade .user-heading .userinitials {
  width: 50px;
  height: 50px;
}

#page-grade-grading-manage #activemethodselector label {
  display: inline-block;
}
#page-grade-grading-manage #activemethodselector .helptooltip {
  margin-right: 0.5em;
}
#page-grade-grading-manage .actions {
  display: block;
  text-align: center;
  margin-bottom: 1em;
}
#page-grade-grading-manage .actions .action {
  display: inline-block;
  position: relative;
  vertical-align: top;
  width: 150px;
  text-align: center;
  overflow: hidden;
  margin: 0.5em;
  padding: 1em;
  border: 1px solid #aaa;
}
#page-grade-grading-manage .actions .action .action-text {
  position: relative;
  top: 0.4em;
  font-size: 14px;
  white-space: normal;
}

#page-grade-grading-form-rubric-edit .gradingform_rubric_editform .status {
  font-size: 70%;
}

.gradingform_rubric {
  margin-bottom: 1em;
}
.gradingform_rubric.evaluate .criterion .levels .level:hover, .gradingform_rubric.evaluate .criterion .levels .level.checked {
  background: #dff0d8;
}
.gradingform_rubric.evaluate .criterion .levels .level.checked {
  border: none;
  border-left: 1px solid #dee2e6;
}
.gradingform_rubric .criterion .description {
  vertical-align: top;
  padding: 6px;
}
.gradingform_rubric .criterion .description textarea {
  margin-bottom: 0;
  height: 115px;
}
.gradingform_rubric .criterion .definition textarea {
  width: 80%;
  margin-bottom: 0;
}
.gradingform_rubric .criterion .score {
  margin-top: 5px;
  margin-right: 28px;
  font-style: italic;
  font-weight: bold;
  color: #2a6228;
}
.gradingform_rubric .criterion .score input {
  margin-bottom: 0;
}
.gradingform_rubric .criterion .level {
  vertical-align: top;
  padding: 6px;
}
.gradingform_rubric .criterion .level.currentchecked {
  background: #fff0f0;
}
.gradingform_rubric .criterion .level.checked {
  background: #d0ffd0;
  border: 1px solid #555;
}
.gradingform_rubric .criterion .level .delete {
  position: relative;
  width: 32px;
  height: 32px;
  margin-top: -32px;
  clear: both;
  float: right;
}
.gradingform_rubric .criterion .level .delete input {
  display: block;
  position: absolute;
  right: 0;
  bottom: 0;
  height: 24px;
  width: 24px;
  margin: 0;
}
.gradingform_rubric .criterion .level .delete input:hover {
  background-color: #ddd;
}
.gradingform_rubric .criterion .scorevalue input {
  float: none;
  width: 2em;
}
.gradingform_rubric .criterion .scorevalue input.hiddenelement, .gradingform_rubric .criterion .scorevalue input.pseudotablink {
  width: 0;
}
.gradingform_rubric .criterion .addlevel {
  vertical-align: top;
  padding-top: 6px;
}
.gradingform_rubric .criterion .addlevel input {
  height: 30px;
  line-height: 1rem;
}
.gradingform_rubric .addcriterion {
  margin-left: 5px;
  padding: 0;
  margin-bottom: 1em;
}
.gradingform_rubric .addcriterion input {
  margin: 0;
  color: inherit;
  text-shadow: inherit;
  border: 0 none;
  line-height: inherit;
  background: transparent url([[pix:t/add]]) no-repeat 7px 8px;
  padding-left: 26px;
}
.gradingform_rubric .options {
  clear: both;
}
.gradingform_rubric .options .option label {
  margin: 0;
  padding: 0;
  font-size: inherit;
  font-weight: normal;
  line-height: 2em;
  color: inherit;
  text-shadow: none;
  background-color: transparent;
}
.gradingform_rubric .options .option input {
  margin-left: 5px;
  margin-right: 12px;
}

.grade-display .description {
  font-size: 1rem;
}

.criterion .description {
  font-size: 1rem;
}
.criterion .criterion-toggle .expanded-icon {
  display: block;
}
.criterion .criterion-toggle .collapsed-icon {
  display: none;
}
.criterion .criterion-toggle.collapsed .expanded-icon {
  display: none;
}
.criterion .criterion-toggle.collapsed .collapsed-icon {
  display: block;
}

.path-grade-edit-tree .collapse-list .unlist {
  padding-left: 2rem;
}
.path-grade-edit-tree .collapse-list .unlist [data-for=sectionnode]:focus > .collapse-list-item:first-child {
  background-color: #e6f2f5;
  border-color: #b3d9e0;
}
.path-grade-edit-tree .collapse-list .unlist [data-for=sectionnode][data-selected=true] > .collapse-list-item:first-child {
  background-color: #e6f2f5;
  border-color: #b3d9e0;
  color: #0f6cbf;
}
.path-grade-edit-tree .collapse-list .unlist [data-for=sectionnode] .collapse-list-item-content[aria-hidden=true] {
  display: none;
}
.path-grade-edit-tree .collapse-list .unlist [data-for=sectionnode][aria-expanded=true] > .collapse-list-item .collapsed-icon {
  display: none;
}
.path-grade-edit-tree .collapse-list .unlist [data-for=sectionnode][aria-expanded=false] > .collapse-list-item .expanded-icon {
  display: none;
}
.path-grade-edit-tree .collapse-list .unlist .collapse-list-item {
  padding: 0.5rem 1rem;
  cursor: pointer;
}
.path-grade-edit-tree .collapse-list .unlist .collapse-list-item .collapse-list-item-name {
  font-weight: bold;
}
.path-grade-edit-tree .collapse-list .unlist .collapse-list-item .collapse-list-link {
  color: #1d2125;
  padding: 0 0.2rem;
  margin-right: 0.3rem;
}
.path-grade-edit-tree .collapse-list .unlist .collapse-list-item .collapse-list-link i {
  font-size: 12px;
  width: 12px;
  height: 12px;
  margin: 0;
}
.path-grade-edit-tree .gradetree-wrapper {
  padding: 10px 10px;
  background-color: #f8f9fa;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades h4, .path-grade-edit-tree .gradetree-wrapper .setup-grades .h4 {
  margin: 0;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades .column-rowspan {
  padding: 0;
  width: 24px;
  min-width: 24px;
  max-width: 24px;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades .emptyrow {
  display: none;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades .gradeitemdescription {
  font-weight: normal;
  padding-left: 24px;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.spacer {
  height: 0.5rem;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr[data-hidden=true] {
  display: none;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr th {
  vertical-align: bottom;
  border: none;
  text-align: left;
  background-color: #f8f9fa;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr th.rowspan {
  padding: 0;
  width: 24px;
  min-width: 24px;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr td {
  min-width: 4.5em;
  background-color: #f8f9fa;
  border: none;
  vertical-align: middle;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr td.column-name .small {
  font-size: 70%;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr td.column-name .itemselect {
  margin-right: 15px;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr td.column-weight {
  min-width: 15em;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr td.column-weight .weightoverride {
  margin-right: 5px;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr td.column-actions .dropdown-toggle::after {
  display: none;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr td.movehere {
  padding: 0;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr td.movehere a.movehere {
  display: block;
  width: 100%;
  margin: 5px 0 5px 0;
  padding: 3px 0 3px 0;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr td.movehere a.movehere hr {
  border-top: 2px dashed #8f959e;
  margin: 0;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr td.movehere a.movehere:hover hr {
  border-top: 2px dashed #0f6cbf;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.category td {
  background-color: #fff;
  border-top: 1px solid #dee2e6;
  border-bottom: 1px solid #dee2e6;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.category td:first-child {
  border-left: 1px solid #dee2e6;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.category td:last-child {
  border-right: 1px solid #dee2e6;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.category td.column-name {
  font-weight: bold;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.category td.column-name div {
  display: flex;
  min-height: 30px;
  align-items: center;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.category td.column-name div a.toggle-category {
  height: 24px;
  width: 24px;
  font-size: 12px;
  line-height: 24px;
  margin-right: 3px;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.category td.column-name div a.toggle-category[aria-expanded=true] .expanded, .path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.category td.column-name div a.toggle-category[aria-expanded=false] .collapsed {
  display: none;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.category td.column-name div a.toggle-category i {
  font-size: 12px;
  width: 12px;
  height: 12px;
  color: #1d2125;
  margin: 0;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.item td {
  background-color: #fff;
  border-top: 3px solid #f8f9fa;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.item.categoryitem td, .path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.item.courseitem td {
  min-width: 4.5em;
  background-color: #f8f9fa;
  border: none;
  vertical-align: middle;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.item.categoryitem td.column-name, .path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.item.courseitem td.column-name {
  padding-left: 0;
}
.path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.item.categoryitem td:not(.column-actions), .path-grade-edit-tree .gradetree-wrapper .setup-grades.table tr.item.courseitem td:not(.column-actions) {
  font-weight: bold;
}
.path-grade-edit-tree .gradetree-wrapper .badge-light {
  color: #1d2125;
  background-color: #ced4da;
  margin-right: 0.5em;
  margin-bottom: 0.5em;
}

/**
 * Grader report.
 */
.path-grade-report-grader .gradeparent tr .cell,
.path-grade-report-grader .gradeparent .floater .cell {
  background-color: var(--bs-body-bg);
}
.path-grade-report-grader .gradeparent tr .cell.gradecell .dropdown-menu.show,
.path-grade-report-grader .gradeparent .floater .cell.gradecell .dropdown-menu.show {
  z-index: 1;
}
.path-grade-report-grader .gradeparent table,
.path-grade-report-grader .gradeparent .cell {
  border-color: var(--bs-border-color);
}
.path-grade-report-grader .gradeparent .heading .cell,
.path-grade-report-grader .gradeparent .cell.category,
.path-grade-report-grader .gradeparent .avg .cell {
  background-color: #f8f9fa;
}
.path-grade-report-grader .gradeparent table .clickable {
  cursor: pointer;
}
.path-grade-report-grader .gradeparent tr.heading {
  position: sticky;
  top: 60px;
  z-index: 4;
}
.path-grade-report-grader .gradeparent tr.userrow th {
  z-index: 2;
}
.path-grade-report-grader .gradeparent tr.userrow th.actions-menu-active {
  z-index: 3;
}
.path-grade-report-grader .gradeparent tr.lastrow:not(.userrow) {
  position: sticky;
  bottom: -1px;
}
.path-grade-report-grader .gradeparent tr.lastrow:not(.userrow).pinned {
  z-index: 4;
}
.path-grade-report-grader .gradeparent tr.lastrow td,
.path-grade-report-grader .gradeparent tr.lastrow th {
  border-top: 1px solid var(--bs-border-color);
}
.path-grade-report-grader .gradeparent th.header {
  left: 0;
  position: sticky;
}
.path-grade-report-grader .gradeparent th.header#studentheader {
  z-index: 1;
}
.path-grade-report-grader .gradeparent td.noborder {
  border-right: transparent;
}
.path-grade-report-grader.hasstickyfooter .gradeparent tr.lastrow {
  bottom: calc(max(80px, 0.9375rem * 3) - 1px);
}

/**
 * User report.
 */
.path-grade-report-user .user-grade {
  border: none;
}
.path-grade-report-user .user-grade.table .levelodd {
  background-color: transparent;
}
.path-grade-report-user .user-grade.table .leveleven {
  background-color: var(--bs-body-bg);
}
.path-grade-report-user .user-grade .column-contributiontocoursetotal,
.path-grade-report-user .user-grade .column-range,
.path-grade-report-user .user-grade .column-percentage,
.path-grade-report-user .user-grade .column-weight {
  /*rtl:ignore*/
  direction: ltr;
}

/**
 * Single view.
 */
.path-grade-report-singleview .reporttable input[name^=finalgrade] {
  width: 80px;
  display: inline-block;
}
.path-grade-report-singleview .reporttable .action-menu {
  display: inline-block;
  margin-left: 0.5rem;
  float: right;
}
.path-grade-report-singleview .reporttable .dropdown-toggle::after {
  display: none;
}

.gradereport-grader-table input[name^=grade] {
  width: 80px;
  display: inline-block;
}
.gradereport-grader-table .dropdown-toggle::after {
  display: none;
}

.search-widget .dropdown-menu {
  padding: 0.8rem 1.2rem;
}
.search-widget .dropdown-menu.wide {
  width: 350px;
}
.search-widget .dropdown-menu.narrow {
  width: 250px;
}
.search-widget .dropdown-menu .dropdown-item span.email {
  color: var(--bs-secondary-color);
}
.search-widget .dropdown-menu .dropdown-item:hover span,
.search-widget .dropdown-menu .dropdown-item:active span {
  color: #fff;
}
.search-widget .dropdown-menu .searchresultscontainer {
  height: 178px;
  font-size: 90%;
}
.search-widget .dropdown-menu .searchresultscontainer .searchresultitemscontainer {
  height: 178px;
  max-height: 178px;
  overflow: auto;
}
.search-widget .dropdown-menu .unsearchablecontentcontainer {
  border-top: 1px solid #dee2e6;
  padding-top: 10px;
  font-size: 90%;
}

#fitem_id_submitbutton {
  padding-right: 2em;
}

.gradestatus {
  padding-top: 10px;
}

.gradestatus .icon {
  margin-right: 1rem;
}

.columns-autoflow-1to1to1 {
  column-count: 3;
}

@media (max-width: 767px) {
  .columns-autoflow-1to1to1 {
    column-count: 1;
  }
}
/* some very targetted corrections to roll back nameclashes between
 * Moodle and Bootstrap like .row, .label, .content, .controls
 *
 * Mostly relies on these styles being more specific than the Bootstrap
 * ones in order to overule them.
 */
li.activity.label,
.file-picker td.label {
  background: inherit;
  color: inherit;
  text-shadow: none;
  white-space: normal;
  display: block;
  font-size: inherit;
  line-height: inherit;
  text-align: inherit;
}

.file-picker td.label {
  border: inherit;
  display: table-cell;
  text-align: right;
  padding: 8px;
}

.choosercontainer #chooseform .option {
  font-size: 12px;
}

/* block.invisible vs .invisible
 * block.hidden vs .invisible
 *
 * uses .invisible where the rest of Moodle uses @mixin dimmed
 * fixible in block renderer?
 *
 * There's seems to be even more naming confusion here since,
 * blocks can be actually 'visible' (or not) to students,
 * marked 'visible' but really just dimmed to indicate to editors
 * that students can't see them or  'visible' to the user who
 * collapses them, 'visible' if you have the right role and in
 * different circumstances different sections of a block can
 * be 'visible' or not.
 *
 * currently worked around in renderers.php function block{}
 * by rewriting the class name "invisible" to "dimmed",
 * though the blocks don't look particularly different apart
 * from their contents disappearing. Maybe try .muted? or
 * dimming all the edit icons apart from unhide, might be a
 * nice effect, though they'd still be active. Maybe reverse
 * it to white?
 */
.section.hidden,
.block.hidden,
.block.invisible {
  visibility: visible;
  display: block;
}

/* .row vs .row
 *
 * very tricky to track down this when it goes wrong,
 * since the styles are applied to generated content
 *
 * basically if you see things shifted left or right compared
 * with where they should be check for a .row
 */
.forumpost .row {
  margin-left: 0 !important; /* stylelint-disable-line declaration-no-important */
}

.forumpost .row:before,
.forumpost .row:after {
  content: none;
}

/* fieldset.hidden vs .hidden
 *
 * Moodle uses fieldset.hidden for mforms, to signify a collection of
 * form elements that don't have a box drawn round them. Bootstrap
 * uses hidden for stuff that is hidden in various responsive modes.
 *
 * Relatedly, there is also fieldset.invisiblefieldset which hides the
 * border and sets the display to inline.
 *
 * Originally this just set block and visible, but it is used
 * in random question dialogue in Quiz,
 * that dialogue is hidden and shown, so when hidden the
 * above workaround leaves you with a button floating around
 */
fieldset.hidden {
  display: inherit;
  visibility: inherit;
}

/* .container vs .container
 *
 * bootstrap uses .container to set the width of the layout at 960px or so, Moodle uses it
 * in the Quiz to contain the questions to add. If you don't overule the Bootstrap code,
 * it becomes near unuseable.
 */
#questionbank + .container {
  width: auto;
}

body:not(.jsenabled) .dropdown:hover > .dropdown-menu {
  display: block;
  margin-top: -6px;
}

body:not(.jsenabled) .langmenu:hover > .dropdown-menu,
.langmenu.open > .dropdown-menu {
  display: block;
  max-height: 150px;
  overflow-y: auto;
}

.navbar.fixed-top .dropdown .dropdown-menu {
  max-height: calc(100vh - 60px);
  overflow-y: auto;
}

.page-item.active .page-link, .page-item.active .page-link:hover, .page-item.active .page-link:focus {
  z-index: inherit;
}

.form-select {
  word-wrap: normal;
}

/**
 * Reset all of the forced style on the page.
 * - Remove borders on header and content.
 * - Remove most of the vertical padding.
 * - Make the content region flex grow so it pushes things like the
 *   next activity selector to the bottom of the page.
 */
body.reset-style #page-header .card {
  border: none;
}
body.reset-style #page-header .card .page-header-headings h1, body.reset-style #page-header .card .page-header-headings .h1 {
  margin-bottom: 0;
}
@media (max-width: 767.98px) {
  body.reset-style #page-header .card .card-body {
    padding-left: 0;
    padding-right: 0;
  }
}
body.reset-style #page-header > div {
  padding-top: 0 !important; /* stylelint-disable-line declaration-no-important */
  padding-bottom: 0 !important; /* stylelint-disable-line declaration-no-important */
}
body.reset-style #page-content {
  padding-bottom: 0 !important; /* stylelint-disable-line declaration-no-important */
}
body.reset-style #page-content #region-main-box #region-main {
  border: none;
  display: inline-flex;
  flex-direction: column;
  padding: 0;
  height: 100%;
  width: 100%;
  padding-left: 1rem;
  padding-right: 1rem;
  vertical-align: top;
}
body.reset-style #page-content #region-main-box #region-main div[role=main] {
  flex: 1 0 auto;
}
body.reset-style #page-content #region-main-box #region-main .activity-navigation {
  overflow: hidden;
}
body.reset-style #page-content #region-main-box #region-main.has-blocks {
  width: calc(100% - calc(360px + (1.5rem * 0.5)));
}
@media (max-width: 1199.98px) {
  body.reset-style #page-content #region-main-box #region-main.has-blocks {
    width: 100%;
  }
}
@media (max-width: 767.98px) {
  body.reset-style #page-content #region-main-box #region-main {
    padding-left: 0;
    padding-right: 0;
  }
}
body.reset-style #page-content #region-main-box [data-region=blocks-column] {
  margin-left: auto;
}
@media (max-width: 1199.98px) {
  body.reset-style #page-content #region-main-box {
    display: flex;
    flex-direction: column;
  }
}
body.reset-style select,
body.reset-style input,
body.reset-style textarea,
body.reset-style .btn:not(.btn-icon) {
  border-radius: 0.6rem;
}

/**
 * Reset the Bootstrap _reboot.scss styles for tables.
 * That reboot mainly affects tables added by the tiny editor.
 * If a table should continue using the original _reboot.scss styles, add the .table-reboot (styleless) class to opt out
 * of this reset.
 */
table:not(.table):not(.ygtvtable):not(.table-reboot):not(.yui3-datatable-table) thead,
table:not(.table):not(.ygtvtable):not(.table-reboot):not(.yui3-datatable-table) tbody,
table:not(.table):not(.ygtvtable):not(.table-reboot):not(.yui3-datatable-table) tfoot,
table:not(.table):not(.ygtvtable):not(.table-reboot):not(.yui3-datatable-table) tr,
table:not(.table):not(.ygtvtable):not(.table-reboot):not(.yui3-datatable-table) td,
table:not(.table):not(.ygtvtable):not(.table-reboot):not(.yui3-datatable-table) th {
  border-color: var(--bs-border-color);
  border-style: solid;
  border-width: var(--bs-border-width);
}

body.behat-site .fixed-top {
  position: absolute;
}
body.behat-site.hasstickyfooter .stickyfooter,
body.behat-site .stickyfooter {
  position: inherit;
  z-index: inherit;
}
body.behat-site.hasstickyfooter .stickyfooter .sticky-footer-content-wrapper,
body.behat-site .stickyfooter .sticky-footer-content-wrapper {
  padding: 0;
  margin: 0;
}
body.behat-site .dropdown-item {
  margin-top: 4px !important; /* stylelint-disable declaration-no-important */
}
body.behat-site.drawer-ease {
  -webkit-transition: initial;
  -moz-transition: initial;
  transition: initial;
}
body.behat-site [data-region=drawer] {
  -webkit-transition: initial;
  -moz-transition: initial;
  transition: initial;
  position: absolute;
}
body.behat-site [data-region=message-drawer] {
  padding-right: 10px;
}
body.behat-site.jsenabled #page-footer .footer-content-popover {
  display: block;
}
body.behat-site.path-grade-report-grader .gradeparent tr.heading,
body.behat-site.path-grade-report-grader .gradeparent tr.lastrow,
body.behat-site.path-grade-report-grader .gradeparent th.header {
  position: relative;
  left: auto;
}
body.behat-site.path-grade-report-grader .gradeparent tr.heading {
  top: auto;
}

.phpinfo table,
.phpinfo th,
.phpinfo h2,
.phpinfo .h2 {
  margin: auto;
}

.phpinfo .e,
.phpinfo .v,
.phpinfo .h {
  border: 1px solid #000;
  font-size: 0.8em;
  vertical-align: baseline;
  color: #000;
  background-color: #ccc;
}

.phpinfo .e {
  background-color: #ccf;
  font-weight: bold;
}

.phpinfo .h {
  background-color: #99c;
  font-weight: bold;
}

body > .debuggingmessage {
  margin-top: 60px;
}

body > .debuggingmessage ~ .debuggingmessage {
  margin-top: 0.5rem;
}

/**
 * This file contains the styles required to make the footer sticky.
 */
html,
body {
  height: 100%;
}

.stickyfooter {
  position: fixed;
  right: 0;
  left: 0;
  height: max(80px, 0.9375rem * 3);
  bottom: calc(max(80px, 0.9375rem * 3) * -1);
  transition: bottom 0.5s;
  z-index: 1020;
  box-shadow: 0 0 1rem rgba(0, 0, 0, 0.15);
  font-size: calc(0.9375rem * 1.10);
}
.stickyfooter .sticky-footer-content {
  overflow: hidden;
}
@media (min-width: 768px) {
  .pagelayout-standard .stickyfooter .sticky-footer-content, body.limitedwidth.uses-drawers .stickyfooter .sticky-footer-content {
    max-width: 830px;
  }
}
.stickyfooter .sticky-footer-content-wrapper {
  transition: 0.2s;
}
@media (prefers-reduced-motion: reduce) {
  .stickyfooter .sticky-footer-content-wrapper {
    transition: none;
  }
}
@media (min-width: 992px) {
  .stickyfooter .sticky-footer-content-wrapper .drawers {
    padding: 0 3rem;
  }
  .show-drawer-left .stickyfooter .sticky-footer-content-wrapper {
    margin-left: 285px;
    padding: 0 3rem 0 1rem;
  }
  .show-drawer-right .stickyfooter .sticky-footer-content-wrapper {
    margin-right: 315px;
    padding: 0 1rem 0 3rem;
  }
  .show-drawer-right.show-drawer-left .stickyfooter .sticky-footer-content-wrapper {
    padding: 0 1rem;
  }
}

.hasstickyfooter .stickyfooter {
  bottom: 0;
}

/* Standard components fixes for sticky footer. */
.stickyfooter ul.pagination {
  margin-bottom: 0.25rem;
}

.stickyfooter .btn {
  font-size: calc(0.9375rem * 1.10);
}

/* Breakpoints fixes. */
@media (min-width: 576px) {
  #page-wrapper {
    height: 100%;
    display: flex;
    flex-direction: column;
  }
  #page-wrapper #page {
    display: flex;
    flex-direction: column;
  }
  #page-wrapper #page:not(.drawers) {
    flex: 1 0 auto;
  }
  #page-wrapper #page #page-content {
    flex: 1 0 auto;
  }
  #page-wrapper #page-footer {
    flex-shrink: 0;
  }
}
@media (max-width: 767.98px) {
  #page-wrapper {
    height: 100%;
    display: flex;
    flex-direction: column;
  }
  #page-wrapper #page {
    display: flex;
    flex-direction: column;
  }
  #page-wrapper #page:not(.drawers) {
    flex: 1 0 auto;
  }
}
.popover-region {
  position: relative;
}
.popover-region.collapsed .popover-region-toggle:before, .popover-region.collapsed .popover-region-toggle:after {
  display: none;
}
.popover-region.collapsed .popover-region-container {
  opacity: 0;
  visibility: hidden;
  height: 0;
  overflow: hidden;
  transition: height 0.25s, opacity 101ms 0.25s, visibility 101ms 0.25s;
}

.popover-region-toggle {
  cursor: pointer;
}
.popover-region-toggle::before {
  content: "";
  display: inline-block;
  border-left: 10px solid transparent;
  border-right: 10px solid transparent;
  border-bottom: 10px solid #ddd;
  position: absolute;
  bottom: 0;
  right: 7px;
}
.popover-region-toggle::after {
  content: "";
  display: inline-block;
  border-left: 9px solid transparent;
  border-right: 9px solid transparent;
  border-bottom: 9px solid #fff;
  position: absolute;
  bottom: -1px;
  right: 8px;
  z-index: 2;
}

.count-container {
  padding: 2px;
  border-radius: 2px;
  background-color: #ca3120;
  color: #fff;
  font-size: 11px;
  line-height: 11px;
  position: absolute;
  top: 5px;
  right: 0;
}

.popover-region-container {
  opacity: 1;
  visibility: visible;
  position: absolute;
  right: 0;
  top: 0;
  height: 500px;
  max-height: 80vh;
  width: 380px;
  border: 1px solid #ddd;
  transition: height 0.25s;
  background-color: #fff;
  z-index: 1;
}

.popover-region-header-container {
  height: 25px;
  line-height: 25px;
  padding-left: 5px;
  padding-right: 5px;
  border-bottom: 1px solid #ddd;
  box-sizing: border-box;
}

.popover-region-footer-container {
  height: 30px;
  text-align: center;
  border-top: 1px solid #ddd;
  background-color: var(--bs-body-bg);
  padding-top: 3px;
}

.popover-region-header-text {
  float: left;
  margin: 0;
  font-size: 14px;
  line-height: 25px;
}

.popover-region-header-actions {
  float: right;
}
.popover-region-header-actions > * {
  margin-left: 10px;
  min-width: 20px;
  display: inline-block;
}
.popover-region-header-actions .loading-icon {
  display: none;
  height: 12px;
  width: 12px;
}
.popover-region-header-actions .newmessage-link {
  margin-right: 10px;
}
.popover-region-header-actions label {
  display: inline-block;
  text-align: center;
  margin-bottom: 0;
}

.popover-region-content-container {
  height: calc(100% - 55px);
  width: 100%;
  overflow-y: auto;
  -webkit-overflow-scrolling: touch;
}
.popover-region-content-container > .loading-icon {
  display: none;
  text-align: center;
  padding: 5px;
  box-sizing: border-box;
}
.popover-region-content-container .empty-message {
  display: none;
  text-align: center;
  padding: 10px;
}
.popover-region-content-container.loading > .loading-icon {
  display: block;
}
.popover-region-content-container.loading .empty-message {
  display: none;
}

.navbar-nav .popover-region .icon {
  font-weight: bolder;
}

.navbar .popover-region.collapsed .popover-region-container {
  opacity: 0;
  visibility: hidden;
  height: 0;
  overflow: hidden;
  transition: height 0.25s, opacity 101ms 0.25s, visibility 101ms 0.25s;
}
.navbar .count-container {
  padding: 2px;
  border-radius: 2px;
  background-color: #ca3120;
  color: #fff;
  font-size: 11px;
  line-height: 11px;
  position: absolute;
  top: 15px;
  right: 0;
}
.navbar .popover-region-container {
  top: 60px;
}

.content-item-container {
  width: 100%;
  border-bottom: 1px solid #ddd;
  box-sizing: border-box;
  padding: 5px;
  position: relative;
  margin: 0;
  display: block;
  color: inherit;
  text-decoration: none;
}
.content-item-container:hover {
  color: #fff;
  background-color: #0f6cbf;
}
.content-item-container:hover .content-item-footer .timestamp {
  color: #fff;
}
.content-item-container:hover .view-more {
  color: inherit;
}
.content-item-container.unread {
  margin: 0;
  background-color: #f4f4f4;
}
.content-item-container.unread:hover {
  color: #fff;
  background-color: #0f6cbf;
}
.content-item-container.unread .content-item-body .notification-message {
  font-weight: 600;
}
.content-item-container .context-link {
  color: inherit;
  text-decoration: none;
}
.content-item-container .content-item-body {
  box-sizing: border-box;
  margin-bottom: 5px;
}
.content-item-container .content-item-footer {
  text-align: left;
  box-sizing: border-box;
}
.content-item-container .content-item-footer .timestamp {
  font-size: 10px;
  line-height: 10px;
  margin: 0;
  color: inherit;
}
.content-item-container .view-more {
  position: absolute;
  bottom: 5px;
  right: 5px;
  font-size: 12px;
  line-height: 12px;
}
.content-item-container .view-more:hover {
  color: inherit;
}
.content-item-container.notification .content-item-body .notification-image {
  display: inline-block;
  width: 24px;
  height: 24px;
  float: left;
}
.content-item-container.notification .content-item-body .notification-image img {
  height: 75%;
}
.content-item-container.notification .content-item-body .notification-message {
  display: inline-block;
  font-size: 12px;
  width: calc(100% - 24px);
}
.content-item-container.selected {
  background-color: #3279b3;
  color: #fff;
  border-color: #3279b3;
}
.content-item-container.selected .content-item-footer .timestamp {
  color: #fff;
}

.popover-region-notifications .popover-region-header-container .mark-all-read-button .normal-icon {
  display: inline-block;
}
.popover-region-notifications .popover-region-header-container .mark-all-read-button.loading .normal-icon {
  display: none;
}
.popover-region-notifications .popover-region-header-container .mark-all-read-button.loading .loading-icon {
  display: inline-block;
}
.popover-region-notifications .all-notifications {
  opacity: 1;
  visibility: visible;
  height: auto;
  overflow: hidden;
}
.popover-region-notifications .all-notifications:empty + .empty-message {
  display: block;
}
.popover-region-notifications .notification-image {
  display: inline-block;
  width: 8%;
  vertical-align: top;
}
.popover-region-notifications .notification-image img {
  height: 75%;
}
.popover-region-notifications .notification-message {
  display: inline-block;
  font-size: 12px;
}
.popover-region-notifications .popover-region-content-container.loading .all-notifications:empty + .empty-message {
  display: none;
}

.popover-region-messages .mark-all-read-button .normal-icon {
  display: inline-block;
}
.popover-region-messages .mark-all-read-button.loading .normal-icon {
  display: none;
}
.popover-region-messages .mark-all-read-button.loading .loading-icon {
  display: inline-block;
}
.popover-region-messages .popover-region-content-container.loading .popover-region-content .messages:empty + .empty-message {
  display: none;
}
.popover-region-messages .messages:empty + .empty-message {
  display: block;
}
.popover-region-messages .content-item-container.unread .content-item-body {
  font-weight: 600;
  width: calc(90% - 30px);
}
.popover-region-messages .content-item-container.unread .unread-count-container {
  display: inline-block;
  width: 10%;
  text-align: center;
  float: right;
}
.popover-region-messages .content-item {
  height: 100%;
  width: 100%;
  box-sizing: border-box;
}
.popover-region-messages .profile-image-container {
  width: 30px;
  display: inline-block;
  text-align: center;
  float: left;
}
.popover-region-messages .profile-image-container img {
  width: 100%;
  display: inline-block;
  vertical-align: middle;
  border-radius: 50%;
}
.popover-region-messages .content-item-body {
  display: inline-block;
  box-sizing: border-box;
  width: calc(100% - 30px);
  font-size: 12px;
  padding-left: 10px;
  overflow: hidden;
}
.popover-region-messages .content-item-body h3, .popover-region-messages .content-item-body .h3 {
  font-size: 12px;
  line-height: 12px;
  margin: 0;
  width: 100%;
}
.popover-region-messages .content-item-body p {
  margin: 0;
}
.popover-region-messages .unread-count-container {
  display: none;
}

@media (max-width: 767px) {
  .navbar .popover-region .popover-region-container {
    right: -70px;
  }
}
@media (max-width: 480px) {
  .navbar .popover-region .popover-region-container {
    position: fixed;
    top: 46px;
    right: 0;
    left: 0;
    bottom: 0;
    width: auto;
    height: auto;
  }
}
/**
 * Tour step must sit above all other UI components.
 * The backdrop is the lowest point in the tour.
 * ----- moodle
 *  ---- step backdrop
 *   --- step container
 */
div[data-flexitour=backdrop] {
  background-color: #000;
  opacity: 0.5;
  z-index: 1040;
}

span[data-flexitour=container] {
  z-index: 1041;
}

span[data-flexitour=container] .modal {
  display: initial;
  position: initial;
}
span[data-flexitour=container] .modal-dialog {
  /**
   * Remove all margins to:
   * 1) ensure that the arrow touches the target; and
   * 2) ensure that the focus border touches the modal.
   */
  margin: 0;
}
span[data-flexitour=container] div[data-role=arrow] {
  border-width: 1rem;
}
span[data-flexitour=container] div[data-role=arrow],
span[data-flexitour=container] div[data-role=arrow]:after {
  position: absolute;
  display: block;
  width: 0;
  height: 0;
  border-color: transparent;
  border-style: solid;
  border-width: 1rem;
}
span[data-flexitour=container][x-placement=top], span[data-flexitour=container][x-placement=top-start] {
  margin-bottom: 1rem;
}
span[data-flexitour=container][x-placement=top] div[data-role=arrow], span[data-flexitour=container][x-placement=top-start] div[data-role=arrow] {
  bottom: -1rem;
  left: 50%;
  margin-left: -1rem;
  border-bottom-width: 0;
  border-top-color: var(--bs-border-color-translucent);
}
span[data-flexitour=container][x-placement=top] div[data-role=arrow]:after, span[data-flexitour=container][x-placement=top-start] div[data-role=arrow]:after {
  bottom: 1px;
  margin-left: -1rem;
  content: " ";
  border-bottom-width: 0;
  border-top-color: var(--bs-body-bg);
}
span[data-flexitour=container][x-placement=bottom], span[data-flexitour=container][x-placement=bottom-start] {
  margin-top: 1rem;
}
span[data-flexitour=container][x-placement=bottom] div[data-role=arrow], span[data-flexitour=container][x-placement=bottom-start] div[data-role=arrow] {
  top: -1rem;
  left: 50%;
  margin-left: -1rem;
  border-top-width: 0;
  border-bottom-color: var(--bs-border-color-translucent);
}
span[data-flexitour=container][x-placement=bottom] div[data-role=arrow]:after, span[data-flexitour=container][x-placement=bottom-start] div[data-role=arrow]:after {
  top: 1px;
  margin-left: -1rem;
  content: " ";
  border-top-width: 0;
  border-bottom-color: var(--bs-body-bg);
}
span[data-flexitour=container][x-placement=left], span[data-flexitour=container][x-placement=left-start] {
  margin-right: 1rem;
}
span[data-flexitour=container][x-placement=left] div[data-role=arrow], span[data-flexitour=container][x-placement=left-start] div[data-role=arrow] {
  right: -1rem;
  top: 50%;
  margin-top: -1rem;
  border-right-width: 0;
  border-left-color: var(--bs-border-color-translucent);
}
span[data-flexitour=container][x-placement=left] div[data-role=arrow]:after, span[data-flexitour=container][x-placement=left-start] div[data-role=arrow]:after {
  right: 1px;
  margin-top: -1rem;
  content: " ";
  border-right-width: 0;
  border-left-color: var(--bs-body-bg);
}
span[data-flexitour=container][x-placement=right], span[data-flexitour=container][x-placement=right-start] {
  margin-left: 1rem;
}
span[data-flexitour=container][x-placement=right] div[data-role=arrow], span[data-flexitour=container][x-placement=right-start] div[data-role=arrow] {
  left: -1rem;
  top: 50%;
  margin-top: -1rem;
  border-left-width: 0;
  border-right-color: var(--bs-border-color-translucent);
}
span[data-flexitour=container][x-placement=right] div[data-role=arrow]:after, span[data-flexitour=container][x-placement=right-start] div[data-role=arrow]:after {
  left: 1px;
  margin-top: -1rem;
  content: " ";
  border-left-width: 0;
  border-right-color: var(--bs-body-bg);
}
.dir-rtl span[data-flexitour=container][x-placement^=right] {
  margin-left: 1rem;
}
.dir-rtl span[data-flexitour=container][x-placement^=right] div[data-role=arrow] {
  transform: rotate(180deg);
  left: -1rem;
  right: auto;
}
.dir-rtl span[data-flexitour=container][x-placement^=left] {
  margin-right: 1rem;
}
.dir-rtl span[data-flexitour=container][x-placement^=left] div[data-role=arrow] {
  transform: rotate(180deg);
  left: auto;
  right: -1rem;
}

[data-region=drawer] [data-flexitour=container] {
  /*rtl:ignore*/
  margin-left: -15px;
  width: 275px;
}

@media print {
  body.drawer-open-left.jsenabled,
  body.drawer-open-right.jsenabled {
    margin: 0;
  }
  .container {
    width: auto;
  }
  .navbar {
    display: none;
  }
  .tertiary-navigation {
    display: none;
  }
}
.modal .modal-body > .loading-icon {
  display: block;
  position: relative;
  width: 100%;
  height: 100%;
}
.modal .modal-body > .loading-icon .icon {
  position: absolute;
  top: 50%;
  /*rtl:ignore*/
  left: 50%;
  transform: translate(-50%, -50%);
}
.modal .btn-close {
  margin: -0.4rem -0.4rem -0.4rem auto;
}
.modal .btn-close:not(:disabled):not(.disabled):hover, .modal .btn-close:not(:disabled):not(.disabled):focus {
  opacity: inherit;
}

/* Change the modal-dialog-scrollable class to position: static to fix the issue with popper.js and autocomplete. */
.modal-dialog-scrollable:has(.form-autocomplete-suggestions) {
  position: static;
}
.modal-dialog-scrollable:has(.form-autocomplete-suggestions) .modal-content {
  position: static;
}
.modal-dialog-scrollable:has(.form-autocomplete-suggestions) .modal-content .modal-body {
  position: static;
}

/* Bug fix for TinyMCE menu when in fullscreen mode. */
body.tox-fullscreen .modal-dialog {
  width: 100%;
  max-width: 100%;
  height: 100%;
}
body.tox-fullscreen .modal-dialog .modal-header {
  height: 0;
  padding: 0;
}
body.tox-fullscreen .modal-dialog .tox-tinymce-aux {
  position: relative;
}

/* Bug fix TinyMCE menu width dropdowns when in fullscreen mode. */
body.tox-fullscreen .tox.tox-tinymce-aux {
  position: relative !important;
}

.layout.fullscreen {
  height: 100vh;
  position: fixed;
  top: 0;
  left: 0;
  z-index: 1050;
  transition: 0.5s;
  width: 100vw;
  margin: 0;
  opacity: 1;
  background-color: var(--bs-body-bg);
}
@media (prefers-reduced-motion: reduce) {
  .layout.fullscreen {
    transition: none;
  }
}
.layout.fullscreen > div {
  height: 100%;
  width: 100%;
}
.layout.fullscreen .loading-icon {
  margin-left: auto;
  margin-right: auto;
  text-align: center;
  display: inline-block;
  width: 100%;
  top: 40%;
  position: fixed;
}
.layout.fullscreen .loading-icon .icon {
  max-width: 4em;
  max-height: 4em;
  font-size: 4em;
}

#page.drawers {
  margin-top: 60px;
  scrollbar-width: thin;
  scrollbar-color: #6a737b #f8f9fa;
}
#page.drawers::-webkit-scrollbar {
  width: 12px;
}
#page.drawers::-webkit-scrollbar-track {
  background: #f8f9fa;
}
#page.drawers::-webkit-scrollbar-thumb {
  background-color: #6a737b;
  border-radius: 20px;
  border: 3px solid #f8f9fa;
}
#page.drawers::-webkit-scrollbar-thumb:hover {
  background-color: #495057;
}
#page.drawers .main-inner {
  max-width: 100%;
  width: 100%;
  margin: 0 auto;
  border-radius: 0.5rem;
  background-color: #fff;
  padding: 1.5rem 0.5rem;
  margin-top: 0.5rem;
  margin-bottom: 3rem;
  flex: 1 0 auto;
}
#page.drawers .activity-header {
  margin-left: 15px;
  margin-right: 15px;
}

@media (min-width: 768px) {
  .pagelayout-standard #page.drawers .main-inner,
  body.limitedwidth #page.drawers .main-inner {
    max-width: 830px;
  }
  .pagelayout-standard #page.drawers .footer-popover,
  body.limitedwidth #page.drawers .footer-popover {
    max-width: 830px;
    width: 100%;
    margin: 0 auto;
    border-radius: 0.5rem;
  }
  .header-maxwidth {
    max-width: 830px;
    margin: 0 auto;
    padding-left: 15px;
    padding-right: 15px;
  }
  .header-maxwidth .header-inner {
    padding-left: 0;
    padding-right: 0;
  }
}
.drawer-toggles .drawer-toggler {
  position: fixed;
  top: calc(60px + 0.7rem);
  z-index: 2;
}
.drawer-toggles .drawer-toggler .btn {
  border-radius: 200px;
  padding: 16px;
  background-color: #dee2e6;
  box-shadow: 0 0.125rem 0.25rem rgba(0, 0, 0, 0.075);
  transition: padding 200ms;
}
.drawer-toggles .drawer-toggler .btn .icon {
  width: auto;
  height: auto;
}
.drawer-toggles .drawer-toggler .btn:focus {
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.drawer-toggles .drawer-left-toggle {
  left: 0;
}
.drawer-toggles .drawer-left-toggle .btn {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
  padding-right: 14px;
  padding-left: 10px;
}
.drawer-toggles .drawer-left-toggle .btn:hover {
  padding-left: 20px;
}
.drawer-toggles .drawer-right-toggle {
  right: 0;
}
.drawer-toggles .drawer-right-toggle .btn {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
  padding-right: 10px;
  padding-left: 14px;
}
.drawer-toggles .drawer-right-toggle .btn:hover {
  padding-right: 20px;
}

#page.drawers.show-drawer-left .drawer-left-toggle {
  display: none;
}

#page.drawers.show-drawer-right .drawer-right-toggle {
  display: none;
}

@media (max-width: 767.98px) {
  .drawer-toggles {
    z-index: 100;
  }
  .drawer-toggles .drawer-right-toggle,
  .drawer-toggles .drawer-left-toggle {
    top: calc(99vh - (60px * 2.5));
  }
  #page.drawers.scroll-down .drawer-right-toggle {
    transform: translateX(150%);
    pointer-events: auto;
    visibility: hidden;
  }
  #page.drawers.scroll-down .drawer-left-toggle {
    transform: translateX(-150%);
    pointer-events: auto;
    visibility: hidden;
  }
  .hasstickyfooter:has(.drawer-right-toggle) .maintenancewarning {
    right: 2rem;
  }
  .hasstickyfooter:has(.drawer-left-toggle) .maintenancewarning {
    margin-left: 3.5rem;
  }
}
@media (min-width: 576px) {
  #page.drawers .main-inner {
    margin-top: 1.5rem;
  }
}
@media (min-width: 768px) {
  #page.drawers {
    padding-left: 3rem;
    padding-right: 3rem;
  }
  #page.drawers .main-inner {
    padding: 1.5rem 0.5rem;
  }
  #page.drawers div[role=main] {
    padding-left: 15px;
    padding-right: 15px;
  }
}
@media (min-width: 992px) {
  .drawer-left,
  .drawer-right {
    top: 60px;
    height: calc(100vh - 60px);
  }
  .hasstickyfooter .drawer-left,
  .hasstickyfooter .drawer-right {
    top: 60px;
    height: calc(100vh - 60px - max(80px, 0.9375rem * 3));
  }
  #page.drawers {
    position: relative;
    overflow-y: visible;
    transition: 0.2s;
    left: 0;
    right: 0;
  }
}
@media (min-width: 992px) and (prefers-reduced-motion: reduce) {
  #page.drawers {
    transition: none;
  }
}
@media (min-width: 992px) {
  #page.drawers.show-drawer-left {
    margin-left: 285px;
    margin-right: 0;
    padding-left: 1rem;
  }
}
@media (min-width: 992px) {
  #page.drawers.show-drawer-right {
    margin-left: 0;
    margin-right: 315px;
    padding-right: 1rem;
  }
  .jsenabled #page.drawers.show-drawer-right .popover-process-monitor, .jsenabled #page.drawers.show-drawer-right .btn-footer-popover, .jsenabled #page.drawers.show-drawer-right .btn-footer-communication {
    right: calc(315px + 2rem);
  }
  .jsenabled:has(#page.drawers.show-drawer-right) .maintenancewarning {
    right: calc(315px + 5rem);
  }
}
@media (min-width: 992px) {
  #page.drawers.show-drawer-left.show-drawer-right {
    margin-left: 285px;
    margin-right: 315px;
  }
}
@media (min-width: 992px) {
  #page.drawers.hasstickyfooter {
    margin-bottom: max(80px, 0.9375rem * 3);
  }
}
.drawercontrolbuttons {
  margin-top: 92px;
}
.drawercontrolbuttons .buttons {
  z-index: 1;
}

.form-control:-ms-input-placeholder {
  color: var(--bs-secondary-color);
}

.form-select {
  -webkit-appearance: none;
  -moz-appearance: none;
}

.form-range {
  -webkit-appearance: none;
  -moz-appearance: none;
}
.form-range::-webkit-slider-thumb, .form-range::-moz-range-thumb, .form-range::-ms-thumb {
  -webkit-appearance: none;
  -moz-appearance: none;
}

input[type=date].form-control,
input[type=time].form-control,
input[type=datetime-local].form-control,
input[type=month].form-control {
  -webkit-appearance: none;
  -moz-appearance: none;
}

@media (min-width: 576px) {
  .card-columns {
    -webkit-column-gap: 1.25rem;
    -moz-column-gap: 1.25rem;
  }
}

.carousel-item {
  -webkit-backface-visibility: hidden;
}

.card {
  -webkit-background-clip: border-box;
}

.carousel-indicators li,
.dropdown-menu,
.form-control,
.modal-content,
.popover,
.toast {
  -webkit-background-clip: padding-box;
}

.btn {
  -webkit-user-select: none;
  -ms-user-select: none;
}

.user-select-all {
  -webkit-user-select: all !important; /* stylelint-disable-line declaration-no-important */
  -ms-user-select: none;
}

.user-select-auto {
  -webkit-user-select: auto !important; /* stylelint-disable-line declaration-no-important */
  -ms-user-select: none;
}

.user-select-none {
  -webkit-user-select: none !important; /* stylelint-disable-line declaration-no-important */
  -ms-user-select: none;
}

.editor_atto_content_wrap {
  background-color: #fff;
  color: #333;
}

.editor_atto_content {
  padding: 4px;
  resize: vertical;
  overflow: auto;
}

.editor_atto_content_wrap,
.editor_atto + textarea {
  width: 100%;
  padding: 0;
}

.editor_atto + textarea {
  border-radius: 0;
  resize: vertical;
  margin-top: -1px;
}

div.editor_atto_toolbar {
  display: block;
  background: #f2f2f2;
  min-height: 35px;
  border: 1px solid var(--bs-gray-500);
  width: 100%;
  padding: 0 0 9px 0;
  border-top-left-radius: 0.5rem;
  border-top-right-radius: 0.5rem;
}

div.editor_atto_toolbar button {
  padding: 4px 9px;
  background: none;
  border: 0;
  margin: 0;
  border-radius: 0;
  cursor: pointer;
}

div.editor_atto_toolbar .menuplaceholder {
  display: inline-block;
}

div.editor_atto_toolbar button + button,
div.editor_atto_toolbar .menuplaceholder + button {
  border-left: 1px solid #ccc;
}

div.editor_atto_toolbar button[disabled] {
  opacity: 0.45;
  background: none;
  cursor: default;
}

.editor_atto_toolbar button:hover {
  background-image: radial-gradient(ellipse at center, #fff 60%, #dfdfdf 100%);
  background-color: #ebebeb;
}

.editor_atto_toolbar button:active,
.editor_atto_toolbar button.highlight {
  background-image: radial-gradient(ellipse at center, #fff 40%, #dfdfdf 100%);
  background-color: #dfdfdf;
}

/* Make firefox button sizes match other browsers */
div.editor_atto_toolbar button::-moz-focus-inner {
  border: 0;
  padding: 0;
}

div.editor_atto_toolbar button .icon {
  padding: 0;
  margin: 2px 0;
}

div.editor_atto_toolbar div.atto_group {
  display: inline-block;
  border: 1px solid #ccc;
  border-bottom: 1px solid #b3b3b3;
  border-radius: 4px;
  margin: 9px 0 0 9px;
  background: #fff;
}

div.editor_atto_toolbar .atto_toolbar_row {
  margin: 6px 0 -3px 5px;
  display: table;
}
div.editor_atto_toolbar .atto_toolbar_row div.atto_group {
  margin: 3px 5px 3px 4px;
}

.editor_atto_content img {
  resize: both;
  overflow: auto;
}

.atto_hasmenu {
  /* IE8 places the images on top of each other if that is not set. */
  white-space: nowrap;
}

.atto_menuentry .icon {
  width: 16px;
  height: 16px;
}

.atto_menuentry {
  clear: left;
}

.atto_menuentry h1, .atto_menuentry .h1,
.atto_menuentry h2,
.atto_menuentry .h2,
.atto_menuentry p {
  margin: 4px;
}

/*.atto_form label.sameline {
    display: inline-block;
    min-width: 10em;
}*/
.atto_form textarea.fullwidth,
.atto_form input.fullwidth {
  width: 100%;
}

.atto_form {
  padding: 0.5rem;
}

/*.atto_form label {
    display: block;
    margin: 0 0 5px 0;
}*/
.atto_control {
  position: absolute;
  right: -6px;
  bottom: -6px;
  display: none;
  cursor: pointer;
}

.atto_control .icon {
  background-color: #fff;
}

div.editor_atto_content:focus .atto_control,
div.editor_atto_content:hover .atto_control {
  display: block;
}

.editor_atto_menu.yui3-menu-hidden {
  display: none;
}

/* Get broken images back in firefox */
.editor_atto_content img:-moz-broken {
  -moz-force-broken-image-icon: 1;
  min-width: 24px;
  min-height: 24px;
}

/* Atto menu styling */
.moodle-dialogue-base .editor_atto_menu .moodle-dialogue-content .moodle-dialogue-bd {
  padding: 0;
  z-index: 1000;
}

.editor_atto_menu .dropdown-menu > li > a {
  margin: 3px 14px;
}

.editor_atto_menu .open ul.dropdown-menu {
  padding-top: 5px;
  padding-bottom: 5px;
}

.editor_atto_wrap {
  position: relative;
}

/*rtl:ignore*/
.editor_atto_wrap textarea {
  direction: ltr;
}

.editor_atto_notification {
  display: inline-block;
  padding: 0.5em;
  padding-left: 1em;
  padding-right: 1em;
  border-bottom-left-radius: 1em;
  border-bottom-right-radius: 1em;
}
.editor_atto_notification .atto_info {
  background-color: #f2f2f2;
}
.editor_atto_notification .atto_warning {
  background-color: #ffd700;
}

.editor_atto_toolbar,
.editor_atto_content_wrap,
.editor_atto + textarea {
  box-sizing: border-box;
}

.editor_atto_content.form-control {
  width: 100%;
  border-top: 0;
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}

/** Atto fields do not have form-control because that would break the layout of the editor.
    So they need these extra styles to highlight the editor when there is a validation error. */
.has-danger .editor_atto_content.form-control .invalid-feedback,
.has-danger .editor_atto_content.form-control-danger .invalid-feedback {
  display: none;
  width: 100%;
  margin-top: 0.25rem;
  font-size: 0.875em;
  color: #ca3120;
}
.has-danger .editor_atto_content.form-control .invalid-tooltip,
.has-danger .editor_atto_content.form-control-danger .invalid-tooltip {
  position: absolute;
  top: 100%;
  z-index: 5;
  display: none;
  max-width: 100%;
  padding: 0.25rem 0.5rem;
  margin-top: 0.1rem;
  font-size: 0.8203125rem;
  color: #fff;
  background-color: rgba(202, 49, 32, 0.9);
  border-radius: var(--bs-border-radius);
}
.was-validated .has-danger .editor_atto_content.form-control:invalid ~ .invalid-feedback,
.was-validated .has-danger .editor_atto_content.form-control:invalid ~ .invalid-tooltip, .has-danger .editor_atto_content.form-control.is-invalid ~ .invalid-feedback,
.has-danger .editor_atto_content.form-control.is-invalid ~ .invalid-tooltip,
.was-validated .has-danger .editor_atto_content.form-control-danger:invalid ~ .invalid-feedback,
.was-validated .has-danger .editor_atto_content.form-control-danger:invalid ~ .invalid-tooltip,
.has-danger .editor_atto_content.form-control-danger.is-invalid ~ .invalid-feedback,
.has-danger .editor_atto_content.form-control-danger.is-invalid ~ .invalid-tooltip {
  display: block;
}
.was-validated .has-danger .editor_atto_content.form-control .form-control:invalid, .has-danger .editor_atto_content.form-control .form-control.is-invalid,
.was-validated .has-danger .editor_atto_content.form-control-danger .form-control:invalid,
.has-danger .editor_atto_content.form-control-danger .form-control.is-invalid {
  border-color: #ca3120;
  padding-right: calc(1.5em + 0.75rem);
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 12 12' width='12' height='12' fill='none' stroke='%23ca3120'%3e%3ccircle cx='6' cy='6' r='4.5'/%3e%3cpath stroke-linejoin='round' d='M5.8 3.6h.4L6 6.5z'/%3e%3ccircle cx='6' cy='8.2' r='.6' fill='%23ca3120' stroke='none'/%3e%3c/svg%3e");
  background-repeat: no-repeat;
  background-position: right calc(0.375em + 0.1875rem) center;
  background-size: calc(0.75em + 0.375rem) calc(0.75em + 0.375rem);
}
.was-validated .has-danger .editor_atto_content.form-control .form-control:invalid:focus, .has-danger .editor_atto_content.form-control .form-control.is-invalid:focus,
.was-validated .has-danger .editor_atto_content.form-control-danger .form-control:invalid:focus,
.has-danger .editor_atto_content.form-control-danger .form-control.is-invalid:focus {
  border-color: #ca3120;
  box-shadow: 0 0 0 0.25rem rgba(202, 49, 32, 0.75);
}
.was-validated .has-danger .editor_atto_content.form-control textarea.form-control:invalid, .has-danger .editor_atto_content.form-control textarea.form-control.is-invalid,
.was-validated .has-danger .editor_atto_content.form-control-danger textarea.form-control:invalid,
.has-danger .editor_atto_content.form-control-danger textarea.form-control.is-invalid {
  padding-right: calc(1.5em + 0.75rem);
  background-position: top calc(0.375em + 0.1875rem) right calc(0.375em + 0.1875rem);
}
.was-validated .has-danger .editor_atto_content.form-control .form-select:invalid, .has-danger .editor_atto_content.form-control .form-select.is-invalid,
.was-validated .has-danger .editor_atto_content.form-control-danger .form-select:invalid,
.has-danger .editor_atto_content.form-control-danger .form-select.is-invalid {
  border-color: #ca3120;
}
.was-validated .has-danger .editor_atto_content.form-control .form-select:invalid:not([multiple]):not([size]), .was-validated .has-danger .editor_atto_content.form-control .form-select:invalid:not([multiple])[size="1"], .has-danger .editor_atto_content.form-control .form-select.is-invalid:not([multiple]):not([size]), .has-danger .editor_atto_content.form-control .form-select.is-invalid:not([multiple])[size="1"],
.was-validated .has-danger .editor_atto_content.form-control-danger .form-select:invalid:not([multiple]):not([size]),
.was-validated .has-danger .editor_atto_content.form-control-danger .form-select:invalid:not([multiple])[size="1"],
.has-danger .editor_atto_content.form-control-danger .form-select.is-invalid:not([multiple]):not([size]),
.has-danger .editor_atto_content.form-control-danger .form-select.is-invalid:not([multiple])[size="1"] {
  --bs-form-select-bg-icon: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' viewBox='0 0 12 12' width='12' height='12' fill='none' stroke='%23ca3120'%3e%3ccircle cx='6' cy='6' r='4.5'/%3e%3cpath stroke-linejoin='round' d='M5.8 3.6h.4L6 6.5z'/%3e%3ccircle cx='6' cy='8.2' r='.6' fill='%23ca3120' stroke='none'/%3e%3c/svg%3e");
  padding-right: 4.125rem;
  background-position: right 0.75rem center, center right 2.25rem;
  background-size: 16px 12px, calc(0.75em + 0.375rem) calc(0.75em + 0.375rem);
}
.was-validated .has-danger .editor_atto_content.form-control .form-select:invalid:focus, .has-danger .editor_atto_content.form-control .form-select.is-invalid:focus,
.was-validated .has-danger .editor_atto_content.form-control-danger .form-select:invalid:focus,
.has-danger .editor_atto_content.form-control-danger .form-select.is-invalid:focus {
  border-color: #ca3120;
  box-shadow: 0 0 0 0.25rem rgba(202, 49, 32, 0.75);
}
.was-validated .has-danger .editor_atto_content.form-control .form-control-color:invalid, .has-danger .editor_atto_content.form-control .form-control-color.is-invalid,
.was-validated .has-danger .editor_atto_content.form-control-danger .form-control-color:invalid,
.has-danger .editor_atto_content.form-control-danger .form-control-color.is-invalid {
  width: calc(3rem + calc(1.5em + 0.75rem));
}
.was-validated .has-danger .editor_atto_content.form-control .form-check-input:invalid, .has-danger .editor_atto_content.form-control .form-check-input.is-invalid,
.was-validated .has-danger .editor_atto_content.form-control-danger .form-check-input:invalid,
.has-danger .editor_atto_content.form-control-danger .form-check-input.is-invalid {
  border-color: #ca3120;
}
.was-validated .has-danger .editor_atto_content.form-control .form-check-input:invalid:checked, .has-danger .editor_atto_content.form-control .form-check-input.is-invalid:checked,
.was-validated .has-danger .editor_atto_content.form-control-danger .form-check-input:invalid:checked,
.has-danger .editor_atto_content.form-control-danger .form-check-input.is-invalid:checked {
  background-color: #ca3120;
}
.was-validated .has-danger .editor_atto_content.form-control .form-check-input:invalid:focus, .has-danger .editor_atto_content.form-control .form-check-input.is-invalid:focus,
.was-validated .has-danger .editor_atto_content.form-control-danger .form-check-input:invalid:focus,
.has-danger .editor_atto_content.form-control-danger .form-check-input.is-invalid:focus {
  box-shadow: 0 0 0 0.25rem rgba(202, 49, 32, 0.75);
}
.was-validated .has-danger .editor_atto_content.form-control .form-check-input:invalid ~ .form-check-label, .has-danger .editor_atto_content.form-control .form-check-input.is-invalid ~ .form-check-label,
.was-validated .has-danger .editor_atto_content.form-control-danger .form-check-input:invalid ~ .form-check-label,
.has-danger .editor_atto_content.form-control-danger .form-check-input.is-invalid ~ .form-check-label {
  color: #ca3120;
}
.has-danger .editor_atto_content.form-control .form-check-inline .form-check-input ~ .invalid-feedback,
.has-danger .editor_atto_content.form-control-danger .form-check-inline .form-check-input ~ .invalid-feedback {
  margin-left: 0.5em;
}
.was-validated .has-danger .editor_atto_content.form-control .input-group > .form-control:not(:focus):invalid, .has-danger .editor_atto_content.form-control .input-group > .form-control:not(:focus).is-invalid,
.was-validated .has-danger .editor_atto_content.form-control .input-group > .form-select:not(:focus):invalid,
.has-danger .editor_atto_content.form-control .input-group > .form-select:not(:focus).is-invalid,
.was-validated .has-danger .editor_atto_content.form-control .input-group > .form-floating:not(:focus-within):invalid,
.has-danger .editor_atto_content.form-control .input-group > .form-floating:not(:focus-within).is-invalid,
.was-validated .has-danger .editor_atto_content.form-control-danger .input-group > .form-control:not(:focus):invalid,
.has-danger .editor_atto_content.form-control-danger .input-group > .form-control:not(:focus).is-invalid,
.was-validated .has-danger .editor_atto_content.form-control-danger .input-group > .form-select:not(:focus):invalid,
.has-danger .editor_atto_content.form-control-danger .input-group > .form-select:not(:focus).is-invalid,
.was-validated .has-danger .editor_atto_content.form-control-danger .input-group > .form-floating:not(:focus-within):invalid,
.has-danger .editor_atto_content.form-control-danger .input-group > .form-floating:not(:focus-within).is-invalid {
  z-index: 4;
}

.open.atto_menu > .dropdown-menu {
  display: block;
}

div.editor_atto_toolbar button .icon {
  color: #495057;
}

.toast {
  border-radius: var(--bs-border-radius);
}
.toast.toast-success {
  background-color: rgba(215, 228, 214, 0.95);
  color: #20491e;
}
.toast.toast-success .toast-header {
  color: #20491e;
}
.toast.toast-success .toast-body:before {
  font: var(--fa-font-solid);
  margin: 2px 5px 0 0;
  content: "\f058";
}
.toast.toast-danger {
  background-color: rgba(244, 214, 210, 0.95);
  color: #791d13;
}
.toast.toast-danger .toast-header {
  color: #791d13;
}
.toast.toast-danger .toast-body:before {
  font: var(--fa-font-solid);
  margin: 2px 5px 0 0;
  content: "\f057";
}
.toast.toast-info {
  background-color: rgba(204, 230, 234, 0.95);
  color: #004d5a;
}
.toast.toast-info .toast-header {
  color: #004d5a;
}
.toast.toast-info .toast-body:before {
  font: var(--fa-font-solid);
  margin: 2px 5px 0 0;
  content: "\f05a";
}
.toast.toast-warning {
  background-color: rgba(252, 239, 220, 0.95);
  color: #90682f;
}
.toast.toast-warning .toast-header {
  color: #90682f;
}
.toast.toast-warning .toast-body:before {
  font: var(--fa-font-solid);
  margin: 2px 5px 0 0;
  content: "\f06a";
}
.toast .btn-close {
  color: inherit;
}

.navbar.fixed-top {
  padding-top: 0;
  padding-bottom: 0;
  box-shadow: none;
  border-bottom: #dee2e6 1px solid;
  align-items: stretch;
  height: 61px;
}
.navbar.fixed-top .navbar-brand .logo {
  max-height: calc(60px - (0.25rem * 2));
}
.navbar.fixed-top .nav-link {
  height: 100%;
  display: flex;
  align-items: center;
  white-space: nowrap;
}
.navbar.fixed-top .divider {
  width: 1px;
  background-color: #dee2e6;
}
.navbar.fixed-top #usernavigation .nav-link {
  padding: 0 0.5rem;
}
.navbar.fixed-top .login {
  display: flex;
  align-items: center;
}
.navbar.fixed-top .usermenu {
  display: flex;
}
.navbar.fixed-top .usermenu .action-menu {
  display: flex;
  align-items: center;
}
.navbar.fixed-top .usermenu .dropdown {
  display: flex;
  align-items: center;
}
.navbar.fixed-top .usermenu .dropdown .dropdown-toggle {
  padding-top: 0;
  padding-bottom: 0;
  border-radius: 0;
  display: flex;
  align-items: center;
  height: 100%;
}
.navbar.fixed-top .usermenu .dropdown-menu {
  min-width: 235px;
}
.navbar.fixed-top .usermenu .dropdown-menu .carousel-navigation-link > * {
  pointer-events: none;
}
.navbar.fixed-top .usermenu .dropdown-menu .dropdown-item {
  padding: 0.25rem 1.75rem 0.25rem 0.75rem;
}
.navbar.fixed-top .usermenu .dropdown-menu .dropdown-item.carousel-navigation-link::after {
  font: var(--fa-font-solid);
  content: "\f0da";
  font-size: 1rem;
  padding-top: 0.2rem;
  right: 0.75rem;
  position: absolute;
}
.navbar.fixed-top .usermenu .dropdown-menu .submenu .header {
  padding: 0.25rem 0.75rem;
  font-size: 0.975rem;
}
.navbar.fixed-top .usermenu .dropdown-menu .submenu .header .icon {
  font-size: 20px;
  height: 20px;
  width: 20px;
  margin: 0;
}
.navbar.fixed-top .usermenu .dropdown-menu .submenu .items .dropdown-item[aria-current=true]::before {
  font: var(--fa-font-solid);
  content: "\f00c";
  font-size: 0.75rem;
  padding-left: 0.25rem;
}
.navbar.fixed-top .usermenu .login {
  display: flex;
  align-items: center;
}
.navbar.fixed-top .usermenu .dropdown,
.navbar.fixed-top .langmenu .dropdown {
  display: flex;
  align-items: center;
  height: 100%;
}
.navbar.fixed-top .usermenu .dropdown .dropdown-toggle,
.navbar.fixed-top .langmenu .dropdown .dropdown-toggle {
  padding-top: 0;
  padding-bottom: 0;
  border-radius: 0;
  display: flex;
  align-items: center;
  height: 100%;
}
.navbar.fixed-top .langmenu .dropdown-menu .dropdown-item[aria-current=true]::before {
  content: "\f00c";
  font-size: 0.75rem;
  padding-left: 0.25rem;
}
@media (max-width: 767.98px) {
  .navbar.fixed-top .langmenu .langbutton {
    display: none;
  }
}
.navbar.fixed-top .moodle-actionmenu .menubar,
.navbar.fixed-top .action-menu-trigger .dropdown {
  height: 100%;
  display: flex;
}

.dir-rtl .navbar.fixed-top .usermenu .dropdown-menu .dropdown-item.carousel-navigation-link::after {
  font: var(--fa-font-solid);
  content: "\f0d9";
}

#page {
  margin-top: 60px;
}

.pagelayout-embedded #page {
  margin-top: 0;
}

.navbar-bootswatch .navbar-brand {
  overflow: hidden;
  text-overflow: ellipsis;
}
.navbar-bootswatch .navbar-brand .sitename {
  overflow: hidden;
  white-space: nowrap;
  text-overflow: ellipsis;
}

:root {
  --navbar-height: 60px;
}

/**
* Reportbuilder styles.
*/
.reportbuilder-wrapper .btn-outline-secondary[data-bs-toggle=collapse]:not(.collapsed),
.reportbuilder-wrapper .dropdown.show .btn-outline-secondary[data-bs-toggle=dropdown] {
  color: #fff;
  background-color: #6a737b;
  border-color: #6a737b;
}

.reportbuilder-table .action-menu .menubar {
  justify-content: end;
}

.reportbuilder-wrapper .filters-dropdown {
  width: 27rem;
  padding: 0 0 4rem 0;
  z-index: 1055;
  overflow: hidden;
}
@media (max-width: 767.98px) {
  .reportbuilder-wrapper .filters-dropdown {
    width: 100%;
  }
}

.reportbuilder-filters-sidebar {
  max-height: 30rem;
  overflow-y: auto;
  border-bottom: 1px solid var(--bs-border-color-translucent);
  scrollbar-width: thin;
  scrollbar-color: #6a737b #fff;
  background: linear-gradient(#fff 30%, rgba(255, 255, 255, 0)) center top, linear-gradient(rgba(255, 255, 255, 0), #fff 70%) center bottom, radial-gradient(farthest-side at 50% 0, rgba(0, 0, 0, 0.2), rgba(0, 0, 0, 0)) center top, radial-gradient(farthest-side at 50% 100%, rgba(0, 0, 0, 0.2), rgba(0, 0, 0, 0)) center bottom;
  background-repeat: no-repeat;
  background-size: 100% 1rem, 100% 1rem, 100% calc(1rem / 2), 100% calc(1rem / 2);
  background-attachment: local, local, scroll, scroll;
}
.reportbuilder-filters-sidebar::-webkit-scrollbar {
  width: 12px;
}
.reportbuilder-filters-sidebar::-webkit-scrollbar-track {
  background: #fff;
}
.reportbuilder-filters-sidebar::-webkit-scrollbar-thumb {
  background-color: #6a737b;
  border-radius: 20px;
  border: 3px solid #fff;
}
.reportbuilder-filters-sidebar::-webkit-scrollbar-thumb:hover {
  background-color: #495057;
}
.reportbuilder-filters-sidebar .filter:not(:nth-last-child(2)) {
  border-bottom: 1px solid var(--bs-border-color-translucent);
}
.reportbuilder-filters-sidebar .filter .filter-name:hover {
  white-space: normal;
  text-overflow: clip;
  word-break: break-all;
}
.reportbuilder-filters-sidebar .fitem[data-groupname=buttonar] {
  position: absolute;
  bottom: 0;
  height: 4rem;
  align-items: center;
}
.reportbuilder-filters-sidebar .fitem[data-groupname=buttonar] .col-form-label {
  display: none !important; /* stylelint-disable-line declaration-no-important */
}

.reportbuilder-filters-wrapper .mform.full-width-labels .fdate_selector,
.reportbuilder-conditions-list .mform.full-width-labels .fdate_selector {
  flex-wrap: wrap;
}
.reportbuilder-filters-wrapper .mform.full-width-labels .fitem,
.reportbuilder-conditions-list .mform.full-width-labels .fitem {
  margin-bottom: 0 !important; /* stylelint-disable-line declaration-no-important */
  max-width: 100%;
}
.reportbuilder-filters-wrapper .mform.full-width-labels .fitem:not(.row):last-child,
.reportbuilder-conditions-list .mform.full-width-labels .fitem:not(.row):last-child {
  flex: 1;
  min-width: 0;
}
.reportbuilder-filters-wrapper .mform.full-width-labels .fitem:not(.row):last-child span,
.reportbuilder-filters-wrapper .mform.full-width-labels .fitem:not(.row):last-child .form-select,
.reportbuilder-conditions-list .mform.full-width-labels .fitem:not(.row):last-child span,
.reportbuilder-conditions-list .mform.full-width-labels .fitem:not(.row):last-child .form-select {
  width: 100%;
}

@media (min-width: 992px) {
  .path-admin-reportbuilder.pagelayout-popup.behat-site .fixed-top {
    position: fixed;
  }
  .path-admin-reportbuilder.pagelayout-popup #region-main {
    border: none;
    padding: 0;
  }
  .path-admin-reportbuilder.pagelayout-popup #maincontent {
    visibility: hidden;
  }
  .path-admin-reportbuilder.pagelayout-popup .dynamictabs .nav-tabs {
    position: fixed;
    z-index: 1030;
    width: calc(100% - 35px);
    padding-top: 1.25rem;
    background-color: #fff;
    box-shadow: 0 1.25rem 0 #fff;
  }
  .path-admin-reportbuilder.pagelayout-popup .dynamictabs .tab-content {
    padding-top: 83px;
  }
  .path-admin-reportbuilder.pagelayout-popup .reportbuilder-sidebar-menu {
    position: fixed;
  }
  .path-admin-reportbuilder.pagelayout-popup .reportbuilder-sidebar-menu .reportbuilder-sidebar-menu-cards {
    max-height: calc(100vh - 163px - 52px);
  }
  .path-admin-reportbuilder.pagelayout-popup .reportbuilder-sidebar-settings {
    position: fixed;
    right: 30px;
    max-height: calc(100vh - 163px);
  }
  .path-admin-reportbuilder.pagelayout-popup .reportbuilder-report[data-editing] .reportbuilder-report-container {
    max-height: calc(100vh - 163px);
    overflow-y: auto;
    scrollbar-width: thin;
    scrollbar-color: #6a737b #f8f9fa;
    margin-left: calc(250px + 1rem);
    margin-right: calc(350px + 1rem);
  }
  .path-admin-reportbuilder.pagelayout-popup .reportbuilder-report[data-editing] .reportbuilder-report-container::-webkit-scrollbar {
    width: 12px;
  }
  .path-admin-reportbuilder.pagelayout-popup .reportbuilder-report[data-editing] .reportbuilder-report-container::-webkit-scrollbar-track {
    background: #f8f9fa;
  }
  .path-admin-reportbuilder.pagelayout-popup .reportbuilder-report[data-editing] .reportbuilder-report-container::-webkit-scrollbar-thumb {
    background-color: #6a737b;
    border-radius: 20px;
    border: 3px solid #f8f9fa;
  }
  .path-admin-reportbuilder.pagelayout-popup .reportbuilder-report[data-editing] .reportbuilder-report-container::-webkit-scrollbar-thumb:hover {
    background-color: #495057;
  }
  .path-admin-reportbuilder.pagelayout-popup .reportbuilder-audiences-container {
    margin-left: calc(250px + 1rem);
  }
}
#page-admin-reportbuilder-edit #page {
  overflow-y: auto;
}

.reportbuilder-report-container {
  min-width: 0;
}
.reportbuilder-report-container button[data-action=toggle-edit-preview] .loading-icon {
  margin-left: 0.5rem;
}

.reportbuilder-editor-table-container {
  overflow-x: auto;
  padding: 1px;
}

.reportbuilder-table th button[data-action=report-remove-column] .icon,
.reportbuilder-table th span[data-drag-type=move] .icon {
  width: 12px;
  height: 12px;
  font-size: 12px;
  vertical-align: text-top;
  color: #1d2125;
}
.reportbuilder-table th button[data-action=report-remove-column] .icon {
  margin-right: 0;
}

@media (min-width: 992px) {
  .reportbuilder-sidebar-menu {
    width: 250px;
    flex-shrink: 0;
  }
}
.reportbuilder-sidebar-menu .card-body .list-group-item {
  padding: 0.75rem;
}
.reportbuilder-sidebar-menu .card-body .list-group-item .icon {
  width: 12px;
  height: 12px;
  font-size: 12px;
}

.reportbuilder-sidebar-menu-cards {
  overflow-y: auto;
  scrollbar-width: thin;
  scrollbar-color: #6a737b #f8f9fa;
}
.reportbuilder-sidebar-menu-cards::-webkit-scrollbar {
  width: 12px;
}
.reportbuilder-sidebar-menu-cards::-webkit-scrollbar-track {
  background: #f8f9fa;
}
.reportbuilder-sidebar-menu-cards::-webkit-scrollbar-thumb {
  background-color: #6a737b;
  border-radius: 20px;
  border: 3px solid #f8f9fa;
}
.reportbuilder-sidebar-menu-cards::-webkit-scrollbar-thumb:hover {
  background-color: #495057;
}

.reportbuilder-sidebar-settings {
  overflow-y: auto;
  scrollbar-width: thin;
  scrollbar-color: #6a737b #f8f9fa;
}
.reportbuilder-sidebar-settings::-webkit-scrollbar {
  width: 12px;
}
.reportbuilder-sidebar-settings::-webkit-scrollbar-track {
  background: #f8f9fa;
}
.reportbuilder-sidebar-settings::-webkit-scrollbar-thumb {
  background-color: #6a737b;
  border-radius: 20px;
  border: 3px solid #f8f9fa;
}
.reportbuilder-sidebar-settings::-webkit-scrollbar-thumb:hover {
  background-color: #495057;
}
@media (min-width: 992px) {
  .reportbuilder-sidebar-settings {
    width: 350px;
    flex-shrink: 0;
  }
}
.reportbuilder-sidebar-settings div[data-region=settings-conditions] .reportbuilder-conditions-select .form-autocomplete-selection,
.reportbuilder-sidebar-settings div[data-region=settings-filters] .reportbuilder-filters-select .form-autocomplete-selection {
  display: none;
}
.reportbuilder-sidebar-settings div[data-region=settings-conditions] .reportbuilder-conditions-select .form-autocomplete-input,
.reportbuilder-sidebar-settings div[data-region=settings-filters] .reportbuilder-filters-select .form-autocomplete-input {
  width: 100%;
}
.reportbuilder-sidebar-settings div[data-region=settings-conditions] .reportbuilder-conditions-select .form-autocomplete-suggestions,
.reportbuilder-sidebar-settings div[data-region=settings-filters] .reportbuilder-filters-select .form-autocomplete-suggestions {
  width: calc(100% - 2 * 0.5rem);
}
.reportbuilder-sidebar-settings .list-group-item {
  padding: 0.75rem;
}
.reportbuilder-sidebar-settings .list-group-item .icon {
  width: 12px;
  height: 12px;
  font-size: 12px;
  color: #1d2125;
}
.reportbuilder-sidebar-settings .list-group-item button[data-action=report-remove-filter] .icon,
.reportbuilder-sidebar-settings .list-group-item button[data-action=report-remove-condition] .icon {
  margin-right: 0;
  vertical-align: text-top;
}
.reportbuilder-sidebar-settings .list-group-item span[data-drag-type=move] .icon {
  vertical-align: text-top;
}
.reportbuilder-sidebar-settings div[data-region=settings-sorting] .list-group-item span[data-drag-type=move] .icon {
  vertical-align: middle;
}
.reportbuilder-sidebar-settings div[data-region=settings-cardview] form .col-md-3,
.reportbuilder-sidebar-settings div[data-region=settings-cardview] form .col-md-9 {
  flex: 1 1;
  max-width: initial;
}
.reportbuilder-sidebar-settings div[data-region=settings-cardview] form div[data-fieldtype=submit] {
  flex-basis: auto;
}
.reportbuilder-sidebar-settings .inplaceeditable.inplaceeditingon input {
  width: 100%;
}

@keyframes sortable-list-drag-color-reset {
  from {
    background-color: #a2cff8;
  }
  to {
    background-color: inherit;
  }
}
.reportbuilder-sortable-list li.sortable-list-current-position,
.reportbuilder-table th.sortable-list-current-position,
.reportbuilder-conditions-list .condition.sortable-list-current-position {
  background-color: #a2cff8;
}
.reportbuilder-sortable-list li.sortable-list-is-dragged,
.reportbuilder-table th.sortable-list-is-dragged,
.reportbuilder-conditions-list .condition.sortable-list-is-dragged {
  background-color: #fff;
  opacity: 0.85;
}
.reportbuilder-sortable-list li.sortable-list-is-dropped,
.reportbuilder-table th.sortable-list-is-dropped,
.reportbuilder-conditions-list .condition.sortable-list-is-dropped {
  animation: sortable-list-drag-color-reset 1s ease;
}

.reportbuilder-toggle-card .card-header {
  border-bottom: none;
}
.reportbuilder-toggle-card .card-body {
  border-top: var(--bs-border-width) solid var(--bs-border-color-translucent);
}
.reportbuilder-toggle-card .toggle-card-button i.toggle-card-icon {
  color: #6a737b;
  font-weight: 700;
}
.reportbuilder-toggle-card .toggle-card-button .collapsed-icon-container {
  display: none;
}
.reportbuilder-toggle-card .toggle-card-button .expanded-icon-container {
  display: inline-block;
}
.reportbuilder-toggle-card .toggle-card-button.collapsed .collapsed-icon-container {
  display: inline-block;
}
.reportbuilder-toggle-card .toggle-card-button.collapsed .expanded-icon-container {
  display: none;
}

.reportbuilder-audiences-container .audience-separator {
  text-transform: uppercase;
}
.reportbuilder-audiences-container .audience-separator::before, .reportbuilder-audiences-container .audience-separator::after {
  content: "";
  flex: 1;
  border-bottom: 1px solid rgba(0, 0, 0, 0.125);
}
.reportbuilder-audiences-container .audience-separator:not(:empty)::before {
  margin-right: 1rem;
}
.reportbuilder-audiences-container .audience-separator:not(:empty)::after {
  margin-left: 1rem;
}
.reportbuilder-audiences-container .instance-card .card-header i.icon {
  margin-right: 0;
}

.reportbuilder-report {
  container-type: inline-size; /* stylelint-disable-line */
}

@container (width < 576px) {
  .reportbuilder-report[data-report-type="0"]:not([data-editing]) { /* stylelint-disable-line */ }
  .reportbuilder-report[data-report-type="0"]:not([data-editing]):not([data-force-table]) table.reportbuilder-table {
    --bs-table-striped-bg: var(--bs-body-bg);
    --bs-table-hover-bg: var(--bs-body-bg);
  }
  .reportbuilder-report[data-report-type="0"]:not([data-editing]):not([data-force-table]) table.reportbuilder-table thead {
    display: none;
  }
  .reportbuilder-report[data-report-type="0"]:not([data-editing]):not([data-force-table]) table.reportbuilder-table tr {
    display: flex;
    flex-direction: column;
    margin: 0.5rem 0;
    padding: 0.25rem 0.5rem 0 0.5rem;
    word-wrap: break-word;
    background-clip: border-box;
    border: var(--bs-border-width) solid var(--bs-border-color-translucent);
    border-radius: var(--bs-border-radius);
  }
  .reportbuilder-report[data-report-type="0"]:not([data-editing]):not([data-force-table]) table.reportbuilder-table tr.emptyrow {
    display: none;
  }
  .reportbuilder-report[data-report-type="0"]:not([data-editing]):not([data-force-table]) table.reportbuilder-table tr:not(.show) td[data-cardviewhidden] {
    display: none;
  }
  .reportbuilder-report[data-report-type="0"]:not([data-editing]):not([data-force-table]) table.reportbuilder-table tr td {
    display: block;
    min-height: 3.6rem;
    padding: 0.5rem 0.25rem;
    border: none;
  }
  .reportbuilder-report[data-report-type="0"]:not([data-editing]):not([data-force-table]) table.reportbuilder-table tr td::before {
    content: attr(data-cardtitle);
    display: block;
    text-transform: uppercase;
    font-size: 70%;
    color: #343a40;
  }
  .reportbuilder-report[data-report-type="0"]:not([data-editing]):not([data-force-table]) table.reportbuilder-table tr td:not([data-cardtitle]) {
    min-height: 3rem;
  }
  .reportbuilder-report[data-report-type="0"]:not([data-editing]):not([data-force-table]) table.reportbuilder-table tr td:not(:first-child):not(.card-toggle) {
    border-top: var(--bs-border-width) solid var(--bs-border-color-translucent);
  }
  .reportbuilder-report[data-report-type="0"]:not([data-editing]):not([data-force-table]) table.reportbuilder-table tr td:first-child {
    padding-right: 2rem;
  }
  .reportbuilder-report[data-report-type="0"]:not([data-editing]):not([data-force-table]) table.reportbuilder-table tr td.card-toggle {
    display: block !important; /* stylelint-disable-line declaration-no-important */
    position: absolute;
    right: 10px;
  }
  .reportbuilder-report[data-report-type="0"]:not([data-editing]):not([data-force-table]) table.reportbuilder-table tr td.card-toggle button {
    padding: 0 0.5rem;
    color: #6a737b;
  }
  .reportbuilder-report[data-report-type="0"]:not([data-editing]):not([data-force-table]) table.reportbuilder-table tr td.card-toggle button i {
    font-size: 1.5em;
    font-weight: bold;
  }
}
.reportbuilder-report[data-report-type="0"]:not([data-editing])[data-force-card] table.reportbuilder-table {
  --bs-table-striped-bg: var(--bs-body-bg);
  --bs-table-hover-bg: var(--bs-body-bg);
}
.reportbuilder-report[data-report-type="0"]:not([data-editing])[data-force-card] table.reportbuilder-table thead {
  display: none;
}
.reportbuilder-report[data-report-type="0"]:not([data-editing])[data-force-card] table.reportbuilder-table tr {
  display: flex;
  flex-direction: column;
  margin: 0.5rem 0;
  padding: 0.25rem 0.5rem 0 0.5rem;
  word-wrap: break-word;
  background-clip: border-box;
  border: var(--bs-border-width) solid var(--bs-border-color-translucent);
  border-radius: var(--bs-border-radius);
}
.reportbuilder-report[data-report-type="0"]:not([data-editing])[data-force-card] table.reportbuilder-table tr.emptyrow {
  display: none;
}
.reportbuilder-report[data-report-type="0"]:not([data-editing])[data-force-card] table.reportbuilder-table tr:not(.show) td[data-cardviewhidden] {
  display: none;
}
.reportbuilder-report[data-report-type="0"]:not([data-editing])[data-force-card] table.reportbuilder-table tr td {
  display: block;
  min-height: 3.6rem;
  padding: 0.5rem 0.25rem;
  border: none;
}
.reportbuilder-report[data-report-type="0"]:not([data-editing])[data-force-card] table.reportbuilder-table tr td::before {
  content: attr(data-cardtitle);
  display: block;
  text-transform: uppercase;
  font-size: 70%;
  color: #343a40;
}
.reportbuilder-report[data-report-type="0"]:not([data-editing])[data-force-card] table.reportbuilder-table tr td:not([data-cardtitle]) {
  min-height: 3rem;
}
.reportbuilder-report[data-report-type="0"]:not([data-editing])[data-force-card] table.reportbuilder-table tr td:not(:first-child):not(.card-toggle) {
  border-top: var(--bs-border-width) solid var(--bs-border-color-translucent);
}
.reportbuilder-report[data-report-type="0"]:not([data-editing])[data-force-card] table.reportbuilder-table tr td:first-child {
  padding-right: 2rem;
}
.reportbuilder-report[data-report-type="0"]:not([data-editing])[data-force-card] table.reportbuilder-table tr td.card-toggle {
  display: block !important; /* stylelint-disable-line declaration-no-important */
  position: absolute;
  right: 10px;
}
.reportbuilder-report[data-report-type="0"]:not([data-editing])[data-force-card] table.reportbuilder-table tr td.card-toggle button {
  padding: 0 0.5rem;
  color: #6a737b;
}
.reportbuilder-report[data-report-type="0"]:not([data-editing])[data-force-card] table.reportbuilder-table tr td.card-toggle button i {
  font-size: 1.5em;
  font-weight: bold;
}

.courseindex .courseindex-item {
  padding: 0.5rem 0.5rem;
  border: 1px solid transparent;
  border-radius: 0.5rem;
}
.courseindex .courseindex-item.courseindex-section-title a {
  font-weight: bold;
}
.courseindex .courseindex-item .icons-collapse-expand {
  padding-right: 4px;
}
.courseindex .courseindex-item .courseindex-link,
.courseindex .courseindex-item .courseindex-chevron {
  color: var(--bs-secondary-color);
}
.courseindex .courseindex-item .courseindex-link:hover, .courseindex .courseindex-item .courseindex-link:focus,
.courseindex .courseindex-item .courseindex-chevron:hover,
.courseindex .courseindex-item .courseindex-chevron:focus {
  color: black;
  text-decoration: none;
}
.courseindex .courseindex-item:hover, .courseindex .courseindex-item:focus {
  color: black;
}
.courseindex .courseindex-item:hover .courseindex-link,
.courseindex .courseindex-item:hover .courseindex-chevron, .courseindex .courseindex-item:focus .courseindex-link,
.courseindex .courseindex-item:focus .courseindex-chevron {
  color: black;
  cursor: pointer;
}
.courseindex .courseindex-item:hover.dimmed, .courseindex .courseindex-item:focus.dimmed {
  color: black;
}
.courseindex .courseindex-item:hover.dimmed .courseindex-link,
.courseindex .courseindex-item:hover.dimmed .courseindex-chevron, .courseindex .courseindex-item:focus.dimmed .courseindex-link,
.courseindex .courseindex-item:focus.dimmed .courseindex-chevron {
  color: black;
}
.courseindex .courseindex-item:hover.draggable, .courseindex .courseindex-item:focus.draggable {
  cursor: move;
}
.courseindex .courseindex-item:hover.pageitem,
.courseindex .courseindex-item:hover .pageitem, .courseindex .courseindex-item:focus.pageitem,
.courseindex .courseindex-item:focus .pageitem {
  background-color: #0c589c;
  color: #e6e6e6;
}
.courseindex .courseindex-item:hover.pageitem .courseindex-link,
.courseindex .courseindex-item:hover.pageitem .courseindex-chevron,
.courseindex .courseindex-item:hover .pageitem .courseindex-link,
.courseindex .courseindex-item:hover .pageitem .courseindex-chevron, .courseindex .courseindex-item:focus.pageitem .courseindex-link,
.courseindex .courseindex-item:focus.pageitem .courseindex-chevron,
.courseindex .courseindex-item:focus .pageitem .courseindex-link,
.courseindex .courseindex-item:focus .pageitem .courseindex-chevron {
  color: #e6e6e6;
}
.courseindex .courseindex-item:hover.pageitem .courseindex-link a,
.courseindex .courseindex-item:hover.pageitem .courseindex-chevron a,
.courseindex .courseindex-item:hover .pageitem .courseindex-link a,
.courseindex .courseindex-item:hover .pageitem .courseindex-chevron a, .courseindex .courseindex-item:focus.pageitem .courseindex-link a,
.courseindex .courseindex-item:focus.pageitem .courseindex-chevron a,
.courseindex .courseindex-item:focus .pageitem .courseindex-link a,
.courseindex .courseindex-item:focus .pageitem .courseindex-chevron a {
  color: #e6e6e6;
}
.courseindex .courseindex-item.dragging {
  border: 1px solid #b3d9e0;
  background-color: #e6f2f5;
}
.courseindex .courseindex-item.active {
  background-color: #f8f9fa;
  border-color: #dee2e6;
}
.courseindex .courseindex-item.dimmed {
  color: #6a737b;
}
.courseindex .courseindex-item.dimmed .courseindex-link,
.courseindex .courseindex-item.dimmed .courseindex-chevron {
  color: #6a737b;
}
.courseindex .courseindex-item.dimmed.pageitem {
  color: #fff;
}
.courseindex .courseindex-item.dimmed.pageitem a {
  color: #fff;
}
.courseindex .courseindex-item .courseindex-locked {
  display: none;
}
.courseindex .courseindex-item.restrictions .courseindex-locked {
  display: block;
}
.courseindex .courseindex-item.pageitem {
  background-color: #0f6cbf;
  color: #fff;
  scroll-margin: 6rem;
}
.courseindex .courseindex-item.pageitem a {
  color: #fff;
}
.courseindex .courseindex-item .completioninfo {
  min-width: 24px;
}
.courseindex .courseindex-item .completioninfo.completion_complete {
  color: #357a32;
}
.courseindex .courseindex-item .completioninfo.completion_fail {
  color: #ca3120;
}
.courseindex .courseindex-item.indented {
  margin-left: 1rem;
}
.courseindex .courseindex-section {
  border-left: solid 3px transparent;
}
.courseindex .courseindex-section.dragging {
  border: 1px solid #b3d9e0;
  background-color: #e6f2f5;
}
.courseindex .courseindex-section .current-badge {
  line-height: 1.5;
  display: none;
}
.courseindex .courseindex-section.current {
  border-left: solid 3px #0f6cbf;
  /* Skip current badges in delegated sections. */
}
.courseindex .courseindex-section.current .current-badge {
  display: inline-block;
}
.courseindex .courseindex-section.current .delegated-section .current-badge {
  display: none;
}
.courseindex .courseindex-section.dropready .courseindex-item-content {
  /* Extra dropzone space */
  padding-bottom: 1em;
}
.courseindex .courseindex-section .courseindex-sectioncontent .courseindex-item {
  padding-left: 0.5rem;
}
.courseindex .courseindex-section.delegated-section {
  width: 100%;
}
.courseindex .icon {
  font-size: 10px;
}
.courseindex .d-flex-noedit {
  display: none;
}
.courseindex.editing .d-flex-noedit {
  display: flex;
}
.courseindex .placeholders .rounded-circle {
  height: 1rem;
  width: 1rem;
}
.courseindex .placeholders .w-100 {
  height: 1rem;
  margin: 0.5rem 0;
}

.moremenu {
  opacity: 0;
  height: 60px;
}
.moremenu.observed {
  opacity: 1;
}
.moremenu .nav-link {
  height: 60px;
  display: flex;
  align-items: center;
  border-right: none;
  border-bottom: solid 3px transparent;
  border-left: none;
  border-top: none;
}
.moremenu .nav-link:hover, .moremenu .nav-link:focus {
  border-color: transparent;
  background-color: #f8f9fa;
}
.moremenu .nav-link.active {
  background-color: #f8f9fa;
  border-color: transparent;
  border-bottom-color: #0f6cbf;
}
.moremenu .nav-link.active:focus, .moremenu .nav-link.active:hover {
  background-color: #f8f9fa;
  border-bottom-color: #0f6cbf;
}
.moremenu .nav-link.focus, .moremenu .nav-link:focus {
  position: relative;
}
.moremenu .nav-link[data-bs-toggle=tab] {
  display: inline-flex;
  flex-direction: column;
  align-items: center;
  justify-content: center;
}
.moremenu .nav-link[data-bs-toggle=tab]::after {
  content: attr(data-text)/"";
  height: 0;
  visibility: hidden;
  overflow: hidden;
  user-select: none;
  pointer-events: none;
  font-weight: bold;
}
@media speech {
  .moremenu .nav-link[data-bs-toggle=tab]::after {
    display: none;
  }
}
.moremenu .nav-tabs {
  margin-left: 0;
  background-color: #fff;
}
.moremenu .show > .nav-link,
.moremenu .active > .nav-link,
.moremenu .nav-link.show,
.moremenu .nav-link.active {
  background: transparent;
}
.moremenu .dropdownmoremenu > .dropdown-menu > .dropdown-item {
  padding: 0;
}
.moremenu .dropdownmoremenu > .dropdown-menu .dropdown-menu {
  position: static;
  padding: 0;
  border: 0;
}
.moremenu .dropdownmoremenu > .dropdown-menu .dropdown-menu.show {
  display: block;
}
.moremenu .dropdownmoremenu > .dropdown-menu .dropdown-menu .dropdown-item {
  background-color: #f8f9fa;
}
.moremenu .dropdownmoremenu > .dropdown-menu .dropdown-menu .dropdown-item:hover, .moremenu .dropdownmoremenu > .dropdown-menu .dropdown-menu .dropdown-item:focus {
  color: #fff;
  background-color: #0f6cbf;
}
.moremenu .dropdownmoremenu > .dropdown-menu .dropdown-menu .dropdown-divider {
  display: none;
}
.moremenu .dropdown-item[aria-current=true],
.moremenu .dropdown-item.active {
  background-color: transparent;
  color: var(--bs-body-color);
}
.moremenu .dropdown-item[aria-current=true]:focus-within, .moremenu .dropdown-item[aria-current=true]:hover,
.moremenu .dropdown-item.active:focus-within,
.moremenu .dropdown-item.active:hover {
  background-color: #0f6cbf;
  color: #fff;
}
.moremenu .dropdown-item[aria-current=true]:focus-within a, .moremenu .dropdown-item[aria-current=true]:hover a,
.moremenu .dropdown-item.active:focus-within a,
.moremenu .dropdown-item.active:hover a {
  color: #fff;
}
.moremenu .dropdown-item[aria-current=true]:before,
.moremenu .dropdown-item.active:before {
  font: var(--fa-font-solid);
  content: "\f00c";
}

.primary-navigation .navigation {
  height: 60px;
}
.primary-navigation .navigation .nav-link {
  height: 60px;
  color: #1d2125;
  border-top: 3px solid transparent;
}

@media (max-width: 767.98px) {
  .primary-navigation {
    display: none;
  }
  .editmode-switch-form label {
    width: 1px !important;
    height: 1px !important;
    padding: 0 !important;
    margin: -1px !important;
    overflow: hidden !important;
    clip: rect(0, 0, 0, 0) !important;
    white-space: nowrap !important;
    border: 0 !important;
  }
  .editmode-switch-form label:not(caption) {
    position: absolute !important;
  }
}
.drawer-primary .drawercontent {
  padding: 0;
}
.drawer-primary .drawercontent .list-group {
  border-radius: 0;
  margin-top: -1px;
}
.drawer-primary .drawercontent .list-group .list-group-item {
  border-left: 0;
  border-right: 0;
}

.secondary-navigation {
  padding-bottom: 15px;
}
.secondary-navigation .navigation {
  border-bottom: 1px solid var(--bs-border-color);
  background-color: #fff;
  margin: 0 -0.5rem;
  padding: 0 0.5rem;
}
.secondary-navigation .navigation .nav-tabs {
  border: none;
  max-width: 830px;
  margin: 0 auto;
}
.secondary-navigation .navigation .nav-tabs .nav-link {
  border-radius: initial;
}

@media (min-width: 768px) {
  .secondary-navigation .navigation {
    padding: 0 calc(0.5rem + 15px);
  }
}
.tertiary-navigation {
  padding-top: 10px;
}
.tertiary-navigation.full-width-bottom-border {
  width: calc(100% + 1rem + 30px);
  margin-left: calc(-0.5rem - 15px);
  margin-right: calc(-0.5rem - 15px);
  border-bottom: 1px solid #dee2e6;
  margin-bottom: 25px;
}
@media (max-width: 767.98px) {
  .tertiary-navigation.full-width-bottom-border {
    width: calc(100% + 1rem);
    margin-left: -0.5rem;
    margin-right: -0.5rem;
  }
}
.tertiary-navigation.full-width-bottom-border .row {
  margin: 0;
  padding-left: 0.5rem;
  padding-right: 0.5rem;
}
.tertiary-navigation .navitem,
.tertiary-navigation .navitem-divider {
  display: flex;
  margin-bottom: 25px;
}
.tertiary-navigation .navitem-divider {
  width: 1px;
  background-color: #dee2e6;
}
.tertiary-navigation > a.btn, .tertiary-navigation > div.urlselect {
  margin-bottom: 25px;
}
.tertiary-navigation .row {
  column-gap: 10px;
}
.tertiary-navigation .tertiary-navigation-selector .dropdown-toggle {
  padding: 0;
  font-size: 1.4rem;
  font-weight: bold;
}
.tertiary-navigation h2, .tertiary-navigation .h2 {
  font-size: 1.4rem;
  font-weight: bold;
  margin: 0;
  align-self: center;
}
.tertiary-navigation .navitem:not(:last-child),
.tertiary-navigation .navitem-divider:not(:last-child) {
  margin-right: 20px;
}
.tertiary-navigation .btn > div {
  max-width: 200px;
}

@media (max-width: 767.98px) {
  .tertiary-navigation .mform {
    padding-left: initial;
  }
}
@media (max-width: 575.98px) {
  .tertiary-navigation .page-toggler > p {
    font-size: 80%;
  }
}
.popover-process-monitor {
  position: fixed;
  right: 2rem;
  bottom: 5rem;
  left: initial;
  top: initial;
  width: 350px;
  height: auto;
}
.popover-process-monitor .process-list {
  max-height: 30vh;
  overflow: auto;
  scrollbar-width: thin;
  scrollbar-color: #6a737b #f8f9fa;
}
.popover-process-monitor .process-list::-webkit-scrollbar {
  width: 12px;
}
.popover-process-monitor .process-list::-webkit-scrollbar-track {
  background: #f8f9fa;
}
.popover-process-monitor .process-list::-webkit-scrollbar-thumb {
  background-color: #6a737b;
  border-radius: 20px;
  border: 3px solid #f8f9fa;
}
.popover-process-monitor .process-list::-webkit-scrollbar-thumb:hover {
  background-color: #495057;
}
.popover-process-monitor .queue-process {
  border-bottom: 1px solid #e9ecef;
}
.popover-process-monitor .queue-process:last-child {
  border-bottom: 0;
}

.moodlenet-share-dialog {
  min-height: 500px;
}
.moodlenet-share-dialog .modal-header .moodlenet-share-moodlenetinfo {
  align-items: baseline;
}
.moodlenet-share-dialog .modal-header .moodlenet-share-moodlenetinfo .moodlenet-logo {
  display: flex;
}
.moodlenet-share-dialog .modal-header .moodlenet-share-moodlenetinfo .moodlenet-logo .icon {
  max-width: initial;
  width: auto;
  height: 1.3rem;
}
.moodlenet-share-dialog .modal-header .moodlenet-share-moodlenetinfo .moodlenet-title {
  display: flex;
  padding-left: 0.5em;
}
.moodlenet-share-dialog .modal-header.no-border {
  border-bottom: none;
}
.moodlenet-share-dialog .modal-header.no-header-text .moodlenet-share-moodlenetinfo .moodlenet-title {
  display: none;
}
.moodlenet-share-dialog .modal-body .moodlenet-share-activity-info {
  color: #1d2125;
  border-color: #3f89cc;
  background-color: #e7f0f9;
  border-radius: 0.5rem;
  border-width: 1px;
  border-style: solid;
  padding: 0.6em 1.5em;
  margin-bottom: 1rem;
}
.moodlenet-share-dialog .modal-body .moodlenet-share-activity-info .moodlenet-share-activity-info-hr {
  border-bottom: 1px solid #dee2e6;
}
.moodlenet-share-dialog .modal-body .moodlenet-share-activity-info .moodlenet-activity-type,
.moodlenet-share-dialog .modal-body .moodlenet-share-activity-info .moodlenet-activity-name {
  display: block;
}
.moodlenet-share-dialog .modal-body .moodlenet-share-notice {
  background-color: #f8f9fa;
  padding: 1rem;
}
.moodlenet-share-dialog .modal-body .moodlenet-share-modal-content .loading-icon .icon {
  max-width: 4em;
  max-height: 4em;
  font-size: 4em;
}
.moodlenet-share-dialog .modal-body .moodlenet-share-modal-content .moodlenet-circle-status {
  height: 18rem;
  margin: auto;
}
.moodlenet-share-dialog .modal-body .moodlenet-share-modal-content .moodlenet-circle-status.success {
  background: radial-gradient(circle, rgba(25, 143, 81, 0.1) 9rem, transparent 9rem);
}
.moodlenet-share-dialog .modal-body .moodlenet-share-modal-content .moodlenet-circle-status.fail {
  background: radial-gradient(circle, rgba(202, 49, 32, 0.1) 9rem, transparent 9rem);
}
.moodlenet-share-dialog .modal-body .moodlenet-share-modal-content .moodlenet-circle-status span {
  display: block;
  margin: auto;
}
.moodlenet-share-dialog .modal-body .moodlenet-share-modal-content .moodlenet-circle-status span.status-icon .icon {
  font-size: 8rem;
  max-width: initial;
  margin: 0;
}
.moodlenet-share-dialog .modal-footer .moodlenet-share-to {
  margin-right: auto;
}

/**
 * Dropdown menu Moodle specific styles.
 */
.dropdown-item a {
  display: block;
  width: 100%;
  color: #1d2125;
}
.dropdown-item.active a, .dropdown-item:active a, .dropdown-item:hover a, .dropdown-item:focus a, .dropdown-item:focus-within a {
  color: #fff;
}
.dropdown-item[aria-current=true], .dropdown-item[aria-selected=true] {
  position: relative;
  display: flex;
  align-items: center;
}
.dropdown-item[aria-current=true]:before, .dropdown-item[aria-selected=true]:before {
  font: var(--fa-font-solid);
  content: "\f00c";
  position: absolute;
  left: 0.4rem;
  font-size: 0.7rem;
}

.dropdown-menu {
  width: fit-content;
}
.dropdown-menu:has(.dropdown-item-active) .dropdown-item {
  padding-left: 1.5rem;
}

.dropdown-item:hover.text-primary,
.dropdown-item:hover a.text-primary,
.dropdown-item:hover span.text-primary, .dropdown-item:focus.text-primary,
.dropdown-item:focus a.text-primary,
.dropdown-item:focus span.text-primary {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-secondary,
.dropdown-item:hover a.text-secondary,
.dropdown-item:hover span.text-secondary, .dropdown-item:focus.text-secondary,
.dropdown-item:focus a.text-secondary,
.dropdown-item:focus span.text-secondary {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-success,
.dropdown-item:hover a.text-success,
.dropdown-item:hover span.text-success, .dropdown-item:focus.text-success,
.dropdown-item:focus a.text-success,
.dropdown-item:focus span.text-success {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-info,
.dropdown-item:hover a.text-info,
.dropdown-item:hover span.text-info, .dropdown-item:focus.text-info,
.dropdown-item:focus a.text-info,
.dropdown-item:focus span.text-info {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-warning,
.dropdown-item:hover a.text-warning,
.dropdown-item:hover span.text-warning, .dropdown-item:focus.text-warning,
.dropdown-item:focus a.text-warning,
.dropdown-item:focus span.text-warning {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-danger,
.dropdown-item:hover a.text-danger,
.dropdown-item:hover span.text-danger, .dropdown-item:focus.text-danger,
.dropdown-item:focus a.text-danger,
.dropdown-item:focus span.text-danger {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-light,
.dropdown-item:hover a.text-light,
.dropdown-item:hover span.text-light, .dropdown-item:focus.text-light,
.dropdown-item:focus a.text-light,
.dropdown-item:focus span.text-light {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-dark,
.dropdown-item:hover a.text-dark,
.dropdown-item:hover span.text-dark, .dropdown-item:focus.text-dark,
.dropdown-item:focus a.text-dark,
.dropdown-item:focus span.text-dark {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-primary-emphasis,
.dropdown-item:hover a.text-primary-emphasis,
.dropdown-item:hover span.text-primary-emphasis, .dropdown-item:focus.text-primary-emphasis,
.dropdown-item:focus a.text-primary-emphasis,
.dropdown-item:focus span.text-primary-emphasis {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-secondary-emphasis,
.dropdown-item:hover a.text-secondary-emphasis,
.dropdown-item:hover span.text-secondary-emphasis, .dropdown-item:focus.text-secondary-emphasis,
.dropdown-item:focus a.text-secondary-emphasis,
.dropdown-item:focus span.text-secondary-emphasis {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-success-emphasis,
.dropdown-item:hover a.text-success-emphasis,
.dropdown-item:hover span.text-success-emphasis, .dropdown-item:focus.text-success-emphasis,
.dropdown-item:focus a.text-success-emphasis,
.dropdown-item:focus span.text-success-emphasis {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-info-emphasis,
.dropdown-item:hover a.text-info-emphasis,
.dropdown-item:hover span.text-info-emphasis, .dropdown-item:focus.text-info-emphasis,
.dropdown-item:focus a.text-info-emphasis,
.dropdown-item:focus span.text-info-emphasis {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-warning-emphasis,
.dropdown-item:hover a.text-warning-emphasis,
.dropdown-item:hover span.text-warning-emphasis, .dropdown-item:focus.text-warning-emphasis,
.dropdown-item:focus a.text-warning-emphasis,
.dropdown-item:focus span.text-warning-emphasis {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-danger-emphasis,
.dropdown-item:hover a.text-danger-emphasis,
.dropdown-item:hover span.text-danger-emphasis, .dropdown-item:focus.text-danger-emphasis,
.dropdown-item:focus a.text-danger-emphasis,
.dropdown-item:focus span.text-danger-emphasis {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-light-emphasis,
.dropdown-item:hover a.text-light-emphasis,
.dropdown-item:hover span.text-light-emphasis, .dropdown-item:focus.text-light-emphasis,
.dropdown-item:focus a.text-light-emphasis,
.dropdown-item:focus span.text-light-emphasis {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

.dropdown-item:hover.text-dark-emphasis,
.dropdown-item:hover a.text-dark-emphasis,
.dropdown-item:hover span.text-dark-emphasis, .dropdown-item:focus.text-dark-emphasis,
.dropdown-item:focus a.text-dark-emphasis,
.dropdown-item:focus span.text-dark-emphasis {
  color: #fff !important; /* stylelint-disable-line declaration-no-important */
}

/**
 Deprecations
 */
.course-content .stateready .section .spinner {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  display: none;
}
body.behat-site .course-content .stateready .section .spinner,
.course-content .stateready .section .spinner body.behat-site.course-content .stateready .section .spinner, body.themedesignermode .course-content .stateready .section .spinner,
.course-content .stateready .section .spinner body.themedesignermode.course-content .stateready .section .spinner {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .course-content .stateready .section .spinner::before,
.course-content .stateready .section .spinner body.behat-site.course-content .stateready .section .spinner::before, body.themedesignermode .course-content .stateready .section .spinner::before,
.course-content .stateready .section .spinner body.themedesignermode.course-content .stateready .section .spinner::before {
  content: "Deprecated style in use (.course-content .stateready .section .spinner)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

/**
 * Some backwards compatibility for Bootstrap v4.
 */
.media {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  display: flex;
  align-items: flex-start;
}
body.behat-site .media,
.media body.behat-site.media, body.themedesignermode .media,
.media body.themedesignermode.media {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .media::before,
.media body.behat-site.media::before, body.themedesignermode .media::before,
.media body.themedesignermode.media::before {
  content: "Deprecated style in use (.media)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.media-body {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  flex: 1;
}
body.behat-site .media-body,
.media-body body.behat-site.media-body, body.themedesignermode .media-body,
.media-body body.themedesignermode.media-body {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .media-body::before,
.media-body body.behat-site.media-body::before, body.themedesignermode .media-body::before,
.media-body body.themedesignermode.media-body::before {
  content: "Deprecated style in use (.media-body)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.badge-primary {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  color: #fff;
  background-color: #0f6cbf;
}
body.behat-site .badge-primary,
.badge-primary body.behat-site.badge-primary, body.themedesignermode .badge-primary,
.badge-primary body.themedesignermode.badge-primary {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .badge-primary::before,
.badge-primary body.behat-site.badge-primary::before, body.themedesignermode .badge-primary::before,
.badge-primary body.themedesignermode.badge-primary::before {
  content: "Deprecated style in use (.badge-primary)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
a.badge-primary:hover, a.badge-primary:focus {
  color: #fff;
  background-color: #0b5190;
}
a.badge-primary:focus, a.badge-primary.focus {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.5);
}

.badge-secondary {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  color: #1d2125;
  background-color: #ced4da;
}
body.behat-site .badge-secondary,
.badge-secondary body.behat-site.badge-secondary, body.themedesignermode .badge-secondary,
.badge-secondary body.themedesignermode.badge-secondary {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .badge-secondary::before,
.badge-secondary body.behat-site.badge-secondary::before, body.themedesignermode .badge-secondary::before,
.badge-secondary body.themedesignermode.badge-secondary::before {
  content: "Deprecated style in use (.badge-secondary)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
a.badge-secondary:hover, a.badge-secondary:focus {
  color: #1d2125;
  background-color: #b1bbc4;
}
a.badge-secondary:focus, a.badge-secondary.focus {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(206, 212, 218, 0.5);
}

.badge-success {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  color: #fff;
  background-color: #357a32;
}
body.behat-site .badge-success,
.badge-success body.behat-site.badge-success, body.themedesignermode .badge-success,
.badge-success body.themedesignermode.badge-success {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .badge-success::before,
.badge-success body.behat-site.badge-success::before, body.themedesignermode .badge-success::before,
.badge-success body.themedesignermode.badge-success::before {
  content: "Deprecated style in use (.badge-success)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
a.badge-success:hover, a.badge-success:focus {
  color: #fff;
  background-color: #255623;
}
a.badge-success:focus, a.badge-success.focus {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(53, 122, 50, 0.5);
}

.badge-info {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  color: #fff;
  background-color: #008196;
}
body.behat-site .badge-info,
.badge-info body.behat-site.badge-info, body.themedesignermode .badge-info,
.badge-info body.themedesignermode.badge-info {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .badge-info::before,
.badge-info body.behat-site.badge-info::before, body.themedesignermode .badge-info::before,
.badge-info body.themedesignermode.badge-info::before {
  content: "Deprecated style in use (.badge-info)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
a.badge-info:hover, a.badge-info:focus {
  color: #fff;
  background-color: #005563;
}
a.badge-info:focus, a.badge-info.focus {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(0, 129, 150, 0.5);
}

.badge-warning {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  color: #1d2125;
  background-color: #f0ad4e;
}
body.behat-site .badge-warning,
.badge-warning body.behat-site.badge-warning, body.themedesignermode .badge-warning,
.badge-warning body.themedesignermode.badge-warning {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .badge-warning::before,
.badge-warning body.behat-site.badge-warning::before, body.themedesignermode .badge-warning::before,
.badge-warning body.themedesignermode.badge-warning::before {
  content: "Deprecated style in use (.badge-warning)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
a.badge-warning:hover, a.badge-warning:focus {
  color: #1d2125;
  background-color: #ec971f;
}
a.badge-warning:focus, a.badge-warning.focus {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(240, 173, 78, 0.5);
}

.badge-danger {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  color: #fff;
  background-color: #ca3120;
}
body.behat-site .badge-danger,
.badge-danger body.behat-site.badge-danger, body.themedesignermode .badge-danger,
.badge-danger body.themedesignermode.badge-danger {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .badge-danger::before,
.badge-danger body.behat-site.badge-danger::before, body.themedesignermode .badge-danger::before,
.badge-danger body.themedesignermode.badge-danger::before {
  content: "Deprecated style in use (.badge-danger)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
a.badge-danger:hover, a.badge-danger:focus {
  color: #fff;
  background-color: #9e2619;
}
a.badge-danger:focus, a.badge-danger.focus {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(202, 49, 32, 0.5);
}

.badge-light {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  color: #1d2125;
  background-color: #f8f9fa;
}
body.behat-site .badge-light,
.badge-light body.behat-site.badge-light, body.themedesignermode .badge-light,
.badge-light body.themedesignermode.badge-light {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .badge-light::before,
.badge-light body.behat-site.badge-light::before, body.themedesignermode .badge-light::before,
.badge-light body.themedesignermode.badge-light::before {
  content: "Deprecated style in use (.badge-light)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
a.badge-light:hover, a.badge-light:focus {
  color: #1d2125;
  background-color: #dae0e5;
}
a.badge-light:focus, a.badge-light.focus {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(248, 249, 250, 0.5);
}

.badge-dark {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  color: #fff;
  background-color: #1d2125;
}
body.behat-site .badge-dark,
.badge-dark body.behat-site.badge-dark, body.themedesignermode .badge-dark,
.badge-dark body.themedesignermode.badge-dark {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .badge-dark::before,
.badge-dark body.behat-site.badge-dark::before, body.themedesignermode .badge-dark::before,
.badge-dark body.themedesignermode.badge-dark::before {
  content: "Deprecated style in use (.badge-dark)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
a.badge-dark:hover, a.badge-dark:focus {
  color: #fff;
  background-color: #070808;
}
a.badge-dark:focus, a.badge-dark.focus {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(29, 33, 37, 0.5);
}

.badge-pill {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-right: 0.6em;
  padding-left: 0.6em;
  border-radius: 10rem;
}
body.behat-site .badge-pill,
.badge-pill body.behat-site.badge-pill, body.themedesignermode .badge-pill,
.badge-pill body.themedesignermode.badge-pill {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .badge-pill::before,
.badge-pill body.behat-site.badge-pill::before, body.themedesignermode .badge-pill::before,
.badge-pill body.themedesignermode.badge-pill::before {
  content: "Deprecated style in use (.badge-pill)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.form-group {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-bottom: 1rem;
}
body.behat-site .form-group,
.form-group body.behat-site.form-group, body.themedesignermode .form-group,
.form-group body.themedesignermode.form-group {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .form-group::before,
.form-group body.behat-site.form-group::before, body.themedesignermode .form-group::before,
.form-group body.themedesignermode.form-group::before {
  content: "Deprecated style in use (.form-group)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.form-inline {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  display: flex;
  flex-flow: row wrap;
  align-items: center;
}
body.behat-site .form-inline,
.form-inline body.behat-site.form-inline, body.themedesignermode .form-inline,
.form-inline body.themedesignermode.form-inline {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .form-inline::before,
.form-inline body.behat-site.form-inline::before, body.themedesignermode .form-inline::before,
.form-inline body.themedesignermode.form-inline::before {
  content: "Deprecated style in use (.form-inline)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
.form-inline .form-check {
  width: 100%;
}
@media (min-width: 576px) {
  .form-inline label {
    display: flex;
    align-items: center;
    justify-content: center;
    margin-bottom: 0;
  }
  .form-inline .form-group {
    display: flex;
    flex: 0 0 auto;
    flex-flow: row wrap;
    align-items: center;
    margin-bottom: 0;
  }
  .form-inline .form-control {
    display: inline-block;
    width: auto;
    vertical-align: middle;
  }
  .form-inline .form-control-plaintext {
    display: inline-block;
  }
  .form-inline .input-group,
  .form-inline .custom-select {
    width: auto;
  }
  .form-inline .form-check {
    display: flex;
    align-items: center;
    justify-content: center;
    width: auto;
    padding-left: 0;
  }
  .form-inline .form-check-input {
    position: relative;
    flex-shrink: 0;
    margin-top: 0;
    margin-right: 0.25rem;
    margin-left: 0;
  }
  .form-inline .custom-control {
    align-items: center;
    justify-content: center;
  }
  .form-inline .custom-control-label {
    margin-bottom: 0;
  }
}

.card-deck {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
}
body.behat-site .card-deck,
.card-deck body.behat-site.card-deck, body.themedesignermode .card-deck,
.card-deck body.themedesignermode.card-deck {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .card-deck::before,
.card-deck body.behat-site.card-deck::before, body.themedesignermode .card-deck::before,
.card-deck body.themedesignermode.card-deck::before {
  content: "Deprecated style in use (.card-deck)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
.card-deck .card {
  margin-bottom: 15px;
}
@media (min-width: 576px) {
  .card-deck {
    display: flex;
    flex-flow: row wrap;
    margin-right: -15px;
    margin-left: -15px;
  }
  .card-deck .card {
    flex: 1 0 0%;
    margin-right: 15px;
    margin-bottom: 0;
    margin-left: 15px;
  }
}

.no-gutters {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: 0;
  margin-left: 0;
}
body.behat-site .no-gutters,
.no-gutters body.behat-site.no-gutters, body.themedesignermode .no-gutters,
.no-gutters body.themedesignermode.no-gutters {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .no-gutters::before,
.no-gutters body.behat-site.no-gutters::before, body.themedesignermode .no-gutters::before,
.no-gutters body.themedesignermode.no-gutters::before {
  content: "Deprecated style in use (.no-gutters)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
.no-gutters > .col,
.no-gutters > [class*=col-] {
  padding-right: 0;
  padding-left: 0;
}

.mr-0 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: 0 !important;
}
body.behat-site .mr-0,
.mr-0 body.behat-site.mr-0, body.themedesignermode .mr-0,
.mr-0 body.themedesignermode.mr-0 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .mr-0::before,
.mr-0 body.behat-site.mr-0::before, body.themedesignermode .mr-0::before,
.mr-0 body.themedesignermode.mr-0::before {
  content: "Deprecated style in use (.mr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.ml-0 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-left: 0 !important;
}
body.behat-site .ml-0,
.ml-0 body.behat-site.ml-0, body.themedesignermode .ml-0,
.ml-0 body.themedesignermode.ml-0 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .ml-0::before,
.ml-0 body.behat-site.ml-0::before, body.themedesignermode .ml-0::before,
.ml-0 body.themedesignermode.ml-0::before {
  content: "Deprecated style in use (.ml)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.mr-1 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: 0.25rem !important;
}
body.behat-site .mr-1,
.mr-1 body.behat-site.mr-1, body.themedesignermode .mr-1,
.mr-1 body.themedesignermode.mr-1 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .mr-1::before,
.mr-1 body.behat-site.mr-1::before, body.themedesignermode .mr-1::before,
.mr-1 body.themedesignermode.mr-1::before {
  content: "Deprecated style in use (.mr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.ml-1 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-left: 0.25rem !important;
}
body.behat-site .ml-1,
.ml-1 body.behat-site.ml-1, body.themedesignermode .ml-1,
.ml-1 body.themedesignermode.ml-1 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .ml-1::before,
.ml-1 body.behat-site.ml-1::before, body.themedesignermode .ml-1::before,
.ml-1 body.themedesignermode.ml-1::before {
  content: "Deprecated style in use (.ml)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.mr-2 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: 0.5rem !important;
}
body.behat-site .mr-2,
.mr-2 body.behat-site.mr-2, body.themedesignermode .mr-2,
.mr-2 body.themedesignermode.mr-2 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .mr-2::before,
.mr-2 body.behat-site.mr-2::before, body.themedesignermode .mr-2::before,
.mr-2 body.themedesignermode.mr-2::before {
  content: "Deprecated style in use (.mr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.ml-2 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-left: 0.5rem !important;
}
body.behat-site .ml-2,
.ml-2 body.behat-site.ml-2, body.themedesignermode .ml-2,
.ml-2 body.themedesignermode.ml-2 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .ml-2::before,
.ml-2 body.behat-site.ml-2::before, body.themedesignermode .ml-2::before,
.ml-2 body.themedesignermode.ml-2::before {
  content: "Deprecated style in use (.ml)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.mr-3 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: 1rem !important;
}
body.behat-site .mr-3,
.mr-3 body.behat-site.mr-3, body.themedesignermode .mr-3,
.mr-3 body.themedesignermode.mr-3 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .mr-3::before,
.mr-3 body.behat-site.mr-3::before, body.themedesignermode .mr-3::before,
.mr-3 body.themedesignermode.mr-3::before {
  content: "Deprecated style in use (.mr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.ml-3 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-left: 1rem !important;
}
body.behat-site .ml-3,
.ml-3 body.behat-site.ml-3, body.themedesignermode .ml-3,
.ml-3 body.themedesignermode.ml-3 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .ml-3::before,
.ml-3 body.behat-site.ml-3::before, body.themedesignermode .ml-3::before,
.ml-3 body.themedesignermode.ml-3::before {
  content: "Deprecated style in use (.ml)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.mr-4 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: 1.5rem !important;
}
body.behat-site .mr-4,
.mr-4 body.behat-site.mr-4, body.themedesignermode .mr-4,
.mr-4 body.themedesignermode.mr-4 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .mr-4::before,
.mr-4 body.behat-site.mr-4::before, body.themedesignermode .mr-4::before,
.mr-4 body.themedesignermode.mr-4::before {
  content: "Deprecated style in use (.mr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.ml-4 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-left: 1.5rem !important;
}
body.behat-site .ml-4,
.ml-4 body.behat-site.ml-4, body.themedesignermode .ml-4,
.ml-4 body.themedesignermode.ml-4 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .ml-4::before,
.ml-4 body.behat-site.ml-4::before, body.themedesignermode .ml-4::before,
.ml-4 body.themedesignermode.ml-4::before {
  content: "Deprecated style in use (.ml)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.mr-5 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: 2rem !important;
}
body.behat-site .mr-5,
.mr-5 body.behat-site.mr-5, body.themedesignermode .mr-5,
.mr-5 body.themedesignermode.mr-5 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .mr-5::before,
.mr-5 body.behat-site.mr-5::before, body.themedesignermode .mr-5::before,
.mr-5 body.themedesignermode.mr-5::before {
  content: "Deprecated style in use (.mr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.ml-5 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-left: 2rem !important;
}
body.behat-site .ml-5,
.ml-5 body.behat-site.ml-5, body.themedesignermode .ml-5,
.ml-5 body.themedesignermode.ml-5 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .ml-5::before,
.ml-5 body.behat-site.ml-5::before, body.themedesignermode .ml-5::before,
.ml-5 body.themedesignermode.ml-5::before {
  content: "Deprecated style in use (.ml)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.mr-6 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: 3rem !important;
}
body.behat-site .mr-6,
.mr-6 body.behat-site.mr-6, body.themedesignermode .mr-6,
.mr-6 body.themedesignermode.mr-6 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .mr-6::before,
.mr-6 body.behat-site.mr-6::before, body.themedesignermode .mr-6::before,
.mr-6 body.themedesignermode.mr-6::before {
  content: "Deprecated style in use (.mr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.ml-6 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-left: 3rem !important;
}
body.behat-site .ml-6,
.ml-6 body.behat-site.ml-6, body.themedesignermode .ml-6,
.ml-6 body.themedesignermode.ml-6 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .ml-6::before,
.ml-6 body.behat-site.ml-6::before, body.themedesignermode .ml-6::before,
.ml-6 body.themedesignermode.ml-6::before {
  content: "Deprecated style in use (.ml)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.pr-0 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-right: 0 !important;
}
body.behat-site .pr-0,
.pr-0 body.behat-site.pr-0, body.themedesignermode .pr-0,
.pr-0 body.themedesignermode.pr-0 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .pr-0::before,
.pr-0 body.behat-site.pr-0::before, body.themedesignermode .pr-0::before,
.pr-0 body.themedesignermode.pr-0::before {
  content: "Deprecated style in use (.pr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.pl-0 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-left: 0 !important;
}
body.behat-site .pl-0,
.pl-0 body.behat-site.pl-0, body.themedesignermode .pl-0,
.pl-0 body.themedesignermode.pl-0 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .pl-0::before,
.pl-0 body.behat-site.pl-0::before, body.themedesignermode .pl-0::before,
.pl-0 body.themedesignermode.pl-0::before {
  content: "Deprecated style in use (.pl)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.pr-1 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-right: 0.25rem !important;
}
body.behat-site .pr-1,
.pr-1 body.behat-site.pr-1, body.themedesignermode .pr-1,
.pr-1 body.themedesignermode.pr-1 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .pr-1::before,
.pr-1 body.behat-site.pr-1::before, body.themedesignermode .pr-1::before,
.pr-1 body.themedesignermode.pr-1::before {
  content: "Deprecated style in use (.pr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.pl-1 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-left: 0.25rem !important;
}
body.behat-site .pl-1,
.pl-1 body.behat-site.pl-1, body.themedesignermode .pl-1,
.pl-1 body.themedesignermode.pl-1 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .pl-1::before,
.pl-1 body.behat-site.pl-1::before, body.themedesignermode .pl-1::before,
.pl-1 body.themedesignermode.pl-1::before {
  content: "Deprecated style in use (.pl)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.pr-2 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-right: 0.5rem !important;
}
body.behat-site .pr-2,
.pr-2 body.behat-site.pr-2, body.themedesignermode .pr-2,
.pr-2 body.themedesignermode.pr-2 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .pr-2::before,
.pr-2 body.behat-site.pr-2::before, body.themedesignermode .pr-2::before,
.pr-2 body.themedesignermode.pr-2::before {
  content: "Deprecated style in use (.pr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.pl-2 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-left: 0.5rem !important;
}
body.behat-site .pl-2,
.pl-2 body.behat-site.pl-2, body.themedesignermode .pl-2,
.pl-2 body.themedesignermode.pl-2 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .pl-2::before,
.pl-2 body.behat-site.pl-2::before, body.themedesignermode .pl-2::before,
.pl-2 body.themedesignermode.pl-2::before {
  content: "Deprecated style in use (.pl)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.pr-3 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-right: 1rem !important;
}
body.behat-site .pr-3,
.pr-3 body.behat-site.pr-3, body.themedesignermode .pr-3,
.pr-3 body.themedesignermode.pr-3 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .pr-3::before,
.pr-3 body.behat-site.pr-3::before, body.themedesignermode .pr-3::before,
.pr-3 body.themedesignermode.pr-3::before {
  content: "Deprecated style in use (.pr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.pl-3 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-left: 1rem !important;
}
body.behat-site .pl-3,
.pl-3 body.behat-site.pl-3, body.themedesignermode .pl-3,
.pl-3 body.themedesignermode.pl-3 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .pl-3::before,
.pl-3 body.behat-site.pl-3::before, body.themedesignermode .pl-3::before,
.pl-3 body.themedesignermode.pl-3::before {
  content: "Deprecated style in use (.pl)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.pr-4 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-right: 1.5rem !important;
}
body.behat-site .pr-4,
.pr-4 body.behat-site.pr-4, body.themedesignermode .pr-4,
.pr-4 body.themedesignermode.pr-4 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .pr-4::before,
.pr-4 body.behat-site.pr-4::before, body.themedesignermode .pr-4::before,
.pr-4 body.themedesignermode.pr-4::before {
  content: "Deprecated style in use (.pr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.pl-4 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-left: 1.5rem !important;
}
body.behat-site .pl-4,
.pl-4 body.behat-site.pl-4, body.themedesignermode .pl-4,
.pl-4 body.themedesignermode.pl-4 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .pl-4::before,
.pl-4 body.behat-site.pl-4::before, body.themedesignermode .pl-4::before,
.pl-4 body.themedesignermode.pl-4::before {
  content: "Deprecated style in use (.pl)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.pr-5 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-right: 2rem !important;
}
body.behat-site .pr-5,
.pr-5 body.behat-site.pr-5, body.themedesignermode .pr-5,
.pr-5 body.themedesignermode.pr-5 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .pr-5::before,
.pr-5 body.behat-site.pr-5::before, body.themedesignermode .pr-5::before,
.pr-5 body.themedesignermode.pr-5::before {
  content: "Deprecated style in use (.pr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.pl-5 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-left: 2rem !important;
}
body.behat-site .pl-5,
.pl-5 body.behat-site.pl-5, body.themedesignermode .pl-5,
.pl-5 body.themedesignermode.pl-5 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .pl-5::before,
.pl-5 body.behat-site.pl-5::before, body.themedesignermode .pl-5::before,
.pl-5 body.themedesignermode.pl-5::before {
  content: "Deprecated style in use (.pl)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.pr-6 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-right: 3rem !important;
}
body.behat-site .pr-6,
.pr-6 body.behat-site.pr-6, body.themedesignermode .pr-6,
.pr-6 body.themedesignermode.pr-6 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .pr-6::before,
.pr-6 body.behat-site.pr-6::before, body.themedesignermode .pr-6::before,
.pr-6 body.themedesignermode.pr-6::before {
  content: "Deprecated style in use (.pr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.pl-6 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-left: 3rem !important;
}
body.behat-site .pl-6,
.pl-6 body.behat-site.pl-6, body.themedesignermode .pl-6,
.pl-6 body.themedesignermode.pl-6 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .pl-6::before,
.pl-6 body.behat-site.pl-6::before, body.themedesignermode .pl-6::before,
.pl-6 body.themedesignermode.pl-6::before {
  content: "Deprecated style in use (.pl)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.mr-n1 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: -0.25rem !important;
}
body.behat-site .mr-n1,
.mr-n1 body.behat-site.mr-n1, body.themedesignermode .mr-n1,
.mr-n1 body.themedesignermode.mr-n1 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .mr-n1::before,
.mr-n1 body.behat-site.mr-n1::before, body.themedesignermode .mr-n1::before,
.mr-n1 body.themedesignermode.mr-n1::before {
  content: "Deprecated style in use (.mr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.ml-n1 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-left: -0.25rem !important;
}
body.behat-site .ml-n1,
.ml-n1 body.behat-site.ml-n1, body.themedesignermode .ml-n1,
.ml-n1 body.themedesignermode.ml-n1 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .ml-n1::before,
.ml-n1 body.behat-site.ml-n1::before, body.themedesignermode .ml-n1::before,
.ml-n1 body.themedesignermode.ml-n1::before {
  content: "Deprecated style in use (.ml)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.mr-n2 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: -0.5rem !important;
}
body.behat-site .mr-n2,
.mr-n2 body.behat-site.mr-n2, body.themedesignermode .mr-n2,
.mr-n2 body.themedesignermode.mr-n2 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .mr-n2::before,
.mr-n2 body.behat-site.mr-n2::before, body.themedesignermode .mr-n2::before,
.mr-n2 body.themedesignermode.mr-n2::before {
  content: "Deprecated style in use (.mr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.ml-n2 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-left: -0.5rem !important;
}
body.behat-site .ml-n2,
.ml-n2 body.behat-site.ml-n2, body.themedesignermode .ml-n2,
.ml-n2 body.themedesignermode.ml-n2 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .ml-n2::before,
.ml-n2 body.behat-site.ml-n2::before, body.themedesignermode .ml-n2::before,
.ml-n2 body.themedesignermode.ml-n2::before {
  content: "Deprecated style in use (.ml)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.mr-n3 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: -1rem !important;
}
body.behat-site .mr-n3,
.mr-n3 body.behat-site.mr-n3, body.themedesignermode .mr-n3,
.mr-n3 body.themedesignermode.mr-n3 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .mr-n3::before,
.mr-n3 body.behat-site.mr-n3::before, body.themedesignermode .mr-n3::before,
.mr-n3 body.themedesignermode.mr-n3::before {
  content: "Deprecated style in use (.mr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.ml-n3 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-left: -1rem !important;
}
body.behat-site .ml-n3,
.ml-n3 body.behat-site.ml-n3, body.themedesignermode .ml-n3,
.ml-n3 body.themedesignermode.ml-n3 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .ml-n3::before,
.ml-n3 body.behat-site.ml-n3::before, body.themedesignermode .ml-n3::before,
.ml-n3 body.themedesignermode.ml-n3::before {
  content: "Deprecated style in use (.ml)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.mr-n4 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: -1.5rem !important;
}
body.behat-site .mr-n4,
.mr-n4 body.behat-site.mr-n4, body.themedesignermode .mr-n4,
.mr-n4 body.themedesignermode.mr-n4 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .mr-n4::before,
.mr-n4 body.behat-site.mr-n4::before, body.themedesignermode .mr-n4::before,
.mr-n4 body.themedesignermode.mr-n4::before {
  content: "Deprecated style in use (.mr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.ml-n4 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-left: -1.5rem !important;
}
body.behat-site .ml-n4,
.ml-n4 body.behat-site.ml-n4, body.themedesignermode .ml-n4,
.ml-n4 body.themedesignermode.ml-n4 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .ml-n4::before,
.ml-n4 body.behat-site.ml-n4::before, body.themedesignermode .ml-n4::before,
.ml-n4 body.themedesignermode.ml-n4::before {
  content: "Deprecated style in use (.ml)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.mr-n5 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: -2rem !important;
}
body.behat-site .mr-n5,
.mr-n5 body.behat-site.mr-n5, body.themedesignermode .mr-n5,
.mr-n5 body.themedesignermode.mr-n5 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .mr-n5::before,
.mr-n5 body.behat-site.mr-n5::before, body.themedesignermode .mr-n5::before,
.mr-n5 body.themedesignermode.mr-n5::before {
  content: "Deprecated style in use (.mr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.ml-n5 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-left: -2rem !important;
}
body.behat-site .ml-n5,
.ml-n5 body.behat-site.ml-n5, body.themedesignermode .ml-n5,
.ml-n5 body.themedesignermode.ml-n5 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .ml-n5::before,
.ml-n5 body.behat-site.ml-n5::before, body.themedesignermode .ml-n5::before,
.ml-n5 body.themedesignermode.ml-n5::before {
  content: "Deprecated style in use (.ml)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.mr-n6 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: -3rem !important;
}
body.behat-site .mr-n6,
.mr-n6 body.behat-site.mr-n6, body.themedesignermode .mr-n6,
.mr-n6 body.themedesignermode.mr-n6 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .mr-n6::before,
.mr-n6 body.behat-site.mr-n6::before, body.themedesignermode .mr-n6::before,
.mr-n6 body.themedesignermode.mr-n6::before {
  content: "Deprecated style in use (.mr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.ml-n6 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-left: -3rem !important;
}
body.behat-site .ml-n6,
.ml-n6 body.behat-site.ml-n6, body.themedesignermode .ml-n6,
.ml-n6 body.themedesignermode.ml-n6 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .ml-n6::before,
.ml-n6 body.behat-site.ml-n6::before, body.themedesignermode .ml-n6::before,
.ml-n6 body.themedesignermode.ml-n6::before {
  content: "Deprecated style in use (.ml)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.mr-auto {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-right: auto !important;
}
body.behat-site .mr-auto,
.mr-auto body.behat-site.mr-auto, body.themedesignermode .mr-auto,
.mr-auto body.themedesignermode.mr-auto {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .mr-auto::before,
.mr-auto body.behat-site.mr-auto::before, body.themedesignermode .mr-auto::before,
.mr-auto body.themedesignermode.mr-auto::before {
  content: "Deprecated style in use (.mr)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.ml-auto {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  margin-left: auto !important;
}
body.behat-site .ml-auto,
.ml-auto body.behat-site.ml-auto, body.themedesignermode .ml-auto,
.ml-auto body.themedesignermode.ml-auto {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .ml-auto::before,
.ml-auto body.behat-site.ml-auto::before, body.themedesignermode .ml-auto::before,
.ml-auto body.themedesignermode.ml-auto::before {
  content: "Deprecated style in use (.ml)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

@media (min-width: 576px) {
  .mr-sm-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0 !important;
  }
  body.behat-site .mr-sm-0,
  .mr-sm-0 body.behat-site.mr-sm-0, body.themedesignermode .mr-sm-0,
  .mr-sm-0 body.themedesignermode.mr-sm-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-sm-0::before,
  .mr-sm-0 body.behat-site.mr-sm-0::before, body.themedesignermode .mr-sm-0::before,
  .mr-sm-0 body.themedesignermode.mr-sm-0::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-sm-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0 !important;
  }
  body.behat-site .ml-sm-0,
  .ml-sm-0 body.behat-site.ml-sm-0, body.themedesignermode .ml-sm-0,
  .ml-sm-0 body.themedesignermode.ml-sm-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-sm-0::before,
  .ml-sm-0 body.behat-site.ml-sm-0::before, body.themedesignermode .ml-sm-0::before,
  .ml-sm-0 body.themedesignermode.ml-sm-0::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-sm-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0.25rem !important;
  }
  body.behat-site .mr-sm-1,
  .mr-sm-1 body.behat-site.mr-sm-1, body.themedesignermode .mr-sm-1,
  .mr-sm-1 body.themedesignermode.mr-sm-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-sm-1::before,
  .mr-sm-1 body.behat-site.mr-sm-1::before, body.themedesignermode .mr-sm-1::before,
  .mr-sm-1 body.themedesignermode.mr-sm-1::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-sm-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0.25rem !important;
  }
  body.behat-site .ml-sm-1,
  .ml-sm-1 body.behat-site.ml-sm-1, body.themedesignermode .ml-sm-1,
  .ml-sm-1 body.themedesignermode.ml-sm-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-sm-1::before,
  .ml-sm-1 body.behat-site.ml-sm-1::before, body.themedesignermode .ml-sm-1::before,
  .ml-sm-1 body.themedesignermode.ml-sm-1::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-sm-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0.5rem !important;
  }
  body.behat-site .mr-sm-2,
  .mr-sm-2 body.behat-site.mr-sm-2, body.themedesignermode .mr-sm-2,
  .mr-sm-2 body.themedesignermode.mr-sm-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-sm-2::before,
  .mr-sm-2 body.behat-site.mr-sm-2::before, body.themedesignermode .mr-sm-2::before,
  .mr-sm-2 body.themedesignermode.mr-sm-2::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-sm-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0.5rem !important;
  }
  body.behat-site .ml-sm-2,
  .ml-sm-2 body.behat-site.ml-sm-2, body.themedesignermode .ml-sm-2,
  .ml-sm-2 body.themedesignermode.ml-sm-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-sm-2::before,
  .ml-sm-2 body.behat-site.ml-sm-2::before, body.themedesignermode .ml-sm-2::before,
  .ml-sm-2 body.themedesignermode.ml-sm-2::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-sm-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 1rem !important;
  }
  body.behat-site .mr-sm-3,
  .mr-sm-3 body.behat-site.mr-sm-3, body.themedesignermode .mr-sm-3,
  .mr-sm-3 body.themedesignermode.mr-sm-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-sm-3::before,
  .mr-sm-3 body.behat-site.mr-sm-3::before, body.themedesignermode .mr-sm-3::before,
  .mr-sm-3 body.themedesignermode.mr-sm-3::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-sm-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 1rem !important;
  }
  body.behat-site .ml-sm-3,
  .ml-sm-3 body.behat-site.ml-sm-3, body.themedesignermode .ml-sm-3,
  .ml-sm-3 body.themedesignermode.ml-sm-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-sm-3::before,
  .ml-sm-3 body.behat-site.ml-sm-3::before, body.themedesignermode .ml-sm-3::before,
  .ml-sm-3 body.themedesignermode.ml-sm-3::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-sm-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 1.5rem !important;
  }
  body.behat-site .mr-sm-4,
  .mr-sm-4 body.behat-site.mr-sm-4, body.themedesignermode .mr-sm-4,
  .mr-sm-4 body.themedesignermode.mr-sm-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-sm-4::before,
  .mr-sm-4 body.behat-site.mr-sm-4::before, body.themedesignermode .mr-sm-4::before,
  .mr-sm-4 body.themedesignermode.mr-sm-4::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-sm-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 1.5rem !important;
  }
  body.behat-site .ml-sm-4,
  .ml-sm-4 body.behat-site.ml-sm-4, body.themedesignermode .ml-sm-4,
  .ml-sm-4 body.themedesignermode.ml-sm-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-sm-4::before,
  .ml-sm-4 body.behat-site.ml-sm-4::before, body.themedesignermode .ml-sm-4::before,
  .ml-sm-4 body.themedesignermode.ml-sm-4::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-sm-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 2rem !important;
  }
  body.behat-site .mr-sm-5,
  .mr-sm-5 body.behat-site.mr-sm-5, body.themedesignermode .mr-sm-5,
  .mr-sm-5 body.themedesignermode.mr-sm-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-sm-5::before,
  .mr-sm-5 body.behat-site.mr-sm-5::before, body.themedesignermode .mr-sm-5::before,
  .mr-sm-5 body.themedesignermode.mr-sm-5::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-sm-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 2rem !important;
  }
  body.behat-site .ml-sm-5,
  .ml-sm-5 body.behat-site.ml-sm-5, body.themedesignermode .ml-sm-5,
  .ml-sm-5 body.themedesignermode.ml-sm-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-sm-5::before,
  .ml-sm-5 body.behat-site.ml-sm-5::before, body.themedesignermode .ml-sm-5::before,
  .ml-sm-5 body.themedesignermode.ml-sm-5::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-sm-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 3rem !important;
  }
  body.behat-site .mr-sm-6,
  .mr-sm-6 body.behat-site.mr-sm-6, body.themedesignermode .mr-sm-6,
  .mr-sm-6 body.themedesignermode.mr-sm-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-sm-6::before,
  .mr-sm-6 body.behat-site.mr-sm-6::before, body.themedesignermode .mr-sm-6::before,
  .mr-sm-6 body.themedesignermode.mr-sm-6::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-sm-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 3rem !important;
  }
  body.behat-site .ml-sm-6,
  .ml-sm-6 body.behat-site.ml-sm-6, body.themedesignermode .ml-sm-6,
  .ml-sm-6 body.themedesignermode.ml-sm-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-sm-6::before,
  .ml-sm-6 body.behat-site.ml-sm-6::before, body.themedesignermode .ml-sm-6::before,
  .ml-sm-6 body.themedesignermode.ml-sm-6::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-sm-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0 !important;
  }
  body.behat-site .pr-sm-0,
  .pr-sm-0 body.behat-site.pr-sm-0, body.themedesignermode .pr-sm-0,
  .pr-sm-0 body.themedesignermode.pr-sm-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-sm-0::before,
  .pr-sm-0 body.behat-site.pr-sm-0::before, body.themedesignermode .pr-sm-0::before,
  .pr-sm-0 body.themedesignermode.pr-sm-0::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-sm-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0 !important;
  }
  body.behat-site .pl-sm-0,
  .pl-sm-0 body.behat-site.pl-sm-0, body.themedesignermode .pl-sm-0,
  .pl-sm-0 body.themedesignermode.pl-sm-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-sm-0::before,
  .pl-sm-0 body.behat-site.pl-sm-0::before, body.themedesignermode .pl-sm-0::before,
  .pl-sm-0 body.themedesignermode.pl-sm-0::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-sm-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0.25rem !important;
  }
  body.behat-site .pr-sm-1,
  .pr-sm-1 body.behat-site.pr-sm-1, body.themedesignermode .pr-sm-1,
  .pr-sm-1 body.themedesignermode.pr-sm-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-sm-1::before,
  .pr-sm-1 body.behat-site.pr-sm-1::before, body.themedesignermode .pr-sm-1::before,
  .pr-sm-1 body.themedesignermode.pr-sm-1::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-sm-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0.25rem !important;
  }
  body.behat-site .pl-sm-1,
  .pl-sm-1 body.behat-site.pl-sm-1, body.themedesignermode .pl-sm-1,
  .pl-sm-1 body.themedesignermode.pl-sm-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-sm-1::before,
  .pl-sm-1 body.behat-site.pl-sm-1::before, body.themedesignermode .pl-sm-1::before,
  .pl-sm-1 body.themedesignermode.pl-sm-1::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-sm-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0.5rem !important;
  }
  body.behat-site .pr-sm-2,
  .pr-sm-2 body.behat-site.pr-sm-2, body.themedesignermode .pr-sm-2,
  .pr-sm-2 body.themedesignermode.pr-sm-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-sm-2::before,
  .pr-sm-2 body.behat-site.pr-sm-2::before, body.themedesignermode .pr-sm-2::before,
  .pr-sm-2 body.themedesignermode.pr-sm-2::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-sm-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0.5rem !important;
  }
  body.behat-site .pl-sm-2,
  .pl-sm-2 body.behat-site.pl-sm-2, body.themedesignermode .pl-sm-2,
  .pl-sm-2 body.themedesignermode.pl-sm-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-sm-2::before,
  .pl-sm-2 body.behat-site.pl-sm-2::before, body.themedesignermode .pl-sm-2::before,
  .pl-sm-2 body.themedesignermode.pl-sm-2::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-sm-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 1rem !important;
  }
  body.behat-site .pr-sm-3,
  .pr-sm-3 body.behat-site.pr-sm-3, body.themedesignermode .pr-sm-3,
  .pr-sm-3 body.themedesignermode.pr-sm-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-sm-3::before,
  .pr-sm-3 body.behat-site.pr-sm-3::before, body.themedesignermode .pr-sm-3::before,
  .pr-sm-3 body.themedesignermode.pr-sm-3::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-sm-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 1rem !important;
  }
  body.behat-site .pl-sm-3,
  .pl-sm-3 body.behat-site.pl-sm-3, body.themedesignermode .pl-sm-3,
  .pl-sm-3 body.themedesignermode.pl-sm-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-sm-3::before,
  .pl-sm-3 body.behat-site.pl-sm-3::before, body.themedesignermode .pl-sm-3::before,
  .pl-sm-3 body.themedesignermode.pl-sm-3::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-sm-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 1.5rem !important;
  }
  body.behat-site .pr-sm-4,
  .pr-sm-4 body.behat-site.pr-sm-4, body.themedesignermode .pr-sm-4,
  .pr-sm-4 body.themedesignermode.pr-sm-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-sm-4::before,
  .pr-sm-4 body.behat-site.pr-sm-4::before, body.themedesignermode .pr-sm-4::before,
  .pr-sm-4 body.themedesignermode.pr-sm-4::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-sm-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 1.5rem !important;
  }
  body.behat-site .pl-sm-4,
  .pl-sm-4 body.behat-site.pl-sm-4, body.themedesignermode .pl-sm-4,
  .pl-sm-4 body.themedesignermode.pl-sm-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-sm-4::before,
  .pl-sm-4 body.behat-site.pl-sm-4::before, body.themedesignermode .pl-sm-4::before,
  .pl-sm-4 body.themedesignermode.pl-sm-4::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-sm-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 2rem !important;
  }
  body.behat-site .pr-sm-5,
  .pr-sm-5 body.behat-site.pr-sm-5, body.themedesignermode .pr-sm-5,
  .pr-sm-5 body.themedesignermode.pr-sm-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-sm-5::before,
  .pr-sm-5 body.behat-site.pr-sm-5::before, body.themedesignermode .pr-sm-5::before,
  .pr-sm-5 body.themedesignermode.pr-sm-5::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-sm-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 2rem !important;
  }
  body.behat-site .pl-sm-5,
  .pl-sm-5 body.behat-site.pl-sm-5, body.themedesignermode .pl-sm-5,
  .pl-sm-5 body.themedesignermode.pl-sm-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-sm-5::before,
  .pl-sm-5 body.behat-site.pl-sm-5::before, body.themedesignermode .pl-sm-5::before,
  .pl-sm-5 body.themedesignermode.pl-sm-5::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-sm-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 3rem !important;
  }
  body.behat-site .pr-sm-6,
  .pr-sm-6 body.behat-site.pr-sm-6, body.themedesignermode .pr-sm-6,
  .pr-sm-6 body.themedesignermode.pr-sm-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-sm-6::before,
  .pr-sm-6 body.behat-site.pr-sm-6::before, body.themedesignermode .pr-sm-6::before,
  .pr-sm-6 body.themedesignermode.pr-sm-6::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-sm-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 3rem !important;
  }
  body.behat-site .pl-sm-6,
  .pl-sm-6 body.behat-site.pl-sm-6, body.themedesignermode .pl-sm-6,
  .pl-sm-6 body.themedesignermode.pl-sm-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-sm-6::before,
  .pl-sm-6 body.behat-site.pl-sm-6::before, body.themedesignermode .pl-sm-6::before,
  .pl-sm-6 body.themedesignermode.pl-sm-6::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-sm-n1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -0.25rem !important;
  }
  body.behat-site .mr-sm-n1,
  .mr-sm-n1 body.behat-site.mr-sm-n1, body.themedesignermode .mr-sm-n1,
  .mr-sm-n1 body.themedesignermode.mr-sm-n1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-sm-n1::before,
  .mr-sm-n1 body.behat-site.mr-sm-n1::before, body.themedesignermode .mr-sm-n1::before,
  .mr-sm-n1 body.themedesignermode.mr-sm-n1::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-sm-n1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -0.25rem !important;
  }
  body.behat-site .ml-sm-n1,
  .ml-sm-n1 body.behat-site.ml-sm-n1, body.themedesignermode .ml-sm-n1,
  .ml-sm-n1 body.themedesignermode.ml-sm-n1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-sm-n1::before,
  .ml-sm-n1 body.behat-site.ml-sm-n1::before, body.themedesignermode .ml-sm-n1::before,
  .ml-sm-n1 body.themedesignermode.ml-sm-n1::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-sm-n2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -0.5rem !important;
  }
  body.behat-site .mr-sm-n2,
  .mr-sm-n2 body.behat-site.mr-sm-n2, body.themedesignermode .mr-sm-n2,
  .mr-sm-n2 body.themedesignermode.mr-sm-n2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-sm-n2::before,
  .mr-sm-n2 body.behat-site.mr-sm-n2::before, body.themedesignermode .mr-sm-n2::before,
  .mr-sm-n2 body.themedesignermode.mr-sm-n2::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-sm-n2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -0.5rem !important;
  }
  body.behat-site .ml-sm-n2,
  .ml-sm-n2 body.behat-site.ml-sm-n2, body.themedesignermode .ml-sm-n2,
  .ml-sm-n2 body.themedesignermode.ml-sm-n2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-sm-n2::before,
  .ml-sm-n2 body.behat-site.ml-sm-n2::before, body.themedesignermode .ml-sm-n2::before,
  .ml-sm-n2 body.themedesignermode.ml-sm-n2::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-sm-n3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -1rem !important;
  }
  body.behat-site .mr-sm-n3,
  .mr-sm-n3 body.behat-site.mr-sm-n3, body.themedesignermode .mr-sm-n3,
  .mr-sm-n3 body.themedesignermode.mr-sm-n3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-sm-n3::before,
  .mr-sm-n3 body.behat-site.mr-sm-n3::before, body.themedesignermode .mr-sm-n3::before,
  .mr-sm-n3 body.themedesignermode.mr-sm-n3::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-sm-n3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -1rem !important;
  }
  body.behat-site .ml-sm-n3,
  .ml-sm-n3 body.behat-site.ml-sm-n3, body.themedesignermode .ml-sm-n3,
  .ml-sm-n3 body.themedesignermode.ml-sm-n3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-sm-n3::before,
  .ml-sm-n3 body.behat-site.ml-sm-n3::before, body.themedesignermode .ml-sm-n3::before,
  .ml-sm-n3 body.themedesignermode.ml-sm-n3::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-sm-n4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -1.5rem !important;
  }
  body.behat-site .mr-sm-n4,
  .mr-sm-n4 body.behat-site.mr-sm-n4, body.themedesignermode .mr-sm-n4,
  .mr-sm-n4 body.themedesignermode.mr-sm-n4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-sm-n4::before,
  .mr-sm-n4 body.behat-site.mr-sm-n4::before, body.themedesignermode .mr-sm-n4::before,
  .mr-sm-n4 body.themedesignermode.mr-sm-n4::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-sm-n4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -1.5rem !important;
  }
  body.behat-site .ml-sm-n4,
  .ml-sm-n4 body.behat-site.ml-sm-n4, body.themedesignermode .ml-sm-n4,
  .ml-sm-n4 body.themedesignermode.ml-sm-n4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-sm-n4::before,
  .ml-sm-n4 body.behat-site.ml-sm-n4::before, body.themedesignermode .ml-sm-n4::before,
  .ml-sm-n4 body.themedesignermode.ml-sm-n4::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-sm-n5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -2rem !important;
  }
  body.behat-site .mr-sm-n5,
  .mr-sm-n5 body.behat-site.mr-sm-n5, body.themedesignermode .mr-sm-n5,
  .mr-sm-n5 body.themedesignermode.mr-sm-n5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-sm-n5::before,
  .mr-sm-n5 body.behat-site.mr-sm-n5::before, body.themedesignermode .mr-sm-n5::before,
  .mr-sm-n5 body.themedesignermode.mr-sm-n5::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-sm-n5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -2rem !important;
  }
  body.behat-site .ml-sm-n5,
  .ml-sm-n5 body.behat-site.ml-sm-n5, body.themedesignermode .ml-sm-n5,
  .ml-sm-n5 body.themedesignermode.ml-sm-n5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-sm-n5::before,
  .ml-sm-n5 body.behat-site.ml-sm-n5::before, body.themedesignermode .ml-sm-n5::before,
  .ml-sm-n5 body.themedesignermode.ml-sm-n5::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-sm-n6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -3rem !important;
  }
  body.behat-site .mr-sm-n6,
  .mr-sm-n6 body.behat-site.mr-sm-n6, body.themedesignermode .mr-sm-n6,
  .mr-sm-n6 body.themedesignermode.mr-sm-n6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-sm-n6::before,
  .mr-sm-n6 body.behat-site.mr-sm-n6::before, body.themedesignermode .mr-sm-n6::before,
  .mr-sm-n6 body.themedesignermode.mr-sm-n6::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-sm-n6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -3rem !important;
  }
  body.behat-site .ml-sm-n6,
  .ml-sm-n6 body.behat-site.ml-sm-n6, body.themedesignermode .ml-sm-n6,
  .ml-sm-n6 body.themedesignermode.ml-sm-n6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-sm-n6::before,
  .ml-sm-n6 body.behat-site.ml-sm-n6::before, body.themedesignermode .ml-sm-n6::before,
  .ml-sm-n6 body.themedesignermode.ml-sm-n6::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-sm-auto {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: auto !important;
  }
  body.behat-site .mr-sm-auto,
  .mr-sm-auto body.behat-site.mr-sm-auto, body.themedesignermode .mr-sm-auto,
  .mr-sm-auto body.themedesignermode.mr-sm-auto {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-sm-auto::before,
  .mr-sm-auto body.behat-site.mr-sm-auto::before, body.themedesignermode .mr-sm-auto::before,
  .mr-sm-auto body.themedesignermode.mr-sm-auto::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-sm-auto {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: auto !important;
  }
  body.behat-site .ml-sm-auto,
  .ml-sm-auto body.behat-site.ml-sm-auto, body.themedesignermode .ml-sm-auto,
  .ml-sm-auto body.themedesignermode.ml-sm-auto {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-sm-auto::before,
  .ml-sm-auto body.behat-site.ml-sm-auto::before, body.themedesignermode .ml-sm-auto::before,
  .ml-sm-auto body.themedesignermode.ml-sm-auto::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
@media (min-width: 768px) {
  .mr-md-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0 !important;
  }
  body.behat-site .mr-md-0,
  .mr-md-0 body.behat-site.mr-md-0, body.themedesignermode .mr-md-0,
  .mr-md-0 body.themedesignermode.mr-md-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-md-0::before,
  .mr-md-0 body.behat-site.mr-md-0::before, body.themedesignermode .mr-md-0::before,
  .mr-md-0 body.themedesignermode.mr-md-0::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-md-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0 !important;
  }
  body.behat-site .ml-md-0,
  .ml-md-0 body.behat-site.ml-md-0, body.themedesignermode .ml-md-0,
  .ml-md-0 body.themedesignermode.ml-md-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-md-0::before,
  .ml-md-0 body.behat-site.ml-md-0::before, body.themedesignermode .ml-md-0::before,
  .ml-md-0 body.themedesignermode.ml-md-0::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-md-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0.25rem !important;
  }
  body.behat-site .mr-md-1,
  .mr-md-1 body.behat-site.mr-md-1, body.themedesignermode .mr-md-1,
  .mr-md-1 body.themedesignermode.mr-md-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-md-1::before,
  .mr-md-1 body.behat-site.mr-md-1::before, body.themedesignermode .mr-md-1::before,
  .mr-md-1 body.themedesignermode.mr-md-1::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-md-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0.25rem !important;
  }
  body.behat-site .ml-md-1,
  .ml-md-1 body.behat-site.ml-md-1, body.themedesignermode .ml-md-1,
  .ml-md-1 body.themedesignermode.ml-md-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-md-1::before,
  .ml-md-1 body.behat-site.ml-md-1::before, body.themedesignermode .ml-md-1::before,
  .ml-md-1 body.themedesignermode.ml-md-1::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-md-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0.5rem !important;
  }
  body.behat-site .mr-md-2,
  .mr-md-2 body.behat-site.mr-md-2, body.themedesignermode .mr-md-2,
  .mr-md-2 body.themedesignermode.mr-md-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-md-2::before,
  .mr-md-2 body.behat-site.mr-md-2::before, body.themedesignermode .mr-md-2::before,
  .mr-md-2 body.themedesignermode.mr-md-2::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-md-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0.5rem !important;
  }
  body.behat-site .ml-md-2,
  .ml-md-2 body.behat-site.ml-md-2, body.themedesignermode .ml-md-2,
  .ml-md-2 body.themedesignermode.ml-md-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-md-2::before,
  .ml-md-2 body.behat-site.ml-md-2::before, body.themedesignermode .ml-md-2::before,
  .ml-md-2 body.themedesignermode.ml-md-2::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-md-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 1rem !important;
  }
  body.behat-site .mr-md-3,
  .mr-md-3 body.behat-site.mr-md-3, body.themedesignermode .mr-md-3,
  .mr-md-3 body.themedesignermode.mr-md-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-md-3::before,
  .mr-md-3 body.behat-site.mr-md-3::before, body.themedesignermode .mr-md-3::before,
  .mr-md-3 body.themedesignermode.mr-md-3::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-md-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 1rem !important;
  }
  body.behat-site .ml-md-3,
  .ml-md-3 body.behat-site.ml-md-3, body.themedesignermode .ml-md-3,
  .ml-md-3 body.themedesignermode.ml-md-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-md-3::before,
  .ml-md-3 body.behat-site.ml-md-3::before, body.themedesignermode .ml-md-3::before,
  .ml-md-3 body.themedesignermode.ml-md-3::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-md-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 1.5rem !important;
  }
  body.behat-site .mr-md-4,
  .mr-md-4 body.behat-site.mr-md-4, body.themedesignermode .mr-md-4,
  .mr-md-4 body.themedesignermode.mr-md-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-md-4::before,
  .mr-md-4 body.behat-site.mr-md-4::before, body.themedesignermode .mr-md-4::before,
  .mr-md-4 body.themedesignermode.mr-md-4::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-md-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 1.5rem !important;
  }
  body.behat-site .ml-md-4,
  .ml-md-4 body.behat-site.ml-md-4, body.themedesignermode .ml-md-4,
  .ml-md-4 body.themedesignermode.ml-md-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-md-4::before,
  .ml-md-4 body.behat-site.ml-md-4::before, body.themedesignermode .ml-md-4::before,
  .ml-md-4 body.themedesignermode.ml-md-4::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-md-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 2rem !important;
  }
  body.behat-site .mr-md-5,
  .mr-md-5 body.behat-site.mr-md-5, body.themedesignermode .mr-md-5,
  .mr-md-5 body.themedesignermode.mr-md-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-md-5::before,
  .mr-md-5 body.behat-site.mr-md-5::before, body.themedesignermode .mr-md-5::before,
  .mr-md-5 body.themedesignermode.mr-md-5::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-md-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 2rem !important;
  }
  body.behat-site .ml-md-5,
  .ml-md-5 body.behat-site.ml-md-5, body.themedesignermode .ml-md-5,
  .ml-md-5 body.themedesignermode.ml-md-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-md-5::before,
  .ml-md-5 body.behat-site.ml-md-5::before, body.themedesignermode .ml-md-5::before,
  .ml-md-5 body.themedesignermode.ml-md-5::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-md-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 3rem !important;
  }
  body.behat-site .mr-md-6,
  .mr-md-6 body.behat-site.mr-md-6, body.themedesignermode .mr-md-6,
  .mr-md-6 body.themedesignermode.mr-md-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-md-6::before,
  .mr-md-6 body.behat-site.mr-md-6::before, body.themedesignermode .mr-md-6::before,
  .mr-md-6 body.themedesignermode.mr-md-6::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-md-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 3rem !important;
  }
  body.behat-site .ml-md-6,
  .ml-md-6 body.behat-site.ml-md-6, body.themedesignermode .ml-md-6,
  .ml-md-6 body.themedesignermode.ml-md-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-md-6::before,
  .ml-md-6 body.behat-site.ml-md-6::before, body.themedesignermode .ml-md-6::before,
  .ml-md-6 body.themedesignermode.ml-md-6::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-md-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0 !important;
  }
  body.behat-site .pr-md-0,
  .pr-md-0 body.behat-site.pr-md-0, body.themedesignermode .pr-md-0,
  .pr-md-0 body.themedesignermode.pr-md-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-md-0::before,
  .pr-md-0 body.behat-site.pr-md-0::before, body.themedesignermode .pr-md-0::before,
  .pr-md-0 body.themedesignermode.pr-md-0::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-md-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0 !important;
  }
  body.behat-site .pl-md-0,
  .pl-md-0 body.behat-site.pl-md-0, body.themedesignermode .pl-md-0,
  .pl-md-0 body.themedesignermode.pl-md-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-md-0::before,
  .pl-md-0 body.behat-site.pl-md-0::before, body.themedesignermode .pl-md-0::before,
  .pl-md-0 body.themedesignermode.pl-md-0::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-md-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0.25rem !important;
  }
  body.behat-site .pr-md-1,
  .pr-md-1 body.behat-site.pr-md-1, body.themedesignermode .pr-md-1,
  .pr-md-1 body.themedesignermode.pr-md-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-md-1::before,
  .pr-md-1 body.behat-site.pr-md-1::before, body.themedesignermode .pr-md-1::before,
  .pr-md-1 body.themedesignermode.pr-md-1::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-md-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0.25rem !important;
  }
  body.behat-site .pl-md-1,
  .pl-md-1 body.behat-site.pl-md-1, body.themedesignermode .pl-md-1,
  .pl-md-1 body.themedesignermode.pl-md-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-md-1::before,
  .pl-md-1 body.behat-site.pl-md-1::before, body.themedesignermode .pl-md-1::before,
  .pl-md-1 body.themedesignermode.pl-md-1::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-md-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0.5rem !important;
  }
  body.behat-site .pr-md-2,
  .pr-md-2 body.behat-site.pr-md-2, body.themedesignermode .pr-md-2,
  .pr-md-2 body.themedesignermode.pr-md-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-md-2::before,
  .pr-md-2 body.behat-site.pr-md-2::before, body.themedesignermode .pr-md-2::before,
  .pr-md-2 body.themedesignermode.pr-md-2::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-md-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0.5rem !important;
  }
  body.behat-site .pl-md-2,
  .pl-md-2 body.behat-site.pl-md-2, body.themedesignermode .pl-md-2,
  .pl-md-2 body.themedesignermode.pl-md-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-md-2::before,
  .pl-md-2 body.behat-site.pl-md-2::before, body.themedesignermode .pl-md-2::before,
  .pl-md-2 body.themedesignermode.pl-md-2::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-md-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 1rem !important;
  }
  body.behat-site .pr-md-3,
  .pr-md-3 body.behat-site.pr-md-3, body.themedesignermode .pr-md-3,
  .pr-md-3 body.themedesignermode.pr-md-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-md-3::before,
  .pr-md-3 body.behat-site.pr-md-3::before, body.themedesignermode .pr-md-3::before,
  .pr-md-3 body.themedesignermode.pr-md-3::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-md-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 1rem !important;
  }
  body.behat-site .pl-md-3,
  .pl-md-3 body.behat-site.pl-md-3, body.themedesignermode .pl-md-3,
  .pl-md-3 body.themedesignermode.pl-md-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-md-3::before,
  .pl-md-3 body.behat-site.pl-md-3::before, body.themedesignermode .pl-md-3::before,
  .pl-md-3 body.themedesignermode.pl-md-3::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-md-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 1.5rem !important;
  }
  body.behat-site .pr-md-4,
  .pr-md-4 body.behat-site.pr-md-4, body.themedesignermode .pr-md-4,
  .pr-md-4 body.themedesignermode.pr-md-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-md-4::before,
  .pr-md-4 body.behat-site.pr-md-4::before, body.themedesignermode .pr-md-4::before,
  .pr-md-4 body.themedesignermode.pr-md-4::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-md-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 1.5rem !important;
  }
  body.behat-site .pl-md-4,
  .pl-md-4 body.behat-site.pl-md-4, body.themedesignermode .pl-md-4,
  .pl-md-4 body.themedesignermode.pl-md-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-md-4::before,
  .pl-md-4 body.behat-site.pl-md-4::before, body.themedesignermode .pl-md-4::before,
  .pl-md-4 body.themedesignermode.pl-md-4::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-md-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 2rem !important;
  }
  body.behat-site .pr-md-5,
  .pr-md-5 body.behat-site.pr-md-5, body.themedesignermode .pr-md-5,
  .pr-md-5 body.themedesignermode.pr-md-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-md-5::before,
  .pr-md-5 body.behat-site.pr-md-5::before, body.themedesignermode .pr-md-5::before,
  .pr-md-5 body.themedesignermode.pr-md-5::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-md-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 2rem !important;
  }
  body.behat-site .pl-md-5,
  .pl-md-5 body.behat-site.pl-md-5, body.themedesignermode .pl-md-5,
  .pl-md-5 body.themedesignermode.pl-md-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-md-5::before,
  .pl-md-5 body.behat-site.pl-md-5::before, body.themedesignermode .pl-md-5::before,
  .pl-md-5 body.themedesignermode.pl-md-5::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-md-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 3rem !important;
  }
  body.behat-site .pr-md-6,
  .pr-md-6 body.behat-site.pr-md-6, body.themedesignermode .pr-md-6,
  .pr-md-6 body.themedesignermode.pr-md-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-md-6::before,
  .pr-md-6 body.behat-site.pr-md-6::before, body.themedesignermode .pr-md-6::before,
  .pr-md-6 body.themedesignermode.pr-md-6::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-md-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 3rem !important;
  }
  body.behat-site .pl-md-6,
  .pl-md-6 body.behat-site.pl-md-6, body.themedesignermode .pl-md-6,
  .pl-md-6 body.themedesignermode.pl-md-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-md-6::before,
  .pl-md-6 body.behat-site.pl-md-6::before, body.themedesignermode .pl-md-6::before,
  .pl-md-6 body.themedesignermode.pl-md-6::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-md-n1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -0.25rem !important;
  }
  body.behat-site .mr-md-n1,
  .mr-md-n1 body.behat-site.mr-md-n1, body.themedesignermode .mr-md-n1,
  .mr-md-n1 body.themedesignermode.mr-md-n1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-md-n1::before,
  .mr-md-n1 body.behat-site.mr-md-n1::before, body.themedesignermode .mr-md-n1::before,
  .mr-md-n1 body.themedesignermode.mr-md-n1::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-md-n1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -0.25rem !important;
  }
  body.behat-site .ml-md-n1,
  .ml-md-n1 body.behat-site.ml-md-n1, body.themedesignermode .ml-md-n1,
  .ml-md-n1 body.themedesignermode.ml-md-n1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-md-n1::before,
  .ml-md-n1 body.behat-site.ml-md-n1::before, body.themedesignermode .ml-md-n1::before,
  .ml-md-n1 body.themedesignermode.ml-md-n1::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-md-n2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -0.5rem !important;
  }
  body.behat-site .mr-md-n2,
  .mr-md-n2 body.behat-site.mr-md-n2, body.themedesignermode .mr-md-n2,
  .mr-md-n2 body.themedesignermode.mr-md-n2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-md-n2::before,
  .mr-md-n2 body.behat-site.mr-md-n2::before, body.themedesignermode .mr-md-n2::before,
  .mr-md-n2 body.themedesignermode.mr-md-n2::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-md-n2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -0.5rem !important;
  }
  body.behat-site .ml-md-n2,
  .ml-md-n2 body.behat-site.ml-md-n2, body.themedesignermode .ml-md-n2,
  .ml-md-n2 body.themedesignermode.ml-md-n2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-md-n2::before,
  .ml-md-n2 body.behat-site.ml-md-n2::before, body.themedesignermode .ml-md-n2::before,
  .ml-md-n2 body.themedesignermode.ml-md-n2::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-md-n3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -1rem !important;
  }
  body.behat-site .mr-md-n3,
  .mr-md-n3 body.behat-site.mr-md-n3, body.themedesignermode .mr-md-n3,
  .mr-md-n3 body.themedesignermode.mr-md-n3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-md-n3::before,
  .mr-md-n3 body.behat-site.mr-md-n3::before, body.themedesignermode .mr-md-n3::before,
  .mr-md-n3 body.themedesignermode.mr-md-n3::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-md-n3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -1rem !important;
  }
  body.behat-site .ml-md-n3,
  .ml-md-n3 body.behat-site.ml-md-n3, body.themedesignermode .ml-md-n3,
  .ml-md-n3 body.themedesignermode.ml-md-n3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-md-n3::before,
  .ml-md-n3 body.behat-site.ml-md-n3::before, body.themedesignermode .ml-md-n3::before,
  .ml-md-n3 body.themedesignermode.ml-md-n3::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-md-n4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -1.5rem !important;
  }
  body.behat-site .mr-md-n4,
  .mr-md-n4 body.behat-site.mr-md-n4, body.themedesignermode .mr-md-n4,
  .mr-md-n4 body.themedesignermode.mr-md-n4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-md-n4::before,
  .mr-md-n4 body.behat-site.mr-md-n4::before, body.themedesignermode .mr-md-n4::before,
  .mr-md-n4 body.themedesignermode.mr-md-n4::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-md-n4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -1.5rem !important;
  }
  body.behat-site .ml-md-n4,
  .ml-md-n4 body.behat-site.ml-md-n4, body.themedesignermode .ml-md-n4,
  .ml-md-n4 body.themedesignermode.ml-md-n4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-md-n4::before,
  .ml-md-n4 body.behat-site.ml-md-n4::before, body.themedesignermode .ml-md-n4::before,
  .ml-md-n4 body.themedesignermode.ml-md-n4::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-md-n5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -2rem !important;
  }
  body.behat-site .mr-md-n5,
  .mr-md-n5 body.behat-site.mr-md-n5, body.themedesignermode .mr-md-n5,
  .mr-md-n5 body.themedesignermode.mr-md-n5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-md-n5::before,
  .mr-md-n5 body.behat-site.mr-md-n5::before, body.themedesignermode .mr-md-n5::before,
  .mr-md-n5 body.themedesignermode.mr-md-n5::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-md-n5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -2rem !important;
  }
  body.behat-site .ml-md-n5,
  .ml-md-n5 body.behat-site.ml-md-n5, body.themedesignermode .ml-md-n5,
  .ml-md-n5 body.themedesignermode.ml-md-n5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-md-n5::before,
  .ml-md-n5 body.behat-site.ml-md-n5::before, body.themedesignermode .ml-md-n5::before,
  .ml-md-n5 body.themedesignermode.ml-md-n5::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-md-n6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -3rem !important;
  }
  body.behat-site .mr-md-n6,
  .mr-md-n6 body.behat-site.mr-md-n6, body.themedesignermode .mr-md-n6,
  .mr-md-n6 body.themedesignermode.mr-md-n6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-md-n6::before,
  .mr-md-n6 body.behat-site.mr-md-n6::before, body.themedesignermode .mr-md-n6::before,
  .mr-md-n6 body.themedesignermode.mr-md-n6::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-md-n6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -3rem !important;
  }
  body.behat-site .ml-md-n6,
  .ml-md-n6 body.behat-site.ml-md-n6, body.themedesignermode .ml-md-n6,
  .ml-md-n6 body.themedesignermode.ml-md-n6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-md-n6::before,
  .ml-md-n6 body.behat-site.ml-md-n6::before, body.themedesignermode .ml-md-n6::before,
  .ml-md-n6 body.themedesignermode.ml-md-n6::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-md-auto {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: auto !important;
  }
  body.behat-site .mr-md-auto,
  .mr-md-auto body.behat-site.mr-md-auto, body.themedesignermode .mr-md-auto,
  .mr-md-auto body.themedesignermode.mr-md-auto {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-md-auto::before,
  .mr-md-auto body.behat-site.mr-md-auto::before, body.themedesignermode .mr-md-auto::before,
  .mr-md-auto body.themedesignermode.mr-md-auto::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-md-auto {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: auto !important;
  }
  body.behat-site .ml-md-auto,
  .ml-md-auto body.behat-site.ml-md-auto, body.themedesignermode .ml-md-auto,
  .ml-md-auto body.themedesignermode.ml-md-auto {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-md-auto::before,
  .ml-md-auto body.behat-site.ml-md-auto::before, body.themedesignermode .ml-md-auto::before,
  .ml-md-auto body.themedesignermode.ml-md-auto::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
@media (min-width: 992px) {
  .mr-lg-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0 !important;
  }
  body.behat-site .mr-lg-0,
  .mr-lg-0 body.behat-site.mr-lg-0, body.themedesignermode .mr-lg-0,
  .mr-lg-0 body.themedesignermode.mr-lg-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-lg-0::before,
  .mr-lg-0 body.behat-site.mr-lg-0::before, body.themedesignermode .mr-lg-0::before,
  .mr-lg-0 body.themedesignermode.mr-lg-0::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-lg-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0 !important;
  }
  body.behat-site .ml-lg-0,
  .ml-lg-0 body.behat-site.ml-lg-0, body.themedesignermode .ml-lg-0,
  .ml-lg-0 body.themedesignermode.ml-lg-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-lg-0::before,
  .ml-lg-0 body.behat-site.ml-lg-0::before, body.themedesignermode .ml-lg-0::before,
  .ml-lg-0 body.themedesignermode.ml-lg-0::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-lg-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0.25rem !important;
  }
  body.behat-site .mr-lg-1,
  .mr-lg-1 body.behat-site.mr-lg-1, body.themedesignermode .mr-lg-1,
  .mr-lg-1 body.themedesignermode.mr-lg-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-lg-1::before,
  .mr-lg-1 body.behat-site.mr-lg-1::before, body.themedesignermode .mr-lg-1::before,
  .mr-lg-1 body.themedesignermode.mr-lg-1::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-lg-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0.25rem !important;
  }
  body.behat-site .ml-lg-1,
  .ml-lg-1 body.behat-site.ml-lg-1, body.themedesignermode .ml-lg-1,
  .ml-lg-1 body.themedesignermode.ml-lg-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-lg-1::before,
  .ml-lg-1 body.behat-site.ml-lg-1::before, body.themedesignermode .ml-lg-1::before,
  .ml-lg-1 body.themedesignermode.ml-lg-1::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-lg-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0.5rem !important;
  }
  body.behat-site .mr-lg-2,
  .mr-lg-2 body.behat-site.mr-lg-2, body.themedesignermode .mr-lg-2,
  .mr-lg-2 body.themedesignermode.mr-lg-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-lg-2::before,
  .mr-lg-2 body.behat-site.mr-lg-2::before, body.themedesignermode .mr-lg-2::before,
  .mr-lg-2 body.themedesignermode.mr-lg-2::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-lg-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0.5rem !important;
  }
  body.behat-site .ml-lg-2,
  .ml-lg-2 body.behat-site.ml-lg-2, body.themedesignermode .ml-lg-2,
  .ml-lg-2 body.themedesignermode.ml-lg-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-lg-2::before,
  .ml-lg-2 body.behat-site.ml-lg-2::before, body.themedesignermode .ml-lg-2::before,
  .ml-lg-2 body.themedesignermode.ml-lg-2::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-lg-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 1rem !important;
  }
  body.behat-site .mr-lg-3,
  .mr-lg-3 body.behat-site.mr-lg-3, body.themedesignermode .mr-lg-3,
  .mr-lg-3 body.themedesignermode.mr-lg-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-lg-3::before,
  .mr-lg-3 body.behat-site.mr-lg-3::before, body.themedesignermode .mr-lg-3::before,
  .mr-lg-3 body.themedesignermode.mr-lg-3::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-lg-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 1rem !important;
  }
  body.behat-site .ml-lg-3,
  .ml-lg-3 body.behat-site.ml-lg-3, body.themedesignermode .ml-lg-3,
  .ml-lg-3 body.themedesignermode.ml-lg-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-lg-3::before,
  .ml-lg-3 body.behat-site.ml-lg-3::before, body.themedesignermode .ml-lg-3::before,
  .ml-lg-3 body.themedesignermode.ml-lg-3::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-lg-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 1.5rem !important;
  }
  body.behat-site .mr-lg-4,
  .mr-lg-4 body.behat-site.mr-lg-4, body.themedesignermode .mr-lg-4,
  .mr-lg-4 body.themedesignermode.mr-lg-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-lg-4::before,
  .mr-lg-4 body.behat-site.mr-lg-4::before, body.themedesignermode .mr-lg-4::before,
  .mr-lg-4 body.themedesignermode.mr-lg-4::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-lg-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 1.5rem !important;
  }
  body.behat-site .ml-lg-4,
  .ml-lg-4 body.behat-site.ml-lg-4, body.themedesignermode .ml-lg-4,
  .ml-lg-4 body.themedesignermode.ml-lg-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-lg-4::before,
  .ml-lg-4 body.behat-site.ml-lg-4::before, body.themedesignermode .ml-lg-4::before,
  .ml-lg-4 body.themedesignermode.ml-lg-4::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-lg-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 2rem !important;
  }
  body.behat-site .mr-lg-5,
  .mr-lg-5 body.behat-site.mr-lg-5, body.themedesignermode .mr-lg-5,
  .mr-lg-5 body.themedesignermode.mr-lg-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-lg-5::before,
  .mr-lg-5 body.behat-site.mr-lg-5::before, body.themedesignermode .mr-lg-5::before,
  .mr-lg-5 body.themedesignermode.mr-lg-5::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-lg-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 2rem !important;
  }
  body.behat-site .ml-lg-5,
  .ml-lg-5 body.behat-site.ml-lg-5, body.themedesignermode .ml-lg-5,
  .ml-lg-5 body.themedesignermode.ml-lg-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-lg-5::before,
  .ml-lg-5 body.behat-site.ml-lg-5::before, body.themedesignermode .ml-lg-5::before,
  .ml-lg-5 body.themedesignermode.ml-lg-5::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-lg-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 3rem !important;
  }
  body.behat-site .mr-lg-6,
  .mr-lg-6 body.behat-site.mr-lg-6, body.themedesignermode .mr-lg-6,
  .mr-lg-6 body.themedesignermode.mr-lg-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-lg-6::before,
  .mr-lg-6 body.behat-site.mr-lg-6::before, body.themedesignermode .mr-lg-6::before,
  .mr-lg-6 body.themedesignermode.mr-lg-6::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-lg-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 3rem !important;
  }
  body.behat-site .ml-lg-6,
  .ml-lg-6 body.behat-site.ml-lg-6, body.themedesignermode .ml-lg-6,
  .ml-lg-6 body.themedesignermode.ml-lg-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-lg-6::before,
  .ml-lg-6 body.behat-site.ml-lg-6::before, body.themedesignermode .ml-lg-6::before,
  .ml-lg-6 body.themedesignermode.ml-lg-6::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-lg-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0 !important;
  }
  body.behat-site .pr-lg-0,
  .pr-lg-0 body.behat-site.pr-lg-0, body.themedesignermode .pr-lg-0,
  .pr-lg-0 body.themedesignermode.pr-lg-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-lg-0::before,
  .pr-lg-0 body.behat-site.pr-lg-0::before, body.themedesignermode .pr-lg-0::before,
  .pr-lg-0 body.themedesignermode.pr-lg-0::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-lg-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0 !important;
  }
  body.behat-site .pl-lg-0,
  .pl-lg-0 body.behat-site.pl-lg-0, body.themedesignermode .pl-lg-0,
  .pl-lg-0 body.themedesignermode.pl-lg-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-lg-0::before,
  .pl-lg-0 body.behat-site.pl-lg-0::before, body.themedesignermode .pl-lg-0::before,
  .pl-lg-0 body.themedesignermode.pl-lg-0::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-lg-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0.25rem !important;
  }
  body.behat-site .pr-lg-1,
  .pr-lg-1 body.behat-site.pr-lg-1, body.themedesignermode .pr-lg-1,
  .pr-lg-1 body.themedesignermode.pr-lg-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-lg-1::before,
  .pr-lg-1 body.behat-site.pr-lg-1::before, body.themedesignermode .pr-lg-1::before,
  .pr-lg-1 body.themedesignermode.pr-lg-1::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-lg-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0.25rem !important;
  }
  body.behat-site .pl-lg-1,
  .pl-lg-1 body.behat-site.pl-lg-1, body.themedesignermode .pl-lg-1,
  .pl-lg-1 body.themedesignermode.pl-lg-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-lg-1::before,
  .pl-lg-1 body.behat-site.pl-lg-1::before, body.themedesignermode .pl-lg-1::before,
  .pl-lg-1 body.themedesignermode.pl-lg-1::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-lg-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0.5rem !important;
  }
  body.behat-site .pr-lg-2,
  .pr-lg-2 body.behat-site.pr-lg-2, body.themedesignermode .pr-lg-2,
  .pr-lg-2 body.themedesignermode.pr-lg-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-lg-2::before,
  .pr-lg-2 body.behat-site.pr-lg-2::before, body.themedesignermode .pr-lg-2::before,
  .pr-lg-2 body.themedesignermode.pr-lg-2::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-lg-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0.5rem !important;
  }
  body.behat-site .pl-lg-2,
  .pl-lg-2 body.behat-site.pl-lg-2, body.themedesignermode .pl-lg-2,
  .pl-lg-2 body.themedesignermode.pl-lg-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-lg-2::before,
  .pl-lg-2 body.behat-site.pl-lg-2::before, body.themedesignermode .pl-lg-2::before,
  .pl-lg-2 body.themedesignermode.pl-lg-2::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-lg-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 1rem !important;
  }
  body.behat-site .pr-lg-3,
  .pr-lg-3 body.behat-site.pr-lg-3, body.themedesignermode .pr-lg-3,
  .pr-lg-3 body.themedesignermode.pr-lg-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-lg-3::before,
  .pr-lg-3 body.behat-site.pr-lg-3::before, body.themedesignermode .pr-lg-3::before,
  .pr-lg-3 body.themedesignermode.pr-lg-3::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-lg-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 1rem !important;
  }
  body.behat-site .pl-lg-3,
  .pl-lg-3 body.behat-site.pl-lg-3, body.themedesignermode .pl-lg-3,
  .pl-lg-3 body.themedesignermode.pl-lg-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-lg-3::before,
  .pl-lg-3 body.behat-site.pl-lg-3::before, body.themedesignermode .pl-lg-3::before,
  .pl-lg-3 body.themedesignermode.pl-lg-3::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-lg-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 1.5rem !important;
  }
  body.behat-site .pr-lg-4,
  .pr-lg-4 body.behat-site.pr-lg-4, body.themedesignermode .pr-lg-4,
  .pr-lg-4 body.themedesignermode.pr-lg-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-lg-4::before,
  .pr-lg-4 body.behat-site.pr-lg-4::before, body.themedesignermode .pr-lg-4::before,
  .pr-lg-4 body.themedesignermode.pr-lg-4::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-lg-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 1.5rem !important;
  }
  body.behat-site .pl-lg-4,
  .pl-lg-4 body.behat-site.pl-lg-4, body.themedesignermode .pl-lg-4,
  .pl-lg-4 body.themedesignermode.pl-lg-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-lg-4::before,
  .pl-lg-4 body.behat-site.pl-lg-4::before, body.themedesignermode .pl-lg-4::before,
  .pl-lg-4 body.themedesignermode.pl-lg-4::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-lg-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 2rem !important;
  }
  body.behat-site .pr-lg-5,
  .pr-lg-5 body.behat-site.pr-lg-5, body.themedesignermode .pr-lg-5,
  .pr-lg-5 body.themedesignermode.pr-lg-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-lg-5::before,
  .pr-lg-5 body.behat-site.pr-lg-5::before, body.themedesignermode .pr-lg-5::before,
  .pr-lg-5 body.themedesignermode.pr-lg-5::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-lg-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 2rem !important;
  }
  body.behat-site .pl-lg-5,
  .pl-lg-5 body.behat-site.pl-lg-5, body.themedesignermode .pl-lg-5,
  .pl-lg-5 body.themedesignermode.pl-lg-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-lg-5::before,
  .pl-lg-5 body.behat-site.pl-lg-5::before, body.themedesignermode .pl-lg-5::before,
  .pl-lg-5 body.themedesignermode.pl-lg-5::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-lg-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 3rem !important;
  }
  body.behat-site .pr-lg-6,
  .pr-lg-6 body.behat-site.pr-lg-6, body.themedesignermode .pr-lg-6,
  .pr-lg-6 body.themedesignermode.pr-lg-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-lg-6::before,
  .pr-lg-6 body.behat-site.pr-lg-6::before, body.themedesignermode .pr-lg-6::before,
  .pr-lg-6 body.themedesignermode.pr-lg-6::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-lg-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 3rem !important;
  }
  body.behat-site .pl-lg-6,
  .pl-lg-6 body.behat-site.pl-lg-6, body.themedesignermode .pl-lg-6,
  .pl-lg-6 body.themedesignermode.pl-lg-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-lg-6::before,
  .pl-lg-6 body.behat-site.pl-lg-6::before, body.themedesignermode .pl-lg-6::before,
  .pl-lg-6 body.themedesignermode.pl-lg-6::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-lg-n1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -0.25rem !important;
  }
  body.behat-site .mr-lg-n1,
  .mr-lg-n1 body.behat-site.mr-lg-n1, body.themedesignermode .mr-lg-n1,
  .mr-lg-n1 body.themedesignermode.mr-lg-n1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-lg-n1::before,
  .mr-lg-n1 body.behat-site.mr-lg-n1::before, body.themedesignermode .mr-lg-n1::before,
  .mr-lg-n1 body.themedesignermode.mr-lg-n1::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-lg-n1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -0.25rem !important;
  }
  body.behat-site .ml-lg-n1,
  .ml-lg-n1 body.behat-site.ml-lg-n1, body.themedesignermode .ml-lg-n1,
  .ml-lg-n1 body.themedesignermode.ml-lg-n1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-lg-n1::before,
  .ml-lg-n1 body.behat-site.ml-lg-n1::before, body.themedesignermode .ml-lg-n1::before,
  .ml-lg-n1 body.themedesignermode.ml-lg-n1::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-lg-n2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -0.5rem !important;
  }
  body.behat-site .mr-lg-n2,
  .mr-lg-n2 body.behat-site.mr-lg-n2, body.themedesignermode .mr-lg-n2,
  .mr-lg-n2 body.themedesignermode.mr-lg-n2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-lg-n2::before,
  .mr-lg-n2 body.behat-site.mr-lg-n2::before, body.themedesignermode .mr-lg-n2::before,
  .mr-lg-n2 body.themedesignermode.mr-lg-n2::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-lg-n2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -0.5rem !important;
  }
  body.behat-site .ml-lg-n2,
  .ml-lg-n2 body.behat-site.ml-lg-n2, body.themedesignermode .ml-lg-n2,
  .ml-lg-n2 body.themedesignermode.ml-lg-n2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-lg-n2::before,
  .ml-lg-n2 body.behat-site.ml-lg-n2::before, body.themedesignermode .ml-lg-n2::before,
  .ml-lg-n2 body.themedesignermode.ml-lg-n2::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-lg-n3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -1rem !important;
  }
  body.behat-site .mr-lg-n3,
  .mr-lg-n3 body.behat-site.mr-lg-n3, body.themedesignermode .mr-lg-n3,
  .mr-lg-n3 body.themedesignermode.mr-lg-n3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-lg-n3::before,
  .mr-lg-n3 body.behat-site.mr-lg-n3::before, body.themedesignermode .mr-lg-n3::before,
  .mr-lg-n3 body.themedesignermode.mr-lg-n3::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-lg-n3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -1rem !important;
  }
  body.behat-site .ml-lg-n3,
  .ml-lg-n3 body.behat-site.ml-lg-n3, body.themedesignermode .ml-lg-n3,
  .ml-lg-n3 body.themedesignermode.ml-lg-n3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-lg-n3::before,
  .ml-lg-n3 body.behat-site.ml-lg-n3::before, body.themedesignermode .ml-lg-n3::before,
  .ml-lg-n3 body.themedesignermode.ml-lg-n3::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-lg-n4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -1.5rem !important;
  }
  body.behat-site .mr-lg-n4,
  .mr-lg-n4 body.behat-site.mr-lg-n4, body.themedesignermode .mr-lg-n4,
  .mr-lg-n4 body.themedesignermode.mr-lg-n4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-lg-n4::before,
  .mr-lg-n4 body.behat-site.mr-lg-n4::before, body.themedesignermode .mr-lg-n4::before,
  .mr-lg-n4 body.themedesignermode.mr-lg-n4::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-lg-n4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -1.5rem !important;
  }
  body.behat-site .ml-lg-n4,
  .ml-lg-n4 body.behat-site.ml-lg-n4, body.themedesignermode .ml-lg-n4,
  .ml-lg-n4 body.themedesignermode.ml-lg-n4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-lg-n4::before,
  .ml-lg-n4 body.behat-site.ml-lg-n4::before, body.themedesignermode .ml-lg-n4::before,
  .ml-lg-n4 body.themedesignermode.ml-lg-n4::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-lg-n5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -2rem !important;
  }
  body.behat-site .mr-lg-n5,
  .mr-lg-n5 body.behat-site.mr-lg-n5, body.themedesignermode .mr-lg-n5,
  .mr-lg-n5 body.themedesignermode.mr-lg-n5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-lg-n5::before,
  .mr-lg-n5 body.behat-site.mr-lg-n5::before, body.themedesignermode .mr-lg-n5::before,
  .mr-lg-n5 body.themedesignermode.mr-lg-n5::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-lg-n5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -2rem !important;
  }
  body.behat-site .ml-lg-n5,
  .ml-lg-n5 body.behat-site.ml-lg-n5, body.themedesignermode .ml-lg-n5,
  .ml-lg-n5 body.themedesignermode.ml-lg-n5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-lg-n5::before,
  .ml-lg-n5 body.behat-site.ml-lg-n5::before, body.themedesignermode .ml-lg-n5::before,
  .ml-lg-n5 body.themedesignermode.ml-lg-n5::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-lg-n6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -3rem !important;
  }
  body.behat-site .mr-lg-n6,
  .mr-lg-n6 body.behat-site.mr-lg-n6, body.themedesignermode .mr-lg-n6,
  .mr-lg-n6 body.themedesignermode.mr-lg-n6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-lg-n6::before,
  .mr-lg-n6 body.behat-site.mr-lg-n6::before, body.themedesignermode .mr-lg-n6::before,
  .mr-lg-n6 body.themedesignermode.mr-lg-n6::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-lg-n6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -3rem !important;
  }
  body.behat-site .ml-lg-n6,
  .ml-lg-n6 body.behat-site.ml-lg-n6, body.themedesignermode .ml-lg-n6,
  .ml-lg-n6 body.themedesignermode.ml-lg-n6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-lg-n6::before,
  .ml-lg-n6 body.behat-site.ml-lg-n6::before, body.themedesignermode .ml-lg-n6::before,
  .ml-lg-n6 body.themedesignermode.ml-lg-n6::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-lg-auto {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: auto !important;
  }
  body.behat-site .mr-lg-auto,
  .mr-lg-auto body.behat-site.mr-lg-auto, body.themedesignermode .mr-lg-auto,
  .mr-lg-auto body.themedesignermode.mr-lg-auto {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-lg-auto::before,
  .mr-lg-auto body.behat-site.mr-lg-auto::before, body.themedesignermode .mr-lg-auto::before,
  .mr-lg-auto body.themedesignermode.mr-lg-auto::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-lg-auto {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: auto !important;
  }
  body.behat-site .ml-lg-auto,
  .ml-lg-auto body.behat-site.ml-lg-auto, body.themedesignermode .ml-lg-auto,
  .ml-lg-auto body.themedesignermode.ml-lg-auto {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-lg-auto::before,
  .ml-lg-auto body.behat-site.ml-lg-auto::before, body.themedesignermode .ml-lg-auto::before,
  .ml-lg-auto body.themedesignermode.ml-lg-auto::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
@media (min-width: 1200px) {
  .mr-xl-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0 !important;
  }
  body.behat-site .mr-xl-0,
  .mr-xl-0 body.behat-site.mr-xl-0, body.themedesignermode .mr-xl-0,
  .mr-xl-0 body.themedesignermode.mr-xl-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xl-0::before,
  .mr-xl-0 body.behat-site.mr-xl-0::before, body.themedesignermode .mr-xl-0::before,
  .mr-xl-0 body.themedesignermode.mr-xl-0::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xl-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0 !important;
  }
  body.behat-site .ml-xl-0,
  .ml-xl-0 body.behat-site.ml-xl-0, body.themedesignermode .ml-xl-0,
  .ml-xl-0 body.themedesignermode.ml-xl-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xl-0::before,
  .ml-xl-0 body.behat-site.ml-xl-0::before, body.themedesignermode .ml-xl-0::before,
  .ml-xl-0 body.themedesignermode.ml-xl-0::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xl-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0.25rem !important;
  }
  body.behat-site .mr-xl-1,
  .mr-xl-1 body.behat-site.mr-xl-1, body.themedesignermode .mr-xl-1,
  .mr-xl-1 body.themedesignermode.mr-xl-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xl-1::before,
  .mr-xl-1 body.behat-site.mr-xl-1::before, body.themedesignermode .mr-xl-1::before,
  .mr-xl-1 body.themedesignermode.mr-xl-1::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xl-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0.25rem !important;
  }
  body.behat-site .ml-xl-1,
  .ml-xl-1 body.behat-site.ml-xl-1, body.themedesignermode .ml-xl-1,
  .ml-xl-1 body.themedesignermode.ml-xl-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xl-1::before,
  .ml-xl-1 body.behat-site.ml-xl-1::before, body.themedesignermode .ml-xl-1::before,
  .ml-xl-1 body.themedesignermode.ml-xl-1::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xl-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0.5rem !important;
  }
  body.behat-site .mr-xl-2,
  .mr-xl-2 body.behat-site.mr-xl-2, body.themedesignermode .mr-xl-2,
  .mr-xl-2 body.themedesignermode.mr-xl-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xl-2::before,
  .mr-xl-2 body.behat-site.mr-xl-2::before, body.themedesignermode .mr-xl-2::before,
  .mr-xl-2 body.themedesignermode.mr-xl-2::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xl-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0.5rem !important;
  }
  body.behat-site .ml-xl-2,
  .ml-xl-2 body.behat-site.ml-xl-2, body.themedesignermode .ml-xl-2,
  .ml-xl-2 body.themedesignermode.ml-xl-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xl-2::before,
  .ml-xl-2 body.behat-site.ml-xl-2::before, body.themedesignermode .ml-xl-2::before,
  .ml-xl-2 body.themedesignermode.ml-xl-2::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xl-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 1rem !important;
  }
  body.behat-site .mr-xl-3,
  .mr-xl-3 body.behat-site.mr-xl-3, body.themedesignermode .mr-xl-3,
  .mr-xl-3 body.themedesignermode.mr-xl-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xl-3::before,
  .mr-xl-3 body.behat-site.mr-xl-3::before, body.themedesignermode .mr-xl-3::before,
  .mr-xl-3 body.themedesignermode.mr-xl-3::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xl-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 1rem !important;
  }
  body.behat-site .ml-xl-3,
  .ml-xl-3 body.behat-site.ml-xl-3, body.themedesignermode .ml-xl-3,
  .ml-xl-3 body.themedesignermode.ml-xl-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xl-3::before,
  .ml-xl-3 body.behat-site.ml-xl-3::before, body.themedesignermode .ml-xl-3::before,
  .ml-xl-3 body.themedesignermode.ml-xl-3::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xl-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 1.5rem !important;
  }
  body.behat-site .mr-xl-4,
  .mr-xl-4 body.behat-site.mr-xl-4, body.themedesignermode .mr-xl-4,
  .mr-xl-4 body.themedesignermode.mr-xl-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xl-4::before,
  .mr-xl-4 body.behat-site.mr-xl-4::before, body.themedesignermode .mr-xl-4::before,
  .mr-xl-4 body.themedesignermode.mr-xl-4::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xl-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 1.5rem !important;
  }
  body.behat-site .ml-xl-4,
  .ml-xl-4 body.behat-site.ml-xl-4, body.themedesignermode .ml-xl-4,
  .ml-xl-4 body.themedesignermode.ml-xl-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xl-4::before,
  .ml-xl-4 body.behat-site.ml-xl-4::before, body.themedesignermode .ml-xl-4::before,
  .ml-xl-4 body.themedesignermode.ml-xl-4::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xl-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 2rem !important;
  }
  body.behat-site .mr-xl-5,
  .mr-xl-5 body.behat-site.mr-xl-5, body.themedesignermode .mr-xl-5,
  .mr-xl-5 body.themedesignermode.mr-xl-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xl-5::before,
  .mr-xl-5 body.behat-site.mr-xl-5::before, body.themedesignermode .mr-xl-5::before,
  .mr-xl-5 body.themedesignermode.mr-xl-5::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xl-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 2rem !important;
  }
  body.behat-site .ml-xl-5,
  .ml-xl-5 body.behat-site.ml-xl-5, body.themedesignermode .ml-xl-5,
  .ml-xl-5 body.themedesignermode.ml-xl-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xl-5::before,
  .ml-xl-5 body.behat-site.ml-xl-5::before, body.themedesignermode .ml-xl-5::before,
  .ml-xl-5 body.themedesignermode.ml-xl-5::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xl-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 3rem !important;
  }
  body.behat-site .mr-xl-6,
  .mr-xl-6 body.behat-site.mr-xl-6, body.themedesignermode .mr-xl-6,
  .mr-xl-6 body.themedesignermode.mr-xl-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xl-6::before,
  .mr-xl-6 body.behat-site.mr-xl-6::before, body.themedesignermode .mr-xl-6::before,
  .mr-xl-6 body.themedesignermode.mr-xl-6::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xl-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 3rem !important;
  }
  body.behat-site .ml-xl-6,
  .ml-xl-6 body.behat-site.ml-xl-6, body.themedesignermode .ml-xl-6,
  .ml-xl-6 body.themedesignermode.ml-xl-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xl-6::before,
  .ml-xl-6 body.behat-site.ml-xl-6::before, body.themedesignermode .ml-xl-6::before,
  .ml-xl-6 body.themedesignermode.ml-xl-6::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-xl-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0 !important;
  }
  body.behat-site .pr-xl-0,
  .pr-xl-0 body.behat-site.pr-xl-0, body.themedesignermode .pr-xl-0,
  .pr-xl-0 body.themedesignermode.pr-xl-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-xl-0::before,
  .pr-xl-0 body.behat-site.pr-xl-0::before, body.themedesignermode .pr-xl-0::before,
  .pr-xl-0 body.themedesignermode.pr-xl-0::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-xl-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0 !important;
  }
  body.behat-site .pl-xl-0,
  .pl-xl-0 body.behat-site.pl-xl-0, body.themedesignermode .pl-xl-0,
  .pl-xl-0 body.themedesignermode.pl-xl-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-xl-0::before,
  .pl-xl-0 body.behat-site.pl-xl-0::before, body.themedesignermode .pl-xl-0::before,
  .pl-xl-0 body.themedesignermode.pl-xl-0::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-xl-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0.25rem !important;
  }
  body.behat-site .pr-xl-1,
  .pr-xl-1 body.behat-site.pr-xl-1, body.themedesignermode .pr-xl-1,
  .pr-xl-1 body.themedesignermode.pr-xl-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-xl-1::before,
  .pr-xl-1 body.behat-site.pr-xl-1::before, body.themedesignermode .pr-xl-1::before,
  .pr-xl-1 body.themedesignermode.pr-xl-1::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-xl-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0.25rem !important;
  }
  body.behat-site .pl-xl-1,
  .pl-xl-1 body.behat-site.pl-xl-1, body.themedesignermode .pl-xl-1,
  .pl-xl-1 body.themedesignermode.pl-xl-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-xl-1::before,
  .pl-xl-1 body.behat-site.pl-xl-1::before, body.themedesignermode .pl-xl-1::before,
  .pl-xl-1 body.themedesignermode.pl-xl-1::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-xl-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0.5rem !important;
  }
  body.behat-site .pr-xl-2,
  .pr-xl-2 body.behat-site.pr-xl-2, body.themedesignermode .pr-xl-2,
  .pr-xl-2 body.themedesignermode.pr-xl-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-xl-2::before,
  .pr-xl-2 body.behat-site.pr-xl-2::before, body.themedesignermode .pr-xl-2::before,
  .pr-xl-2 body.themedesignermode.pr-xl-2::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-xl-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0.5rem !important;
  }
  body.behat-site .pl-xl-2,
  .pl-xl-2 body.behat-site.pl-xl-2, body.themedesignermode .pl-xl-2,
  .pl-xl-2 body.themedesignermode.pl-xl-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-xl-2::before,
  .pl-xl-2 body.behat-site.pl-xl-2::before, body.themedesignermode .pl-xl-2::before,
  .pl-xl-2 body.themedesignermode.pl-xl-2::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-xl-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 1rem !important;
  }
  body.behat-site .pr-xl-3,
  .pr-xl-3 body.behat-site.pr-xl-3, body.themedesignermode .pr-xl-3,
  .pr-xl-3 body.themedesignermode.pr-xl-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-xl-3::before,
  .pr-xl-3 body.behat-site.pr-xl-3::before, body.themedesignermode .pr-xl-3::before,
  .pr-xl-3 body.themedesignermode.pr-xl-3::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-xl-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 1rem !important;
  }
  body.behat-site .pl-xl-3,
  .pl-xl-3 body.behat-site.pl-xl-3, body.themedesignermode .pl-xl-3,
  .pl-xl-3 body.themedesignermode.pl-xl-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-xl-3::before,
  .pl-xl-3 body.behat-site.pl-xl-3::before, body.themedesignermode .pl-xl-3::before,
  .pl-xl-3 body.themedesignermode.pl-xl-3::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-xl-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 1.5rem !important;
  }
  body.behat-site .pr-xl-4,
  .pr-xl-4 body.behat-site.pr-xl-4, body.themedesignermode .pr-xl-4,
  .pr-xl-4 body.themedesignermode.pr-xl-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-xl-4::before,
  .pr-xl-4 body.behat-site.pr-xl-4::before, body.themedesignermode .pr-xl-4::before,
  .pr-xl-4 body.themedesignermode.pr-xl-4::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-xl-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 1.5rem !important;
  }
  body.behat-site .pl-xl-4,
  .pl-xl-4 body.behat-site.pl-xl-4, body.themedesignermode .pl-xl-4,
  .pl-xl-4 body.themedesignermode.pl-xl-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-xl-4::before,
  .pl-xl-4 body.behat-site.pl-xl-4::before, body.themedesignermode .pl-xl-4::before,
  .pl-xl-4 body.themedesignermode.pl-xl-4::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-xl-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 2rem !important;
  }
  body.behat-site .pr-xl-5,
  .pr-xl-5 body.behat-site.pr-xl-5, body.themedesignermode .pr-xl-5,
  .pr-xl-5 body.themedesignermode.pr-xl-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-xl-5::before,
  .pr-xl-5 body.behat-site.pr-xl-5::before, body.themedesignermode .pr-xl-5::before,
  .pr-xl-5 body.themedesignermode.pr-xl-5::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-xl-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 2rem !important;
  }
  body.behat-site .pl-xl-5,
  .pl-xl-5 body.behat-site.pl-xl-5, body.themedesignermode .pl-xl-5,
  .pl-xl-5 body.themedesignermode.pl-xl-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-xl-5::before,
  .pl-xl-5 body.behat-site.pl-xl-5::before, body.themedesignermode .pl-xl-5::before,
  .pl-xl-5 body.themedesignermode.pl-xl-5::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-xl-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 3rem !important;
  }
  body.behat-site .pr-xl-6,
  .pr-xl-6 body.behat-site.pr-xl-6, body.themedesignermode .pr-xl-6,
  .pr-xl-6 body.themedesignermode.pr-xl-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-xl-6::before,
  .pr-xl-6 body.behat-site.pr-xl-6::before, body.themedesignermode .pr-xl-6::before,
  .pr-xl-6 body.themedesignermode.pr-xl-6::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-xl-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 3rem !important;
  }
  body.behat-site .pl-xl-6,
  .pl-xl-6 body.behat-site.pl-xl-6, body.themedesignermode .pl-xl-6,
  .pl-xl-6 body.themedesignermode.pl-xl-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-xl-6::before,
  .pl-xl-6 body.behat-site.pl-xl-6::before, body.themedesignermode .pl-xl-6::before,
  .pl-xl-6 body.themedesignermode.pl-xl-6::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xl-n1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -0.25rem !important;
  }
  body.behat-site .mr-xl-n1,
  .mr-xl-n1 body.behat-site.mr-xl-n1, body.themedesignermode .mr-xl-n1,
  .mr-xl-n1 body.themedesignermode.mr-xl-n1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xl-n1::before,
  .mr-xl-n1 body.behat-site.mr-xl-n1::before, body.themedesignermode .mr-xl-n1::before,
  .mr-xl-n1 body.themedesignermode.mr-xl-n1::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xl-n1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -0.25rem !important;
  }
  body.behat-site .ml-xl-n1,
  .ml-xl-n1 body.behat-site.ml-xl-n1, body.themedesignermode .ml-xl-n1,
  .ml-xl-n1 body.themedesignermode.ml-xl-n1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xl-n1::before,
  .ml-xl-n1 body.behat-site.ml-xl-n1::before, body.themedesignermode .ml-xl-n1::before,
  .ml-xl-n1 body.themedesignermode.ml-xl-n1::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xl-n2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -0.5rem !important;
  }
  body.behat-site .mr-xl-n2,
  .mr-xl-n2 body.behat-site.mr-xl-n2, body.themedesignermode .mr-xl-n2,
  .mr-xl-n2 body.themedesignermode.mr-xl-n2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xl-n2::before,
  .mr-xl-n2 body.behat-site.mr-xl-n2::before, body.themedesignermode .mr-xl-n2::before,
  .mr-xl-n2 body.themedesignermode.mr-xl-n2::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xl-n2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -0.5rem !important;
  }
  body.behat-site .ml-xl-n2,
  .ml-xl-n2 body.behat-site.ml-xl-n2, body.themedesignermode .ml-xl-n2,
  .ml-xl-n2 body.themedesignermode.ml-xl-n2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xl-n2::before,
  .ml-xl-n2 body.behat-site.ml-xl-n2::before, body.themedesignermode .ml-xl-n2::before,
  .ml-xl-n2 body.themedesignermode.ml-xl-n2::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xl-n3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -1rem !important;
  }
  body.behat-site .mr-xl-n3,
  .mr-xl-n3 body.behat-site.mr-xl-n3, body.themedesignermode .mr-xl-n3,
  .mr-xl-n3 body.themedesignermode.mr-xl-n3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xl-n3::before,
  .mr-xl-n3 body.behat-site.mr-xl-n3::before, body.themedesignermode .mr-xl-n3::before,
  .mr-xl-n3 body.themedesignermode.mr-xl-n3::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xl-n3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -1rem !important;
  }
  body.behat-site .ml-xl-n3,
  .ml-xl-n3 body.behat-site.ml-xl-n3, body.themedesignermode .ml-xl-n3,
  .ml-xl-n3 body.themedesignermode.ml-xl-n3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xl-n3::before,
  .ml-xl-n3 body.behat-site.ml-xl-n3::before, body.themedesignermode .ml-xl-n3::before,
  .ml-xl-n3 body.themedesignermode.ml-xl-n3::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xl-n4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -1.5rem !important;
  }
  body.behat-site .mr-xl-n4,
  .mr-xl-n4 body.behat-site.mr-xl-n4, body.themedesignermode .mr-xl-n4,
  .mr-xl-n4 body.themedesignermode.mr-xl-n4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xl-n4::before,
  .mr-xl-n4 body.behat-site.mr-xl-n4::before, body.themedesignermode .mr-xl-n4::before,
  .mr-xl-n4 body.themedesignermode.mr-xl-n4::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xl-n4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -1.5rem !important;
  }
  body.behat-site .ml-xl-n4,
  .ml-xl-n4 body.behat-site.ml-xl-n4, body.themedesignermode .ml-xl-n4,
  .ml-xl-n4 body.themedesignermode.ml-xl-n4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xl-n4::before,
  .ml-xl-n4 body.behat-site.ml-xl-n4::before, body.themedesignermode .ml-xl-n4::before,
  .ml-xl-n4 body.themedesignermode.ml-xl-n4::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xl-n5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -2rem !important;
  }
  body.behat-site .mr-xl-n5,
  .mr-xl-n5 body.behat-site.mr-xl-n5, body.themedesignermode .mr-xl-n5,
  .mr-xl-n5 body.themedesignermode.mr-xl-n5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xl-n5::before,
  .mr-xl-n5 body.behat-site.mr-xl-n5::before, body.themedesignermode .mr-xl-n5::before,
  .mr-xl-n5 body.themedesignermode.mr-xl-n5::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xl-n5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -2rem !important;
  }
  body.behat-site .ml-xl-n5,
  .ml-xl-n5 body.behat-site.ml-xl-n5, body.themedesignermode .ml-xl-n5,
  .ml-xl-n5 body.themedesignermode.ml-xl-n5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xl-n5::before,
  .ml-xl-n5 body.behat-site.ml-xl-n5::before, body.themedesignermode .ml-xl-n5::before,
  .ml-xl-n5 body.themedesignermode.ml-xl-n5::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xl-n6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -3rem !important;
  }
  body.behat-site .mr-xl-n6,
  .mr-xl-n6 body.behat-site.mr-xl-n6, body.themedesignermode .mr-xl-n6,
  .mr-xl-n6 body.themedesignermode.mr-xl-n6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xl-n6::before,
  .mr-xl-n6 body.behat-site.mr-xl-n6::before, body.themedesignermode .mr-xl-n6::before,
  .mr-xl-n6 body.themedesignermode.mr-xl-n6::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xl-n6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -3rem !important;
  }
  body.behat-site .ml-xl-n6,
  .ml-xl-n6 body.behat-site.ml-xl-n6, body.themedesignermode .ml-xl-n6,
  .ml-xl-n6 body.themedesignermode.ml-xl-n6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xl-n6::before,
  .ml-xl-n6 body.behat-site.ml-xl-n6::before, body.themedesignermode .ml-xl-n6::before,
  .ml-xl-n6 body.themedesignermode.ml-xl-n6::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xl-auto {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: auto !important;
  }
  body.behat-site .mr-xl-auto,
  .mr-xl-auto body.behat-site.mr-xl-auto, body.themedesignermode .mr-xl-auto,
  .mr-xl-auto body.themedesignermode.mr-xl-auto {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xl-auto::before,
  .mr-xl-auto body.behat-site.mr-xl-auto::before, body.themedesignermode .mr-xl-auto::before,
  .mr-xl-auto body.themedesignermode.mr-xl-auto::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xl-auto {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: auto !important;
  }
  body.behat-site .ml-xl-auto,
  .ml-xl-auto body.behat-site.ml-xl-auto, body.themedesignermode .ml-xl-auto,
  .ml-xl-auto body.themedesignermode.ml-xl-auto {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xl-auto::before,
  .ml-xl-auto body.behat-site.ml-xl-auto::before, body.themedesignermode .ml-xl-auto::before,
  .ml-xl-auto body.themedesignermode.ml-xl-auto::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
@media (min-width: 1400px) {
  .mr-xxl-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0 !important;
  }
  body.behat-site .mr-xxl-0,
  .mr-xxl-0 body.behat-site.mr-xxl-0, body.themedesignermode .mr-xxl-0,
  .mr-xxl-0 body.themedesignermode.mr-xxl-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xxl-0::before,
  .mr-xxl-0 body.behat-site.mr-xxl-0::before, body.themedesignermode .mr-xxl-0::before,
  .mr-xxl-0 body.themedesignermode.mr-xxl-0::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xxl-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0 !important;
  }
  body.behat-site .ml-xxl-0,
  .ml-xxl-0 body.behat-site.ml-xxl-0, body.themedesignermode .ml-xxl-0,
  .ml-xxl-0 body.themedesignermode.ml-xxl-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xxl-0::before,
  .ml-xxl-0 body.behat-site.ml-xxl-0::before, body.themedesignermode .ml-xxl-0::before,
  .ml-xxl-0 body.themedesignermode.ml-xxl-0::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xxl-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0.25rem !important;
  }
  body.behat-site .mr-xxl-1,
  .mr-xxl-1 body.behat-site.mr-xxl-1, body.themedesignermode .mr-xxl-1,
  .mr-xxl-1 body.themedesignermode.mr-xxl-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xxl-1::before,
  .mr-xxl-1 body.behat-site.mr-xxl-1::before, body.themedesignermode .mr-xxl-1::before,
  .mr-xxl-1 body.themedesignermode.mr-xxl-1::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xxl-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0.25rem !important;
  }
  body.behat-site .ml-xxl-1,
  .ml-xxl-1 body.behat-site.ml-xxl-1, body.themedesignermode .ml-xxl-1,
  .ml-xxl-1 body.themedesignermode.ml-xxl-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xxl-1::before,
  .ml-xxl-1 body.behat-site.ml-xxl-1::before, body.themedesignermode .ml-xxl-1::before,
  .ml-xxl-1 body.themedesignermode.ml-xxl-1::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xxl-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 0.5rem !important;
  }
  body.behat-site .mr-xxl-2,
  .mr-xxl-2 body.behat-site.mr-xxl-2, body.themedesignermode .mr-xxl-2,
  .mr-xxl-2 body.themedesignermode.mr-xxl-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xxl-2::before,
  .mr-xxl-2 body.behat-site.mr-xxl-2::before, body.themedesignermode .mr-xxl-2::before,
  .mr-xxl-2 body.themedesignermode.mr-xxl-2::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xxl-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 0.5rem !important;
  }
  body.behat-site .ml-xxl-2,
  .ml-xxl-2 body.behat-site.ml-xxl-2, body.themedesignermode .ml-xxl-2,
  .ml-xxl-2 body.themedesignermode.ml-xxl-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xxl-2::before,
  .ml-xxl-2 body.behat-site.ml-xxl-2::before, body.themedesignermode .ml-xxl-2::before,
  .ml-xxl-2 body.themedesignermode.ml-xxl-2::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xxl-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 1rem !important;
  }
  body.behat-site .mr-xxl-3,
  .mr-xxl-3 body.behat-site.mr-xxl-3, body.themedesignermode .mr-xxl-3,
  .mr-xxl-3 body.themedesignermode.mr-xxl-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xxl-3::before,
  .mr-xxl-3 body.behat-site.mr-xxl-3::before, body.themedesignermode .mr-xxl-3::before,
  .mr-xxl-3 body.themedesignermode.mr-xxl-3::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xxl-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 1rem !important;
  }
  body.behat-site .ml-xxl-3,
  .ml-xxl-3 body.behat-site.ml-xxl-3, body.themedesignermode .ml-xxl-3,
  .ml-xxl-3 body.themedesignermode.ml-xxl-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xxl-3::before,
  .ml-xxl-3 body.behat-site.ml-xxl-3::before, body.themedesignermode .ml-xxl-3::before,
  .ml-xxl-3 body.themedesignermode.ml-xxl-3::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xxl-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 1.5rem !important;
  }
  body.behat-site .mr-xxl-4,
  .mr-xxl-4 body.behat-site.mr-xxl-4, body.themedesignermode .mr-xxl-4,
  .mr-xxl-4 body.themedesignermode.mr-xxl-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xxl-4::before,
  .mr-xxl-4 body.behat-site.mr-xxl-4::before, body.themedesignermode .mr-xxl-4::before,
  .mr-xxl-4 body.themedesignermode.mr-xxl-4::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xxl-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 1.5rem !important;
  }
  body.behat-site .ml-xxl-4,
  .ml-xxl-4 body.behat-site.ml-xxl-4, body.themedesignermode .ml-xxl-4,
  .ml-xxl-4 body.themedesignermode.ml-xxl-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xxl-4::before,
  .ml-xxl-4 body.behat-site.ml-xxl-4::before, body.themedesignermode .ml-xxl-4::before,
  .ml-xxl-4 body.themedesignermode.ml-xxl-4::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xxl-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 2rem !important;
  }
  body.behat-site .mr-xxl-5,
  .mr-xxl-5 body.behat-site.mr-xxl-5, body.themedesignermode .mr-xxl-5,
  .mr-xxl-5 body.themedesignermode.mr-xxl-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xxl-5::before,
  .mr-xxl-5 body.behat-site.mr-xxl-5::before, body.themedesignermode .mr-xxl-5::before,
  .mr-xxl-5 body.themedesignermode.mr-xxl-5::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xxl-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 2rem !important;
  }
  body.behat-site .ml-xxl-5,
  .ml-xxl-5 body.behat-site.ml-xxl-5, body.themedesignermode .ml-xxl-5,
  .ml-xxl-5 body.themedesignermode.ml-xxl-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xxl-5::before,
  .ml-xxl-5 body.behat-site.ml-xxl-5::before, body.themedesignermode .ml-xxl-5::before,
  .ml-xxl-5 body.themedesignermode.ml-xxl-5::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xxl-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: 3rem !important;
  }
  body.behat-site .mr-xxl-6,
  .mr-xxl-6 body.behat-site.mr-xxl-6, body.themedesignermode .mr-xxl-6,
  .mr-xxl-6 body.themedesignermode.mr-xxl-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xxl-6::before,
  .mr-xxl-6 body.behat-site.mr-xxl-6::before, body.themedesignermode .mr-xxl-6::before,
  .mr-xxl-6 body.themedesignermode.mr-xxl-6::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xxl-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: 3rem !important;
  }
  body.behat-site .ml-xxl-6,
  .ml-xxl-6 body.behat-site.ml-xxl-6, body.themedesignermode .ml-xxl-6,
  .ml-xxl-6 body.themedesignermode.ml-xxl-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xxl-6::before,
  .ml-xxl-6 body.behat-site.ml-xxl-6::before, body.themedesignermode .ml-xxl-6::before,
  .ml-xxl-6 body.themedesignermode.ml-xxl-6::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-xxl-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0 !important;
  }
  body.behat-site .pr-xxl-0,
  .pr-xxl-0 body.behat-site.pr-xxl-0, body.themedesignermode .pr-xxl-0,
  .pr-xxl-0 body.themedesignermode.pr-xxl-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-xxl-0::before,
  .pr-xxl-0 body.behat-site.pr-xxl-0::before, body.themedesignermode .pr-xxl-0::before,
  .pr-xxl-0 body.themedesignermode.pr-xxl-0::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-xxl-0 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0 !important;
  }
  body.behat-site .pl-xxl-0,
  .pl-xxl-0 body.behat-site.pl-xxl-0, body.themedesignermode .pl-xxl-0,
  .pl-xxl-0 body.themedesignermode.pl-xxl-0 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-xxl-0::before,
  .pl-xxl-0 body.behat-site.pl-xxl-0::before, body.themedesignermode .pl-xxl-0::before,
  .pl-xxl-0 body.themedesignermode.pl-xxl-0::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-xxl-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0.25rem !important;
  }
  body.behat-site .pr-xxl-1,
  .pr-xxl-1 body.behat-site.pr-xxl-1, body.themedesignermode .pr-xxl-1,
  .pr-xxl-1 body.themedesignermode.pr-xxl-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-xxl-1::before,
  .pr-xxl-1 body.behat-site.pr-xxl-1::before, body.themedesignermode .pr-xxl-1::before,
  .pr-xxl-1 body.themedesignermode.pr-xxl-1::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-xxl-1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0.25rem !important;
  }
  body.behat-site .pl-xxl-1,
  .pl-xxl-1 body.behat-site.pl-xxl-1, body.themedesignermode .pl-xxl-1,
  .pl-xxl-1 body.themedesignermode.pl-xxl-1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-xxl-1::before,
  .pl-xxl-1 body.behat-site.pl-xxl-1::before, body.themedesignermode .pl-xxl-1::before,
  .pl-xxl-1 body.themedesignermode.pl-xxl-1::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-xxl-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 0.5rem !important;
  }
  body.behat-site .pr-xxl-2,
  .pr-xxl-2 body.behat-site.pr-xxl-2, body.themedesignermode .pr-xxl-2,
  .pr-xxl-2 body.themedesignermode.pr-xxl-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-xxl-2::before,
  .pr-xxl-2 body.behat-site.pr-xxl-2::before, body.themedesignermode .pr-xxl-2::before,
  .pr-xxl-2 body.themedesignermode.pr-xxl-2::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-xxl-2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 0.5rem !important;
  }
  body.behat-site .pl-xxl-2,
  .pl-xxl-2 body.behat-site.pl-xxl-2, body.themedesignermode .pl-xxl-2,
  .pl-xxl-2 body.themedesignermode.pl-xxl-2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-xxl-2::before,
  .pl-xxl-2 body.behat-site.pl-xxl-2::before, body.themedesignermode .pl-xxl-2::before,
  .pl-xxl-2 body.themedesignermode.pl-xxl-2::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-xxl-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 1rem !important;
  }
  body.behat-site .pr-xxl-3,
  .pr-xxl-3 body.behat-site.pr-xxl-3, body.themedesignermode .pr-xxl-3,
  .pr-xxl-3 body.themedesignermode.pr-xxl-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-xxl-3::before,
  .pr-xxl-3 body.behat-site.pr-xxl-3::before, body.themedesignermode .pr-xxl-3::before,
  .pr-xxl-3 body.themedesignermode.pr-xxl-3::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-xxl-3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 1rem !important;
  }
  body.behat-site .pl-xxl-3,
  .pl-xxl-3 body.behat-site.pl-xxl-3, body.themedesignermode .pl-xxl-3,
  .pl-xxl-3 body.themedesignermode.pl-xxl-3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-xxl-3::before,
  .pl-xxl-3 body.behat-site.pl-xxl-3::before, body.themedesignermode .pl-xxl-3::before,
  .pl-xxl-3 body.themedesignermode.pl-xxl-3::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-xxl-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 1.5rem !important;
  }
  body.behat-site .pr-xxl-4,
  .pr-xxl-4 body.behat-site.pr-xxl-4, body.themedesignermode .pr-xxl-4,
  .pr-xxl-4 body.themedesignermode.pr-xxl-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-xxl-4::before,
  .pr-xxl-4 body.behat-site.pr-xxl-4::before, body.themedesignermode .pr-xxl-4::before,
  .pr-xxl-4 body.themedesignermode.pr-xxl-4::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-xxl-4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 1.5rem !important;
  }
  body.behat-site .pl-xxl-4,
  .pl-xxl-4 body.behat-site.pl-xxl-4, body.themedesignermode .pl-xxl-4,
  .pl-xxl-4 body.themedesignermode.pl-xxl-4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-xxl-4::before,
  .pl-xxl-4 body.behat-site.pl-xxl-4::before, body.themedesignermode .pl-xxl-4::before,
  .pl-xxl-4 body.themedesignermode.pl-xxl-4::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-xxl-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 2rem !important;
  }
  body.behat-site .pr-xxl-5,
  .pr-xxl-5 body.behat-site.pr-xxl-5, body.themedesignermode .pr-xxl-5,
  .pr-xxl-5 body.themedesignermode.pr-xxl-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-xxl-5::before,
  .pr-xxl-5 body.behat-site.pr-xxl-5::before, body.themedesignermode .pr-xxl-5::before,
  .pr-xxl-5 body.themedesignermode.pr-xxl-5::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-xxl-5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 2rem !important;
  }
  body.behat-site .pl-xxl-5,
  .pl-xxl-5 body.behat-site.pl-xxl-5, body.themedesignermode .pl-xxl-5,
  .pl-xxl-5 body.themedesignermode.pl-xxl-5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-xxl-5::before,
  .pl-xxl-5 body.behat-site.pl-xxl-5::before, body.themedesignermode .pl-xxl-5::before,
  .pl-xxl-5 body.themedesignermode.pl-xxl-5::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pr-xxl-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-right: 3rem !important;
  }
  body.behat-site .pr-xxl-6,
  .pr-xxl-6 body.behat-site.pr-xxl-6, body.themedesignermode .pr-xxl-6,
  .pr-xxl-6 body.themedesignermode.pr-xxl-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pr-xxl-6::before,
  .pr-xxl-6 body.behat-site.pr-xxl-6::before, body.themedesignermode .pr-xxl-6::before,
  .pr-xxl-6 body.themedesignermode.pr-xxl-6::before {
    content: "Deprecated style in use (.pr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .pl-xxl-6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    padding-left: 3rem !important;
  }
  body.behat-site .pl-xxl-6,
  .pl-xxl-6 body.behat-site.pl-xxl-6, body.themedesignermode .pl-xxl-6,
  .pl-xxl-6 body.themedesignermode.pl-xxl-6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .pl-xxl-6::before,
  .pl-xxl-6 body.behat-site.pl-xxl-6::before, body.themedesignermode .pl-xxl-6::before,
  .pl-xxl-6 body.themedesignermode.pl-xxl-6::before {
    content: "Deprecated style in use (.pl)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xxl-n1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -0.25rem !important;
  }
  body.behat-site .mr-xxl-n1,
  .mr-xxl-n1 body.behat-site.mr-xxl-n1, body.themedesignermode .mr-xxl-n1,
  .mr-xxl-n1 body.themedesignermode.mr-xxl-n1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xxl-n1::before,
  .mr-xxl-n1 body.behat-site.mr-xxl-n1::before, body.themedesignermode .mr-xxl-n1::before,
  .mr-xxl-n1 body.themedesignermode.mr-xxl-n1::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xxl-n1 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -0.25rem !important;
  }
  body.behat-site .ml-xxl-n1,
  .ml-xxl-n1 body.behat-site.ml-xxl-n1, body.themedesignermode .ml-xxl-n1,
  .ml-xxl-n1 body.themedesignermode.ml-xxl-n1 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xxl-n1::before,
  .ml-xxl-n1 body.behat-site.ml-xxl-n1::before, body.themedesignermode .ml-xxl-n1::before,
  .ml-xxl-n1 body.themedesignermode.ml-xxl-n1::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xxl-n2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -0.5rem !important;
  }
  body.behat-site .mr-xxl-n2,
  .mr-xxl-n2 body.behat-site.mr-xxl-n2, body.themedesignermode .mr-xxl-n2,
  .mr-xxl-n2 body.themedesignermode.mr-xxl-n2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xxl-n2::before,
  .mr-xxl-n2 body.behat-site.mr-xxl-n2::before, body.themedesignermode .mr-xxl-n2::before,
  .mr-xxl-n2 body.themedesignermode.mr-xxl-n2::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xxl-n2 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -0.5rem !important;
  }
  body.behat-site .ml-xxl-n2,
  .ml-xxl-n2 body.behat-site.ml-xxl-n2, body.themedesignermode .ml-xxl-n2,
  .ml-xxl-n2 body.themedesignermode.ml-xxl-n2 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xxl-n2::before,
  .ml-xxl-n2 body.behat-site.ml-xxl-n2::before, body.themedesignermode .ml-xxl-n2::before,
  .ml-xxl-n2 body.themedesignermode.ml-xxl-n2::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xxl-n3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -1rem !important;
  }
  body.behat-site .mr-xxl-n3,
  .mr-xxl-n3 body.behat-site.mr-xxl-n3, body.themedesignermode .mr-xxl-n3,
  .mr-xxl-n3 body.themedesignermode.mr-xxl-n3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xxl-n3::before,
  .mr-xxl-n3 body.behat-site.mr-xxl-n3::before, body.themedesignermode .mr-xxl-n3::before,
  .mr-xxl-n3 body.themedesignermode.mr-xxl-n3::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xxl-n3 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -1rem !important;
  }
  body.behat-site .ml-xxl-n3,
  .ml-xxl-n3 body.behat-site.ml-xxl-n3, body.themedesignermode .ml-xxl-n3,
  .ml-xxl-n3 body.themedesignermode.ml-xxl-n3 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xxl-n3::before,
  .ml-xxl-n3 body.behat-site.ml-xxl-n3::before, body.themedesignermode .ml-xxl-n3::before,
  .ml-xxl-n3 body.themedesignermode.ml-xxl-n3::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xxl-n4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -1.5rem !important;
  }
  body.behat-site .mr-xxl-n4,
  .mr-xxl-n4 body.behat-site.mr-xxl-n4, body.themedesignermode .mr-xxl-n4,
  .mr-xxl-n4 body.themedesignermode.mr-xxl-n4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xxl-n4::before,
  .mr-xxl-n4 body.behat-site.mr-xxl-n4::before, body.themedesignermode .mr-xxl-n4::before,
  .mr-xxl-n4 body.themedesignermode.mr-xxl-n4::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xxl-n4 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -1.5rem !important;
  }
  body.behat-site .ml-xxl-n4,
  .ml-xxl-n4 body.behat-site.ml-xxl-n4, body.themedesignermode .ml-xxl-n4,
  .ml-xxl-n4 body.themedesignermode.ml-xxl-n4 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xxl-n4::before,
  .ml-xxl-n4 body.behat-site.ml-xxl-n4::before, body.themedesignermode .ml-xxl-n4::before,
  .ml-xxl-n4 body.themedesignermode.ml-xxl-n4::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xxl-n5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -2rem !important;
  }
  body.behat-site .mr-xxl-n5,
  .mr-xxl-n5 body.behat-site.mr-xxl-n5, body.themedesignermode .mr-xxl-n5,
  .mr-xxl-n5 body.themedesignermode.mr-xxl-n5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xxl-n5::before,
  .mr-xxl-n5 body.behat-site.mr-xxl-n5::before, body.themedesignermode .mr-xxl-n5::before,
  .mr-xxl-n5 body.themedesignermode.mr-xxl-n5::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xxl-n5 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -2rem !important;
  }
  body.behat-site .ml-xxl-n5,
  .ml-xxl-n5 body.behat-site.ml-xxl-n5, body.themedesignermode .ml-xxl-n5,
  .ml-xxl-n5 body.themedesignermode.ml-xxl-n5 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xxl-n5::before,
  .ml-xxl-n5 body.behat-site.ml-xxl-n5::before, body.themedesignermode .ml-xxl-n5::before,
  .ml-xxl-n5 body.themedesignermode.ml-xxl-n5::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xxl-n6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: -3rem !important;
  }
  body.behat-site .mr-xxl-n6,
  .mr-xxl-n6 body.behat-site.mr-xxl-n6, body.themedesignermode .mr-xxl-n6,
  .mr-xxl-n6 body.themedesignermode.mr-xxl-n6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xxl-n6::before,
  .mr-xxl-n6 body.behat-site.mr-xxl-n6::before, body.themedesignermode .mr-xxl-n6::before,
  .mr-xxl-n6 body.themedesignermode.mr-xxl-n6::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xxl-n6 {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: -3rem !important;
  }
  body.behat-site .ml-xxl-n6,
  .ml-xxl-n6 body.behat-site.ml-xxl-n6, body.themedesignermode .ml-xxl-n6,
  .ml-xxl-n6 body.themedesignermode.ml-xxl-n6 {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xxl-n6::before,
  .ml-xxl-n6 body.behat-site.ml-xxl-n6::before, body.themedesignermode .ml-xxl-n6::before,
  .ml-xxl-n6 body.themedesignermode.ml-xxl-n6::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .mr-xxl-auto {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-right: auto !important;
  }
  body.behat-site .mr-xxl-auto,
  .mr-xxl-auto body.behat-site.mr-xxl-auto, body.themedesignermode .mr-xxl-auto,
  .mr-xxl-auto body.themedesignermode.mr-xxl-auto {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .mr-xxl-auto::before,
  .mr-xxl-auto body.behat-site.mr-xxl-auto::before, body.themedesignermode .mr-xxl-auto::before,
  .mr-xxl-auto body.themedesignermode.mr-xxl-auto::before {
    content: "Deprecated style in use (.mr)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .ml-xxl-auto {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    margin-left: auto !important;
  }
  body.behat-site .ml-xxl-auto,
  .ml-xxl-auto body.behat-site.ml-xxl-auto, body.themedesignermode .ml-xxl-auto,
  .ml-xxl-auto body.themedesignermode.ml-xxl-auto {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .ml-xxl-auto::before,
  .ml-xxl-auto body.behat-site.ml-xxl-auto::before, body.themedesignermode .ml-xxl-auto::before,
  .ml-xxl-auto body.themedesignermode.ml-xxl-auto::before {
    content: "Deprecated style in use (.ml)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
.text-left {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  text-align: left !important;
}
body.behat-site .text-left,
.text-left body.behat-site.text-left, body.themedesignermode .text-left,
.text-left body.themedesignermode.text-left {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .text-left::before,
.text-left body.behat-site.text-left::before, body.themedesignermode .text-left::before,
.text-left body.themedesignermode.text-left::before {
  content: "Deprecated style in use (.text-left)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.text-right {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  text-align: right !important;
}
body.behat-site .text-right,
.text-right body.behat-site.text-right, body.themedesignermode .text-right,
.text-right body.themedesignermode.text-right {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .text-right::before,
.text-right body.behat-site.text-right::before, body.themedesignermode .text-right::before,
.text-right body.themedesignermode.text-right::before {
  content: "Deprecated style in use (.text-right)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

@media (min-width: 576px) {
  .text-sm-left {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    text-align: left !important;
  }
  body.behat-site .text-sm-left,
  .text-sm-left body.behat-site.text-sm-left, body.themedesignermode .text-sm-left,
  .text-sm-left body.themedesignermode.text-sm-left {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .text-sm-left::before,
  .text-sm-left body.behat-site.text-sm-left::before, body.themedesignermode .text-sm-left::before,
  .text-sm-left body.themedesignermode.text-sm-left::before {
    content: "Deprecated style in use (.text-left)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .text-sm-right {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    text-align: right !important;
  }
  body.behat-site .text-sm-right,
  .text-sm-right body.behat-site.text-sm-right, body.themedesignermode .text-sm-right,
  .text-sm-right body.themedesignermode.text-sm-right {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .text-sm-right::before,
  .text-sm-right body.behat-site.text-sm-right::before, body.themedesignermode .text-sm-right::before,
  .text-sm-right body.themedesignermode.text-sm-right::before {
    content: "Deprecated style in use (.text-right)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
@media (min-width: 768px) {
  .text-md-left {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    text-align: left !important;
  }
  body.behat-site .text-md-left,
  .text-md-left body.behat-site.text-md-left, body.themedesignermode .text-md-left,
  .text-md-left body.themedesignermode.text-md-left {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .text-md-left::before,
  .text-md-left body.behat-site.text-md-left::before, body.themedesignermode .text-md-left::before,
  .text-md-left body.themedesignermode.text-md-left::before {
    content: "Deprecated style in use (.text-left)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .text-md-right {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    text-align: right !important;
  }
  body.behat-site .text-md-right,
  .text-md-right body.behat-site.text-md-right, body.themedesignermode .text-md-right,
  .text-md-right body.themedesignermode.text-md-right {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .text-md-right::before,
  .text-md-right body.behat-site.text-md-right::before, body.themedesignermode .text-md-right::before,
  .text-md-right body.themedesignermode.text-md-right::before {
    content: "Deprecated style in use (.text-right)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
@media (min-width: 992px) {
  .text-lg-left {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    text-align: left !important;
  }
  body.behat-site .text-lg-left,
  .text-lg-left body.behat-site.text-lg-left, body.themedesignermode .text-lg-left,
  .text-lg-left body.themedesignermode.text-lg-left {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .text-lg-left::before,
  .text-lg-left body.behat-site.text-lg-left::before, body.themedesignermode .text-lg-left::before,
  .text-lg-left body.themedesignermode.text-lg-left::before {
    content: "Deprecated style in use (.text-left)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .text-lg-right {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    text-align: right !important;
  }
  body.behat-site .text-lg-right,
  .text-lg-right body.behat-site.text-lg-right, body.themedesignermode .text-lg-right,
  .text-lg-right body.themedesignermode.text-lg-right {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .text-lg-right::before,
  .text-lg-right body.behat-site.text-lg-right::before, body.themedesignermode .text-lg-right::before,
  .text-lg-right body.themedesignermode.text-lg-right::before {
    content: "Deprecated style in use (.text-right)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
@media (min-width: 1200px) {
  .text-xl-left {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    text-align: left !important;
  }
  body.behat-site .text-xl-left,
  .text-xl-left body.behat-site.text-xl-left, body.themedesignermode .text-xl-left,
  .text-xl-left body.themedesignermode.text-xl-left {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .text-xl-left::before,
  .text-xl-left body.behat-site.text-xl-left::before, body.themedesignermode .text-xl-left::before,
  .text-xl-left body.themedesignermode.text-xl-left::before {
    content: "Deprecated style in use (.text-left)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .text-xl-right {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    text-align: right !important;
  }
  body.behat-site .text-xl-right,
  .text-xl-right body.behat-site.text-xl-right, body.themedesignermode .text-xl-right,
  .text-xl-right body.themedesignermode.text-xl-right {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .text-xl-right::before,
  .text-xl-right body.behat-site.text-xl-right::before, body.themedesignermode .text-xl-right::before,
  .text-xl-right body.themedesignermode.text-xl-right::before {
    content: "Deprecated style in use (.text-right)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
@media (min-width: 1400px) {
  .text-xxl-left {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    text-align: left !important;
  }
  body.behat-site .text-xxl-left,
  .text-xxl-left body.behat-site.text-xxl-left, body.themedesignermode .text-xxl-left,
  .text-xxl-left body.themedesignermode.text-xxl-left {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .text-xxl-left::before,
  .text-xxl-left body.behat-site.text-xxl-left::before, body.themedesignermode .text-xxl-left::before,
  .text-xxl-left body.themedesignermode.text-xxl-left::before {
    content: "Deprecated style in use (.text-left)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .text-xxl-right {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    text-align: right !important;
  }
  body.behat-site .text-xxl-right,
  .text-xxl-right body.behat-site.text-xxl-right, body.themedesignermode .text-xxl-right,
  .text-xxl-right body.themedesignermode.text-xxl-right {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .text-xxl-right::before,
  .text-xxl-right body.behat-site.text-xxl-right::before, body.themedesignermode .text-xxl-right::before,
  .text-xxl-right body.themedesignermode.text-xxl-right::before {
    content: "Deprecated style in use (.text-right)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
.border-right {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  border-right: 1px solid #dee2e6 !important;
}
body.behat-site .border-right,
.border-right body.behat-site.border-right, body.themedesignermode .border-right,
.border-right body.themedesignermode.border-right {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .border-right::before,
.border-right body.behat-site.border-right::before, body.themedesignermode .border-right::before,
.border-right body.themedesignermode.border-right::before {
  content: "Deprecated style in use (.border-right)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.border-left {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  border-left: 1px solid #dee2e6 !important;
}
body.behat-site .border-left,
.border-left body.behat-site.border-left, body.themedesignermode .border-left,
.border-left body.themedesignermode.border-left {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .border-left::before,
.border-left body.behat-site.border-left::before, body.themedesignermode .border-left::before,
.border-left body.themedesignermode.border-left::before {
  content: "Deprecated style in use (.border-left)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.border-right-0 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  border-right: 0 !important;
}
body.behat-site .border-right-0,
.border-right-0 body.behat-site.border-right-0, body.themedesignermode .border-right-0,
.border-right-0 body.themedesignermode.border-right-0 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .border-right-0::before,
.border-right-0 body.behat-site.border-right-0::before, body.themedesignermode .border-right-0::before,
.border-right-0 body.themedesignermode.border-right-0::before {
  content: "Deprecated style in use (.border-right)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.border-left-0 {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  border-left: 0 !important;
}
body.behat-site .border-left-0,
.border-left-0 body.behat-site.border-left-0, body.themedesignermode .border-left-0,
.border-left-0 body.themedesignermode.border-left-0 {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .border-left-0::before,
.border-left-0 body.behat-site.border-left-0::before, body.themedesignermode .border-left-0::before,
.border-left-0 body.themedesignermode.border-left-0::before {
  content: "Deprecated style in use (.border-left)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.rounded-right {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  border-top-right-radius: 0.5rem !important;
  border-bottom-right-radius: 0.5rem !important;
}
body.behat-site .rounded-right,
.rounded-right body.behat-site.rounded-right, body.themedesignermode .rounded-right,
.rounded-right body.themedesignermode.rounded-right {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .rounded-right::before,
.rounded-right body.behat-site.rounded-right::before, body.themedesignermode .rounded-right::before,
.rounded-right body.themedesignermode.rounded-right::before {
  content: "Deprecated style in use (.rounded-right)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.rounded-left {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  border-top-left-radius: 0.5rem !important;
  border-bottom-left-radius: 0.5rem !important;
}
body.behat-site .rounded-left,
.rounded-left body.behat-site.rounded-left, body.themedesignermode .rounded-left,
.rounded-left body.themedesignermode.rounded-left {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .rounded-left::before,
.rounded-left body.behat-site.rounded-left::before, body.themedesignermode .rounded-left::before,
.rounded-left body.themedesignermode.rounded-left::before {
  content: "Deprecated style in use (.rounded-left)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.float-left {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  float: left !important;
}
body.behat-site .float-left,
.float-left body.behat-site.float-left, body.themedesignermode .float-left,
.float-left body.themedesignermode.float-left {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .float-left::before,
.float-left body.behat-site.float-left::before, body.themedesignermode .float-left::before,
.float-left body.themedesignermode.float-left::before {
  content: "Deprecated style in use (.float-left)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.float-right {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  float: right !important;
}
body.behat-site .float-right,
.float-right body.behat-site.float-right, body.themedesignermode .float-right,
.float-right body.themedesignermode.float-right {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .float-right::before,
.float-right body.behat-site.float-right::before, body.themedesignermode .float-right::before,
.float-right body.themedesignermode.float-right::before {
  content: "Deprecated style in use (.float-right)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

@media (min-width: 576px) {
  .float-sm-left {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    float: left !important;
  }
  body.behat-site .float-sm-left,
  .float-sm-left body.behat-site.float-sm-left, body.themedesignermode .float-sm-left,
  .float-sm-left body.themedesignermode.float-sm-left {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .float-sm-left::before,
  .float-sm-left body.behat-site.float-sm-left::before, body.themedesignermode .float-sm-left::before,
  .float-sm-left body.themedesignermode.float-sm-left::before {
    content: "Deprecated style in use (.float-left)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .float-sm-right {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    float: right !important;
  }
  body.behat-site .float-sm-right,
  .float-sm-right body.behat-site.float-sm-right, body.themedesignermode .float-sm-right,
  .float-sm-right body.themedesignermode.float-sm-right {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .float-sm-right::before,
  .float-sm-right body.behat-site.float-sm-right::before, body.themedesignermode .float-sm-right::before,
  .float-sm-right body.themedesignermode.float-sm-right::before {
    content: "Deprecated style in use (.float-right)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
@media (min-width: 768px) {
  .float-md-left {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    float: left !important;
  }
  body.behat-site .float-md-left,
  .float-md-left body.behat-site.float-md-left, body.themedesignermode .float-md-left,
  .float-md-left body.themedesignermode.float-md-left {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .float-md-left::before,
  .float-md-left body.behat-site.float-md-left::before, body.themedesignermode .float-md-left::before,
  .float-md-left body.themedesignermode.float-md-left::before {
    content: "Deprecated style in use (.float-left)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .float-md-right {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    float: right !important;
  }
  body.behat-site .float-md-right,
  .float-md-right body.behat-site.float-md-right, body.themedesignermode .float-md-right,
  .float-md-right body.themedesignermode.float-md-right {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .float-md-right::before,
  .float-md-right body.behat-site.float-md-right::before, body.themedesignermode .float-md-right::before,
  .float-md-right body.themedesignermode.float-md-right::before {
    content: "Deprecated style in use (.float-right)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
@media (min-width: 992px) {
  .float-lg-left {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    float: left !important;
  }
  body.behat-site .float-lg-left,
  .float-lg-left body.behat-site.float-lg-left, body.themedesignermode .float-lg-left,
  .float-lg-left body.themedesignermode.float-lg-left {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .float-lg-left::before,
  .float-lg-left body.behat-site.float-lg-left::before, body.themedesignermode .float-lg-left::before,
  .float-lg-left body.themedesignermode.float-lg-left::before {
    content: "Deprecated style in use (.float-left)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .float-lg-right {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    float: right !important;
  }
  body.behat-site .float-lg-right,
  .float-lg-right body.behat-site.float-lg-right, body.themedesignermode .float-lg-right,
  .float-lg-right body.themedesignermode.float-lg-right {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .float-lg-right::before,
  .float-lg-right body.behat-site.float-lg-right::before, body.themedesignermode .float-lg-right::before,
  .float-lg-right body.themedesignermode.float-lg-right::before {
    content: "Deprecated style in use (.float-right)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
@media (min-width: 1200px) {
  .float-xl-left {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    float: left !important;
  }
  body.behat-site .float-xl-left,
  .float-xl-left body.behat-site.float-xl-left, body.themedesignermode .float-xl-left,
  .float-xl-left body.themedesignermode.float-xl-left {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .float-xl-left::before,
  .float-xl-left body.behat-site.float-xl-left::before, body.themedesignermode .float-xl-left::before,
  .float-xl-left body.themedesignermode.float-xl-left::before {
    content: "Deprecated style in use (.float-left)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .float-xl-right {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    float: right !important;
  }
  body.behat-site .float-xl-right,
  .float-xl-right body.behat-site.float-xl-right, body.themedesignermode .float-xl-right,
  .float-xl-right body.themedesignermode.float-xl-right {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .float-xl-right::before,
  .float-xl-right body.behat-site.float-xl-right::before, body.themedesignermode .float-xl-right::before,
  .float-xl-right body.themedesignermode.float-xl-right::before {
    content: "Deprecated style in use (.float-right)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
@media (min-width: 1400px) {
  .float-xxl-left {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    float: left !important;
  }
  body.behat-site .float-xxl-left,
  .float-xxl-left body.behat-site.float-xxl-left, body.themedesignermode .float-xxl-left,
  .float-xxl-left body.themedesignermode.float-xxl-left {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .float-xxl-left::before,
  .float-xxl-left body.behat-site.float-xxl-left::before, body.themedesignermode .float-xxl-left::before,
  .float-xxl-left body.themedesignermode.float-xxl-left::before {
    content: "Deprecated style in use (.float-left)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
  .float-xxl-right {
    /* stylelint-disable declaration-no-important */
    /* stylelint-enable */
    float: right !important;
  }
  body.behat-site .float-xxl-right,
  .float-xxl-right body.behat-site.float-xxl-right, body.themedesignermode .float-xxl-right,
  .float-xxl-right body.themedesignermode.float-xxl-right {
    outline: 3px dotted #ca3120 !important;
    background-color: #fceeed !important;
  }
  body.behat-site .float-xxl-right::before,
  .float-xxl-right body.behat-site.float-xxl-right::before, body.themedesignermode .float-xxl-right::before,
  .float-xxl-right body.themedesignermode.float-xxl-right::before {
    content: "Deprecated style in use (.float-right)" !important;
    color: #ca3120 !important;
    font-size: smaller !important;
  }
}
.close {
  float: right;
  font-size: calc(1.265625rem + 0.1875vw);
  font-weight: 700;
  line-height: 1;
  color: #000;
  text-shadow: 0 1px 0 #fff;
  opacity: 0.5;
}
@media (min-width: 1200px) {
  .close {
    font-size: 1.40625rem;
  }
}
.close:hover {
  color: #000;
  text-decoration: none;
}
.close:not(:disabled):not(.disabled):hover, .close:not(:disabled):not(.disabled):focus {
  opacity: 0.75;
}

button.close {
  padding: 0;
  background-color: transparent;
  border: 0;
}

a.close.disabled {
  pointer-events: none;
}

.font-weight-light {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  font-weight: 300 !important;
}
body.behat-site .font-weight-light,
.font-weight-light body.behat-site.font-weight-light, body.themedesignermode .font-weight-light,
.font-weight-light body.themedesignermode.font-weight-light {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .font-weight-light::before,
.font-weight-light body.behat-site.font-weight-light::before, body.themedesignermode .font-weight-light::before,
.font-weight-light body.themedesignermode.font-weight-light::before {
  content: "Deprecated style in use (.font-weight)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.font-weight-lighter {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  font-weight: lighter !important;
}
body.behat-site .font-weight-lighter,
.font-weight-lighter body.behat-site.font-weight-lighter, body.themedesignermode .font-weight-lighter,
.font-weight-lighter body.themedesignermode.font-weight-lighter {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .font-weight-lighter::before,
.font-weight-lighter body.behat-site.font-weight-lighter::before, body.themedesignermode .font-weight-lighter::before,
.font-weight-lighter body.themedesignermode.font-weight-lighter::before {
  content: "Deprecated style in use (.font-weight)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.font-weight-normal {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  font-weight: 400 !important;
}
body.behat-site .font-weight-normal,
.font-weight-normal body.behat-site.font-weight-normal, body.themedesignermode .font-weight-normal,
.font-weight-normal body.themedesignermode.font-weight-normal {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .font-weight-normal::before,
.font-weight-normal body.behat-site.font-weight-normal::before, body.themedesignermode .font-weight-normal::before,
.font-weight-normal body.themedesignermode.font-weight-normal::before {
  content: "Deprecated style in use (.font-weight)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.font-weight-bold {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  font-weight: 700 !important;
}
body.behat-site .font-weight-bold,
.font-weight-bold body.behat-site.font-weight-bold, body.themedesignermode .font-weight-bold,
.font-weight-bold body.themedesignermode.font-weight-bold {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .font-weight-bold::before,
.font-weight-bold body.behat-site.font-weight-bold::before, body.themedesignermode .font-weight-bold::before,
.font-weight-bold body.themedesignermode.font-weight-bold::before {
  content: "Deprecated style in use (.font-weight)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.font-weight-bolder {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  font-weight: bolder !important;
}
body.behat-site .font-weight-bolder,
.font-weight-bolder body.behat-site.font-weight-bolder, body.themedesignermode .font-weight-bolder,
.font-weight-bolder body.themedesignermode.font-weight-bolder {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .font-weight-bolder::before,
.font-weight-bolder body.behat-site.font-weight-bolder::before, body.themedesignermode .font-weight-bolder::before,
.font-weight-bolder body.themedesignermode.font-weight-bolder::before {
  content: "Deprecated style in use (.font-weight)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.font-italic {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  font-style: italic !important;
}
body.behat-site .font-italic,
.font-italic body.behat-site.font-italic, body.themedesignermode .font-italic,
.font-italic body.themedesignermode.font-italic {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .font-italic::before,
.font-italic body.behat-site.font-italic::before, body.themedesignermode .font-italic::before,
.font-italic body.themedesignermode.font-italic::before {
  content: "Deprecated style in use (.font-weight)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.rounded-sm {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  border-radius: 0.25rem !important;
}
body.behat-site .rounded-sm,
.rounded-sm body.behat-site.rounded-sm, body.themedesignermode .rounded-sm,
.rounded-sm body.themedesignermode.rounded-sm {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .rounded-sm::before,
.rounded-sm body.behat-site.rounded-sm::before, body.themedesignermode .rounded-sm::before,
.rounded-sm body.themedesignermode.rounded-sm::before {
  content: "Deprecated style in use (.rounded-sm)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.rounded-lg {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  border-radius: 0.6rem !important;
}
body.behat-site .rounded-lg,
.rounded-lg body.behat-site.rounded-lg, body.themedesignermode .rounded-lg,
.rounded-lg body.themedesignermode.rounded-lg {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .rounded-lg::before,
.rounded-lg body.behat-site.rounded-lg::before, body.themedesignermode .rounded-lg::before,
.rounded-lg body.themedesignermode.rounded-lg::before {
  content: "Deprecated style in use (.rounded-lg)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.sr-only {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  position: absolute;
  width: 1px;
  height: 1px;
  padding: 0;
  margin: -1px;
  overflow: hidden;
  clip: rect(0, 0, 0, 0);
  white-space: nowrap;
  border: 0;
}
body.behat-site .sr-only,
.sr-only body.behat-site.sr-only, body.themedesignermode .sr-only,
.sr-only body.themedesignermode.sr-only {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .sr-only::before,
.sr-only body.behat-site.sr-only::before, body.themedesignermode .sr-only::before,
.sr-only body.themedesignermode.sr-only::before {
  content: "Deprecated style in use (.sr-only)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.sr-only-focusable {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
}
body.behat-site .sr-only-focusable,
.sr-only-focusable body.behat-site.sr-only-focusable, body.themedesignermode .sr-only-focusable,
.sr-only-focusable body.themedesignermode.sr-only-focusable {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .sr-only-focusable::before,
.sr-only-focusable body.behat-site.sr-only-focusable::before, body.themedesignermode .sr-only-focusable::before,
.sr-only-focusable body.themedesignermode.sr-only-focusable::before {
  content: "Deprecated style in use (.sr-only-focusable)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
.sr-only-focusable:active, .sr-only-focusable:focus {
  position: static;
  width: auto;
  height: auto;
  overflow: visible;
  clip: auto;
  white-space: normal;
}

.custom-control {
  position: relative;
  z-index: 1;
  display: block;
  min-height: 1.40625rem;
  padding-left: 1.5rem;
  print-color-adjust: exact;
}

.custom-control-inline {
  display: inline-flex;
  margin-right: 1rem;
}

.custom-control-input {
  position: absolute;
  left: 0;
  z-index: -1;
  width: 1rem;
  height: 1.203125rem;
  opacity: 0;
}
.custom-control-input:checked ~ .custom-control-label::before {
  color: #fff;
  border-color: #0f6cbf;
  background-color: #0f6cbf;
}
.custom-control-input:focus ~ .custom-control-label::before {
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.custom-control-input:focus:not(:checked) ~ .custom-control-label::before {
  border-color: #87b6df;
}
.custom-control-input:not(:disabled):active ~ .custom-control-label::before {
  color: #fff;
  background-color: #8bc3f6;
  border-color: #8bc3f6;
}
.custom-control-input[disabled] ~ .custom-control-label, .custom-control-input:disabled ~ .custom-control-label {
  color: #6a737b;
}
.custom-control-input[disabled] ~ .custom-control-label::before, .custom-control-input:disabled ~ .custom-control-label::before {
  background-color: var(--bs-secondary-bg);
}

.custom-control-label {
  position: relative;
  margin-bottom: 0;
  vertical-align: top;
}
.custom-control-label::before {
  position: absolute;
  top: 0.203125rem;
  left: -1.5rem;
  display: block;
  width: 1rem;
  height: 1rem;
  pointer-events: none;
  content: "";
  background-color: var(--bs-body-bg);
  border: 1px solid #8f959e;
}
.custom-control-label::after {
  position: absolute;
  top: 0.203125rem;
  left: -1.5rem;
  display: block;
  width: 1rem;
  height: 1rem;
  content: "";
  background: 50%/50% 50% no-repeat;
}

.custom-checkbox {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
}
body.behat-site .custom-checkbox,
.custom-checkbox body.behat-site.custom-checkbox, body.themedesignermode .custom-checkbox,
.custom-checkbox body.themedesignermode.custom-checkbox {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .custom-checkbox::before,
.custom-checkbox body.behat-site.custom-checkbox::before, body.themedesignermode .custom-checkbox::before,
.custom-checkbox body.themedesignermode.custom-checkbox::before {
  content: "Deprecated style in use (.custom-checkbox)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
.custom-checkbox .custom-control-label::before {
  border-radius: 0.5rem;
}
.custom-checkbox .custom-control-input:checked ~ .custom-control-label::after {
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' width='8' height='8' viewBox='0 0 8 8'%3e%3cpath fill='%23fff' d='M6.564.75l-3.59 3.612-1.538-1.55L0 4.26l2.974 2.99L8 2.193z'/%3e%3c/svg%3e");
}
.custom-checkbox .custom-control-input:indeterminate ~ .custom-control-label::before {
  border-color: #0f6cbf;
  background-color: #0f6cbf;
}
.custom-checkbox .custom-control-input:indeterminate ~ .custom-control-label::after {
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' width='4' height='4' viewBox='0 0 4 4'%3e%3cpath stroke='%23fff' d='M0 2h4'/%3e%3c/svg%3e");
}
.custom-checkbox .custom-control-input:disabled:checked ~ .custom-control-label::before {
  background-color: rgba(15, 108, 191, 0.5);
}
.custom-checkbox .custom-control-input:disabled:indeterminate ~ .custom-control-label::before {
  background-color: rgba(15, 108, 191, 0.5);
}

.custom-radio {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
}
body.behat-site .custom-radio,
.custom-radio body.behat-site.custom-radio, body.themedesignermode .custom-radio,
.custom-radio body.themedesignermode.custom-radio {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .custom-radio::before,
.custom-radio body.behat-site.custom-radio::before, body.themedesignermode .custom-radio::before,
.custom-radio body.themedesignermode.custom-radio::before {
  content: "Deprecated style in use (.custom-radio)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
.custom-radio .custom-control-label::before {
  border-radius: 50%;
}
.custom-radio .custom-control-input:checked ~ .custom-control-label::after {
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' width='12' height='12' viewBox='-4 -4 8 8'%3e%3ccircle r='3' fill='%23fff'/%3e%3c/svg%3e");
}
.custom-radio .custom-control-input:disabled:checked ~ .custom-control-label::before {
  background-color: rgba(15, 108, 191, 0.5);
}

.custom-switch {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  padding-left: 2.25rem;
}
body.behat-site .custom-switch,
.custom-switch body.behat-site.custom-switch, body.themedesignermode .custom-switch,
.custom-switch body.themedesignermode.custom-switch {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .custom-switch::before,
.custom-switch body.behat-site.custom-switch::before, body.themedesignermode .custom-switch::before,
.custom-switch body.themedesignermode.custom-switch::before {
  content: "Deprecated style in use (.custom-switch)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
.custom-switch .custom-control-label::before {
  left: -2.25rem;
  width: 1.75rem;
  pointer-events: all;
  border-radius: 0.5rem;
}
.custom-switch .custom-control-label::after {
  top: calc(0.203125rem + 2px);
  left: calc(-2.25rem + 2px);
  width: calc(1rem - 4px);
  height: calc(1rem - 4px);
  background-color: #8f959e;
  border-radius: 0.5rem;
  transition: transform 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .custom-switch .custom-control-label::after {
    transition: none;
  }
}
.custom-switch .custom-control-input:checked ~ .custom-control-label::after {
  background-color: var(--bs-body-bg);
  transform: translateX(0.75rem);
}
.custom-switch .custom-control-input:disabled:checked ~ .custom-control-label::before {
  background-color: rgba(15, 108, 191, 0.5);
}

.custom-select {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  display: inline-block;
  width: 100%;
  height: calc(1.5em + 0.75rem + calc(var(--bs-border-width) * 2));
  padding: 0.375rem 1.75rem 0.375rem 0.75rem;
  font-size: 0.9375rem;
  font-weight: 400;
  line-height: 1.5;
  color: var(--bs-body-color);
  vertical-align: middle;
  background: var(--bs-body-bg) url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' width='4' height='5' viewBox='0 0 4 5'%3e%3cpath fill='%23343a40' d='M2 0L0 2h4zm0 5L0 3h4z'/%3e%3c/svg%3e") right 0.75rem center/8px 10px no-repeat;
  border: var(--bs-border-width) solid var(--bs-gray-500);
  border-radius: 0.5rem;
  appearance: none;
}
body.behat-site .custom-select,
.custom-select body.behat-site.custom-select, body.themedesignermode .custom-select,
.custom-select body.themedesignermode.custom-select {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .custom-select::before,
.custom-select body.behat-site.custom-select::before, body.themedesignermode .custom-select::before,
.custom-select body.themedesignermode.custom-select::before {
  content: "Deprecated style in use (.custom-select)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
.custom-select:focus {
  border-color: #87b6df;
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.custom-select:focus::-ms-value {
  color: var(--bs-body-color);
  background-color: var(--bs-body-bg);
}
.custom-select[multiple], .custom-select[size]:not([size="1"]) {
  height: auto;
  padding-right: 0.75rem;
  background-image: none;
}
.custom-select:disabled {
  color: #6a737b;
  background-color: #e9ecef;
}
.custom-select::-ms-expand {
  display: none;
}
.custom-select:-moz-focusring {
  color: transparent;
  text-shadow: 0 0 0 var(--bs-body-color);
}

.custom-select-sm {
  height: calc(1.5em + 0.5rem + calc(var(--bs-border-width) * 2));
  padding-top: 0.25rem;
  padding-bottom: 0.25rem;
  padding-left: 0.5rem;
  font-size: 0.8203125rem;
}

.custom-select-lg {
  height: calc(1.5em + 1rem + calc(var(--bs-border-width) * 2));
  padding-top: 0.5rem;
  padding-bottom: 0.5rem;
  padding-left: 1rem;
  font-size: 1.171875rem;
}

.custom-file {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  position: relative;
  display: inline-block;
  width: 100%;
  height: calc(1.5em + 0.75rem + calc(var(--bs-border-width) * 2));
  margin-bottom: 0;
}
body.behat-site .custom-file,
.custom-file body.behat-site.custom-file, body.themedesignermode .custom-file,
.custom-file body.themedesignermode.custom-file {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .custom-file::before,
.custom-file body.behat-site.custom-file::before, body.themedesignermode .custom-file::before,
.custom-file body.themedesignermode.custom-file::before {
  content: "Deprecated style in use (.custom-file)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}

.custom-file-input {
  position: relative;
  z-index: 2;
  width: 100%;
  height: calc(1.5em + 0.75rem + calc(var(--bs-border-width) * 2));
  margin: 0;
  overflow: hidden;
  opacity: 0;
}
.custom-file-input:focus ~ .custom-file-label {
  border-color: #87b6df;
  box-shadow: 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.custom-file-input[disabled] ~ .custom-file-label, .custom-file-input:disabled ~ .custom-file-label {
  background-color: var(--bs-secondary-bg);
}
.custom-file-input:lang(en) ~ .custom-file-label::after {
  content: "Browse";
}
.custom-file-input ~ .custom-file-label[data-browse]::after {
  content: attr(data-browse);
}

.custom-file-label {
  position: absolute;
  top: 0;
  right: 0;
  left: 0;
  z-index: 1;
  height: calc(1.5em + 0.75rem + calc(var(--bs-border-width) * 2));
  padding: 0.375rem 0.75rem;
  overflow: hidden;
  font-weight: 400;
  line-height: 1.5;
  color: var(--bs-body-color);
  background-color: var(--bs-body-bg);
  border: var(--bs-border-width) solid var(--bs-gray-500);
  border-radius: var(--bs-border-radius);
}
.custom-file-label::after {
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  z-index: 3;
  display: block;
  height: calc(1.5em + 0.75rem);
  padding: 0.375rem 0.75rem;
  line-height: 1.5;
  color: var(--bs-body-color);
  content: "Browse";
  background-color: var(--bs-tertiary-bg);
  border-left: inherit;
  border-radius: 0 var(--bs-border-radius) var(--bs-border-radius) 0;
}

.custom-range {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  width: 100%;
  height: 1.5rem;
  padding: 0;
  background-color: transparent;
  appearance: none;
}
body.behat-site .custom-range,
.custom-range body.behat-site.custom-range, body.themedesignermode .custom-range,
.custom-range body.themedesignermode.custom-range {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .custom-range::before,
.custom-range body.behat-site.custom-range::before, body.themedesignermode .custom-range::before,
.custom-range body.themedesignermode.custom-range::before {
  content: "Deprecated style in use (.custom-range)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
.custom-range:focus {
  outline: 0;
}
.custom-range:focus::-webkit-slider-thumb {
  box-shadow: 0 0 0 1px #fff, 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.custom-range:focus::-moz-range-thumb {
  box-shadow: 0 0 0 1px #fff, 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.custom-range:focus::-ms-thumb {
  box-shadow: 0 0 0 1px #fff, 0 0 0 0.25rem rgba(15, 108, 191, 0.75);
}
.custom-range::-moz-focus-outer {
  border: 0;
}
.custom-range::-webkit-slider-thumb {
  width: 1rem;
  height: 1rem;
  margin-top: -0.25rem;
  background-color: #0f6cbf;
  border: 0;
  border-radius: 1rem;
  transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
  appearance: none;
}
@media (prefers-reduced-motion: reduce) {
  .custom-range::-webkit-slider-thumb {
    transition: none;
  }
}
.custom-range::-webkit-slider-thumb:active {
  background-color: #8bc3f6;
}
.custom-range::-webkit-slider-runnable-track {
  width: 100%;
  height: 0.5rem;
  color: transparent;
  cursor: pointer;
  background-color: #dee2e6;
  border-color: transparent;
  border-radius: 1rem;
}
.custom-range::-moz-range-thumb {
  width: 1rem;
  height: 1rem;
  background-color: #0f6cbf;
  border: 0;
  border-radius: 1rem;
  transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
  appearance: none;
}
@media (prefers-reduced-motion: reduce) {
  .custom-range::-moz-range-thumb {
    transition: none;
  }
}
.custom-range::-moz-range-thumb:active {
  background-color: #8bc3f6;
}
.custom-range::-moz-range-track {
  width: 100%;
  height: 0.5rem;
  color: transparent;
  cursor: pointer;
  background-color: #dee2e6;
  border-color: transparent;
  border-radius: 1rem;
}
.custom-range::-ms-thumb {
  width: 1rem;
  height: 1rem;
  margin-top: 0;
  margin-right: 0.25rem;
  margin-left: 0.25rem;
  background-color: #0f6cbf;
  border: 0;
  border-radius: 1rem;
  transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
  appearance: none;
}
@media (prefers-reduced-motion: reduce) {
  .custom-range::-ms-thumb {
    transition: none;
  }
}
.custom-range::-ms-thumb:active {
  background-color: #8bc3f6;
}
.custom-range::-ms-track {
  width: 100%;
  height: 0.5rem;
  color: transparent;
  cursor: pointer;
  background-color: transparent;
  border-color: transparent;
  border-width: 0.5rem;
}
.custom-range::-ms-fill-lower {
  background-color: #dee2e6;
  border-radius: 1rem;
}
.custom-range::-ms-fill-upper {
  margin-right: 15px;
  background-color: #dee2e6;
  border-radius: 1rem;
}
.custom-range:disabled::-webkit-slider-thumb {
  background-color: #8f959e;
}
.custom-range:disabled::-webkit-slider-runnable-track {
  cursor: default;
}
.custom-range:disabled::-moz-range-thumb {
  background-color: #8f959e;
}
.custom-range:disabled::-moz-range-track {
  cursor: default;
}
.custom-range:disabled::-ms-thumb {
  background-color: #8f959e;
}

.custom-control-label::before,
.custom-file-label,
.custom-select {
  transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .custom-control-label::before,
  .custom-file-label,
  .custom-select {
    transition: none;
  }
}

.btn-block {
  /* stylelint-disable declaration-no-important */
  /* stylelint-enable */
  display: block;
  width: 100%;
}
body.behat-site .btn-block,
.btn-block body.behat-site.btn-block, body.themedesignermode .btn-block,
.btn-block body.themedesignermode.btn-block {
  outline: 3px dotted #ca3120 !important;
  background-color: #fceeed !important;
}
body.behat-site .btn-block::before,
.btn-block body.behat-site.btn-block::before, body.themedesignermode .btn-block::before,
.btn-block body.themedesignermode.btn-block::before {
  content: "Deprecated style in use (.btn-block)" !important;
  color: #ca3120 !important;
  font-size: smaller !important;
}
.btn-block + .btn-block {
  margin-top: 0.5rem;
}

input[type=submit].btn-block,
input[type=reset].btn-block,
input[type=button].btn-block {
  width: 100%;
}

.task-indicator {
  padding-top: 2rem;
}
.task-indicator img.icon {
  width: 145px;
  height: 145px;
  max-height: 145px;
  max-width: 145px;
  font-size: 145px;
  padding-bottom: 2rem;
}
.task-indicator .progressbar_container {
  padding-top: 2rem;
}
.task-indicator .stored-progress-bar {
  opacity: 1;
  transition: opacity 0.5s;
}
.task-indicator .stored-progress-notstarted {
  opacity: 0;
}

body {
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

.navbar {
  box-shadow: 0 2px 4px rgba(0, 0, 0, 0.08);
}

.userpicture {
  border-radius: 50%;
}

.btn-outline-secondary {
  --bs-btn-color: #6a737b;
  --bs-btn-border-color: #6a737b;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #6a737b;
  --bs-btn-hover-border-color: #6a737b;
  --bs-btn-focus-shadow-rgb: 106, 115, 123;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #6a737b;
  --bs-btn-active-border-color: #6a737b;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #6a737b;
  --bs-btn-disabled-bg: transparent;
  --bs-btn-disabled-border-color: #6a737b;
  --bs-gradient: none;
  border-color: #6a737b;
}

.btn-outline-info {
  --bs-btn-color: #1f7e9a;
  --bs-btn-border-color: #1f7e9a;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #1f7e9a;
  --bs-btn-hover-border-color: #1f7e9a;
  --bs-btn-focus-shadow-rgb: 31, 126, 154;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #1f7e9a;
  --bs-btn-active-border-color: #1f7e9a;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #1f7e9a;
  --bs-btn-disabled-bg: transparent;
  --bs-btn-disabled-border-color: #1f7e9a;
  --bs-gradient: none;
}

.btn-outline-warning {
  --bs-btn-color: #a6670e;
  --bs-btn-border-color: #a6670e;
  --bs-btn-hover-color: #fff;
  --bs-btn-hover-bg: #a6670e;
  --bs-btn-hover-border-color: #a6670e;
  --bs-btn-focus-shadow-rgb: 166, 103, 14;
  --bs-btn-active-color: #fff;
  --bs-btn-active-bg: #a6670e;
  --bs-btn-active-border-color: #a6670e;
  --bs-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --bs-btn-disabled-color: #a6670e;
  --bs-btn-disabled-bg: transparent;
  --bs-btn-disabled-border-color: #a6670e;
  --bs-gradient: none;
}